function canvasManager(id, type){
	this.id = id;
	this.obj = null;
	this.line = null;
	this.triangle = null;
	this.hiddenTriangle = null;
	this.rectangle = null;
	this.targets = new Array();
	this.top = 0;
	this.left = 0;
	this.width = 0;
	this.height = 0;
	this.canvasSize = {};
	this.dragObj = false;
	this.intersection = false;
	this.lineColors = {};
	this.type = type;
}

canvasManager.prototype.init = function(){
	var self = this;
	
	$("canvas").remove();
	var _canvas = $("<canvas/>")
		.attr('id','canvas');
	
	$("body").append(_canvas);

	self.setCoords();
	self.setDimensions();
	
	this.obj = oCanvas.create({
		canvas: "#"+self.id
	});
	

};
canvasManager.prototype.getCanvas = function(){return this.obj;};
canvasManager.prototype.getLine = function(){return this.line;};
canvasManager.prototype.getTriangle = function(){return this.triangle;};
canvasManager.prototype.getRectangle = function(){return this.rectangle;};
canvasManager.prototype.getTargets = function(){return this.targets;};
canvasManager.prototype.getTop = function(){return this.top;};
canvasManager.prototype.getLeft = function(){return this.left;};
canvasManager.prototype.getWidth = function(){return this.width;};
canvasManager.prototype.getHeight = function(){return this.height;};
canvasManager.prototype.getType = function(){return this.type;};
canvasManager.prototype.getLineColor = function(index){return this.lineColors[index]};
canvasManager.prototype.getCanvasSize = function(size){
	var response;
	switch(size){
		case "width":
		case "height":
			response = this.canvasSize[size];
			break;
		default:
			response = this.canvasSize;
			break;
	}
	return response;
};
/*canvasManager.prototype.isControlEnabled = function(control){return this.controls[control];};*/
canvasManager.prototype.setLine = function(x, y, color){
	this.line = this.obj.display.line({
		start: {x: x - (10 * ims.interfaz.zoomRatio), y: y},
		end: {x: x, y: y},
		stroke: (6 * ims.interfaz.zoomRatio) + "px "+color
	});
	this.obj.addChild(this.line);
};
canvasManager.prototype.setHiddenTriangle = function(id, x, y, slides, radius, color){
	var self = this;
	
	this.hiddenTriangle = this.obj.display.polygon({
		x: x,
		y: y,
		sx: x,
		sy: y,
		ix: x,
		iy: y,
		sides: slides,
		radius: radius,
		rotation: 0,
		fill: color,
		idi: id,
		rel: -1,
		linea: self.getLine(),
		triangle: self.getTriangle()
	}).scale(ims.interfaz.zoomRatio, ims.interfaz.zoomRatio);
	
	this.hiddenTriangle.bind( 'tap', function(){

		if( !self.dragObj ){
//			if ( !yaCorregida ){
				self.dragObj = this;

				this.triangle.fill = "#3333BB";
				this.triangle.linea.strokeColor = "#3333BB";
				self.getCanvas().redraw();
//			}
		}
		else if ( this == self.dragObj ){
			this.x = this.triangle.ix;
			this.y = this.triangle.iy;
			this.triangle.x = this.triangle.ix;
			this.triangle.y = this.triangle.iy;
			this.triangle.rotateTo( 0 );

			this.triangle.linea.end.x = this.triangle.ix;
			this.triangle.linea.end.y = this.triangle.iy;
			this.triangle.rel = -1;
			this.triangle.fill = "#CC6633";
			this.triangle.linea.strokeColor = "#CC6633";

			self.dragObj = false;
			self.getCanvas().redraw();
		}
	});
};
canvasManager.prototype.setTriangle = function(id, x, y, slides, radius, color){
	var self = this;
	
	this.triangle = this.obj.display.polygon({
		x: x,
		y: y,
		sx: x,
		sy: y,
		ix: x,
		iy: y,
		sides: slides,
		radius: radius,
		rotation: 0,
		fill: color,
		idi: id,
		rel: -1,
		linea: self.getLine()
	}).scale(ims.interfaz.zoomRatio, ims.interfaz.zoomRatio);

	this.triangle.dragAndDrop({
		changeZindex:true,
		bubble:true,
		start:function(){
			self.intersection = false;
			self.dragObj = this;
		},
		move:function(x,y){
			this.linea.end.x = this.x;
			this.linea.end.y = this.y;
			var angulo = self.rotation(this.x,this.y,this.linea.start.x,this.linea.start.y);
			this.rotateTo(angulo);
			self.obj.redraw();
		},
		end:function(){
			this.sx = this.x;
			this.sy = this.y;
			self.setCanvasSize("width", self.getWidth());
			self.setCanvasSize("height", self.getHeight());

			var _targets = self.getTargets();

			for(var i in _targets){
				var _this = $(_targets[i].publicId);
				if(self.intersects(_this, self.dragObj)){
					self.intersection = true;
					self.dragObj.rel = _targets[i].id;
					return false;
				}
			}
			
			if(self.intersection == false){
				self.dragObj.x = self.dragObj.ix;
				self.dragObj.y = self.dragObj.iy;
				var angulo = self.rotation(self.dragObj.x, self.dragObj.y, self.dragObj.linea.start.x, self.dragObj.linea.start.y);
				self.dragObj.rotateTo(angulo);
				self.dragObj.linea.end.x = self.dragObj.ix;
				self.dragObj.linea.end.y = self.dragObj.iy;
				self.getCanvas().redraw();
				self.dragObj.rel = -1;
			}
			self.dragObj = null;
		}
	});

	self.obj.addChild(self.triangle);
};
canvasManager.prototype.setRectangle = function(id, x, y, width, height, color){
	var self = this;
	this.rectangle = self.getCanvas().display.rectangle({
		x: x,
		y: y,
		width: width,
		height: height,
		fill: color,
		idi: id
	}).scale(ims.interfaz.zoomRatio, ims.interfaz.zoomRatio);
	
	self.rectangle.bind( 'tap', function(){
		
		if( self.dragObj !== false ){
			self.dragObj.sx = this.x;
			self.dragObj.x = this.x;
			self.dragObj.y = this.y+(this.height/2);
			self.dragObj.sy = this.y+(this.height/2);

			self.dragObj.triangle.sx = this.x;
			self.dragObj.triangle.x = this.x;
			self.dragObj.triangle.y = this.y+(this.height/2);
			self.dragObj.triangle.sy = this.y+(this.height/2);

			self.dragObj.triangle.linea.end.x = self.dragObj.x;
			self.dragObj.triangle.linea.end.y = self.dragObj.y;

			// var angulo = self.dragObj.rotation( self.dragObj.x, self.dragObj.y, self.dragObj.triangle.linea.start.x, self.dragObj.triangle.linea.start.y );
			var _linea = self.dragObj.triangle.linea;
			var vX = _linea.end.x - _linea.start.x;
			var vY = _linea.end.y - _linea.start.y;
			angulo = 90 - Math.atan2( vX, vY )*180/Math.PI;
			// alert( angulo );
			self.dragObj.triangle.rotateTo(angulo);

			self.dragObj.triangle.rel = this.idi.replace( 'item_','' );
			
			// self.redraw();
			self.dragObj.triangle.fill = this.fill;
			self.dragObj.triangle.linea.strokeColor = this.fill;
			self.dragObj = false;

			self.obj.redraw();
		}
	});
	
	self.obj.addChild(self.rectangle);
};
canvasManager.prototype.setTarget = function(id){
	var _target = {
		id: id,
		publicId: '#item_' + id
	}
	this.targets.push(_target);
};
canvasManager.prototype.setTop = function(top){this.top = top;};
canvasManager.prototype.setLeft = function(left){this.left = left;};
canvasManager.prototype.setWidth = function(width){this.width = width;};
canvasManager.prototype.setHeight = function(height){this.height = height;};
canvasManager.prototype.setLineColor = function(index, value){this.lineColors[index] = value};
canvasManager.prototype.setCanvasSize = function(size, measure){
	this.canvasSize[size] = measure;
};
canvasManager.prototype.rotation = function(x1,y1,x2,y2){
	var Y = y1 - y2;
	var X = x1 - x2;
	return (Math.atan2(Y, X) * 180 / Math.PI);
};
canvasManager.prototype.setCoords = function(){
console.log('set coords', $("#visor").offset().top, ims.interfaz.zoomRatio);
	var self = this;
	var _top = $("#visor").offset().top;
	var _left = $("#visor").offset().left;
console.log('mozilla', isMoz);
	if(!isMoz){
		_top *= ims.interfaz.zoomRatio;
		_left *= ims.interfaz.zoomRatio;
//		_top = $("#visor").get(0).offsetTop;
//		_left = $("#visor").get(0).offsetLeft;
//		_left = eventObj.originalEvent.layerX - $(eventObj.currentTarget).offset().left;
//		_top = eventObj.originalEvent.layerY - $(eventObj.currentTarget).offset().top;
	}
	
	self.setTop(_top);
	self.setLeft(_left);
	
	$("canvas").css({
		'top': self.getTop(),
		'left': self.getLeft()
	});
};
canvasManager.prototype.setDimensions = function(){
console.log('set dimensions');
	this.setWidth($("#visor").width() * ims.interfaz.zoomRatio);
	this.setHeight($("#visor").height() * ims.interfaz.zoomRatio);
	
	$("canvas").attr('width', this.getWidth()).attr('height', this.getHeight());
};
canvasManager.prototype.intersects = function(obj1, obj2) {
	obj2x = parseInt(obj2.x) * (this.getWidth() / this.getCanvasSize("width"));
	obj2y = parseInt(obj2.y) * (this.getHeight() / this.getCanvasSize("height"));
	obj1width = obj1.width();
	obj1height = obj1.height();
	obj2width = 10;
	obj2height = 10;
	obj1offLeft = (obj1.offset().left - $("#visor").offset().left) * ims.interfaz.zoomRatio;
	obj1offTop = (obj1.offset().top - $("#visor").offset().top) * ims.interfaz.zoomRatio;
	if (obj1offLeft + obj1width < obj2x) {
		return false;
	};
	if (obj1offTop + obj1height < obj2y) {
		return false;
	};
	if (obj1offLeft > obj2x + obj2width) {
		return false;
	};
	if (obj1offTop > obj2y + obj2height) {
		return false;
	};
	return true;
};
canvasManager.prototype.objFind = function(cvchild){
    return $.grep(cvchild, function(item){
      return item.linea != undefined;
    });
};
canvasManager.prototype.check = function(){
	var self = this;

	var cvchild = self.getCanvas().children;
	var rowObj = self.objFind(cvchild);
	var total = 0;
	var right = 0;
	
	$.each(rowObj,function(index,item){
		total++;
		var idi = item.idi.replace('item_','');
		item.dragAndDrop(false);
		
		var _color = self.getLineColor("wrong");
		if(ims.actividad.results[idi] !== undefined && ims.actividad.results[idi] == item.rel){//correcta
			_color = self.getLineColor("right");
			right++;
		}
		item.fill = _color;
		item.linea.stroke = (6 * ims.interfaz.zoomRatio) + "px " + _color;
	});
	ims.interfaz.showResults(right, total);
	
	self.getCanvas().redraw();
};
canvasManager.prototype.solve = function(){
	switch(this.type){
		case 'arrow':
			this.solveArrow();
			break;
	}
};
canvasManager.prototype.solveArrow = function(){
	var self = this;
	var cvchild = self.getCanvas().children;
console.log('children', cvchild);
	var rowObj = self.objFind(cvchild);
console.log('solution', ims.actividad.results, rowObj);
	$.each(rowObj,function(index,item){
console.log(index, item);
		var idi = item.idi.replace('item_','');
		if(ims.actividad.results[idi] !== undefined){
console.log('idi', idi, ims.actividad.results, ims.actividad.results[idi]);
			var itTrg = '#item_' + ims.actividad.results[idi];
			var trg = $(itTrg+'.target');
			var trgH = (trg.outerHeight()/2);
			var trgOff = trg.offset();

			item.x = (trgOff.left - $("#visor").offset().left) * ims.interfaz.zoomRatio;
			item.sx = item.x;
			item.y = (trgOff.top - $("#visor").offset().top +trgH) * ims.interfaz.zoomRatio;
			item.sy = item.y;
			item.linea.end.x = item.x;
			item.linea.end.y = item.y;
			var angulo = self.rotation(item.linea.end.x, item.linea.end.y, item.linea.start.x, item.linea.start.y);
			item.rotateTo(angulo);
			item.fill = self.getLineColor("right");
			item.linea.stroke = (6 * ims.interfaz.zoomRatio) + "px " + item.fill;
			
			ims.interfaz.showStars($('#item_'+ims.actividad.results[idi]));				
		}
	});
	self.getCanvas().redraw();
};
canvasManager.prototype.resize = function(){
	var self = this;
	self.setCoords();
	self.setDimensions();
	
console.log('canvas', self);
	var cvchild = self.getCanvas().children;
	var rowObj = self.objFind(cvchild);

	$.each(rowObj,function(index,item){
		var _this = $('#'+item.idi);
		var _x = self.getLineCoords(_this).x;
		var _y = self.getLineCoords(_this).y;
		var color = item.linea.stroke.split(' ');
		var stroke = (6 * ims.interfaz.zoomRatio) + "px " + color[1];
		
		item.x = item.sx * (self.getWidth() / self.getCanvasSize("width"));
		item.y = item.sy * (self.getHeight() / self.getCanvasSize("height"));
		item.linea.start.x =_x - (10 * ims.interfaz.zoomRatio);
		item.linea.start.y =_y;
		item.linea.end.x = item.x;
		item.linea.end.y = item.y;
		item.scale(ims.interfaz.zoomRatio, ims.interfaz.zoomRatio);
		item.linea.stroke = stroke;
	});

	self.getCanvas().redraw();	
};
canvasManager.prototype.getLineCoords = function(_this){
	var _x = _this.offset().left - $("#visor").offset().left;
	var _y = _this.offset().top - $("#visor").offset().top;
	if(!isMoz){
		_x *= ims.interfaz.zoomRatio;
		_y *= ims.interfaz.zoomRatio;
	}
	var _coords = {
		x:  _x + _this.outerWidth() * ims.interfaz.zoomRatio + (10 * ims.interfaz.zoomRatio),
		y: _y + (_this.outerHeight() / 2) * ims.interfaz.zoomRatio
	};
	return _coords;
};