/** vim: et:ts=4:sw=4:sts=4
 * @license RequireJS 2.1.22 Copyright (c) 2010-2015, The Dojo Foundation All Rights Reserved.
 * Available via the MIT or new BSD license.
 * see: http://github.com/jrburke/requirejs for details
 */
//Not using strict: uneven strict support in browsers, #392, and causes
//problems with requirejs.exec()/transpiler plugins that may not be strict.
/*jslint regexp: true, nomen: true, sloppy: true */
/*global window, navigator, document, importScripts, setTimeout, opera */

var requirejs, require, define;
(function (global) {
    var req, s, head, baseElement, dataMain, src,
        interactiveScript, currentlyAddingScript, mainScript, subPath,
        version = '2.1.22',
        commentRegExp = /(\/\*([\s\S]*?)\*\/|([^:]|^)\/\/(.*)$)/mg,
        cjsRequireRegExp = /[^.]\s*require\s*\(\s*["']([^'"\s]+)["']\s*\)/g,
        jsSuffixRegExp = /\.js$/,
        currDirRegExp = /^\.\//,
        op = Object.prototype,
        ostring = op.toString,
        hasOwn = op.hasOwnProperty,
        ap = Array.prototype,
        isBrowser = !!(typeof window !== 'undefined' && typeof navigator !== 'undefined' && window.document),
        isWebWorker = !isBrowser && typeof importScripts !== 'undefined',
        //PS3 indicates loaded and complete, but need to wait for complete
        //specifically. Sequence is 'loading', 'loaded', execution,
        // then 'complete'. The UA check is unfortunate, but not sure how
        //to feature test w/o causing perf issues.
        readyRegExp = isBrowser && navigator.platform === 'PLAYSTATION 3' ?
                      /^complete$/ : /^(complete|loaded)$/,
        defContextName = '_',
        //Oh the tragedy, detecting opera. See the usage of isOpera for reason.
        isOpera = typeof opera !== 'undefined' && opera.toString() === '[object Opera]',
        contexts = {},
        cfg = {},
        globalDefQueue = [],
        useInteractive = false;

    function isFunction(it) {
        return ostring.call(it) === '[object Function]';
    }

    function isArray(it) {
        return ostring.call(it) === '[object Array]';
    }

    /**
     * Helper function for iterating over an array. If the func returns
     * a true value, it will break out of the loop.
     */
    function each(ary, func) {
        if (ary) {
            var i;
            for (i = 0; i < ary.length; i += 1) {
                if (ary[i] && func(ary[i], i, ary)) {
                    break;
                }
            }
        }
    }

    /**
     * Helper function for iterating over an array backwards. If the func
     * returns a true value, it will break out of the loop.
     */
    function eachReverse(ary, func) {
        if (ary) {
            var i;
            for (i = ary.length - 1; i > -1; i -= 1) {
                if (ary[i] && func(ary[i], i, ary)) {
                    break;
                }
            }
        }
    }

    function hasProp(obj, prop) {
        return hasOwn.call(obj, prop);
    }

    function getOwn(obj, prop) {
        return hasProp(obj, prop) && obj[prop];
    }

    /**
     * Cycles over properties in an object and calls a function for each
     * property value. If the function returns a truthy value, then the
     * iteration is stopped.
     */
    function eachProp(obj, func) {
        var prop;
        for (prop in obj) {
            if (hasProp(obj, prop)) {
                if (func(obj[prop], prop)) {
                    break;
                }
            }
        }
    }

    /**
     * Simple function to mix in properties from source into target,
     * but only if target does not already have a property of the same name.
     */
    function mixin(target, source, force, deepStringMixin) {
        if (source) {
            eachProp(source, function (value, prop) {
                if (force || !hasProp(target, prop)) {
                    if (deepStringMixin && typeof value === 'object' && value &&
                        !isArray(value) && !isFunction(value) &&
                        !(value instanceof RegExp)) {

                        if (!target[prop]) {
                            target[prop] = {};
                        }
                        mixin(target[prop], value, force, deepStringMixin);
                    } else {
                        target[prop] = value;
                    }
                }
            });
        }
        return target;
    }

    //Similar to Function.prototype.bind, but the 'this' object is specified
    //first, since it is easier to read/figure out what 'this' will be.
    function bind(obj, fn) {
        return function () {
            return fn.apply(obj, arguments);
        };
    }

    function scripts() {
        return document.getElementsByTagName('script');
    }

    function defaultOnError(err) {
        throw err;
    }

    //Allow getting a global that is expressed in
    //dot notation, like 'a.b.c'.
    function getGlobal(value) {
        if (!value) {
            return value;
        }
        var g = global;
        each(value.split('.'), function (part) {
            g = g[part];
        });
        return g;
    }

    /**
     * Constructs an error with a pointer to an URL with more information.
     * @param {String} id the error ID that maps to an ID on a web page.
     * @param {String} message human readable error.
     * @param {Error} [err] the original error, if there is one.
     *
     * @returns {Error}
     */
    function makeError(id, msg, err, requireModules) {
        var e = new Error(msg + '\nhttp://requirejs.org/docs/errors.html#' + id);
        e.requireType = id;
        e.requireModules = requireModules;
        if (err) {
            e.originalError = err;
        }
        return e;
    }

    if (typeof define !== 'undefined') {
        //If a define is already in play via another AMD loader,
        //do not overwrite.
        return;
    }

    if (typeof requirejs !== 'undefined') {
        if (isFunction(requirejs)) {
            //Do not overwrite an existing requirejs instance.
            return;
        }
        cfg = requirejs;
        requirejs = undefined;
    }

    //Allow for a require config object
    if (typeof require !== 'undefined' && !isFunction(require)) {
        //assume it is a config object.
        cfg = require;
        require = undefined;
    }

    function newContext(contextName) {
        var inCheckLoaded, Module, context, handlers,
            checkLoadedTimeoutId,
            config = {
                //Defaults. Do not set a default for map
                //config to speed up normalize(), which
                //will run faster if there is no default.
                waitSeconds: 7,
                baseUrl: './',
                paths: {},
                bundles: {},
                pkgs: {},
                shim: {},
                config: {}
            },
            registry = {},
            //registry of just enabled modules, to speed
            //cycle breaking code when lots of modules
            //are registered, but not activated.
            enabledRegistry = {},
            undefEvents = {},
            defQueue = [],
            defined = {},
            urlFetched = {},
            bundlesMap = {},
            requireCounter = 1,
            unnormalizedCounter = 1;

        /**
         * Trims the . and .. from an array of path segments.
         * It will keep a leading path segment if a .. will become
         * the first path segment, to help with module name lookups,
         * which act like paths, but can be remapped. But the end result,
         * all paths that use this function should look normalized.
         * NOTE: this method MODIFIES the input array.
         * @param {Array} ary the array of path segments.
         */
        function trimDots(ary) {
            var i, part;
            for (i = 0; i < ary.length; i++) {
                part = ary[i];
                if (part === '.') {
                    ary.splice(i, 1);
                    i -= 1;
                } else if (part === '..') {
                    // If at the start, or previous value is still ..,
                    // keep them so that when converted to a path it may
                    // still work when converted to a path, even though
                    // as an ID it is less than ideal. In larger point
                    // releases, may be better to just kick out an error.
                    if (i === 0 || (i === 1 && ary[2] === '..') || ary[i - 1] === '..') {
                        continue;
                    } else if (i > 0) {
                        ary.splice(i - 1, 2);
                        i -= 2;
                    }
                }
            }
        }

        /**
         * Given a relative module name, like ./something, normalize it to
         * a real name that can be mapped to a path.
         * @param {String} name the relative name
         * @param {String} baseName a real name that the name arg is relative
         * to.
         * @param {Boolean} applyMap apply the map config to the value. Should
         * only be done if this normalization is for a dependency ID.
         * @returns {String} normalized name
         */
        function normalize(name, baseName, applyMap) {
            var pkgMain, mapValue, nameParts, i, j, nameSegment, lastIndex,
                foundMap, foundI, foundStarMap, starI, normalizedBaseParts,
                baseParts = (baseName && baseName.split('/')),
                map = config.map,
                starMap = map && map['*'];

            //Adjust any relative paths.
            if (name) {
                name = name.split('/');
                lastIndex = name.length - 1;

                // If wanting node ID compatibility, strip .js from end
                // of IDs. Have to do this here, and not in nameToUrl
                // because node allows either .js or non .js to map
                // to same file.
                if (config.nodeIdCompat && jsSuffixRegExp.test(name[lastIndex])) {
                    name[lastIndex] = name[lastIndex].replace(jsSuffixRegExp, '');
                }

                // Starts with a '.' so need the baseName
                if (name[0].charAt(0) === '.' && baseParts) {
                    //Convert baseName to array, and lop off the last part,
                    //so that . matches that 'directory' and not name of the baseName's
                    //module. For instance, baseName of 'one/two/three', maps to
                    //'one/two/three.js', but we want the directory, 'one/two' for
                    //this normalization.
                    normalizedBaseParts = baseParts.slice(0, baseParts.length - 1);
                    name = normalizedBaseParts.concat(name);
                }

                trimDots(name);
                name = name.join('/');
            }

            //Apply map config if available.
            if (applyMap && map && (baseParts || starMap)) {
                nameParts = name.split('/');

                outerLoop: for (i = nameParts.length; i > 0; i -= 1) {
                    nameSegment = nameParts.slice(0, i).join('/');

                    if (baseParts) {
                        //Find the longest baseName segment match in the config.
                        //So, do joins on the biggest to smallest lengths of baseParts.
                        for (j = baseParts.length; j > 0; j -= 1) {
                            mapValue = getOwn(map, baseParts.slice(0, j).join('/'));

                            //baseName segment has config, find if it has one for
                            //this name.
                            if (mapValue) {
                                mapValue = getOwn(mapValue, nameSegment);
                                if (mapValue) {
                                    //Match, update name to the new value.
                                    foundMap = mapValue;
                                    foundI = i;
                                    break outerLoop;
                                }
                            }
                        }
                    }

                    //Check for a star map match, but just hold on to it,
                    //if there is a shorter segment match later in a matching
                    //config, then favor over this star map.
                    if (!foundStarMap && starMap && getOwn(starMap, nameSegment)) {
                        foundStarMap = getOwn(starMap, nameSegment);
                        starI = i;
                    }
                }

                if (!foundMap && foundStarMap) {
                    foundMap = foundStarMap;
                    foundI = starI;
                }

                if (foundMap) {
                    nameParts.splice(0, foundI, foundMap);
                    name = nameParts.join('/');
                }
            }

            // If the name points to a package's name, use
            // the package main instead.
            pkgMain = getOwn(config.pkgs, name);

            return pkgMain ? pkgMain : name;
        }

        function removeScript(name) {
            if (isBrowser) {
                each(scripts(), function (scriptNode) {
                    if (scriptNode.getAttribute('data-requiremodule') === name &&
                            scriptNode.getAttribute('data-requirecontext') === context.contextName) {
                        scriptNode.parentNode.removeChild(scriptNode);
                        return true;
                    }
                });
            }
        }

        function hasPathFallback(id) {
            var pathConfig = getOwn(config.paths, id);
            if (pathConfig && isArray(pathConfig) && pathConfig.length > 1) {
                //Pop off the first array value, since it failed, and
                //retry
                pathConfig.shift();
                context.require.undef(id);

                //Custom require that does not do map translation, since
                //ID is "absolute", already mapped/resolved.
                context.makeRequire(null, {
                    skipMap: true
                })([id]);

                return true;
            }
        }

        //Turns a plugin!resource to [plugin, resource]
        //with the plugin being undefined if the name
        //did not have a plugin prefix.
        function splitPrefix(name) {
            var prefix,
                index = name ? name.indexOf('!') : -1;
            if (index > -1) {
                prefix = name.substring(0, index);
                name = name.substring(index + 1, name.length);
            }
            return [prefix, name];
        }

        /**
         * Creates a module mapping that includes plugin prefix, module
         * name, and path. If parentModuleMap is provided it will
         * also normalize the name via require.normalize()
         *
         * @param {String} name the module name
         * @param {String} [parentModuleMap] parent module map
         * for the module name, used to resolve relative names.
         * @param {Boolean} isNormalized: is the ID already normalized.
         * This is true if this call is done for a define() module ID.
         * @param {Boolean} applyMap: apply the map config to the ID.
         * Should only be true if this map is for a dependency.
         *
         * @returns {Object}
         */
        function makeModuleMap(name, parentModuleMap, isNormalized, applyMap) {
            var url, pluginModule, suffix, nameParts,
                prefix = null,
                parentName = parentModuleMap ? parentModuleMap.name : null,
                originalName = name,
                isDefine = true,
                normalizedName = '';

            //If no name, then it means it is a require call, generate an
            //internal name.
            if (!name) {
                isDefine = false;
                name = '_@r' + (requireCounter += 1);
            }

            nameParts = splitPrefix(name);
            prefix = nameParts[0];
            name = nameParts[1];

            if (prefix) {
                prefix = normalize(prefix, parentName, applyMap);
                pluginModule = getOwn(defined, prefix);
            }

            //Account for relative paths if there is a base name.
            if (name) {
                if (prefix) {
                    if (pluginModule && pluginModule.normalize) {
                        //Plugin is loaded, use its normalize method.
                        normalizedName = pluginModule.normalize(name, function (name) {
                            return normalize(name, parentName, applyMap);
                        });
                    } else {
                        // If nested plugin references, then do not try to
                        // normalize, as it will not normalize correctly. This
                        // places a restriction on resourceIds, and the longer
                        // term solution is not to normalize until plugins are
                        // loaded and all normalizations to allow for async
                        // loading of a loader plugin. But for now, fixes the
                        // common uses. Details in #1131
                        normalizedName = name.indexOf('!') === -1 ?
                                         normalize(name, parentName, applyMap) :
                                         name;
                    }
                } else {
                    //A regular module.
                    normalizedName = normalize(name, parentName, applyMap);

                    //Normalized name may be a plugin ID due to map config
                    //application in normalize. The map config values must
                    //already be normalized, so do not need to redo that part.
                    nameParts = splitPrefix(normalizedName);
                    prefix = nameParts[0];
                    normalizedName = nameParts[1];
                    isNormalized = true;

                    url = context.nameToUrl(normalizedName);
                }
            }

            //If the id is a plugin id that cannot be determined if it needs
            //normalization, stamp it with a unique ID so two matching relative
            //ids that may conflict can be separate.
            suffix = prefix && !pluginModule && !isNormalized ?
                     '_unnormalized' + (unnormalizedCounter += 1) :
                     '';

            return {
                prefix: prefix,
                name: normalizedName,
                parentMap: parentModuleMap,
                unnormalized: !!suffix,
                url: url,
                originalName: originalName,
                isDefine: isDefine,
                id: (prefix ?
                        prefix + '!' + normalizedName :
                        normalizedName) + suffix
            };
        }

        function getModule(depMap) {
            var id = depMap.id,
                mod = getOwn(registry, id);

            if (!mod) {
                mod = registry[id] = new context.Module(depMap);
            }

            return mod;
        }

        function on(depMap, name, fn) {
            var id = depMap.id,
                mod = getOwn(registry, id);

            if (hasProp(defined, id) &&
                    (!mod || mod.defineEmitComplete)) {
                if (name === 'defined') {
                    fn(defined[id]);
                }
            } else {
                mod = getModule(depMap);
                if (mod.error && name === 'error') {
                    fn(mod.error);
                } else {
                    mod.on(name, fn);
                }
            }
        }

        function onError(err, errback) {
            var ids = err.requireModules,
                notified = false;

            if (errback) {
                errback(err);
            } else {
                each(ids, function (id) {
                    var mod = getOwn(registry, id);
                    if (mod) {
                        //Set error on module, so it skips timeout checks.
                        mod.error = err;
                        if (mod.events.error) {
                            notified = true;
                            mod.emit('error', err);
                        }
                    }
                });

                if (!notified) {
                    req.onError(err);
                }
            }
        }

        /**
         * Internal method to transfer globalQueue items to this context's
         * defQueue.
         */
        function takeGlobalQueue() {
            //Push all the globalDefQueue items into the context's defQueue
            if (globalDefQueue.length) {
                each(globalDefQueue, function(queueItem) {
                    var id = queueItem[0];
                    if (typeof id === 'string') {
                        context.defQueueMap[id] = true;
                    }
                    defQueue.push(queueItem);
                });
                globalDefQueue = [];
            }
        }

        handlers = {
            'require': function (mod) {
                if (mod.require) {
                    return mod.require;
                } else {
                    return (mod.require = context.makeRequire(mod.map));
                }
            },
            'exports': function (mod) {
                mod.usingExports = true;
                if (mod.map.isDefine) {
                    if (mod.exports) {
                        return (defined[mod.map.id] = mod.exports);
                    } else {
                        return (mod.exports = defined[mod.map.id] = {});
                    }
                }
            },
            'module': function (mod) {
                if (mod.module) {
                    return mod.module;
                } else {
                    return (mod.module = {
                        id: mod.map.id,
                        uri: mod.map.url,
                        config: function () {
                            return getOwn(config.config, mod.map.id) || {};
                        },
                        exports: mod.exports || (mod.exports = {})
                    });
                }
            }
        };

        function cleanRegistry(id) {
            //Clean up machinery used for waiting modules.
            delete registry[id];
            delete enabledRegistry[id];
        }

        function breakCycle(mod, traced, processed) {
            var id = mod.map.id;

            if (mod.error) {
                mod.emit('error', mod.error);
            } else {
                traced[id] = true;
                each(mod.depMaps, function (depMap, i) {
                    var depId = depMap.id,
                        dep = getOwn(registry, depId);

                    //Only force things that have not completed
                    //being defined, so still in the registry,
                    //and only if it has not been matched up
                    //in the module already.
                    if (dep && !mod.depMatched[i] && !processed[depId]) {
                        if (getOwn(traced, depId)) {
                            mod.defineDep(i, defined[depId]);
                            mod.check(); //pass false?
                        } else {
                            breakCycle(dep, traced, processed);
                        }
                    }
                });
                processed[id] = true;
            }
        }

        function checkLoaded() {
            var err, usingPathFallback,
                waitInterval = config.waitSeconds * 1000,
                //It is possible to disable the wait interval by using waitSeconds of 0.
                expired = waitInterval && (context.startTime + waitInterval) < new Date().getTime(),
                noLoads = [],
                reqCalls = [],
                stillLoading = false,
                needCycleCheck = true;

            //Do not bother if this call was a result of a cycle break.
            if (inCheckLoaded) {
                return;
            }

            inCheckLoaded = true;

            //Figure out the state of all the modules.
            eachProp(enabledRegistry, function (mod) {
                var map = mod.map,
                    modId = map.id;

                //Skip things that are not enabled or in error state.
                if (!mod.enabled) {
                    return;
                }

                if (!map.isDefine) {
                    reqCalls.push(mod);
                }

                if (!mod.error) {
                    //If the module should be executed, and it has not
                    //been inited and time is up, remember it.
                    if (!mod.inited && expired) {
                        if (hasPathFallback(modId)) {
                            usingPathFallback = true;
                            stillLoading = true;
                        } else {
                            noLoads.push(modId);
                            removeScript(modId);
                        }
                    } else if (!mod.inited && mod.fetched && map.isDefine) {
                        stillLoading = true;
                        if (!map.prefix) {
                            //No reason to keep looking for unfinished
                            //loading. If the only stillLoading is a
                            //plugin resource though, keep going,
                            //because it may be that a plugin resource
                            //is waiting on a non-plugin cycle.
                            return (needCycleCheck = false);
                        }
                    }
                }
            });

            if (expired && noLoads.length) {
                //If wait time expired, throw error of unloaded modules.
                err = makeError('timeout', 'Load timeout for modules: ' + noLoads, null, noLoads);
                err.contextName = context.contextName;
                return onError(err);
            }

            //Not expired, check for a cycle.
            if (needCycleCheck) {
                each(reqCalls, function (mod) {
                    breakCycle(mod, {}, {});
                });
            }

            //If still waiting on loads, and the waiting load is something
            //other than a plugin resource, or there are still outstanding
            //scripts, then just try back later.
            if ((!expired || usingPathFallback) && stillLoading) {
                //Something is still waiting to load. Wait for it, but only
                //if a timeout is not already in effect.
                if ((isBrowser || isWebWorker) && !checkLoadedTimeoutId) {
                    checkLoadedTimeoutId = setTimeout(function () {
                        checkLoadedTimeoutId = 0;
                        checkLoaded();
                    }, 50);
                }
            }

            inCheckLoaded = false;
        }

        Module = function (map) {
            this.events = getOwn(undefEvents, map.id) || {};
            this.map = map;
            this.shim = getOwn(config.shim, map.id);
            this.depExports = [];
            this.depMaps = [];
            this.depMatched = [];
            this.pluginMaps = {};
            this.depCount = 0;

            /* this.exports this.factory
               this.depMaps = [],
               this.enabled, this.fetched
            */
        };

        Module.prototype = {
            init: function (depMaps, factory, errback, options) {
                options = options || {};

                //Do not do more inits if already done. Can happen if there
                //are multiple define calls for the same module. That is not
                //a normal, common case, but it is also not unexpected.
                if (this.inited) {
                    return;
                }

                this.factory = factory;

                if (errback) {
                    //Register for errors on this module.
                    this.on('error', errback);
                } else if (this.events.error) {
                    //If no errback already, but there are error listeners
                    //on this module, set up an errback to pass to the deps.
                    errback = bind(this, function (err) {
                        this.emit('error', err);
                    });
                }

                //Do a copy of the dependency array, so that
                //source inputs are not modified. For example
                //"shim" deps are passed in here directly, and
                //doing a direct modification of the depMaps array
                //would affect that config.
                this.depMaps = depMaps && depMaps.slice(0);

                this.errback = errback;

                //Indicate this module has be initialized
                this.inited = true;

                this.ignore = options.ignore;

                //Could have option to init this module in enabled mode,
                //or could have been previously marked as enabled. However,
                //the dependencies are not known until init is called. So
                //if enabled previously, now trigger dependencies as enabled.
                if (options.enabled || this.enabled) {
                    //Enable this module and dependencies.
                    //Will call this.check()
                    this.enable();
                } else {
                    this.check();
                }
            },

            defineDep: function (i, depExports) {
                //Because of cycles, defined callback for a given
                //export can be called more than once.
                if (!this.depMatched[i]) {
                    this.depMatched[i] = true;
                    this.depCount -= 1;
                    this.depExports[i] = depExports;
                }
            },

            fetch: function () {
                if (this.fetched) {
                    return;
                }
                this.fetched = true;

                context.startTime = (new Date()).getTime();

                var map = this.map;

                //If the manager is for a plugin managed resource,
                //ask the plugin to load it now.
                if (this.shim) {
                    context.makeRequire(this.map, {
                        enableBuildCallback: true
                    })(this.shim.deps || [], bind(this, function () {
                        return map.prefix ? this.callPlugin() : this.load();
                    }));
                } else {
                    //Regular dependency.
                    return map.prefix ? this.callPlugin() : this.load();
                }
            },

            load: function () {
                var url = this.map.url;

                //Regular dependency.
                if (!urlFetched[url]) {
                    urlFetched[url] = true;
                    context.load(this.map.id, url);
                }
            },

            /**
             * Checks if the module is ready to define itself, and if so,
             * define it.
             */
            check: function () {
                if (!this.enabled || this.enabling) {
                    return;
                }

                var err, cjsModule,
                    id = this.map.id,
                    depExports = this.depExports,
                    exports = this.exports,
                    factory = this.factory;

                if (!this.inited) {
                    // Only fetch if not already in the defQueue.
                    if (!hasProp(context.defQueueMap, id)) {
                        this.fetch();
                    }
                } else if (this.error) {
                    this.emit('error', this.error);
                } else if (!this.defining) {
                    //The factory could trigger another require call
                    //that would result in checking this module to
                    //define itself again. If already in the process
                    //of doing that, skip this work.
                    this.defining = true;

                    if (this.depCount < 1 && !this.defined) {
                        if (isFunction(factory)) {
                            try {
                                exports = context.execCb(id, factory, depExports, exports);
                            } catch (e) {
                                err = e;
                            }

                            // Favor return value over exports. If node/cjs in play,
                            // then will not have a return value anyway. Favor
                            // module.exports assignment over exports object.
                            if (this.map.isDefine && exports === undefined) {
                                cjsModule = this.module;
                                if (cjsModule) {
                                    exports = cjsModule.exports;
                                } else if (this.usingExports) {
                                    //exports already set the defined value.
                                    exports = this.exports;
                                }
                            }

                            if (err) {
                                // If there is an error listener, favor passing
                                // to that instead of throwing an error. However,
                                // only do it for define()'d  modules. require
                                // errbacks should not be called for failures in
                                // their callbacks (#699). However if a global
                                // onError is set, use that.
                                if ((this.events.error && this.map.isDefine) ||
                                    req.onError !== defaultOnError) {
                                    err.requireMap = this.map;
                                    err.requireModules = this.map.isDefine ? [this.map.id] : null;
                                    err.requireType = this.map.isDefine ? 'define' : 'require';
                                    return onError((this.error = err));
                                } else if (typeof console !== 'undefined' &&
                                           console.error) {
                                    // Log the error for debugging. If promises could be
                                    // used, this would be different, but making do.
                                    console.error(err);
                                } else {
                                    // Do not want to completely lose the error. While this
                                    // will mess up processing and lead to similar results
                                    // as bug 1440, it at least surfaces the error.
                                    req.onError(err);
                                }
                            }
                        } else {
                            //Just a literal value
                            exports = factory;
                        }

                        this.exports = exports;

                        if (this.map.isDefine && !this.ignore) {
                            defined[id] = exports;

                            if (req.onResourceLoad) {
                                var resLoadMaps = [];
                                each(this.depMaps, function (depMap) {
                                    resLoadMaps.push(depMap.normalizedMap || depMap);
                                });
                                req.onResourceLoad(context, this.map, resLoadMaps);
                            }
                        }

                        //Clean up
                        cleanRegistry(id);

                        this.defined = true;
                    }

                    //Finished the define stage. Allow calling check again
                    //to allow define notifications below in the case of a
                    //cycle.
                    this.defining = false;

                    if (this.defined && !this.defineEmitted) {
                        this.defineEmitted = true;
                        this.emit('defined', this.exports);
                        this.defineEmitComplete = true;
                    }

                }
            },

            callPlugin: function () {
                var map = this.map,
                    id = map.id,
                    //Map already normalized the prefix.
                    pluginMap = makeModuleMap(map.prefix);

                //Mark this as a dependency for this plugin, so it
                //can be traced for cycles.
                this.depMaps.push(pluginMap);

                on(pluginMap, 'defined', bind(this, function (plugin) {
                    var load, normalizedMap, normalizedMod,
                        bundleId = getOwn(bundlesMap, this.map.id),
                        name = this.map.name,
                        parentName = this.map.parentMap ? this.map.parentMap.name : null,
                        localRequire = context.makeRequire(map.parentMap, {
                            enableBuildCallback: true
                        });

                    //If current map is not normalized, wait for that
                    //normalized name to load instead of continuing.
                    if (this.map.unnormalized) {
                        //Normalize the ID if the plugin allows it.
                        if (plugin.normalize) {
                            name = plugin.normalize(name, function (name) {
                                return normalize(name, parentName, true);
                            }) || '';
                        }

                        //prefix and name should already be normalized, no need
                        //for applying map config again either.
                        normalizedMap = makeModuleMap(map.prefix + '!' + name,
                                                      this.map.parentMap);
                        on(normalizedMap,
                            'defined', bind(this, function (value) {
                                this.map.normalizedMap = normalizedMap;
                                this.init([], function () { return value; }, null, {
                                    enabled: true,
                                    ignore: true
                                });
                            }));

                        normalizedMod = getOwn(registry, normalizedMap.id);
                        if (normalizedMod) {
                            //Mark this as a dependency for this plugin, so it
                            //can be traced for cycles.
                            this.depMaps.push(normalizedMap);

                            if (this.events.error) {
                                normalizedMod.on('error', bind(this, function (err) {
                                    this.emit('error', err);
                                }));
                            }
                            normalizedMod.enable();
                        }

                        return;
                    }

                    //If a paths config, then just load that file instead to
                    //resolve the plugin, as it is built into that paths layer.
                    if (bundleId) {
                        this.map.url = context.nameToUrl(bundleId);
                        this.load();
                        return;
                    }

                    load = bind(this, function (value) {
                        this.init([], function () { return value; }, null, {
                            enabled: true
                        });
                    });

                    load.error = bind(this, function (err) {
                        this.inited = true;
                        this.error = err;
                        err.requireModules = [id];

                        //Remove temp unnormalized modules for this module,
                        //since they will never be resolved otherwise now.
                        eachProp(registry, function (mod) {
                            if (mod.map.id.indexOf(id + '_unnormalized') === 0) {
                                cleanRegistry(mod.map.id);
                            }
                        });

                        onError(err);
                    });

                    //Allow plugins to load other code without having to know the
                    //context or how to 'complete' the load.
                    load.fromText = bind(this, function (text, textAlt) {
                        /*jslint evil: true */
                        var moduleName = map.name,
                            moduleMap = makeModuleMap(moduleName),
                            hasInteractive = useInteractive;

                        //As of 2.1.0, support just passing the text, to reinforce
                        //fromText only being called once per resource. Still
                        //support old style of passing moduleName but discard
                        //that moduleName in favor of the internal ref.
                        if (textAlt) {
                            text = textAlt;
                        }

                        //Turn off interactive script matching for IE for any define
                        //calls in the text, then turn it back on at the end.
                        if (hasInteractive) {
                            useInteractive = false;
                        }

                        //Prime the system by creating a module instance for
                        //it.
                        getModule(moduleMap);

                        //Transfer any config to this other module.
                        if (hasProp(config.config, id)) {
                            config.config[moduleName] = config.config[id];
                        }

                        try {
                            req.exec(text);
                        } catch (e) {
                            return onError(makeError('fromtexteval',
                                             'fromText eval for ' + id +
                                            ' failed: ' + e,
                                             e,
                                             [id]));
                        }

                        if (hasInteractive) {
                            useInteractive = true;
                        }

                        //Mark this as a dependency for the plugin
                        //resource
                        this.depMaps.push(moduleMap);

                        //Support anonymous modules.
                        context.completeLoad(moduleName);

                        //Bind the value of that module to the value for this
                        //resource ID.
                        localRequire([moduleName], load);
                    });

                    //Use parentName here since the plugin's name is not reliable,
                    //could be some weird string with no path that actually wants to
                    //reference the parentName's path.
                    plugin.load(map.name, localRequire, load, config);
                }));

                context.enable(pluginMap, this);
                this.pluginMaps[pluginMap.id] = pluginMap;
            },

            enable: function () {
                enabledRegistry[this.map.id] = this;
                this.enabled = true;

                //Set flag mentioning that the module is enabling,
                //so that immediate calls to the defined callbacks
                //for dependencies do not trigger inadvertent load
                //with the depCount still being zero.
                this.enabling = true;

                //Enable each dependency
                each(this.depMaps, bind(this, function (depMap, i) {
                    var id, mod, handler;

                    if (typeof depMap === 'string') {
                        //Dependency needs to be converted to a depMap
                        //and wired up to this module.
                        depMap = makeModuleMap(depMap,
                                               (this.map.isDefine ? this.map : this.map.parentMap),
                                               false,
                                               !this.skipMap);
                        this.depMaps[i] = depMap;

                        handler = getOwn(handlers, depMap.id);

                        if (handler) {
                            this.depExports[i] = handler(this);
                            return;
                        }

                        this.depCount += 1;

                        on(depMap, 'defined', bind(this, function (depExports) {
                            if (this.undefed) {
                                return;
                            }
                            this.defineDep(i, depExports);
                            this.check();
                        }));

                        if (this.errback) {
                            on(depMap, 'error', bind(this, this.errback));
                        } else if (this.events.error) {
                            // No direct errback on this module, but something
                            // else is listening for errors, so be sure to
                            // propagate the error correctly.
                            on(depMap, 'error', bind(this, function(err) {
                                this.emit('error', err);
                            }));
                        }
                    }

                    id = depMap.id;
                    mod = registry[id];

                    //Skip special modules like 'require', 'exports', 'module'
                    //Also, don't call enable if it is already enabled,
                    //important in circular dependency cases.
                    if (!hasProp(handlers, id) && mod && !mod.enabled) {
                        context.enable(depMap, this);
                    }
                }));

                //Enable each plugin that is used in
                //a dependency
                eachProp(this.pluginMaps, bind(this, function (pluginMap) {
                    var mod = getOwn(registry, pluginMap.id);
                    if (mod && !mod.enabled) {
                        context.enable(pluginMap, this);
                    }
                }));

                this.enabling = false;

                this.check();
            },

            on: function (name, cb) {
                var cbs = this.events[name];
                if (!cbs) {
                    cbs = this.events[name] = [];
                }
                cbs.push(cb);
            },

            emit: function (name, evt) {
                each(this.events[name], function (cb) {
                    cb(evt);
                });
                if (name === 'error') {
                    //Now that the error handler was triggered, remove
                    //the listeners, since this broken Module instance
                    //can stay around for a while in the registry.
                    delete this.events[name];
                }
            }
        };

        function callGetModule(args) {
            //Skip modules already defined.
            if (!hasProp(defined, args[0])) {
                getModule(makeModuleMap(args[0], null, true)).init(args[1], args[2]);
            }
        }

        function removeListener(node, func, name, ieName) {
            //Favor detachEvent because of IE9
            //issue, see attachEvent/addEventListener comment elsewhere
            //in this file.
            if (node.detachEvent && !isOpera) {
                //Probably IE. If not it will throw an error, which will be
                //useful to know.
                if (ieName) {
                    node.detachEvent(ieName, func);
                }
            } else {
                node.removeEventListener(name, func, false);
            }
        }

        /**
         * Given an event from a script node, get the requirejs info from it,
         * and then removes the event listeners on the node.
         * @param {Event} evt
         * @returns {Object}
         */
        function getScriptData(evt) {
            //Using currentTarget instead of target for Firefox 2.0's sake. Not
            //all old browsers will be supported, but this one was easy enough
            //to support and still makes sense.
            var node = evt.currentTarget || evt.srcElement;

            //Remove the listeners once here.
            removeListener(node, context.onScriptLoad, 'load', 'onreadystatechange');
            removeListener(node, context.onScriptError, 'error');

            return {
                node: node,
                id: node && node.getAttribute('data-requiremodule')
            };
        }

        function intakeDefines() {
            var args;

            //Any defined modules in the global queue, intake them now.
            takeGlobalQueue();

            //Make sure any remaining defQueue items get properly processed.
            while (defQueue.length) {
                args = defQueue.shift();
                if (args[0] === null) {
                    return onError(makeError('mismatch', 'Mismatched anonymous define() module: ' +
                        args[args.length - 1]));
                } else {
                    //args are id, deps, factory. Should be normalized by the
                    //define() function.
                    callGetModule(args);
                }
            }
            context.defQueueMap = {};
        }

        context = {
            config: config,
            contextName: contextName,
            registry: registry,
            defined: defined,
            urlFetched: urlFetched,
            defQueue: defQueue,
            defQueueMap: {},
            Module: Module,
            makeModuleMap: makeModuleMap,
            nextTick: req.nextTick,
            onError: onError,

            /**
             * Set a configuration for the context.
             * @param {Object} cfg config object to integrate.
             */
            configure: function (cfg) {
                //Make sure the baseUrl ends in a slash.
                if (cfg.baseUrl) {
                    if (cfg.baseUrl.charAt(cfg.baseUrl.length - 1) !== '/') {
                        cfg.baseUrl += '/';
                    }
                }

                //Save off the paths since they require special processing,
                //they are additive.
                var shim = config.shim,
                    objs = {
                        paths: true,
                        bundles: true,
                        config: true,
                        map: true
                    };

                eachProp(cfg, function (value, prop) {
                    if (objs[prop]) {
                        if (!config[prop]) {
                            config[prop] = {};
                        }
                        mixin(config[prop], value, true, true);
                    } else {
                        config[prop] = value;
                    }
                });

                //Reverse map the bundles
                if (cfg.bundles) {
                    eachProp(cfg.bundles, function (value, prop) {
                        each(value, function (v) {
                            if (v !== prop) {
                                bundlesMap[v] = prop;
                            }
                        });
                    });
                }

                //Merge shim
                if (cfg.shim) {
                    eachProp(cfg.shim, function (value, id) {
                        //Normalize the structure
                        if (isArray(value)) {
                            value = {
                                deps: value
                            };
                        }
                        if ((value.exports || value.init) && !value.exportsFn) {
                            value.exportsFn = context.makeShimExports(value);
                        }
                        shim[id] = value;
                    });
                    config.shim = shim;
                }

                //Adjust packages if necessary.
                if (cfg.packages) {
                    each(cfg.packages, function (pkgObj) {
                        var location, name;

                        pkgObj = typeof pkgObj === 'string' ? {name: pkgObj} : pkgObj;

                        name = pkgObj.name;
                        location = pkgObj.location;
                        if (location) {
                            config.paths[name] = pkgObj.location;
                        }

                        //Save pointer to main module ID for pkg name.
                        //Remove leading dot in main, so main paths are normalized,
                        //and remove any trailing .js, since different package
                        //envs have different conventions: some use a module name,
                        //some use a file name.
                        config.pkgs[name] = pkgObj.name + '/' + (pkgObj.main || 'main')
                                     .replace(currDirRegExp, '')
                                     .replace(jsSuffixRegExp, '');
                    });
                }

                //If there are any "waiting to execute" modules in the registry,
                //update the maps for them, since their info, like URLs to load,
                //may have changed.
                eachProp(registry, function (mod, id) {
                    //If module already has init called, since it is too
                    //late to modify them, and ignore unnormalized ones
                    //since they are transient.
                    if (!mod.inited && !mod.map.unnormalized) {
                        mod.map = makeModuleMap(id, null, true);
                    }
                });

                //If a deps array or a config callback is specified, then call
                //require with those args. This is useful when require is defined as a
                //config object before require.js is loaded.
                if (cfg.deps || cfg.callback) {
                    context.require(cfg.deps || [], cfg.callback);
                }
            },

            makeShimExports: function (value) {
                function fn() {
                    var ret;
                    if (value.init) {
                        ret = value.init.apply(global, arguments);
                    }
                    return ret || (value.exports && getGlobal(value.exports));
                }
                return fn;
            },

            makeRequire: function (relMap, options) {
                options = options || {};

                function localRequire(deps, callback, errback) {
                    var id, map, requireMod;

                    if (options.enableBuildCallback && callback && isFunction(callback)) {
                        callback.__requireJsBuild = true;
                    }

                    if (typeof deps === 'string') {
                        if (isFunction(callback)) {
                            //Invalid call
                            return onError(makeError('requireargs', 'Invalid require call'), errback);
                        }

                        //If require|exports|module are requested, get the
                        //value for them from the special handlers. Caveat:
                        //this only works while module is being defined.
                        if (relMap && hasProp(handlers, deps)) {
                            return handlers[deps](registry[relMap.id]);
                        }

                        //Synchronous access to one module. If require.get is
                        //available (as in the Node adapter), prefer that.
                        if (req.get) {
                            return req.get(context, deps, relMap, localRequire);
                        }

                        //Normalize module name, if it contains . or ..
                        map = makeModuleMap(deps, relMap, false, true);
                        id = map.id;

                        if (!hasProp(defined, id)) {
                            return onError(makeError('notloaded', 'Module name "' +
                                        id +
                                        '" has not been loaded yet for context: ' +
                                        contextName +
                                        (relMap ? '' : '. Use require([])')));
                        }
                        return defined[id];
                    }

                    //Grab defines waiting in the global queue.
                    intakeDefines();

                    //Mark all the dependencies as needing to be loaded.
                    context.nextTick(function () {
                        //Some defines could have been added since the
                        //require call, collect them.
                        intakeDefines();

                        requireMod = getModule(makeModuleMap(null, relMap));

                        //Store if map config should be applied to this require
                        //call for dependencies.
                        requireMod.skipMap = options.skipMap;

                        requireMod.init(deps, callback, errback, {
                            enabled: true
                        });

                        checkLoaded();
                    });

                    return localRequire;
                }

                mixin(localRequire, {
                    isBrowser: isBrowser,

                    /**
                     * Converts a module name + .extension into an URL path.
                     * *Requires* the use of a module name. It does not support using
                     * plain URLs like nameToUrl.
                     */
                    toUrl: function (moduleNamePlusExt) {
                        var ext,
                            index = moduleNamePlusExt.lastIndexOf('.'),
                            segment = moduleNamePlusExt.split('/')[0],
                            isRelative = segment === '.' || segment === '..';

                        //Have a file extension alias, and it is not the
                        //dots from a relative path.
                        if (index !== -1 && (!isRelative || index > 1)) {
                            ext = moduleNamePlusExt.substring(index, moduleNamePlusExt.length);
                            moduleNamePlusExt = moduleNamePlusExt.substring(0, index);
                        }

                        return context.nameToUrl(normalize(moduleNamePlusExt,
                                                relMap && relMap.id, true), ext,  true);
                    },

                    defined: function (id) {
                        return hasProp(defined, makeModuleMap(id, relMap, false, true).id);
                    },

                    specified: function (id) {
                        id = makeModuleMap(id, relMap, false, true).id;
                        return hasProp(defined, id) || hasProp(registry, id);
                    }
                });

                //Only allow undef on top level require calls
                if (!relMap) {
                    localRequire.undef = function (id) {
                        //Bind any waiting define() calls to this context,
                        //fix for #408
                        takeGlobalQueue();

                        var map = makeModuleMap(id, relMap, true),
                            mod = getOwn(registry, id);

                        mod.undefed = true;
                        removeScript(id);

                        delete defined[id];
                        delete urlFetched[map.url];
                        delete undefEvents[id];

                        //Clean queued defines too. Go backwards
                        //in array so that the splices do not
                        //mess up the iteration.
                        eachReverse(defQueue, function(args, i) {
                            if (args[0] === id) {
                                defQueue.splice(i, 1);
                            }
                        });
                        delete context.defQueueMap[id];

                        if (mod) {
                            //Hold on to listeners in case the
                            //module will be attempted to be reloaded
                            //using a different config.
                            if (mod.events.defined) {
                                undefEvents[id] = mod.events;
                            }

                            cleanRegistry(id);
                        }
                    };
                }

                return localRequire;
            },

            /**
             * Called to enable a module if it is still in the registry
             * awaiting enablement. A second arg, parent, the parent module,
             * is passed in for context, when this method is overridden by
             * the optimizer. Not shown here to keep code compact.
             */
            enable: function (depMap) {
                var mod = getOwn(registry, depMap.id);
                if (mod) {
                    getModule(depMap).enable();
                }
            },

            /**
             * Internal method used by environment adapters to complete a load event.
             * A load event could be a script load or just a load pass from a synchronous
             * load call.
             * @param {String} moduleName the name of the module to potentially complete.
             */
            completeLoad: function (moduleName) {
                var found, args, mod,
                    shim = getOwn(config.shim, moduleName) || {},
                    shExports = shim.exports;

                takeGlobalQueue();

                while (defQueue.length) {
                    args = defQueue.shift();
                    if (args[0] === null) {
                        args[0] = moduleName;
                        //If already found an anonymous module and bound it
                        //to this name, then this is some other anon module
                        //waiting for its completeLoad to fire.
                        if (found) {
                            break;
                        }
                        found = true;
                    } else if (args[0] === moduleName) {
                        //Found matching define call for this script!
                        found = true;
                    }

                    callGetModule(args);
                }
                context.defQueueMap = {};

                //Do this after the cycle of callGetModule in case the result
                //of those calls/init calls changes the registry.
                mod = getOwn(registry, moduleName);

                if (!found && !hasProp(defined, moduleName) && mod && !mod.inited) {
                    if (config.enforceDefine && (!shExports || !getGlobal(shExports))) {
                        if (hasPathFallback(moduleName)) {
                            return;
                        } else {
                            return onError(makeError('nodefine',
                                             'No define call for ' + moduleName,
                                             null,
                                             [moduleName]));
                        }
                    } else {
                        //A script that does not call define(), so just simulate
                        //the call for it.
                        callGetModule([moduleName, (shim.deps || []), shim.exportsFn]);
                    }
                }

                checkLoaded();
            },

            /**
             * Converts a module name to a file path. Supports cases where
             * moduleName may actually be just an URL.
             * Note that it **does not** call normalize on the moduleName,
             * it is assumed to have already been normalized. This is an
             * internal API, not a public one. Use toUrl for the public API.
             */
            nameToUrl: function (moduleName, ext, skipExt) {
                var paths, syms, i, parentModule, url,
                    parentPath, bundleId,
                    pkgMain = getOwn(config.pkgs, moduleName);

                if (pkgMain) {
                    moduleName = pkgMain;
                }

                bundleId = getOwn(bundlesMap, moduleName);

                if (bundleId) {
                    return context.nameToUrl(bundleId, ext, skipExt);
                }

                //If a colon is in the URL, it indicates a protocol is used and it is just
                //an URL to a file, or if it starts with a slash, contains a query arg (i.e. ?)
                //or ends with .js, then assume the user meant to use an url and not a module id.
                //The slash is important for protocol-less URLs as well as full paths.
                if (req.jsExtRegExp.test(moduleName)) {
                    //Just a plain path, not module name lookup, so just return it.
                    //Add extension if it is included. This is a bit wonky, only non-.js things pass
                    //an extension, this method probably needs to be reworked.
                    url = moduleName + (ext || '');
                } else {
                    //A module that needs to be converted to a path.
                    paths = config.paths;

                    syms = moduleName.split('/');
                    //For each module name segment, see if there is a path
                    //registered for it. Start with most specific name
                    //and work up from it.
                    for (i = syms.length; i > 0; i -= 1) {
                        parentModule = syms.slice(0, i).join('/');

                        parentPath = getOwn(paths, parentModule);
                        if (parentPath) {
                            //If an array, it means there are a few choices,
                            //Choose the one that is desired
                            if (isArray(parentPath)) {
                                parentPath = parentPath[0];
                            }
                            syms.splice(0, i, parentPath);
                            break;
                        }
                    }

                    //Join the path parts together, then figure out if baseUrl is needed.
                    url = syms.join('/');
                    url += (ext || (/^data\:|\?/.test(url) || skipExt ? '' : '.js'));
                    url = (url.charAt(0) === '/' || url.match(/^[\w\+\.\-]+:/) ? '' : config.baseUrl) + url;
                }

                return config.urlArgs ? url +
                                        ((url.indexOf('?') === -1 ? '?' : '&') +
                                         config.urlArgs) : url;
            },

            //Delegates to req.load. Broken out as a separate function to
            //allow overriding in the optimizer.
            load: function (id, url) {
                req.load(context, id, url);
            },

            /**
             * Executes a module callback function. Broken out as a separate function
             * solely to allow the build system to sequence the files in the built
             * layer in the right sequence.
             *
             * @private
             */
            execCb: function (name, callback, args, exports) {
                return callback.apply(exports, args);
            },

            /**
             * callback for script loads, used to check status of loading.
             *
             * @param {Event} evt the event from the browser for the script
             * that was loaded.
             */
            onScriptLoad: function (evt) {
                //Using currentTarget instead of target for Firefox 2.0's sake. Not
                //all old browsers will be supported, but this one was easy enough
                //to support and still makes sense.
                if (evt.type === 'load' ||
                        (readyRegExp.test((evt.currentTarget || evt.srcElement).readyState))) {
                    //Reset interactive script so a script node is not held onto for
                    //to long.
                    interactiveScript = null;

                    //Pull out the name of the module and the context.
                    var data = getScriptData(evt);
                    context.completeLoad(data.id);
                }
            },

            /**
             * Callback for script errors.
             */
            onScriptError: function (evt) {
                var data = getScriptData(evt);
                if (!hasPathFallback(data.id)) {
                    var parents = [];
                    eachProp(registry, function(value, key) {
                        if (key.indexOf('_@r') !== 0) {
                            each(value.depMaps, function(depMap) {
                                if (depMap.id === data.id) {
                                    parents.push(key);
                                }
                                return true;
                            });
                        }
                    });
                    return onError(makeError('scripterror', 'Script error for "' + data.id +
                                             (parents.length ?
                                             '", needed by: ' + parents.join(', ') :
                                             '"'), evt, [data.id]));
                }
            }
        };

        context.require = context.makeRequire();
        return context;
    }

    /**
     * Main entry point.
     *
     * If the only argument to require is a string, then the module that
     * is represented by that string is fetched for the appropriate context.
     *
     * If the first argument is an array, then it will be treated as an array
     * of dependency string names to fetch. An optional function callback can
     * be specified to execute when all of those dependencies are available.
     *
     * Make a local req variable to help Caja compliance (it assumes things
     * on a require that are not standardized), and to give a short
     * name for minification/local scope use.
     */
    req = requirejs = function (deps, callback, errback, optional) {

        //Find the right context, use default
        var context, config,
            contextName = defContextName;

        // Determine if have config object in the call.
        if (!isArray(deps) && typeof deps !== 'string') {
            // deps is a config object
            config = deps;
            if (isArray(callback)) {
                // Adjust args if there are dependencies
                deps = callback;
                callback = errback;
                errback = optional;
            } else {
                deps = [];
            }
        }

        if (config && config.context) {
            contextName = config.context;
        }

        context = getOwn(contexts, contextName);
        if (!context) {
            context = contexts[contextName] = req.s.newContext(contextName);
        }

        if (config) {
            context.configure(config);
        }

        return context.require(deps, callback, errback);
    };

    /**
     * Support require.config() to make it easier to cooperate with other
     * AMD loaders on globally agreed names.
     */
    req.config = function (config) {
        return req(config);
    };

    /**
     * Execute something after the current tick
     * of the event loop. Override for other envs
     * that have a better solution than setTimeout.
     * @param  {Function} fn function to execute later.
     */
    req.nextTick = typeof setTimeout !== 'undefined' ? function (fn) {
        setTimeout(fn, 4);
    } : function (fn) { fn(); };

    /**
     * Export require as a global, but only if it does not already exist.
     */
    if (!require) {
        require = req;
    }

    req.version = version;

    //Used to filter out dependencies that are already paths.
    req.jsExtRegExp = /^\/|:|\?|\.js$/;
    req.isBrowser = isBrowser;
    s = req.s = {
        contexts: contexts,
        newContext: newContext
    };

    //Create default context.
    req({});

    //Exports some context-sensitive methods on global require.
    each([
        'toUrl',
        'undef',
        'defined',
        'specified'
    ], function (prop) {
        //Reference from contexts instead of early binding to default context,
        //so that during builds, the latest instance of the default context
        //with its config gets used.
        req[prop] = function () {
            var ctx = contexts[defContextName];
            return ctx.require[prop].apply(ctx, arguments);
        };
    });

    if (isBrowser) {
        head = s.head = document.getElementsByTagName('head')[0];
        //If BASE tag is in play, using appendChild is a problem for IE6.
        //When that browser dies, this can be removed. Details in this jQuery bug:
        //http://dev.jquery.com/ticket/2709
        baseElement = document.getElementsByTagName('base')[0];
        if (baseElement) {
            head = s.head = baseElement.parentNode;
        }
    }

    /**
     * Any errors that require explicitly generates will be passed to this
     * function. Intercept/override it if you want custom error handling.
     * @param {Error} err the error object.
     */
    req.onError = defaultOnError;

    /**
     * Creates the node for the load command. Only used in browser envs.
     */
    req.createNode = function (config, moduleName, url) {
        var node = config.xhtml ?
                document.createElementNS('http://www.w3.org/1999/xhtml', 'html:script') :
                document.createElement('script');
        node.type = config.scriptType || 'text/javascript';
        node.charset = 'utf-8';
        node.async = true;
        return node;
    };

    /**
     * Does the request to load a module for the browser case.
     * Make this a separate function to allow other environments
     * to override it.
     *
     * @param {Object} context the require context to find state.
     * @param {String} moduleName the name of the module.
     * @param {Object} url the URL to the module.
     */
    req.load = function (context, moduleName, url) {
        var config = (context && context.config) || {},
            node;
        if (isBrowser) {
            //In the browser so use a script tag
            node = req.createNode(config, moduleName, url);
            if (config.onNodeCreated) {
                config.onNodeCreated(node, config, moduleName, url);
            }

            node.setAttribute('data-requirecontext', context.contextName);
            node.setAttribute('data-requiremodule', moduleName);

            //Set up load listener. Test attachEvent first because IE9 has
            //a subtle issue in its addEventListener and script onload firings
            //that do not match the behavior of all other browsers with
            //addEventListener support, which fire the onload event for a
            //script right after the script execution. See:
            //https://connect.microsoft.com/IE/feedback/details/648057/script-onload-event-is-not-fired-immediately-after-script-execution
            //UNFORTUNATELY Opera implements attachEvent but does not follow the script
            //script execution mode.
            if (node.attachEvent &&
                    //Check if node.attachEvent is artificially added by custom script or
                    //natively supported by browser
                    //read https://github.com/jrburke/requirejs/issues/187
                    //if we can NOT find [native code] then it must NOT natively supported.
                    //in IE8, node.attachEvent does not have toString()
                    //Note the test for "[native code" with no closing brace, see:
                    //https://github.com/jrburke/requirejs/issues/273
                    !(node.attachEvent.toString && node.attachEvent.toString().indexOf('[native code') < 0) &&
                    !isOpera) {
                //Probably IE. IE (at least 6-8) do not fire
                //script onload right after executing the script, so
                //we cannot tie the anonymous define call to a name.
                //However, IE reports the script as being in 'interactive'
                //readyState at the time of the define call.
                useInteractive = true;

                node.attachEvent('onreadystatechange', context.onScriptLoad);
                //It would be great to add an error handler here to catch
                //404s in IE9+. However, onreadystatechange will fire before
                //the error handler, so that does not help. If addEventListener
                //is used, then IE will fire error before load, but we cannot
                //use that pathway given the connect.microsoft.com issue
                //mentioned above about not doing the 'script execute,
                //then fire the script load event listener before execute
                //next script' that other browsers do.
                //Best hope: IE10 fixes the issues,
                //and then destroys all installs of IE 6-9.
                //node.attachEvent('onerror', context.onScriptError);
            } else {
                node.addEventListener('load', context.onScriptLoad, false);
                node.addEventListener('error', context.onScriptError, false);
            }
            node.src = url;

            //For some cache cases in IE 6-8, the script executes before the end
            //of the appendChild execution, so to tie an anonymous define
            //call to the module name (which is stored on the node), hold on
            //to a reference to this node, but clear after the DOM insertion.
            currentlyAddingScript = node;
            if (baseElement) {
                head.insertBefore(node, baseElement);
            } else {
                head.appendChild(node);
            }
            currentlyAddingScript = null;

            return node;
        } else if (isWebWorker) {
            try {
                //In a web worker, use importScripts. This is not a very
                //efficient use of importScripts, importScripts will block until
                //its script is downloaded and evaluated. However, if web workers
                //are in play, the expectation is that a build has been done so
                //that only one script needs to be loaded anyway. This may need
                //to be reevaluated if other use cases become common.
                importScripts(url);

                //Account for anonymous modules
                context.completeLoad(moduleName);
            } catch (e) {
                context.onError(makeError('importscripts',
                                'importScripts failed for ' +
                                    moduleName + ' at ' + url,
                                e,
                                [moduleName]));
            }
        }
    };

    function getInteractiveScript() {
        if (interactiveScript && interactiveScript.readyState === 'interactive') {
            return interactiveScript;
        }

        eachReverse(scripts(), function (script) {
            if (script.readyState === 'interactive') {
                return (interactiveScript = script);
            }
        });
        return interactiveScript;
    }

    //Look for a data-main script attribute, which could also adjust the baseUrl.
    if (isBrowser && !cfg.skipDataMain) {
        //Figure out baseUrl. Get it from the script tag with require.js in it.
        eachReverse(scripts(), function (script) {
            //Set the 'head' where we can append children by
            //using the script's parent.
            if (!head) {
                head = script.parentNode;
            }

            //Look for a data-main attribute to set main script for the page
            //to load. If it is there, the path to data main becomes the
            //baseUrl, if it is not already set.
            dataMain = script.getAttribute('data-main');
            if (dataMain) {
                //Preserve dataMain in case it is a path (i.e. contains '?')
                mainScript = dataMain;

                //Set final baseUrl if there is not already an explicit one.
                if (!cfg.baseUrl) {
                    //Pull off the directory of data-main for use as the
                    //baseUrl.
                    src = mainScript.split('/');
                    mainScript = src.pop();
                    subPath = src.length ? src.join('/')  + '/' : './';

                    cfg.baseUrl = subPath;
                }

                //Strip off any trailing .js since mainScript is now
                //like a module name.
                mainScript = mainScript.replace(jsSuffixRegExp, '');

                //If mainScript is still a path, fall back to dataMain
                if (req.jsExtRegExp.test(mainScript)) {
                    mainScript = dataMain;
                }

                //Put the data-main script in the files to load.
                cfg.deps = cfg.deps ? cfg.deps.concat(mainScript) : [mainScript];

                return true;
            }
        });
    }

    /**
     * The function that handles definitions of modules. Differs from
     * require() in that a string for the module should be the first argument,
     * and the function to execute after dependencies are loaded should
     * return a value to define the module corresponding to the first argument's
     * name.
     */
    define = function (name, deps, callback) {
        var node, context;

        //Allow for anonymous modules
        if (typeof name !== 'string') {
            //Adjust args appropriately
            callback = deps;
            deps = name;
            name = null;
        }

        //This module may not have dependencies
        if (!isArray(deps)) {
            callback = deps;
            deps = null;
        }

        //If no name, and callback is a function, then figure out if it a
        //CommonJS thing with dependencies.
        if (!deps && isFunction(callback)) {
            deps = [];
            //Remove comments from the callback string,
            //look for require calls, and pull them into the dependencies,
            //but only if there are function args.
            if (callback.length) {
                callback
                    .toString()
                    .replace(commentRegExp, '')
                    .replace(cjsRequireRegExp, function (match, dep) {
                        deps.push(dep);
                    });

                //May be a CommonJS thing even without require calls, but still
                //could use exports, and module. Avoid doing exports and module
                //work though if it just needs require.
                //REQUIRES the function to expect the CommonJS variables in the
                //order listed below.
                deps = (callback.length === 1 ? ['require'] : ['require', 'exports', 'module']).concat(deps);
            }
        }

        //If in IE 6-8 and hit an anonymous define() call, do the interactive
        //work.
        if (useInteractive) {
            node = currentlyAddingScript || getInteractiveScript();
            if (node) {
                if (!name) {
                    name = node.getAttribute('data-requiremodule');
                }
                context = contexts[node.getAttribute('data-requirecontext')];
            }
        }

        //Always save off evaluating the def call until the script onload handler.
        //This allows multiple modules to be in a file without prematurely
        //tracing dependencies, and allows for anonymous module support,
        //where the module name is not known until the script onload event
        //occurs. If no context, use the global queue, and get it processed
        //in the onscript load callback.
        if (context) {
            context.defQueue.push([name, deps, callback]);
            context.defQueueMap[name] = true;
        } else {
            globalDefQueue.push([name, deps, callback]);
        }
    };

    define.amd = {
        jQuery: true
    };

    /**
     * Executes the text. Normally just uses eval, but can be modified
     * to use a better, environment-specific call. Only used for transpiling
     * loader plugins, not for plain JS modules.
     * @param {String} text the text to execute/evaluate.
     */
    req.exec = function (text) {
        /*jslint evil: true */
        return eval(text);
    };

    //Set up with config info.
    req(cfg);
}(this));

define("../../../bower_components/requirejs/require.js", function(){});

/*!
 * jQuery JavaScript Library v2.2.4
 * http://jquery.com/
 *
 * Includes Sizzle.js
 * http://sizzlejs.com/
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2016-05-20T17:23Z
 */

(function( global, factory ) {

	if ( typeof module === "object" && typeof module.exports === "object" ) {
		// For CommonJS and CommonJS-like environments where a proper `window`
		// is present, execute the factory and get jQuery.
		// For environments that do not have a `window` with a `document`
		// (such as Node.js), expose a factory as module.exports.
		// This accentuates the need for the creation of a real `window`.
		// e.g. var jQuery = require("jquery")(window);
		// See ticket #14549 for more info.
		module.exports = global.document ?
			factory( global, true ) :
			function( w ) {
				if ( !w.document ) {
					throw new Error( "jQuery requires a window with a document" );
				}
				return factory( w );
			};
	} else {
		factory( global );
	}

// Pass this if window is not defined yet
}(typeof window !== "undefined" ? window : this, function( window, noGlobal ) {

// Support: Firefox 18+
// Can't be in strict mode, several libs including ASP.NET trace
// the stack via arguments.caller.callee and Firefox dies if
// you try to trace through "use strict" call chains. (#13335)
//"use strict";
var arr = [];

var document = window.document;

var slice = arr.slice;

var concat = arr.concat;

var push = arr.push;

var indexOf = arr.indexOf;

var class2type = {};

var toString = class2type.toString;

var hasOwn = class2type.hasOwnProperty;

var support = {};



var
	version = "2.2.4",

	// Define a local copy of jQuery
	jQuery = function( selector, context ) {

		// The jQuery object is actually just the init constructor 'enhanced'
		// Need init if jQuery is called (just allow error to be thrown if not included)
		return new jQuery.fn.init( selector, context );
	},

	// Support: Android<4.1
	// Make sure we trim BOM and NBSP
	rtrim = /^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,

	// Matches dashed string for camelizing
	rmsPrefix = /^-ms-/,
	rdashAlpha = /-([\da-z])/gi,

	// Used by jQuery.camelCase as callback to replace()
	fcamelCase = function( all, letter ) {
		return letter.toUpperCase();
	};

jQuery.fn = jQuery.prototype = {

	// The current version of jQuery being used
	jquery: version,

	constructor: jQuery,

	// Start with an empty selector
	selector: "",

	// The default length of a jQuery object is 0
	length: 0,

	toArray: function() {
		return slice.call( this );
	},

	// Get the Nth element in the matched element set OR
	// Get the whole matched element set as a clean array
	get: function( num ) {
		return num != null ?

			// Return just the one element from the set
			( num < 0 ? this[ num + this.length ] : this[ num ] ) :

			// Return all the elements in a clean array
			slice.call( this );
	},

	// Take an array of elements and push it onto the stack
	// (returning the new matched element set)
	pushStack: function( elems ) {

		// Build a new jQuery matched element set
		var ret = jQuery.merge( this.constructor(), elems );

		// Add the old object onto the stack (as a reference)
		ret.prevObject = this;
		ret.context = this.context;

		// Return the newly-formed element set
		return ret;
	},

	// Execute a callback for every element in the matched set.
	each: function( callback ) {
		return jQuery.each( this, callback );
	},

	map: function( callback ) {
		return this.pushStack( jQuery.map( this, function( elem, i ) {
			return callback.call( elem, i, elem );
		} ) );
	},

	slice: function() {
		return this.pushStack( slice.apply( this, arguments ) );
	},

	first: function() {
		return this.eq( 0 );
	},

	last: function() {
		return this.eq( -1 );
	},

	eq: function( i ) {
		var len = this.length,
			j = +i + ( i < 0 ? len : 0 );
		return this.pushStack( j >= 0 && j < len ? [ this[ j ] ] : [] );
	},

	end: function() {
		return this.prevObject || this.constructor();
	},

	// For internal use only.
	// Behaves like an Array's method, not like a jQuery method.
	push: push,
	sort: arr.sort,
	splice: arr.splice
};

jQuery.extend = jQuery.fn.extend = function() {
	var options, name, src, copy, copyIsArray, clone,
		target = arguments[ 0 ] || {},
		i = 1,
		length = arguments.length,
		deep = false;

	// Handle a deep copy situation
	if ( typeof target === "boolean" ) {
		deep = target;

		// Skip the boolean and the target
		target = arguments[ i ] || {};
		i++;
	}

	// Handle case when target is a string or something (possible in deep copy)
	if ( typeof target !== "object" && !jQuery.isFunction( target ) ) {
		target = {};
	}

	// Extend jQuery itself if only one argument is passed
	if ( i === length ) {
		target = this;
		i--;
	}

	for ( ; i < length; i++ ) {

		// Only deal with non-null/undefined values
		if ( ( options = arguments[ i ] ) != null ) {

			// Extend the base object
			for ( name in options ) {
				src = target[ name ];
				copy = options[ name ];

				// Prevent never-ending loop
				if ( target === copy ) {
					continue;
				}

				// Recurse if we're merging plain objects or arrays
				if ( deep && copy && ( jQuery.isPlainObject( copy ) ||
					( copyIsArray = jQuery.isArray( copy ) ) ) ) {

					if ( copyIsArray ) {
						copyIsArray = false;
						clone = src && jQuery.isArray( src ) ? src : [];

					} else {
						clone = src && jQuery.isPlainObject( src ) ? src : {};
					}

					// Never move original objects, clone them
					target[ name ] = jQuery.extend( deep, clone, copy );

				// Don't bring in undefined values
				} else if ( copy !== undefined ) {
					target[ name ] = copy;
				}
			}
		}
	}

	// Return the modified object
	return target;
};

jQuery.extend( {

	// Unique for each copy of jQuery on the page
	expando: "jQuery" + ( version + Math.random() ).replace( /\D/g, "" ),

	// Assume jQuery is ready without the ready module
	isReady: true,

	error: function( msg ) {
		throw new Error( msg );
	},

	noop: function() {},

	isFunction: function( obj ) {
		return jQuery.type( obj ) === "function";
	},

	isArray: Array.isArray,

	isWindow: function( obj ) {
		return obj != null && obj === obj.window;
	},

	isNumeric: function( obj ) {

		// parseFloat NaNs numeric-cast false positives (null|true|false|"")
		// ...but misinterprets leading-number strings, particularly hex literals ("0x...")
		// subtraction forces infinities to NaN
		// adding 1 corrects loss of precision from parseFloat (#15100)
		var realStringObj = obj && obj.toString();
		return !jQuery.isArray( obj ) && ( realStringObj - parseFloat( realStringObj ) + 1 ) >= 0;
	},

	isPlainObject: function( obj ) {
		var key;

		// Not plain objects:
		// - Any object or value whose internal [[Class]] property is not "[object Object]"
		// - DOM nodes
		// - window
		if ( jQuery.type( obj ) !== "object" || obj.nodeType || jQuery.isWindow( obj ) ) {
			return false;
		}

		// Not own constructor property must be Object
		if ( obj.constructor &&
				!hasOwn.call( obj, "constructor" ) &&
				!hasOwn.call( obj.constructor.prototype || {}, "isPrototypeOf" ) ) {
			return false;
		}

		// Own properties are enumerated firstly, so to speed up,
		// if last one is own, then all properties are own
		for ( key in obj ) {}

		return key === undefined || hasOwn.call( obj, key );
	},

	isEmptyObject: function( obj ) {
		var name;
		for ( name in obj ) {
			return false;
		}
		return true;
	},

	type: function( obj ) {
		if ( obj == null ) {
			return obj + "";
		}

		// Support: Android<4.0, iOS<6 (functionish RegExp)
		return typeof obj === "object" || typeof obj === "function" ?
			class2type[ toString.call( obj ) ] || "object" :
			typeof obj;
	},

	// Evaluates a script in a global context
	globalEval: function( code ) {
		var script,
			indirect = eval;

		code = jQuery.trim( code );

		if ( code ) {

			// If the code includes a valid, prologue position
			// strict mode pragma, execute code by injecting a
			// script tag into the document.
			if ( code.indexOf( "use strict" ) === 1 ) {
				script = document.createElement( "script" );
				script.text = code;
				document.head.appendChild( script ).parentNode.removeChild( script );
			} else {

				// Otherwise, avoid the DOM node creation, insertion
				// and removal by using an indirect global eval

				indirect( code );
			}
		}
	},

	// Convert dashed to camelCase; used by the css and data modules
	// Support: IE9-11+
	// Microsoft forgot to hump their vendor prefix (#9572)
	camelCase: function( string ) {
		return string.replace( rmsPrefix, "ms-" ).replace( rdashAlpha, fcamelCase );
	},

	nodeName: function( elem, name ) {
		return elem.nodeName && elem.nodeName.toLowerCase() === name.toLowerCase();
	},

	each: function( obj, callback ) {
		var length, i = 0;

		if ( isArrayLike( obj ) ) {
			length = obj.length;
			for ( ; i < length; i++ ) {
				if ( callback.call( obj[ i ], i, obj[ i ] ) === false ) {
					break;
				}
			}
		} else {
			for ( i in obj ) {
				if ( callback.call( obj[ i ], i, obj[ i ] ) === false ) {
					break;
				}
			}
		}

		return obj;
	},

	// Support: Android<4.1
	trim: function( text ) {
		return text == null ?
			"" :
			( text + "" ).replace( rtrim, "" );
	},

	// results is for internal usage only
	makeArray: function( arr, results ) {
		var ret = results || [];

		if ( arr != null ) {
			if ( isArrayLike( Object( arr ) ) ) {
				jQuery.merge( ret,
					typeof arr === "string" ?
					[ arr ] : arr
				);
			} else {
				push.call( ret, arr );
			}
		}

		return ret;
	},

	inArray: function( elem, arr, i ) {
		return arr == null ? -1 : indexOf.call( arr, elem, i );
	},

	merge: function( first, second ) {
		var len = +second.length,
			j = 0,
			i = first.length;

		for ( ; j < len; j++ ) {
			first[ i++ ] = second[ j ];
		}

		first.length = i;

		return first;
	},

	grep: function( elems, callback, invert ) {
		var callbackInverse,
			matches = [],
			i = 0,
			length = elems.length,
			callbackExpect = !invert;

		// Go through the array, only saving the items
		// that pass the validator function
		for ( ; i < length; i++ ) {
			callbackInverse = !callback( elems[ i ], i );
			if ( callbackInverse !== callbackExpect ) {
				matches.push( elems[ i ] );
			}
		}

		return matches;
	},

	// arg is for internal usage only
	map: function( elems, callback, arg ) {
		var length, value,
			i = 0,
			ret = [];

		// Go through the array, translating each of the items to their new values
		if ( isArrayLike( elems ) ) {
			length = elems.length;
			for ( ; i < length; i++ ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret.push( value );
				}
			}

		// Go through every key on the object,
		} else {
			for ( i in elems ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret.push( value );
				}
			}
		}

		// Flatten any nested arrays
		return concat.apply( [], ret );
	},

	// A global GUID counter for objects
	guid: 1,

	// Bind a function to a context, optionally partially applying any
	// arguments.
	proxy: function( fn, context ) {
		var tmp, args, proxy;

		if ( typeof context === "string" ) {
			tmp = fn[ context ];
			context = fn;
			fn = tmp;
		}

		// Quick check to determine if target is callable, in the spec
		// this throws a TypeError, but we will just return undefined.
		if ( !jQuery.isFunction( fn ) ) {
			return undefined;
		}

		// Simulated bind
		args = slice.call( arguments, 2 );
		proxy = function() {
			return fn.apply( context || this, args.concat( slice.call( arguments ) ) );
		};

		// Set the guid of unique handler to the same of original handler, so it can be removed
		proxy.guid = fn.guid = fn.guid || jQuery.guid++;

		return proxy;
	},

	now: Date.now,

	// jQuery.support is not used in Core but other projects attach their
	// properties to it so it needs to exist.
	support: support
} );

// JSHint would error on this code due to the Symbol not being defined in ES5.
// Defining this global in .jshintrc would create a danger of using the global
// unguarded in another place, it seems safer to just disable JSHint for these
// three lines.
/* jshint ignore: start */
if ( typeof Symbol === "function" ) {
	jQuery.fn[ Symbol.iterator ] = arr[ Symbol.iterator ];
}
/* jshint ignore: end */

// Populate the class2type map
jQuery.each( "Boolean Number String Function Array Date RegExp Object Error Symbol".split( " " ),
function( i, name ) {
	class2type[ "[object " + name + "]" ] = name.toLowerCase();
} );

function isArrayLike( obj ) {

	// Support: iOS 8.2 (not reproducible in simulator)
	// `in` check used to prevent JIT error (gh-2145)
	// hasOwn isn't used here due to false negatives
	// regarding Nodelist length in IE
	var length = !!obj && "length" in obj && obj.length,
		type = jQuery.type( obj );

	if ( type === "function" || jQuery.isWindow( obj ) ) {
		return false;
	}

	return type === "array" || length === 0 ||
		typeof length === "number" && length > 0 && ( length - 1 ) in obj;
}
var Sizzle =
/*!
 * Sizzle CSS Selector Engine v2.2.1
 * http://sizzlejs.com/
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2015-10-17
 */
(function( window ) {

var i,
	support,
	Expr,
	getText,
	isXML,
	tokenize,
	compile,
	select,
	outermostContext,
	sortInput,
	hasDuplicate,

	// Local document vars
	setDocument,
	document,
	docElem,
	documentIsHTML,
	rbuggyQSA,
	rbuggyMatches,
	matches,
	contains,

	// Instance-specific data
	expando = "sizzle" + 1 * new Date(),
	preferredDoc = window.document,
	dirruns = 0,
	done = 0,
	classCache = createCache(),
	tokenCache = createCache(),
	compilerCache = createCache(),
	sortOrder = function( a, b ) {
		if ( a === b ) {
			hasDuplicate = true;
		}
		return 0;
	},

	// General-purpose constants
	MAX_NEGATIVE = 1 << 31,

	// Instance methods
	hasOwn = ({}).hasOwnProperty,
	arr = [],
	pop = arr.pop,
	push_native = arr.push,
	push = arr.push,
	slice = arr.slice,
	// Use a stripped-down indexOf as it's faster than native
	// http://jsperf.com/thor-indexof-vs-for/5
	indexOf = function( list, elem ) {
		var i = 0,
			len = list.length;
		for ( ; i < len; i++ ) {
			if ( list[i] === elem ) {
				return i;
			}
		}
		return -1;
	},

	booleans = "checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",

	// Regular expressions

	// http://www.w3.org/TR/css3-selectors/#whitespace
	whitespace = "[\\x20\\t\\r\\n\\f]",

	// http://www.w3.org/TR/CSS21/syndata.html#value-def-identifier
	identifier = "(?:\\\\.|[\\w-]|[^\\x00-\\xa0])+",

	// Attribute selectors: http://www.w3.org/TR/selectors/#attribute-selectors
	attributes = "\\[" + whitespace + "*(" + identifier + ")(?:" + whitespace +
		// Operator (capture 2)
		"*([*^$|!~]?=)" + whitespace +
		// "Attribute values must be CSS identifiers [capture 5] or strings [capture 3 or capture 4]"
		"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|(" + identifier + "))|)" + whitespace +
		"*\\]",

	pseudos = ":(" + identifier + ")(?:\\((" +
		// To reduce the number of selectors needing tokenize in the preFilter, prefer arguments:
		// 1. quoted (capture 3; capture 4 or capture 5)
		"('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|" +
		// 2. simple (capture 6)
		"((?:\\\\.|[^\\\\()[\\]]|" + attributes + ")*)|" +
		// 3. anything else (capture 2)
		".*" +
		")\\)|)",

	// Leading and non-escaped trailing whitespace, capturing some non-whitespace characters preceding the latter
	rwhitespace = new RegExp( whitespace + "+", "g" ),
	rtrim = new RegExp( "^" + whitespace + "+|((?:^|[^\\\\])(?:\\\\.)*)" + whitespace + "+$", "g" ),

	rcomma = new RegExp( "^" + whitespace + "*," + whitespace + "*" ),
	rcombinators = new RegExp( "^" + whitespace + "*([>+~]|" + whitespace + ")" + whitespace + "*" ),

	rattributeQuotes = new RegExp( "=" + whitespace + "*([^\\]'\"]*?)" + whitespace + "*\\]", "g" ),

	rpseudo = new RegExp( pseudos ),
	ridentifier = new RegExp( "^" + identifier + "$" ),

	matchExpr = {
		"ID": new RegExp( "^#(" + identifier + ")" ),
		"CLASS": new RegExp( "^\\.(" + identifier + ")" ),
		"TAG": new RegExp( "^(" + identifier + "|[*])" ),
		"ATTR": new RegExp( "^" + attributes ),
		"PSEUDO": new RegExp( "^" + pseudos ),
		"CHILD": new RegExp( "^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\(" + whitespace +
			"*(even|odd|(([+-]|)(\\d*)n|)" + whitespace + "*(?:([+-]|)" + whitespace +
			"*(\\d+)|))" + whitespace + "*\\)|)", "i" ),
		"bool": new RegExp( "^(?:" + booleans + ")$", "i" ),
		// For use in libraries implementing .is()
		// We use this for POS matching in `select`
		"needsContext": new RegExp( "^" + whitespace + "*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\(" +
			whitespace + "*((?:-\\d)?\\d*)" + whitespace + "*\\)|)(?=[^-]|$)", "i" )
	},

	rinputs = /^(?:input|select|textarea|button)$/i,
	rheader = /^h\d$/i,

	rnative = /^[^{]+\{\s*\[native \w/,

	// Easily-parseable/retrievable ID or TAG or CLASS selectors
	rquickExpr = /^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,

	rsibling = /[+~]/,
	rescape = /'|\\/g,

	// CSS escapes http://www.w3.org/TR/CSS21/syndata.html#escaped-characters
	runescape = new RegExp( "\\\\([\\da-f]{1,6}" + whitespace + "?|(" + whitespace + ")|.)", "ig" ),
	funescape = function( _, escaped, escapedWhitespace ) {
		var high = "0x" + escaped - 0x10000;
		// NaN means non-codepoint
		// Support: Firefox<24
		// Workaround erroneous numeric interpretation of +"0x"
		return high !== high || escapedWhitespace ?
			escaped :
			high < 0 ?
				// BMP codepoint
				String.fromCharCode( high + 0x10000 ) :
				// Supplemental Plane codepoint (surrogate pair)
				String.fromCharCode( high >> 10 | 0xD800, high & 0x3FF | 0xDC00 );
	},

	// Used for iframes
	// See setDocument()
	// Removing the function wrapper causes a "Permission Denied"
	// error in IE
	unloadHandler = function() {
		setDocument();
	};

// Optimize for push.apply( _, NodeList )
try {
	push.apply(
		(arr = slice.call( preferredDoc.childNodes )),
		preferredDoc.childNodes
	);
	// Support: Android<4.0
	// Detect silently failing push.apply
	arr[ preferredDoc.childNodes.length ].nodeType;
} catch ( e ) {
	push = { apply: arr.length ?

		// Leverage slice if possible
		function( target, els ) {
			push_native.apply( target, slice.call(els) );
		} :

		// Support: IE<9
		// Otherwise append directly
		function( target, els ) {
			var j = target.length,
				i = 0;
			// Can't trust NodeList.length
			while ( (target[j++] = els[i++]) ) {}
			target.length = j - 1;
		}
	};
}

function Sizzle( selector, context, results, seed ) {
	var m, i, elem, nid, nidselect, match, groups, newSelector,
		newContext = context && context.ownerDocument,

		// nodeType defaults to 9, since context defaults to document
		nodeType = context ? context.nodeType : 9;

	results = results || [];

	// Return early from calls with invalid selector or context
	if ( typeof selector !== "string" || !selector ||
		nodeType !== 1 && nodeType !== 9 && nodeType !== 11 ) {

		return results;
	}

	// Try to shortcut find operations (as opposed to filters) in HTML documents
	if ( !seed ) {

		if ( ( context ? context.ownerDocument || context : preferredDoc ) !== document ) {
			setDocument( context );
		}
		context = context || document;

		if ( documentIsHTML ) {

			// If the selector is sufficiently simple, try using a "get*By*" DOM method
			// (excepting DocumentFragment context, where the methods don't exist)
			if ( nodeType !== 11 && (match = rquickExpr.exec( selector )) ) {

				// ID selector
				if ( (m = match[1]) ) {

					// Document context
					if ( nodeType === 9 ) {
						if ( (elem = context.getElementById( m )) ) {

							// Support: IE, Opera, Webkit
							// TODO: identify versions
							// getElementById can match elements by name instead of ID
							if ( elem.id === m ) {
								results.push( elem );
								return results;
							}
						} else {
							return results;
						}

					// Element context
					} else {

						// Support: IE, Opera, Webkit
						// TODO: identify versions
						// getElementById can match elements by name instead of ID
						if ( newContext && (elem = newContext.getElementById( m )) &&
							contains( context, elem ) &&
							elem.id === m ) {

							results.push( elem );
							return results;
						}
					}

				// Type selector
				} else if ( match[2] ) {
					push.apply( results, context.getElementsByTagName( selector ) );
					return results;

				// Class selector
				} else if ( (m = match[3]) && support.getElementsByClassName &&
					context.getElementsByClassName ) {

					push.apply( results, context.getElementsByClassName( m ) );
					return results;
				}
			}

			// Take advantage of querySelectorAll
			if ( support.qsa &&
				!compilerCache[ selector + " " ] &&
				(!rbuggyQSA || !rbuggyQSA.test( selector )) ) {

				if ( nodeType !== 1 ) {
					newContext = context;
					newSelector = selector;

				// qSA looks outside Element context, which is not what we want
				// Thanks to Andrew Dupont for this workaround technique
				// Support: IE <=8
				// Exclude object elements
				} else if ( context.nodeName.toLowerCase() !== "object" ) {

					// Capture the context ID, setting it first if necessary
					if ( (nid = context.getAttribute( "id" )) ) {
						nid = nid.replace( rescape, "\\$&" );
					} else {
						context.setAttribute( "id", (nid = expando) );
					}

					// Prefix every selector in the list
					groups = tokenize( selector );
					i = groups.length;
					nidselect = ridentifier.test( nid ) ? "#" + nid : "[id='" + nid + "']";
					while ( i-- ) {
						groups[i] = nidselect + " " + toSelector( groups[i] );
					}
					newSelector = groups.join( "," );

					// Expand context for sibling selectors
					newContext = rsibling.test( selector ) && testContext( context.parentNode ) ||
						context;
				}

				if ( newSelector ) {
					try {
						push.apply( results,
							newContext.querySelectorAll( newSelector )
						);
						return results;
					} catch ( qsaError ) {
					} finally {
						if ( nid === expando ) {
							context.removeAttribute( "id" );
						}
					}
				}
			}
		}
	}

	// All others
	return select( selector.replace( rtrim, "$1" ), context, results, seed );
}

/**
 * Create key-value caches of limited size
 * @returns {function(string, object)} Returns the Object data after storing it on itself with
 *	property name the (space-suffixed) string and (if the cache is larger than Expr.cacheLength)
 *	deleting the oldest entry
 */
function createCache() {
	var keys = [];

	function cache( key, value ) {
		// Use (key + " ") to avoid collision with native prototype properties (see Issue #157)
		if ( keys.push( key + " " ) > Expr.cacheLength ) {
			// Only keep the most recent entries
			delete cache[ keys.shift() ];
		}
		return (cache[ key + " " ] = value);
	}
	return cache;
}

/**
 * Mark a function for special use by Sizzle
 * @param {Function} fn The function to mark
 */
function markFunction( fn ) {
	fn[ expando ] = true;
	return fn;
}

/**
 * Support testing using an element
 * @param {Function} fn Passed the created div and expects a boolean result
 */
function assert( fn ) {
	var div = document.createElement("div");

	try {
		return !!fn( div );
	} catch (e) {
		return false;
	} finally {
		// Remove from its parent by default
		if ( div.parentNode ) {
			div.parentNode.removeChild( div );
		}
		// release memory in IE
		div = null;
	}
}

/**
 * Adds the same handler for all of the specified attrs
 * @param {String} attrs Pipe-separated list of attributes
 * @param {Function} handler The method that will be applied
 */
function addHandle( attrs, handler ) {
	var arr = attrs.split("|"),
		i = arr.length;

	while ( i-- ) {
		Expr.attrHandle[ arr[i] ] = handler;
	}
}

/**
 * Checks document order of two siblings
 * @param {Element} a
 * @param {Element} b
 * @returns {Number} Returns less than 0 if a precedes b, greater than 0 if a follows b
 */
function siblingCheck( a, b ) {
	var cur = b && a,
		diff = cur && a.nodeType === 1 && b.nodeType === 1 &&
			( ~b.sourceIndex || MAX_NEGATIVE ) -
			( ~a.sourceIndex || MAX_NEGATIVE );

	// Use IE sourceIndex if available on both nodes
	if ( diff ) {
		return diff;
	}

	// Check if b follows a
	if ( cur ) {
		while ( (cur = cur.nextSibling) ) {
			if ( cur === b ) {
				return -1;
			}
		}
	}

	return a ? 1 : -1;
}

/**
 * Returns a function to use in pseudos for input types
 * @param {String} type
 */
function createInputPseudo( type ) {
	return function( elem ) {
		var name = elem.nodeName.toLowerCase();
		return name === "input" && elem.type === type;
	};
}

/**
 * Returns a function to use in pseudos for buttons
 * @param {String} type
 */
function createButtonPseudo( type ) {
	return function( elem ) {
		var name = elem.nodeName.toLowerCase();
		return (name === "input" || name === "button") && elem.type === type;
	};
}

/**
 * Returns a function to use in pseudos for positionals
 * @param {Function} fn
 */
function createPositionalPseudo( fn ) {
	return markFunction(function( argument ) {
		argument = +argument;
		return markFunction(function( seed, matches ) {
			var j,
				matchIndexes = fn( [], seed.length, argument ),
				i = matchIndexes.length;

			// Match elements found at the specified indexes
			while ( i-- ) {
				if ( seed[ (j = matchIndexes[i]) ] ) {
					seed[j] = !(matches[j] = seed[j]);
				}
			}
		});
	});
}

/**
 * Checks a node for validity as a Sizzle context
 * @param {Element|Object=} context
 * @returns {Element|Object|Boolean} The input node if acceptable, otherwise a falsy value
 */
function testContext( context ) {
	return context && typeof context.getElementsByTagName !== "undefined" && context;
}

// Expose support vars for convenience
support = Sizzle.support = {};

/**
 * Detects XML nodes
 * @param {Element|Object} elem An element or a document
 * @returns {Boolean} True iff elem is a non-HTML XML node
 */
isXML = Sizzle.isXML = function( elem ) {
	// documentElement is verified for cases where it doesn't yet exist
	// (such as loading iframes in IE - #4833)
	var documentElement = elem && (elem.ownerDocument || elem).documentElement;
	return documentElement ? documentElement.nodeName !== "HTML" : false;
};

/**
 * Sets document-related variables once based on the current document
 * @param {Element|Object} [doc] An element or document object to use to set the document
 * @returns {Object} Returns the current document
 */
setDocument = Sizzle.setDocument = function( node ) {
	var hasCompare, parent,
		doc = node ? node.ownerDocument || node : preferredDoc;

	// Return early if doc is invalid or already selected
	if ( doc === document || doc.nodeType !== 9 || !doc.documentElement ) {
		return document;
	}

	// Update global variables
	document = doc;
	docElem = document.documentElement;
	documentIsHTML = !isXML( document );

	// Support: IE 9-11, Edge
	// Accessing iframe documents after unload throws "permission denied" errors (jQuery #13936)
	if ( (parent = document.defaultView) && parent.top !== parent ) {
		// Support: IE 11
		if ( parent.addEventListener ) {
			parent.addEventListener( "unload", unloadHandler, false );

		// Support: IE 9 - 10 only
		} else if ( parent.attachEvent ) {
			parent.attachEvent( "onunload", unloadHandler );
		}
	}

	/* Attributes
	---------------------------------------------------------------------- */

	// Support: IE<8
	// Verify that getAttribute really returns attributes and not properties
	// (excepting IE8 booleans)
	support.attributes = assert(function( div ) {
		div.className = "i";
		return !div.getAttribute("className");
	});

	/* getElement(s)By*
	---------------------------------------------------------------------- */

	// Check if getElementsByTagName("*") returns only elements
	support.getElementsByTagName = assert(function( div ) {
		div.appendChild( document.createComment("") );
		return !div.getElementsByTagName("*").length;
	});

	// Support: IE<9
	support.getElementsByClassName = rnative.test( document.getElementsByClassName );

	// Support: IE<10
	// Check if getElementById returns elements by name
	// The broken getElementById methods don't pick up programatically-set names,
	// so use a roundabout getElementsByName test
	support.getById = assert(function( div ) {
		docElem.appendChild( div ).id = expando;
		return !document.getElementsByName || !document.getElementsByName( expando ).length;
	});

	// ID find and filter
	if ( support.getById ) {
		Expr.find["ID"] = function( id, context ) {
			if ( typeof context.getElementById !== "undefined" && documentIsHTML ) {
				var m = context.getElementById( id );
				return m ? [ m ] : [];
			}
		};
		Expr.filter["ID"] = function( id ) {
			var attrId = id.replace( runescape, funescape );
			return function( elem ) {
				return elem.getAttribute("id") === attrId;
			};
		};
	} else {
		// Support: IE6/7
		// getElementById is not reliable as a find shortcut
		delete Expr.find["ID"];

		Expr.filter["ID"] =  function( id ) {
			var attrId = id.replace( runescape, funescape );
			return function( elem ) {
				var node = typeof elem.getAttributeNode !== "undefined" &&
					elem.getAttributeNode("id");
				return node && node.value === attrId;
			};
		};
	}

	// Tag
	Expr.find["TAG"] = support.getElementsByTagName ?
		function( tag, context ) {
			if ( typeof context.getElementsByTagName !== "undefined" ) {
				return context.getElementsByTagName( tag );

			// DocumentFragment nodes don't have gEBTN
			} else if ( support.qsa ) {
				return context.querySelectorAll( tag );
			}
		} :

		function( tag, context ) {
			var elem,
				tmp = [],
				i = 0,
				// By happy coincidence, a (broken) gEBTN appears on DocumentFragment nodes too
				results = context.getElementsByTagName( tag );

			// Filter out possible comments
			if ( tag === "*" ) {
				while ( (elem = results[i++]) ) {
					if ( elem.nodeType === 1 ) {
						tmp.push( elem );
					}
				}

				return tmp;
			}
			return results;
		};

	// Class
	Expr.find["CLASS"] = support.getElementsByClassName && function( className, context ) {
		if ( typeof context.getElementsByClassName !== "undefined" && documentIsHTML ) {
			return context.getElementsByClassName( className );
		}
	};

	/* QSA/matchesSelector
	---------------------------------------------------------------------- */

	// QSA and matchesSelector support

	// matchesSelector(:active) reports false when true (IE9/Opera 11.5)
	rbuggyMatches = [];

	// qSa(:focus) reports false when true (Chrome 21)
	// We allow this because of a bug in IE8/9 that throws an error
	// whenever `document.activeElement` is accessed on an iframe
	// So, we allow :focus to pass through QSA all the time to avoid the IE error
	// See http://bugs.jquery.com/ticket/13378
	rbuggyQSA = [];

	if ( (support.qsa = rnative.test( document.querySelectorAll )) ) {
		// Build QSA regex
		// Regex strategy adopted from Diego Perini
		assert(function( div ) {
			// Select is set to empty string on purpose
			// This is to test IE's treatment of not explicitly
			// setting a boolean content attribute,
			// since its presence should be enough
			// http://bugs.jquery.com/ticket/12359
			docElem.appendChild( div ).innerHTML = "<a id='" + expando + "'></a>" +
				"<select id='" + expando + "-\r\\' msallowcapture=''>" +
				"<option selected=''></option></select>";

			// Support: IE8, Opera 11-12.16
			// Nothing should be selected when empty strings follow ^= or $= or *=
			// The test attribute must be unknown in Opera but "safe" for WinRT
			// http://msdn.microsoft.com/en-us/library/ie/hh465388.aspx#attribute_section
			if ( div.querySelectorAll("[msallowcapture^='']").length ) {
				rbuggyQSA.push( "[*^$]=" + whitespace + "*(?:''|\"\")" );
			}

			// Support: IE8
			// Boolean attributes and "value" are not treated correctly
			if ( !div.querySelectorAll("[selected]").length ) {
				rbuggyQSA.push( "\\[" + whitespace + "*(?:value|" + booleans + ")" );
			}

			// Support: Chrome<29, Android<4.4, Safari<7.0+, iOS<7.0+, PhantomJS<1.9.8+
			if ( !div.querySelectorAll( "[id~=" + expando + "-]" ).length ) {
				rbuggyQSA.push("~=");
			}

			// Webkit/Opera - :checked should return selected option elements
			// http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
			// IE8 throws error here and will not see later tests
			if ( !div.querySelectorAll(":checked").length ) {
				rbuggyQSA.push(":checked");
			}

			// Support: Safari 8+, iOS 8+
			// https://bugs.webkit.org/show_bug.cgi?id=136851
			// In-page `selector#id sibing-combinator selector` fails
			if ( !div.querySelectorAll( "a#" + expando + "+*" ).length ) {
				rbuggyQSA.push(".#.+[+~]");
			}
		});

		assert(function( div ) {
			// Support: Windows 8 Native Apps
			// The type and name attributes are restricted during .innerHTML assignment
			var input = document.createElement("input");
			input.setAttribute( "type", "hidden" );
			div.appendChild( input ).setAttribute( "name", "D" );

			// Support: IE8
			// Enforce case-sensitivity of name attribute
			if ( div.querySelectorAll("[name=d]").length ) {
				rbuggyQSA.push( "name" + whitespace + "*[*^$|!~]?=" );
			}

			// FF 3.5 - :enabled/:disabled and hidden elements (hidden elements are still enabled)
			// IE8 throws error here and will not see later tests
			if ( !div.querySelectorAll(":enabled").length ) {
				rbuggyQSA.push( ":enabled", ":disabled" );
			}

			// Opera 10-11 does not throw on post-comma invalid pseudos
			div.querySelectorAll("*,:x");
			rbuggyQSA.push(",.*:");
		});
	}

	if ( (support.matchesSelector = rnative.test( (matches = docElem.matches ||
		docElem.webkitMatchesSelector ||
		docElem.mozMatchesSelector ||
		docElem.oMatchesSelector ||
		docElem.msMatchesSelector) )) ) {

		assert(function( div ) {
			// Check to see if it's possible to do matchesSelector
			// on a disconnected node (IE 9)
			support.disconnectedMatch = matches.call( div, "div" );

			// This should fail with an exception
			// Gecko does not error, returns false instead
			matches.call( div, "[s!='']:x" );
			rbuggyMatches.push( "!=", pseudos );
		});
	}

	rbuggyQSA = rbuggyQSA.length && new RegExp( rbuggyQSA.join("|") );
	rbuggyMatches = rbuggyMatches.length && new RegExp( rbuggyMatches.join("|") );

	/* Contains
	---------------------------------------------------------------------- */
	hasCompare = rnative.test( docElem.compareDocumentPosition );

	// Element contains another
	// Purposefully self-exclusive
	// As in, an element does not contain itself
	contains = hasCompare || rnative.test( docElem.contains ) ?
		function( a, b ) {
			var adown = a.nodeType === 9 ? a.documentElement : a,
				bup = b && b.parentNode;
			return a === bup || !!( bup && bup.nodeType === 1 && (
				adown.contains ?
					adown.contains( bup ) :
					a.compareDocumentPosition && a.compareDocumentPosition( bup ) & 16
			));
		} :
		function( a, b ) {
			if ( b ) {
				while ( (b = b.parentNode) ) {
					if ( b === a ) {
						return true;
					}
				}
			}
			return false;
		};

	/* Sorting
	---------------------------------------------------------------------- */

	// Document order sorting
	sortOrder = hasCompare ?
	function( a, b ) {

		// Flag for duplicate removal
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		// Sort on method existence if only one input has compareDocumentPosition
		var compare = !a.compareDocumentPosition - !b.compareDocumentPosition;
		if ( compare ) {
			return compare;
		}

		// Calculate position if both inputs belong to the same document
		compare = ( a.ownerDocument || a ) === ( b.ownerDocument || b ) ?
			a.compareDocumentPosition( b ) :

			// Otherwise we know they are disconnected
			1;

		// Disconnected nodes
		if ( compare & 1 ||
			(!support.sortDetached && b.compareDocumentPosition( a ) === compare) ) {

			// Choose the first element that is related to our preferred document
			if ( a === document || a.ownerDocument === preferredDoc && contains(preferredDoc, a) ) {
				return -1;
			}
			if ( b === document || b.ownerDocument === preferredDoc && contains(preferredDoc, b) ) {
				return 1;
			}

			// Maintain original order
			return sortInput ?
				( indexOf( sortInput, a ) - indexOf( sortInput, b ) ) :
				0;
		}

		return compare & 4 ? -1 : 1;
	} :
	function( a, b ) {
		// Exit early if the nodes are identical
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		var cur,
			i = 0,
			aup = a.parentNode,
			bup = b.parentNode,
			ap = [ a ],
			bp = [ b ];

		// Parentless nodes are either documents or disconnected
		if ( !aup || !bup ) {
			return a === document ? -1 :
				b === document ? 1 :
				aup ? -1 :
				bup ? 1 :
				sortInput ?
				( indexOf( sortInput, a ) - indexOf( sortInput, b ) ) :
				0;

		// If the nodes are siblings, we can do a quick check
		} else if ( aup === bup ) {
			return siblingCheck( a, b );
		}

		// Otherwise we need full lists of their ancestors for comparison
		cur = a;
		while ( (cur = cur.parentNode) ) {
			ap.unshift( cur );
		}
		cur = b;
		while ( (cur = cur.parentNode) ) {
			bp.unshift( cur );
		}

		// Walk down the tree looking for a discrepancy
		while ( ap[i] === bp[i] ) {
			i++;
		}

		return i ?
			// Do a sibling check if the nodes have a common ancestor
			siblingCheck( ap[i], bp[i] ) :

			// Otherwise nodes in our document sort first
			ap[i] === preferredDoc ? -1 :
			bp[i] === preferredDoc ? 1 :
			0;
	};

	return document;
};

Sizzle.matches = function( expr, elements ) {
	return Sizzle( expr, null, null, elements );
};

Sizzle.matchesSelector = function( elem, expr ) {
	// Set document vars if needed
	if ( ( elem.ownerDocument || elem ) !== document ) {
		setDocument( elem );
	}

	// Make sure that attribute selectors are quoted
	expr = expr.replace( rattributeQuotes, "='$1']" );

	if ( support.matchesSelector && documentIsHTML &&
		!compilerCache[ expr + " " ] &&
		( !rbuggyMatches || !rbuggyMatches.test( expr ) ) &&
		( !rbuggyQSA     || !rbuggyQSA.test( expr ) ) ) {

		try {
			var ret = matches.call( elem, expr );

			// IE 9's matchesSelector returns false on disconnected nodes
			if ( ret || support.disconnectedMatch ||
					// As well, disconnected nodes are said to be in a document
					// fragment in IE 9
					elem.document && elem.document.nodeType !== 11 ) {
				return ret;
			}
		} catch (e) {}
	}

	return Sizzle( expr, document, null, [ elem ] ).length > 0;
};

Sizzle.contains = function( context, elem ) {
	// Set document vars if needed
	if ( ( context.ownerDocument || context ) !== document ) {
		setDocument( context );
	}
	return contains( context, elem );
};

Sizzle.attr = function( elem, name ) {
	// Set document vars if needed
	if ( ( elem.ownerDocument || elem ) !== document ) {
		setDocument( elem );
	}

	var fn = Expr.attrHandle[ name.toLowerCase() ],
		// Don't get fooled by Object.prototype properties (jQuery #13807)
		val = fn && hasOwn.call( Expr.attrHandle, name.toLowerCase() ) ?
			fn( elem, name, !documentIsHTML ) :
			undefined;

	return val !== undefined ?
		val :
		support.attributes || !documentIsHTML ?
			elem.getAttribute( name ) :
			(val = elem.getAttributeNode(name)) && val.specified ?
				val.value :
				null;
};

Sizzle.error = function( msg ) {
	throw new Error( "Syntax error, unrecognized expression: " + msg );
};

/**
 * Document sorting and removing duplicates
 * @param {ArrayLike} results
 */
Sizzle.uniqueSort = function( results ) {
	var elem,
		duplicates = [],
		j = 0,
		i = 0;

	// Unless we *know* we can detect duplicates, assume their presence
	hasDuplicate = !support.detectDuplicates;
	sortInput = !support.sortStable && results.slice( 0 );
	results.sort( sortOrder );

	if ( hasDuplicate ) {
		while ( (elem = results[i++]) ) {
			if ( elem === results[ i ] ) {
				j = duplicates.push( i );
			}
		}
		while ( j-- ) {
			results.splice( duplicates[ j ], 1 );
		}
	}

	// Clear input after sorting to release objects
	// See https://github.com/jquery/sizzle/pull/225
	sortInput = null;

	return results;
};

/**
 * Utility function for retrieving the text value of an array of DOM nodes
 * @param {Array|Element} elem
 */
getText = Sizzle.getText = function( elem ) {
	var node,
		ret = "",
		i = 0,
		nodeType = elem.nodeType;

	if ( !nodeType ) {
		// If no nodeType, this is expected to be an array
		while ( (node = elem[i++]) ) {
			// Do not traverse comment nodes
			ret += getText( node );
		}
	} else if ( nodeType === 1 || nodeType === 9 || nodeType === 11 ) {
		// Use textContent for elements
		// innerText usage removed for consistency of new lines (jQuery #11153)
		if ( typeof elem.textContent === "string" ) {
			return elem.textContent;
		} else {
			// Traverse its children
			for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
				ret += getText( elem );
			}
		}
	} else if ( nodeType === 3 || nodeType === 4 ) {
		return elem.nodeValue;
	}
	// Do not include comment or processing instruction nodes

	return ret;
};

Expr = Sizzle.selectors = {

	// Can be adjusted by the user
	cacheLength: 50,

	createPseudo: markFunction,

	match: matchExpr,

	attrHandle: {},

	find: {},

	relative: {
		">": { dir: "parentNode", first: true },
		" ": { dir: "parentNode" },
		"+": { dir: "previousSibling", first: true },
		"~": { dir: "previousSibling" }
	},

	preFilter: {
		"ATTR": function( match ) {
			match[1] = match[1].replace( runescape, funescape );

			// Move the given value to match[3] whether quoted or unquoted
			match[3] = ( match[3] || match[4] || match[5] || "" ).replace( runescape, funescape );

			if ( match[2] === "~=" ) {
				match[3] = " " + match[3] + " ";
			}

			return match.slice( 0, 4 );
		},

		"CHILD": function( match ) {
			/* matches from matchExpr["CHILD"]
				1 type (only|nth|...)
				2 what (child|of-type)
				3 argument (even|odd|\d*|\d*n([+-]\d+)?|...)
				4 xn-component of xn+y argument ([+-]?\d*n|)
				5 sign of xn-component
				6 x of xn-component
				7 sign of y-component
				8 y of y-component
			*/
			match[1] = match[1].toLowerCase();

			if ( match[1].slice( 0, 3 ) === "nth" ) {
				// nth-* requires argument
				if ( !match[3] ) {
					Sizzle.error( match[0] );
				}

				// numeric x and y parameters for Expr.filter.CHILD
				// remember that false/true cast respectively to 0/1
				match[4] = +( match[4] ? match[5] + (match[6] || 1) : 2 * ( match[3] === "even" || match[3] === "odd" ) );
				match[5] = +( ( match[7] + match[8] ) || match[3] === "odd" );

			// other types prohibit arguments
			} else if ( match[3] ) {
				Sizzle.error( match[0] );
			}

			return match;
		},

		"PSEUDO": function( match ) {
			var excess,
				unquoted = !match[6] && match[2];

			if ( matchExpr["CHILD"].test( match[0] ) ) {
				return null;
			}

			// Accept quoted arguments as-is
			if ( match[3] ) {
				match[2] = match[4] || match[5] || "";

			// Strip excess characters from unquoted arguments
			} else if ( unquoted && rpseudo.test( unquoted ) &&
				// Get excess from tokenize (recursively)
				(excess = tokenize( unquoted, true )) &&
				// advance to the next closing parenthesis
				(excess = unquoted.indexOf( ")", unquoted.length - excess ) - unquoted.length) ) {

				// excess is a negative index
				match[0] = match[0].slice( 0, excess );
				match[2] = unquoted.slice( 0, excess );
			}

			// Return only captures needed by the pseudo filter method (type and argument)
			return match.slice( 0, 3 );
		}
	},

	filter: {

		"TAG": function( nodeNameSelector ) {
			var nodeName = nodeNameSelector.replace( runescape, funescape ).toLowerCase();
			return nodeNameSelector === "*" ?
				function() { return true; } :
				function( elem ) {
					return elem.nodeName && elem.nodeName.toLowerCase() === nodeName;
				};
		},

		"CLASS": function( className ) {
			var pattern = classCache[ className + " " ];

			return pattern ||
				(pattern = new RegExp( "(^|" + whitespace + ")" + className + "(" + whitespace + "|$)" )) &&
				classCache( className, function( elem ) {
					return pattern.test( typeof elem.className === "string" && elem.className || typeof elem.getAttribute !== "undefined" && elem.getAttribute("class") || "" );
				});
		},

		"ATTR": function( name, operator, check ) {
			return function( elem ) {
				var result = Sizzle.attr( elem, name );

				if ( result == null ) {
					return operator === "!=";
				}
				if ( !operator ) {
					return true;
				}

				result += "";

				return operator === "=" ? result === check :
					operator === "!=" ? result !== check :
					operator === "^=" ? check && result.indexOf( check ) === 0 :
					operator === "*=" ? check && result.indexOf( check ) > -1 :
					operator === "$=" ? check && result.slice( -check.length ) === check :
					operator === "~=" ? ( " " + result.replace( rwhitespace, " " ) + " " ).indexOf( check ) > -1 :
					operator === "|=" ? result === check || result.slice( 0, check.length + 1 ) === check + "-" :
					false;
			};
		},

		"CHILD": function( type, what, argument, first, last ) {
			var simple = type.slice( 0, 3 ) !== "nth",
				forward = type.slice( -4 ) !== "last",
				ofType = what === "of-type";

			return first === 1 && last === 0 ?

				// Shortcut for :nth-*(n)
				function( elem ) {
					return !!elem.parentNode;
				} :

				function( elem, context, xml ) {
					var cache, uniqueCache, outerCache, node, nodeIndex, start,
						dir = simple !== forward ? "nextSibling" : "previousSibling",
						parent = elem.parentNode,
						name = ofType && elem.nodeName.toLowerCase(),
						useCache = !xml && !ofType,
						diff = false;

					if ( parent ) {

						// :(first|last|only)-(child|of-type)
						if ( simple ) {
							while ( dir ) {
								node = elem;
								while ( (node = node[ dir ]) ) {
									if ( ofType ?
										node.nodeName.toLowerCase() === name :
										node.nodeType === 1 ) {

										return false;
									}
								}
								// Reverse direction for :only-* (if we haven't yet done so)
								start = dir = type === "only" && !start && "nextSibling";
							}
							return true;
						}

						start = [ forward ? parent.firstChild : parent.lastChild ];

						// non-xml :nth-child(...) stores cache data on `parent`
						if ( forward && useCache ) {

							// Seek `elem` from a previously-cached index

							// ...in a gzip-friendly way
							node = parent;
							outerCache = node[ expando ] || (node[ expando ] = {});

							// Support: IE <9 only
							// Defend against cloned attroperties (jQuery gh-1709)
							uniqueCache = outerCache[ node.uniqueID ] ||
								(outerCache[ node.uniqueID ] = {});

							cache = uniqueCache[ type ] || [];
							nodeIndex = cache[ 0 ] === dirruns && cache[ 1 ];
							diff = nodeIndex && cache[ 2 ];
							node = nodeIndex && parent.childNodes[ nodeIndex ];

							while ( (node = ++nodeIndex && node && node[ dir ] ||

								// Fallback to seeking `elem` from the start
								(diff = nodeIndex = 0) || start.pop()) ) {

								// When found, cache indexes on `parent` and break
								if ( node.nodeType === 1 && ++diff && node === elem ) {
									uniqueCache[ type ] = [ dirruns, nodeIndex, diff ];
									break;
								}
							}

						} else {
							// Use previously-cached element index if available
							if ( useCache ) {
								// ...in a gzip-friendly way
								node = elem;
								outerCache = node[ expando ] || (node[ expando ] = {});

								// Support: IE <9 only
								// Defend against cloned attroperties (jQuery gh-1709)
								uniqueCache = outerCache[ node.uniqueID ] ||
									(outerCache[ node.uniqueID ] = {});

								cache = uniqueCache[ type ] || [];
								nodeIndex = cache[ 0 ] === dirruns && cache[ 1 ];
								diff = nodeIndex;
							}

							// xml :nth-child(...)
							// or :nth-last-child(...) or :nth(-last)?-of-type(...)
							if ( diff === false ) {
								// Use the same loop as above to seek `elem` from the start
								while ( (node = ++nodeIndex && node && node[ dir ] ||
									(diff = nodeIndex = 0) || start.pop()) ) {

									if ( ( ofType ?
										node.nodeName.toLowerCase() === name :
										node.nodeType === 1 ) &&
										++diff ) {

										// Cache the index of each encountered element
										if ( useCache ) {
											outerCache = node[ expando ] || (node[ expando ] = {});

											// Support: IE <9 only
											// Defend against cloned attroperties (jQuery gh-1709)
											uniqueCache = outerCache[ node.uniqueID ] ||
												(outerCache[ node.uniqueID ] = {});

											uniqueCache[ type ] = [ dirruns, diff ];
										}

										if ( node === elem ) {
											break;
										}
									}
								}
							}
						}

						// Incorporate the offset, then check against cycle size
						diff -= last;
						return diff === first || ( diff % first === 0 && diff / first >= 0 );
					}
				};
		},

		"PSEUDO": function( pseudo, argument ) {
			// pseudo-class names are case-insensitive
			// http://www.w3.org/TR/selectors/#pseudo-classes
			// Prioritize by case sensitivity in case custom pseudos are added with uppercase letters
			// Remember that setFilters inherits from pseudos
			var args,
				fn = Expr.pseudos[ pseudo ] || Expr.setFilters[ pseudo.toLowerCase() ] ||
					Sizzle.error( "unsupported pseudo: " + pseudo );

			// The user may use createPseudo to indicate that
			// arguments are needed to create the filter function
			// just as Sizzle does
			if ( fn[ expando ] ) {
				return fn( argument );
			}

			// But maintain support for old signatures
			if ( fn.length > 1 ) {
				args = [ pseudo, pseudo, "", argument ];
				return Expr.setFilters.hasOwnProperty( pseudo.toLowerCase() ) ?
					markFunction(function( seed, matches ) {
						var idx,
							matched = fn( seed, argument ),
							i = matched.length;
						while ( i-- ) {
							idx = indexOf( seed, matched[i] );
							seed[ idx ] = !( matches[ idx ] = matched[i] );
						}
					}) :
					function( elem ) {
						return fn( elem, 0, args );
					};
			}

			return fn;
		}
	},

	pseudos: {
		// Potentially complex pseudos
		"not": markFunction(function( selector ) {
			// Trim the selector passed to compile
			// to avoid treating leading and trailing
			// spaces as combinators
			var input = [],
				results = [],
				matcher = compile( selector.replace( rtrim, "$1" ) );

			return matcher[ expando ] ?
				markFunction(function( seed, matches, context, xml ) {
					var elem,
						unmatched = matcher( seed, null, xml, [] ),
						i = seed.length;

					// Match elements unmatched by `matcher`
					while ( i-- ) {
						if ( (elem = unmatched[i]) ) {
							seed[i] = !(matches[i] = elem);
						}
					}
				}) :
				function( elem, context, xml ) {
					input[0] = elem;
					matcher( input, null, xml, results );
					// Don't keep the element (issue #299)
					input[0] = null;
					return !results.pop();
				};
		}),

		"has": markFunction(function( selector ) {
			return function( elem ) {
				return Sizzle( selector, elem ).length > 0;
			};
		}),

		"contains": markFunction(function( text ) {
			text = text.replace( runescape, funescape );
			return function( elem ) {
				return ( elem.textContent || elem.innerText || getText( elem ) ).indexOf( text ) > -1;
			};
		}),

		// "Whether an element is represented by a :lang() selector
		// is based solely on the element's language value
		// being equal to the identifier C,
		// or beginning with the identifier C immediately followed by "-".
		// The matching of C against the element's language value is performed case-insensitively.
		// The identifier C does not have to be a valid language name."
		// http://www.w3.org/TR/selectors/#lang-pseudo
		"lang": markFunction( function( lang ) {
			// lang value must be a valid identifier
			if ( !ridentifier.test(lang || "") ) {
				Sizzle.error( "unsupported lang: " + lang );
			}
			lang = lang.replace( runescape, funescape ).toLowerCase();
			return function( elem ) {
				var elemLang;
				do {
					if ( (elemLang = documentIsHTML ?
						elem.lang :
						elem.getAttribute("xml:lang") || elem.getAttribute("lang")) ) {

						elemLang = elemLang.toLowerCase();
						return elemLang === lang || elemLang.indexOf( lang + "-" ) === 0;
					}
				} while ( (elem = elem.parentNode) && elem.nodeType === 1 );
				return false;
			};
		}),

		// Miscellaneous
		"target": function( elem ) {
			var hash = window.location && window.location.hash;
			return hash && hash.slice( 1 ) === elem.id;
		},

		"root": function( elem ) {
			return elem === docElem;
		},

		"focus": function( elem ) {
			return elem === document.activeElement && (!document.hasFocus || document.hasFocus()) && !!(elem.type || elem.href || ~elem.tabIndex);
		},

		// Boolean properties
		"enabled": function( elem ) {
			return elem.disabled === false;
		},

		"disabled": function( elem ) {
			return elem.disabled === true;
		},

		"checked": function( elem ) {
			// In CSS3, :checked should return both checked and selected elements
			// http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
			var nodeName = elem.nodeName.toLowerCase();
			return (nodeName === "input" && !!elem.checked) || (nodeName === "option" && !!elem.selected);
		},

		"selected": function( elem ) {
			// Accessing this property makes selected-by-default
			// options in Safari work properly
			if ( elem.parentNode ) {
				elem.parentNode.selectedIndex;
			}

			return elem.selected === true;
		},

		// Contents
		"empty": function( elem ) {
			// http://www.w3.org/TR/selectors/#empty-pseudo
			// :empty is negated by element (1) or content nodes (text: 3; cdata: 4; entity ref: 5),
			//   but not by others (comment: 8; processing instruction: 7; etc.)
			// nodeType < 6 works because attributes (2) do not appear as children
			for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
				if ( elem.nodeType < 6 ) {
					return false;
				}
			}
			return true;
		},

		"parent": function( elem ) {
			return !Expr.pseudos["empty"]( elem );
		},

		// Element/input types
		"header": function( elem ) {
			return rheader.test( elem.nodeName );
		},

		"input": function( elem ) {
			return rinputs.test( elem.nodeName );
		},

		"button": function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return name === "input" && elem.type === "button" || name === "button";
		},

		"text": function( elem ) {
			var attr;
			return elem.nodeName.toLowerCase() === "input" &&
				elem.type === "text" &&

				// Support: IE<8
				// New HTML5 attribute values (e.g., "search") appear with elem.type === "text"
				( (attr = elem.getAttribute("type")) == null || attr.toLowerCase() === "text" );
		},

		// Position-in-collection
		"first": createPositionalPseudo(function() {
			return [ 0 ];
		}),

		"last": createPositionalPseudo(function( matchIndexes, length ) {
			return [ length - 1 ];
		}),

		"eq": createPositionalPseudo(function( matchIndexes, length, argument ) {
			return [ argument < 0 ? argument + length : argument ];
		}),

		"even": createPositionalPseudo(function( matchIndexes, length ) {
			var i = 0;
			for ( ; i < length; i += 2 ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"odd": createPositionalPseudo(function( matchIndexes, length ) {
			var i = 1;
			for ( ; i < length; i += 2 ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"lt": createPositionalPseudo(function( matchIndexes, length, argument ) {
			var i = argument < 0 ? argument + length : argument;
			for ( ; --i >= 0; ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"gt": createPositionalPseudo(function( matchIndexes, length, argument ) {
			var i = argument < 0 ? argument + length : argument;
			for ( ; ++i < length; ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		})
	}
};

Expr.pseudos["nth"] = Expr.pseudos["eq"];

// Add button/input type pseudos
for ( i in { radio: true, checkbox: true, file: true, password: true, image: true } ) {
	Expr.pseudos[ i ] = createInputPseudo( i );
}
for ( i in { submit: true, reset: true } ) {
	Expr.pseudos[ i ] = createButtonPseudo( i );
}

// Easy API for creating new setFilters
function setFilters() {}
setFilters.prototype = Expr.filters = Expr.pseudos;
Expr.setFilters = new setFilters();

tokenize = Sizzle.tokenize = function( selector, parseOnly ) {
	var matched, match, tokens, type,
		soFar, groups, preFilters,
		cached = tokenCache[ selector + " " ];

	if ( cached ) {
		return parseOnly ? 0 : cached.slice( 0 );
	}

	soFar = selector;
	groups = [];
	preFilters = Expr.preFilter;

	while ( soFar ) {

		// Comma and first run
		if ( !matched || (match = rcomma.exec( soFar )) ) {
			if ( match ) {
				// Don't consume trailing commas as valid
				soFar = soFar.slice( match[0].length ) || soFar;
			}
			groups.push( (tokens = []) );
		}

		matched = false;

		// Combinators
		if ( (match = rcombinators.exec( soFar )) ) {
			matched = match.shift();
			tokens.push({
				value: matched,
				// Cast descendant combinators to space
				type: match[0].replace( rtrim, " " )
			});
			soFar = soFar.slice( matched.length );
		}

		// Filters
		for ( type in Expr.filter ) {
			if ( (match = matchExpr[ type ].exec( soFar )) && (!preFilters[ type ] ||
				(match = preFilters[ type ]( match ))) ) {
				matched = match.shift();
				tokens.push({
					value: matched,
					type: type,
					matches: match
				});
				soFar = soFar.slice( matched.length );
			}
		}

		if ( !matched ) {
			break;
		}
	}

	// Return the length of the invalid excess
	// if we're just parsing
	// Otherwise, throw an error or return tokens
	return parseOnly ?
		soFar.length :
		soFar ?
			Sizzle.error( selector ) :
			// Cache the tokens
			tokenCache( selector, groups ).slice( 0 );
};

function toSelector( tokens ) {
	var i = 0,
		len = tokens.length,
		selector = "";
	for ( ; i < len; i++ ) {
		selector += tokens[i].value;
	}
	return selector;
}

function addCombinator( matcher, combinator, base ) {
	var dir = combinator.dir,
		checkNonElements = base && dir === "parentNode",
		doneName = done++;

	return combinator.first ?
		// Check against closest ancestor/preceding element
		function( elem, context, xml ) {
			while ( (elem = elem[ dir ]) ) {
				if ( elem.nodeType === 1 || checkNonElements ) {
					return matcher( elem, context, xml );
				}
			}
		} :

		// Check against all ancestor/preceding elements
		function( elem, context, xml ) {
			var oldCache, uniqueCache, outerCache,
				newCache = [ dirruns, doneName ];

			// We can't set arbitrary data on XML nodes, so they don't benefit from combinator caching
			if ( xml ) {
				while ( (elem = elem[ dir ]) ) {
					if ( elem.nodeType === 1 || checkNonElements ) {
						if ( matcher( elem, context, xml ) ) {
							return true;
						}
					}
				}
			} else {
				while ( (elem = elem[ dir ]) ) {
					if ( elem.nodeType === 1 || checkNonElements ) {
						outerCache = elem[ expando ] || (elem[ expando ] = {});

						// Support: IE <9 only
						// Defend against cloned attroperties (jQuery gh-1709)
						uniqueCache = outerCache[ elem.uniqueID ] || (outerCache[ elem.uniqueID ] = {});

						if ( (oldCache = uniqueCache[ dir ]) &&
							oldCache[ 0 ] === dirruns && oldCache[ 1 ] === doneName ) {

							// Assign to newCache so results back-propagate to previous elements
							return (newCache[ 2 ] = oldCache[ 2 ]);
						} else {
							// Reuse newcache so results back-propagate to previous elements
							uniqueCache[ dir ] = newCache;

							// A match means we're done; a fail means we have to keep checking
							if ( (newCache[ 2 ] = matcher( elem, context, xml )) ) {
								return true;
							}
						}
					}
				}
			}
		};
}

function elementMatcher( matchers ) {
	return matchers.length > 1 ?
		function( elem, context, xml ) {
			var i = matchers.length;
			while ( i-- ) {
				if ( !matchers[i]( elem, context, xml ) ) {
					return false;
				}
			}
			return true;
		} :
		matchers[0];
}

function multipleContexts( selector, contexts, results ) {
	var i = 0,
		len = contexts.length;
	for ( ; i < len; i++ ) {
		Sizzle( selector, contexts[i], results );
	}
	return results;
}

function condense( unmatched, map, filter, context, xml ) {
	var elem,
		newUnmatched = [],
		i = 0,
		len = unmatched.length,
		mapped = map != null;

	for ( ; i < len; i++ ) {
		if ( (elem = unmatched[i]) ) {
			if ( !filter || filter( elem, context, xml ) ) {
				newUnmatched.push( elem );
				if ( mapped ) {
					map.push( i );
				}
			}
		}
	}

	return newUnmatched;
}

function setMatcher( preFilter, selector, matcher, postFilter, postFinder, postSelector ) {
	if ( postFilter && !postFilter[ expando ] ) {
		postFilter = setMatcher( postFilter );
	}
	if ( postFinder && !postFinder[ expando ] ) {
		postFinder = setMatcher( postFinder, postSelector );
	}
	return markFunction(function( seed, results, context, xml ) {
		var temp, i, elem,
			preMap = [],
			postMap = [],
			preexisting = results.length,

			// Get initial elements from seed or context
			elems = seed || multipleContexts( selector || "*", context.nodeType ? [ context ] : context, [] ),

			// Prefilter to get matcher input, preserving a map for seed-results synchronization
			matcherIn = preFilter && ( seed || !selector ) ?
				condense( elems, preMap, preFilter, context, xml ) :
				elems,

			matcherOut = matcher ?
				// If we have a postFinder, or filtered seed, or non-seed postFilter or preexisting results,
				postFinder || ( seed ? preFilter : preexisting || postFilter ) ?

					// ...intermediate processing is necessary
					[] :

					// ...otherwise use results directly
					results :
				matcherIn;

		// Find primary matches
		if ( matcher ) {
			matcher( matcherIn, matcherOut, context, xml );
		}

		// Apply postFilter
		if ( postFilter ) {
			temp = condense( matcherOut, postMap );
			postFilter( temp, [], context, xml );

			// Un-match failing elements by moving them back to matcherIn
			i = temp.length;
			while ( i-- ) {
				if ( (elem = temp[i]) ) {
					matcherOut[ postMap[i] ] = !(matcherIn[ postMap[i] ] = elem);
				}
			}
		}

		if ( seed ) {
			if ( postFinder || preFilter ) {
				if ( postFinder ) {
					// Get the final matcherOut by condensing this intermediate into postFinder contexts
					temp = [];
					i = matcherOut.length;
					while ( i-- ) {
						if ( (elem = matcherOut[i]) ) {
							// Restore matcherIn since elem is not yet a final match
							temp.push( (matcherIn[i] = elem) );
						}
					}
					postFinder( null, (matcherOut = []), temp, xml );
				}

				// Move matched elements from seed to results to keep them synchronized
				i = matcherOut.length;
				while ( i-- ) {
					if ( (elem = matcherOut[i]) &&
						(temp = postFinder ? indexOf( seed, elem ) : preMap[i]) > -1 ) {

						seed[temp] = !(results[temp] = elem);
					}
				}
			}

		// Add elements to results, through postFinder if defined
		} else {
			matcherOut = condense(
				matcherOut === results ?
					matcherOut.splice( preexisting, matcherOut.length ) :
					matcherOut
			);
			if ( postFinder ) {
				postFinder( null, results, matcherOut, xml );
			} else {
				push.apply( results, matcherOut );
			}
		}
	});
}

function matcherFromTokens( tokens ) {
	var checkContext, matcher, j,
		len = tokens.length,
		leadingRelative = Expr.relative[ tokens[0].type ],
		implicitRelative = leadingRelative || Expr.relative[" "],
		i = leadingRelative ? 1 : 0,

		// The foundational matcher ensures that elements are reachable from top-level context(s)
		matchContext = addCombinator( function( elem ) {
			return elem === checkContext;
		}, implicitRelative, true ),
		matchAnyContext = addCombinator( function( elem ) {
			return indexOf( checkContext, elem ) > -1;
		}, implicitRelative, true ),
		matchers = [ function( elem, context, xml ) {
			var ret = ( !leadingRelative && ( xml || context !== outermostContext ) ) || (
				(checkContext = context).nodeType ?
					matchContext( elem, context, xml ) :
					matchAnyContext( elem, context, xml ) );
			// Avoid hanging onto element (issue #299)
			checkContext = null;
			return ret;
		} ];

	for ( ; i < len; i++ ) {
		if ( (matcher = Expr.relative[ tokens[i].type ]) ) {
			matchers = [ addCombinator(elementMatcher( matchers ), matcher) ];
		} else {
			matcher = Expr.filter[ tokens[i].type ].apply( null, tokens[i].matches );

			// Return special upon seeing a positional matcher
			if ( matcher[ expando ] ) {
				// Find the next relative operator (if any) for proper handling
				j = ++i;
				for ( ; j < len; j++ ) {
					if ( Expr.relative[ tokens[j].type ] ) {
						break;
					}
				}
				return setMatcher(
					i > 1 && elementMatcher( matchers ),
					i > 1 && toSelector(
						// If the preceding token was a descendant combinator, insert an implicit any-element `*`
						tokens.slice( 0, i - 1 ).concat({ value: tokens[ i - 2 ].type === " " ? "*" : "" })
					).replace( rtrim, "$1" ),
					matcher,
					i < j && matcherFromTokens( tokens.slice( i, j ) ),
					j < len && matcherFromTokens( (tokens = tokens.slice( j )) ),
					j < len && toSelector( tokens )
				);
			}
			matchers.push( matcher );
		}
	}

	return elementMatcher( matchers );
}

function matcherFromGroupMatchers( elementMatchers, setMatchers ) {
	var bySet = setMatchers.length > 0,
		byElement = elementMatchers.length > 0,
		superMatcher = function( seed, context, xml, results, outermost ) {
			var elem, j, matcher,
				matchedCount = 0,
				i = "0",
				unmatched = seed && [],
				setMatched = [],
				contextBackup = outermostContext,
				// We must always have either seed elements or outermost context
				elems = seed || byElement && Expr.find["TAG"]( "*", outermost ),
				// Use integer dirruns iff this is the outermost matcher
				dirrunsUnique = (dirruns += contextBackup == null ? 1 : Math.random() || 0.1),
				len = elems.length;

			if ( outermost ) {
				outermostContext = context === document || context || outermost;
			}

			// Add elements passing elementMatchers directly to results
			// Support: IE<9, Safari
			// Tolerate NodeList properties (IE: "length"; Safari: <number>) matching elements by id
			for ( ; i !== len && (elem = elems[i]) != null; i++ ) {
				if ( byElement && elem ) {
					j = 0;
					if ( !context && elem.ownerDocument !== document ) {
						setDocument( elem );
						xml = !documentIsHTML;
					}
					while ( (matcher = elementMatchers[j++]) ) {
						if ( matcher( elem, context || document, xml) ) {
							results.push( elem );
							break;
						}
					}
					if ( outermost ) {
						dirruns = dirrunsUnique;
					}
				}

				// Track unmatched elements for set filters
				if ( bySet ) {
					// They will have gone through all possible matchers
					if ( (elem = !matcher && elem) ) {
						matchedCount--;
					}

					// Lengthen the array for every element, matched or not
					if ( seed ) {
						unmatched.push( elem );
					}
				}
			}

			// `i` is now the count of elements visited above, and adding it to `matchedCount`
			// makes the latter nonnegative.
			matchedCount += i;

			// Apply set filters to unmatched elements
			// NOTE: This can be skipped if there are no unmatched elements (i.e., `matchedCount`
			// equals `i`), unless we didn't visit _any_ elements in the above loop because we have
			// no element matchers and no seed.
			// Incrementing an initially-string "0" `i` allows `i` to remain a string only in that
			// case, which will result in a "00" `matchedCount` that differs from `i` but is also
			// numerically zero.
			if ( bySet && i !== matchedCount ) {
				j = 0;
				while ( (matcher = setMatchers[j++]) ) {
					matcher( unmatched, setMatched, context, xml );
				}

				if ( seed ) {
					// Reintegrate element matches to eliminate the need for sorting
					if ( matchedCount > 0 ) {
						while ( i-- ) {
							if ( !(unmatched[i] || setMatched[i]) ) {
								setMatched[i] = pop.call( results );
							}
						}
					}

					// Discard index placeholder values to get only actual matches
					setMatched = condense( setMatched );
				}

				// Add matches to results
				push.apply( results, setMatched );

				// Seedless set matches succeeding multiple successful matchers stipulate sorting
				if ( outermost && !seed && setMatched.length > 0 &&
					( matchedCount + setMatchers.length ) > 1 ) {

					Sizzle.uniqueSort( results );
				}
			}

			// Override manipulation of globals by nested matchers
			if ( outermost ) {
				dirruns = dirrunsUnique;
				outermostContext = contextBackup;
			}

			return unmatched;
		};

	return bySet ?
		markFunction( superMatcher ) :
		superMatcher;
}

compile = Sizzle.compile = function( selector, match /* Internal Use Only */ ) {
	var i,
		setMatchers = [],
		elementMatchers = [],
		cached = compilerCache[ selector + " " ];

	if ( !cached ) {
		// Generate a function of recursive functions that can be used to check each element
		if ( !match ) {
			match = tokenize( selector );
		}
		i = match.length;
		while ( i-- ) {
			cached = matcherFromTokens( match[i] );
			if ( cached[ expando ] ) {
				setMatchers.push( cached );
			} else {
				elementMatchers.push( cached );
			}
		}

		// Cache the compiled function
		cached = compilerCache( selector, matcherFromGroupMatchers( elementMatchers, setMatchers ) );

		// Save selector and tokenization
		cached.selector = selector;
	}
	return cached;
};

/**
 * A low-level selection function that works with Sizzle's compiled
 *  selector functions
 * @param {String|Function} selector A selector or a pre-compiled
 *  selector function built with Sizzle.compile
 * @param {Element} context
 * @param {Array} [results]
 * @param {Array} [seed] A set of elements to match against
 */
select = Sizzle.select = function( selector, context, results, seed ) {
	var i, tokens, token, type, find,
		compiled = typeof selector === "function" && selector,
		match = !seed && tokenize( (selector = compiled.selector || selector) );

	results = results || [];

	// Try to minimize operations if there is only one selector in the list and no seed
	// (the latter of which guarantees us context)
	if ( match.length === 1 ) {

		// Reduce context if the leading compound selector is an ID
		tokens = match[0] = match[0].slice( 0 );
		if ( tokens.length > 2 && (token = tokens[0]).type === "ID" &&
				support.getById && context.nodeType === 9 && documentIsHTML &&
				Expr.relative[ tokens[1].type ] ) {

			context = ( Expr.find["ID"]( token.matches[0].replace(runescape, funescape), context ) || [] )[0];
			if ( !context ) {
				return results;

			// Precompiled matchers will still verify ancestry, so step up a level
			} else if ( compiled ) {
				context = context.parentNode;
			}

			selector = selector.slice( tokens.shift().value.length );
		}

		// Fetch a seed set for right-to-left matching
		i = matchExpr["needsContext"].test( selector ) ? 0 : tokens.length;
		while ( i-- ) {
			token = tokens[i];

			// Abort if we hit a combinator
			if ( Expr.relative[ (type = token.type) ] ) {
				break;
			}
			if ( (find = Expr.find[ type ]) ) {
				// Search, expanding context for leading sibling combinators
				if ( (seed = find(
					token.matches[0].replace( runescape, funescape ),
					rsibling.test( tokens[0].type ) && testContext( context.parentNode ) || context
				)) ) {

					// If seed is empty or no tokens remain, we can return early
					tokens.splice( i, 1 );
					selector = seed.length && toSelector( tokens );
					if ( !selector ) {
						push.apply( results, seed );
						return results;
					}

					break;
				}
			}
		}
	}

	// Compile and execute a filtering function if one is not provided
	// Provide `match` to avoid retokenization if we modified the selector above
	( compiled || compile( selector, match ) )(
		seed,
		context,
		!documentIsHTML,
		results,
		!context || rsibling.test( selector ) && testContext( context.parentNode ) || context
	);
	return results;
};

// One-time assignments

// Sort stability
support.sortStable = expando.split("").sort( sortOrder ).join("") === expando;

// Support: Chrome 14-35+
// Always assume duplicates if they aren't passed to the comparison function
support.detectDuplicates = !!hasDuplicate;

// Initialize against the default document
setDocument();

// Support: Webkit<537.32 - Safari 6.0.3/Chrome 25 (fixed in Chrome 27)
// Detached nodes confoundingly follow *each other*
support.sortDetached = assert(function( div1 ) {
	// Should return 1, but returns 4 (following)
	return div1.compareDocumentPosition( document.createElement("div") ) & 1;
});

// Support: IE<8
// Prevent attribute/property "interpolation"
// http://msdn.microsoft.com/en-us/library/ms536429%28VS.85%29.aspx
if ( !assert(function( div ) {
	div.innerHTML = "<a href='#'></a>";
	return div.firstChild.getAttribute("href") === "#" ;
}) ) {
	addHandle( "type|href|height|width", function( elem, name, isXML ) {
		if ( !isXML ) {
			return elem.getAttribute( name, name.toLowerCase() === "type" ? 1 : 2 );
		}
	});
}

// Support: IE<9
// Use defaultValue in place of getAttribute("value")
if ( !support.attributes || !assert(function( div ) {
	div.innerHTML = "<input/>";
	div.firstChild.setAttribute( "value", "" );
	return div.firstChild.getAttribute( "value" ) === "";
}) ) {
	addHandle( "value", function( elem, name, isXML ) {
		if ( !isXML && elem.nodeName.toLowerCase() === "input" ) {
			return elem.defaultValue;
		}
	});
}

// Support: IE<9
// Use getAttributeNode to fetch booleans when getAttribute lies
if ( !assert(function( div ) {
	return div.getAttribute("disabled") == null;
}) ) {
	addHandle( booleans, function( elem, name, isXML ) {
		var val;
		if ( !isXML ) {
			return elem[ name ] === true ? name.toLowerCase() :
					(val = elem.getAttributeNode( name )) && val.specified ?
					val.value :
				null;
		}
	});
}

return Sizzle;

})( window );



jQuery.find = Sizzle;
jQuery.expr = Sizzle.selectors;
jQuery.expr[ ":" ] = jQuery.expr.pseudos;
jQuery.uniqueSort = jQuery.unique = Sizzle.uniqueSort;
jQuery.text = Sizzle.getText;
jQuery.isXMLDoc = Sizzle.isXML;
jQuery.contains = Sizzle.contains;



var dir = function( elem, dir, until ) {
	var matched = [],
		truncate = until !== undefined;

	while ( ( elem = elem[ dir ] ) && elem.nodeType !== 9 ) {
		if ( elem.nodeType === 1 ) {
			if ( truncate && jQuery( elem ).is( until ) ) {
				break;
			}
			matched.push( elem );
		}
	}
	return matched;
};


var siblings = function( n, elem ) {
	var matched = [];

	for ( ; n; n = n.nextSibling ) {
		if ( n.nodeType === 1 && n !== elem ) {
			matched.push( n );
		}
	}

	return matched;
};


var rneedsContext = jQuery.expr.match.needsContext;

var rsingleTag = ( /^<([\w-]+)\s*\/?>(?:<\/\1>|)$/ );



var risSimple = /^.[^:#\[\.,]*$/;

// Implement the identical functionality for filter and not
function winnow( elements, qualifier, not ) {
	if ( jQuery.isFunction( qualifier ) ) {
		return jQuery.grep( elements, function( elem, i ) {
			/* jshint -W018 */
			return !!qualifier.call( elem, i, elem ) !== not;
		} );

	}

	if ( qualifier.nodeType ) {
		return jQuery.grep( elements, function( elem ) {
			return ( elem === qualifier ) !== not;
		} );

	}

	if ( typeof qualifier === "string" ) {
		if ( risSimple.test( qualifier ) ) {
			return jQuery.filter( qualifier, elements, not );
		}

		qualifier = jQuery.filter( qualifier, elements );
	}

	return jQuery.grep( elements, function( elem ) {
		return ( indexOf.call( qualifier, elem ) > -1 ) !== not;
	} );
}

jQuery.filter = function( expr, elems, not ) {
	var elem = elems[ 0 ];

	if ( not ) {
		expr = ":not(" + expr + ")";
	}

	return elems.length === 1 && elem.nodeType === 1 ?
		jQuery.find.matchesSelector( elem, expr ) ? [ elem ] : [] :
		jQuery.find.matches( expr, jQuery.grep( elems, function( elem ) {
			return elem.nodeType === 1;
		} ) );
};

jQuery.fn.extend( {
	find: function( selector ) {
		var i,
			len = this.length,
			ret = [],
			self = this;

		if ( typeof selector !== "string" ) {
			return this.pushStack( jQuery( selector ).filter( function() {
				for ( i = 0; i < len; i++ ) {
					if ( jQuery.contains( self[ i ], this ) ) {
						return true;
					}
				}
			} ) );
		}

		for ( i = 0; i < len; i++ ) {
			jQuery.find( selector, self[ i ], ret );
		}

		// Needed because $( selector, context ) becomes $( context ).find( selector )
		ret = this.pushStack( len > 1 ? jQuery.unique( ret ) : ret );
		ret.selector = this.selector ? this.selector + " " + selector : selector;
		return ret;
	},
	filter: function( selector ) {
		return this.pushStack( winnow( this, selector || [], false ) );
	},
	not: function( selector ) {
		return this.pushStack( winnow( this, selector || [], true ) );
	},
	is: function( selector ) {
		return !!winnow(
			this,

			// If this is a positional/relative selector, check membership in the returned set
			// so $("p:first").is("p:last") won't return true for a doc with two "p".
			typeof selector === "string" && rneedsContext.test( selector ) ?
				jQuery( selector ) :
				selector || [],
			false
		).length;
	}
} );


// Initialize a jQuery object


// A central reference to the root jQuery(document)
var rootjQuery,

	// A simple way to check for HTML strings
	// Prioritize #id over <tag> to avoid XSS via location.hash (#9521)
	// Strict HTML recognition (#11290: must start with <)
	rquickExpr = /^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]*))$/,

	init = jQuery.fn.init = function( selector, context, root ) {
		var match, elem;

		// HANDLE: $(""), $(null), $(undefined), $(false)
		if ( !selector ) {
			return this;
		}

		// Method init() accepts an alternate rootjQuery
		// so migrate can support jQuery.sub (gh-2101)
		root = root || rootjQuery;

		// Handle HTML strings
		if ( typeof selector === "string" ) {
			if ( selector[ 0 ] === "<" &&
				selector[ selector.length - 1 ] === ">" &&
				selector.length >= 3 ) {

				// Assume that strings that start and end with <> are HTML and skip the regex check
				match = [ null, selector, null ];

			} else {
				match = rquickExpr.exec( selector );
			}

			// Match html or make sure no context is specified for #id
			if ( match && ( match[ 1 ] || !context ) ) {

				// HANDLE: $(html) -> $(array)
				if ( match[ 1 ] ) {
					context = context instanceof jQuery ? context[ 0 ] : context;

					// Option to run scripts is true for back-compat
					// Intentionally let the error be thrown if parseHTML is not present
					jQuery.merge( this, jQuery.parseHTML(
						match[ 1 ],
						context && context.nodeType ? context.ownerDocument || context : document,
						true
					) );

					// HANDLE: $(html, props)
					if ( rsingleTag.test( match[ 1 ] ) && jQuery.isPlainObject( context ) ) {
						for ( match in context ) {

							// Properties of context are called as methods if possible
							if ( jQuery.isFunction( this[ match ] ) ) {
								this[ match ]( context[ match ] );

							// ...and otherwise set as attributes
							} else {
								this.attr( match, context[ match ] );
							}
						}
					}

					return this;

				// HANDLE: $(#id)
				} else {
					elem = document.getElementById( match[ 2 ] );

					// Support: Blackberry 4.6
					// gEBID returns nodes no longer in the document (#6963)
					if ( elem && elem.parentNode ) {

						// Inject the element directly into the jQuery object
						this.length = 1;
						this[ 0 ] = elem;
					}

					this.context = document;
					this.selector = selector;
					return this;
				}

			// HANDLE: $(expr, $(...))
			} else if ( !context || context.jquery ) {
				return ( context || root ).find( selector );

			// HANDLE: $(expr, context)
			// (which is just equivalent to: $(context).find(expr)
			} else {
				return this.constructor( context ).find( selector );
			}

		// HANDLE: $(DOMElement)
		} else if ( selector.nodeType ) {
			this.context = this[ 0 ] = selector;
			this.length = 1;
			return this;

		// HANDLE: $(function)
		// Shortcut for document ready
		} else if ( jQuery.isFunction( selector ) ) {
			return root.ready !== undefined ?
				root.ready( selector ) :

				// Execute immediately if ready is not present
				selector( jQuery );
		}

		if ( selector.selector !== undefined ) {
			this.selector = selector.selector;
			this.context = selector.context;
		}

		return jQuery.makeArray( selector, this );
	};

// Give the init function the jQuery prototype for later instantiation
init.prototype = jQuery.fn;

// Initialize central reference
rootjQuery = jQuery( document );


var rparentsprev = /^(?:parents|prev(?:Until|All))/,

	// Methods guaranteed to produce a unique set when starting from a unique set
	guaranteedUnique = {
		children: true,
		contents: true,
		next: true,
		prev: true
	};

jQuery.fn.extend( {
	has: function( target ) {
		var targets = jQuery( target, this ),
			l = targets.length;

		return this.filter( function() {
			var i = 0;
			for ( ; i < l; i++ ) {
				if ( jQuery.contains( this, targets[ i ] ) ) {
					return true;
				}
			}
		} );
	},

	closest: function( selectors, context ) {
		var cur,
			i = 0,
			l = this.length,
			matched = [],
			pos = rneedsContext.test( selectors ) || typeof selectors !== "string" ?
				jQuery( selectors, context || this.context ) :
				0;

		for ( ; i < l; i++ ) {
			for ( cur = this[ i ]; cur && cur !== context; cur = cur.parentNode ) {

				// Always skip document fragments
				if ( cur.nodeType < 11 && ( pos ?
					pos.index( cur ) > -1 :

					// Don't pass non-elements to Sizzle
					cur.nodeType === 1 &&
						jQuery.find.matchesSelector( cur, selectors ) ) ) {

					matched.push( cur );
					break;
				}
			}
		}

		return this.pushStack( matched.length > 1 ? jQuery.uniqueSort( matched ) : matched );
	},

	// Determine the position of an element within the set
	index: function( elem ) {

		// No argument, return index in parent
		if ( !elem ) {
			return ( this[ 0 ] && this[ 0 ].parentNode ) ? this.first().prevAll().length : -1;
		}

		// Index in selector
		if ( typeof elem === "string" ) {
			return indexOf.call( jQuery( elem ), this[ 0 ] );
		}

		// Locate the position of the desired element
		return indexOf.call( this,

			// If it receives a jQuery object, the first element is used
			elem.jquery ? elem[ 0 ] : elem
		);
	},

	add: function( selector, context ) {
		return this.pushStack(
			jQuery.uniqueSort(
				jQuery.merge( this.get(), jQuery( selector, context ) )
			)
		);
	},

	addBack: function( selector ) {
		return this.add( selector == null ?
			this.prevObject : this.prevObject.filter( selector )
		);
	}
} );

function sibling( cur, dir ) {
	while ( ( cur = cur[ dir ] ) && cur.nodeType !== 1 ) {}
	return cur;
}

jQuery.each( {
	parent: function( elem ) {
		var parent = elem.parentNode;
		return parent && parent.nodeType !== 11 ? parent : null;
	},
	parents: function( elem ) {
		return dir( elem, "parentNode" );
	},
	parentsUntil: function( elem, i, until ) {
		return dir( elem, "parentNode", until );
	},
	next: function( elem ) {
		return sibling( elem, "nextSibling" );
	},
	prev: function( elem ) {
		return sibling( elem, "previousSibling" );
	},
	nextAll: function( elem ) {
		return dir( elem, "nextSibling" );
	},
	prevAll: function( elem ) {
		return dir( elem, "previousSibling" );
	},
	nextUntil: function( elem, i, until ) {
		return dir( elem, "nextSibling", until );
	},
	prevUntil: function( elem, i, until ) {
		return dir( elem, "previousSibling", until );
	},
	siblings: function( elem ) {
		return siblings( ( elem.parentNode || {} ).firstChild, elem );
	},
	children: function( elem ) {
		return siblings( elem.firstChild );
	},
	contents: function( elem ) {
		return elem.contentDocument || jQuery.merge( [], elem.childNodes );
	}
}, function( name, fn ) {
	jQuery.fn[ name ] = function( until, selector ) {
		var matched = jQuery.map( this, fn, until );

		if ( name.slice( -5 ) !== "Until" ) {
			selector = until;
		}

		if ( selector && typeof selector === "string" ) {
			matched = jQuery.filter( selector, matched );
		}

		if ( this.length > 1 ) {

			// Remove duplicates
			if ( !guaranteedUnique[ name ] ) {
				jQuery.uniqueSort( matched );
			}

			// Reverse order for parents* and prev-derivatives
			if ( rparentsprev.test( name ) ) {
				matched.reverse();
			}
		}

		return this.pushStack( matched );
	};
} );
var rnotwhite = ( /\S+/g );



// Convert String-formatted options into Object-formatted ones
function createOptions( options ) {
	var object = {};
	jQuery.each( options.match( rnotwhite ) || [], function( _, flag ) {
		object[ flag ] = true;
	} );
	return object;
}

/*
 * Create a callback list using the following parameters:
 *
 *	options: an optional list of space-separated options that will change how
 *			the callback list behaves or a more traditional option object
 *
 * By default a callback list will act like an event callback list and can be
 * "fired" multiple times.
 *
 * Possible options:
 *
 *	once:			will ensure the callback list can only be fired once (like a Deferred)
 *
 *	memory:			will keep track of previous values and will call any callback added
 *					after the list has been fired right away with the latest "memorized"
 *					values (like a Deferred)
 *
 *	unique:			will ensure a callback can only be added once (no duplicate in the list)
 *
 *	stopOnFalse:	interrupt callings when a callback returns false
 *
 */
jQuery.Callbacks = function( options ) {

	// Convert options from String-formatted to Object-formatted if needed
	// (we check in cache first)
	options = typeof options === "string" ?
		createOptions( options ) :
		jQuery.extend( {}, options );

	var // Flag to know if list is currently firing
		firing,

		// Last fire value for non-forgettable lists
		memory,

		// Flag to know if list was already fired
		fired,

		// Flag to prevent firing
		locked,

		// Actual callback list
		list = [],

		// Queue of execution data for repeatable lists
		queue = [],

		// Index of currently firing callback (modified by add/remove as needed)
		firingIndex = -1,

		// Fire callbacks
		fire = function() {

			// Enforce single-firing
			locked = options.once;

			// Execute callbacks for all pending executions,
			// respecting firingIndex overrides and runtime changes
			fired = firing = true;
			for ( ; queue.length; firingIndex = -1 ) {
				memory = queue.shift();
				while ( ++firingIndex < list.length ) {

					// Run callback and check for early termination
					if ( list[ firingIndex ].apply( memory[ 0 ], memory[ 1 ] ) === false &&
						options.stopOnFalse ) {

						// Jump to end and forget the data so .add doesn't re-fire
						firingIndex = list.length;
						memory = false;
					}
				}
			}

			// Forget the data if we're done with it
			if ( !options.memory ) {
				memory = false;
			}

			firing = false;

			// Clean up if we're done firing for good
			if ( locked ) {

				// Keep an empty list if we have data for future add calls
				if ( memory ) {
					list = [];

				// Otherwise, this object is spent
				} else {
					list = "";
				}
			}
		},

		// Actual Callbacks object
		self = {

			// Add a callback or a collection of callbacks to the list
			add: function() {
				if ( list ) {

					// If we have memory from a past run, we should fire after adding
					if ( memory && !firing ) {
						firingIndex = list.length - 1;
						queue.push( memory );
					}

					( function add( args ) {
						jQuery.each( args, function( _, arg ) {
							if ( jQuery.isFunction( arg ) ) {
								if ( !options.unique || !self.has( arg ) ) {
									list.push( arg );
								}
							} else if ( arg && arg.length && jQuery.type( arg ) !== "string" ) {

								// Inspect recursively
								add( arg );
							}
						} );
					} )( arguments );

					if ( memory && !firing ) {
						fire();
					}
				}
				return this;
			},

			// Remove a callback from the list
			remove: function() {
				jQuery.each( arguments, function( _, arg ) {
					var index;
					while ( ( index = jQuery.inArray( arg, list, index ) ) > -1 ) {
						list.splice( index, 1 );

						// Handle firing indexes
						if ( index <= firingIndex ) {
							firingIndex--;
						}
					}
				} );
				return this;
			},

			// Check if a given callback is in the list.
			// If no argument is given, return whether or not list has callbacks attached.
			has: function( fn ) {
				return fn ?
					jQuery.inArray( fn, list ) > -1 :
					list.length > 0;
			},

			// Remove all callbacks from the list
			empty: function() {
				if ( list ) {
					list = [];
				}
				return this;
			},

			// Disable .fire and .add
			// Abort any current/pending executions
			// Clear all callbacks and values
			disable: function() {
				locked = queue = [];
				list = memory = "";
				return this;
			},
			disabled: function() {
				return !list;
			},

			// Disable .fire
			// Also disable .add unless we have memory (since it would have no effect)
			// Abort any pending executions
			lock: function() {
				locked = queue = [];
				if ( !memory ) {
					list = memory = "";
				}
				return this;
			},
			locked: function() {
				return !!locked;
			},

			// Call all callbacks with the given context and arguments
			fireWith: function( context, args ) {
				if ( !locked ) {
					args = args || [];
					args = [ context, args.slice ? args.slice() : args ];
					queue.push( args );
					if ( !firing ) {
						fire();
					}
				}
				return this;
			},

			// Call all the callbacks with the given arguments
			fire: function() {
				self.fireWith( this, arguments );
				return this;
			},

			// To know if the callbacks have already been called at least once
			fired: function() {
				return !!fired;
			}
		};

	return self;
};


jQuery.extend( {

	Deferred: function( func ) {
		var tuples = [

				// action, add listener, listener list, final state
				[ "resolve", "done", jQuery.Callbacks( "once memory" ), "resolved" ],
				[ "reject", "fail", jQuery.Callbacks( "once memory" ), "rejected" ],
				[ "notify", "progress", jQuery.Callbacks( "memory" ) ]
			],
			state = "pending",
			promise = {
				state: function() {
					return state;
				},
				always: function() {
					deferred.done( arguments ).fail( arguments );
					return this;
				},
				then: function( /* fnDone, fnFail, fnProgress */ ) {
					var fns = arguments;
					return jQuery.Deferred( function( newDefer ) {
						jQuery.each( tuples, function( i, tuple ) {
							var fn = jQuery.isFunction( fns[ i ] ) && fns[ i ];

							// deferred[ done | fail | progress ] for forwarding actions to newDefer
							deferred[ tuple[ 1 ] ]( function() {
								var returned = fn && fn.apply( this, arguments );
								if ( returned && jQuery.isFunction( returned.promise ) ) {
									returned.promise()
										.progress( newDefer.notify )
										.done( newDefer.resolve )
										.fail( newDefer.reject );
								} else {
									newDefer[ tuple[ 0 ] + "With" ](
										this === promise ? newDefer.promise() : this,
										fn ? [ returned ] : arguments
									);
								}
							} );
						} );
						fns = null;
					} ).promise();
				},

				// Get a promise for this deferred
				// If obj is provided, the promise aspect is added to the object
				promise: function( obj ) {
					return obj != null ? jQuery.extend( obj, promise ) : promise;
				}
			},
			deferred = {};

		// Keep pipe for back-compat
		promise.pipe = promise.then;

		// Add list-specific methods
		jQuery.each( tuples, function( i, tuple ) {
			var list = tuple[ 2 ],
				stateString = tuple[ 3 ];

			// promise[ done | fail | progress ] = list.add
			promise[ tuple[ 1 ] ] = list.add;

			// Handle state
			if ( stateString ) {
				list.add( function() {

					// state = [ resolved | rejected ]
					state = stateString;

				// [ reject_list | resolve_list ].disable; progress_list.lock
				}, tuples[ i ^ 1 ][ 2 ].disable, tuples[ 2 ][ 2 ].lock );
			}

			// deferred[ resolve | reject | notify ]
			deferred[ tuple[ 0 ] ] = function() {
				deferred[ tuple[ 0 ] + "With" ]( this === deferred ? promise : this, arguments );
				return this;
			};
			deferred[ tuple[ 0 ] + "With" ] = list.fireWith;
		} );

		// Make the deferred a promise
		promise.promise( deferred );

		// Call given func if any
		if ( func ) {
			func.call( deferred, deferred );
		}

		// All done!
		return deferred;
	},

	// Deferred helper
	when: function( subordinate /* , ..., subordinateN */ ) {
		var i = 0,
			resolveValues = slice.call( arguments ),
			length = resolveValues.length,

			// the count of uncompleted subordinates
			remaining = length !== 1 ||
				( subordinate && jQuery.isFunction( subordinate.promise ) ) ? length : 0,

			// the master Deferred.
			// If resolveValues consist of only a single Deferred, just use that.
			deferred = remaining === 1 ? subordinate : jQuery.Deferred(),

			// Update function for both resolve and progress values
			updateFunc = function( i, contexts, values ) {
				return function( value ) {
					contexts[ i ] = this;
					values[ i ] = arguments.length > 1 ? slice.call( arguments ) : value;
					if ( values === progressValues ) {
						deferred.notifyWith( contexts, values );
					} else if ( !( --remaining ) ) {
						deferred.resolveWith( contexts, values );
					}
				};
			},

			progressValues, progressContexts, resolveContexts;

		// Add listeners to Deferred subordinates; treat others as resolved
		if ( length > 1 ) {
			progressValues = new Array( length );
			progressContexts = new Array( length );
			resolveContexts = new Array( length );
			for ( ; i < length; i++ ) {
				if ( resolveValues[ i ] && jQuery.isFunction( resolveValues[ i ].promise ) ) {
					resolveValues[ i ].promise()
						.progress( updateFunc( i, progressContexts, progressValues ) )
						.done( updateFunc( i, resolveContexts, resolveValues ) )
						.fail( deferred.reject );
				} else {
					--remaining;
				}
			}
		}

		// If we're not waiting on anything, resolve the master
		if ( !remaining ) {
			deferred.resolveWith( resolveContexts, resolveValues );
		}

		return deferred.promise();
	}
} );


// The deferred used on DOM ready
var readyList;

jQuery.fn.ready = function( fn ) {

	// Add the callback
	jQuery.ready.promise().done( fn );

	return this;
};

jQuery.extend( {

	// Is the DOM ready to be used? Set to true once it occurs.
	isReady: false,

	// A counter to track how many items to wait for before
	// the ready event fires. See #6781
	readyWait: 1,

	// Hold (or release) the ready event
	holdReady: function( hold ) {
		if ( hold ) {
			jQuery.readyWait++;
		} else {
			jQuery.ready( true );
		}
	},

	// Handle when the DOM is ready
	ready: function( wait ) {

		// Abort if there are pending holds or we're already ready
		if ( wait === true ? --jQuery.readyWait : jQuery.isReady ) {
			return;
		}

		// Remember that the DOM is ready
		jQuery.isReady = true;

		// If a normal DOM Ready event fired, decrement, and wait if need be
		if ( wait !== true && --jQuery.readyWait > 0 ) {
			return;
		}

		// If there are functions bound, to execute
		readyList.resolveWith( document, [ jQuery ] );

		// Trigger any bound ready events
		if ( jQuery.fn.triggerHandler ) {
			jQuery( document ).triggerHandler( "ready" );
			jQuery( document ).off( "ready" );
		}
	}
} );

/**
 * The ready event handler and self cleanup method
 */
function completed() {
	document.removeEventListener( "DOMContentLoaded", completed );
	window.removeEventListener( "load", completed );
	jQuery.ready();
}

jQuery.ready.promise = function( obj ) {
	if ( !readyList ) {

		readyList = jQuery.Deferred();

		// Catch cases where $(document).ready() is called
		// after the browser event has already occurred.
		// Support: IE9-10 only
		// Older IE sometimes signals "interactive" too soon
		if ( document.readyState === "complete" ||
			( document.readyState !== "loading" && !document.documentElement.doScroll ) ) {

			// Handle it asynchronously to allow scripts the opportunity to delay ready
			window.setTimeout( jQuery.ready );

		} else {

			// Use the handy event callback
			document.addEventListener( "DOMContentLoaded", completed );

			// A fallback to window.onload, that will always work
			window.addEventListener( "load", completed );
		}
	}
	return readyList.promise( obj );
};

// Kick off the DOM ready check even if the user does not
jQuery.ready.promise();




// Multifunctional method to get and set values of a collection
// The value/s can optionally be executed if it's a function
var access = function( elems, fn, key, value, chainable, emptyGet, raw ) {
	var i = 0,
		len = elems.length,
		bulk = key == null;

	// Sets many values
	if ( jQuery.type( key ) === "object" ) {
		chainable = true;
		for ( i in key ) {
			access( elems, fn, i, key[ i ], true, emptyGet, raw );
		}

	// Sets one value
	} else if ( value !== undefined ) {
		chainable = true;

		if ( !jQuery.isFunction( value ) ) {
			raw = true;
		}

		if ( bulk ) {

			// Bulk operations run against the entire set
			if ( raw ) {
				fn.call( elems, value );
				fn = null;

			// ...except when executing function values
			} else {
				bulk = fn;
				fn = function( elem, key, value ) {
					return bulk.call( jQuery( elem ), value );
				};
			}
		}

		if ( fn ) {
			for ( ; i < len; i++ ) {
				fn(
					elems[ i ], key, raw ?
					value :
					value.call( elems[ i ], i, fn( elems[ i ], key ) )
				);
			}
		}
	}

	return chainable ?
		elems :

		// Gets
		bulk ?
			fn.call( elems ) :
			len ? fn( elems[ 0 ], key ) : emptyGet;
};
var acceptData = function( owner ) {

	// Accepts only:
	//  - Node
	//    - Node.ELEMENT_NODE
	//    - Node.DOCUMENT_NODE
	//  - Object
	//    - Any
	/* jshint -W018 */
	return owner.nodeType === 1 || owner.nodeType === 9 || !( +owner.nodeType );
};




function Data() {
	this.expando = jQuery.expando + Data.uid++;
}

Data.uid = 1;

Data.prototype = {

	register: function( owner, initial ) {
		var value = initial || {};

		// If it is a node unlikely to be stringify-ed or looped over
		// use plain assignment
		if ( owner.nodeType ) {
			owner[ this.expando ] = value;

		// Otherwise secure it in a non-enumerable, non-writable property
		// configurability must be true to allow the property to be
		// deleted with the delete operator
		} else {
			Object.defineProperty( owner, this.expando, {
				value: value,
				writable: true,
				configurable: true
			} );
		}
		return owner[ this.expando ];
	},
	cache: function( owner ) {

		// We can accept data for non-element nodes in modern browsers,
		// but we should not, see #8335.
		// Always return an empty object.
		if ( !acceptData( owner ) ) {
			return {};
		}

		// Check if the owner object already has a cache
		var value = owner[ this.expando ];

		// If not, create one
		if ( !value ) {
			value = {};

			// We can accept data for non-element nodes in modern browsers,
			// but we should not, see #8335.
			// Always return an empty object.
			if ( acceptData( owner ) ) {

				// If it is a node unlikely to be stringify-ed or looped over
				// use plain assignment
				if ( owner.nodeType ) {
					owner[ this.expando ] = value;

				// Otherwise secure it in a non-enumerable property
				// configurable must be true to allow the property to be
				// deleted when data is removed
				} else {
					Object.defineProperty( owner, this.expando, {
						value: value,
						configurable: true
					} );
				}
			}
		}

		return value;
	},
	set: function( owner, data, value ) {
		var prop,
			cache = this.cache( owner );

		// Handle: [ owner, key, value ] args
		if ( typeof data === "string" ) {
			cache[ data ] = value;

		// Handle: [ owner, { properties } ] args
		} else {

			// Copy the properties one-by-one to the cache object
			for ( prop in data ) {
				cache[ prop ] = data[ prop ];
			}
		}
		return cache;
	},
	get: function( owner, key ) {
		return key === undefined ?
			this.cache( owner ) :
			owner[ this.expando ] && owner[ this.expando ][ key ];
	},
	access: function( owner, key, value ) {
		var stored;

		// In cases where either:
		//
		//   1. No key was specified
		//   2. A string key was specified, but no value provided
		//
		// Take the "read" path and allow the get method to determine
		// which value to return, respectively either:
		//
		//   1. The entire cache object
		//   2. The data stored at the key
		//
		if ( key === undefined ||
				( ( key && typeof key === "string" ) && value === undefined ) ) {

			stored = this.get( owner, key );

			return stored !== undefined ?
				stored : this.get( owner, jQuery.camelCase( key ) );
		}

		// When the key is not a string, or both a key and value
		// are specified, set or extend (existing objects) with either:
		//
		//   1. An object of properties
		//   2. A key and value
		//
		this.set( owner, key, value );

		// Since the "set" path can have two possible entry points
		// return the expected data based on which path was taken[*]
		return value !== undefined ? value : key;
	},
	remove: function( owner, key ) {
		var i, name, camel,
			cache = owner[ this.expando ];

		if ( cache === undefined ) {
			return;
		}

		if ( key === undefined ) {
			this.register( owner );

		} else {

			// Support array or space separated string of keys
			if ( jQuery.isArray( key ) ) {

				// If "name" is an array of keys...
				// When data is initially created, via ("key", "val") signature,
				// keys will be converted to camelCase.
				// Since there is no way to tell _how_ a key was added, remove
				// both plain key and camelCase key. #12786
				// This will only penalize the array argument path.
				name = key.concat( key.map( jQuery.camelCase ) );
			} else {
				camel = jQuery.camelCase( key );

				// Try the string as a key before any manipulation
				if ( key in cache ) {
					name = [ key, camel ];
				} else {

					// If a key with the spaces exists, use it.
					// Otherwise, create an array by matching non-whitespace
					name = camel;
					name = name in cache ?
						[ name ] : ( name.match( rnotwhite ) || [] );
				}
			}

			i = name.length;

			while ( i-- ) {
				delete cache[ name[ i ] ];
			}
		}

		// Remove the expando if there's no more data
		if ( key === undefined || jQuery.isEmptyObject( cache ) ) {

			// Support: Chrome <= 35-45+
			// Webkit & Blink performance suffers when deleting properties
			// from DOM nodes, so set to undefined instead
			// https://code.google.com/p/chromium/issues/detail?id=378607
			if ( owner.nodeType ) {
				owner[ this.expando ] = undefined;
			} else {
				delete owner[ this.expando ];
			}
		}
	},
	hasData: function( owner ) {
		var cache = owner[ this.expando ];
		return cache !== undefined && !jQuery.isEmptyObject( cache );
	}
};
var dataPriv = new Data();

var dataUser = new Data();



//	Implementation Summary
//
//	1. Enforce API surface and semantic compatibility with 1.9.x branch
//	2. Improve the module's maintainability by reducing the storage
//		paths to a single mechanism.
//	3. Use the same single mechanism to support "private" and "user" data.
//	4. _Never_ expose "private" data to user code (TODO: Drop _data, _removeData)
//	5. Avoid exposing implementation details on user objects (eg. expando properties)
//	6. Provide a clear path for implementation upgrade to WeakMap in 2014

var rbrace = /^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,
	rmultiDash = /[A-Z]/g;

function dataAttr( elem, key, data ) {
	var name;

	// If nothing was found internally, try to fetch any
	// data from the HTML5 data-* attribute
	if ( data === undefined && elem.nodeType === 1 ) {
		name = "data-" + key.replace( rmultiDash, "-$&" ).toLowerCase();
		data = elem.getAttribute( name );

		if ( typeof data === "string" ) {
			try {
				data = data === "true" ? true :
					data === "false" ? false :
					data === "null" ? null :

					// Only convert to a number if it doesn't change the string
					+data + "" === data ? +data :
					rbrace.test( data ) ? jQuery.parseJSON( data ) :
					data;
			} catch ( e ) {}

			// Make sure we set the data so it isn't changed later
			dataUser.set( elem, key, data );
		} else {
			data = undefined;
		}
	}
	return data;
}

jQuery.extend( {
	hasData: function( elem ) {
		return dataUser.hasData( elem ) || dataPriv.hasData( elem );
	},

	data: function( elem, name, data ) {
		return dataUser.access( elem, name, data );
	},

	removeData: function( elem, name ) {
		dataUser.remove( elem, name );
	},

	// TODO: Now that all calls to _data and _removeData have been replaced
	// with direct calls to dataPriv methods, these can be deprecated.
	_data: function( elem, name, data ) {
		return dataPriv.access( elem, name, data );
	},

	_removeData: function( elem, name ) {
		dataPriv.remove( elem, name );
	}
} );

jQuery.fn.extend( {
	data: function( key, value ) {
		var i, name, data,
			elem = this[ 0 ],
			attrs = elem && elem.attributes;

		// Gets all values
		if ( key === undefined ) {
			if ( this.length ) {
				data = dataUser.get( elem );

				if ( elem.nodeType === 1 && !dataPriv.get( elem, "hasDataAttrs" ) ) {
					i = attrs.length;
					while ( i-- ) {

						// Support: IE11+
						// The attrs elements can be null (#14894)
						if ( attrs[ i ] ) {
							name = attrs[ i ].name;
							if ( name.indexOf( "data-" ) === 0 ) {
								name = jQuery.camelCase( name.slice( 5 ) );
								dataAttr( elem, name, data[ name ] );
							}
						}
					}
					dataPriv.set( elem, "hasDataAttrs", true );
				}
			}

			return data;
		}

		// Sets multiple values
		if ( typeof key === "object" ) {
			return this.each( function() {
				dataUser.set( this, key );
			} );
		}

		return access( this, function( value ) {
			var data, camelKey;

			// The calling jQuery object (element matches) is not empty
			// (and therefore has an element appears at this[ 0 ]) and the
			// `value` parameter was not undefined. An empty jQuery object
			// will result in `undefined` for elem = this[ 0 ] which will
			// throw an exception if an attempt to read a data cache is made.
			if ( elem && value === undefined ) {

				// Attempt to get data from the cache
				// with the key as-is
				data = dataUser.get( elem, key ) ||

					// Try to find dashed key if it exists (gh-2779)
					// This is for 2.2.x only
					dataUser.get( elem, key.replace( rmultiDash, "-$&" ).toLowerCase() );

				if ( data !== undefined ) {
					return data;
				}

				camelKey = jQuery.camelCase( key );

				// Attempt to get data from the cache
				// with the key camelized
				data = dataUser.get( elem, camelKey );
				if ( data !== undefined ) {
					return data;
				}

				// Attempt to "discover" the data in
				// HTML5 custom data-* attrs
				data = dataAttr( elem, camelKey, undefined );
				if ( data !== undefined ) {
					return data;
				}

				// We tried really hard, but the data doesn't exist.
				return;
			}

			// Set the data...
			camelKey = jQuery.camelCase( key );
			this.each( function() {

				// First, attempt to store a copy or reference of any
				// data that might've been store with a camelCased key.
				var data = dataUser.get( this, camelKey );

				// For HTML5 data-* attribute interop, we have to
				// store property names with dashes in a camelCase form.
				// This might not apply to all properties...*
				dataUser.set( this, camelKey, value );

				// *... In the case of properties that might _actually_
				// have dashes, we need to also store a copy of that
				// unchanged property.
				if ( key.indexOf( "-" ) > -1 && data !== undefined ) {
					dataUser.set( this, key, value );
				}
			} );
		}, null, value, arguments.length > 1, null, true );
	},

	removeData: function( key ) {
		return this.each( function() {
			dataUser.remove( this, key );
		} );
	}
} );


jQuery.extend( {
	queue: function( elem, type, data ) {
		var queue;

		if ( elem ) {
			type = ( type || "fx" ) + "queue";
			queue = dataPriv.get( elem, type );

			// Speed up dequeue by getting out quickly if this is just a lookup
			if ( data ) {
				if ( !queue || jQuery.isArray( data ) ) {
					queue = dataPriv.access( elem, type, jQuery.makeArray( data ) );
				} else {
					queue.push( data );
				}
			}
			return queue || [];
		}
	},

	dequeue: function( elem, type ) {
		type = type || "fx";

		var queue = jQuery.queue( elem, type ),
			startLength = queue.length,
			fn = queue.shift(),
			hooks = jQuery._queueHooks( elem, type ),
			next = function() {
				jQuery.dequeue( elem, type );
			};

		// If the fx queue is dequeued, always remove the progress sentinel
		if ( fn === "inprogress" ) {
			fn = queue.shift();
			startLength--;
		}

		if ( fn ) {

			// Add a progress sentinel to prevent the fx queue from being
			// automatically dequeued
			if ( type === "fx" ) {
				queue.unshift( "inprogress" );
			}

			// Clear up the last queue stop function
			delete hooks.stop;
			fn.call( elem, next, hooks );
		}

		if ( !startLength && hooks ) {
			hooks.empty.fire();
		}
	},

	// Not public - generate a queueHooks object, or return the current one
	_queueHooks: function( elem, type ) {
		var key = type + "queueHooks";
		return dataPriv.get( elem, key ) || dataPriv.access( elem, key, {
			empty: jQuery.Callbacks( "once memory" ).add( function() {
				dataPriv.remove( elem, [ type + "queue", key ] );
			} )
		} );
	}
} );

jQuery.fn.extend( {
	queue: function( type, data ) {
		var setter = 2;

		if ( typeof type !== "string" ) {
			data = type;
			type = "fx";
			setter--;
		}

		if ( arguments.length < setter ) {
			return jQuery.queue( this[ 0 ], type );
		}

		return data === undefined ?
			this :
			this.each( function() {
				var queue = jQuery.queue( this, type, data );

				// Ensure a hooks for this queue
				jQuery._queueHooks( this, type );

				if ( type === "fx" && queue[ 0 ] !== "inprogress" ) {
					jQuery.dequeue( this, type );
				}
			} );
	},
	dequeue: function( type ) {
		return this.each( function() {
			jQuery.dequeue( this, type );
		} );
	},
	clearQueue: function( type ) {
		return this.queue( type || "fx", [] );
	},

	// Get a promise resolved when queues of a certain type
	// are emptied (fx is the type by default)
	promise: function( type, obj ) {
		var tmp,
			count = 1,
			defer = jQuery.Deferred(),
			elements = this,
			i = this.length,
			resolve = function() {
				if ( !( --count ) ) {
					defer.resolveWith( elements, [ elements ] );
				}
			};

		if ( typeof type !== "string" ) {
			obj = type;
			type = undefined;
		}
		type = type || "fx";

		while ( i-- ) {
			tmp = dataPriv.get( elements[ i ], type + "queueHooks" );
			if ( tmp && tmp.empty ) {
				count++;
				tmp.empty.add( resolve );
			}
		}
		resolve();
		return defer.promise( obj );
	}
} );
var pnum = ( /[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/ ).source;

var rcssNum = new RegExp( "^(?:([+-])=|)(" + pnum + ")([a-z%]*)$", "i" );


var cssExpand = [ "Top", "Right", "Bottom", "Left" ];

var isHidden = function( elem, el ) {

		// isHidden might be called from jQuery#filter function;
		// in that case, element will be second argument
		elem = el || elem;
		return jQuery.css( elem, "display" ) === "none" ||
			!jQuery.contains( elem.ownerDocument, elem );
	};



function adjustCSS( elem, prop, valueParts, tween ) {
	var adjusted,
		scale = 1,
		maxIterations = 20,
		currentValue = tween ?
			function() { return tween.cur(); } :
			function() { return jQuery.css( elem, prop, "" ); },
		initial = currentValue(),
		unit = valueParts && valueParts[ 3 ] || ( jQuery.cssNumber[ prop ] ? "" : "px" ),

		// Starting value computation is required for potential unit mismatches
		initialInUnit = ( jQuery.cssNumber[ prop ] || unit !== "px" && +initial ) &&
			rcssNum.exec( jQuery.css( elem, prop ) );

	if ( initialInUnit && initialInUnit[ 3 ] !== unit ) {

		// Trust units reported by jQuery.css
		unit = unit || initialInUnit[ 3 ];

		// Make sure we update the tween properties later on
		valueParts = valueParts || [];

		// Iteratively approximate from a nonzero starting point
		initialInUnit = +initial || 1;

		do {

			// If previous iteration zeroed out, double until we get *something*.
			// Use string for doubling so we don't accidentally see scale as unchanged below
			scale = scale || ".5";

			// Adjust and apply
			initialInUnit = initialInUnit / scale;
			jQuery.style( elem, prop, initialInUnit + unit );

		// Update scale, tolerating zero or NaN from tween.cur()
		// Break the loop if scale is unchanged or perfect, or if we've just had enough.
		} while (
			scale !== ( scale = currentValue() / initial ) && scale !== 1 && --maxIterations
		);
	}

	if ( valueParts ) {
		initialInUnit = +initialInUnit || +initial || 0;

		// Apply relative offset (+=/-=) if specified
		adjusted = valueParts[ 1 ] ?
			initialInUnit + ( valueParts[ 1 ] + 1 ) * valueParts[ 2 ] :
			+valueParts[ 2 ];
		if ( tween ) {
			tween.unit = unit;
			tween.start = initialInUnit;
			tween.end = adjusted;
		}
	}
	return adjusted;
}
var rcheckableType = ( /^(?:checkbox|radio)$/i );

var rtagName = ( /<([\w:-]+)/ );

var rscriptType = ( /^$|\/(?:java|ecma)script/i );



// We have to close these tags to support XHTML (#13200)
var wrapMap = {

	// Support: IE9
	option: [ 1, "<select multiple='multiple'>", "</select>" ],

	// XHTML parsers do not magically insert elements in the
	// same way that tag soup parsers do. So we cannot shorten
	// this by omitting <tbody> or other required elements.
	thead: [ 1, "<table>", "</table>" ],
	col: [ 2, "<table><colgroup>", "</colgroup></table>" ],
	tr: [ 2, "<table><tbody>", "</tbody></table>" ],
	td: [ 3, "<table><tbody><tr>", "</tr></tbody></table>" ],

	_default: [ 0, "", "" ]
};

// Support: IE9
wrapMap.optgroup = wrapMap.option;

wrapMap.tbody = wrapMap.tfoot = wrapMap.colgroup = wrapMap.caption = wrapMap.thead;
wrapMap.th = wrapMap.td;


function getAll( context, tag ) {

	// Support: IE9-11+
	// Use typeof to avoid zero-argument method invocation on host objects (#15151)
	var ret = typeof context.getElementsByTagName !== "undefined" ?
			context.getElementsByTagName( tag || "*" ) :
			typeof context.querySelectorAll !== "undefined" ?
				context.querySelectorAll( tag || "*" ) :
			[];

	return tag === undefined || tag && jQuery.nodeName( context, tag ) ?
		jQuery.merge( [ context ], ret ) :
		ret;
}


// Mark scripts as having already been evaluated
function setGlobalEval( elems, refElements ) {
	var i = 0,
		l = elems.length;

	for ( ; i < l; i++ ) {
		dataPriv.set(
			elems[ i ],
			"globalEval",
			!refElements || dataPriv.get( refElements[ i ], "globalEval" )
		);
	}
}


var rhtml = /<|&#?\w+;/;

function buildFragment( elems, context, scripts, selection, ignored ) {
	var elem, tmp, tag, wrap, contains, j,
		fragment = context.createDocumentFragment(),
		nodes = [],
		i = 0,
		l = elems.length;

	for ( ; i < l; i++ ) {
		elem = elems[ i ];

		if ( elem || elem === 0 ) {

			// Add nodes directly
			if ( jQuery.type( elem ) === "object" ) {

				// Support: Android<4.1, PhantomJS<2
				// push.apply(_, arraylike) throws on ancient WebKit
				jQuery.merge( nodes, elem.nodeType ? [ elem ] : elem );

			// Convert non-html into a text node
			} else if ( !rhtml.test( elem ) ) {
				nodes.push( context.createTextNode( elem ) );

			// Convert html into DOM nodes
			} else {
				tmp = tmp || fragment.appendChild( context.createElement( "div" ) );

				// Deserialize a standard representation
				tag = ( rtagName.exec( elem ) || [ "", "" ] )[ 1 ].toLowerCase();
				wrap = wrapMap[ tag ] || wrapMap._default;
				tmp.innerHTML = wrap[ 1 ] + jQuery.htmlPrefilter( elem ) + wrap[ 2 ];

				// Descend through wrappers to the right content
				j = wrap[ 0 ];
				while ( j-- ) {
					tmp = tmp.lastChild;
				}

				// Support: Android<4.1, PhantomJS<2
				// push.apply(_, arraylike) throws on ancient WebKit
				jQuery.merge( nodes, tmp.childNodes );

				// Remember the top-level container
				tmp = fragment.firstChild;

				// Ensure the created nodes are orphaned (#12392)
				tmp.textContent = "";
			}
		}
	}

	// Remove wrapper from fragment
	fragment.textContent = "";

	i = 0;
	while ( ( elem = nodes[ i++ ] ) ) {

		// Skip elements already in the context collection (trac-4087)
		if ( selection && jQuery.inArray( elem, selection ) > -1 ) {
			if ( ignored ) {
				ignored.push( elem );
			}
			continue;
		}

		contains = jQuery.contains( elem.ownerDocument, elem );

		// Append to fragment
		tmp = getAll( fragment.appendChild( elem ), "script" );

		// Preserve script evaluation history
		if ( contains ) {
			setGlobalEval( tmp );
		}

		// Capture executables
		if ( scripts ) {
			j = 0;
			while ( ( elem = tmp[ j++ ] ) ) {
				if ( rscriptType.test( elem.type || "" ) ) {
					scripts.push( elem );
				}
			}
		}
	}

	return fragment;
}


( function() {
	var fragment = document.createDocumentFragment(),
		div = fragment.appendChild( document.createElement( "div" ) ),
		input = document.createElement( "input" );

	// Support: Android 4.0-4.3, Safari<=5.1
	// Check state lost if the name is set (#11217)
	// Support: Windows Web Apps (WWA)
	// `name` and `type` must use .setAttribute for WWA (#14901)
	input.setAttribute( "type", "radio" );
	input.setAttribute( "checked", "checked" );
	input.setAttribute( "name", "t" );

	div.appendChild( input );

	// Support: Safari<=5.1, Android<4.2
	// Older WebKit doesn't clone checked state correctly in fragments
	support.checkClone = div.cloneNode( true ).cloneNode( true ).lastChild.checked;

	// Support: IE<=11+
	// Make sure textarea (and checkbox) defaultValue is properly cloned
	div.innerHTML = "<textarea>x</textarea>";
	support.noCloneChecked = !!div.cloneNode( true ).lastChild.defaultValue;
} )();


var
	rkeyEvent = /^key/,
	rmouseEvent = /^(?:mouse|pointer|contextmenu|drag|drop)|click/,
	rtypenamespace = /^([^.]*)(?:\.(.+)|)/;

function returnTrue() {
	return true;
}

function returnFalse() {
	return false;
}

// Support: IE9
// See #13393 for more info
function safeActiveElement() {
	try {
		return document.activeElement;
	} catch ( err ) { }
}

function on( elem, types, selector, data, fn, one ) {
	var origFn, type;

	// Types can be a map of types/handlers
	if ( typeof types === "object" ) {

		// ( types-Object, selector, data )
		if ( typeof selector !== "string" ) {

			// ( types-Object, data )
			data = data || selector;
			selector = undefined;
		}
		for ( type in types ) {
			on( elem, type, selector, data, types[ type ], one );
		}
		return elem;
	}

	if ( data == null && fn == null ) {

		// ( types, fn )
		fn = selector;
		data = selector = undefined;
	} else if ( fn == null ) {
		if ( typeof selector === "string" ) {

			// ( types, selector, fn )
			fn = data;
			data = undefined;
		} else {

			// ( types, data, fn )
			fn = data;
			data = selector;
			selector = undefined;
		}
	}
	if ( fn === false ) {
		fn = returnFalse;
	} else if ( !fn ) {
		return elem;
	}

	if ( one === 1 ) {
		origFn = fn;
		fn = function( event ) {

			// Can use an empty set, since event contains the info
			jQuery().off( event );
			return origFn.apply( this, arguments );
		};

		// Use same guid so caller can remove using origFn
		fn.guid = origFn.guid || ( origFn.guid = jQuery.guid++ );
	}
	return elem.each( function() {
		jQuery.event.add( this, types, fn, data, selector );
	} );
}

/*
 * Helper functions for managing events -- not part of the public interface.
 * Props to Dean Edwards' addEvent library for many of the ideas.
 */
jQuery.event = {

	global: {},

	add: function( elem, types, handler, data, selector ) {

		var handleObjIn, eventHandle, tmp,
			events, t, handleObj,
			special, handlers, type, namespaces, origType,
			elemData = dataPriv.get( elem );

		// Don't attach events to noData or text/comment nodes (but allow plain objects)
		if ( !elemData ) {
			return;
		}

		// Caller can pass in an object of custom data in lieu of the handler
		if ( handler.handler ) {
			handleObjIn = handler;
			handler = handleObjIn.handler;
			selector = handleObjIn.selector;
		}

		// Make sure that the handler has a unique ID, used to find/remove it later
		if ( !handler.guid ) {
			handler.guid = jQuery.guid++;
		}

		// Init the element's event structure and main handler, if this is the first
		if ( !( events = elemData.events ) ) {
			events = elemData.events = {};
		}
		if ( !( eventHandle = elemData.handle ) ) {
			eventHandle = elemData.handle = function( e ) {

				// Discard the second event of a jQuery.event.trigger() and
				// when an event is called after a page has unloaded
				return typeof jQuery !== "undefined" && jQuery.event.triggered !== e.type ?
					jQuery.event.dispatch.apply( elem, arguments ) : undefined;
			};
		}

		// Handle multiple events separated by a space
		types = ( types || "" ).match( rnotwhite ) || [ "" ];
		t = types.length;
		while ( t-- ) {
			tmp = rtypenamespace.exec( types[ t ] ) || [];
			type = origType = tmp[ 1 ];
			namespaces = ( tmp[ 2 ] || "" ).split( "." ).sort();

			// There *must* be a type, no attaching namespace-only handlers
			if ( !type ) {
				continue;
			}

			// If event changes its type, use the special event handlers for the changed type
			special = jQuery.event.special[ type ] || {};

			// If selector defined, determine special event api type, otherwise given type
			type = ( selector ? special.delegateType : special.bindType ) || type;

			// Update special based on newly reset type
			special = jQuery.event.special[ type ] || {};

			// handleObj is passed to all event handlers
			handleObj = jQuery.extend( {
				type: type,
				origType: origType,
				data: data,
				handler: handler,
				guid: handler.guid,
				selector: selector,
				needsContext: selector && jQuery.expr.match.needsContext.test( selector ),
				namespace: namespaces.join( "." )
			}, handleObjIn );

			// Init the event handler queue if we're the first
			if ( !( handlers = events[ type ] ) ) {
				handlers = events[ type ] = [];
				handlers.delegateCount = 0;

				// Only use addEventListener if the special events handler returns false
				if ( !special.setup ||
					special.setup.call( elem, data, namespaces, eventHandle ) === false ) {

					if ( elem.addEventListener ) {
						elem.addEventListener( type, eventHandle );
					}
				}
			}

			if ( special.add ) {
				special.add.call( elem, handleObj );

				if ( !handleObj.handler.guid ) {
					handleObj.handler.guid = handler.guid;
				}
			}

			// Add to the element's handler list, delegates in front
			if ( selector ) {
				handlers.splice( handlers.delegateCount++, 0, handleObj );
			} else {
				handlers.push( handleObj );
			}

			// Keep track of which events have ever been used, for event optimization
			jQuery.event.global[ type ] = true;
		}

	},

	// Detach an event or set of events from an element
	remove: function( elem, types, handler, selector, mappedTypes ) {

		var j, origCount, tmp,
			events, t, handleObj,
			special, handlers, type, namespaces, origType,
			elemData = dataPriv.hasData( elem ) && dataPriv.get( elem );

		if ( !elemData || !( events = elemData.events ) ) {
			return;
		}

		// Once for each type.namespace in types; type may be omitted
		types = ( types || "" ).match( rnotwhite ) || [ "" ];
		t = types.length;
		while ( t-- ) {
			tmp = rtypenamespace.exec( types[ t ] ) || [];
			type = origType = tmp[ 1 ];
			namespaces = ( tmp[ 2 ] || "" ).split( "." ).sort();

			// Unbind all events (on this namespace, if provided) for the element
			if ( !type ) {
				for ( type in events ) {
					jQuery.event.remove( elem, type + types[ t ], handler, selector, true );
				}
				continue;
			}

			special = jQuery.event.special[ type ] || {};
			type = ( selector ? special.delegateType : special.bindType ) || type;
			handlers = events[ type ] || [];
			tmp = tmp[ 2 ] &&
				new RegExp( "(^|\\.)" + namespaces.join( "\\.(?:.*\\.|)" ) + "(\\.|$)" );

			// Remove matching events
			origCount = j = handlers.length;
			while ( j-- ) {
				handleObj = handlers[ j ];

				if ( ( mappedTypes || origType === handleObj.origType ) &&
					( !handler || handler.guid === handleObj.guid ) &&
					( !tmp || tmp.test( handleObj.namespace ) ) &&
					( !selector || selector === handleObj.selector ||
						selector === "**" && handleObj.selector ) ) {
					handlers.splice( j, 1 );

					if ( handleObj.selector ) {
						handlers.delegateCount--;
					}
					if ( special.remove ) {
						special.remove.call( elem, handleObj );
					}
				}
			}

			// Remove generic event handler if we removed something and no more handlers exist
			// (avoids potential for endless recursion during removal of special event handlers)
			if ( origCount && !handlers.length ) {
				if ( !special.teardown ||
					special.teardown.call( elem, namespaces, elemData.handle ) === false ) {

					jQuery.removeEvent( elem, type, elemData.handle );
				}

				delete events[ type ];
			}
		}

		// Remove data and the expando if it's no longer used
		if ( jQuery.isEmptyObject( events ) ) {
			dataPriv.remove( elem, "handle events" );
		}
	},

	dispatch: function( event ) {

		// Make a writable jQuery.Event from the native event object
		event = jQuery.event.fix( event );

		var i, j, ret, matched, handleObj,
			handlerQueue = [],
			args = slice.call( arguments ),
			handlers = ( dataPriv.get( this, "events" ) || {} )[ event.type ] || [],
			special = jQuery.event.special[ event.type ] || {};

		// Use the fix-ed jQuery.Event rather than the (read-only) native event
		args[ 0 ] = event;
		event.delegateTarget = this;

		// Call the preDispatch hook for the mapped type, and let it bail if desired
		if ( special.preDispatch && special.preDispatch.call( this, event ) === false ) {
			return;
		}

		// Determine handlers
		handlerQueue = jQuery.event.handlers.call( this, event, handlers );

		// Run delegates first; they may want to stop propagation beneath us
		i = 0;
		while ( ( matched = handlerQueue[ i++ ] ) && !event.isPropagationStopped() ) {
			event.currentTarget = matched.elem;

			j = 0;
			while ( ( handleObj = matched.handlers[ j++ ] ) &&
				!event.isImmediatePropagationStopped() ) {

				// Triggered event must either 1) have no namespace, or 2) have namespace(s)
				// a subset or equal to those in the bound event (both can have no namespace).
				if ( !event.rnamespace || event.rnamespace.test( handleObj.namespace ) ) {

					event.handleObj = handleObj;
					event.data = handleObj.data;

					ret = ( ( jQuery.event.special[ handleObj.origType ] || {} ).handle ||
						handleObj.handler ).apply( matched.elem, args );

					if ( ret !== undefined ) {
						if ( ( event.result = ret ) === false ) {
							event.preventDefault();
							event.stopPropagation();
						}
					}
				}
			}
		}

		// Call the postDispatch hook for the mapped type
		if ( special.postDispatch ) {
			special.postDispatch.call( this, event );
		}

		return event.result;
	},

	handlers: function( event, handlers ) {
		var i, matches, sel, handleObj,
			handlerQueue = [],
			delegateCount = handlers.delegateCount,
			cur = event.target;

		// Support (at least): Chrome, IE9
		// Find delegate handlers
		// Black-hole SVG <use> instance trees (#13180)
		//
		// Support: Firefox<=42+
		// Avoid non-left-click in FF but don't block IE radio events (#3861, gh-2343)
		if ( delegateCount && cur.nodeType &&
			( event.type !== "click" || isNaN( event.button ) || event.button < 1 ) ) {

			for ( ; cur !== this; cur = cur.parentNode || this ) {

				// Don't check non-elements (#13208)
				// Don't process clicks on disabled elements (#6911, #8165, #11382, #11764)
				if ( cur.nodeType === 1 && ( cur.disabled !== true || event.type !== "click" ) ) {
					matches = [];
					for ( i = 0; i < delegateCount; i++ ) {
						handleObj = handlers[ i ];

						// Don't conflict with Object.prototype properties (#13203)
						sel = handleObj.selector + " ";

						if ( matches[ sel ] === undefined ) {
							matches[ sel ] = handleObj.needsContext ?
								jQuery( sel, this ).index( cur ) > -1 :
								jQuery.find( sel, this, null, [ cur ] ).length;
						}
						if ( matches[ sel ] ) {
							matches.push( handleObj );
						}
					}
					if ( matches.length ) {
						handlerQueue.push( { elem: cur, handlers: matches } );
					}
				}
			}
		}

		// Add the remaining (directly-bound) handlers
		if ( delegateCount < handlers.length ) {
			handlerQueue.push( { elem: this, handlers: handlers.slice( delegateCount ) } );
		}

		return handlerQueue;
	},

	// Includes some event props shared by KeyEvent and MouseEvent
	props: ( "altKey bubbles cancelable ctrlKey currentTarget detail eventPhase " +
		"metaKey relatedTarget shiftKey target timeStamp view which" ).split( " " ),

	fixHooks: {},

	keyHooks: {
		props: "char charCode key keyCode".split( " " ),
		filter: function( event, original ) {

			// Add which for key events
			if ( event.which == null ) {
				event.which = original.charCode != null ? original.charCode : original.keyCode;
			}

			return event;
		}
	},

	mouseHooks: {
		props: ( "button buttons clientX clientY offsetX offsetY pageX pageY " +
			"screenX screenY toElement" ).split( " " ),
		filter: function( event, original ) {
			var eventDoc, doc, body,
				button = original.button;

			// Calculate pageX/Y if missing and clientX/Y available
			if ( event.pageX == null && original.clientX != null ) {
				eventDoc = event.target.ownerDocument || document;
				doc = eventDoc.documentElement;
				body = eventDoc.body;

				event.pageX = original.clientX +
					( doc && doc.scrollLeft || body && body.scrollLeft || 0 ) -
					( doc && doc.clientLeft || body && body.clientLeft || 0 );
				event.pageY = original.clientY +
					( doc && doc.scrollTop  || body && body.scrollTop  || 0 ) -
					( doc && doc.clientTop  || body && body.clientTop  || 0 );
			}

			// Add which for click: 1 === left; 2 === middle; 3 === right
			// Note: button is not normalized, so don't use it
			if ( !event.which && button !== undefined ) {
				event.which = ( button & 1 ? 1 : ( button & 2 ? 3 : ( button & 4 ? 2 : 0 ) ) );
			}

			return event;
		}
	},

	fix: function( event ) {
		if ( event[ jQuery.expando ] ) {
			return event;
		}

		// Create a writable copy of the event object and normalize some properties
		var i, prop, copy,
			type = event.type,
			originalEvent = event,
			fixHook = this.fixHooks[ type ];

		if ( !fixHook ) {
			this.fixHooks[ type ] = fixHook =
				rmouseEvent.test( type ) ? this.mouseHooks :
				rkeyEvent.test( type ) ? this.keyHooks :
				{};
		}
		copy = fixHook.props ? this.props.concat( fixHook.props ) : this.props;

		event = new jQuery.Event( originalEvent );

		i = copy.length;
		while ( i-- ) {
			prop = copy[ i ];
			event[ prop ] = originalEvent[ prop ];
		}

		// Support: Cordova 2.5 (WebKit) (#13255)
		// All events should have a target; Cordova deviceready doesn't
		if ( !event.target ) {
			event.target = document;
		}

		// Support: Safari 6.0+, Chrome<28
		// Target should not be a text node (#504, #13143)
		if ( event.target.nodeType === 3 ) {
			event.target = event.target.parentNode;
		}

		return fixHook.filter ? fixHook.filter( event, originalEvent ) : event;
	},

	special: {
		load: {

			// Prevent triggered image.load events from bubbling to window.load
			noBubble: true
		},
		focus: {

			// Fire native event if possible so blur/focus sequence is correct
			trigger: function() {
				if ( this !== safeActiveElement() && this.focus ) {
					this.focus();
					return false;
				}
			},
			delegateType: "focusin"
		},
		blur: {
			trigger: function() {
				if ( this === safeActiveElement() && this.blur ) {
					this.blur();
					return false;
				}
			},
			delegateType: "focusout"
		},
		click: {

			// For checkbox, fire native event so checked state will be right
			trigger: function() {
				if ( this.type === "checkbox" && this.click && jQuery.nodeName( this, "input" ) ) {
					this.click();
					return false;
				}
			},

			// For cross-browser consistency, don't fire native .click() on links
			_default: function( event ) {
				return jQuery.nodeName( event.target, "a" );
			}
		},

		beforeunload: {
			postDispatch: function( event ) {

				// Support: Firefox 20+
				// Firefox doesn't alert if the returnValue field is not set.
				if ( event.result !== undefined && event.originalEvent ) {
					event.originalEvent.returnValue = event.result;
				}
			}
		}
	}
};

jQuery.removeEvent = function( elem, type, handle ) {

	// This "if" is needed for plain objects
	if ( elem.removeEventListener ) {
		elem.removeEventListener( type, handle );
	}
};

jQuery.Event = function( src, props ) {

	// Allow instantiation without the 'new' keyword
	if ( !( this instanceof jQuery.Event ) ) {
		return new jQuery.Event( src, props );
	}

	// Event object
	if ( src && src.type ) {
		this.originalEvent = src;
		this.type = src.type;

		// Events bubbling up the document may have been marked as prevented
		// by a handler lower down the tree; reflect the correct value.
		this.isDefaultPrevented = src.defaultPrevented ||
				src.defaultPrevented === undefined &&

				// Support: Android<4.0
				src.returnValue === false ?
			returnTrue :
			returnFalse;

	// Event type
	} else {
		this.type = src;
	}

	// Put explicitly provided properties onto the event object
	if ( props ) {
		jQuery.extend( this, props );
	}

	// Create a timestamp if incoming event doesn't have one
	this.timeStamp = src && src.timeStamp || jQuery.now();

	// Mark it as fixed
	this[ jQuery.expando ] = true;
};

// jQuery.Event is based on DOM3 Events as specified by the ECMAScript Language Binding
// http://www.w3.org/TR/2003/WD-DOM-Level-3-Events-20030331/ecma-script-binding.html
jQuery.Event.prototype = {
	constructor: jQuery.Event,
	isDefaultPrevented: returnFalse,
	isPropagationStopped: returnFalse,
	isImmediatePropagationStopped: returnFalse,
	isSimulated: false,

	preventDefault: function() {
		var e = this.originalEvent;

		this.isDefaultPrevented = returnTrue;

		if ( e && !this.isSimulated ) {
			e.preventDefault();
		}
	},
	stopPropagation: function() {
		var e = this.originalEvent;

		this.isPropagationStopped = returnTrue;

		if ( e && !this.isSimulated ) {
			e.stopPropagation();
		}
	},
	stopImmediatePropagation: function() {
		var e = this.originalEvent;

		this.isImmediatePropagationStopped = returnTrue;

		if ( e && !this.isSimulated ) {
			e.stopImmediatePropagation();
		}

		this.stopPropagation();
	}
};

// Create mouseenter/leave events using mouseover/out and event-time checks
// so that event delegation works in jQuery.
// Do the same for pointerenter/pointerleave and pointerover/pointerout
//
// Support: Safari 7 only
// Safari sends mouseenter too often; see:
// https://code.google.com/p/chromium/issues/detail?id=470258
// for the description of the bug (it existed in older Chrome versions as well).
jQuery.each( {
	mouseenter: "mouseover",
	mouseleave: "mouseout",
	pointerenter: "pointerover",
	pointerleave: "pointerout"
}, function( orig, fix ) {
	jQuery.event.special[ orig ] = {
		delegateType: fix,
		bindType: fix,

		handle: function( event ) {
			var ret,
				target = this,
				related = event.relatedTarget,
				handleObj = event.handleObj;

			// For mouseenter/leave call the handler if related is outside the target.
			// NB: No relatedTarget if the mouse left/entered the browser window
			if ( !related || ( related !== target && !jQuery.contains( target, related ) ) ) {
				event.type = handleObj.origType;
				ret = handleObj.handler.apply( this, arguments );
				event.type = fix;
			}
			return ret;
		}
	};
} );

jQuery.fn.extend( {
	on: function( types, selector, data, fn ) {
		return on( this, types, selector, data, fn );
	},
	one: function( types, selector, data, fn ) {
		return on( this, types, selector, data, fn, 1 );
	},
	off: function( types, selector, fn ) {
		var handleObj, type;
		if ( types && types.preventDefault && types.handleObj ) {

			// ( event )  dispatched jQuery.Event
			handleObj = types.handleObj;
			jQuery( types.delegateTarget ).off(
				handleObj.namespace ?
					handleObj.origType + "." + handleObj.namespace :
					handleObj.origType,
				handleObj.selector,
				handleObj.handler
			);
			return this;
		}
		if ( typeof types === "object" ) {

			// ( types-object [, selector] )
			for ( type in types ) {
				this.off( type, selector, types[ type ] );
			}
			return this;
		}
		if ( selector === false || typeof selector === "function" ) {

			// ( types [, fn] )
			fn = selector;
			selector = undefined;
		}
		if ( fn === false ) {
			fn = returnFalse;
		}
		return this.each( function() {
			jQuery.event.remove( this, types, fn, selector );
		} );
	}
} );


var
	rxhtmlTag = /<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:-]+)[^>]*)\/>/gi,

	// Support: IE 10-11, Edge 10240+
	// In IE/Edge using regex groups here causes severe slowdowns.
	// See https://connect.microsoft.com/IE/feedback/details/1736512/
	rnoInnerhtml = /<script|<style|<link/i,

	// checked="checked" or checked
	rchecked = /checked\s*(?:[^=]|=\s*.checked.)/i,
	rscriptTypeMasked = /^true\/(.*)/,
	rcleanScript = /^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;

// Manipulating tables requires a tbody
function manipulationTarget( elem, content ) {
	return jQuery.nodeName( elem, "table" ) &&
		jQuery.nodeName( content.nodeType !== 11 ? content : content.firstChild, "tr" ) ?

		elem.getElementsByTagName( "tbody" )[ 0 ] ||
			elem.appendChild( elem.ownerDocument.createElement( "tbody" ) ) :
		elem;
}

// Replace/restore the type attribute of script elements for safe DOM manipulation
function disableScript( elem ) {
	elem.type = ( elem.getAttribute( "type" ) !== null ) + "/" + elem.type;
	return elem;
}
function restoreScript( elem ) {
	var match = rscriptTypeMasked.exec( elem.type );

	if ( match ) {
		elem.type = match[ 1 ];
	} else {
		elem.removeAttribute( "type" );
	}

	return elem;
}

function cloneCopyEvent( src, dest ) {
	var i, l, type, pdataOld, pdataCur, udataOld, udataCur, events;

	if ( dest.nodeType !== 1 ) {
		return;
	}

	// 1. Copy private data: events, handlers, etc.
	if ( dataPriv.hasData( src ) ) {
		pdataOld = dataPriv.access( src );
		pdataCur = dataPriv.set( dest, pdataOld );
		events = pdataOld.events;

		if ( events ) {
			delete pdataCur.handle;
			pdataCur.events = {};

			for ( type in events ) {
				for ( i = 0, l = events[ type ].length; i < l; i++ ) {
					jQuery.event.add( dest, type, events[ type ][ i ] );
				}
			}
		}
	}

	// 2. Copy user data
	if ( dataUser.hasData( src ) ) {
		udataOld = dataUser.access( src );
		udataCur = jQuery.extend( {}, udataOld );

		dataUser.set( dest, udataCur );
	}
}

// Fix IE bugs, see support tests
function fixInput( src, dest ) {
	var nodeName = dest.nodeName.toLowerCase();

	// Fails to persist the checked state of a cloned checkbox or radio button.
	if ( nodeName === "input" && rcheckableType.test( src.type ) ) {
		dest.checked = src.checked;

	// Fails to return the selected option to the default selected state when cloning options
	} else if ( nodeName === "input" || nodeName === "textarea" ) {
		dest.defaultValue = src.defaultValue;
	}
}

function domManip( collection, args, callback, ignored ) {

	// Flatten any nested arrays
	args = concat.apply( [], args );

	var fragment, first, scripts, hasScripts, node, doc,
		i = 0,
		l = collection.length,
		iNoClone = l - 1,
		value = args[ 0 ],
		isFunction = jQuery.isFunction( value );

	// We can't cloneNode fragments that contain checked, in WebKit
	if ( isFunction ||
			( l > 1 && typeof value === "string" &&
				!support.checkClone && rchecked.test( value ) ) ) {
		return collection.each( function( index ) {
			var self = collection.eq( index );
			if ( isFunction ) {
				args[ 0 ] = value.call( this, index, self.html() );
			}
			domManip( self, args, callback, ignored );
		} );
	}

	if ( l ) {
		fragment = buildFragment( args, collection[ 0 ].ownerDocument, false, collection, ignored );
		first = fragment.firstChild;

		if ( fragment.childNodes.length === 1 ) {
			fragment = first;
		}

		// Require either new content or an interest in ignored elements to invoke the callback
		if ( first || ignored ) {
			scripts = jQuery.map( getAll( fragment, "script" ), disableScript );
			hasScripts = scripts.length;

			// Use the original fragment for the last item
			// instead of the first because it can end up
			// being emptied incorrectly in certain situations (#8070).
			for ( ; i < l; i++ ) {
				node = fragment;

				if ( i !== iNoClone ) {
					node = jQuery.clone( node, true, true );

					// Keep references to cloned scripts for later restoration
					if ( hasScripts ) {

						// Support: Android<4.1, PhantomJS<2
						// push.apply(_, arraylike) throws on ancient WebKit
						jQuery.merge( scripts, getAll( node, "script" ) );
					}
				}

				callback.call( collection[ i ], node, i );
			}

			if ( hasScripts ) {
				doc = scripts[ scripts.length - 1 ].ownerDocument;

				// Reenable scripts
				jQuery.map( scripts, restoreScript );

				// Evaluate executable scripts on first document insertion
				for ( i = 0; i < hasScripts; i++ ) {
					node = scripts[ i ];
					if ( rscriptType.test( node.type || "" ) &&
						!dataPriv.access( node, "globalEval" ) &&
						jQuery.contains( doc, node ) ) {

						if ( node.src ) {

							// Optional AJAX dependency, but won't run scripts if not present
							if ( jQuery._evalUrl ) {
								jQuery._evalUrl( node.src );
							}
						} else {
							jQuery.globalEval( node.textContent.replace( rcleanScript, "" ) );
						}
					}
				}
			}
		}
	}

	return collection;
}

function remove( elem, selector, keepData ) {
	var node,
		nodes = selector ? jQuery.filter( selector, elem ) : elem,
		i = 0;

	for ( ; ( node = nodes[ i ] ) != null; i++ ) {
		if ( !keepData && node.nodeType === 1 ) {
			jQuery.cleanData( getAll( node ) );
		}

		if ( node.parentNode ) {
			if ( keepData && jQuery.contains( node.ownerDocument, node ) ) {
				setGlobalEval( getAll( node, "script" ) );
			}
			node.parentNode.removeChild( node );
		}
	}

	return elem;
}

jQuery.extend( {
	htmlPrefilter: function( html ) {
		return html.replace( rxhtmlTag, "<$1></$2>" );
	},

	clone: function( elem, dataAndEvents, deepDataAndEvents ) {
		var i, l, srcElements, destElements,
			clone = elem.cloneNode( true ),
			inPage = jQuery.contains( elem.ownerDocument, elem );

		// Fix IE cloning issues
		if ( !support.noCloneChecked && ( elem.nodeType === 1 || elem.nodeType === 11 ) &&
				!jQuery.isXMLDoc( elem ) ) {

			// We eschew Sizzle here for performance reasons: http://jsperf.com/getall-vs-sizzle/2
			destElements = getAll( clone );
			srcElements = getAll( elem );

			for ( i = 0, l = srcElements.length; i < l; i++ ) {
				fixInput( srcElements[ i ], destElements[ i ] );
			}
		}

		// Copy the events from the original to the clone
		if ( dataAndEvents ) {
			if ( deepDataAndEvents ) {
				srcElements = srcElements || getAll( elem );
				destElements = destElements || getAll( clone );

				for ( i = 0, l = srcElements.length; i < l; i++ ) {
					cloneCopyEvent( srcElements[ i ], destElements[ i ] );
				}
			} else {
				cloneCopyEvent( elem, clone );
			}
		}

		// Preserve script evaluation history
		destElements = getAll( clone, "script" );
		if ( destElements.length > 0 ) {
			setGlobalEval( destElements, !inPage && getAll( elem, "script" ) );
		}

		// Return the cloned set
		return clone;
	},

	cleanData: function( elems ) {
		var data, elem, type,
			special = jQuery.event.special,
			i = 0;

		for ( ; ( elem = elems[ i ] ) !== undefined; i++ ) {
			if ( acceptData( elem ) ) {
				if ( ( data = elem[ dataPriv.expando ] ) ) {
					if ( data.events ) {
						for ( type in data.events ) {
							if ( special[ type ] ) {
								jQuery.event.remove( elem, type );

							// This is a shortcut to avoid jQuery.event.remove's overhead
							} else {
								jQuery.removeEvent( elem, type, data.handle );
							}
						}
					}

					// Support: Chrome <= 35-45+
					// Assign undefined instead of using delete, see Data#remove
					elem[ dataPriv.expando ] = undefined;
				}
				if ( elem[ dataUser.expando ] ) {

					// Support: Chrome <= 35-45+
					// Assign undefined instead of using delete, see Data#remove
					elem[ dataUser.expando ] = undefined;
				}
			}
		}
	}
} );

jQuery.fn.extend( {

	// Keep domManip exposed until 3.0 (gh-2225)
	domManip: domManip,

	detach: function( selector ) {
		return remove( this, selector, true );
	},

	remove: function( selector ) {
		return remove( this, selector );
	},

	text: function( value ) {
		return access( this, function( value ) {
			return value === undefined ?
				jQuery.text( this ) :
				this.empty().each( function() {
					if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
						this.textContent = value;
					}
				} );
		}, null, value, arguments.length );
	},

	append: function() {
		return domManip( this, arguments, function( elem ) {
			if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
				var target = manipulationTarget( this, elem );
				target.appendChild( elem );
			}
		} );
	},

	prepend: function() {
		return domManip( this, arguments, function( elem ) {
			if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
				var target = manipulationTarget( this, elem );
				target.insertBefore( elem, target.firstChild );
			}
		} );
	},

	before: function() {
		return domManip( this, arguments, function( elem ) {
			if ( this.parentNode ) {
				this.parentNode.insertBefore( elem, this );
			}
		} );
	},

	after: function() {
		return domManip( this, arguments, function( elem ) {
			if ( this.parentNode ) {
				this.parentNode.insertBefore( elem, this.nextSibling );
			}
		} );
	},

	empty: function() {
		var elem,
			i = 0;

		for ( ; ( elem = this[ i ] ) != null; i++ ) {
			if ( elem.nodeType === 1 ) {

				// Prevent memory leaks
				jQuery.cleanData( getAll( elem, false ) );

				// Remove any remaining nodes
				elem.textContent = "";
			}
		}

		return this;
	},

	clone: function( dataAndEvents, deepDataAndEvents ) {
		dataAndEvents = dataAndEvents == null ? false : dataAndEvents;
		deepDataAndEvents = deepDataAndEvents == null ? dataAndEvents : deepDataAndEvents;

		return this.map( function() {
			return jQuery.clone( this, dataAndEvents, deepDataAndEvents );
		} );
	},

	html: function( value ) {
		return access( this, function( value ) {
			var elem = this[ 0 ] || {},
				i = 0,
				l = this.length;

			if ( value === undefined && elem.nodeType === 1 ) {
				return elem.innerHTML;
			}

			// See if we can take a shortcut and just use innerHTML
			if ( typeof value === "string" && !rnoInnerhtml.test( value ) &&
				!wrapMap[ ( rtagName.exec( value ) || [ "", "" ] )[ 1 ].toLowerCase() ] ) {

				value = jQuery.htmlPrefilter( value );

				try {
					for ( ; i < l; i++ ) {
						elem = this[ i ] || {};

						// Remove element nodes and prevent memory leaks
						if ( elem.nodeType === 1 ) {
							jQuery.cleanData( getAll( elem, false ) );
							elem.innerHTML = value;
						}
					}

					elem = 0;

				// If using innerHTML throws an exception, use the fallback method
				} catch ( e ) {}
			}

			if ( elem ) {
				this.empty().append( value );
			}
		}, null, value, arguments.length );
	},

	replaceWith: function() {
		var ignored = [];

		// Make the changes, replacing each non-ignored context element with the new content
		return domManip( this, arguments, function( elem ) {
			var parent = this.parentNode;

			if ( jQuery.inArray( this, ignored ) < 0 ) {
				jQuery.cleanData( getAll( this ) );
				if ( parent ) {
					parent.replaceChild( elem, this );
				}
			}

		// Force callback invocation
		}, ignored );
	}
} );

jQuery.each( {
	appendTo: "append",
	prependTo: "prepend",
	insertBefore: "before",
	insertAfter: "after",
	replaceAll: "replaceWith"
}, function( name, original ) {
	jQuery.fn[ name ] = function( selector ) {
		var elems,
			ret = [],
			insert = jQuery( selector ),
			last = insert.length - 1,
			i = 0;

		for ( ; i <= last; i++ ) {
			elems = i === last ? this : this.clone( true );
			jQuery( insert[ i ] )[ original ]( elems );

			// Support: QtWebKit
			// .get() because push.apply(_, arraylike) throws
			push.apply( ret, elems.get() );
		}

		return this.pushStack( ret );
	};
} );


var iframe,
	elemdisplay = {

		// Support: Firefox
		// We have to pre-define these values for FF (#10227)
		HTML: "block",
		BODY: "block"
	};

/**
 * Retrieve the actual display of a element
 * @param {String} name nodeName of the element
 * @param {Object} doc Document object
 */

// Called only from within defaultDisplay
function actualDisplay( name, doc ) {
	var elem = jQuery( doc.createElement( name ) ).appendTo( doc.body ),

		display = jQuery.css( elem[ 0 ], "display" );

	// We don't have any data stored on the element,
	// so use "detach" method as fast way to get rid of the element
	elem.detach();

	return display;
}

/**
 * Try to determine the default display value of an element
 * @param {String} nodeName
 */
function defaultDisplay( nodeName ) {
	var doc = document,
		display = elemdisplay[ nodeName ];

	if ( !display ) {
		display = actualDisplay( nodeName, doc );

		// If the simple way fails, read from inside an iframe
		if ( display === "none" || !display ) {

			// Use the already-created iframe if possible
			iframe = ( iframe || jQuery( "<iframe frameborder='0' width='0' height='0'/>" ) )
				.appendTo( doc.documentElement );

			// Always write a new HTML skeleton so Webkit and Firefox don't choke on reuse
			doc = iframe[ 0 ].contentDocument;

			// Support: IE
			doc.write();
			doc.close();

			display = actualDisplay( nodeName, doc );
			iframe.detach();
		}

		// Store the correct default display
		elemdisplay[ nodeName ] = display;
	}

	return display;
}
var rmargin = ( /^margin/ );

var rnumnonpx = new RegExp( "^(" + pnum + ")(?!px)[a-z%]+$", "i" );

var getStyles = function( elem ) {

		// Support: IE<=11+, Firefox<=30+ (#15098, #14150)
		// IE throws on elements created in popups
		// FF meanwhile throws on frame elements through "defaultView.getComputedStyle"
		var view = elem.ownerDocument.defaultView;

		if ( !view || !view.opener ) {
			view = window;
		}

		return view.getComputedStyle( elem );
	};

var swap = function( elem, options, callback, args ) {
	var ret, name,
		old = {};

	// Remember the old values, and insert the new ones
	for ( name in options ) {
		old[ name ] = elem.style[ name ];
		elem.style[ name ] = options[ name ];
	}

	ret = callback.apply( elem, args || [] );

	// Revert the old values
	for ( name in options ) {
		elem.style[ name ] = old[ name ];
	}

	return ret;
};


var documentElement = document.documentElement;



( function() {
	var pixelPositionVal, boxSizingReliableVal, pixelMarginRightVal, reliableMarginLeftVal,
		container = document.createElement( "div" ),
		div = document.createElement( "div" );

	// Finish early in limited (non-browser) environments
	if ( !div.style ) {
		return;
	}

	// Support: IE9-11+
	// Style of cloned element affects source element cloned (#8908)
	div.style.backgroundClip = "content-box";
	div.cloneNode( true ).style.backgroundClip = "";
	support.clearCloneStyle = div.style.backgroundClip === "content-box";

	container.style.cssText = "border:0;width:8px;height:0;top:0;left:-9999px;" +
		"padding:0;margin-top:1px;position:absolute";
	container.appendChild( div );

	// Executing both pixelPosition & boxSizingReliable tests require only one layout
	// so they're executed at the same time to save the second computation.
	function computeStyleTests() {
		div.style.cssText =

			// Support: Firefox<29, Android 2.3
			// Vendor-prefix box-sizing
			"-webkit-box-sizing:border-box;-moz-box-sizing:border-box;box-sizing:border-box;" +
			"position:relative;display:block;" +
			"margin:auto;border:1px;padding:1px;" +
			"top:1%;width:50%";
		div.innerHTML = "";
		documentElement.appendChild( container );

		var divStyle = window.getComputedStyle( div );
		pixelPositionVal = divStyle.top !== "1%";
		reliableMarginLeftVal = divStyle.marginLeft === "2px";
		boxSizingReliableVal = divStyle.width === "4px";

		// Support: Android 4.0 - 4.3 only
		// Some styles come back with percentage values, even though they shouldn't
		div.style.marginRight = "50%";
		pixelMarginRightVal = divStyle.marginRight === "4px";

		documentElement.removeChild( container );
	}

	jQuery.extend( support, {
		pixelPosition: function() {

			// This test is executed only once but we still do memoizing
			// since we can use the boxSizingReliable pre-computing.
			// No need to check if the test was already performed, though.
			computeStyleTests();
			return pixelPositionVal;
		},
		boxSizingReliable: function() {
			if ( boxSizingReliableVal == null ) {
				computeStyleTests();
			}
			return boxSizingReliableVal;
		},
		pixelMarginRight: function() {

			// Support: Android 4.0-4.3
			// We're checking for boxSizingReliableVal here instead of pixelMarginRightVal
			// since that compresses better and they're computed together anyway.
			if ( boxSizingReliableVal == null ) {
				computeStyleTests();
			}
			return pixelMarginRightVal;
		},
		reliableMarginLeft: function() {

			// Support: IE <=8 only, Android 4.0 - 4.3 only, Firefox <=3 - 37
			if ( boxSizingReliableVal == null ) {
				computeStyleTests();
			}
			return reliableMarginLeftVal;
		},
		reliableMarginRight: function() {

			// Support: Android 2.3
			// Check if div with explicit width and no margin-right incorrectly
			// gets computed margin-right based on width of container. (#3333)
			// WebKit Bug 13343 - getComputedStyle returns wrong value for margin-right
			// This support function is only executed once so no memoizing is needed.
			var ret,
				marginDiv = div.appendChild( document.createElement( "div" ) );

			// Reset CSS: box-sizing; display; margin; border; padding
			marginDiv.style.cssText = div.style.cssText =

				// Support: Android 2.3
				// Vendor-prefix box-sizing
				"-webkit-box-sizing:content-box;box-sizing:content-box;" +
				"display:block;margin:0;border:0;padding:0";
			marginDiv.style.marginRight = marginDiv.style.width = "0";
			div.style.width = "1px";
			documentElement.appendChild( container );

			ret = !parseFloat( window.getComputedStyle( marginDiv ).marginRight );

			documentElement.removeChild( container );
			div.removeChild( marginDiv );

			return ret;
		}
	} );
} )();


function curCSS( elem, name, computed ) {
	var width, minWidth, maxWidth, ret,
		style = elem.style;

	computed = computed || getStyles( elem );
	ret = computed ? computed.getPropertyValue( name ) || computed[ name ] : undefined;

	// Support: Opera 12.1x only
	// Fall back to style even without computed
	// computed is undefined for elems on document fragments
	if ( ( ret === "" || ret === undefined ) && !jQuery.contains( elem.ownerDocument, elem ) ) {
		ret = jQuery.style( elem, name );
	}

	// Support: IE9
	// getPropertyValue is only needed for .css('filter') (#12537)
	if ( computed ) {

		// A tribute to the "awesome hack by Dean Edwards"
		// Android Browser returns percentage for some values,
		// but width seems to be reliably pixels.
		// This is against the CSSOM draft spec:
		// http://dev.w3.org/csswg/cssom/#resolved-values
		if ( !support.pixelMarginRight() && rnumnonpx.test( ret ) && rmargin.test( name ) ) {

			// Remember the original values
			width = style.width;
			minWidth = style.minWidth;
			maxWidth = style.maxWidth;

			// Put in the new values to get a computed value out
			style.minWidth = style.maxWidth = style.width = ret;
			ret = computed.width;

			// Revert the changed values
			style.width = width;
			style.minWidth = minWidth;
			style.maxWidth = maxWidth;
		}
	}

	return ret !== undefined ?

		// Support: IE9-11+
		// IE returns zIndex value as an integer.
		ret + "" :
		ret;
}


function addGetHookIf( conditionFn, hookFn ) {

	// Define the hook, we'll check on the first run if it's really needed.
	return {
		get: function() {
			if ( conditionFn() ) {

				// Hook not needed (or it's not possible to use it due
				// to missing dependency), remove it.
				delete this.get;
				return;
			}

			// Hook needed; redefine it so that the support test is not executed again.
			return ( this.get = hookFn ).apply( this, arguments );
		}
	};
}


var

	// Swappable if display is none or starts with table
	// except "table", "table-cell", or "table-caption"
	// See here for display values: https://developer.mozilla.org/en-US/docs/CSS/display
	rdisplayswap = /^(none|table(?!-c[ea]).+)/,

	cssShow = { position: "absolute", visibility: "hidden", display: "block" },
	cssNormalTransform = {
		letterSpacing: "0",
		fontWeight: "400"
	},

	cssPrefixes = [ "Webkit", "O", "Moz", "ms" ],
	emptyStyle = document.createElement( "div" ).style;

// Return a css property mapped to a potentially vendor prefixed property
function vendorPropName( name ) {

	// Shortcut for names that are not vendor prefixed
	if ( name in emptyStyle ) {
		return name;
	}

	// Check for vendor prefixed names
	var capName = name[ 0 ].toUpperCase() + name.slice( 1 ),
		i = cssPrefixes.length;

	while ( i-- ) {
		name = cssPrefixes[ i ] + capName;
		if ( name in emptyStyle ) {
			return name;
		}
	}
}

function setPositiveNumber( elem, value, subtract ) {

	// Any relative (+/-) values have already been
	// normalized at this point
	var matches = rcssNum.exec( value );
	return matches ?

		// Guard against undefined "subtract", e.g., when used as in cssHooks
		Math.max( 0, matches[ 2 ] - ( subtract || 0 ) ) + ( matches[ 3 ] || "px" ) :
		value;
}

function augmentWidthOrHeight( elem, name, extra, isBorderBox, styles ) {
	var i = extra === ( isBorderBox ? "border" : "content" ) ?

		// If we already have the right measurement, avoid augmentation
		4 :

		// Otherwise initialize for horizontal or vertical properties
		name === "width" ? 1 : 0,

		val = 0;

	for ( ; i < 4; i += 2 ) {

		// Both box models exclude margin, so add it if we want it
		if ( extra === "margin" ) {
			val += jQuery.css( elem, extra + cssExpand[ i ], true, styles );
		}

		if ( isBorderBox ) {

			// border-box includes padding, so remove it if we want content
			if ( extra === "content" ) {
				val -= jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );
			}

			// At this point, extra isn't border nor margin, so remove border
			if ( extra !== "margin" ) {
				val -= jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
			}
		} else {

			// At this point, extra isn't content, so add padding
			val += jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );

			// At this point, extra isn't content nor padding, so add border
			if ( extra !== "padding" ) {
				val += jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
			}
		}
	}

	return val;
}

function getWidthOrHeight( elem, name, extra ) {

	// Start with offset property, which is equivalent to the border-box value
	var valueIsBorderBox = true,
		val = name === "width" ? elem.offsetWidth : elem.offsetHeight,
		styles = getStyles( elem ),
		isBorderBox = jQuery.css( elem, "boxSizing", false, styles ) === "border-box";

	// Some non-html elements return undefined for offsetWidth, so check for null/undefined
	// svg - https://bugzilla.mozilla.org/show_bug.cgi?id=649285
	// MathML - https://bugzilla.mozilla.org/show_bug.cgi?id=491668
	if ( val <= 0 || val == null ) {

		// Fall back to computed then uncomputed css if necessary
		val = curCSS( elem, name, styles );
		if ( val < 0 || val == null ) {
			val = elem.style[ name ];
		}

		// Computed unit is not pixels. Stop here and return.
		if ( rnumnonpx.test( val ) ) {
			return val;
		}

		// Check for style in case a browser which returns unreliable values
		// for getComputedStyle silently falls back to the reliable elem.style
		valueIsBorderBox = isBorderBox &&
			( support.boxSizingReliable() || val === elem.style[ name ] );

		// Normalize "", auto, and prepare for extra
		val = parseFloat( val ) || 0;
	}

	// Use the active box-sizing model to add/subtract irrelevant styles
	return ( val +
		augmentWidthOrHeight(
			elem,
			name,
			extra || ( isBorderBox ? "border" : "content" ),
			valueIsBorderBox,
			styles
		)
	) + "px";
}

function showHide( elements, show ) {
	var display, elem, hidden,
		values = [],
		index = 0,
		length = elements.length;

	for ( ; index < length; index++ ) {
		elem = elements[ index ];
		if ( !elem.style ) {
			continue;
		}

		values[ index ] = dataPriv.get( elem, "olddisplay" );
		display = elem.style.display;
		if ( show ) {

			// Reset the inline display of this element to learn if it is
			// being hidden by cascaded rules or not
			if ( !values[ index ] && display === "none" ) {
				elem.style.display = "";
			}

			// Set elements which have been overridden with display: none
			// in a stylesheet to whatever the default browser style is
			// for such an element
			if ( elem.style.display === "" && isHidden( elem ) ) {
				values[ index ] = dataPriv.access(
					elem,
					"olddisplay",
					defaultDisplay( elem.nodeName )
				);
			}
		} else {
			hidden = isHidden( elem );

			if ( display !== "none" || !hidden ) {
				dataPriv.set(
					elem,
					"olddisplay",
					hidden ? display : jQuery.css( elem, "display" )
				);
			}
		}
	}

	// Set the display of most of the elements in a second loop
	// to avoid the constant reflow
	for ( index = 0; index < length; index++ ) {
		elem = elements[ index ];
		if ( !elem.style ) {
			continue;
		}
		if ( !show || elem.style.display === "none" || elem.style.display === "" ) {
			elem.style.display = show ? values[ index ] || "" : "none";
		}
	}

	return elements;
}

jQuery.extend( {

	// Add in style property hooks for overriding the default
	// behavior of getting and setting a style property
	cssHooks: {
		opacity: {
			get: function( elem, computed ) {
				if ( computed ) {

					// We should always get a number back from opacity
					var ret = curCSS( elem, "opacity" );
					return ret === "" ? "1" : ret;
				}
			}
		}
	},

	// Don't automatically add "px" to these possibly-unitless properties
	cssNumber: {
		"animationIterationCount": true,
		"columnCount": true,
		"fillOpacity": true,
		"flexGrow": true,
		"flexShrink": true,
		"fontWeight": true,
		"lineHeight": true,
		"opacity": true,
		"order": true,
		"orphans": true,
		"widows": true,
		"zIndex": true,
		"zoom": true
	},

	// Add in properties whose names you wish to fix before
	// setting or getting the value
	cssProps: {
		"float": "cssFloat"
	},

	// Get and set the style property on a DOM Node
	style: function( elem, name, value, extra ) {

		// Don't set styles on text and comment nodes
		if ( !elem || elem.nodeType === 3 || elem.nodeType === 8 || !elem.style ) {
			return;
		}

		// Make sure that we're working with the right name
		var ret, type, hooks,
			origName = jQuery.camelCase( name ),
			style = elem.style;

		name = jQuery.cssProps[ origName ] ||
			( jQuery.cssProps[ origName ] = vendorPropName( origName ) || origName );

		// Gets hook for the prefixed version, then unprefixed version
		hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

		// Check if we're setting a value
		if ( value !== undefined ) {
			type = typeof value;

			// Convert "+=" or "-=" to relative numbers (#7345)
			if ( type === "string" && ( ret = rcssNum.exec( value ) ) && ret[ 1 ] ) {
				value = adjustCSS( elem, name, ret );

				// Fixes bug #9237
				type = "number";
			}

			// Make sure that null and NaN values aren't set (#7116)
			if ( value == null || value !== value ) {
				return;
			}

			// If a number was passed in, add the unit (except for certain CSS properties)
			if ( type === "number" ) {
				value += ret && ret[ 3 ] || ( jQuery.cssNumber[ origName ] ? "" : "px" );
			}

			// Support: IE9-11+
			// background-* props affect original clone's values
			if ( !support.clearCloneStyle && value === "" && name.indexOf( "background" ) === 0 ) {
				style[ name ] = "inherit";
			}

			// If a hook was provided, use that value, otherwise just set the specified value
			if ( !hooks || !( "set" in hooks ) ||
				( value = hooks.set( elem, value, extra ) ) !== undefined ) {

				style[ name ] = value;
			}

		} else {

			// If a hook was provided get the non-computed value from there
			if ( hooks && "get" in hooks &&
				( ret = hooks.get( elem, false, extra ) ) !== undefined ) {

				return ret;
			}

			// Otherwise just get the value from the style object
			return style[ name ];
		}
	},

	css: function( elem, name, extra, styles ) {
		var val, num, hooks,
			origName = jQuery.camelCase( name );

		// Make sure that we're working with the right name
		name = jQuery.cssProps[ origName ] ||
			( jQuery.cssProps[ origName ] = vendorPropName( origName ) || origName );

		// Try prefixed name followed by the unprefixed name
		hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

		// If a hook was provided get the computed value from there
		if ( hooks && "get" in hooks ) {
			val = hooks.get( elem, true, extra );
		}

		// Otherwise, if a way to get the computed value exists, use that
		if ( val === undefined ) {
			val = curCSS( elem, name, styles );
		}

		// Convert "normal" to computed value
		if ( val === "normal" && name in cssNormalTransform ) {
			val = cssNormalTransform[ name ];
		}

		// Make numeric if forced or a qualifier was provided and val looks numeric
		if ( extra === "" || extra ) {
			num = parseFloat( val );
			return extra === true || isFinite( num ) ? num || 0 : val;
		}
		return val;
	}
} );

jQuery.each( [ "height", "width" ], function( i, name ) {
	jQuery.cssHooks[ name ] = {
		get: function( elem, computed, extra ) {
			if ( computed ) {

				// Certain elements can have dimension info if we invisibly show them
				// but it must have a current display style that would benefit
				return rdisplayswap.test( jQuery.css( elem, "display" ) ) &&
					elem.offsetWidth === 0 ?
						swap( elem, cssShow, function() {
							return getWidthOrHeight( elem, name, extra );
						} ) :
						getWidthOrHeight( elem, name, extra );
			}
		},

		set: function( elem, value, extra ) {
			var matches,
				styles = extra && getStyles( elem ),
				subtract = extra && augmentWidthOrHeight(
					elem,
					name,
					extra,
					jQuery.css( elem, "boxSizing", false, styles ) === "border-box",
					styles
				);

			// Convert to pixels if value adjustment is needed
			if ( subtract && ( matches = rcssNum.exec( value ) ) &&
				( matches[ 3 ] || "px" ) !== "px" ) {

				elem.style[ name ] = value;
				value = jQuery.css( elem, name );
			}

			return setPositiveNumber( elem, value, subtract );
		}
	};
} );

jQuery.cssHooks.marginLeft = addGetHookIf( support.reliableMarginLeft,
	function( elem, computed ) {
		if ( computed ) {
			return ( parseFloat( curCSS( elem, "marginLeft" ) ) ||
				elem.getBoundingClientRect().left -
					swap( elem, { marginLeft: 0 }, function() {
						return elem.getBoundingClientRect().left;
					} )
				) + "px";
		}
	}
);

// Support: Android 2.3
jQuery.cssHooks.marginRight = addGetHookIf( support.reliableMarginRight,
	function( elem, computed ) {
		if ( computed ) {
			return swap( elem, { "display": "inline-block" },
				curCSS, [ elem, "marginRight" ] );
		}
	}
);

// These hooks are used by animate to expand properties
jQuery.each( {
	margin: "",
	padding: "",
	border: "Width"
}, function( prefix, suffix ) {
	jQuery.cssHooks[ prefix + suffix ] = {
		expand: function( value ) {
			var i = 0,
				expanded = {},

				// Assumes a single number if not a string
				parts = typeof value === "string" ? value.split( " " ) : [ value ];

			for ( ; i < 4; i++ ) {
				expanded[ prefix + cssExpand[ i ] + suffix ] =
					parts[ i ] || parts[ i - 2 ] || parts[ 0 ];
			}

			return expanded;
		}
	};

	if ( !rmargin.test( prefix ) ) {
		jQuery.cssHooks[ prefix + suffix ].set = setPositiveNumber;
	}
} );

jQuery.fn.extend( {
	css: function( name, value ) {
		return access( this, function( elem, name, value ) {
			var styles, len,
				map = {},
				i = 0;

			if ( jQuery.isArray( name ) ) {
				styles = getStyles( elem );
				len = name.length;

				for ( ; i < len; i++ ) {
					map[ name[ i ] ] = jQuery.css( elem, name[ i ], false, styles );
				}

				return map;
			}

			return value !== undefined ?
				jQuery.style( elem, name, value ) :
				jQuery.css( elem, name );
		}, name, value, arguments.length > 1 );
	},
	show: function() {
		return showHide( this, true );
	},
	hide: function() {
		return showHide( this );
	},
	toggle: function( state ) {
		if ( typeof state === "boolean" ) {
			return state ? this.show() : this.hide();
		}

		return this.each( function() {
			if ( isHidden( this ) ) {
				jQuery( this ).show();
			} else {
				jQuery( this ).hide();
			}
		} );
	}
} );


function Tween( elem, options, prop, end, easing ) {
	return new Tween.prototype.init( elem, options, prop, end, easing );
}
jQuery.Tween = Tween;

Tween.prototype = {
	constructor: Tween,
	init: function( elem, options, prop, end, easing, unit ) {
		this.elem = elem;
		this.prop = prop;
		this.easing = easing || jQuery.easing._default;
		this.options = options;
		this.start = this.now = this.cur();
		this.end = end;
		this.unit = unit || ( jQuery.cssNumber[ prop ] ? "" : "px" );
	},
	cur: function() {
		var hooks = Tween.propHooks[ this.prop ];

		return hooks && hooks.get ?
			hooks.get( this ) :
			Tween.propHooks._default.get( this );
	},
	run: function( percent ) {
		var eased,
			hooks = Tween.propHooks[ this.prop ];

		if ( this.options.duration ) {
			this.pos = eased = jQuery.easing[ this.easing ](
				percent, this.options.duration * percent, 0, 1, this.options.duration
			);
		} else {
			this.pos = eased = percent;
		}
		this.now = ( this.end - this.start ) * eased + this.start;

		if ( this.options.step ) {
			this.options.step.call( this.elem, this.now, this );
		}

		if ( hooks && hooks.set ) {
			hooks.set( this );
		} else {
			Tween.propHooks._default.set( this );
		}
		return this;
	}
};

Tween.prototype.init.prototype = Tween.prototype;

Tween.propHooks = {
	_default: {
		get: function( tween ) {
			var result;

			// Use a property on the element directly when it is not a DOM element,
			// or when there is no matching style property that exists.
			if ( tween.elem.nodeType !== 1 ||
				tween.elem[ tween.prop ] != null && tween.elem.style[ tween.prop ] == null ) {
				return tween.elem[ tween.prop ];
			}

			// Passing an empty string as a 3rd parameter to .css will automatically
			// attempt a parseFloat and fallback to a string if the parse fails.
			// Simple values such as "10px" are parsed to Float;
			// complex values such as "rotate(1rad)" are returned as-is.
			result = jQuery.css( tween.elem, tween.prop, "" );

			// Empty strings, null, undefined and "auto" are converted to 0.
			return !result || result === "auto" ? 0 : result;
		},
		set: function( tween ) {

			// Use step hook for back compat.
			// Use cssHook if its there.
			// Use .style if available and use plain properties where available.
			if ( jQuery.fx.step[ tween.prop ] ) {
				jQuery.fx.step[ tween.prop ]( tween );
			} else if ( tween.elem.nodeType === 1 &&
				( tween.elem.style[ jQuery.cssProps[ tween.prop ] ] != null ||
					jQuery.cssHooks[ tween.prop ] ) ) {
				jQuery.style( tween.elem, tween.prop, tween.now + tween.unit );
			} else {
				tween.elem[ tween.prop ] = tween.now;
			}
		}
	}
};

// Support: IE9
// Panic based approach to setting things on disconnected nodes
Tween.propHooks.scrollTop = Tween.propHooks.scrollLeft = {
	set: function( tween ) {
		if ( tween.elem.nodeType && tween.elem.parentNode ) {
			tween.elem[ tween.prop ] = tween.now;
		}
	}
};

jQuery.easing = {
	linear: function( p ) {
		return p;
	},
	swing: function( p ) {
		return 0.5 - Math.cos( p * Math.PI ) / 2;
	},
	_default: "swing"
};

jQuery.fx = Tween.prototype.init;

// Back Compat <1.8 extension point
jQuery.fx.step = {};




var
	fxNow, timerId,
	rfxtypes = /^(?:toggle|show|hide)$/,
	rrun = /queueHooks$/;

// Animations created synchronously will run synchronously
function createFxNow() {
	window.setTimeout( function() {
		fxNow = undefined;
	} );
	return ( fxNow = jQuery.now() );
}

// Generate parameters to create a standard animation
function genFx( type, includeWidth ) {
	var which,
		i = 0,
		attrs = { height: type };

	// If we include width, step value is 1 to do all cssExpand values,
	// otherwise step value is 2 to skip over Left and Right
	includeWidth = includeWidth ? 1 : 0;
	for ( ; i < 4 ; i += 2 - includeWidth ) {
		which = cssExpand[ i ];
		attrs[ "margin" + which ] = attrs[ "padding" + which ] = type;
	}

	if ( includeWidth ) {
		attrs.opacity = attrs.width = type;
	}

	return attrs;
}

function createTween( value, prop, animation ) {
	var tween,
		collection = ( Animation.tweeners[ prop ] || [] ).concat( Animation.tweeners[ "*" ] ),
		index = 0,
		length = collection.length;
	for ( ; index < length; index++ ) {
		if ( ( tween = collection[ index ].call( animation, prop, value ) ) ) {

			// We're done with this property
			return tween;
		}
	}
}

function defaultPrefilter( elem, props, opts ) {
	/* jshint validthis: true */
	var prop, value, toggle, tween, hooks, oldfire, display, checkDisplay,
		anim = this,
		orig = {},
		style = elem.style,
		hidden = elem.nodeType && isHidden( elem ),
		dataShow = dataPriv.get( elem, "fxshow" );

	// Handle queue: false promises
	if ( !opts.queue ) {
		hooks = jQuery._queueHooks( elem, "fx" );
		if ( hooks.unqueued == null ) {
			hooks.unqueued = 0;
			oldfire = hooks.empty.fire;
			hooks.empty.fire = function() {
				if ( !hooks.unqueued ) {
					oldfire();
				}
			};
		}
		hooks.unqueued++;

		anim.always( function() {

			// Ensure the complete handler is called before this completes
			anim.always( function() {
				hooks.unqueued--;
				if ( !jQuery.queue( elem, "fx" ).length ) {
					hooks.empty.fire();
				}
			} );
		} );
	}

	// Height/width overflow pass
	if ( elem.nodeType === 1 && ( "height" in props || "width" in props ) ) {

		// Make sure that nothing sneaks out
		// Record all 3 overflow attributes because IE9-10 do not
		// change the overflow attribute when overflowX and
		// overflowY are set to the same value
		opts.overflow = [ style.overflow, style.overflowX, style.overflowY ];

		// Set display property to inline-block for height/width
		// animations on inline elements that are having width/height animated
		display = jQuery.css( elem, "display" );

		// Test default display if display is currently "none"
		checkDisplay = display === "none" ?
			dataPriv.get( elem, "olddisplay" ) || defaultDisplay( elem.nodeName ) : display;

		if ( checkDisplay === "inline" && jQuery.css( elem, "float" ) === "none" ) {
			style.display = "inline-block";
		}
	}

	if ( opts.overflow ) {
		style.overflow = "hidden";
		anim.always( function() {
			style.overflow = opts.overflow[ 0 ];
			style.overflowX = opts.overflow[ 1 ];
			style.overflowY = opts.overflow[ 2 ];
		} );
	}

	// show/hide pass
	for ( prop in props ) {
		value = props[ prop ];
		if ( rfxtypes.exec( value ) ) {
			delete props[ prop ];
			toggle = toggle || value === "toggle";
			if ( value === ( hidden ? "hide" : "show" ) ) {

				// If there is dataShow left over from a stopped hide or show
				// and we are going to proceed with show, we should pretend to be hidden
				if ( value === "show" && dataShow && dataShow[ prop ] !== undefined ) {
					hidden = true;
				} else {
					continue;
				}
			}
			orig[ prop ] = dataShow && dataShow[ prop ] || jQuery.style( elem, prop );

		// Any non-fx value stops us from restoring the original display value
		} else {
			display = undefined;
		}
	}

	if ( !jQuery.isEmptyObject( orig ) ) {
		if ( dataShow ) {
			if ( "hidden" in dataShow ) {
				hidden = dataShow.hidden;
			}
		} else {
			dataShow = dataPriv.access( elem, "fxshow", {} );
		}

		// Store state if its toggle - enables .stop().toggle() to "reverse"
		if ( toggle ) {
			dataShow.hidden = !hidden;
		}
		if ( hidden ) {
			jQuery( elem ).show();
		} else {
			anim.done( function() {
				jQuery( elem ).hide();
			} );
		}
		anim.done( function() {
			var prop;

			dataPriv.remove( elem, "fxshow" );
			for ( prop in orig ) {
				jQuery.style( elem, prop, orig[ prop ] );
			}
		} );
		for ( prop in orig ) {
			tween = createTween( hidden ? dataShow[ prop ] : 0, prop, anim );

			if ( !( prop in dataShow ) ) {
				dataShow[ prop ] = tween.start;
				if ( hidden ) {
					tween.end = tween.start;
					tween.start = prop === "width" || prop === "height" ? 1 : 0;
				}
			}
		}

	// If this is a noop like .hide().hide(), restore an overwritten display value
	} else if ( ( display === "none" ? defaultDisplay( elem.nodeName ) : display ) === "inline" ) {
		style.display = display;
	}
}

function propFilter( props, specialEasing ) {
	var index, name, easing, value, hooks;

	// camelCase, specialEasing and expand cssHook pass
	for ( index in props ) {
		name = jQuery.camelCase( index );
		easing = specialEasing[ name ];
		value = props[ index ];
		if ( jQuery.isArray( value ) ) {
			easing = value[ 1 ];
			value = props[ index ] = value[ 0 ];
		}

		if ( index !== name ) {
			props[ name ] = value;
			delete props[ index ];
		}

		hooks = jQuery.cssHooks[ name ];
		if ( hooks && "expand" in hooks ) {
			value = hooks.expand( value );
			delete props[ name ];

			// Not quite $.extend, this won't overwrite existing keys.
			// Reusing 'index' because we have the correct "name"
			for ( index in value ) {
				if ( !( index in props ) ) {
					props[ index ] = value[ index ];
					specialEasing[ index ] = easing;
				}
			}
		} else {
			specialEasing[ name ] = easing;
		}
	}
}

function Animation( elem, properties, options ) {
	var result,
		stopped,
		index = 0,
		length = Animation.prefilters.length,
		deferred = jQuery.Deferred().always( function() {

			// Don't match elem in the :animated selector
			delete tick.elem;
		} ),
		tick = function() {
			if ( stopped ) {
				return false;
			}
			var currentTime = fxNow || createFxNow(),
				remaining = Math.max( 0, animation.startTime + animation.duration - currentTime ),

				// Support: Android 2.3
				// Archaic crash bug won't allow us to use `1 - ( 0.5 || 0 )` (#12497)
				temp = remaining / animation.duration || 0,
				percent = 1 - temp,
				index = 0,
				length = animation.tweens.length;

			for ( ; index < length ; index++ ) {
				animation.tweens[ index ].run( percent );
			}

			deferred.notifyWith( elem, [ animation, percent, remaining ] );

			if ( percent < 1 && length ) {
				return remaining;
			} else {
				deferred.resolveWith( elem, [ animation ] );
				return false;
			}
		},
		animation = deferred.promise( {
			elem: elem,
			props: jQuery.extend( {}, properties ),
			opts: jQuery.extend( true, {
				specialEasing: {},
				easing: jQuery.easing._default
			}, options ),
			originalProperties: properties,
			originalOptions: options,
			startTime: fxNow || createFxNow(),
			duration: options.duration,
			tweens: [],
			createTween: function( prop, end ) {
				var tween = jQuery.Tween( elem, animation.opts, prop, end,
						animation.opts.specialEasing[ prop ] || animation.opts.easing );
				animation.tweens.push( tween );
				return tween;
			},
			stop: function( gotoEnd ) {
				var index = 0,

					// If we are going to the end, we want to run all the tweens
					// otherwise we skip this part
					length = gotoEnd ? animation.tweens.length : 0;
				if ( stopped ) {
					return this;
				}
				stopped = true;
				for ( ; index < length ; index++ ) {
					animation.tweens[ index ].run( 1 );
				}

				// Resolve when we played the last frame; otherwise, reject
				if ( gotoEnd ) {
					deferred.notifyWith( elem, [ animation, 1, 0 ] );
					deferred.resolveWith( elem, [ animation, gotoEnd ] );
				} else {
					deferred.rejectWith( elem, [ animation, gotoEnd ] );
				}
				return this;
			}
		} ),
		props = animation.props;

	propFilter( props, animation.opts.specialEasing );

	for ( ; index < length ; index++ ) {
		result = Animation.prefilters[ index ].call( animation, elem, props, animation.opts );
		if ( result ) {
			if ( jQuery.isFunction( result.stop ) ) {
				jQuery._queueHooks( animation.elem, animation.opts.queue ).stop =
					jQuery.proxy( result.stop, result );
			}
			return result;
		}
	}

	jQuery.map( props, createTween, animation );

	if ( jQuery.isFunction( animation.opts.start ) ) {
		animation.opts.start.call( elem, animation );
	}

	jQuery.fx.timer(
		jQuery.extend( tick, {
			elem: elem,
			anim: animation,
			queue: animation.opts.queue
		} )
	);

	// attach callbacks from options
	return animation.progress( animation.opts.progress )
		.done( animation.opts.done, animation.opts.complete )
		.fail( animation.opts.fail )
		.always( animation.opts.always );
}

jQuery.Animation = jQuery.extend( Animation, {
	tweeners: {
		"*": [ function( prop, value ) {
			var tween = this.createTween( prop, value );
			adjustCSS( tween.elem, prop, rcssNum.exec( value ), tween );
			return tween;
		} ]
	},

	tweener: function( props, callback ) {
		if ( jQuery.isFunction( props ) ) {
			callback = props;
			props = [ "*" ];
		} else {
			props = props.match( rnotwhite );
		}

		var prop,
			index = 0,
			length = props.length;

		for ( ; index < length ; index++ ) {
			prop = props[ index ];
			Animation.tweeners[ prop ] = Animation.tweeners[ prop ] || [];
			Animation.tweeners[ prop ].unshift( callback );
		}
	},

	prefilters: [ defaultPrefilter ],

	prefilter: function( callback, prepend ) {
		if ( prepend ) {
			Animation.prefilters.unshift( callback );
		} else {
			Animation.prefilters.push( callback );
		}
	}
} );

jQuery.speed = function( speed, easing, fn ) {
	var opt = speed && typeof speed === "object" ? jQuery.extend( {}, speed ) : {
		complete: fn || !fn && easing ||
			jQuery.isFunction( speed ) && speed,
		duration: speed,
		easing: fn && easing || easing && !jQuery.isFunction( easing ) && easing
	};

	opt.duration = jQuery.fx.off ? 0 : typeof opt.duration === "number" ?
		opt.duration : opt.duration in jQuery.fx.speeds ?
			jQuery.fx.speeds[ opt.duration ] : jQuery.fx.speeds._default;

	// Normalize opt.queue - true/undefined/null -> "fx"
	if ( opt.queue == null || opt.queue === true ) {
		opt.queue = "fx";
	}

	// Queueing
	opt.old = opt.complete;

	opt.complete = function() {
		if ( jQuery.isFunction( opt.old ) ) {
			opt.old.call( this );
		}

		if ( opt.queue ) {
			jQuery.dequeue( this, opt.queue );
		}
	};

	return opt;
};

jQuery.fn.extend( {
	fadeTo: function( speed, to, easing, callback ) {

		// Show any hidden elements after setting opacity to 0
		return this.filter( isHidden ).css( "opacity", 0 ).show()

			// Animate to the value specified
			.end().animate( { opacity: to }, speed, easing, callback );
	},
	animate: function( prop, speed, easing, callback ) {
		var empty = jQuery.isEmptyObject( prop ),
			optall = jQuery.speed( speed, easing, callback ),
			doAnimation = function() {

				// Operate on a copy of prop so per-property easing won't be lost
				var anim = Animation( this, jQuery.extend( {}, prop ), optall );

				// Empty animations, or finishing resolves immediately
				if ( empty || dataPriv.get( this, "finish" ) ) {
					anim.stop( true );
				}
			};
			doAnimation.finish = doAnimation;

		return empty || optall.queue === false ?
			this.each( doAnimation ) :
			this.queue( optall.queue, doAnimation );
	},
	stop: function( type, clearQueue, gotoEnd ) {
		var stopQueue = function( hooks ) {
			var stop = hooks.stop;
			delete hooks.stop;
			stop( gotoEnd );
		};

		if ( typeof type !== "string" ) {
			gotoEnd = clearQueue;
			clearQueue = type;
			type = undefined;
		}
		if ( clearQueue && type !== false ) {
			this.queue( type || "fx", [] );
		}

		return this.each( function() {
			var dequeue = true,
				index = type != null && type + "queueHooks",
				timers = jQuery.timers,
				data = dataPriv.get( this );

			if ( index ) {
				if ( data[ index ] && data[ index ].stop ) {
					stopQueue( data[ index ] );
				}
			} else {
				for ( index in data ) {
					if ( data[ index ] && data[ index ].stop && rrun.test( index ) ) {
						stopQueue( data[ index ] );
					}
				}
			}

			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this &&
					( type == null || timers[ index ].queue === type ) ) {

					timers[ index ].anim.stop( gotoEnd );
					dequeue = false;
					timers.splice( index, 1 );
				}
			}

			// Start the next in the queue if the last step wasn't forced.
			// Timers currently will call their complete callbacks, which
			// will dequeue but only if they were gotoEnd.
			if ( dequeue || !gotoEnd ) {
				jQuery.dequeue( this, type );
			}
		} );
	},
	finish: function( type ) {
		if ( type !== false ) {
			type = type || "fx";
		}
		return this.each( function() {
			var index,
				data = dataPriv.get( this ),
				queue = data[ type + "queue" ],
				hooks = data[ type + "queueHooks" ],
				timers = jQuery.timers,
				length = queue ? queue.length : 0;

			// Enable finishing flag on private data
			data.finish = true;

			// Empty the queue first
			jQuery.queue( this, type, [] );

			if ( hooks && hooks.stop ) {
				hooks.stop.call( this, true );
			}

			// Look for any active animations, and finish them
			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this && timers[ index ].queue === type ) {
					timers[ index ].anim.stop( true );
					timers.splice( index, 1 );
				}
			}

			// Look for any animations in the old queue and finish them
			for ( index = 0; index < length; index++ ) {
				if ( queue[ index ] && queue[ index ].finish ) {
					queue[ index ].finish.call( this );
				}
			}

			// Turn off finishing flag
			delete data.finish;
		} );
	}
} );

jQuery.each( [ "toggle", "show", "hide" ], function( i, name ) {
	var cssFn = jQuery.fn[ name ];
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return speed == null || typeof speed === "boolean" ?
			cssFn.apply( this, arguments ) :
			this.animate( genFx( name, true ), speed, easing, callback );
	};
} );

// Generate shortcuts for custom animations
jQuery.each( {
	slideDown: genFx( "show" ),
	slideUp: genFx( "hide" ),
	slideToggle: genFx( "toggle" ),
	fadeIn: { opacity: "show" },
	fadeOut: { opacity: "hide" },
	fadeToggle: { opacity: "toggle" }
}, function( name, props ) {
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return this.animate( props, speed, easing, callback );
	};
} );

jQuery.timers = [];
jQuery.fx.tick = function() {
	var timer,
		i = 0,
		timers = jQuery.timers;

	fxNow = jQuery.now();

	for ( ; i < timers.length; i++ ) {
		timer = timers[ i ];

		// Checks the timer has not already been removed
		if ( !timer() && timers[ i ] === timer ) {
			timers.splice( i--, 1 );
		}
	}

	if ( !timers.length ) {
		jQuery.fx.stop();
	}
	fxNow = undefined;
};

jQuery.fx.timer = function( timer ) {
	jQuery.timers.push( timer );
	if ( timer() ) {
		jQuery.fx.start();
	} else {
		jQuery.timers.pop();
	}
};

jQuery.fx.interval = 13;
jQuery.fx.start = function() {
	if ( !timerId ) {
		timerId = window.setInterval( jQuery.fx.tick, jQuery.fx.interval );
	}
};

jQuery.fx.stop = function() {
	window.clearInterval( timerId );

	timerId = null;
};

jQuery.fx.speeds = {
	slow: 600,
	fast: 200,

	// Default speed
	_default: 400
};


// Based off of the plugin by Clint Helfers, with permission.
// http://web.archive.org/web/20100324014747/http://blindsignals.com/index.php/2009/07/jquery-delay/
jQuery.fn.delay = function( time, type ) {
	time = jQuery.fx ? jQuery.fx.speeds[ time ] || time : time;
	type = type || "fx";

	return this.queue( type, function( next, hooks ) {
		var timeout = window.setTimeout( next, time );
		hooks.stop = function() {
			window.clearTimeout( timeout );
		};
	} );
};


( function() {
	var input = document.createElement( "input" ),
		select = document.createElement( "select" ),
		opt = select.appendChild( document.createElement( "option" ) );

	input.type = "checkbox";

	// Support: iOS<=5.1, Android<=4.2+
	// Default value for a checkbox should be "on"
	support.checkOn = input.value !== "";

	// Support: IE<=11+
	// Must access selectedIndex to make default options select
	support.optSelected = opt.selected;

	// Support: Android<=2.3
	// Options inside disabled selects are incorrectly marked as disabled
	select.disabled = true;
	support.optDisabled = !opt.disabled;

	// Support: IE<=11+
	// An input loses its value after becoming a radio
	input = document.createElement( "input" );
	input.value = "t";
	input.type = "radio";
	support.radioValue = input.value === "t";
} )();


var boolHook,
	attrHandle = jQuery.expr.attrHandle;

jQuery.fn.extend( {
	attr: function( name, value ) {
		return access( this, jQuery.attr, name, value, arguments.length > 1 );
	},

	removeAttr: function( name ) {
		return this.each( function() {
			jQuery.removeAttr( this, name );
		} );
	}
} );

jQuery.extend( {
	attr: function( elem, name, value ) {
		var ret, hooks,
			nType = elem.nodeType;

		// Don't get/set attributes on text, comment and attribute nodes
		if ( nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		// Fallback to prop when attributes are not supported
		if ( typeof elem.getAttribute === "undefined" ) {
			return jQuery.prop( elem, name, value );
		}

		// All attributes are lowercase
		// Grab necessary hook if one is defined
		if ( nType !== 1 || !jQuery.isXMLDoc( elem ) ) {
			name = name.toLowerCase();
			hooks = jQuery.attrHooks[ name ] ||
				( jQuery.expr.match.bool.test( name ) ? boolHook : undefined );
		}

		if ( value !== undefined ) {
			if ( value === null ) {
				jQuery.removeAttr( elem, name );
				return;
			}

			if ( hooks && "set" in hooks &&
				( ret = hooks.set( elem, value, name ) ) !== undefined ) {
				return ret;
			}

			elem.setAttribute( name, value + "" );
			return value;
		}

		if ( hooks && "get" in hooks && ( ret = hooks.get( elem, name ) ) !== null ) {
			return ret;
		}

		ret = jQuery.find.attr( elem, name );

		// Non-existent attributes return null, we normalize to undefined
		return ret == null ? undefined : ret;
	},

	attrHooks: {
		type: {
			set: function( elem, value ) {
				if ( !support.radioValue && value === "radio" &&
					jQuery.nodeName( elem, "input" ) ) {
					var val = elem.value;
					elem.setAttribute( "type", value );
					if ( val ) {
						elem.value = val;
					}
					return value;
				}
			}
		}
	},

	removeAttr: function( elem, value ) {
		var name, propName,
			i = 0,
			attrNames = value && value.match( rnotwhite );

		if ( attrNames && elem.nodeType === 1 ) {
			while ( ( name = attrNames[ i++ ] ) ) {
				propName = jQuery.propFix[ name ] || name;

				// Boolean attributes get special treatment (#10870)
				if ( jQuery.expr.match.bool.test( name ) ) {

					// Set corresponding property to false
					elem[ propName ] = false;
				}

				elem.removeAttribute( name );
			}
		}
	}
} );

// Hooks for boolean attributes
boolHook = {
	set: function( elem, value, name ) {
		if ( value === false ) {

			// Remove boolean attributes when set to false
			jQuery.removeAttr( elem, name );
		} else {
			elem.setAttribute( name, name );
		}
		return name;
	}
};
jQuery.each( jQuery.expr.match.bool.source.match( /\w+/g ), function( i, name ) {
	var getter = attrHandle[ name ] || jQuery.find.attr;

	attrHandle[ name ] = function( elem, name, isXML ) {
		var ret, handle;
		if ( !isXML ) {

			// Avoid an infinite loop by temporarily removing this function from the getter
			handle = attrHandle[ name ];
			attrHandle[ name ] = ret;
			ret = getter( elem, name, isXML ) != null ?
				name.toLowerCase() :
				null;
			attrHandle[ name ] = handle;
		}
		return ret;
	};
} );




var rfocusable = /^(?:input|select|textarea|button)$/i,
	rclickable = /^(?:a|area)$/i;

jQuery.fn.extend( {
	prop: function( name, value ) {
		return access( this, jQuery.prop, name, value, arguments.length > 1 );
	},

	removeProp: function( name ) {
		return this.each( function() {
			delete this[ jQuery.propFix[ name ] || name ];
		} );
	}
} );

jQuery.extend( {
	prop: function( elem, name, value ) {
		var ret, hooks,
			nType = elem.nodeType;

		// Don't get/set properties on text, comment and attribute nodes
		if ( nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		if ( nType !== 1 || !jQuery.isXMLDoc( elem ) ) {

			// Fix name and attach hooks
			name = jQuery.propFix[ name ] || name;
			hooks = jQuery.propHooks[ name ];
		}

		if ( value !== undefined ) {
			if ( hooks && "set" in hooks &&
				( ret = hooks.set( elem, value, name ) ) !== undefined ) {
				return ret;
			}

			return ( elem[ name ] = value );
		}

		if ( hooks && "get" in hooks && ( ret = hooks.get( elem, name ) ) !== null ) {
			return ret;
		}

		return elem[ name ];
	},

	propHooks: {
		tabIndex: {
			get: function( elem ) {

				// elem.tabIndex doesn't always return the
				// correct value when it hasn't been explicitly set
				// http://fluidproject.org/blog/2008/01/09/getting-setting-and-removing-tabindex-values-with-javascript/
				// Use proper attribute retrieval(#12072)
				var tabindex = jQuery.find.attr( elem, "tabindex" );

				return tabindex ?
					parseInt( tabindex, 10 ) :
					rfocusable.test( elem.nodeName ) ||
						rclickable.test( elem.nodeName ) && elem.href ?
							0 :
							-1;
			}
		}
	},

	propFix: {
		"for": "htmlFor",
		"class": "className"
	}
} );

// Support: IE <=11 only
// Accessing the selectedIndex property
// forces the browser to respect setting selected
// on the option
// The getter ensures a default option is selected
// when in an optgroup
if ( !support.optSelected ) {
	jQuery.propHooks.selected = {
		get: function( elem ) {
			var parent = elem.parentNode;
			if ( parent && parent.parentNode ) {
				parent.parentNode.selectedIndex;
			}
			return null;
		},
		set: function( elem ) {
			var parent = elem.parentNode;
			if ( parent ) {
				parent.selectedIndex;

				if ( parent.parentNode ) {
					parent.parentNode.selectedIndex;
				}
			}
		}
	};
}

jQuery.each( [
	"tabIndex",
	"readOnly",
	"maxLength",
	"cellSpacing",
	"cellPadding",
	"rowSpan",
	"colSpan",
	"useMap",
	"frameBorder",
	"contentEditable"
], function() {
	jQuery.propFix[ this.toLowerCase() ] = this;
} );




var rclass = /[\t\r\n\f]/g;

function getClass( elem ) {
	return elem.getAttribute && elem.getAttribute( "class" ) || "";
}

jQuery.fn.extend( {
	addClass: function( value ) {
		var classes, elem, cur, curValue, clazz, j, finalValue,
			i = 0;

		if ( jQuery.isFunction( value ) ) {
			return this.each( function( j ) {
				jQuery( this ).addClass( value.call( this, j, getClass( this ) ) );
			} );
		}

		if ( typeof value === "string" && value ) {
			classes = value.match( rnotwhite ) || [];

			while ( ( elem = this[ i++ ] ) ) {
				curValue = getClass( elem );
				cur = elem.nodeType === 1 &&
					( " " + curValue + " " ).replace( rclass, " " );

				if ( cur ) {
					j = 0;
					while ( ( clazz = classes[ j++ ] ) ) {
						if ( cur.indexOf( " " + clazz + " " ) < 0 ) {
							cur += clazz + " ";
						}
					}

					// Only assign if different to avoid unneeded rendering.
					finalValue = jQuery.trim( cur );
					if ( curValue !== finalValue ) {
						elem.setAttribute( "class", finalValue );
					}
				}
			}
		}

		return this;
	},

	removeClass: function( value ) {
		var classes, elem, cur, curValue, clazz, j, finalValue,
			i = 0;

		if ( jQuery.isFunction( value ) ) {
			return this.each( function( j ) {
				jQuery( this ).removeClass( value.call( this, j, getClass( this ) ) );
			} );
		}

		if ( !arguments.length ) {
			return this.attr( "class", "" );
		}

		if ( typeof value === "string" && value ) {
			classes = value.match( rnotwhite ) || [];

			while ( ( elem = this[ i++ ] ) ) {
				curValue = getClass( elem );

				// This expression is here for better compressibility (see addClass)
				cur = elem.nodeType === 1 &&
					( " " + curValue + " " ).replace( rclass, " " );

				if ( cur ) {
					j = 0;
					while ( ( clazz = classes[ j++ ] ) ) {

						// Remove *all* instances
						while ( cur.indexOf( " " + clazz + " " ) > -1 ) {
							cur = cur.replace( " " + clazz + " ", " " );
						}
					}

					// Only assign if different to avoid unneeded rendering.
					finalValue = jQuery.trim( cur );
					if ( curValue !== finalValue ) {
						elem.setAttribute( "class", finalValue );
					}
				}
			}
		}

		return this;
	},

	toggleClass: function( value, stateVal ) {
		var type = typeof value;

		if ( typeof stateVal === "boolean" && type === "string" ) {
			return stateVal ? this.addClass( value ) : this.removeClass( value );
		}

		if ( jQuery.isFunction( value ) ) {
			return this.each( function( i ) {
				jQuery( this ).toggleClass(
					value.call( this, i, getClass( this ), stateVal ),
					stateVal
				);
			} );
		}

		return this.each( function() {
			var className, i, self, classNames;

			if ( type === "string" ) {

				// Toggle individual class names
				i = 0;
				self = jQuery( this );
				classNames = value.match( rnotwhite ) || [];

				while ( ( className = classNames[ i++ ] ) ) {

					// Check each className given, space separated list
					if ( self.hasClass( className ) ) {
						self.removeClass( className );
					} else {
						self.addClass( className );
					}
				}

			// Toggle whole class name
			} else if ( value === undefined || type === "boolean" ) {
				className = getClass( this );
				if ( className ) {

					// Store className if set
					dataPriv.set( this, "__className__", className );
				}

				// If the element has a class name or if we're passed `false`,
				// then remove the whole classname (if there was one, the above saved it).
				// Otherwise bring back whatever was previously saved (if anything),
				// falling back to the empty string if nothing was stored.
				if ( this.setAttribute ) {
					this.setAttribute( "class",
						className || value === false ?
						"" :
						dataPriv.get( this, "__className__" ) || ""
					);
				}
			}
		} );
	},

	hasClass: function( selector ) {
		var className, elem,
			i = 0;

		className = " " + selector + " ";
		while ( ( elem = this[ i++ ] ) ) {
			if ( elem.nodeType === 1 &&
				( " " + getClass( elem ) + " " ).replace( rclass, " " )
					.indexOf( className ) > -1
			) {
				return true;
			}
		}

		return false;
	}
} );




var rreturn = /\r/g,
	rspaces = /[\x20\t\r\n\f]+/g;

jQuery.fn.extend( {
	val: function( value ) {
		var hooks, ret, isFunction,
			elem = this[ 0 ];

		if ( !arguments.length ) {
			if ( elem ) {
				hooks = jQuery.valHooks[ elem.type ] ||
					jQuery.valHooks[ elem.nodeName.toLowerCase() ];

				if ( hooks &&
					"get" in hooks &&
					( ret = hooks.get( elem, "value" ) ) !== undefined
				) {
					return ret;
				}

				ret = elem.value;

				return typeof ret === "string" ?

					// Handle most common string cases
					ret.replace( rreturn, "" ) :

					// Handle cases where value is null/undef or number
					ret == null ? "" : ret;
			}

			return;
		}

		isFunction = jQuery.isFunction( value );

		return this.each( function( i ) {
			var val;

			if ( this.nodeType !== 1 ) {
				return;
			}

			if ( isFunction ) {
				val = value.call( this, i, jQuery( this ).val() );
			} else {
				val = value;
			}

			// Treat null/undefined as ""; convert numbers to string
			if ( val == null ) {
				val = "";

			} else if ( typeof val === "number" ) {
				val += "";

			} else if ( jQuery.isArray( val ) ) {
				val = jQuery.map( val, function( value ) {
					return value == null ? "" : value + "";
				} );
			}

			hooks = jQuery.valHooks[ this.type ] || jQuery.valHooks[ this.nodeName.toLowerCase() ];

			// If set returns undefined, fall back to normal setting
			if ( !hooks || !( "set" in hooks ) || hooks.set( this, val, "value" ) === undefined ) {
				this.value = val;
			}
		} );
	}
} );

jQuery.extend( {
	valHooks: {
		option: {
			get: function( elem ) {

				var val = jQuery.find.attr( elem, "value" );
				return val != null ?
					val :

					// Support: IE10-11+
					// option.text throws exceptions (#14686, #14858)
					// Strip and collapse whitespace
					// https://html.spec.whatwg.org/#strip-and-collapse-whitespace
					jQuery.trim( jQuery.text( elem ) ).replace( rspaces, " " );
			}
		},
		select: {
			get: function( elem ) {
				var value, option,
					options = elem.options,
					index = elem.selectedIndex,
					one = elem.type === "select-one" || index < 0,
					values = one ? null : [],
					max = one ? index + 1 : options.length,
					i = index < 0 ?
						max :
						one ? index : 0;

				// Loop through all the selected options
				for ( ; i < max; i++ ) {
					option = options[ i ];

					// IE8-9 doesn't update selected after form reset (#2551)
					if ( ( option.selected || i === index ) &&

							// Don't return options that are disabled or in a disabled optgroup
							( support.optDisabled ?
								!option.disabled : option.getAttribute( "disabled" ) === null ) &&
							( !option.parentNode.disabled ||
								!jQuery.nodeName( option.parentNode, "optgroup" ) ) ) {

						// Get the specific value for the option
						value = jQuery( option ).val();

						// We don't need an array for one selects
						if ( one ) {
							return value;
						}

						// Multi-Selects return an array
						values.push( value );
					}
				}

				return values;
			},

			set: function( elem, value ) {
				var optionSet, option,
					options = elem.options,
					values = jQuery.makeArray( value ),
					i = options.length;

				while ( i-- ) {
					option = options[ i ];
					if ( option.selected =
						jQuery.inArray( jQuery.valHooks.option.get( option ), values ) > -1
					) {
						optionSet = true;
					}
				}

				// Force browsers to behave consistently when non-matching value is set
				if ( !optionSet ) {
					elem.selectedIndex = -1;
				}
				return values;
			}
		}
	}
} );

// Radios and checkboxes getter/setter
jQuery.each( [ "radio", "checkbox" ], function() {
	jQuery.valHooks[ this ] = {
		set: function( elem, value ) {
			if ( jQuery.isArray( value ) ) {
				return ( elem.checked = jQuery.inArray( jQuery( elem ).val(), value ) > -1 );
			}
		}
	};
	if ( !support.checkOn ) {
		jQuery.valHooks[ this ].get = function( elem ) {
			return elem.getAttribute( "value" ) === null ? "on" : elem.value;
		};
	}
} );




// Return jQuery for attributes-only inclusion


var rfocusMorph = /^(?:focusinfocus|focusoutblur)$/;

jQuery.extend( jQuery.event, {

	trigger: function( event, data, elem, onlyHandlers ) {

		var i, cur, tmp, bubbleType, ontype, handle, special,
			eventPath = [ elem || document ],
			type = hasOwn.call( event, "type" ) ? event.type : event,
			namespaces = hasOwn.call( event, "namespace" ) ? event.namespace.split( "." ) : [];

		cur = tmp = elem = elem || document;

		// Don't do events on text and comment nodes
		if ( elem.nodeType === 3 || elem.nodeType === 8 ) {
			return;
		}

		// focus/blur morphs to focusin/out; ensure we're not firing them right now
		if ( rfocusMorph.test( type + jQuery.event.triggered ) ) {
			return;
		}

		if ( type.indexOf( "." ) > -1 ) {

			// Namespaced trigger; create a regexp to match event type in handle()
			namespaces = type.split( "." );
			type = namespaces.shift();
			namespaces.sort();
		}
		ontype = type.indexOf( ":" ) < 0 && "on" + type;

		// Caller can pass in a jQuery.Event object, Object, or just an event type string
		event = event[ jQuery.expando ] ?
			event :
			new jQuery.Event( type, typeof event === "object" && event );

		// Trigger bitmask: & 1 for native handlers; & 2 for jQuery (always true)
		event.isTrigger = onlyHandlers ? 2 : 3;
		event.namespace = namespaces.join( "." );
		event.rnamespace = event.namespace ?
			new RegExp( "(^|\\.)" + namespaces.join( "\\.(?:.*\\.|)" ) + "(\\.|$)" ) :
			null;

		// Clean up the event in case it is being reused
		event.result = undefined;
		if ( !event.target ) {
			event.target = elem;
		}

		// Clone any incoming data and prepend the event, creating the handler arg list
		data = data == null ?
			[ event ] :
			jQuery.makeArray( data, [ event ] );

		// Allow special events to draw outside the lines
		special = jQuery.event.special[ type ] || {};
		if ( !onlyHandlers && special.trigger && special.trigger.apply( elem, data ) === false ) {
			return;
		}

		// Determine event propagation path in advance, per W3C events spec (#9951)
		// Bubble up to document, then to window; watch for a global ownerDocument var (#9724)
		if ( !onlyHandlers && !special.noBubble && !jQuery.isWindow( elem ) ) {

			bubbleType = special.delegateType || type;
			if ( !rfocusMorph.test( bubbleType + type ) ) {
				cur = cur.parentNode;
			}
			for ( ; cur; cur = cur.parentNode ) {
				eventPath.push( cur );
				tmp = cur;
			}

			// Only add window if we got to document (e.g., not plain obj or detached DOM)
			if ( tmp === ( elem.ownerDocument || document ) ) {
				eventPath.push( tmp.defaultView || tmp.parentWindow || window );
			}
		}

		// Fire handlers on the event path
		i = 0;
		while ( ( cur = eventPath[ i++ ] ) && !event.isPropagationStopped() ) {

			event.type = i > 1 ?
				bubbleType :
				special.bindType || type;

			// jQuery handler
			handle = ( dataPriv.get( cur, "events" ) || {} )[ event.type ] &&
				dataPriv.get( cur, "handle" );
			if ( handle ) {
				handle.apply( cur, data );
			}

			// Native handler
			handle = ontype && cur[ ontype ];
			if ( handle && handle.apply && acceptData( cur ) ) {
				event.result = handle.apply( cur, data );
				if ( event.result === false ) {
					event.preventDefault();
				}
			}
		}
		event.type = type;

		// If nobody prevented the default action, do it now
		if ( !onlyHandlers && !event.isDefaultPrevented() ) {

			if ( ( !special._default ||
				special._default.apply( eventPath.pop(), data ) === false ) &&
				acceptData( elem ) ) {

				// Call a native DOM method on the target with the same name name as the event.
				// Don't do default actions on window, that's where global variables be (#6170)
				if ( ontype && jQuery.isFunction( elem[ type ] ) && !jQuery.isWindow( elem ) ) {

					// Don't re-trigger an onFOO event when we call its FOO() method
					tmp = elem[ ontype ];

					if ( tmp ) {
						elem[ ontype ] = null;
					}

					// Prevent re-triggering of the same event, since we already bubbled it above
					jQuery.event.triggered = type;
					elem[ type ]();
					jQuery.event.triggered = undefined;

					if ( tmp ) {
						elem[ ontype ] = tmp;
					}
				}
			}
		}

		return event.result;
	},

	// Piggyback on a donor event to simulate a different one
	// Used only for `focus(in | out)` events
	simulate: function( type, elem, event ) {
		var e = jQuery.extend(
			new jQuery.Event(),
			event,
			{
				type: type,
				isSimulated: true
			}
		);

		jQuery.event.trigger( e, null, elem );
	}

} );

jQuery.fn.extend( {

	trigger: function( type, data ) {
		return this.each( function() {
			jQuery.event.trigger( type, data, this );
		} );
	},
	triggerHandler: function( type, data ) {
		var elem = this[ 0 ];
		if ( elem ) {
			return jQuery.event.trigger( type, data, elem, true );
		}
	}
} );


jQuery.each( ( "blur focus focusin focusout load resize scroll unload click dblclick " +
	"mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave " +
	"change select submit keydown keypress keyup error contextmenu" ).split( " " ),
	function( i, name ) {

	// Handle event binding
	jQuery.fn[ name ] = function( data, fn ) {
		return arguments.length > 0 ?
			this.on( name, null, data, fn ) :
			this.trigger( name );
	};
} );

jQuery.fn.extend( {
	hover: function( fnOver, fnOut ) {
		return this.mouseenter( fnOver ).mouseleave( fnOut || fnOver );
	}
} );




support.focusin = "onfocusin" in window;


// Support: Firefox
// Firefox doesn't have focus(in | out) events
// Related ticket - https://bugzilla.mozilla.org/show_bug.cgi?id=687787
//
// Support: Chrome, Safari
// focus(in | out) events fire after focus & blur events,
// which is spec violation - http://www.w3.org/TR/DOM-Level-3-Events/#events-focusevent-event-order
// Related ticket - https://code.google.com/p/chromium/issues/detail?id=449857
if ( !support.focusin ) {
	jQuery.each( { focus: "focusin", blur: "focusout" }, function( orig, fix ) {

		// Attach a single capturing handler on the document while someone wants focusin/focusout
		var handler = function( event ) {
			jQuery.event.simulate( fix, event.target, jQuery.event.fix( event ) );
		};

		jQuery.event.special[ fix ] = {
			setup: function() {
				var doc = this.ownerDocument || this,
					attaches = dataPriv.access( doc, fix );

				if ( !attaches ) {
					doc.addEventListener( orig, handler, true );
				}
				dataPriv.access( doc, fix, ( attaches || 0 ) + 1 );
			},
			teardown: function() {
				var doc = this.ownerDocument || this,
					attaches = dataPriv.access( doc, fix ) - 1;

				if ( !attaches ) {
					doc.removeEventListener( orig, handler, true );
					dataPriv.remove( doc, fix );

				} else {
					dataPriv.access( doc, fix, attaches );
				}
			}
		};
	} );
}
var location = window.location;

var nonce = jQuery.now();

var rquery = ( /\?/ );



// Support: Android 2.3
// Workaround failure to string-cast null input
jQuery.parseJSON = function( data ) {
	return JSON.parse( data + "" );
};


// Cross-browser xml parsing
jQuery.parseXML = function( data ) {
	var xml;
	if ( !data || typeof data !== "string" ) {
		return null;
	}

	// Support: IE9
	try {
		xml = ( new window.DOMParser() ).parseFromString( data, "text/xml" );
	} catch ( e ) {
		xml = undefined;
	}

	if ( !xml || xml.getElementsByTagName( "parsererror" ).length ) {
		jQuery.error( "Invalid XML: " + data );
	}
	return xml;
};


var
	rhash = /#.*$/,
	rts = /([?&])_=[^&]*/,
	rheaders = /^(.*?):[ \t]*([^\r\n]*)$/mg,

	// #7653, #8125, #8152: local protocol detection
	rlocalProtocol = /^(?:about|app|app-storage|.+-extension|file|res|widget):$/,
	rnoContent = /^(?:GET|HEAD)$/,
	rprotocol = /^\/\//,

	/* Prefilters
	 * 1) They are useful to introduce custom dataTypes (see ajax/jsonp.js for an example)
	 * 2) These are called:
	 *    - BEFORE asking for a transport
	 *    - AFTER param serialization (s.data is a string if s.processData is true)
	 * 3) key is the dataType
	 * 4) the catchall symbol "*" can be used
	 * 5) execution will start with transport dataType and THEN continue down to "*" if needed
	 */
	prefilters = {},

	/* Transports bindings
	 * 1) key is the dataType
	 * 2) the catchall symbol "*" can be used
	 * 3) selection will start with transport dataType and THEN go to "*" if needed
	 */
	transports = {},

	// Avoid comment-prolog char sequence (#10098); must appease lint and evade compression
	allTypes = "*/".concat( "*" ),

	// Anchor tag for parsing the document origin
	originAnchor = document.createElement( "a" );
	originAnchor.href = location.href;

// Base "constructor" for jQuery.ajaxPrefilter and jQuery.ajaxTransport
function addToPrefiltersOrTransports( structure ) {

	// dataTypeExpression is optional and defaults to "*"
	return function( dataTypeExpression, func ) {

		if ( typeof dataTypeExpression !== "string" ) {
			func = dataTypeExpression;
			dataTypeExpression = "*";
		}

		var dataType,
			i = 0,
			dataTypes = dataTypeExpression.toLowerCase().match( rnotwhite ) || [];

		if ( jQuery.isFunction( func ) ) {

			// For each dataType in the dataTypeExpression
			while ( ( dataType = dataTypes[ i++ ] ) ) {

				// Prepend if requested
				if ( dataType[ 0 ] === "+" ) {
					dataType = dataType.slice( 1 ) || "*";
					( structure[ dataType ] = structure[ dataType ] || [] ).unshift( func );

				// Otherwise append
				} else {
					( structure[ dataType ] = structure[ dataType ] || [] ).push( func );
				}
			}
		}
	};
}

// Base inspection function for prefilters and transports
function inspectPrefiltersOrTransports( structure, options, originalOptions, jqXHR ) {

	var inspected = {},
		seekingTransport = ( structure === transports );

	function inspect( dataType ) {
		var selected;
		inspected[ dataType ] = true;
		jQuery.each( structure[ dataType ] || [], function( _, prefilterOrFactory ) {
			var dataTypeOrTransport = prefilterOrFactory( options, originalOptions, jqXHR );
			if ( typeof dataTypeOrTransport === "string" &&
				!seekingTransport && !inspected[ dataTypeOrTransport ] ) {

				options.dataTypes.unshift( dataTypeOrTransport );
				inspect( dataTypeOrTransport );
				return false;
			} else if ( seekingTransport ) {
				return !( selected = dataTypeOrTransport );
			}
		} );
		return selected;
	}

	return inspect( options.dataTypes[ 0 ] ) || !inspected[ "*" ] && inspect( "*" );
}

// A special extend for ajax options
// that takes "flat" options (not to be deep extended)
// Fixes #9887
function ajaxExtend( target, src ) {
	var key, deep,
		flatOptions = jQuery.ajaxSettings.flatOptions || {};

	for ( key in src ) {
		if ( src[ key ] !== undefined ) {
			( flatOptions[ key ] ? target : ( deep || ( deep = {} ) ) )[ key ] = src[ key ];
		}
	}
	if ( deep ) {
		jQuery.extend( true, target, deep );
	}

	return target;
}

/* Handles responses to an ajax request:
 * - finds the right dataType (mediates between content-type and expected dataType)
 * - returns the corresponding response
 */
function ajaxHandleResponses( s, jqXHR, responses ) {

	var ct, type, finalDataType, firstDataType,
		contents = s.contents,
		dataTypes = s.dataTypes;

	// Remove auto dataType and get content-type in the process
	while ( dataTypes[ 0 ] === "*" ) {
		dataTypes.shift();
		if ( ct === undefined ) {
			ct = s.mimeType || jqXHR.getResponseHeader( "Content-Type" );
		}
	}

	// Check if we're dealing with a known content-type
	if ( ct ) {
		for ( type in contents ) {
			if ( contents[ type ] && contents[ type ].test( ct ) ) {
				dataTypes.unshift( type );
				break;
			}
		}
	}

	// Check to see if we have a response for the expected dataType
	if ( dataTypes[ 0 ] in responses ) {
		finalDataType = dataTypes[ 0 ];
	} else {

		// Try convertible dataTypes
		for ( type in responses ) {
			if ( !dataTypes[ 0 ] || s.converters[ type + " " + dataTypes[ 0 ] ] ) {
				finalDataType = type;
				break;
			}
			if ( !firstDataType ) {
				firstDataType = type;
			}
		}

		// Or just use first one
		finalDataType = finalDataType || firstDataType;
	}

	// If we found a dataType
	// We add the dataType to the list if needed
	// and return the corresponding response
	if ( finalDataType ) {
		if ( finalDataType !== dataTypes[ 0 ] ) {
			dataTypes.unshift( finalDataType );
		}
		return responses[ finalDataType ];
	}
}

/* Chain conversions given the request and the original response
 * Also sets the responseXXX fields on the jqXHR instance
 */
function ajaxConvert( s, response, jqXHR, isSuccess ) {
	var conv2, current, conv, tmp, prev,
		converters = {},

		// Work with a copy of dataTypes in case we need to modify it for conversion
		dataTypes = s.dataTypes.slice();

	// Create converters map with lowercased keys
	if ( dataTypes[ 1 ] ) {
		for ( conv in s.converters ) {
			converters[ conv.toLowerCase() ] = s.converters[ conv ];
		}
	}

	current = dataTypes.shift();

	// Convert to each sequential dataType
	while ( current ) {

		if ( s.responseFields[ current ] ) {
			jqXHR[ s.responseFields[ current ] ] = response;
		}

		// Apply the dataFilter if provided
		if ( !prev && isSuccess && s.dataFilter ) {
			response = s.dataFilter( response, s.dataType );
		}

		prev = current;
		current = dataTypes.shift();

		if ( current ) {

		// There's only work to do if current dataType is non-auto
			if ( current === "*" ) {

				current = prev;

			// Convert response if prev dataType is non-auto and differs from current
			} else if ( prev !== "*" && prev !== current ) {

				// Seek a direct converter
				conv = converters[ prev + " " + current ] || converters[ "* " + current ];

				// If none found, seek a pair
				if ( !conv ) {
					for ( conv2 in converters ) {

						// If conv2 outputs current
						tmp = conv2.split( " " );
						if ( tmp[ 1 ] === current ) {

							// If prev can be converted to accepted input
							conv = converters[ prev + " " + tmp[ 0 ] ] ||
								converters[ "* " + tmp[ 0 ] ];
							if ( conv ) {

								// Condense equivalence converters
								if ( conv === true ) {
									conv = converters[ conv2 ];

								// Otherwise, insert the intermediate dataType
								} else if ( converters[ conv2 ] !== true ) {
									current = tmp[ 0 ];
									dataTypes.unshift( tmp[ 1 ] );
								}
								break;
							}
						}
					}
				}

				// Apply converter (if not an equivalence)
				if ( conv !== true ) {

					// Unless errors are allowed to bubble, catch and return them
					if ( conv && s.throws ) {
						response = conv( response );
					} else {
						try {
							response = conv( response );
						} catch ( e ) {
							return {
								state: "parsererror",
								error: conv ? e : "No conversion from " + prev + " to " + current
							};
						}
					}
				}
			}
		}
	}

	return { state: "success", data: response };
}

jQuery.extend( {

	// Counter for holding the number of active queries
	active: 0,

	// Last-Modified header cache for next request
	lastModified: {},
	etag: {},

	ajaxSettings: {
		url: location.href,
		type: "GET",
		isLocal: rlocalProtocol.test( location.protocol ),
		global: true,
		processData: true,
		async: true,
		contentType: "application/x-www-form-urlencoded; charset=UTF-8",
		/*
		timeout: 0,
		data: null,
		dataType: null,
		username: null,
		password: null,
		cache: null,
		throws: false,
		traditional: false,
		headers: {},
		*/

		accepts: {
			"*": allTypes,
			text: "text/plain",
			html: "text/html",
			xml: "application/xml, text/xml",
			json: "application/json, text/javascript"
		},

		contents: {
			xml: /\bxml\b/,
			html: /\bhtml/,
			json: /\bjson\b/
		},

		responseFields: {
			xml: "responseXML",
			text: "responseText",
			json: "responseJSON"
		},

		// Data converters
		// Keys separate source (or catchall "*") and destination types with a single space
		converters: {

			// Convert anything to text
			"* text": String,

			// Text to html (true = no transformation)
			"text html": true,

			// Evaluate text as a json expression
			"text json": jQuery.parseJSON,

			// Parse text as xml
			"text xml": jQuery.parseXML
		},

		// For options that shouldn't be deep extended:
		// you can add your own custom options here if
		// and when you create one that shouldn't be
		// deep extended (see ajaxExtend)
		flatOptions: {
			url: true,
			context: true
		}
	},

	// Creates a full fledged settings object into target
	// with both ajaxSettings and settings fields.
	// If target is omitted, writes into ajaxSettings.
	ajaxSetup: function( target, settings ) {
		return settings ?

			// Building a settings object
			ajaxExtend( ajaxExtend( target, jQuery.ajaxSettings ), settings ) :

			// Extending ajaxSettings
			ajaxExtend( jQuery.ajaxSettings, target );
	},

	ajaxPrefilter: addToPrefiltersOrTransports( prefilters ),
	ajaxTransport: addToPrefiltersOrTransports( transports ),

	// Main method
	ajax: function( url, options ) {

		// If url is an object, simulate pre-1.5 signature
		if ( typeof url === "object" ) {
			options = url;
			url = undefined;
		}

		// Force options to be an object
		options = options || {};

		var transport,

			// URL without anti-cache param
			cacheURL,

			// Response headers
			responseHeadersString,
			responseHeaders,

			// timeout handle
			timeoutTimer,

			// Url cleanup var
			urlAnchor,

			// To know if global events are to be dispatched
			fireGlobals,

			// Loop variable
			i,

			// Create the final options object
			s = jQuery.ajaxSetup( {}, options ),

			// Callbacks context
			callbackContext = s.context || s,

			// Context for global events is callbackContext if it is a DOM node or jQuery collection
			globalEventContext = s.context &&
				( callbackContext.nodeType || callbackContext.jquery ) ?
					jQuery( callbackContext ) :
					jQuery.event,

			// Deferreds
			deferred = jQuery.Deferred(),
			completeDeferred = jQuery.Callbacks( "once memory" ),

			// Status-dependent callbacks
			statusCode = s.statusCode || {},

			// Headers (they are sent all at once)
			requestHeaders = {},
			requestHeadersNames = {},

			// The jqXHR state
			state = 0,

			// Default abort message
			strAbort = "canceled",

			// Fake xhr
			jqXHR = {
				readyState: 0,

				// Builds headers hashtable if needed
				getResponseHeader: function( key ) {
					var match;
					if ( state === 2 ) {
						if ( !responseHeaders ) {
							responseHeaders = {};
							while ( ( match = rheaders.exec( responseHeadersString ) ) ) {
								responseHeaders[ match[ 1 ].toLowerCase() ] = match[ 2 ];
							}
						}
						match = responseHeaders[ key.toLowerCase() ];
					}
					return match == null ? null : match;
				},

				// Raw string
				getAllResponseHeaders: function() {
					return state === 2 ? responseHeadersString : null;
				},

				// Caches the header
				setRequestHeader: function( name, value ) {
					var lname = name.toLowerCase();
					if ( !state ) {
						name = requestHeadersNames[ lname ] = requestHeadersNames[ lname ] || name;
						requestHeaders[ name ] = value;
					}
					return this;
				},

				// Overrides response content-type header
				overrideMimeType: function( type ) {
					if ( !state ) {
						s.mimeType = type;
					}
					return this;
				},

				// Status-dependent callbacks
				statusCode: function( map ) {
					var code;
					if ( map ) {
						if ( state < 2 ) {
							for ( code in map ) {

								// Lazy-add the new callback in a way that preserves old ones
								statusCode[ code ] = [ statusCode[ code ], map[ code ] ];
							}
						} else {

							// Execute the appropriate callbacks
							jqXHR.always( map[ jqXHR.status ] );
						}
					}
					return this;
				},

				// Cancel the request
				abort: function( statusText ) {
					var finalText = statusText || strAbort;
					if ( transport ) {
						transport.abort( finalText );
					}
					done( 0, finalText );
					return this;
				}
			};

		// Attach deferreds
		deferred.promise( jqXHR ).complete = completeDeferred.add;
		jqXHR.success = jqXHR.done;
		jqXHR.error = jqXHR.fail;

		// Remove hash character (#7531: and string promotion)
		// Add protocol if not provided (prefilters might expect it)
		// Handle falsy url in the settings object (#10093: consistency with old signature)
		// We also use the url parameter if available
		s.url = ( ( url || s.url || location.href ) + "" ).replace( rhash, "" )
			.replace( rprotocol, location.protocol + "//" );

		// Alias method option to type as per ticket #12004
		s.type = options.method || options.type || s.method || s.type;

		// Extract dataTypes list
		s.dataTypes = jQuery.trim( s.dataType || "*" ).toLowerCase().match( rnotwhite ) || [ "" ];

		// A cross-domain request is in order when the origin doesn't match the current origin.
		if ( s.crossDomain == null ) {
			urlAnchor = document.createElement( "a" );

			// Support: IE8-11+
			// IE throws exception if url is malformed, e.g. http://example.com:80x/
			try {
				urlAnchor.href = s.url;

				// Support: IE8-11+
				// Anchor's host property isn't correctly set when s.url is relative
				urlAnchor.href = urlAnchor.href;
				s.crossDomain = originAnchor.protocol + "//" + originAnchor.host !==
					urlAnchor.protocol + "//" + urlAnchor.host;
			} catch ( e ) {

				// If there is an error parsing the URL, assume it is crossDomain,
				// it can be rejected by the transport if it is invalid
				s.crossDomain = true;
			}
		}

		// Convert data if not already a string
		if ( s.data && s.processData && typeof s.data !== "string" ) {
			s.data = jQuery.param( s.data, s.traditional );
		}

		// Apply prefilters
		inspectPrefiltersOrTransports( prefilters, s, options, jqXHR );

		// If request was aborted inside a prefilter, stop there
		if ( state === 2 ) {
			return jqXHR;
		}

		// We can fire global events as of now if asked to
		// Don't fire events if jQuery.event is undefined in an AMD-usage scenario (#15118)
		fireGlobals = jQuery.event && s.global;

		// Watch for a new set of requests
		if ( fireGlobals && jQuery.active++ === 0 ) {
			jQuery.event.trigger( "ajaxStart" );
		}

		// Uppercase the type
		s.type = s.type.toUpperCase();

		// Determine if request has content
		s.hasContent = !rnoContent.test( s.type );

		// Save the URL in case we're toying with the If-Modified-Since
		// and/or If-None-Match header later on
		cacheURL = s.url;

		// More options handling for requests with no content
		if ( !s.hasContent ) {

			// If data is available, append data to url
			if ( s.data ) {
				cacheURL = ( s.url += ( rquery.test( cacheURL ) ? "&" : "?" ) + s.data );

				// #9682: remove data so that it's not used in an eventual retry
				delete s.data;
			}

			// Add anti-cache in url if needed
			if ( s.cache === false ) {
				s.url = rts.test( cacheURL ) ?

					// If there is already a '_' parameter, set its value
					cacheURL.replace( rts, "$1_=" + nonce++ ) :

					// Otherwise add one to the end
					cacheURL + ( rquery.test( cacheURL ) ? "&" : "?" ) + "_=" + nonce++;
			}
		}

		// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
		if ( s.ifModified ) {
			if ( jQuery.lastModified[ cacheURL ] ) {
				jqXHR.setRequestHeader( "If-Modified-Since", jQuery.lastModified[ cacheURL ] );
			}
			if ( jQuery.etag[ cacheURL ] ) {
				jqXHR.setRequestHeader( "If-None-Match", jQuery.etag[ cacheURL ] );
			}
		}

		// Set the correct header, if data is being sent
		if ( s.data && s.hasContent && s.contentType !== false || options.contentType ) {
			jqXHR.setRequestHeader( "Content-Type", s.contentType );
		}

		// Set the Accepts header for the server, depending on the dataType
		jqXHR.setRequestHeader(
			"Accept",
			s.dataTypes[ 0 ] && s.accepts[ s.dataTypes[ 0 ] ] ?
				s.accepts[ s.dataTypes[ 0 ] ] +
					( s.dataTypes[ 0 ] !== "*" ? ", " + allTypes + "; q=0.01" : "" ) :
				s.accepts[ "*" ]
		);

		// Check for headers option
		for ( i in s.headers ) {
			jqXHR.setRequestHeader( i, s.headers[ i ] );
		}

		// Allow custom headers/mimetypes and early abort
		if ( s.beforeSend &&
			( s.beforeSend.call( callbackContext, jqXHR, s ) === false || state === 2 ) ) {

			// Abort if not done already and return
			return jqXHR.abort();
		}

		// Aborting is no longer a cancellation
		strAbort = "abort";

		// Install callbacks on deferreds
		for ( i in { success: 1, error: 1, complete: 1 } ) {
			jqXHR[ i ]( s[ i ] );
		}

		// Get transport
		transport = inspectPrefiltersOrTransports( transports, s, options, jqXHR );

		// If no transport, we auto-abort
		if ( !transport ) {
			done( -1, "No Transport" );
		} else {
			jqXHR.readyState = 1;

			// Send global event
			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxSend", [ jqXHR, s ] );
			}

			// If request was aborted inside ajaxSend, stop there
			if ( state === 2 ) {
				return jqXHR;
			}

			// Timeout
			if ( s.async && s.timeout > 0 ) {
				timeoutTimer = window.setTimeout( function() {
					jqXHR.abort( "timeout" );
				}, s.timeout );
			}

			try {
				state = 1;
				transport.send( requestHeaders, done );
			} catch ( e ) {

				// Propagate exception as error if not done
				if ( state < 2 ) {
					done( -1, e );

				// Simply rethrow otherwise
				} else {
					throw e;
				}
			}
		}

		// Callback for when everything is done
		function done( status, nativeStatusText, responses, headers ) {
			var isSuccess, success, error, response, modified,
				statusText = nativeStatusText;

			// Called once
			if ( state === 2 ) {
				return;
			}

			// State is "done" now
			state = 2;

			// Clear timeout if it exists
			if ( timeoutTimer ) {
				window.clearTimeout( timeoutTimer );
			}

			// Dereference transport for early garbage collection
			// (no matter how long the jqXHR object will be used)
			transport = undefined;

			// Cache response headers
			responseHeadersString = headers || "";

			// Set readyState
			jqXHR.readyState = status > 0 ? 4 : 0;

			// Determine if successful
			isSuccess = status >= 200 && status < 300 || status === 304;

			// Get response data
			if ( responses ) {
				response = ajaxHandleResponses( s, jqXHR, responses );
			}

			// Convert no matter what (that way responseXXX fields are always set)
			response = ajaxConvert( s, response, jqXHR, isSuccess );

			// If successful, handle type chaining
			if ( isSuccess ) {

				// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
				if ( s.ifModified ) {
					modified = jqXHR.getResponseHeader( "Last-Modified" );
					if ( modified ) {
						jQuery.lastModified[ cacheURL ] = modified;
					}
					modified = jqXHR.getResponseHeader( "etag" );
					if ( modified ) {
						jQuery.etag[ cacheURL ] = modified;
					}
				}

				// if no content
				if ( status === 204 || s.type === "HEAD" ) {
					statusText = "nocontent";

				// if not modified
				} else if ( status === 304 ) {
					statusText = "notmodified";

				// If we have data, let's convert it
				} else {
					statusText = response.state;
					success = response.data;
					error = response.error;
					isSuccess = !error;
				}
			} else {

				// Extract error from statusText and normalize for non-aborts
				error = statusText;
				if ( status || !statusText ) {
					statusText = "error";
					if ( status < 0 ) {
						status = 0;
					}
				}
			}

			// Set data for the fake xhr object
			jqXHR.status = status;
			jqXHR.statusText = ( nativeStatusText || statusText ) + "";

			// Success/Error
			if ( isSuccess ) {
				deferred.resolveWith( callbackContext, [ success, statusText, jqXHR ] );
			} else {
				deferred.rejectWith( callbackContext, [ jqXHR, statusText, error ] );
			}

			// Status-dependent callbacks
			jqXHR.statusCode( statusCode );
			statusCode = undefined;

			if ( fireGlobals ) {
				globalEventContext.trigger( isSuccess ? "ajaxSuccess" : "ajaxError",
					[ jqXHR, s, isSuccess ? success : error ] );
			}

			// Complete
			completeDeferred.fireWith( callbackContext, [ jqXHR, statusText ] );

			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxComplete", [ jqXHR, s ] );

				// Handle the global AJAX counter
				if ( !( --jQuery.active ) ) {
					jQuery.event.trigger( "ajaxStop" );
				}
			}
		}

		return jqXHR;
	},

	getJSON: function( url, data, callback ) {
		return jQuery.get( url, data, callback, "json" );
	},

	getScript: function( url, callback ) {
		return jQuery.get( url, undefined, callback, "script" );
	}
} );

jQuery.each( [ "get", "post" ], function( i, method ) {
	jQuery[ method ] = function( url, data, callback, type ) {

		// Shift arguments if data argument was omitted
		if ( jQuery.isFunction( data ) ) {
			type = type || callback;
			callback = data;
			data = undefined;
		}

		// The url can be an options object (which then must have .url)
		return jQuery.ajax( jQuery.extend( {
			url: url,
			type: method,
			dataType: type,
			data: data,
			success: callback
		}, jQuery.isPlainObject( url ) && url ) );
	};
} );


jQuery._evalUrl = function( url ) {
	return jQuery.ajax( {
		url: url,

		// Make this explicit, since user can override this through ajaxSetup (#11264)
		type: "GET",
		dataType: "script",
		async: false,
		global: false,
		"throws": true
	} );
};


jQuery.fn.extend( {
	wrapAll: function( html ) {
		var wrap;

		if ( jQuery.isFunction( html ) ) {
			return this.each( function( i ) {
				jQuery( this ).wrapAll( html.call( this, i ) );
			} );
		}

		if ( this[ 0 ] ) {

			// The elements to wrap the target around
			wrap = jQuery( html, this[ 0 ].ownerDocument ).eq( 0 ).clone( true );

			if ( this[ 0 ].parentNode ) {
				wrap.insertBefore( this[ 0 ] );
			}

			wrap.map( function() {
				var elem = this;

				while ( elem.firstElementChild ) {
					elem = elem.firstElementChild;
				}

				return elem;
			} ).append( this );
		}

		return this;
	},

	wrapInner: function( html ) {
		if ( jQuery.isFunction( html ) ) {
			return this.each( function( i ) {
				jQuery( this ).wrapInner( html.call( this, i ) );
			} );
		}

		return this.each( function() {
			var self = jQuery( this ),
				contents = self.contents();

			if ( contents.length ) {
				contents.wrapAll( html );

			} else {
				self.append( html );
			}
		} );
	},

	wrap: function( html ) {
		var isFunction = jQuery.isFunction( html );

		return this.each( function( i ) {
			jQuery( this ).wrapAll( isFunction ? html.call( this, i ) : html );
		} );
	},

	unwrap: function() {
		return this.parent().each( function() {
			if ( !jQuery.nodeName( this, "body" ) ) {
				jQuery( this ).replaceWith( this.childNodes );
			}
		} ).end();
	}
} );


jQuery.expr.filters.hidden = function( elem ) {
	return !jQuery.expr.filters.visible( elem );
};
jQuery.expr.filters.visible = function( elem ) {

	// Support: Opera <= 12.12
	// Opera reports offsetWidths and offsetHeights less than zero on some elements
	// Use OR instead of AND as the element is not visible if either is true
	// See tickets #10406 and #13132
	return elem.offsetWidth > 0 || elem.offsetHeight > 0 || elem.getClientRects().length > 0;
};




var r20 = /%20/g,
	rbracket = /\[\]$/,
	rCRLF = /\r?\n/g,
	rsubmitterTypes = /^(?:submit|button|image|reset|file)$/i,
	rsubmittable = /^(?:input|select|textarea|keygen)/i;

function buildParams( prefix, obj, traditional, add ) {
	var name;

	if ( jQuery.isArray( obj ) ) {

		// Serialize array item.
		jQuery.each( obj, function( i, v ) {
			if ( traditional || rbracket.test( prefix ) ) {

				// Treat each array item as a scalar.
				add( prefix, v );

			} else {

				// Item is non-scalar (array or object), encode its numeric index.
				buildParams(
					prefix + "[" + ( typeof v === "object" && v != null ? i : "" ) + "]",
					v,
					traditional,
					add
				);
			}
		} );

	} else if ( !traditional && jQuery.type( obj ) === "object" ) {

		// Serialize object item.
		for ( name in obj ) {
			buildParams( prefix + "[" + name + "]", obj[ name ], traditional, add );
		}

	} else {

		// Serialize scalar item.
		add( prefix, obj );
	}
}

// Serialize an array of form elements or a set of
// key/values into a query string
jQuery.param = function( a, traditional ) {
	var prefix,
		s = [],
		add = function( key, value ) {

			// If value is a function, invoke it and return its value
			value = jQuery.isFunction( value ) ? value() : ( value == null ? "" : value );
			s[ s.length ] = encodeURIComponent( key ) + "=" + encodeURIComponent( value );
		};

	// Set traditional to true for jQuery <= 1.3.2 behavior.
	if ( traditional === undefined ) {
		traditional = jQuery.ajaxSettings && jQuery.ajaxSettings.traditional;
	}

	// If an array was passed in, assume that it is an array of form elements.
	if ( jQuery.isArray( a ) || ( a.jquery && !jQuery.isPlainObject( a ) ) ) {

		// Serialize the form elements
		jQuery.each( a, function() {
			add( this.name, this.value );
		} );

	} else {

		// If traditional, encode the "old" way (the way 1.3.2 or older
		// did it), otherwise encode params recursively.
		for ( prefix in a ) {
			buildParams( prefix, a[ prefix ], traditional, add );
		}
	}

	// Return the resulting serialization
	return s.join( "&" ).replace( r20, "+" );
};

jQuery.fn.extend( {
	serialize: function() {
		return jQuery.param( this.serializeArray() );
	},
	serializeArray: function() {
		return this.map( function() {

			// Can add propHook for "elements" to filter or add form elements
			var elements = jQuery.prop( this, "elements" );
			return elements ? jQuery.makeArray( elements ) : this;
		} )
		.filter( function() {
			var type = this.type;

			// Use .is( ":disabled" ) so that fieldset[disabled] works
			return this.name && !jQuery( this ).is( ":disabled" ) &&
				rsubmittable.test( this.nodeName ) && !rsubmitterTypes.test( type ) &&
				( this.checked || !rcheckableType.test( type ) );
		} )
		.map( function( i, elem ) {
			var val = jQuery( this ).val();

			return val == null ?
				null :
				jQuery.isArray( val ) ?
					jQuery.map( val, function( val ) {
						return { name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
					} ) :
					{ name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
		} ).get();
	}
} );


jQuery.ajaxSettings.xhr = function() {
	try {
		return new window.XMLHttpRequest();
	} catch ( e ) {}
};

var xhrSuccessStatus = {

		// File protocol always yields status code 0, assume 200
		0: 200,

		// Support: IE9
		// #1450: sometimes IE returns 1223 when it should be 204
		1223: 204
	},
	xhrSupported = jQuery.ajaxSettings.xhr();

support.cors = !!xhrSupported && ( "withCredentials" in xhrSupported );
support.ajax = xhrSupported = !!xhrSupported;

jQuery.ajaxTransport( function( options ) {
	var callback, errorCallback;

	// Cross domain only allowed if supported through XMLHttpRequest
	if ( support.cors || xhrSupported && !options.crossDomain ) {
		return {
			send: function( headers, complete ) {
				var i,
					xhr = options.xhr();

				xhr.open(
					options.type,
					options.url,
					options.async,
					options.username,
					options.password
				);

				// Apply custom fields if provided
				if ( options.xhrFields ) {
					for ( i in options.xhrFields ) {
						xhr[ i ] = options.xhrFields[ i ];
					}
				}

				// Override mime type if needed
				if ( options.mimeType && xhr.overrideMimeType ) {
					xhr.overrideMimeType( options.mimeType );
				}

				// X-Requested-With header
				// For cross-domain requests, seeing as conditions for a preflight are
				// akin to a jigsaw puzzle, we simply never set it to be sure.
				// (it can always be set on a per-request basis or even using ajaxSetup)
				// For same-domain requests, won't change header if already provided.
				if ( !options.crossDomain && !headers[ "X-Requested-With" ] ) {
					headers[ "X-Requested-With" ] = "XMLHttpRequest";
				}

				// Set headers
				for ( i in headers ) {
					xhr.setRequestHeader( i, headers[ i ] );
				}

				// Callback
				callback = function( type ) {
					return function() {
						if ( callback ) {
							callback = errorCallback = xhr.onload =
								xhr.onerror = xhr.onabort = xhr.onreadystatechange = null;

							if ( type === "abort" ) {
								xhr.abort();
							} else if ( type === "error" ) {

								// Support: IE9
								// On a manual native abort, IE9 throws
								// errors on any property access that is not readyState
								if ( typeof xhr.status !== "number" ) {
									complete( 0, "error" );
								} else {
									complete(

										// File: protocol always yields status 0; see #8605, #14207
										xhr.status,
										xhr.statusText
									);
								}
							} else {
								complete(
									xhrSuccessStatus[ xhr.status ] || xhr.status,
									xhr.statusText,

									// Support: IE9 only
									// IE9 has no XHR2 but throws on binary (trac-11426)
									// For XHR2 non-text, let the caller handle it (gh-2498)
									( xhr.responseType || "text" ) !== "text"  ||
									typeof xhr.responseText !== "string" ?
										{ binary: xhr.response } :
										{ text: xhr.responseText },
									xhr.getAllResponseHeaders()
								);
							}
						}
					};
				};

				// Listen to events
				xhr.onload = callback();
				errorCallback = xhr.onerror = callback( "error" );

				// Support: IE9
				// Use onreadystatechange to replace onabort
				// to handle uncaught aborts
				if ( xhr.onabort !== undefined ) {
					xhr.onabort = errorCallback;
				} else {
					xhr.onreadystatechange = function() {

						// Check readyState before timeout as it changes
						if ( xhr.readyState === 4 ) {

							// Allow onerror to be called first,
							// but that will not handle a native abort
							// Also, save errorCallback to a variable
							// as xhr.onerror cannot be accessed
							window.setTimeout( function() {
								if ( callback ) {
									errorCallback();
								}
							} );
						}
					};
				}

				// Create the abort callback
				callback = callback( "abort" );

				try {

					// Do send the request (this may raise an exception)
					xhr.send( options.hasContent && options.data || null );
				} catch ( e ) {

					// #14683: Only rethrow if this hasn't been notified as an error yet
					if ( callback ) {
						throw e;
					}
				}
			},

			abort: function() {
				if ( callback ) {
					callback();
				}
			}
		};
	}
} );




// Install script dataType
jQuery.ajaxSetup( {
	accepts: {
		script: "text/javascript, application/javascript, " +
			"application/ecmascript, application/x-ecmascript"
	},
	contents: {
		script: /\b(?:java|ecma)script\b/
	},
	converters: {
		"text script": function( text ) {
			jQuery.globalEval( text );
			return text;
		}
	}
} );

// Handle cache's special case and crossDomain
jQuery.ajaxPrefilter( "script", function( s ) {
	if ( s.cache === undefined ) {
		s.cache = false;
	}
	if ( s.crossDomain ) {
		s.type = "GET";
	}
} );

// Bind script tag hack transport
jQuery.ajaxTransport( "script", function( s ) {

	// This transport only deals with cross domain requests
	if ( s.crossDomain ) {
		var script, callback;
		return {
			send: function( _, complete ) {
				script = jQuery( "<script>" ).prop( {
					charset: s.scriptCharset,
					src: s.url
				} ).on(
					"load error",
					callback = function( evt ) {
						script.remove();
						callback = null;
						if ( evt ) {
							complete( evt.type === "error" ? 404 : 200, evt.type );
						}
					}
				);

				// Use native DOM manipulation to avoid our domManip AJAX trickery
				document.head.appendChild( script[ 0 ] );
			},
			abort: function() {
				if ( callback ) {
					callback();
				}
			}
		};
	}
} );




var oldCallbacks = [],
	rjsonp = /(=)\?(?=&|$)|\?\?/;

// Default jsonp settings
jQuery.ajaxSetup( {
	jsonp: "callback",
	jsonpCallback: function() {
		var callback = oldCallbacks.pop() || ( jQuery.expando + "_" + ( nonce++ ) );
		this[ callback ] = true;
		return callback;
	}
} );

// Detect, normalize options and install callbacks for jsonp requests
jQuery.ajaxPrefilter( "json jsonp", function( s, originalSettings, jqXHR ) {

	var callbackName, overwritten, responseContainer,
		jsonProp = s.jsonp !== false && ( rjsonp.test( s.url ) ?
			"url" :
			typeof s.data === "string" &&
				( s.contentType || "" )
					.indexOf( "application/x-www-form-urlencoded" ) === 0 &&
				rjsonp.test( s.data ) && "data"
		);

	// Handle iff the expected data type is "jsonp" or we have a parameter to set
	if ( jsonProp || s.dataTypes[ 0 ] === "jsonp" ) {

		// Get callback name, remembering preexisting value associated with it
		callbackName = s.jsonpCallback = jQuery.isFunction( s.jsonpCallback ) ?
			s.jsonpCallback() :
			s.jsonpCallback;

		// Insert callback into url or form data
		if ( jsonProp ) {
			s[ jsonProp ] = s[ jsonProp ].replace( rjsonp, "$1" + callbackName );
		} else if ( s.jsonp !== false ) {
			s.url += ( rquery.test( s.url ) ? "&" : "?" ) + s.jsonp + "=" + callbackName;
		}

		// Use data converter to retrieve json after script execution
		s.converters[ "script json" ] = function() {
			if ( !responseContainer ) {
				jQuery.error( callbackName + " was not called" );
			}
			return responseContainer[ 0 ];
		};

		// Force json dataType
		s.dataTypes[ 0 ] = "json";

		// Install callback
		overwritten = window[ callbackName ];
		window[ callbackName ] = function() {
			responseContainer = arguments;
		};

		// Clean-up function (fires after converters)
		jqXHR.always( function() {

			// If previous value didn't exist - remove it
			if ( overwritten === undefined ) {
				jQuery( window ).removeProp( callbackName );

			// Otherwise restore preexisting value
			} else {
				window[ callbackName ] = overwritten;
			}

			// Save back as free
			if ( s[ callbackName ] ) {

				// Make sure that re-using the options doesn't screw things around
				s.jsonpCallback = originalSettings.jsonpCallback;

				// Save the callback name for future use
				oldCallbacks.push( callbackName );
			}

			// Call if it was a function and we have a response
			if ( responseContainer && jQuery.isFunction( overwritten ) ) {
				overwritten( responseContainer[ 0 ] );
			}

			responseContainer = overwritten = undefined;
		} );

		// Delegate to script
		return "script";
	}
} );




// Argument "data" should be string of html
// context (optional): If specified, the fragment will be created in this context,
// defaults to document
// keepScripts (optional): If true, will include scripts passed in the html string
jQuery.parseHTML = function( data, context, keepScripts ) {
	if ( !data || typeof data !== "string" ) {
		return null;
	}
	if ( typeof context === "boolean" ) {
		keepScripts = context;
		context = false;
	}
	context = context || document;

	var parsed = rsingleTag.exec( data ),
		scripts = !keepScripts && [];

	// Single tag
	if ( parsed ) {
		return [ context.createElement( parsed[ 1 ] ) ];
	}

	parsed = buildFragment( [ data ], context, scripts );

	if ( scripts && scripts.length ) {
		jQuery( scripts ).remove();
	}

	return jQuery.merge( [], parsed.childNodes );
};


// Keep a copy of the old load method
var _load = jQuery.fn.load;

/**
 * Load a url into a page
 */
jQuery.fn.load = function( url, params, callback ) {
	if ( typeof url !== "string" && _load ) {
		return _load.apply( this, arguments );
	}

	var selector, type, response,
		self = this,
		off = url.indexOf( " " );

	if ( off > -1 ) {
		selector = jQuery.trim( url.slice( off ) );
		url = url.slice( 0, off );
	}

	// If it's a function
	if ( jQuery.isFunction( params ) ) {

		// We assume that it's the callback
		callback = params;
		params = undefined;

	// Otherwise, build a param string
	} else if ( params && typeof params === "object" ) {
		type = "POST";
	}

	// If we have elements to modify, make the request
	if ( self.length > 0 ) {
		jQuery.ajax( {
			url: url,

			// If "type" variable is undefined, then "GET" method will be used.
			// Make value of this field explicit since
			// user can override it through ajaxSetup method
			type: type || "GET",
			dataType: "html",
			data: params
		} ).done( function( responseText ) {

			// Save response for use in complete callback
			response = arguments;

			self.html( selector ?

				// If a selector was specified, locate the right elements in a dummy div
				// Exclude scripts to avoid IE 'Permission Denied' errors
				jQuery( "<div>" ).append( jQuery.parseHTML( responseText ) ).find( selector ) :

				// Otherwise use the full result
				responseText );

		// If the request succeeds, this function gets "data", "status", "jqXHR"
		// but they are ignored because response was set above.
		// If it fails, this function gets "jqXHR", "status", "error"
		} ).always( callback && function( jqXHR, status ) {
			self.each( function() {
				callback.apply( this, response || [ jqXHR.responseText, status, jqXHR ] );
			} );
		} );
	}

	return this;
};




// Attach a bunch of functions for handling common AJAX events
jQuery.each( [
	"ajaxStart",
	"ajaxStop",
	"ajaxComplete",
	"ajaxError",
	"ajaxSuccess",
	"ajaxSend"
], function( i, type ) {
	jQuery.fn[ type ] = function( fn ) {
		return this.on( type, fn );
	};
} );




jQuery.expr.filters.animated = function( elem ) {
	return jQuery.grep( jQuery.timers, function( fn ) {
		return elem === fn.elem;
	} ).length;
};




/**
 * Gets a window from an element
 */
function getWindow( elem ) {
	return jQuery.isWindow( elem ) ? elem : elem.nodeType === 9 && elem.defaultView;
}

jQuery.offset = {
	setOffset: function( elem, options, i ) {
		var curPosition, curLeft, curCSSTop, curTop, curOffset, curCSSLeft, calculatePosition,
			position = jQuery.css( elem, "position" ),
			curElem = jQuery( elem ),
			props = {};

		// Set position first, in-case top/left are set even on static elem
		if ( position === "static" ) {
			elem.style.position = "relative";
		}

		curOffset = curElem.offset();
		curCSSTop = jQuery.css( elem, "top" );
		curCSSLeft = jQuery.css( elem, "left" );
		calculatePosition = ( position === "absolute" || position === "fixed" ) &&
			( curCSSTop + curCSSLeft ).indexOf( "auto" ) > -1;

		// Need to be able to calculate position if either
		// top or left is auto and position is either absolute or fixed
		if ( calculatePosition ) {
			curPosition = curElem.position();
			curTop = curPosition.top;
			curLeft = curPosition.left;

		} else {
			curTop = parseFloat( curCSSTop ) || 0;
			curLeft = parseFloat( curCSSLeft ) || 0;
		}

		if ( jQuery.isFunction( options ) ) {

			// Use jQuery.extend here to allow modification of coordinates argument (gh-1848)
			options = options.call( elem, i, jQuery.extend( {}, curOffset ) );
		}

		if ( options.top != null ) {
			props.top = ( options.top - curOffset.top ) + curTop;
		}
		if ( options.left != null ) {
			props.left = ( options.left - curOffset.left ) + curLeft;
		}

		if ( "using" in options ) {
			options.using.call( elem, props );

		} else {
			curElem.css( props );
		}
	}
};

jQuery.fn.extend( {
	offset: function( options ) {
		if ( arguments.length ) {
			return options === undefined ?
				this :
				this.each( function( i ) {
					jQuery.offset.setOffset( this, options, i );
				} );
		}

		var docElem, win,
			elem = this[ 0 ],
			box = { top: 0, left: 0 },
			doc = elem && elem.ownerDocument;

		if ( !doc ) {
			return;
		}

		docElem = doc.documentElement;

		// Make sure it's not a disconnected DOM node
		if ( !jQuery.contains( docElem, elem ) ) {
			return box;
		}

		box = elem.getBoundingClientRect();
		win = getWindow( doc );
		return {
			top: box.top + win.pageYOffset - docElem.clientTop,
			left: box.left + win.pageXOffset - docElem.clientLeft
		};
	},

	position: function() {
		if ( !this[ 0 ] ) {
			return;
		}

		var offsetParent, offset,
			elem = this[ 0 ],
			parentOffset = { top: 0, left: 0 };

		// Fixed elements are offset from window (parentOffset = {top:0, left: 0},
		// because it is its only offset parent
		if ( jQuery.css( elem, "position" ) === "fixed" ) {

			// Assume getBoundingClientRect is there when computed position is fixed
			offset = elem.getBoundingClientRect();

		} else {

			// Get *real* offsetParent
			offsetParent = this.offsetParent();

			// Get correct offsets
			offset = this.offset();
			if ( !jQuery.nodeName( offsetParent[ 0 ], "html" ) ) {
				parentOffset = offsetParent.offset();
			}

			// Add offsetParent borders
			parentOffset.top += jQuery.css( offsetParent[ 0 ], "borderTopWidth", true );
			parentOffset.left += jQuery.css( offsetParent[ 0 ], "borderLeftWidth", true );
		}

		// Subtract parent offsets and element margins
		return {
			top: offset.top - parentOffset.top - jQuery.css( elem, "marginTop", true ),
			left: offset.left - parentOffset.left - jQuery.css( elem, "marginLeft", true )
		};
	},

	// This method will return documentElement in the following cases:
	// 1) For the element inside the iframe without offsetParent, this method will return
	//    documentElement of the parent window
	// 2) For the hidden or detached element
	// 3) For body or html element, i.e. in case of the html node - it will return itself
	//
	// but those exceptions were never presented as a real life use-cases
	// and might be considered as more preferable results.
	//
	// This logic, however, is not guaranteed and can change at any point in the future
	offsetParent: function() {
		return this.map( function() {
			var offsetParent = this.offsetParent;

			while ( offsetParent && jQuery.css( offsetParent, "position" ) === "static" ) {
				offsetParent = offsetParent.offsetParent;
			}

			return offsetParent || documentElement;
		} );
	}
} );

// Create scrollLeft and scrollTop methods
jQuery.each( { scrollLeft: "pageXOffset", scrollTop: "pageYOffset" }, function( method, prop ) {
	var top = "pageYOffset" === prop;

	jQuery.fn[ method ] = function( val ) {
		return access( this, function( elem, method, val ) {
			var win = getWindow( elem );

			if ( val === undefined ) {
				return win ? win[ prop ] : elem[ method ];
			}

			if ( win ) {
				win.scrollTo(
					!top ? val : win.pageXOffset,
					top ? val : win.pageYOffset
				);

			} else {
				elem[ method ] = val;
			}
		}, method, val, arguments.length );
	};
} );

// Support: Safari<7-8+, Chrome<37-44+
// Add the top/left cssHooks using jQuery.fn.position
// Webkit bug: https://bugs.webkit.org/show_bug.cgi?id=29084
// Blink bug: https://code.google.com/p/chromium/issues/detail?id=229280
// getComputedStyle returns percent when specified for top/left/bottom/right;
// rather than make the css module depend on the offset module, just check for it here
jQuery.each( [ "top", "left" ], function( i, prop ) {
	jQuery.cssHooks[ prop ] = addGetHookIf( support.pixelPosition,
		function( elem, computed ) {
			if ( computed ) {
				computed = curCSS( elem, prop );

				// If curCSS returns percentage, fallback to offset
				return rnumnonpx.test( computed ) ?
					jQuery( elem ).position()[ prop ] + "px" :
					computed;
			}
		}
	);
} );


// Create innerHeight, innerWidth, height, width, outerHeight and outerWidth methods
jQuery.each( { Height: "height", Width: "width" }, function( name, type ) {
	jQuery.each( { padding: "inner" + name, content: type, "": "outer" + name },
		function( defaultExtra, funcName ) {

		// Margin is only for outerHeight, outerWidth
		jQuery.fn[ funcName ] = function( margin, value ) {
			var chainable = arguments.length && ( defaultExtra || typeof margin !== "boolean" ),
				extra = defaultExtra || ( margin === true || value === true ? "margin" : "border" );

			return access( this, function( elem, type, value ) {
				var doc;

				if ( jQuery.isWindow( elem ) ) {

					// As of 5/8/2012 this will yield incorrect results for Mobile Safari, but there
					// isn't a whole lot we can do. See pull request at this URL for discussion:
					// https://github.com/jquery/jquery/pull/764
					return elem.document.documentElement[ "client" + name ];
				}

				// Get document width or height
				if ( elem.nodeType === 9 ) {
					doc = elem.documentElement;

					// Either scroll[Width/Height] or offset[Width/Height] or client[Width/Height],
					// whichever is greatest
					return Math.max(
						elem.body[ "scroll" + name ], doc[ "scroll" + name ],
						elem.body[ "offset" + name ], doc[ "offset" + name ],
						doc[ "client" + name ]
					);
				}

				return value === undefined ?

					// Get width or height on the element, requesting but not forcing parseFloat
					jQuery.css( elem, type, extra ) :

					// Set width or height on the element
					jQuery.style( elem, type, value, extra );
			}, type, chainable ? margin : undefined, chainable, null );
		};
	} );
} );


jQuery.fn.extend( {

	bind: function( types, data, fn ) {
		return this.on( types, null, data, fn );
	},
	unbind: function( types, fn ) {
		return this.off( types, null, fn );
	},

	delegate: function( selector, types, data, fn ) {
		return this.on( types, selector, data, fn );
	},
	undelegate: function( selector, types, fn ) {

		// ( namespace ) or ( selector, types [, fn] )
		return arguments.length === 1 ?
			this.off( selector, "**" ) :
			this.off( types, selector || "**", fn );
	},
	size: function() {
		return this.length;
	}
} );

jQuery.fn.andSelf = jQuery.fn.addBack;




// Register as a named AMD module, since jQuery can be concatenated with other
// files that may use define, but not via a proper concatenation script that
// understands anonymous AMD modules. A named AMD is safest and most robust
// way to register. Lowercase jquery is used because AMD module names are
// derived from file names, and jQuery is normally delivered in a lowercase
// file name. Do this after creating the global so that if an AMD module wants
// to call noConflict to hide this version of jQuery, it will work.

// Note that for maximum portability, libraries that are not jQuery should
// declare themselves as anonymous modules, and avoid setting a global if an
// AMD loader is present. jQuery is a special case. For more information, see
// https://github.com/jrburke/requirejs/wiki/Updating-existing-libraries#wiki-anon

if ( typeof define === "function" && define.amd ) {
	define( "jquery", [], function() {
		return jQuery;
	} );
}



var

	// Map over jQuery in case of overwrite
	_jQuery = window.jQuery,

	// Map over the $ in case of overwrite
	_$ = window.$;

jQuery.noConflict = function( deep ) {
	if ( window.$ === jQuery ) {
		window.$ = _$;
	}

	if ( deep && window.jQuery === jQuery ) {
		window.jQuery = _jQuery;
	}

	return jQuery;
};

// Expose jQuery and $ identifiers, even in AMD
// (#7102#comment:10, https://github.com/jquery/jquery/pull/557)
// and CommonJS for browser emulators (#13566)
if ( !noGlobal ) {
	window.jQuery = window.$ = jQuery;
}

return jQuery;
}));

//     Underscore.js 1.6.0
//     http://underscorejs.org
//     (c) 2009-2014 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
//     Underscore may be freely distributed under the MIT license.

(function() {

  // Baseline setup
  // --------------

  // Establish the root object, `window` in the browser, or `exports` on the server.
  var root = this;

  // Save the previous value of the `_` variable.
  var previousUnderscore = root._;

  // Establish the object that gets returned to break out of a loop iteration.
  var breaker = {};

  // Save bytes in the minified (but not gzipped) version:
  var ArrayProto = Array.prototype, ObjProto = Object.prototype, FuncProto = Function.prototype;

  // Create quick reference variables for speed access to core prototypes.
  var
    push             = ArrayProto.push,
    slice            = ArrayProto.slice,
    concat           = ArrayProto.concat,
    toString         = ObjProto.toString,
    hasOwnProperty   = ObjProto.hasOwnProperty;

  // All **ECMAScript 5** native function implementations that we hope to use
  // are declared here.
  var
    nativeForEach      = ArrayProto.forEach,
    nativeMap          = ArrayProto.map,
    nativeReduce       = ArrayProto.reduce,
    nativeReduceRight  = ArrayProto.reduceRight,
    nativeFilter       = ArrayProto.filter,
    nativeEvery        = ArrayProto.every,
    nativeSome         = ArrayProto.some,
    nativeIndexOf      = ArrayProto.indexOf,
    nativeLastIndexOf  = ArrayProto.lastIndexOf,
    nativeIsArray      = Array.isArray,
    nativeKeys         = Object.keys,
    nativeBind         = FuncProto.bind;

  // Create a safe reference to the Underscore object for use below.
  var _ = function(obj) {
    if (obj instanceof _) return obj;
    if (!(this instanceof _)) return new _(obj);
    this._wrapped = obj;
  };

  // Export the Underscore object for **Node.js**, with
  // backwards-compatibility for the old `require()` API. If we're in
  // the browser, add `_` as a global object via a string identifier,
  // for Closure Compiler "advanced" mode.
  if (typeof exports !== 'undefined') {
    if (typeof module !== 'undefined' && module.exports) {
      exports = module.exports = _;
    }
    exports._ = _;
  } else {
    root._ = _;
  }

  // Current version.
  _.VERSION = '1.6.0';

  // Collection Functions
  // --------------------

  // The cornerstone, an `each` implementation, aka `forEach`.
  // Handles objects with the built-in `forEach`, arrays, and raw objects.
  // Delegates to **ECMAScript 5**'s native `forEach` if available.
  var each = _.each = _.forEach = function(obj, iterator, context) {
    if (obj == null) return obj;
    if (nativeForEach && obj.forEach === nativeForEach) {
      obj.forEach(iterator, context);
    } else if (obj.length === +obj.length) {
      for (var i = 0, length = obj.length; i < length; i++) {
        if (iterator.call(context, obj[i], i, obj) === breaker) return;
      }
    } else {
      var keys = _.keys(obj);
      for (var i = 0, length = keys.length; i < length; i++) {
        if (iterator.call(context, obj[keys[i]], keys[i], obj) === breaker) return;
      }
    }
    return obj;
  };

  // Return the results of applying the iterator to each element.
  // Delegates to **ECMAScript 5**'s native `map` if available.
  _.map = _.collect = function(obj, iterator, context) {
    var results = [];
    if (obj == null) return results;
    if (nativeMap && obj.map === nativeMap) return obj.map(iterator, context);
    each(obj, function(value, index, list) {
      results.push(iterator.call(context, value, index, list));
    });
    return results;
  };

  var reduceError = 'Reduce of empty array with no initial value';

  // **Reduce** builds up a single result from a list of values, aka `inject`,
  // or `foldl`. Delegates to **ECMAScript 5**'s native `reduce` if available.
  _.reduce = _.foldl = _.inject = function(obj, iterator, memo, context) {
    var initial = arguments.length > 2;
    if (obj == null) obj = [];
    if (nativeReduce && obj.reduce === nativeReduce) {
      if (context) iterator = _.bind(iterator, context);
      return initial ? obj.reduce(iterator, memo) : obj.reduce(iterator);
    }
    each(obj, function(value, index, list) {
      if (!initial) {
        memo = value;
        initial = true;
      } else {
        memo = iterator.call(context, memo, value, index, list);
      }
    });
    if (!initial) throw new TypeError(reduceError);
    return memo;
  };

  // The right-associative version of reduce, also known as `foldr`.
  // Delegates to **ECMAScript 5**'s native `reduceRight` if available.
  _.reduceRight = _.foldr = function(obj, iterator, memo, context) {
    var initial = arguments.length > 2;
    if (obj == null) obj = [];
    if (nativeReduceRight && obj.reduceRight === nativeReduceRight) {
      if (context) iterator = _.bind(iterator, context);
      return initial ? obj.reduceRight(iterator, memo) : obj.reduceRight(iterator);
    }
    var length = obj.length;
    if (length !== +length) {
      var keys = _.keys(obj);
      length = keys.length;
    }
    each(obj, function(value, index, list) {
      index = keys ? keys[--length] : --length;
      if (!initial) {
        memo = obj[index];
        initial = true;
      } else {
        memo = iterator.call(context, memo, obj[index], index, list);
      }
    });
    if (!initial) throw new TypeError(reduceError);
    return memo;
  };

  // Return the first value which passes a truth test. Aliased as `detect`.
  _.find = _.detect = function(obj, predicate, context) {
    var result;
    any(obj, function(value, index, list) {
      if (predicate.call(context, value, index, list)) {
        result = value;
        return true;
      }
    });
    return result;
  };

  // Return all the elements that pass a truth test.
  // Delegates to **ECMAScript 5**'s native `filter` if available.
  // Aliased as `select`.
  _.filter = _.select = function(obj, predicate, context) {
    var results = [];
    if (obj == null) return results;
    if (nativeFilter && obj.filter === nativeFilter) return obj.filter(predicate, context);
    each(obj, function(value, index, list) {
      if (predicate.call(context, value, index, list)) results.push(value);
    });
    return results;
  };

  // Return all the elements for which a truth test fails.
  _.reject = function(obj, predicate, context) {
    return _.filter(obj, function(value, index, list) {
      return !predicate.call(context, value, index, list);
    }, context);
  };

  // Determine whether all of the elements match a truth test.
  // Delegates to **ECMAScript 5**'s native `every` if available.
  // Aliased as `all`.
  _.every = _.all = function(obj, predicate, context) {
    predicate || (predicate = _.identity);
    var result = true;
    if (obj == null) return result;
    if (nativeEvery && obj.every === nativeEvery) return obj.every(predicate, context);
    each(obj, function(value, index, list) {
      if (!(result = result && predicate.call(context, value, index, list))) return breaker;
    });
    return !!result;
  };

  // Determine if at least one element in the object matches a truth test.
  // Delegates to **ECMAScript 5**'s native `some` if available.
  // Aliased as `any`.
  var any = _.some = _.any = function(obj, predicate, context) {
    predicate || (predicate = _.identity);
    var result = false;
    if (obj == null) return result;
    if (nativeSome && obj.some === nativeSome) return obj.some(predicate, context);
    each(obj, function(value, index, list) {
      if (result || (result = predicate.call(context, value, index, list))) return breaker;
    });
    return !!result;
  };

  // Determine if the array or object contains a given value (using `===`).
  // Aliased as `include`.
  _.contains = _.include = function(obj, target) {
    if (obj == null) return false;
    if (nativeIndexOf && obj.indexOf === nativeIndexOf) return obj.indexOf(target) != -1;
    return any(obj, function(value) {
      return value === target;
    });
  };

  // Invoke a method (with arguments) on every item in a collection.
  _.invoke = function(obj, method) {
    var args = slice.call(arguments, 2);
    var isFunc = _.isFunction(method);
    return _.map(obj, function(value) {
      return (isFunc ? method : value[method]).apply(value, args);
    });
  };

  // Convenience version of a common use case of `map`: fetching a property.
  _.pluck = function(obj, key) {
    return _.map(obj, _.property(key));
  };

  // Convenience version of a common use case of `filter`: selecting only objects
  // containing specific `key:value` pairs.
  _.where = function(obj, attrs) {
    return _.filter(obj, _.matches(attrs));
  };

  // Convenience version of a common use case of `find`: getting the first object
  // containing specific `key:value` pairs.
  _.findWhere = function(obj, attrs) {
    return _.find(obj, _.matches(attrs));
  };

  // Return the maximum element or (element-based computation).
  // Can't optimize arrays of integers longer than 65,535 elements.
  // See [WebKit Bug 80797](https://bugs.webkit.org/show_bug.cgi?id=80797)
  _.max = function(obj, iterator, context) {
    if (!iterator && _.isArray(obj) && obj[0] === +obj[0] && obj.length < 65535) {
      return Math.max.apply(Math, obj);
    }
    var result = -Infinity, lastComputed = -Infinity;
    each(obj, function(value, index, list) {
      var computed = iterator ? iterator.call(context, value, index, list) : value;
      if (computed > lastComputed) {
        result = value;
        lastComputed = computed;
      }
    });
    return result;
  };

  // Return the minimum element (or element-based computation).
  _.min = function(obj, iterator, context) {
    if (!iterator && _.isArray(obj) && obj[0] === +obj[0] && obj.length < 65535) {
      return Math.min.apply(Math, obj);
    }
    var result = Infinity, lastComputed = Infinity;
    each(obj, function(value, index, list) {
      var computed = iterator ? iterator.call(context, value, index, list) : value;
      if (computed < lastComputed) {
        result = value;
        lastComputed = computed;
      }
    });
    return result;
  };

  // Shuffle an array, using the modern version of the
  // [Fisher-Yates shuffle](http://en.wikipedia.org/wiki/Fisher–Yates_shuffle).
  _.shuffle = function(obj) {
    var rand;
    var index = 0;
    var shuffled = [];
    each(obj, function(value) {
      rand = _.random(index++);
      shuffled[index - 1] = shuffled[rand];
      shuffled[rand] = value;
    });
    return shuffled;
  };

  // Sample **n** random values from a collection.
  // If **n** is not specified, returns a single random element.
  // The internal `guard` argument allows it to work with `map`.
  _.sample = function(obj, n, guard) {
    if (n == null || guard) {
      if (obj.length !== +obj.length) obj = _.values(obj);
      return obj[_.random(obj.length - 1)];
    }
    return _.shuffle(obj).slice(0, Math.max(0, n));
  };

  // An internal function to generate lookup iterators.
  var lookupIterator = function(value) {
    if (value == null) return _.identity;
    if (_.isFunction(value)) return value;
    return _.property(value);
  };

  // Sort the object's values by a criterion produced by an iterator.
  _.sortBy = function(obj, iterator, context) {
    iterator = lookupIterator(iterator);
    return _.pluck(_.map(obj, function(value, index, list) {
      return {
        value: value,
        index: index,
        criteria: iterator.call(context, value, index, list)
      };
    }).sort(function(left, right) {
      var a = left.criteria;
      var b = right.criteria;
      if (a !== b) {
        if (a > b || a === void 0) return 1;
        if (a < b || b === void 0) return -1;
      }
      return left.index - right.index;
    }), 'value');
  };

  // An internal function used for aggregate "group by" operations.
  var group = function(behavior) {
    return function(obj, iterator, context) {
      var result = {};
      iterator = lookupIterator(iterator);
      each(obj, function(value, index) {
        var key = iterator.call(context, value, index, obj);
        behavior(result, key, value);
      });
      return result;
    };
  };

  // Groups the object's values by a criterion. Pass either a string attribute
  // to group by, or a function that returns the criterion.
  _.groupBy = group(function(result, key, value) {
    _.has(result, key) ? result[key].push(value) : result[key] = [value];
  });

  // Indexes the object's values by a criterion, similar to `groupBy`, but for
  // when you know that your index values will be unique.
  _.indexBy = group(function(result, key, value) {
    result[key] = value;
  });

  // Counts instances of an object that group by a certain criterion. Pass
  // either a string attribute to count by, or a function that returns the
  // criterion.
  _.countBy = group(function(result, key) {
    _.has(result, key) ? result[key]++ : result[key] = 1;
  });

  // Use a comparator function to figure out the smallest index at which
  // an object should be inserted so as to maintain order. Uses binary search.
  _.sortedIndex = function(array, obj, iterator, context) {
    iterator = lookupIterator(iterator);
    var value = iterator.call(context, obj);
    var low = 0, high = array.length;
    while (low < high) {
      var mid = (low + high) >>> 1;
      iterator.call(context, array[mid]) < value ? low = mid + 1 : high = mid;
    }
    return low;
  };

  // Safely create a real, live array from anything iterable.
  _.toArray = function(obj) {
    if (!obj) return [];
    if (_.isArray(obj)) return slice.call(obj);
    if (obj.length === +obj.length) return _.map(obj, _.identity);
    return _.values(obj);
  };

  // Return the number of elements in an object.
  _.size = function(obj) {
    if (obj == null) return 0;
    return (obj.length === +obj.length) ? obj.length : _.keys(obj).length;
  };

  // Array Functions
  // ---------------

  // Get the first element of an array. Passing **n** will return the first N
  // values in the array. Aliased as `head` and `take`. The **guard** check
  // allows it to work with `_.map`.
  _.first = _.head = _.take = function(array, n, guard) {
    if (array == null) return void 0;
    if ((n == null) || guard) return array[0];
    if (n < 0) return [];
    return slice.call(array, 0, n);
  };

  // Returns everything but the last entry of the array. Especially useful on
  // the arguments object. Passing **n** will return all the values in
  // the array, excluding the last N. The **guard** check allows it to work with
  // `_.map`.
  _.initial = function(array, n, guard) {
    return slice.call(array, 0, array.length - ((n == null) || guard ? 1 : n));
  };

  // Get the last element of an array. Passing **n** will return the last N
  // values in the array. The **guard** check allows it to work with `_.map`.
  _.last = function(array, n, guard) {
    if (array == null) return void 0;
    if ((n == null) || guard) return array[array.length - 1];
    return slice.call(array, Math.max(array.length - n, 0));
  };

  // Returns everything but the first entry of the array. Aliased as `tail` and `drop`.
  // Especially useful on the arguments object. Passing an **n** will return
  // the rest N values in the array. The **guard**
  // check allows it to work with `_.map`.
  _.rest = _.tail = _.drop = function(array, n, guard) {
    return slice.call(array, (n == null) || guard ? 1 : n);
  };

  // Trim out all falsy values from an array.
  _.compact = function(array) {
    return _.filter(array, _.identity);
  };

  // Internal implementation of a recursive `flatten` function.
  var flatten = function(input, shallow, output) {
    if (shallow && _.every(input, _.isArray)) {
      return concat.apply(output, input);
    }
    each(input, function(value) {
      if (_.isArray(value) || _.isArguments(value)) {
        shallow ? push.apply(output, value) : flatten(value, shallow, output);
      } else {
        output.push(value);
      }
    });
    return output;
  };

  // Flatten out an array, either recursively (by default), or just one level.
  _.flatten = function(array, shallow) {
    return flatten(array, shallow, []);
  };

  // Return a version of the array that does not contain the specified value(s).
  _.without = function(array) {
    return _.difference(array, slice.call(arguments, 1));
  };

  // Split an array into two arrays: one whose elements all satisfy the given
  // predicate, and one whose elements all do not satisfy the predicate.
  _.partition = function(array, predicate) {
    var pass = [], fail = [];
    each(array, function(elem) {
      (predicate(elem) ? pass : fail).push(elem);
    });
    return [pass, fail];
  };

  // Produce a duplicate-free version of the array. If the array has already
  // been sorted, you have the option of using a faster algorithm.
  // Aliased as `unique`.
  _.uniq = _.unique = function(array, isSorted, iterator, context) {
    if (_.isFunction(isSorted)) {
      context = iterator;
      iterator = isSorted;
      isSorted = false;
    }
    var initial = iterator ? _.map(array, iterator, context) : array;
    var results = [];
    var seen = [];
    each(initial, function(value, index) {
      if (isSorted ? (!index || seen[seen.length - 1] !== value) : !_.contains(seen, value)) {
        seen.push(value);
        results.push(array[index]);
      }
    });
    return results;
  };

  // Produce an array that contains the union: each distinct element from all of
  // the passed-in arrays.
  _.union = function() {
    return _.uniq(_.flatten(arguments, true));
  };

  // Produce an array that contains every item shared between all the
  // passed-in arrays.
  _.intersection = function(array) {
    var rest = slice.call(arguments, 1);
    return _.filter(_.uniq(array), function(item) {
      return _.every(rest, function(other) {
        return _.contains(other, item);
      });
    });
  };

  // Take the difference between one array and a number of other arrays.
  // Only the elements present in just the first array will remain.
  _.difference = function(array) {
    var rest = concat.apply(ArrayProto, slice.call(arguments, 1));
    return _.filter(array, function(value){ return !_.contains(rest, value); });
  };

  // Zip together multiple lists into a single array -- elements that share
  // an index go together.
  _.zip = function() {
    var length = _.max(_.pluck(arguments, 'length').concat(0));
    var results = new Array(length);
    for (var i = 0; i < length; i++) {
      results[i] = _.pluck(arguments, '' + i);
    }
    return results;
  };

  // Converts lists into objects. Pass either a single array of `[key, value]`
  // pairs, or two parallel arrays of the same length -- one of keys, and one of
  // the corresponding values.
  _.object = function(list, values) {
    if (list == null) return {};
    var result = {};
    for (var i = 0, length = list.length; i < length; i++) {
      if (values) {
        result[list[i]] = values[i];
      } else {
        result[list[i][0]] = list[i][1];
      }
    }
    return result;
  };

  // If the browser doesn't supply us with indexOf (I'm looking at you, **MSIE**),
  // we need this function. Return the position of the first occurrence of an
  // item in an array, or -1 if the item is not included in the array.
  // Delegates to **ECMAScript 5**'s native `indexOf` if available.
  // If the array is large and already in sort order, pass `true`
  // for **isSorted** to use binary search.
  _.indexOf = function(array, item, isSorted) {
    if (array == null) return -1;
    var i = 0, length = array.length;
    if (isSorted) {
      if (typeof isSorted == 'number') {
        i = (isSorted < 0 ? Math.max(0, length + isSorted) : isSorted);
      } else {
        i = _.sortedIndex(array, item);
        return array[i] === item ? i : -1;
      }
    }
    if (nativeIndexOf && array.indexOf === nativeIndexOf) return array.indexOf(item, isSorted);
    for (; i < length; i++) if (array[i] === item) return i;
    return -1;
  };

  // Delegates to **ECMAScript 5**'s native `lastIndexOf` if available.
  _.lastIndexOf = function(array, item, from) {
    if (array == null) return -1;
    var hasIndex = from != null;
    if (nativeLastIndexOf && array.lastIndexOf === nativeLastIndexOf) {
      return hasIndex ? array.lastIndexOf(item, from) : array.lastIndexOf(item);
    }
    var i = (hasIndex ? from : array.length);
    while (i--) if (array[i] === item) return i;
    return -1;
  };

  // Generate an integer Array containing an arithmetic progression. A port of
  // the native Python `range()` function. See
  // [the Python documentation](http://docs.python.org/library/functions.html#range).
  _.range = function(start, stop, step) {
    if (arguments.length <= 1) {
      stop = start || 0;
      start = 0;
    }
    step = arguments[2] || 1;

    var length = Math.max(Math.ceil((stop - start) / step), 0);
    var idx = 0;
    var range = new Array(length);

    while(idx < length) {
      range[idx++] = start;
      start += step;
    }

    return range;
  };

  // Function (ahem) Functions
  // ------------------

  // Reusable constructor function for prototype setting.
  var ctor = function(){};

  // Create a function bound to a given object (assigning `this`, and arguments,
  // optionally). Delegates to **ECMAScript 5**'s native `Function.bind` if
  // available.
  _.bind = function(func, context) {
    var args, bound;
    if (nativeBind && func.bind === nativeBind) return nativeBind.apply(func, slice.call(arguments, 1));
    if (!_.isFunction(func)) throw new TypeError;
    args = slice.call(arguments, 2);
    return bound = function() {
      if (!(this instanceof bound)) return func.apply(context, args.concat(slice.call(arguments)));
      ctor.prototype = func.prototype;
      var self = new ctor;
      ctor.prototype = null;
      var result = func.apply(self, args.concat(slice.call(arguments)));
      if (Object(result) === result) return result;
      return self;
    };
  };

  // Partially apply a function by creating a version that has had some of its
  // arguments pre-filled, without changing its dynamic `this` context. _ acts
  // as a placeholder, allowing any combination of arguments to be pre-filled.
  _.partial = function(func) {
    var boundArgs = slice.call(arguments, 1);
    return function() {
      var position = 0;
      var args = boundArgs.slice();
      for (var i = 0, length = args.length; i < length; i++) {
        if (args[i] === _) args[i] = arguments[position++];
      }
      while (position < arguments.length) args.push(arguments[position++]);
      return func.apply(this, args);
    };
  };

  // Bind a number of an object's methods to that object. Remaining arguments
  // are the method names to be bound. Useful for ensuring that all callbacks
  // defined on an object belong to it.
  _.bindAll = function(obj) {
    var funcs = slice.call(arguments, 1);
    if (funcs.length === 0) throw new Error('bindAll must be passed function names');
    each(funcs, function(f) { obj[f] = _.bind(obj[f], obj); });
    return obj;
  };

  // Memoize an expensive function by storing its results.
  _.memoize = function(func, hasher) {
    var memo = {};
    hasher || (hasher = _.identity);
    return function() {
      var key = hasher.apply(this, arguments);
      return _.has(memo, key) ? memo[key] : (memo[key] = func.apply(this, arguments));
    };
  };

  // Delays a function for the given number of milliseconds, and then calls
  // it with the arguments supplied.
  _.delay = function(func, wait) {
    var args = slice.call(arguments, 2);
    return setTimeout(function(){ return func.apply(null, args); }, wait);
  };

  // Defers a function, scheduling it to run after the current call stack has
  // cleared.
  _.defer = function(func) {
    return _.delay.apply(_, [func, 1].concat(slice.call(arguments, 1)));
  };

  // Returns a function, that, when invoked, will only be triggered at most once
  // during a given window of time. Normally, the throttled function will run
  // as much as it can, without ever going more than once per `wait` duration;
  // but if you'd like to disable the execution on the leading edge, pass
  // `{leading: false}`. To disable execution on the trailing edge, ditto.
  _.throttle = function(func, wait, options) {
    var context, args, result;
    var timeout = null;
    var previous = 0;
    options || (options = {});
    var later = function() {
      previous = options.leading === false ? 0 : _.now();
      timeout = null;
      result = func.apply(context, args);
      context = args = null;
    };
    return function() {
      var now = _.now();
      if (!previous && options.leading === false) previous = now;
      var remaining = wait - (now - previous);
      context = this;
      args = arguments;
      if (remaining <= 0) {
        clearTimeout(timeout);
        timeout = null;
        previous = now;
        result = func.apply(context, args);
        context = args = null;
      } else if (!timeout && options.trailing !== false) {
        timeout = setTimeout(later, remaining);
      }
      return result;
    };
  };

  // Returns a function, that, as long as it continues to be invoked, will not
  // be triggered. The function will be called after it stops being called for
  // N milliseconds. If `immediate` is passed, trigger the function on the
  // leading edge, instead of the trailing.
  _.debounce = function(func, wait, immediate) {
    var timeout, args, context, timestamp, result;

    var later = function() {
      var last = _.now() - timestamp;
      if (last < wait) {
        timeout = setTimeout(later, wait - last);
      } else {
        timeout = null;
        if (!immediate) {
          result = func.apply(context, args);
          context = args = null;
        }
      }
    };

    return function() {
      context = this;
      args = arguments;
      timestamp = _.now();
      var callNow = immediate && !timeout;
      if (!timeout) {
        timeout = setTimeout(later, wait);
      }
      if (callNow) {
        result = func.apply(context, args);
        context = args = null;
      }

      return result;
    };
  };

  // Returns a function that will be executed at most one time, no matter how
  // often you call it. Useful for lazy initialization.
  _.once = function(func) {
    var ran = false, memo;
    return function() {
      if (ran) return memo;
      ran = true;
      memo = func.apply(this, arguments);
      func = null;
      return memo;
    };
  };

  // Returns the first function passed as an argument to the second,
  // allowing you to adjust arguments, run code before and after, and
  // conditionally execute the original function.
  _.wrap = function(func, wrapper) {
    return _.partial(wrapper, func);
  };

  // Returns a function that is the composition of a list of functions, each
  // consuming the return value of the function that follows.
  _.compose = function() {
    var funcs = arguments;
    return function() {
      var args = arguments;
      for (var i = funcs.length - 1; i >= 0; i--) {
        args = [funcs[i].apply(this, args)];
      }
      return args[0];
    };
  };

  // Returns a function that will only be executed after being called N times.
  _.after = function(times, func) {
    return function() {
      if (--times < 1) {
        return func.apply(this, arguments);
      }
    };
  };

  // Object Functions
  // ----------------

  // Retrieve the names of an object's properties.
  // Delegates to **ECMAScript 5**'s native `Object.keys`
  _.keys = function(obj) {
    if (!_.isObject(obj)) return [];
    if (nativeKeys) return nativeKeys(obj);
    var keys = [];
    for (var key in obj) if (_.has(obj, key)) keys.push(key);
    return keys;
  };

  // Retrieve the values of an object's properties.
  _.values = function(obj) {
    var keys = _.keys(obj);
    var length = keys.length;
    var values = new Array(length);
    for (var i = 0; i < length; i++) {
      values[i] = obj[keys[i]];
    }
    return values;
  };

  // Convert an object into a list of `[key, value]` pairs.
  _.pairs = function(obj) {
    var keys = _.keys(obj);
    var length = keys.length;
    var pairs = new Array(length);
    for (var i = 0; i < length; i++) {
      pairs[i] = [keys[i], obj[keys[i]]];
    }
    return pairs;
  };

  // Invert the keys and values of an object. The values must be serializable.
  _.invert = function(obj) {
    var result = {};
    var keys = _.keys(obj);
    for (var i = 0, length = keys.length; i < length; i++) {
      result[obj[keys[i]]] = keys[i];
    }
    return result;
  };

  // Return a sorted list of the function names available on the object.
  // Aliased as `methods`
  _.functions = _.methods = function(obj) {
    var names = [];
    for (var key in obj) {
      if (_.isFunction(obj[key])) names.push(key);
    }
    return names.sort();
  };

  // Extend a given object with all the properties in passed-in object(s).
  _.extend = function(obj) {
    each(slice.call(arguments, 1), function(source) {
      if (source) {
        for (var prop in source) {
          obj[prop] = source[prop];
        }
      }
    });
    return obj;
  };

  // Return a copy of the object only containing the whitelisted properties.
  _.pick = function(obj) {
    var copy = {};
    var keys = concat.apply(ArrayProto, slice.call(arguments, 1));
    each(keys, function(key) {
      if (key in obj) copy[key] = obj[key];
    });
    return copy;
  };

   // Return a copy of the object without the blacklisted properties.
  _.omit = function(obj) {
    var copy = {};
    var keys = concat.apply(ArrayProto, slice.call(arguments, 1));
    for (var key in obj) {
      if (!_.contains(keys, key)) copy[key] = obj[key];
    }
    return copy;
  };

  // Fill in a given object with default properties.
  _.defaults = function(obj) {
    each(slice.call(arguments, 1), function(source) {
      if (source) {
        for (var prop in source) {
          if (obj[prop] === void 0) obj[prop] = source[prop];
        }
      }
    });
    return obj;
  };

  // Create a (shallow-cloned) duplicate of an object.
  _.clone = function(obj) {
    if (!_.isObject(obj)) return obj;
    return _.isArray(obj) ? obj.slice() : _.extend({}, obj);
  };

  // Invokes interceptor with the obj, and then returns obj.
  // The primary purpose of this method is to "tap into" a method chain, in
  // order to perform operations on intermediate results within the chain.
  _.tap = function(obj, interceptor) {
    interceptor(obj);
    return obj;
  };

  // Internal recursive comparison function for `isEqual`.
  var eq = function(a, b, aStack, bStack) {
    // Identical objects are equal. `0 === -0`, but they aren't identical.
    // See the [Harmony `egal` proposal](http://wiki.ecmascript.org/doku.php?id=harmony:egal).
    if (a === b) return a !== 0 || 1 / a == 1 / b;
    // A strict comparison is necessary because `null == undefined`.
    if (a == null || b == null) return a === b;
    // Unwrap any wrapped objects.
    if (a instanceof _) a = a._wrapped;
    if (b instanceof _) b = b._wrapped;
    // Compare `[[Class]]` names.
    var className = toString.call(a);
    if (className != toString.call(b)) return false;
    switch (className) {
      // Strings, numbers, dates, and booleans are compared by value.
      case '[object String]':
        // Primitives and their corresponding object wrappers are equivalent; thus, `"5"` is
        // equivalent to `new String("5")`.
        return a == String(b);
      case '[object Number]':
        // `NaN`s are equivalent, but non-reflexive. An `egal` comparison is performed for
        // other numeric values.
        return a != +a ? b != +b : (a == 0 ? 1 / a == 1 / b : a == +b);
      case '[object Date]':
      case '[object Boolean]':
        // Coerce dates and booleans to numeric primitive values. Dates are compared by their
        // millisecond representations. Note that invalid dates with millisecond representations
        // of `NaN` are not equivalent.
        return +a == +b;
      // RegExps are compared by their source patterns and flags.
      case '[object RegExp]':
        return a.source == b.source &&
               a.global == b.global &&
               a.multiline == b.multiline &&
               a.ignoreCase == b.ignoreCase;
    }
    if (typeof a != 'object' || typeof b != 'object') return false;
    // Assume equality for cyclic structures. The algorithm for detecting cyclic
    // structures is adapted from ES 5.1 section 15.12.3, abstract operation `JO`.
    var length = aStack.length;
    while (length--) {
      // Linear search. Performance is inversely proportional to the number of
      // unique nested structures.
      if (aStack[length] == a) return bStack[length] == b;
    }
    // Objects with different constructors are not equivalent, but `Object`s
    // from different frames are.
    var aCtor = a.constructor, bCtor = b.constructor;
    if (aCtor !== bCtor && !(_.isFunction(aCtor) && (aCtor instanceof aCtor) &&
                             _.isFunction(bCtor) && (bCtor instanceof bCtor))
                        && ('constructor' in a && 'constructor' in b)) {
      return false;
    }
    // Add the first object to the stack of traversed objects.
    aStack.push(a);
    bStack.push(b);
    var size = 0, result = true;
    // Recursively compare objects and arrays.
    if (className == '[object Array]') {
      // Compare array lengths to determine if a deep comparison is necessary.
      size = a.length;
      result = size == b.length;
      if (result) {
        // Deep compare the contents, ignoring non-numeric properties.
        while (size--) {
          if (!(result = eq(a[size], b[size], aStack, bStack))) break;
        }
      }
    } else {
      // Deep compare objects.
      for (var key in a) {
        if (_.has(a, key)) {
          // Count the expected number of properties.
          size++;
          // Deep compare each member.
          if (!(result = _.has(b, key) && eq(a[key], b[key], aStack, bStack))) break;
        }
      }
      // Ensure that both objects contain the same number of properties.
      if (result) {
        for (key in b) {
          if (_.has(b, key) && !(size--)) break;
        }
        result = !size;
      }
    }
    // Remove the first object from the stack of traversed objects.
    aStack.pop();
    bStack.pop();
    return result;
  };

  // Perform a deep comparison to check if two objects are equal.
  _.isEqual = function(a, b) {
    return eq(a, b, [], []);
  };

  // Is a given array, string, or object empty?
  // An "empty" object has no enumerable own-properties.
  _.isEmpty = function(obj) {
    if (obj == null) return true;
    if (_.isArray(obj) || _.isString(obj)) return obj.length === 0;
    for (var key in obj) if (_.has(obj, key)) return false;
    return true;
  };

  // Is a given value a DOM element?
  _.isElement = function(obj) {
    return !!(obj && obj.nodeType === 1);
  };

  // Is a given value an array?
  // Delegates to ECMA5's native Array.isArray
  _.isArray = nativeIsArray || function(obj) {
    return toString.call(obj) == '[object Array]';
  };

  // Is a given variable an object?
  _.isObject = function(obj) {
    return obj === Object(obj);
  };

  // Add some isType methods: isArguments, isFunction, isString, isNumber, isDate, isRegExp.
  each(['Arguments', 'Function', 'String', 'Number', 'Date', 'RegExp'], function(name) {
    _['is' + name] = function(obj) {
      return toString.call(obj) == '[object ' + name + ']';
    };
  });

  // Define a fallback version of the method in browsers (ahem, IE), where
  // there isn't any inspectable "Arguments" type.
  if (!_.isArguments(arguments)) {
    _.isArguments = function(obj) {
      return !!(obj && _.has(obj, 'callee'));
    };
  }

  // Optimize `isFunction` if appropriate.
  if (typeof (/./) !== 'function') {
    _.isFunction = function(obj) {
      return typeof obj === 'function';
    };
  }

  // Is a given object a finite number?
  _.isFinite = function(obj) {
    return isFinite(obj) && !isNaN(parseFloat(obj));
  };

  // Is the given value `NaN`? (NaN is the only number which does not equal itself).
  _.isNaN = function(obj) {
    return _.isNumber(obj) && obj != +obj;
  };

  // Is a given value a boolean?
  _.isBoolean = function(obj) {
    return obj === true || obj === false || toString.call(obj) == '[object Boolean]';
  };

  // Is a given value equal to null?
  _.isNull = function(obj) {
    return obj === null;
  };

  // Is a given variable undefined?
  _.isUndefined = function(obj) {
    return obj === void 0;
  };

  // Shortcut function for checking if an object has a given property directly
  // on itself (in other words, not on a prototype).
  _.has = function(obj, key) {
    return hasOwnProperty.call(obj, key);
  };

  // Utility Functions
  // -----------------

  // Run Underscore.js in *noConflict* mode, returning the `_` variable to its
  // previous owner. Returns a reference to the Underscore object.
  _.noConflict = function() {
    root._ = previousUnderscore;
    return this;
  };

  // Keep the identity function around for default iterators.
  _.identity = function(value) {
    return value;
  };

  _.constant = function(value) {
    return function () {
      return value;
    };
  };

  _.property = function(key) {
    return function(obj) {
      return obj[key];
    };
  };

  // Returns a predicate for checking whether an object has a given set of `key:value` pairs.
  _.matches = function(attrs) {
    return function(obj) {
      if (obj === attrs) return true; //avoid comparing an object to itself.
      for (var key in attrs) {
        if (attrs[key] !== obj[key])
          return false;
      }
      return true;
    }
  };

  // Run a function **n** times.
  _.times = function(n, iterator, context) {
    var accum = Array(Math.max(0, n));
    for (var i = 0; i < n; i++) accum[i] = iterator.call(context, i);
    return accum;
  };

  // Return a random integer between min and max (inclusive).
  _.random = function(min, max) {
    if (max == null) {
      max = min;
      min = 0;
    }
    return min + Math.floor(Math.random() * (max - min + 1));
  };

  // A (possibly faster) way to get the current timestamp as an integer.
  _.now = Date.now || function() { return new Date().getTime(); };

  // List of HTML entities for escaping.
  var entityMap = {
    escape: {
      '&': '&amp;',
      '<': '&lt;',
      '>': '&gt;',
      '"': '&quot;',
      "'": '&#x27;'
    }
  };
  entityMap.unescape = _.invert(entityMap.escape);

  // Regexes containing the keys and values listed immediately above.
  var entityRegexes = {
    escape:   new RegExp('[' + _.keys(entityMap.escape).join('') + ']', 'g'),
    unescape: new RegExp('(' + _.keys(entityMap.unescape).join('|') + ')', 'g')
  };

  // Functions for escaping and unescaping strings to/from HTML interpolation.
  _.each(['escape', 'unescape'], function(method) {
    _[method] = function(string) {
      if (string == null) return '';
      return ('' + string).replace(entityRegexes[method], function(match) {
        return entityMap[method][match];
      });
    };
  });

  // If the value of the named `property` is a function then invoke it with the
  // `object` as context; otherwise, return it.
  _.result = function(object, property) {
    if (object == null) return void 0;
    var value = object[property];
    return _.isFunction(value) ? value.call(object) : value;
  };

  // Add your own custom functions to the Underscore object.
  _.mixin = function(obj) {
    each(_.functions(obj), function(name) {
      var func = _[name] = obj[name];
      _.prototype[name] = function() {
        var args = [this._wrapped];
        push.apply(args, arguments);
        return result.call(this, func.apply(_, args));
      };
    });
  };

  // Generate a unique integer id (unique within the entire client session).
  // Useful for temporary DOM ids.
  var idCounter = 0;
  _.uniqueId = function(prefix) {
    var id = ++idCounter + '';
    return prefix ? prefix + id : id;
  };

  // By default, Underscore uses ERB-style template delimiters, change the
  // following template settings to use alternative delimiters.
  _.templateSettings = {
    evaluate    : /<%([\s\S]+?)%>/g,
    interpolate : /<%=([\s\S]+?)%>/g,
    escape      : /<%-([\s\S]+?)%>/g
  };

  // When customizing `templateSettings`, if you don't want to define an
  // interpolation, evaluation or escaping regex, we need one that is
  // guaranteed not to match.
  var noMatch = /(.)^/;

  // Certain characters need to be escaped so that they can be put into a
  // string literal.
  var escapes = {
    "'":      "'",
    '\\':     '\\',
    '\r':     'r',
    '\n':     'n',
    '\t':     't',
    '\u2028': 'u2028',
    '\u2029': 'u2029'
  };

  var escaper = /\\|'|\r|\n|\t|\u2028|\u2029/g;

  // JavaScript micro-templating, similar to John Resig's implementation.
  // Underscore templating handles arbitrary delimiters, preserves whitespace,
  // and correctly escapes quotes within interpolated code.
  _.template = function(text, data, settings) {
    var render;
    settings = _.defaults({}, settings, _.templateSettings);

    // Combine delimiters into one regular expression via alternation.
    var matcher = new RegExp([
      (settings.escape || noMatch).source,
      (settings.interpolate || noMatch).source,
      (settings.evaluate || noMatch).source
    ].join('|') + '|$', 'g');

    // Compile the template source, escaping string literals appropriately.
    var index = 0;
    var source = "__p+='";
    text.replace(matcher, function(match, escape, interpolate, evaluate, offset) {
      source += text.slice(index, offset)
        .replace(escaper, function(match) { return '\\' + escapes[match]; });

      if (escape) {
        source += "'+\n((__t=(" + escape + "))==null?'':_.escape(__t))+\n'";
      }
      if (interpolate) {
        source += "'+\n((__t=(" + interpolate + "))==null?'':__t)+\n'";
      }
      if (evaluate) {
        source += "';\n" + evaluate + "\n__p+='";
      }
      index = offset + match.length;
      return match;
    });
    source += "';\n";

    // If a variable is not specified, place data values in local scope.
    if (!settings.variable) source = 'with(obj||{}){\n' + source + '}\n';

    source = "var __t,__p='',__j=Array.prototype.join," +
      "print=function(){__p+=__j.call(arguments,'');};\n" +
      source + "return __p;\n";

    try {
      render = new Function(settings.variable || 'obj', '_', source);
    } catch (e) {
      e.source = source;
      throw e;
    }

    if (data) return render(data, _);
    var template = function(data) {
      return render.call(this, data, _);
    };

    // Provide the compiled function source as a convenience for precompilation.
    template.source = 'function(' + (settings.variable || 'obj') + '){\n' + source + '}';

    return template;
  };

  // Add a "chain" function, which will delegate to the wrapper.
  _.chain = function(obj) {
    return _(obj).chain();
  };

  // OOP
  // ---------------
  // If Underscore is called as a function, it returns a wrapped object that
  // can be used OO-style. This wrapper holds altered versions of all the
  // underscore functions. Wrapped objects may be chained.

  // Helper function to continue chaining intermediate results.
  var result = function(obj) {
    return this._chain ? _(obj).chain() : obj;
  };

  // Add all of the Underscore functions to the wrapper object.
  _.mixin(_);

  // Add all mutator Array functions to the wrapper.
  each(['pop', 'push', 'reverse', 'shift', 'sort', 'splice', 'unshift'], function(name) {
    var method = ArrayProto[name];
    _.prototype[name] = function() {
      var obj = this._wrapped;
      method.apply(obj, arguments);
      if ((name == 'shift' || name == 'splice') && obj.length === 0) delete obj[0];
      return result.call(this, obj);
    };
  });

  // Add all accessor Array functions to the wrapper.
  each(['concat', 'join', 'slice'], function(name) {
    var method = ArrayProto[name];
    _.prototype[name] = function() {
      return result.call(this, method.apply(this._wrapped, arguments));
    };
  });

  _.extend(_.prototype, {

    // Start chaining a wrapped Underscore object.
    chain: function() {
      this._chain = true;
      return this;
    },

    // Extracts the result from a wrapped and chained object.
    value: function() {
      return this._wrapped;
    }

  });

  // AMD registration happens at the end for compatibility with AMD loaders
  // that may not enforce next-turn semantics on modules. Even though general
  // practice for AMD registration is to be anonymous, underscore registers
  // as a named module because, like jQuery, it is a base library that is
  // popular enough to be bundled in a third party lib, but not be part of
  // an AMD load request. Those cases could generate an error when an
  // anonymous define() is called outside of a loader request.
  if (typeof define === 'function' && define.amd) {
    define('underscore', [], function() {
      return _;
    });
  }
}).call(this);

//     Backbone.js 1.1.2

//     (c) 2010-2014 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
//     Backbone may be freely distributed under the MIT license.
//     For all details and documentation:
//     http://backbonejs.org

(function(root, factory) {

  // Set up Backbone appropriately for the environment. Start with AMD.
  if (typeof define === 'function' && define.amd) {
    define('backbone',['underscore', 'jquery', 'exports'], function(_, $, exports) {
      // Export global even in AMD case in case this script is loaded with
      // others that may still expect a global Backbone.
      root.Backbone = factory(root, exports, _, $);
    });

  // Next for Node.js or CommonJS. jQuery may not be needed as a module.
  } else if (typeof exports !== 'undefined') {
    var _ = require('underscore');
    factory(root, exports, _);

  // Finally, as a browser global.
  } else {
    root.Backbone = factory(root, {}, root._, (root.jQuery || root.Zepto || root.ender || root.$));
  }

}(this, function(root, Backbone, _, $) {

  // Initial Setup
  // -------------

  // Save the previous value of the `Backbone` variable, so that it can be
  // restored later on, if `noConflict` is used.
  var previousBackbone = root.Backbone;

  // Create local references to array methods we'll want to use later.
  var array = [];
  var push = array.push;
  var slice = array.slice;
  var splice = array.splice;

  // Current version of the library. Keep in sync with `package.json`.
  Backbone.VERSION = '1.1.2';

  // For Backbone's purposes, jQuery, Zepto, Ender, or My Library (kidding) owns
  // the `$` variable.
  Backbone.$ = $;

  // Runs Backbone.js in *noConflict* mode, returning the `Backbone` variable
  // to its previous owner. Returns a reference to this Backbone object.
  Backbone.noConflict = function() {
    root.Backbone = previousBackbone;
    return this;
  };

  // Turn on `emulateHTTP` to support legacy HTTP servers. Setting this option
  // will fake `"PATCH"`, `"PUT"` and `"DELETE"` requests via the `_method` parameter and
  // set a `X-Http-Method-Override` header.
  Backbone.emulateHTTP = false;

  // Turn on `emulateJSON` to support legacy servers that can't deal with direct
  // `application/json` requests ... will encode the body as
  // `application/x-www-form-urlencoded` instead and will send the model in a
  // form param named `model`.
  Backbone.emulateJSON = false;

  // Backbone.Events
  // ---------------

  // A module that can be mixed in to *any object* in order to provide it with
  // custom events. You may bind with `on` or remove with `off` callback
  // functions to an event; `trigger`-ing an event fires all callbacks in
  // succession.
  //
  //     var object = {};
  //     _.extend(object, Backbone.Events);
  //     object.on('expand', function(){ alert('expanded'); });
  //     object.trigger('expand');
  //
  var Events = Backbone.Events = {

    // Bind an event to a `callback` function. Passing `"all"` will bind
    // the callback to all events fired.
    on: function(name, callback, context) {
      if (!eventsApi(this, 'on', name, [callback, context]) || !callback) return this;
      this._events || (this._events = {});
      var events = this._events[name] || (this._events[name] = []);
      events.push({callback: callback, context: context, ctx: context || this});
      return this;
    },

    // Bind an event to only be triggered a single time. After the first time
    // the callback is invoked, it will be removed.
    once: function(name, callback, context) {
      if (!eventsApi(this, 'once', name, [callback, context]) || !callback) return this;
      var self = this;
      var once = _.once(function() {
        self.off(name, once);
        callback.apply(this, arguments);
      });
      once._callback = callback;
      return this.on(name, once, context);
    },

    // Remove one or many callbacks. If `context` is null, removes all
    // callbacks with that function. If `callback` is null, removes all
    // callbacks for the event. If `name` is null, removes all bound
    // callbacks for all events.
    off: function(name, callback, context) {
      var retain, ev, events, names, i, l, j, k;
      if (!this._events || !eventsApi(this, 'off', name, [callback, context])) return this;
      if (!name && !callback && !context) {
        this._events = void 0;
        return this;
      }
      names = name ? [name] : _.keys(this._events);
      for (i = 0, l = names.length; i < l; i++) {
        name = names[i];
        if (events = this._events[name]) {
          this._events[name] = retain = [];
          if (callback || context) {
            for (j = 0, k = events.length; j < k; j++) {
              ev = events[j];
              if ((callback && callback !== ev.callback && callback !== ev.callback._callback) ||
                  (context && context !== ev.context)) {
                retain.push(ev);
              }
            }
          }
          if (!retain.length) delete this._events[name];
        }
      }

      return this;
    },

    // Trigger one or many events, firing all bound callbacks. Callbacks are
    // passed the same arguments as `trigger` is, apart from the event name
    // (unless you're listening on `"all"`, which will cause your callback to
    // receive the true name of the event as the first argument).
    trigger: function(name) {
      if (!this._events) return this;
      var args = slice.call(arguments, 1);
      if (!eventsApi(this, 'trigger', name, args)) return this;
      var events = this._events[name];
      var allEvents = this._events.all;
      if (events) triggerEvents(events, args);
      if (allEvents) triggerEvents(allEvents, arguments);
      return this;
    },

    // Tell this object to stop listening to either specific events ... or
    // to every object it's currently listening to.
    stopListening: function(obj, name, callback) {
      var listeningTo = this._listeningTo;
      if (!listeningTo) return this;
      var remove = !name && !callback;
      if (!callback && typeof name === 'object') callback = this;
      if (obj) (listeningTo = {})[obj._listenId] = obj;
      for (var id in listeningTo) {
        obj = listeningTo[id];
        obj.off(name, callback, this);
        if (remove || _.isEmpty(obj._events)) delete this._listeningTo[id];
      }
      return this;
    }

  };

  // Regular expression used to split event strings.
  var eventSplitter = /\s+/;

  // Implement fancy features of the Events API such as multiple event
  // names `"change blur"` and jQuery-style event maps `{change: action}`
  // in terms of the existing API.
  var eventsApi = function(obj, action, name, rest) {
    if (!name) return true;

    // Handle event maps.
    if (typeof name === 'object') {
      for (var key in name) {
        obj[action].apply(obj, [key, name[key]].concat(rest));
      }
      return false;
    }

    // Handle space separated event names.
    if (eventSplitter.test(name)) {
      var names = name.split(eventSplitter);
      for (var i = 0, l = names.length; i < l; i++) {
        obj[action].apply(obj, [names[i]].concat(rest));
      }
      return false;
    }

    return true;
  };

  // A difficult-to-believe, but optimized internal dispatch function for
  // triggering events. Tries to keep the usual cases speedy (most internal
  // Backbone events have 3 arguments).
  var triggerEvents = function(events, args) {
    var ev, i = -1, l = events.length, a1 = args[0], a2 = args[1], a3 = args[2];
    switch (args.length) {
      case 0: while (++i < l) (ev = events[i]).callback.call(ev.ctx); return;
      case 1: while (++i < l) (ev = events[i]).callback.call(ev.ctx, a1); return;
      case 2: while (++i < l) (ev = events[i]).callback.call(ev.ctx, a1, a2); return;
      case 3: while (++i < l) (ev = events[i]).callback.call(ev.ctx, a1, a2, a3); return;
      default: while (++i < l) (ev = events[i]).callback.apply(ev.ctx, args); return;
    }
  };

  var listenMethods = {listenTo: 'on', listenToOnce: 'once'};

  // Inversion-of-control versions of `on` and `once`. Tell *this* object to
  // listen to an event in another object ... keeping track of what it's
  // listening to.
  _.each(listenMethods, function(implementation, method) {
    Events[method] = function(obj, name, callback) {
      var listeningTo = this._listeningTo || (this._listeningTo = {});
      var id = obj._listenId || (obj._listenId = _.uniqueId('l'));
      listeningTo[id] = obj;
      if (!callback && typeof name === 'object') callback = this;
      obj[implementation](name, callback, this);
      return this;
    };
  });

  // Aliases for backwards compatibility.
  Events.bind   = Events.on;
  Events.unbind = Events.off;

  // Allow the `Backbone` object to serve as a global event bus, for folks who
  // want global "pubsub" in a convenient place.
  _.extend(Backbone, Events);

  // Backbone.Model
  // --------------

  // Backbone **Models** are the basic data object in the framework --
  // frequently representing a row in a table in a database on your server.
  // A discrete chunk of data and a bunch of useful, related methods for
  // performing computations and transformations on that data.

  // Create a new model with the specified attributes. A client id (`cid`)
  // is automatically generated and assigned for you.
  var Model = Backbone.Model = function(attributes, options) {
    var attrs = attributes || {};
    options || (options = {});
    this.cid = _.uniqueId('c');
    this.attributes = {};
    if (options.collection) this.collection = options.collection;
    if (options.parse) attrs = this.parse(attrs, options) || {};
    attrs = _.defaults({}, attrs, _.result(this, 'defaults'));
    this.set(attrs, options);
    this.changed = {};
    this.initialize.apply(this, arguments);
  };

  // Attach all inheritable methods to the Model prototype.
  _.extend(Model.prototype, Events, {

    // A hash of attributes whose current and previous value differ.
    changed: null,

    // The value returned during the last failed validation.
    validationError: null,

    // The default name for the JSON `id` attribute is `"id"`. MongoDB and
    // CouchDB users may want to set this to `"_id"`.
    idAttribute: 'id',

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // Return a copy of the model's `attributes` object.
    toJSON: function(options) {
      return _.clone(this.attributes);
    },

    // Proxy `Backbone.sync` by default -- but override this if you need
    // custom syncing semantics for *this* particular model.
    sync: function() {
      return Backbone.sync.apply(this, arguments);
    },

    // Get the value of an attribute.
    get: function(attr) {
      return this.attributes[attr];
    },

    // Get the HTML-escaped value of an attribute.
    escape: function(attr) {
      return _.escape(this.get(attr));
    },

    // Returns `true` if the attribute contains a value that is not null
    // or undefined.
    has: function(attr) {
      return this.get(attr) != null;
    },

    // Set a hash of model attributes on the object, firing `"change"`. This is
    // the core primitive operation of a model, updating the data and notifying
    // anyone who needs to know about the change in state. The heart of the beast.
    set: function(key, val, options) {
      var attr, attrs, unset, changes, silent, changing, prev, current;
      if (key == null) return this;

      // Handle both `"key", value` and `{key: value}` -style arguments.
      if (typeof key === 'object') {
        attrs = key;
        options = val;
      } else {
        (attrs = {})[key] = val;
      }

      options || (options = {});

      // Run validation.
      if (!this._validate(attrs, options)) return false;

      // Extract attributes and options.
      unset           = options.unset;
      silent          = options.silent;
      changes         = [];
      changing        = this._changing;
      this._changing  = true;

      if (!changing) {
        this._previousAttributes = _.clone(this.attributes);
        this.changed = {};
      }
      current = this.attributes, prev = this._previousAttributes;

      // Check for changes of `id`.
      if (this.idAttribute in attrs) this.id = attrs[this.idAttribute];

      // For each `set` attribute, update or delete the current value.
      for (attr in attrs) {
        val = attrs[attr];
        if (!_.isEqual(current[attr], val)) changes.push(attr);
        if (!_.isEqual(prev[attr], val)) {
          this.changed[attr] = val;
        } else {
          delete this.changed[attr];
        }
        unset ? delete current[attr] : current[attr] = val;
      }

      // Trigger all relevant attribute changes.
      if (!silent) {
        if (changes.length) this._pending = options;
        for (var i = 0, l = changes.length; i < l; i++) {
          this.trigger('change:' + changes[i], this, current[changes[i]], options);
        }
      }

      // You might be wondering why there's a `while` loop here. Changes can
      // be recursively nested within `"change"` events.
      if (changing) return this;
      if (!silent) {
        while (this._pending) {
          options = this._pending;
          this._pending = false;
          this.trigger('change', this, options);
        }
      }
      this._pending = false;
      this._changing = false;
      return this;
    },

    // Remove an attribute from the model, firing `"change"`. `unset` is a noop
    // if the attribute doesn't exist.
    unset: function(attr, options) {
      return this.set(attr, void 0, _.extend({}, options, {unset: true}));
    },

    // Clear all attributes on the model, firing `"change"`.
    clear: function(options) {
      var attrs = {};
      for (var key in this.attributes) attrs[key] = void 0;
      return this.set(attrs, _.extend({}, options, {unset: true}));
    },

    // Determine if the model has changed since the last `"change"` event.
    // If you specify an attribute name, determine if that attribute has changed.
    hasChanged: function(attr) {
      if (attr == null) return !_.isEmpty(this.changed);
      return _.has(this.changed, attr);
    },

    // Return an object containing all the attributes that have changed, or
    // false if there are no changed attributes. Useful for determining what
    // parts of a view need to be updated and/or what attributes need to be
    // persisted to the server. Unset attributes will be set to undefined.
    // You can also pass an attributes object to diff against the model,
    // determining if there *would be* a change.
    changedAttributes: function(diff) {
      if (!diff) return this.hasChanged() ? _.clone(this.changed) : false;
      var val, changed = false;
      var old = this._changing ? this._previousAttributes : this.attributes;
      for (var attr in diff) {
        if (_.isEqual(old[attr], (val = diff[attr]))) continue;
        (changed || (changed = {}))[attr] = val;
      }
      return changed;
    },

    // Get the previous value of an attribute, recorded at the time the last
    // `"change"` event was fired.
    previous: function(attr) {
      if (attr == null || !this._previousAttributes) return null;
      return this._previousAttributes[attr];
    },

    // Get all of the attributes of the model at the time of the previous
    // `"change"` event.
    previousAttributes: function() {
      return _.clone(this._previousAttributes);
    },

    // Fetch the model from the server. If the server's representation of the
    // model differs from its current attributes, they will be overridden,
    // triggering a `"change"` event.
    fetch: function(options) {
      options = options ? _.clone(options) : {};
      if (options.parse === void 0) options.parse = true;
      var model = this;
      var success = options.success;
      options.success = function(resp) {
        if (!model.set(model.parse(resp, options), options)) return false;
        if (success) success(model, resp, options);
        model.trigger('sync', model, resp, options);
      };
      wrapError(this, options);
      return this.sync('read', this, options);
    },

    // Set a hash of model attributes, and sync the model to the server.
    // If the server returns an attributes hash that differs, the model's
    // state will be `set` again.
    save: function(key, val, options) {
      var attrs, method, xhr, attributes = this.attributes;

      // Handle both `"key", value` and `{key: value}` -style arguments.
      if (key == null || typeof key === 'object') {
        attrs = key;
        options = val;
      } else {
        (attrs = {})[key] = val;
      }

      options = _.extend({validate: true}, options);

      // If we're not waiting and attributes exist, save acts as
      // `set(attr).save(null, opts)` with validation. Otherwise, check if
      // the model will be valid when the attributes, if any, are set.
      if (attrs && !options.wait) {
        if (!this.set(attrs, options)) return false;
      } else {
        if (!this._validate(attrs, options)) return false;
      }

      // Set temporary attributes if `{wait: true}`.
      if (attrs && options.wait) {
        this.attributes = _.extend({}, attributes, attrs);
      }

      // After a successful server-side save, the client is (optionally)
      // updated with the server-side state.
      if (options.parse === void 0) options.parse = true;
      var model = this;
      var success = options.success;
      options.success = function(resp) {
        // Ensure attributes are restored during synchronous saves.
        model.attributes = attributes;
        var serverAttrs = model.parse(resp, options);
        if (options.wait) serverAttrs = _.extend(attrs || {}, serverAttrs);
        if (_.isObject(serverAttrs) && !model.set(serverAttrs, options)) {
          return false;
        }
        if (success) success(model, resp, options);
        model.trigger('sync', model, resp, options);
      };
      wrapError(this, options);

      method = this.isNew() ? 'create' : (options.patch ? 'patch' : 'update');
      if (method === 'patch') options.attrs = attrs;
      xhr = this.sync(method, this, options);

      // Restore attributes.
      if (attrs && options.wait) this.attributes = attributes;

      return xhr;
    },

    // Destroy this model on the server if it was already persisted.
    // Optimistically removes the model from its collection, if it has one.
    // If `wait: true` is passed, waits for the server to respond before removal.
    destroy: function(options) {
      options = options ? _.clone(options) : {};
      var model = this;
      var success = options.success;

      var destroy = function() {
        model.trigger('destroy', model, model.collection, options);
      };

      options.success = function(resp) {
        if (options.wait || model.isNew()) destroy();
        if (success) success(model, resp, options);
        if (!model.isNew()) model.trigger('sync', model, resp, options);
      };

      if (this.isNew()) {
        options.success();
        return false;
      }
      wrapError(this, options);

      var xhr = this.sync('delete', this, options);
      if (!options.wait) destroy();
      return xhr;
    },

    // Default URL for the model's representation on the server -- if you're
    // using Backbone's restful methods, override this to change the endpoint
    // that will be called.
    url: function() {
      var base =
        _.result(this, 'urlRoot') ||
        _.result(this.collection, 'url') ||
        urlError();
      if (this.isNew()) return base;
      return base.replace(/([^\/])$/, '$1/') + encodeURIComponent(this.id);
    },

    // **parse** converts a response into the hash of attributes to be `set` on
    // the model. The default implementation is just to pass the response along.
    parse: function(resp, options) {
      return resp;
    },

    // Create a new model with identical attributes to this one.
    clone: function() {
      return new this.constructor(this.attributes);
    },

    // A model is new if it has never been saved to the server, and lacks an id.
    isNew: function() {
      return !this.has(this.idAttribute);
    },

    // Check if the model is currently in a valid state.
    isValid: function(options) {
      return this._validate({}, _.extend(options || {}, { validate: true }));
    },

    // Run validation against the next complete set of model attributes,
    // returning `true` if all is well. Otherwise, fire an `"invalid"` event.
    _validate: function(attrs, options) {
      if (!options.validate || !this.validate) return true;
      attrs = _.extend({}, this.attributes, attrs);
      var error = this.validationError = this.validate(attrs, options) || null;
      if (!error) return true;
      this.trigger('invalid', this, error, _.extend(options, {validationError: error}));
      return false;
    }

  });

  // Underscore methods that we want to implement on the Model.
  var modelMethods = ['keys', 'values', 'pairs', 'invert', 'pick', 'omit'];

  // Mix in each Underscore method as a proxy to `Model#attributes`.
  _.each(modelMethods, function(method) {
    Model.prototype[method] = function() {
      var args = slice.call(arguments);
      args.unshift(this.attributes);
      return _[method].apply(_, args);
    };
  });

  // Backbone.Collection
  // -------------------

  // If models tend to represent a single row of data, a Backbone Collection is
  // more analagous to a table full of data ... or a small slice or page of that
  // table, or a collection of rows that belong together for a particular reason
  // -- all of the messages in this particular folder, all of the documents
  // belonging to this particular author, and so on. Collections maintain
  // indexes of their models, both in order, and for lookup by `id`.

  // Create a new **Collection**, perhaps to contain a specific type of `model`.
  // If a `comparator` is specified, the Collection will maintain
  // its models in sort order, as they're added and removed.
  var Collection = Backbone.Collection = function(models, options) {
    options || (options = {});
    if (options.model) this.model = options.model;
    if (options.comparator !== void 0) this.comparator = options.comparator;
    this._reset();
    this.initialize.apply(this, arguments);
    if (models) this.reset(models, _.extend({silent: true}, options));
  };

  // Default options for `Collection#set`.
  var setOptions = {add: true, remove: true, merge: true};
  var addOptions = {add: true, remove: false};

  // Define the Collection's inheritable methods.
  _.extend(Collection.prototype, Events, {

    // The default model for a collection is just a **Backbone.Model**.
    // This should be overridden in most cases.
    model: Model,

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // The JSON representation of a Collection is an array of the
    // models' attributes.
    toJSON: function(options) {
      return this.map(function(model){ return model.toJSON(options); });
    },

    // Proxy `Backbone.sync` by default.
    sync: function() {
      return Backbone.sync.apply(this, arguments);
    },

    // Add a model, or list of models to the set.
    add: function(models, options) {
      return this.set(models, _.extend({merge: false}, options, addOptions));
    },

    // Remove a model, or a list of models from the set.
    remove: function(models, options) {
      var singular = !_.isArray(models);
      models = singular ? [models] : _.clone(models);
      options || (options = {});
      var i, l, index, model;
      for (i = 0, l = models.length; i < l; i++) {
        model = models[i] = this.get(models[i]);
        if (!model) continue;
        delete this._byId[model.id];
        delete this._byId[model.cid];
        index = this.indexOf(model);
        this.models.splice(index, 1);
        this.length--;
        if (!options.silent) {
          options.index = index;
          model.trigger('remove', model, this, options);
        }
        this._removeReference(model, options);
      }
      return singular ? models[0] : models;
    },

    // Update a collection by `set`-ing a new list of models, adding new ones,
    // removing models that are no longer present, and merging models that
    // already exist in the collection, as necessary. Similar to **Model#set**,
    // the core operation for updating the data contained by the collection.
    set: function(models, options) {
      options = _.defaults({}, options, setOptions);
      if (options.parse) models = this.parse(models, options);
      var singular = !_.isArray(models);
      models = singular ? (models ? [models] : []) : _.clone(models);
      var i, l, id, model, attrs, existing, sort;
      var at = options.at;
      var targetModel = this.model;
      var sortable = this.comparator && (at == null) && options.sort !== false;
      var sortAttr = _.isString(this.comparator) ? this.comparator : null;
      var toAdd = [], toRemove = [], modelMap = {};
      var add = options.add, merge = options.merge, remove = options.remove;
      var order = !sortable && add && remove ? [] : false;

      // Turn bare objects into model references, and prevent invalid models
      // from being added.
      for (i = 0, l = models.length; i < l; i++) {
        attrs = models[i] || {};
        if (attrs instanceof Model) {
          id = model = attrs;
        } else {
          id = attrs[targetModel.prototype.idAttribute || 'id'];
        }

        // If a duplicate is found, prevent it from being added and
        // optionally merge it into the existing model.
        if (existing = this.get(id)) {
          if (remove) modelMap[existing.cid] = true;
          if (merge) {
            attrs = attrs === model ? model.attributes : attrs;
            if (options.parse) attrs = existing.parse(attrs, options);
            existing.set(attrs, options);
            if (sortable && !sort && existing.hasChanged(sortAttr)) sort = true;
          }
          models[i] = existing;

        // If this is a new, valid model, push it to the `toAdd` list.
        } else if (add) {
          model = models[i] = this._prepareModel(attrs, options);
          if (!model) continue;
          toAdd.push(model);
          this._addReference(model, options);
        }

        // Do not add multiple models with the same `id`.
        model = existing || model;
        if (order && (model.isNew() || !modelMap[model.id])) order.push(model);
        modelMap[model.id] = true;
      }

      // Remove nonexistent models if appropriate.
      if (remove) {
        for (i = 0, l = this.length; i < l; ++i) {
          if (!modelMap[(model = this.models[i]).cid]) toRemove.push(model);
        }
        if (toRemove.length) this.remove(toRemove, options);
      }

      // See if sorting is needed, update `length` and splice in new models.
      if (toAdd.length || (order && order.length)) {
        if (sortable) sort = true;
        this.length += toAdd.length;
        if (at != null) {
          for (i = 0, l = toAdd.length; i < l; i++) {
            this.models.splice(at + i, 0, toAdd[i]);
          }
        } else {
          if (order) this.models.length = 0;
          var orderedModels = order || toAdd;
          for (i = 0, l = orderedModels.length; i < l; i++) {
            this.models.push(orderedModels[i]);
          }
        }
      }

      // Silently sort the collection if appropriate.
      if (sort) this.sort({silent: true});

      // Unless silenced, it's time to fire all appropriate add/sort events.
      if (!options.silent) {
        for (i = 0, l = toAdd.length; i < l; i++) {
          (model = toAdd[i]).trigger('add', model, this, options);
        }
        if (sort || (order && order.length)) this.trigger('sort', this, options);
      }

      // Return the added (or merged) model (or models).
      return singular ? models[0] : models;
    },

    // When you have more items than you want to add or remove individually,
    // you can reset the entire set with a new list of models, without firing
    // any granular `add` or `remove` events. Fires `reset` when finished.
    // Useful for bulk operations and optimizations.
    reset: function(models, options) {
      options || (options = {});
      for (var i = 0, l = this.models.length; i < l; i++) {
        this._removeReference(this.models[i], options);
      }
      options.previousModels = this.models;
      this._reset();
      models = this.add(models, _.extend({silent: true}, options));
      if (!options.silent) this.trigger('reset', this, options);
      return models;
    },

    // Add a model to the end of the collection.
    push: function(model, options) {
      return this.add(model, _.extend({at: this.length}, options));
    },

    // Remove a model from the end of the collection.
    pop: function(options) {
      var model = this.at(this.length - 1);
      this.remove(model, options);
      return model;
    },

    // Add a model to the beginning of the collection.
    unshift: function(model, options) {
      return this.add(model, _.extend({at: 0}, options));
    },

    // Remove a model from the beginning of the collection.
    shift: function(options) {
      var model = this.at(0);
      this.remove(model, options);
      return model;
    },

    // Slice out a sub-array of models from the collection.
    slice: function() {
      return slice.apply(this.models, arguments);
    },

    // Get a model from the set by id.
    get: function(obj) {
      if (obj == null) return void 0;
      return this._byId[obj] || this._byId[obj.id] || this._byId[obj.cid];
    },

    // Get the model at the given index.
    at: function(index) {
      return this.models[index];
    },

    // Return models with matching attributes. Useful for simple cases of
    // `filter`.
    where: function(attrs, first) {
      if (_.isEmpty(attrs)) return first ? void 0 : [];
      return this[first ? 'find' : 'filter'](function(model) {
        for (var key in attrs) {
          if (attrs[key] !== model.get(key)) return false;
        }
        return true;
      });
    },

    // Return the first model with matching attributes. Useful for simple cases
    // of `find`.
    findWhere: function(attrs) {
      return this.where(attrs, true);
    },

    // Force the collection to re-sort itself. You don't need to call this under
    // normal circumstances, as the set will maintain sort order as each item
    // is added.
    sort: function(options) {
      if (!this.comparator) throw new Error('Cannot sort a set without a comparator');
      options || (options = {});

      // Run sort based on type of `comparator`.
      if (_.isString(this.comparator) || this.comparator.length === 1) {
        this.models = this.sortBy(this.comparator, this);
      } else {
        this.models.sort(_.bind(this.comparator, this));
      }

      if (!options.silent) this.trigger('sort', this, options);
      return this;
    },

    // Pluck an attribute from each model in the collection.
    pluck: function(attr) {
      return _.invoke(this.models, 'get', attr);
    },

    // Fetch the default set of models for this collection, resetting the
    // collection when they arrive. If `reset: true` is passed, the response
    // data will be passed through the `reset` method instead of `set`.
    fetch: function(options) {
      options = options ? _.clone(options) : {};
      if (options.parse === void 0) options.parse = true;
      var success = options.success;
      var collection = this;
      options.success = function(resp) {
        var method = options.reset ? 'reset' : 'set';
        collection[method](resp, options);
        if (success) success(collection, resp, options);
        collection.trigger('sync', collection, resp, options);
      };
      wrapError(this, options);
      return this.sync('read', this, options);
    },

    // Create a new instance of a model in this collection. Add the model to the
    // collection immediately, unless `wait: true` is passed, in which case we
    // wait for the server to agree.
    create: function(model, options) {
      options = options ? _.clone(options) : {};
      if (!(model = this._prepareModel(model, options))) return false;
      if (!options.wait) this.add(model, options);
      var collection = this;
      var success = options.success;
      options.success = function(model, resp) {
        if (options.wait) collection.add(model, options);
        if (success) success(model, resp, options);
      };
      model.save(null, options);
      return model;
    },

    // **parse** converts a response into a list of models to be added to the
    // collection. The default implementation is just to pass it through.
    parse: function(resp, options) {
      return resp;
    },

    // Create a new collection with an identical list of models as this one.
    clone: function() {
      return new this.constructor(this.models);
    },

    // Private method to reset all internal state. Called when the collection
    // is first initialized or reset.
    _reset: function() {
      this.length = 0;
      this.models = [];
      this._byId  = {};
    },

    // Prepare a hash of attributes (or other model) to be added to this
    // collection.
    _prepareModel: function(attrs, options) {
      if (attrs instanceof Model) return attrs;
      options = options ? _.clone(options) : {};
      options.collection = this;
      var model = new this.model(attrs, options);
      if (!model.validationError) return model;
      this.trigger('invalid', this, model.validationError, options);
      return false;
    },

    // Internal method to create a model's ties to a collection.
    _addReference: function(model, options) {
      this._byId[model.cid] = model;
      if (model.id != null) this._byId[model.id] = model;
      if (!model.collection) model.collection = this;
      model.on('all', this._onModelEvent, this);
    },

    // Internal method to sever a model's ties to a collection.
    _removeReference: function(model, options) {
      if (this === model.collection) delete model.collection;
      model.off('all', this._onModelEvent, this);
    },

    // Internal method called every time a model in the set fires an event.
    // Sets need to update their indexes when models change ids. All other
    // events simply proxy through. "add" and "remove" events that originate
    // in other collections are ignored.
    _onModelEvent: function(event, model, collection, options) {
      if ((event === 'add' || event === 'remove') && collection !== this) return;
      if (event === 'destroy') this.remove(model, options);
      if (model && event === 'change:' + model.idAttribute) {
        delete this._byId[model.previous(model.idAttribute)];
        if (model.id != null) this._byId[model.id] = model;
      }
      this.trigger.apply(this, arguments);
    }

  });

  // Underscore methods that we want to implement on the Collection.
  // 90% of the core usefulness of Backbone Collections is actually implemented
  // right here:
  var methods = ['forEach', 'each', 'map', 'collect', 'reduce', 'foldl',
    'inject', 'reduceRight', 'foldr', 'find', 'detect', 'filter', 'select',
    'reject', 'every', 'all', 'some', 'any', 'include', 'contains', 'invoke',
    'max', 'min', 'toArray', 'size', 'first', 'head', 'take', 'initial', 'rest',
    'tail', 'drop', 'last', 'without', 'difference', 'indexOf', 'shuffle',
    'lastIndexOf', 'isEmpty', 'chain', 'sample'];

  // Mix in each Underscore method as a proxy to `Collection#models`.
  _.each(methods, function(method) {
    Collection.prototype[method] = function() {
      var args = slice.call(arguments);
      args.unshift(this.models);
      return _[method].apply(_, args);
    };
  });

  // Underscore methods that take a property name as an argument.
  var attributeMethods = ['groupBy', 'countBy', 'sortBy', 'indexBy'];

  // Use attributes instead of properties.
  _.each(attributeMethods, function(method) {
    Collection.prototype[method] = function(value, context) {
      var iterator = _.isFunction(value) ? value : function(model) {
        return model.get(value);
      };
      return _[method](this.models, iterator, context);
    };
  });

  // Backbone.View
  // -------------

  // Backbone Views are almost more convention than they are actual code. A View
  // is simply a JavaScript object that represents a logical chunk of UI in the
  // DOM. This might be a single item, an entire list, a sidebar or panel, or
  // even the surrounding frame which wraps your whole app. Defining a chunk of
  // UI as a **View** allows you to define your DOM events declaratively, without
  // having to worry about render order ... and makes it easy for the view to
  // react to specific changes in the state of your models.

  // Creating a Backbone.View creates its initial element outside of the DOM,
  // if an existing element is not provided...
  var View = Backbone.View = function(options) {
    this.cid = _.uniqueId('view');
    options || (options = {});
    _.extend(this, _.pick(options, viewOptions));
    this._ensureElement();
    this.initialize.apply(this, arguments);
    this.delegateEvents();
  };

  // Cached regex to split keys for `delegate`.
  var delegateEventSplitter = /^(\S+)\s*(.*)$/;

  // List of view options to be merged as properties.
  var viewOptions = ['model', 'collection', 'el', 'id', 'attributes', 'className', 'tagName', 'events'];

  // Set up all inheritable **Backbone.View** properties and methods.
  _.extend(View.prototype, Events, {

    // The default `tagName` of a View's element is `"div"`.
    tagName: 'div',

    // jQuery delegate for element lookup, scoped to DOM elements within the
    // current view. This should be preferred to global lookups where possible.
    $: function(selector) {
      return this.$el.find(selector);
    },

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // **render** is the core function that your view should override, in order
    // to populate its element (`this.el`), with the appropriate HTML. The
    // convention is for **render** to always return `this`.
    render: function() {
      return this;
    },

    // Remove this view by taking the element out of the DOM, and removing any
    // applicable Backbone.Events listeners.
    remove: function() {
      this.$el.remove();
      this.stopListening();
      return this;
    },

    // Change the view's element (`this.el` property), including event
    // re-delegation.
    setElement: function(element, delegate) {
      if (this.$el) this.undelegateEvents();
      this.$el = element instanceof Backbone.$ ? element : Backbone.$(element);
      this.el = this.$el[0];
      if (delegate !== false) this.delegateEvents();
      return this;
    },

    // Set callbacks, where `this.events` is a hash of
    //
    // *{"event selector": "callback"}*
    //
    //     {
    //       'mousedown .title':  'edit',
    //       'click .button':     'save',
    //       'click .open':       function(e) { ... }
    //     }
    //
    // pairs. Callbacks will be bound to the view, with `this` set properly.
    // Uses event delegation for efficiency.
    // Omitting the selector binds the event to `this.el`.
    // This only works for delegate-able events: not `focus`, `blur`, and
    // not `change`, `submit`, and `reset` in Internet Explorer.
    delegateEvents: function(events) {
      if (!(events || (events = _.result(this, 'events')))) return this;
      this.undelegateEvents();
      for (var key in events) {
        var method = events[key];
        if (!_.isFunction(method)) method = this[events[key]];
        if (!method) continue;

        var match = key.match(delegateEventSplitter);
        var eventName = match[1], selector = match[2];
        method = _.bind(method, this);
        eventName += '.delegateEvents' + this.cid;
        if (selector === '') {
          this.$el.on(eventName, method);
        } else {
          this.$el.on(eventName, selector, method);
        }
      }
      return this;
    },

    // Clears all callbacks previously bound to the view with `delegateEvents`.
    // You usually don't need to use this, but may wish to if you have multiple
    // Backbone views attached to the same DOM element.
    undelegateEvents: function() {
      this.$el.off('.delegateEvents' + this.cid);
      return this;
    },

    // Ensure that the View has a DOM element to render into.
    // If `this.el` is a string, pass it through `$()`, take the first
    // matching element, and re-assign it to `el`. Otherwise, create
    // an element from the `id`, `className` and `tagName` properties.
    _ensureElement: function() {
      if (!this.el) {
        var attrs = _.extend({}, _.result(this, 'attributes'));
        if (this.id) attrs.id = _.result(this, 'id');
        if (this.className) attrs['class'] = _.result(this, 'className');
        var $el = Backbone.$('<' + _.result(this, 'tagName') + '>').attr(attrs);
        this.setElement($el, false);
      } else {
        this.setElement(_.result(this, 'el'), false);
      }
    }

  });

  // Backbone.sync
  // -------------

  // Override this function to change the manner in which Backbone persists
  // models to the server. You will be passed the type of request, and the
  // model in question. By default, makes a RESTful Ajax request
  // to the model's `url()`. Some possible customizations could be:
  //
  // * Use `setTimeout` to batch rapid-fire updates into a single request.
  // * Send up the models as XML instead of JSON.
  // * Persist models via WebSockets instead of Ajax.
  //
  // Turn on `Backbone.emulateHTTP` in order to send `PUT` and `DELETE` requests
  // as `POST`, with a `_method` parameter containing the true HTTP method,
  // as well as all requests with the body as `application/x-www-form-urlencoded`
  // instead of `application/json` with the model in a param named `model`.
  // Useful when interfacing with server-side languages like **PHP** that make
  // it difficult to read the body of `PUT` requests.
  Backbone.sync = function(method, model, options) {
    var type = methodMap[method];

    // Default options, unless specified.
    _.defaults(options || (options = {}), {
      emulateHTTP: Backbone.emulateHTTP,
      emulateJSON: Backbone.emulateJSON
    });

    // Default JSON-request options.
    var params = {type: type, dataType: 'json'};

    // Ensure that we have a URL.
    if (!options.url) {
      params.url = _.result(model, 'url') || urlError();
    }

    // Ensure that we have the appropriate request data.
    if (options.data == null && model && (method === 'create' || method === 'update' || method === 'patch')) {
      params.contentType = 'application/json';
      params.data = JSON.stringify(options.attrs || model.toJSON(options));
    }

    // For older servers, emulate JSON by encoding the request into an HTML-form.
    if (options.emulateJSON) {
      params.contentType = 'application/x-www-form-urlencoded';
      params.data = params.data ? {model: params.data} : {};
    }

    // For older servers, emulate HTTP by mimicking the HTTP method with `_method`
    // And an `X-HTTP-Method-Override` header.
    if (options.emulateHTTP && (type === 'PUT' || type === 'DELETE' || type === 'PATCH')) {
      params.type = 'POST';
      if (options.emulateJSON) params.data._method = type;
      var beforeSend = options.beforeSend;
      options.beforeSend = function(xhr) {
        xhr.setRequestHeader('X-HTTP-Method-Override', type);
        if (beforeSend) return beforeSend.apply(this, arguments);
      };
    }

    // Don't process data on a non-GET request.
    if (params.type !== 'GET' && !options.emulateJSON) {
      params.processData = false;
    }

    // If we're sending a `PATCH` request, and we're in an old Internet Explorer
    // that still has ActiveX enabled by default, override jQuery to use that
    // for XHR instead. Remove this line when jQuery supports `PATCH` on IE8.
    if (params.type === 'PATCH' && noXhrPatch) {
      params.xhr = function() {
        return new ActiveXObject("Microsoft.XMLHTTP");
      };
    }

    // Make the request, allowing the user to override any Ajax options.
    var xhr = options.xhr = Backbone.ajax(_.extend(params, options));
    model.trigger('request', model, xhr, options);
    return xhr;
  };

  var noXhrPatch =
    typeof window !== 'undefined' && !!window.ActiveXObject &&
      !(window.XMLHttpRequest && (new XMLHttpRequest).dispatchEvent);

  // Map from CRUD to HTTP for our default `Backbone.sync` implementation.
  var methodMap = {
    'create': 'POST',
    'update': 'PUT',
    'patch':  'PATCH',
    'delete': 'DELETE',
    'read':   'GET'
  };

  // Set the default implementation of `Backbone.ajax` to proxy through to `$`.
  // Override this if you'd like to use a different library.
  Backbone.ajax = function() {
    return Backbone.$.ajax.apply(Backbone.$, arguments);
  };

  // Backbone.Router
  // ---------------

  // Routers map faux-URLs to actions, and fire events when routes are
  // matched. Creating a new one sets its `routes` hash, if not set statically.
  var Router = Backbone.Router = function(options) {
    options || (options = {});
    if (options.routes) this.routes = options.routes;
    this._bindRoutes();
    this.initialize.apply(this, arguments);
  };

  // Cached regular expressions for matching named param parts and splatted
  // parts of route strings.
  var optionalParam = /\((.*?)\)/g;
  var namedParam    = /(\(\?)?:\w+/g;
  var splatParam    = /\*\w+/g;
  var escapeRegExp  = /[\-{}\[\]+?.,\\\^$|#\s]/g;

  // Set up all inheritable **Backbone.Router** properties and methods.
  _.extend(Router.prototype, Events, {

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // Manually bind a single named route to a callback. For example:
    //
    //     this.route('search/:query/p:num', 'search', function(query, num) {
    //       ...
    //     });
    //
    route: function(route, name, callback) {
      if (!_.isRegExp(route)) route = this._routeToRegExp(route);
      if (_.isFunction(name)) {
        callback = name;
        name = '';
      }
      if (!callback) callback = this[name];
      var router = this;
      Backbone.history.route(route, function(fragment) {
        var args = router._extractParameters(route, fragment);
        router.execute(callback, args);
        router.trigger.apply(router, ['route:' + name].concat(args));
        router.trigger('route', name, args);
        Backbone.history.trigger('route', router, name, args);
      });
      return this;
    },

    // Execute a route handler with the provided parameters.  This is an
    // excellent place to do pre-route setup or post-route cleanup.
    execute: function(callback, args) {
      if (callback) callback.apply(this, args);
    },

    // Simple proxy to `Backbone.history` to save a fragment into the history.
    navigate: function(fragment, options) {
      Backbone.history.navigate(fragment, options);
      return this;
    },

    // Bind all defined routes to `Backbone.history`. We have to reverse the
    // order of the routes here to support behavior where the most general
    // routes can be defined at the bottom of the route map.
    _bindRoutes: function() {
      if (!this.routes) return;
      this.routes = _.result(this, 'routes');
      var route, routes = _.keys(this.routes);
      while ((route = routes.pop()) != null) {
        this.route(route, this.routes[route]);
      }
    },

    // Convert a route string into a regular expression, suitable for matching
    // against the current location hash.
    _routeToRegExp: function(route) {
      route = route.replace(escapeRegExp, '\\$&')
                   .replace(optionalParam, '(?:$1)?')
                   .replace(namedParam, function(match, optional) {
                     return optional ? match : '([^/?]+)';
                   })
                   .replace(splatParam, '([^?]*?)');
      return new RegExp('^' + route + '(?:\\?([\\s\\S]*))?$');
    },

    // Given a route, and a URL fragment that it matches, return the array of
    // extracted decoded parameters. Empty or unmatched parameters will be
    // treated as `null` to normalize cross-browser behavior.
    _extractParameters: function(route, fragment) {
      var params = route.exec(fragment).slice(1);
      return _.map(params, function(param, i) {
        // Don't decode the search params.
        if (i === params.length - 1) return param || null;
        return param ? decodeURIComponent(param) : null;
      });
    }

  });

  // Backbone.History
  // ----------------

  // Handles cross-browser history management, based on either
  // [pushState](http://diveintohtml5.info/history.html) and real URLs, or
  // [onhashchange](https://developer.mozilla.org/en-US/docs/DOM/window.onhashchange)
  // and URL fragments. If the browser supports neither (old IE, natch),
  // falls back to polling.
  var History = Backbone.History = function() {
    this.handlers = [];
    _.bindAll(this, 'checkUrl');

    // Ensure that `History` can be used outside of the browser.
    if (typeof window !== 'undefined') {
      this.location = window.location;
      this.history = window.history;
    }
  };

  // Cached regex for stripping a leading hash/slash and trailing space.
  var routeStripper = /^[#\/]|\s+$/g;

  // Cached regex for stripping leading and trailing slashes.
  var rootStripper = /^\/+|\/+$/g;

  // Cached regex for detecting MSIE.
  var isExplorer = /msie [\w.]+/;

  // Cached regex for removing a trailing slash.
  var trailingSlash = /\/$/;

  // Cached regex for stripping urls of hash.
  var pathStripper = /#.*$/;

  // Has the history handling already been started?
  History.started = false;

  // Set up all inheritable **Backbone.History** properties and methods.
  _.extend(History.prototype, Events, {

    // The default interval to poll for hash changes, if necessary, is
    // twenty times a second.
    interval: 50,

    // Are we at the app root?
    atRoot: function() {
      return this.location.pathname.replace(/[^\/]$/, '$&/') === this.root;
    },

    // Gets the true hash value. Cannot use location.hash directly due to bug
    // in Firefox where location.hash will always be decoded.
    getHash: function(window) {
      var match = (window || this).location.href.match(/#(.*)$/);
      return match ? match[1] : '';
    },

    // Get the cross-browser normalized URL fragment, either from the URL,
    // the hash, or the override.
    getFragment: function(fragment, forcePushState) {
      if (fragment == null) {
        if (this._hasPushState || !this._wantsHashChange || forcePushState) {
          fragment = decodeURI(this.location.pathname + this.location.search);
          var root = this.root.replace(trailingSlash, '');
          if (!fragment.indexOf(root)) fragment = fragment.slice(root.length);
        } else {
          fragment = this.getHash();
        }
      }
      return fragment.replace(routeStripper, '');
    },

    // Start the hash change handling, returning `true` if the current URL matches
    // an existing route, and `false` otherwise.
    start: function(options) {
      if (History.started) throw new Error("Backbone.history has already been started");
      History.started = true;

      // Figure out the initial configuration. Do we need an iframe?
      // Is pushState desired ... is it available?
      this.options          = _.extend({root: '/'}, this.options, options);
      this.root             = this.options.root;
      this._wantsHashChange = this.options.hashChange !== false;
      this._wantsPushState  = !!this.options.pushState;
      this._hasPushState    = !!(this.options.pushState && this.history && this.history.pushState);
      var fragment          = this.getFragment();
      var docMode           = document.documentMode;
      var oldIE             = (isExplorer.exec(navigator.userAgent.toLowerCase()) && (!docMode || docMode <= 7));

      // Normalize root to always include a leading and trailing slash.
      this.root = ('/' + this.root + '/').replace(rootStripper, '/');

      if (oldIE && this._wantsHashChange) {
        var frame = Backbone.$('<iframe src="javascript:0" tabindex="-1">');
        this.iframe = frame.hide().appendTo('body')[0].contentWindow;
        this.navigate(fragment);
      }

      // Depending on whether we're using pushState or hashes, and whether
      // 'onhashchange' is supported, determine how we check the URL state.
      if (this._hasPushState) {
        Backbone.$(window).on('popstate', this.checkUrl);
      } else if (this._wantsHashChange && ('onhashchange' in window) && !oldIE) {
        Backbone.$(window).on('hashchange', this.checkUrl);
      } else if (this._wantsHashChange) {
        this._checkUrlInterval = setInterval(this.checkUrl, this.interval);
      }

      // Determine if we need to change the base url, for a pushState link
      // opened by a non-pushState browser.
      this.fragment = fragment;
      var loc = this.location;

      // Transition from hashChange to pushState or vice versa if both are
      // requested.
      if (this._wantsHashChange && this._wantsPushState) {

        // If we've started off with a route from a `pushState`-enabled
        // browser, but we're currently in a browser that doesn't support it...
        if (!this._hasPushState && !this.atRoot()) {
          this.fragment = this.getFragment(null, true);
          this.location.replace(this.root + '#' + this.fragment);
          // Return immediately as browser will do redirect to new url
          return true;

        // Or if we've started out with a hash-based route, but we're currently
        // in a browser where it could be `pushState`-based instead...
        } else if (this._hasPushState && this.atRoot() && loc.hash) {
          this.fragment = this.getHash().replace(routeStripper, '');
          this.history.replaceState({}, document.title, this.root + this.fragment);
        }

      }

      if (!this.options.silent) return this.loadUrl();
    },

    // Disable Backbone.history, perhaps temporarily. Not useful in a real app,
    // but possibly useful for unit testing Routers.
    stop: function() {
      Backbone.$(window).off('popstate', this.checkUrl).off('hashchange', this.checkUrl);
      if (this._checkUrlInterval) clearInterval(this._checkUrlInterval);
      History.started = false;
    },

    // Add a route to be tested when the fragment changes. Routes added later
    // may override previous routes.
    route: function(route, callback) {
      this.handlers.unshift({route: route, callback: callback});
    },

    // Checks the current URL to see if it has changed, and if it has,
    // calls `loadUrl`, normalizing across the hidden iframe.
    checkUrl: function(e) {
      var current = this.getFragment();
      if (current === this.fragment && this.iframe) {
        current = this.getFragment(this.getHash(this.iframe));
      }
      if (current === this.fragment) return false;
      if (this.iframe) this.navigate(current);
      this.loadUrl();
    },

    // Attempt to load the current URL fragment. If a route succeeds with a
    // match, returns `true`. If no defined routes matches the fragment,
    // returns `false`.
    loadUrl: function(fragment) {
      fragment = this.fragment = this.getFragment(fragment);
      return _.any(this.handlers, function(handler) {
        if (handler.route.test(fragment)) {
          handler.callback(fragment);
          return true;
        }
      });
    },

    // Save a fragment into the hash history, or replace the URL state if the
    // 'replace' option is passed. You are responsible for properly URL-encoding
    // the fragment in advance.
    //
    // The options object can contain `trigger: true` if you wish to have the
    // route callback be fired (not usually desirable), or `replace: true`, if
    // you wish to modify the current URL without adding an entry to the history.
    navigate: function(fragment, options) {
      if (!History.started) return false;
      if (!options || options === true) options = {trigger: !!options};

      var url = this.root + (fragment = this.getFragment(fragment || ''));

      // Strip the hash for matching.
      fragment = fragment.replace(pathStripper, '');

      if (this.fragment === fragment) return;
      this.fragment = fragment;

      // Don't include a trailing slash on the root.
      if (fragment === '' && url !== '/') url = url.slice(0, -1);

      // If pushState is available, we use it to set the fragment as a real URL.
      if (this._hasPushState) {
        this.history[options.replace ? 'replaceState' : 'pushState']({}, document.title, url);

      // If hash changes haven't been explicitly disabled, update the hash
      // fragment to store history.
      } else if (this._wantsHashChange) {
        this._updateHash(this.location, fragment, options.replace);
        if (this.iframe && (fragment !== this.getFragment(this.getHash(this.iframe)))) {
          // Opening and closing the iframe tricks IE7 and earlier to push a
          // history entry on hash-tag change.  When replace is true, we don't
          // want this.
          if(!options.replace) this.iframe.document.open().close();
          this._updateHash(this.iframe.location, fragment, options.replace);
        }

      // If you've told us that you explicitly don't want fallback hashchange-
      // based history, then `navigate` becomes a page refresh.
      } else {
        return this.location.assign(url);
      }
      if (options.trigger) return this.loadUrl(fragment);
    },

    // Update the hash location, either replacing the current entry, or adding
    // a new one to the browser history.
    _updateHash: function(location, fragment, replace) {
      if (replace) {
        var href = location.href.replace(/(javascript:|#).*$/, '');
        location.replace(href + '#' + fragment);
      } else {
        // Some browsers require that `hash` contains a leading #.
        location.hash = '#' + fragment;
      }
    }

  });

  // Create the default Backbone.history.
  Backbone.history = new History;

  // Helpers
  // -------

  // Helper function to correctly set up the prototype chain, for subclasses.
  // Similar to `goog.inherits`, but uses a hash of prototype properties and
  // class properties to be extended.
  var extend = function(protoProps, staticProps) {
    var parent = this;
    var child;

    // The constructor function for the new subclass is either defined by you
    // (the "constructor" property in your `extend` definition), or defaulted
    // by us to simply call the parent's constructor.
    if (protoProps && _.has(protoProps, 'constructor')) {
      child = protoProps.constructor;
    } else {
      child = function(){ return parent.apply(this, arguments); };
    }

    // Add static properties to the constructor function, if supplied.
    _.extend(child, parent, staticProps);

    // Set the prototype chain to inherit from `parent`, without calling
    // `parent`'s constructor function.
    var Surrogate = function(){ this.constructor = child; };
    Surrogate.prototype = parent.prototype;
    child.prototype = new Surrogate;

    // Add prototype properties (instance properties) to the subclass,
    // if supplied.
    if (protoProps) _.extend(child.prototype, protoProps);

    // Set a convenience property in case the parent's prototype is needed
    // later.
    child.__super__ = parent.prototype;

    return child;
  };

  // Set up inheritance for the model, collection, router, view and history.
  Model.extend = Collection.extend = Router.extend = View.extend = History.extend = extend;

  // Throw an error when a URL is needed, and none is supplied.
  var urlError = function() {
    throw new Error('A "url" property or function must be specified');
  };

  // Wrap an optional error callback with a fallback error event.
  var wrapError = function(model, options) {
    var error = options.error;
    options.error = function(resp) {
      if (error) error(model, resp, options);
      model.trigger('error', model, resp, options);
    };
  };

  return Backbone;

}));

// Backbone.Wreqr (Backbone.Marionette)
// ----------------------------------
// v1.3.6
//
// Copyright (c)2016 Derick Bailey, Muted Solutions, LLC.
// Distributed under MIT license
//
// http://github.com/marionettejs/backbone.wreqr


(function(root, factory) {

  if (typeof define === 'function' && define.amd) {
    define('backbone.wreqr',['backbone', 'underscore'], function(Backbone, _) {
      return factory(Backbone, _);
    });
  } else if (typeof exports !== 'undefined') {
    var Backbone = require('backbone');
    var _ = require('underscore');
    module.exports = factory(Backbone, _);
  } else {
    factory(root.Backbone, root._);
  }

}(this, function(Backbone, _) {
  "use strict";

  var previousWreqr = Backbone.Wreqr;

  var Wreqr = Backbone.Wreqr = {};

  Backbone.Wreqr.VERSION = '1.3.6';

  Backbone.Wreqr.noConflict = function () {
    Backbone.Wreqr = previousWreqr;
    return this;
  };

  // Handlers
  // --------
  // A registry of functions to call, given a name
  
  Wreqr.Handlers = (function(Backbone, _){
    "use strict";
    
    // Constructor
    // -----------
  
    var Handlers = function(options){
      this.options = options;
      this._wreqrHandlers = {};
      
      if (_.isFunction(this.initialize)){
        this.initialize(options);
      }
    };
  
    Handlers.extend = Backbone.Model.extend;
  
    // Instance Members
    // ----------------
  
    _.extend(Handlers.prototype, Backbone.Events, {
  
      // Add multiple handlers using an object literal configuration
      setHandlers: function(handlers){
        _.each(handlers, function(handler, name){
          var context = null;
  
          if (_.isObject(handler) && !_.isFunction(handler)){
            context = handler.context;
            handler = handler.callback;
          }
  
          this.setHandler(name, handler, context);
        }, this);
      },
  
      // Add a handler for the given name, with an
      // optional context to run the handler within
      setHandler: function(name, handler, context){
        var config = {
          callback: handler,
          context: context
        };
  
        this._wreqrHandlers[name] = config;
  
        this.trigger("handler:add", name, handler, context);
      },
  
      // Determine whether or not a handler is registered
      hasHandler: function(name){
        return !! this._wreqrHandlers[name];
      },
  
      // Get the currently registered handler for
      // the specified name. Throws an exception if
      // no handler is found.
      getHandler: function(name){
        var config = this._wreqrHandlers[name];
  
        if (!config){
          return;
        }
  
        return function(){
          return config.callback.apply(config.context, arguments);
        };
      },
  
      // Remove a handler for the specified name
      removeHandler: function(name){
        delete this._wreqrHandlers[name];
      },
  
      // Remove all handlers from this registry
      removeAllHandlers: function(){
        this._wreqrHandlers = {};
      }
    });
  
    return Handlers;
  })(Backbone, _);
  
  // Wreqr.CommandStorage
  // --------------------
  //
  // Store and retrieve commands for execution.
  Wreqr.CommandStorage = (function(){
    "use strict";
  
    // Constructor function
    var CommandStorage = function(options){
      this.options = options;
      this._commands = {};
  
      if (_.isFunction(this.initialize)){
        this.initialize(options);
      }
    };
  
    // Instance methods
    _.extend(CommandStorage.prototype, Backbone.Events, {
  
      // Get an object literal by command name, that contains
      // the `commandName` and the `instances` of all commands
      // represented as an array of arguments to process
      getCommands: function(commandName){
        var commands = this._commands[commandName];
  
        // we don't have it, so add it
        if (!commands){
  
          // build the configuration
          commands = {
            command: commandName, 
            instances: []
          };
  
          // store it
          this._commands[commandName] = commands;
        }
  
        return commands;
      },
  
      // Add a command by name, to the storage and store the
      // args for the command
      addCommand: function(commandName, args){
        var command = this.getCommands(commandName);
        command.instances.push(args);
      },
  
      // Clear all commands for the given `commandName`
      clearCommands: function(commandName){
        var command = this.getCommands(commandName);
        command.instances = [];
      }
    });
  
    return CommandStorage;
  })();
  
  // Wreqr.Commands
  // --------------
  //
  // A simple command pattern implementation. Register a command
  // handler and execute it.
  Wreqr.Commands = (function(Wreqr, _){
    "use strict";
  
    return Wreqr.Handlers.extend({
      // default storage type
      storageType: Wreqr.CommandStorage,
  
      constructor: function(options){
        this.options = options || {};
  
        this._initializeStorage(this.options);
        this.on("handler:add", this._executeCommands, this);
  
        Wreqr.Handlers.prototype.constructor.apply(this, arguments);
      },
  
      // Execute a named command with the supplied args
      execute: function(name){
        name = arguments[0];
        var args = _.rest(arguments);
  
        if (this.hasHandler(name)){
          this.getHandler(name).apply(this, args);
        } else {
          this.storage.addCommand(name, args);
        }
  
      },
  
      // Internal method to handle bulk execution of stored commands
      _executeCommands: function(name, handler, context){
        var command = this.storage.getCommands(name);
  
        // loop through and execute all the stored command instances
        _.each(command.instances, function(args){
          handler.apply(context, args);
        });
  
        this.storage.clearCommands(name);
      },
  
      // Internal method to initialize storage either from the type's
      // `storageType` or the instance `options.storageType`.
      _initializeStorage: function(options){
        var storage;
  
        var StorageType = options.storageType || this.storageType;
        if (_.isFunction(StorageType)){
          storage = new StorageType();
        } else {
          storage = StorageType;
        }
  
        this.storage = storage;
      }
    });
  
  })(Wreqr, _);
  
  // Wreqr.RequestResponse
  // ---------------------
  //
  // A simple request/response implementation. Register a
  // request handler, and return a response from it
  Wreqr.RequestResponse = (function(Wreqr, _){
    "use strict";
  
    return Wreqr.Handlers.extend({
      request: function(name){
        if (this.hasHandler(name)) {
          return this.getHandler(name).apply(this, _.rest(arguments));
        }
      }
    });
  
  })(Wreqr, _);
  
  // Event Aggregator
  // ----------------
  // A pub-sub object that can be used to decouple various parts
  // of an application through event-driven architecture.
  
  Wreqr.EventAggregator = (function(Backbone, _){
    "use strict";
    var EA = function(){};
  
    // Copy the `extend` function used by Backbone's classes
    EA.extend = Backbone.Model.extend;
  
    // Copy the basic Backbone.Events on to the event aggregator
    _.extend(EA.prototype, Backbone.Events);
  
    return EA;
  })(Backbone, _);
  
  // Wreqr.Channel
  // --------------
  //
  // An object that wraps the three messaging systems:
  // EventAggregator, RequestResponse, Commands
  Wreqr.Channel = (function(Wreqr){
    "use strict";
  
    var Channel = function(channelName) {
      this.vent        = new Backbone.Wreqr.EventAggregator();
      this.reqres      = new Backbone.Wreqr.RequestResponse();
      this.commands    = new Backbone.Wreqr.Commands();
      this.channelName = channelName;
    };
  
    _.extend(Channel.prototype, {
  
      // Remove all handlers from the messaging systems of this channel
      reset: function() {
        this.vent.off();
        this.vent.stopListening();
        this.reqres.removeAllHandlers();
        this.commands.removeAllHandlers();
        return this;
      },
  
      // Connect a hash of events; one for each messaging system
      connectEvents: function(hash, context) {
        this._connect('vent', hash, context);
        return this;
      },
  
      connectCommands: function(hash, context) {
        this._connect('commands', hash, context);
        return this;
      },
  
      connectRequests: function(hash, context) {
        this._connect('reqres', hash, context);
        return this;
      },
  
      // Attach the handlers to a given message system `type`
      _connect: function(type, hash, context) {
        if (!hash) {
          return;
        }
  
        context = context || this;
        var method = (type === 'vent') ? 'on' : 'setHandler';
  
        _.each(hash, function(fn, eventName) {
          this[type][method](eventName, _.bind(fn, context));
        }, this);
      }
    });
  
  
    return Channel;
  })(Wreqr);
  
  // Wreqr.Radio
  // --------------
  //
  // An object that lets you communicate with many channels.
  Wreqr.radio = (function(Wreqr, _){
    "use strict";
  
    var Radio = function() {
      this._channels = {};
      this.vent = {};
      this.commands = {};
      this.reqres = {};
      this._proxyMethods();
    };
  
    _.extend(Radio.prototype, {
  
      channel: function(channelName) {
        if (!channelName) {
          throw new Error('Channel must receive a name');
        }
  
        return this._getChannel( channelName );
      },
  
      _getChannel: function(channelName) {
        var channel = this._channels[channelName];
  
        if(!channel) {
          channel = new Wreqr.Channel(channelName);
          this._channels[channelName] = channel;
        }
  
        return channel;
      },
  
      _proxyMethods: function() {
        _.each(['vent', 'commands', 'reqres'], function(system) {
          _.each( messageSystems[system], function(method) {
            this[system][method] = proxyMethod(this, system, method);
          }, this);
        }, this);
      }
    });
  
  
    var messageSystems = {
      vent: [
        'on',
        'off',
        'trigger',
        'once',
        'stopListening',
        'listenTo',
        'listenToOnce'
      ],
  
      commands: [
        'execute',
        'setHandler',
        'setHandlers',
        'removeHandler',
        'removeAllHandlers'
      ],
  
      reqres: [
        'request',
        'setHandler',
        'setHandlers',
        'removeHandler',
        'removeAllHandlers'
      ]
    };
  
    var proxyMethod = function(radio, system, method) {
      return function(channelName) {
        var messageSystem = radio._getChannel(channelName)[system];
  
        return messageSystem[method].apply(messageSystem, _.rest(arguments));
      };
    };
  
    return new Radio();
  
  })(Wreqr, _);
  

  return Backbone.Wreqr;

}));

// Backbone.BabySitter
// -------------------
// v0.1.11
//
// Copyright (c)2016 Derick Bailey, Muted Solutions, LLC.
// Distributed under MIT license
//
// http://github.com/marionettejs/backbone.babysitter

(function(root, factory) {

  if (typeof define === 'function' && define.amd) {
    define('backbone.babysitter',['backbone', 'underscore'], function(Backbone, _) {
      return factory(Backbone, _);
    });
  } else if (typeof exports !== 'undefined') {
    var Backbone = require('backbone');
    var _ = require('underscore');
    module.exports = factory(Backbone, _);
  } else {
    factory(root.Backbone, root._);
  }

}(this, function(Backbone, _) {
  'use strict';

  var previousChildViewContainer = Backbone.ChildViewContainer;

  // BabySitter.ChildViewContainer
  // -----------------------------
  //
  // Provide a container to store, retrieve and
  // shut down child views.
  
  Backbone.ChildViewContainer = (function (Backbone, _) {
  
    // Container Constructor
    // ---------------------
  
    var Container = function(views){
      this._views = {};
      this._indexByModel = {};
      this._indexByCustom = {};
      this._updateLength();
  
      _.each(views, this.add, this);
    };
  
    // Container Methods
    // -----------------
  
    _.extend(Container.prototype, {
  
      // Add a view to this container. Stores the view
      // by `cid` and makes it searchable by the model
      // cid (and model itself). Optionally specify
      // a custom key to store an retrieve the view.
      add: function(view, customIndex){
        var viewCid = view.cid;
  
        // store the view
        this._views[viewCid] = view;
  
        // index it by model
        if (view.model){
          this._indexByModel[view.model.cid] = viewCid;
        }
  
        // index by custom
        if (customIndex){
          this._indexByCustom[customIndex] = viewCid;
        }
  
        this._updateLength();
        return this;
      },
  
      // Find a view by the model that was attached to
      // it. Uses the model's `cid` to find it.
      findByModel: function(model){
        return this.findByModelCid(model.cid);
      },
  
      // Find a view by the `cid` of the model that was attached to
      // it. Uses the model's `cid` to find the view `cid` and
      // retrieve the view using it.
      findByModelCid: function(modelCid){
        var viewCid = this._indexByModel[modelCid];
        return this.findByCid(viewCid);
      },
  
      // Find a view by a custom indexer.
      findByCustom: function(index){
        var viewCid = this._indexByCustom[index];
        return this.findByCid(viewCid);
      },
  
      // Find by index. This is not guaranteed to be a
      // stable index.
      findByIndex: function(index){
        return _.values(this._views)[index];
      },
  
      // retrieve a view by its `cid` directly
      findByCid: function(cid){
        return this._views[cid];
      },
  
      // Remove a view
      remove: function(view){
        var viewCid = view.cid;
  
        // delete model index
        if (view.model){
          delete this._indexByModel[view.model.cid];
        }
  
        // delete custom index
        _.any(this._indexByCustom, function(cid, key) {
          if (cid === viewCid) {
            delete this._indexByCustom[key];
            return true;
          }
        }, this);
  
        // remove the view from the container
        delete this._views[viewCid];
  
        // update the length
        this._updateLength();
        return this;
      },
  
      // Call a method on every view in the container,
      // passing parameters to the call method one at a
      // time, like `function.call`.
      call: function(method){
        this.apply(method, _.tail(arguments));
      },
  
      // Apply a method on every view in the container,
      // passing parameters to the call method one at a
      // time, like `function.apply`.
      apply: function(method, args){
        _.each(this._views, function(view){
          if (_.isFunction(view[method])){
            view[method].apply(view, args || []);
          }
        });
      },
  
      // Update the `.length` attribute on this container
      _updateLength: function(){
        this.length = _.size(this._views);
      }
    });
  
    // Borrowing this code from Backbone.Collection:
    // http://backbonejs.org/docs/backbone.html#section-106
    //
    // Mix in methods from Underscore, for iteration, and other
    // collection related features.
    var methods = ['forEach', 'each', 'map', 'find', 'detect', 'filter',
      'select', 'reject', 'every', 'all', 'some', 'any', 'include',
      'contains', 'invoke', 'toArray', 'first', 'initial', 'rest',
      'last', 'without', 'isEmpty', 'pluck', 'reduce'];
  
    _.each(methods, function(method) {
      Container.prototype[method] = function() {
        var views = _.values(this._views);
        var args = [views].concat(_.toArray(arguments));
        return _[method].apply(_, args);
      };
    });
  
    // return the public API
    return Container;
  })(Backbone, _);
  

  Backbone.ChildViewContainer.VERSION = '0.1.11';

  Backbone.ChildViewContainer.noConflict = function () {
    Backbone.ChildViewContainer = previousChildViewContainer;
    return this;
  };

  return Backbone.ChildViewContainer;

}));

// MarionetteJS (Backbone.Marionette)
// ----------------------------------
// v1.8.8
//
// Copyright (c)2014 Derick Bailey, Muted Solutions, LLC.
// Distributed under MIT license
//
// http://marionettejs.com

(function (root, factory) {
  if (typeof exports === 'object') {

    var underscore = require('underscore');
    var backbone = require('backbone');
    var wreqr = require('backbone.wreqr');
    var babysitter = require('backbone.babysitter');

    module.exports = factory(underscore, backbone, wreqr, babysitter);

  } else if (typeof define === 'function' && define.amd) {

    define('marionette',['underscore', 'backbone', 'backbone.wreqr', 'backbone.babysitter'], factory);

  }
}(this, function (_, Backbone) {

  var Marionette = (function(global, Backbone, _){
    "use strict";
  
    // Define and export the Marionette namespace
    var Marionette = {};
    Backbone.Marionette = Marionette;
  
    // Get the DOM manipulator for later use
    Marionette.$ = Backbone.$;
  
  // Helpers
  // -------
  
  // For slicing `arguments` in functions
  var slice = Array.prototype.slice;
  
  function throwError(message, name) {
    var error = new Error(message);
    error.name = name || 'Error';
    throw error;
  }
  
  // Marionette.extend
  // -----------------
  
  // Borrow the Backbone `extend` method so we can use it as needed
  Marionette.extend = Backbone.Model.extend;
  
  // Marionette.getOption
  // --------------------
  
  // Retrieve an object, function or other value from a target
  // object or its `options`, with `options` taking precedence.
  Marionette.getOption = function(target, optionName){
    if (!target || !optionName){ return; }
    var value;
  
    if (target.options && (optionName in target.options) && (target.options[optionName] !== undefined)){
      value = target.options[optionName];
    } else {
      value = target[optionName];
    }
  
    return value;
  };
  
  // Marionette.normalizeMethods
  // ----------------------
  
  // Pass in a mapping of events => functions or function names
  // and return a mapping of events => functions
  Marionette.normalizeMethods = function(hash) {
    var normalizedHash = {}, method;
    _.each(hash, function(fn, name) {
      method = fn;
      if (!_.isFunction(method)) {
        method = this[method];
      }
      if (!method) {
        return;
      }
      normalizedHash[name] = method;
    }, this);
    return normalizedHash;
  };
  
  
  // allows for the use of the @ui. syntax within
  // a given key for triggers and events
  // swaps the @ui with the associated selector
  Marionette.normalizeUIKeys = function(hash, ui) {
    if (typeof(hash) === "undefined") {
      return;
    }
  
    _.each(_.keys(hash), function(v) {
      var pattern = /@ui.[a-zA-Z_$0-9]*/g;
      if (v.match(pattern)) {
        hash[v.replace(pattern, function(r) {
          return ui[r.slice(4)];
        })] = hash[v];
        delete hash[v];
      }
    });
  
    return hash;
  };
  
  // Mix in methods from Underscore, for iteration, and other
  // collection related features.
  // Borrowing this code from Backbone.Collection:
  // http://backbonejs.org/docs/backbone.html#section-106
  Marionette.actAsCollection = function(object, listProperty) {
    var methods = ['forEach', 'each', 'map', 'find', 'detect', 'filter',
      'select', 'reject', 'every', 'all', 'some', 'any', 'include',
      'contains', 'invoke', 'toArray', 'first', 'initial', 'rest',
      'last', 'without', 'isEmpty', 'pluck'];
  
    _.each(methods, function(method) {
      object[method] = function() {
        var list = _.values(_.result(this, listProperty));
        var args = [list].concat(_.toArray(arguments));
        return _[method].apply(_, args);
      };
    });
  };
  
  // Trigger an event and/or a corresponding method name. Examples:
  //
  // `this.triggerMethod("foo")` will trigger the "foo" event and
  // call the "onFoo" method.
  //
  // `this.triggerMethod("foo:bar")` will trigger the "foo:bar" event and
  // call the "onFooBar" method.
  Marionette.triggerMethod = (function(){
  
    // split the event name on the ":"
    var splitter = /(^|:)(\w)/gi;
  
    // take the event section ("section1:section2:section3")
    // and turn it in to uppercase name
    function getEventName(match, prefix, eventName) {
      return eventName.toUpperCase();
    }
  
    // actual triggerMethod implementation
    var triggerMethod = function(event) {
      // get the method name from the event name
      var methodName = 'on' + event.replace(splitter, getEventName);
      var method = this[methodName];
  
      // trigger the event, if a trigger method exists
      if(_.isFunction(this.trigger)) {
        this.trigger.apply(this, arguments);
      }
  
      // call the onMethodName if it exists
      if (_.isFunction(method)) {
        // pass all arguments, except the event name
        return method.apply(this, _.tail(arguments));
      }
    };
  
    return triggerMethod;
  })();
  
  // DOMRefresh
  // ----------
  //
  // Monitor a view's state, and after it has been rendered and shown
  // in the DOM, trigger a "dom:refresh" event every time it is
  // re-rendered.
  
  Marionette.MonitorDOMRefresh = (function(documentElement){
    // track when the view has been shown in the DOM,
    // using a Marionette.Region (or by other means of triggering "show")
    function handleShow(view){
      view._isShown = true;
      triggerDOMRefresh(view);
    }
  
    // track when the view has been rendered
    function handleRender(view){
      view._isRendered = true;
      triggerDOMRefresh(view);
    }
  
    // Trigger the "dom:refresh" event and corresponding "onDomRefresh" method
    function triggerDOMRefresh(view){
      if (view._isShown && view._isRendered && isInDOM(view)){
        if (_.isFunction(view.triggerMethod)){
          view.triggerMethod("dom:refresh");
        }
      }
    }
  
    function isInDOM(view) {
      return documentElement.contains(view.el);
    }
  
    // Export public API
    return function(view){
      view.listenTo(view, "show", function(){
        handleShow(view);
      });
  
      view.listenTo(view, "render", function(){
        handleRender(view);
      });
    };
  })(document.documentElement);
  
  
  // Marionette.bindEntityEvents & unbindEntityEvents
  // ---------------------------
  //
  // These methods are used to bind/unbind a backbone "entity" (collection/model)
  // to methods on a target object.
  //
  // The first parameter, `target`, must have a `listenTo` method from the
  // EventBinder object.
  //
  // The second parameter is the entity (Backbone.Model or Backbone.Collection)
  // to bind the events from.
  //
  // The third parameter is a hash of { "event:name": "eventHandler" }
  // configuration. Multiple handlers can be separated by a space. A
  // function can be supplied instead of a string handler name.
  
  (function(Marionette){
    "use strict";
  
    // Bind the event to handlers specified as a string of
    // handler names on the target object
    function bindFromStrings(target, entity, evt, methods){
      var methodNames = methods.split(/\s+/);
  
      _.each(methodNames, function(methodName) {
  
        var method = target[methodName];
        if(!method) {
          throwError("Method '"+ methodName +"' was configured as an event handler, but does not exist.");
        }
  
        target.listenTo(entity, evt, method);
      });
    }
  
    // Bind the event to a supplied callback function
    function bindToFunction(target, entity, evt, method){
        target.listenTo(entity, evt, method);
    }
  
    // Bind the event to handlers specified as a string of
    // handler names on the target object
    function unbindFromStrings(target, entity, evt, methods){
      var methodNames = methods.split(/\s+/);
  
      _.each(methodNames, function(methodName) {
        var method = target[methodName];
        target.stopListening(entity, evt, method);
      });
    }
  
    // Bind the event to a supplied callback function
    function unbindToFunction(target, entity, evt, method){
        target.stopListening(entity, evt, method);
    }
  
  
    // generic looping function
    function iterateEvents(target, entity, bindings, functionCallback, stringCallback){
      if (!entity || !bindings) { return; }
  
      // allow the bindings to be a function
      if (_.isFunction(bindings)){
        bindings = bindings.call(target);
      }
  
      // iterate the bindings and bind them
      _.each(bindings, function(methods, evt){
  
        // allow for a function as the handler,
        // or a list of event names as a string
        if (_.isFunction(methods)){
          functionCallback(target, entity, evt, methods);
        } else {
          stringCallback(target, entity, evt, methods);
        }
  
      });
    }
  
    // Export Public API
    Marionette.bindEntityEvents = function(target, entity, bindings){
      iterateEvents(target, entity, bindings, bindToFunction, bindFromStrings);
    };
  
    Marionette.unbindEntityEvents = function(target, entity, bindings){
      iterateEvents(target, entity, bindings, unbindToFunction, unbindFromStrings);
    };
  
  })(Marionette);
  
  
  // Callbacks
  // ---------
  
  // A simple way of managing a collection of callbacks
  // and executing them at a later point in time, using jQuery's
  // `Deferred` object.
  Marionette.Callbacks = function(){
    this._deferred = Marionette.$.Deferred();
    this._callbacks = [];
  };
  
  _.extend(Marionette.Callbacks.prototype, {
  
    // Add a callback to be executed. Callbacks added here are
    // guaranteed to execute, even if they are added after the
    // `run` method is called.
    add: function(callback, contextOverride){
      this._callbacks.push({cb: callback, ctx: contextOverride});
  
      this._deferred.done(function(context, options){
        if (contextOverride){ context = contextOverride; }
        callback.call(context, options);
      });
    },
  
    // Run all registered callbacks with the context specified.
    // Additional callbacks can be added after this has been run
    // and they will still be executed.
    run: function(options, context){
      this._deferred.resolve(context, options);
    },
  
    // Resets the list of callbacks to be run, allowing the same list
    // to be run multiple times - whenever the `run` method is called.
    reset: function(){
      var callbacks = this._callbacks;
      this._deferred = Marionette.$.Deferred();
      this._callbacks = [];
  
      _.each(callbacks, function(cb){
        this.add(cb.cb, cb.ctx);
      }, this);
    }
  });
  
  // Marionette Controller
  // ---------------------
  //
  // A multi-purpose object to use as a controller for
  // modules and routers, and as a mediator for workflow
  // and coordination of other objects, views, and more.
  Marionette.Controller = function(options){
    this.triggerMethod = Marionette.triggerMethod;
    this.options = options || {};
  
    if (_.isFunction(this.initialize)){
      this.initialize(this.options);
    }
  };
  
  Marionette.Controller.extend = Marionette.extend;
  
  // Controller Methods
  // --------------
  
  // Ensure it can trigger events with Backbone.Events
  _.extend(Marionette.Controller.prototype, Backbone.Events, {
    close: function(){
      this.stopListening();
      var args = Array.prototype.slice.call(arguments);
      this.triggerMethod.apply(this, ["close"].concat(args));
      this.off();
    }
  });
  
  // Region
  // ------
  //
  // Manage the visual regions of your composite application. See
  // http://lostechies.com/derickbailey/2011/12/12/composite-js-apps-regions-and-region-managers/
  
  Marionette.Region = function(options){
    this.options = options || {};
    this.el = Marionette.getOption(this, "el");
  
    if (!this.el){
      throwError("An 'el' must be specified for a region.", "NoElError");
    }
  
    if (this.initialize){
      var args = Array.prototype.slice.apply(arguments);
      this.initialize.apply(this, args);
    }
  };
  
  
  // Region Type methods
  // -------------------
  
  _.extend(Marionette.Region, {
  
    // Build an instance of a region by passing in a configuration object
    // and a default region type to use if none is specified in the config.
    //
    // The config object should either be a string as a jQuery DOM selector,
    // a Region type directly, or an object literal that specifies both
    // a selector and regionType:
    //
    // ```js
    // {
    //   selector: "#foo",
    //   regionType: MyCustomRegion
    // }
    // ```
    //
    buildRegion: function(regionConfig, defaultRegionType){
      var regionIsString = _.isString(regionConfig);
      var regionSelectorIsString = _.isString(regionConfig.selector);
      var regionTypeIsUndefined = _.isUndefined(regionConfig.regionType);
      var regionIsType = _.isFunction(regionConfig);
  
      if (!regionIsType && !regionIsString && !regionSelectorIsString) {
        throwError("Region must be specified as a Region type, a selector string or an object with selector property");
      }
  
      var selector, RegionType;
  
      // get the selector for the region
  
      if (regionIsString) {
        selector = regionConfig;
      }
  
      if (regionConfig.selector) {
        selector = regionConfig.selector;
        delete regionConfig.selector;
      }
  
      // get the type for the region
  
      if (regionIsType){
        RegionType = regionConfig;
      }
  
      if (!regionIsType && regionTypeIsUndefined) {
        RegionType = defaultRegionType;
      }
  
      if (regionConfig.regionType) {
        RegionType = regionConfig.regionType;
        delete regionConfig.regionType;
      }
  
      if (regionIsString || regionIsType) {
        regionConfig = {};
      }
  
      regionConfig.el = selector;
  
      // build the region instance
      var region = new RegionType(regionConfig);
  
      // override the `getEl` function if we have a parentEl
      // this must be overridden to ensure the selector is found
      // on the first use of the region. if we try to assign the
      // region's `el` to `parentEl.find(selector)` in the object
      // literal to build the region, the element will not be
      // guaranteed to be in the DOM already, and will cause problems
      if (regionConfig.parentEl){
        region.getEl = function(selector) {
          var parentEl = regionConfig.parentEl;
          if (_.isFunction(parentEl)){
            parentEl = parentEl();
          }
          return parentEl.find(selector);
        };
      }
  
      return region;
    }
  
  });
  
  // Region Instance Methods
  // -----------------------
  
  _.extend(Marionette.Region.prototype, Backbone.Events, {
  
    // Displays a backbone view instance inside of the region.
    // Handles calling the `render` method for you. Reads content
    // directly from the `el` attribute. Also calls an optional
    // `onShow` and `close` method on your view, just after showing
    // or just before closing the view, respectively.
    // The `preventClose` option can be used to prevent a view from being destroyed on show.
    show: function(view, options){
      this.ensureEl();
  
      var showOptions = options || {};
      var isViewClosed = view.isClosed || _.isUndefined(view.$el);
      var isDifferentView = view !== this.currentView;
      var preventClose =  !!showOptions.preventClose;
  
      // only close the view if we don't want to preventClose and the view is different
      var _shouldCloseView = !preventClose && isDifferentView;
  
      if (_shouldCloseView) {
        this.close();
      }
  
      view.render();
      Marionette.triggerMethod.call(this, "before:show", view);
  
      if (_.isFunction(view.triggerMethod)) {
        view.triggerMethod("before:show");
      } else {
        Marionette.triggerMethod.call(view, "before:show");
      }
  
      if (isDifferentView || isViewClosed) {
        this.open(view);
      }
  
      this.currentView = view;
  
      Marionette.triggerMethod.call(this, "show", view);
  
      if (_.isFunction(view.triggerMethod)) {
        view.triggerMethod("show");
      } else {
        Marionette.triggerMethod.call(view, "show");
      }
  
      return this;
    },
  
    ensureEl: function(){
      if (!this.$el || this.$el.length === 0){
        this.$el = this.getEl(this.el);
      }
    },
  
    // Override this method to change how the region finds the
    // DOM element that it manages. Return a jQuery selector object.
    getEl: function(selector){
      return Marionette.$(selector);
    },
  
    // Override this method to change how the new view is
    // appended to the `$el` that the region is managing
    open: function(view){
      this.$el.empty().append(view.el);
    },
  
    // Close the current view, if there is one. If there is no
    // current view, it does nothing and returns immediately.
    close: function(){
      var view = this.currentView;
      if (!view || view.isClosed){ return; }
  
      // call 'close' or 'remove', depending on which is found
      if (view.close) { view.close(); }
      else if (view.remove) { view.remove(); }
  
      Marionette.triggerMethod.call(this, "close", view);
  
      delete this.currentView;
    },
  
    // Attach an existing view to the region. This
    // will not call `render` or `onShow` for the new view,
    // and will not replace the current HTML for the `el`
    // of the region.
    attachView: function(view){
      this.currentView = view;
    },
  
    // Reset the region by closing any existing view and
    // clearing out the cached `$el`. The next time a view
    // is shown via this region, the region will re-query the
    // DOM for the region's `el`.
    reset: function(){
      this.close();
      delete this.$el;
    }
  });
  
  // Copy the `extend` function used by Backbone's classes
  Marionette.Region.extend = Marionette.extend;
  
  // Marionette.RegionManager
  // ------------------------
  //
  // Manage one or more related `Marionette.Region` objects.
  Marionette.RegionManager = (function(Marionette){
  
    var RegionManager = Marionette.Controller.extend({
      constructor: function(options){
        this._regions = {};
        Marionette.Controller.prototype.constructor.call(this, options);
      },
  
      // Add multiple regions using an object literal, where
      // each key becomes the region name, and each value is
      // the region definition.
      addRegions: function(regionDefinitions, defaults){
        var regions = {};
  
        _.each(regionDefinitions, function(definition, name){
          if (_.isString(definition)){
            definition = { selector: definition };
          }
  
          if (definition.selector){
            definition = _.defaults({}, definition, defaults);
          }
  
          var region = this.addRegion(name, definition);
          regions[name] = region;
        }, this);
  
        return regions;
      },
  
      // Add an individual region to the region manager,
      // and return the region instance
      addRegion: function(name, definition){
        var region;
  
        var isObject = _.isObject(definition);
        var isString = _.isString(definition);
        var hasSelector = !!definition.selector;
  
        if (isString || (isObject && hasSelector)){
          region = Marionette.Region.buildRegion(definition, Marionette.Region);
        } else if (_.isFunction(definition)){
          region = Marionette.Region.buildRegion(definition, Marionette.Region);
        } else {
          region = definition;
        }
  
        this._store(name, region);
        this.triggerMethod("region:add", name, region);
        return region;
      },
  
      // Get a region by name
      get: function(name){
        return this._regions[name];
      },
  
      // Remove a region by name
      removeRegion: function(name){
        var region = this._regions[name];
        this._remove(name, region);
      },
  
      // Close all regions in the region manager, and
      // remove them
      removeRegions: function(){
        _.each(this._regions, function(region, name){
          this._remove(name, region);
        }, this);
      },
  
      // Close all regions in the region manager, but
      // leave them attached
      closeRegions: function(){
        _.each(this._regions, function(region, name){
          region.close();
        }, this);
      },
  
      // Close all regions and shut down the region
      // manager entirely
      close: function(){
        this.removeRegions();
        Marionette.Controller.prototype.close.apply(this, arguments);
      },
  
      // internal method to store regions
      _store: function(name, region){
        this._regions[name] = region;
        this._setLength();
      },
  
      // internal method to remove a region
      _remove: function(name, region){
        region.close();
        region.stopListening();
        delete this._regions[name];
        this._setLength();
        this.triggerMethod("region:remove", name, region);
      },
  
      // set the number of regions current held
      _setLength: function(){
        this.length = _.size(this._regions);
      }
  
    });
  
    Marionette.actAsCollection(RegionManager.prototype, '_regions');
  
    return RegionManager;
  })(Marionette);
  
  
  // Template Cache
  // --------------
  
  // Manage templates stored in `<script>` blocks,
  // caching them for faster access.
  Marionette.TemplateCache = function(templateId){
    this.templateId = templateId;
  };
  
  // TemplateCache object-level methods. Manage the template
  // caches from these method calls instead of creating
  // your own TemplateCache instances
  _.extend(Marionette.TemplateCache, {
    templateCaches: {},
  
    // Get the specified template by id. Either
    // retrieves the cached version, or loads it
    // from the DOM.
    get: function(templateId){
      var cachedTemplate = this.templateCaches[templateId];
  
      if (!cachedTemplate){
        cachedTemplate = new Marionette.TemplateCache(templateId);
        this.templateCaches[templateId] = cachedTemplate;
      }
  
      return cachedTemplate.load();
    },
  
    // Clear templates from the cache. If no arguments
    // are specified, clears all templates:
    // `clear()`
    //
    // If arguments are specified, clears each of the
    // specified templates from the cache:
    // `clear("#t1", "#t2", "...")`
    clear: function(){
      var i;
      var args = slice.call(arguments);
      var length = args.length;
  
      if (length > 0){
        for(i=0; i<length; i++){
          delete this.templateCaches[args[i]];
        }
      } else {
        this.templateCaches = {};
      }
    }
  });
  
  // TemplateCache instance methods, allowing each
  // template cache object to manage its own state
  // and know whether or not it has been loaded
  _.extend(Marionette.TemplateCache.prototype, {
  
    // Internal method to load the template
    load: function(){
      // Guard clause to prevent loading this template more than once
      if (this.compiledTemplate){
        return this.compiledTemplate;
      }
  
      // Load the template and compile it
      var template = this.loadTemplate(this.templateId);
      this.compiledTemplate = this.compileTemplate(template);
  
      return this.compiledTemplate;
    },
  
    // Load a template from the DOM, by default. Override
    // this method to provide your own template retrieval
    // For asynchronous loading with AMD/RequireJS, consider
    // using a template-loader plugin as described here:
    // https://github.com/marionettejs/backbone.marionette/wiki/Using-marionette-with-requirejs
    loadTemplate: function(templateId){
      var template = Marionette.$(templateId).html();
  
      if (!template || template.length === 0){
        throwError("Could not find template: '" + templateId + "'", "NoTemplateError");
      }
  
      return template;
    },
  
    // Pre-compile the template before caching it. Override
    // this method if you do not need to pre-compile a template
    // (JST / RequireJS for example) or if you want to change
    // the template engine used (Handebars, etc).
    compileTemplate: function(rawTemplate){
      return _.template(rawTemplate);
    }
  });
  
  // Renderer
  // --------
  
  // Render a template with data by passing in the template
  // selector and the data to render.
  Marionette.Renderer = {
  
    // Render a template with data. The `template` parameter is
    // passed to the `TemplateCache` object to retrieve the
    // template function. Override this method to provide your own
    // custom rendering and template handling for all of Marionette.
    render: function(template, data){
  
      if (!template) {
        throwError("Cannot render the template since it's false, null or undefined.", "TemplateNotFoundError");
      }
  
      var templateFunc;
      if (typeof template === "function"){
        templateFunc = template;
      } else {
        templateFunc = Marionette.TemplateCache.get(template);
      }
  
      return templateFunc(data);
    }
  };
  
  
  // Marionette.View
  // ---------------
  
  // The core view type that other Marionette views extend from.
  Marionette.View = Backbone.View.extend({
  
    constructor: function(options){
      _.bindAll(this, "render");
  
      // this exposes view options to the view initializer
      // this is a backfill since backbone removed the assignment
      // of this.options
      // at some point however this may be removed
      this.options = _.extend({}, _.result(this, 'options'), _.isFunction(options) ? options.call(this) : options);
  
      // parses out the @ui DSL for events
      this.events = this.normalizeUIKeys(_.result(this, 'events'));
  
      if (_.isObject(this.behaviors)) {
        new Marionette.Behaviors(this);
      }
  
      Backbone.View.prototype.constructor.apply(this, arguments);
  
      Marionette.MonitorDOMRefresh(this);
      this.listenTo(this, "show", this.onShowCalled);
    },
  
    // import the "triggerMethod" to trigger events with corresponding
    // methods if the method exists
    triggerMethod: Marionette.triggerMethod,
  
    // Imports the "normalizeMethods" to transform hashes of
    // events=>function references/names to a hash of events=>function references
    normalizeMethods: Marionette.normalizeMethods,
  
    // Get the template for this view
    // instance. You can set a `template` attribute in the view
    // definition or pass a `template: "whatever"` parameter in
    // to the constructor options.
    getTemplate: function(){
      return Marionette.getOption(this, "template");
    },
  
    // Mix in template helper methods. Looks for a
    // `templateHelpers` attribute, which can either be an
    // object literal, or a function that returns an object
    // literal. All methods and attributes from this object
    // are copies to the object passed in.
    mixinTemplateHelpers: function(target){
      target = target || {};
      var templateHelpers = Marionette.getOption(this, "templateHelpers");
      if (_.isFunction(templateHelpers)){
        templateHelpers = templateHelpers.call(this);
      }
      return _.extend(target, templateHelpers);
    },
  
  
    normalizeUIKeys: function(hash) {
      var ui = _.result(this, 'ui');
      return Marionette.normalizeUIKeys(hash, ui);
    },
  
    // Configure `triggers` to forward DOM events to view
    // events. `triggers: {"click .foo": "do:foo"}`
    configureTriggers: function(){
      if (!this.triggers) { return; }
  
      var triggerEvents = {};
  
      // Allow `triggers` to be configured as a function
      var triggers = this.normalizeUIKeys(_.result(this, "triggers"));
  
      // Configure the triggers, prevent default
      // action and stop propagation of DOM events
      _.each(triggers, function(value, key){
  
        var hasOptions = _.isObject(value);
        var eventName = hasOptions ? value.event : value;
  
        // build the event handler function for the DOM event
        triggerEvents[key] = function(e){
  
          // stop the event in its tracks
          if (e) {
            var prevent = e.preventDefault;
            var stop = e.stopPropagation;
  
            var shouldPrevent = hasOptions ? value.preventDefault : prevent;
            var shouldStop = hasOptions ? value.stopPropagation : stop;
  
            if (shouldPrevent && prevent) { prevent.apply(e); }
            if (shouldStop && stop) { stop.apply(e); }
          }
  
          // build the args for the event
          var args = {
            view: this,
            model: this.model,
            collection: this.collection
          };
  
          // trigger the event
          this.triggerMethod(eventName, args);
        };
  
      }, this);
  
      return triggerEvents;
    },
  
    // Overriding Backbone.View's delegateEvents to handle
    // the `triggers`, `modelEvents`, and `collectionEvents` configuration
    delegateEvents: function(events){
      this._delegateDOMEvents(events);
      Marionette.bindEntityEvents(this, this.model, Marionette.getOption(this, "modelEvents"));
      Marionette.bindEntityEvents(this, this.collection, Marionette.getOption(this, "collectionEvents"));
    },
  
    // internal method to delegate DOM events and triggers
    _delegateDOMEvents: function(events){
      events = events || this.events;
      if (_.isFunction(events)){ events = events.call(this); }
  
      var combinedEvents = {};
  
      // look up if this view has behavior events
      var behaviorEvents = _.result(this, 'behaviorEvents') || {};
      var triggers = this.configureTriggers();
  
      // behavior events will be overriden by view events and or triggers
      _.extend(combinedEvents, behaviorEvents, events, triggers);
  
      Backbone.View.prototype.delegateEvents.call(this, combinedEvents);
    },
  
    // Overriding Backbone.View's undelegateEvents to handle unbinding
    // the `triggers`, `modelEvents`, and `collectionEvents` config
    undelegateEvents: function(){
      var args = Array.prototype.slice.call(arguments);
      Backbone.View.prototype.undelegateEvents.apply(this, args);
  
      Marionette.unbindEntityEvents(this, this.model, Marionette.getOption(this, "modelEvents"));
      Marionette.unbindEntityEvents(this, this.collection, Marionette.getOption(this, "collectionEvents"));
    },
  
    // Internal method, handles the `show` event.
    onShowCalled: function(){},
  
    // Default `close` implementation, for removing a view from the
    // DOM and unbinding it. Regions will call this method
    // for you. You can specify an `onClose` method in your view to
    // add custom code that is called after the view is closed.
    close: function(){
      if (this.isClosed) { return; }
  
      var args = Array.prototype.slice.call(arguments);
  
      // allow the close to be stopped by returning `false`
      // from the `onBeforeClose` method
      var shouldClose = this.triggerMethod.apply(this, ["before:close"].concat(args));
      if (shouldClose === false){
        return;
      }
  
      // mark as closed before doing the actual close, to
      // prevent infinite loops within "close" event handlers
      // that are trying to close other views
      this.isClosed = true;
      this.triggerMethod.apply(this, ["close"].concat(args));
  
      // unbind UI elements
      this.unbindUIElements();
  
      // remove the view from the DOM
      this.remove();
    },
  
    // This method binds the elements specified in the "ui" hash inside the view's code with
    // the associated jQuery selectors.
    bindUIElements: function(){
      if (!this.ui) { return; }
  
      // store the ui hash in _uiBindings so they can be reset later
      // and so re-rendering the view will be able to find the bindings
      if (!this._uiBindings){
        this._uiBindings = this.ui;
      }
  
      // get the bindings result, as a function or otherwise
      var bindings = _.result(this, "_uiBindings");
  
      // empty the ui so we don't have anything to start with
      this.ui = {};
  
      // bind each of the selectors
      _.each(_.keys(bindings), function(key) {
        var selector = bindings[key];
        this.ui[key] = this.$(selector);
      }, this);
    },
  
    // This method unbinds the elements specified in the "ui" hash
    unbindUIElements: function(){
      if (!this.ui || !this._uiBindings){ return; }
  
      // delete all of the existing ui bindings
      _.each(this.ui, function($el, name){
        delete this.ui[name];
      }, this);
  
      // reset the ui element to the original bindings configuration
      this.ui = this._uiBindings;
      delete this._uiBindings;
    }
  });
  
  // Item View
  // ---------
  
  // A single item view implementation that contains code for rendering
  // with underscore.js templates, serializing the view's model or collection,
  // and calling several methods on extended views, such as `onRender`.
  Marionette.ItemView = Marionette.View.extend({
  
    // Setting up the inheritance chain which allows changes to
    // Marionette.View.prototype.constructor which allows overriding
    constructor: function(){
      Marionette.View.prototype.constructor.apply(this, arguments);
    },
  
    // Serialize the model or collection for the view. If a model is
    // found, `.toJSON()` is called. If a collection is found, `.toJSON()`
    // is also called, but is used to populate an `items` array in the
    // resulting data. If both are found, defaults to the model.
    // You can override the `serializeData` method in your own view
    // definition, to provide custom serialization for your view's data.
    serializeData: function(){
      var data = {};
  
      if (this.model) {
        data = this.model.toJSON();
      }
      else if (this.collection) {
        data = { items: this.collection.toJSON() };
      }
  
      return data;
    },
  
    // Render the view, defaulting to underscore.js templates.
    // You can override this in your view definition to provide
    // a very specific rendering for your view. In general, though,
    // you should override the `Marionette.Renderer` object to
    // change how Marionette renders views.
    render: function(){
      this.isClosed = false;
  
      this.triggerMethod("before:render", this);
      this.triggerMethod("item:before:render", this);
  
      var data = this.serializeData();
      data = this.mixinTemplateHelpers(data);
  
      var template = this.getTemplate();
      var html = Marionette.Renderer.render(template, data);
  
      this.$el.html(html);
      this.bindUIElements();
  
      this.triggerMethod("render", this);
      this.triggerMethod("item:rendered", this);
  
      return this;
    },
  
    // Override the default close event to add a few
    // more events that are triggered.
    close: function(){
      if (this.isClosed){ return; }
  
      this.triggerMethod('item:before:close');
  
      Marionette.View.prototype.close.apply(this, arguments);
  
      this.triggerMethod('item:closed');
    }
  });
  
  // Collection View
  // ---------------
  
  // A view that iterates over a Backbone.Collection
  // and renders an individual ItemView for each model.
  Marionette.CollectionView = Marionette.View.extend({
    // used as the prefix for item view events
    // that are forwarded through the collectionview
    itemViewEventPrefix: "itemview",
  
    // constructor
    constructor: function(options){
      this._initChildViewStorage();
  
      Marionette.View.prototype.constructor.apply(this, arguments);
  
      this._initialEvents();
      this.initRenderBuffer();
    },
  
    // Instead of inserting elements one by one into the page,
    // it's much more performant to insert elements into a document
    // fragment and then insert that document fragment into the page
    initRenderBuffer: function() {
      this.elBuffer = document.createDocumentFragment();
      this._bufferedChildren = [];
    },
  
    startBuffering: function() {
      this.initRenderBuffer();
      this.isBuffering = true;
    },
  
    endBuffering: function() {
      this.isBuffering = false;
      this.appendBuffer(this, this.elBuffer);
      this._triggerShowBufferedChildren();
      this.initRenderBuffer();
    },
  
    _triggerShowBufferedChildren: function () {
      if (this._isShown) {
        _.each(this._bufferedChildren, function (child) {
          if (_.isFunction(child.triggerMethod)) {
            child.triggerMethod('show');
          } else {
            Marionette.triggerMethod.call(child, 'show');
          }
        });
        this._bufferedChildren = [];
      }
    },
  
    // Configured the initial events that the collection view
    // binds to.
    _initialEvents: function(){
      if (this.collection){
        this.listenTo(this.collection, "add", this.addChildView);
        this.listenTo(this.collection, "remove", this.removeItemView);
        this.listenTo(this.collection, "reset", this.render);
      }
    },
  
    // Handle a child item added to the collection
    addChildView: function(item, collection, options){
      this.closeEmptyView();
      var ItemView = this.getItemView(item);
      var index = this.collection.indexOf(item);
      this.addItemView(item, ItemView, index);
    },
  
    // Override from `Marionette.View` to guarantee the `onShow` method
    // of child views is called.
    onShowCalled: function(){
      this.children.each(function(child){
        if (_.isFunction(child.triggerMethod)) {
          child.triggerMethod('show');
        } else {
          Marionette.triggerMethod.call(child, 'show');
        }
      });
    },
  
    // Internal method to trigger the before render callbacks
    // and events
    triggerBeforeRender: function(){
      this.triggerMethod("before:render", this);
      this.triggerMethod("collection:before:render", this);
    },
  
    // Internal method to trigger the rendered callbacks and
    // events
    triggerRendered: function(){
      this.triggerMethod("render", this);
      this.triggerMethod("collection:rendered", this);
    },
  
    // Render the collection of items. Override this method to
    // provide your own implementation of a render function for
    // the collection view.
    render: function(){
      this.isClosed = false;
      this.triggerBeforeRender();
      this._renderChildren();
      this.triggerRendered();
      return this;
    },
  
    // Internal method. Separated so that CompositeView can have
    // more control over events being triggered, around the rendering
    // process
    _renderChildren: function(){
      this.startBuffering();
  
      this.closeEmptyView();
      this.closeChildren();
  
      if (!this.isEmpty(this.collection)) {
        this.showCollection();
      } else {
        this.showEmptyView();
      }
  
      this.endBuffering();
    },
  
    // Internal method to loop through each item in the
    // collection view and show it
    showCollection: function(){
      var ItemView;
      this.collection.each(function(item, index){
        ItemView = this.getItemView(item);
        this.addItemView(item, ItemView, index);
      }, this);
    },
  
    // Internal method to show an empty view in place of
    // a collection of item views, when the collection is
    // empty
    showEmptyView: function(){
      var EmptyView = this.getEmptyView();
  
      if (EmptyView && !this._showingEmptyView){
        this._showingEmptyView = true;
        var model = new Backbone.Model();
        this.addItemView(model, EmptyView, 0);
      }
    },
  
    // Internal method to close an existing emptyView instance
    // if one exists. Called when a collection view has been
    // rendered empty, and then an item is added to the collection.
    closeEmptyView: function(){
      if (this._showingEmptyView){
        this.closeChildren();
        delete this._showingEmptyView;
      }
    },
  
    // Retrieve the empty view type
    getEmptyView: function(){
      return Marionette.getOption(this, "emptyView");
    },
  
    // Retrieve the itemView type, either from `this.options.itemView`
    // or from the `itemView` in the object definition. The "options"
    // takes precedence.
    getItemView: function(item){
      var itemView = Marionette.getOption(this, "itemView");
  
      if (!itemView){
        throwError("An `itemView` must be specified", "NoItemViewError");
      }
  
      return itemView;
    },
  
    // Render the child item's view and add it to the
    // HTML for the collection view.
    addItemView: function(item, ItemView, index){
      // get the itemViewOptions if any were specified
      var itemViewOptions = Marionette.getOption(this, "itemViewOptions");
      if (_.isFunction(itemViewOptions)){
        itemViewOptions = itemViewOptions.call(this, item, index);
      }
  
      // build the view
      var view = this.buildItemView(item, ItemView, itemViewOptions);
  
      // set up the child view event forwarding
      this.addChildViewEventForwarding(view);
  
      // this view is about to be added
      this.triggerMethod("before:item:added", view);
  
      // Store the child view itself so we can properly
      // remove and/or close it later
      this.children.add(view);
  
      // Render it and show it
      this.renderItemView(view, index);
  
      // call the "show" method if the collection view
      // has already been shown
      if (this._isShown && !this.isBuffering){
        if (_.isFunction(view.triggerMethod)) {
          view.triggerMethod('show');
        } else {
          Marionette.triggerMethod.call(view, 'show');
        }
      }
  
      // this view was added
      this.triggerMethod("after:item:added", view);
  
      return view;
    },
  
    // Set up the child view event forwarding. Uses an "itemview:"
    // prefix in front of all forwarded events.
    addChildViewEventForwarding: function(view){
      var prefix = Marionette.getOption(this, "itemViewEventPrefix");
  
      // Forward all child item view events through the parent,
      // prepending "itemview:" to the event name
      this.listenTo(view, "all", function(){
        var args = slice.call(arguments);
        var rootEvent = args[0];
        var itemEvents = this.normalizeMethods(this.getItemEvents());
  
        args[0] = prefix + ":" + rootEvent;
        args.splice(1, 0, view);
  
        // call collectionView itemEvent if defined
        if (typeof itemEvents !== "undefined" && _.isFunction(itemEvents[rootEvent])) {
          itemEvents[rootEvent].apply(this, args);
        }
  
        Marionette.triggerMethod.apply(this, args);
      }, this);
    },
  
    // returns the value of itemEvents depending on if a function
    getItemEvents: function() {
      if (_.isFunction(this.itemEvents)) {
        return this.itemEvents.call(this);
      }
  
      return this.itemEvents;
    },
  
    // render the item view
    renderItemView: function(view, index) {
      view.render();
      this.appendHtml(this, view, index);
    },
  
    // Build an `itemView` for every model in the collection.
    buildItemView: function(item, ItemViewType, itemViewOptions){
      var options = _.extend({model: item}, itemViewOptions);
      return new ItemViewType(options);
    },
  
    // get the child view by item it holds, and remove it
    removeItemView: function(item){
      var view = this.children.findByModel(item);
      this.removeChildView(view);
      this.checkEmpty();
    },
  
    // Remove the child view and close it
    removeChildView: function(view){
  
      // shut down the child view properly,
      // including events that the collection has from it
      if (view){
        // call 'close' or 'remove', depending on which is found
        if (view.close) { view.close(); }
        else if (view.remove) { view.remove(); }
  
        this.stopListening(view);
        this.children.remove(view);
      }
  
      this.triggerMethod("item:removed", view);
    },
  
    // helper to check if the collection is empty
    isEmpty: function(collection){
      // check if we're empty now
      return !this.collection || this.collection.length === 0;
    },
  
    // If empty, show the empty view
    checkEmpty: function (){
      if (this.isEmpty(this.collection)){
        this.showEmptyView();
      }
    },
  
    // You might need to override this if you've overridden appendHtml
    appendBuffer: function(collectionView, buffer) {
      collectionView.$el.append(buffer);
    },
  
    // Append the HTML to the collection's `el`.
    // Override this method to do something other
    // than `.append`.
    appendHtml: function(collectionView, itemView, index){
      if (collectionView.isBuffering) {
        // buffering happens on reset events and initial renders
        // in order to reduce the number of inserts into the
        // document, which are expensive.
        collectionView.elBuffer.appendChild(itemView.el);
        collectionView._bufferedChildren.push(itemView);
      }
      else {
        // If we've already rendered the main collection, just
        // append the new items directly into the element.
        collectionView.$el.append(itemView.el);
      }
    },
  
    // Internal method to set up the `children` object for
    // storing all of the child views
    _initChildViewStorage: function(){
      this.children = new Backbone.ChildViewContainer();
    },
  
    // Handle cleanup and other closing needs for
    // the collection of views.
    close: function(){
      if (this.isClosed){ return; }
  
      this.triggerMethod("collection:before:close");
      this.closeChildren();
      this.triggerMethod("collection:closed");
  
      Marionette.View.prototype.close.apply(this, arguments);
    },
  
    // Close the child views that this collection view
    // is holding on to, if any
    closeChildren: function(){
      this.children.each(function(child){
        this.removeChildView(child);
      }, this);
      this.checkEmpty();
    }
  });
  
  // Composite View
  // --------------
  
  // Used for rendering a branch-leaf, hierarchical structure.
  // Extends directly from CollectionView and also renders an
  // an item view as `modelView`, for the top leaf
  Marionette.CompositeView = Marionette.CollectionView.extend({
  
    // Setting up the inheritance chain which allows changes to
    // Marionette.CollectionView.prototype.constructor which allows overriding
    constructor: function(){
      Marionette.CollectionView.prototype.constructor.apply(this, arguments);
    },
  
    // Configured the initial events that the composite view
    // binds to. Override this method to prevent the initial
    // events, or to add your own initial events.
    _initialEvents: function(){
  
      // Bind only after composite view is rendered to avoid adding child views
      // to nonexistent itemViewContainer
      this.once('render', function () {
        if (this.collection){
          this.listenTo(this.collection, "add", this.addChildView);
          this.listenTo(this.collection, "remove", this.removeItemView);
          this.listenTo(this.collection, "reset", this._renderChildren);
        }
      });
  
    },
  
    // Retrieve the `itemView` to be used when rendering each of
    // the items in the collection. The default is to return
    // `this.itemView` or Marionette.CompositeView if no `itemView`
    // has been defined
    getItemView: function(item){
      var itemView = Marionette.getOption(this, "itemView") || this.constructor;
  
      if (!itemView){
        throwError("An `itemView` must be specified", "NoItemViewError");
      }
  
      return itemView;
    },
  
    // Serialize the collection for the view.
    // You can override the `serializeData` method in your own view
    // definition, to provide custom serialization for your view's data.
    serializeData: function(){
      var data = {};
  
      if (this.model){
        data = this.model.toJSON();
      }
  
      return data;
    },
  
    // Renders the model once, and the collection once. Calling
    // this again will tell the model's view to re-render itself
    // but the collection will not re-render.
    render: function(){
      this.isRendered = true;
      this.isClosed = false;
      this.resetItemViewContainer();
  
      this.triggerBeforeRender();
      var html = this.renderModel();
      this.$el.html(html);
      // the ui bindings is done here and not at the end of render since they
      // will not be available until after the model is rendered, but should be
      // available before the collection is rendered.
      this.bindUIElements();
      this.triggerMethod("composite:model:rendered");
  
      this._renderChildren();
  
      this.triggerMethod("composite:rendered");
      this.triggerRendered();
      return this;
    },
  
    _renderChildren: function(){
      if (this.isRendered){
        this.triggerMethod("composite:collection:before:render");
        Marionette.CollectionView.prototype._renderChildren.call(this);
        this.triggerMethod("composite:collection:rendered");
      }
    },
  
    // Render an individual model, if we have one, as
    // part of a composite view (branch / leaf). For example:
    // a treeview.
    renderModel: function(){
      var data = {};
      data = this.serializeData();
      data = this.mixinTemplateHelpers(data);
  
      var template = this.getTemplate();
      return Marionette.Renderer.render(template, data);
    },
  
  
    // You might need to override this if you've overridden appendHtml
    appendBuffer: function(compositeView, buffer) {
      var $container = this.getItemViewContainer(compositeView);
      $container.append(buffer);
    },
  
    // Appends the `el` of itemView instances to the specified
    // `itemViewContainer` (a jQuery selector). Override this method to
    // provide custom logic of how the child item view instances have their
    // HTML appended to the composite view instance.
    appendHtml: function(compositeView, itemView, index){
      if (compositeView.isBuffering) {
        compositeView.elBuffer.appendChild(itemView.el);
        compositeView._bufferedChildren.push(itemView);
      }
      else {
        // If we've already rendered the main collection, just
        // append the new items directly into the element.
        var $container = this.getItemViewContainer(compositeView);
        $container.append(itemView.el);
      }
    },
  
    // Internal method to ensure an `$itemViewContainer` exists, for the
    // `appendHtml` method to use.
    getItemViewContainer: function(containerView){
      if ("$itemViewContainer" in containerView){
        return containerView.$itemViewContainer;
      }
  
      var container;
      var itemViewContainer = Marionette.getOption(containerView, "itemViewContainer");
      if (itemViewContainer){
  
        var selector = _.isFunction(itemViewContainer) ? itemViewContainer.call(containerView) : itemViewContainer;
  
        if (selector.charAt(0) === "@" && containerView.ui) {
          container = containerView.ui[selector.substr(4)];
        } else {
          container = containerView.$(selector);
        }
  
        if (container.length <= 0) {
          throwError("The specified `itemViewContainer` was not found: " + containerView.itemViewContainer, "ItemViewContainerMissingError");
        }
  
      } else {
        container = containerView.$el;
      }
  
      containerView.$itemViewContainer = container;
      return container;
    },
  
    // Internal method to reset the `$itemViewContainer` on render
    resetItemViewContainer: function(){
      if (this.$itemViewContainer){
        delete this.$itemViewContainer;
      }
    }
  });
  
  // Layout
  // ------
  
  // Used for managing application layouts, nested layouts and
  // multiple regions within an application or sub-application.
  //
  // A specialized view type that renders an area of HTML and then
  // attaches `Region` instances to the specified `regions`.
  // Used for composite view management and sub-application areas.
  Marionette.Layout = Marionette.ItemView.extend({
    regionType: Marionette.Region,
  
    // Ensure the regions are available when the `initialize` method
    // is called.
    constructor: function (options) {
      options = options || {};
  
      this._firstRender = true;
      this._initializeRegions(options);
  
      Marionette.ItemView.prototype.constructor.call(this, options);
    },
  
    // Layout's render will use the existing region objects the
    // first time it is called. Subsequent calls will close the
    // views that the regions are showing and then reset the `el`
    // for the regions to the newly rendered DOM elements.
    render: function(){
  
      if (this.isClosed){
        // a previously closed layout means we need to
        // completely re-initialize the regions
        this._initializeRegions();
      }
      if (this._firstRender) {
        // if this is the first render, don't do anything to
        // reset the regions
        this._firstRender = false;
      } else if (!this.isClosed){
        // If this is not the first render call, then we need to
        // re-initializing the `el` for each region
        this._reInitializeRegions();
      }
  
      return Marionette.ItemView.prototype.render.apply(this, arguments);
    },
  
    // Handle closing regions, and then close the view itself.
    close: function () {
      if (this.isClosed){ return; }
      this.regionManager.close();
      Marionette.ItemView.prototype.close.apply(this, arguments);
    },
  
    // Add a single region, by name, to the layout
    addRegion: function(name, definition){
      var regions = {};
      regions[name] = definition;
      return this._buildRegions(regions)[name];
    },
  
    // Add multiple regions as a {name: definition, name2: def2} object literal
    addRegions: function(regions){
      this.regions = _.extend({}, this.regions, regions);
      return this._buildRegions(regions);
    },
  
    // Remove a single region from the Layout, by name
    removeRegion: function(name){
      delete this.regions[name];
      return this.regionManager.removeRegion(name);
    },
  
    // Provides alternative access to regions
    // Accepts the region name
    // getRegion('main')
    getRegion: function(region) {
      return this.regionManager.get(region);
    },
  
    // internal method to build regions
    _buildRegions: function(regions){
      var that = this;
  
      var defaults = {
        regionType: Marionette.getOption(this, "regionType"),
        parentEl: function(){ return that.$el; }
      };
  
      return this.regionManager.addRegions(regions, defaults);
    },
  
    // Internal method to initialize the regions that have been defined in a
    // `regions` attribute on this layout.
    _initializeRegions: function (options) {
      var regions;
      this._initRegionManager();
  
      if (_.isFunction(this.regions)) {
        regions = this.regions(options);
      } else {
        regions = this.regions || {};
      }
  
      this.addRegions(regions);
    },
  
    // Internal method to re-initialize all of the regions by updating the `el` that
    // they point to
    _reInitializeRegions: function(){
      this.regionManager.closeRegions();
      this.regionManager.each(function(region){
        region.reset();
      });
    },
  
    // Internal method to initialize the region manager
    // and all regions in it
    _initRegionManager: function(){
      this.regionManager = new Marionette.RegionManager();
  
      this.listenTo(this.regionManager, "region:add", function(name, region){
        this[name] = region;
        this.trigger("region:add", name, region);
      });
  
      this.listenTo(this.regionManager, "region:remove", function(name, region){
        delete this[name];
        this.trigger("region:remove", name, region);
      });
    }
  });
  
  
  // Behavior
  // -----------
  
  // A Behavior is an isolated set of DOM /
  // user interactions that can be mixed into any View.
  // Behaviors allow you to blackbox View specific interactions
  // into portable logical chunks, keeping your views simple and your code DRY.
  
  Marionette.Behavior = (function(_, Backbone){
    function Behavior(options, view){
      // Setup reference to the view.
      // this comes in handle when a behavior
      // wants to directly talk up the chain
      // to the view.
      this.view = view;
      this.defaults = _.result(this, "defaults") || {};
      this.options  = _.extend({}, this.defaults, options);
  
      // proxy behavior $ method to the view
      // this is useful for doing jquery DOM lookups
      // scoped to behaviors view.
      this.$ = function() {
        return this.view.$.apply(this.view, arguments);
      };
  
      // Call the initialize method passing
      // the arguments from the instance constructor
      this.initialize.apply(this, arguments);
    }
  
    _.extend(Behavior.prototype, Backbone.Events, {
      initialize: function(){},
  
      // stopListening to behavior `onListen` events.
      close: function() {
        this.stopListening();
      },
  
      // Setup class level proxy for triggerMethod.
      triggerMethod: Marionette.triggerMethod
    });
  
    // Borrow Backbones extend implementation
    // this allows us to setup a proper
    // inheritence pattern that follow in suite
    // with the rest of Marionette views.
    Behavior.extend = Marionette.extend;
  
    return Behavior;
  })(_, Backbone);
  
  // Marionette.Behaviors
  // --------
  
  // Behaviors is a utility class that takes care of
  // glueing your behavior instances to their given View.
  // The most important part of this class is that you
  // **MUST** override the class level behaviorsLookup
  // method for things to work properly.
  
  Marionette.Behaviors = (function(Marionette, _) {
  
    function Behaviors(view) {
      // Behaviors defined on a view can be a flat object literal
      // or it can be a function that returns an object.
      this.behaviors = Behaviors.parseBehaviors(view, _.result(view, 'behaviors'));
  
      // Wraps several of the view's methods
      // calling the methods first on each behavior
      // and then eventually calling the method on the view.
      Behaviors.wrap(view, this.behaviors, [
        'bindUIElements', 'unbindUIElements',
        'delegateEvents', 'undelegateEvents',
        'behaviorEvents', 'triggerMethod',
        'setElement', 'close'
      ]);
    }
  
    var methods = {
      setElement: function(setElement, behaviors) {
        setElement.apply(this, _.tail(arguments, 2));
  
        // proxy behavior $el to the view's $el.
        // This is needed because a view's $el proxy
        // is not set until after setElement is called.
        _.each(behaviors, function(b) {
          b.$el = this.$el;
        }, this);
      },
  
      close: function(close, behaviors) {
        var args = _.tail(arguments, 2);
        close.apply(this, args);
  
        // Call close on each behavior after
        // closing down the view.
        // This unbinds event listeners
        // that behaviors have registerd for.
        _.invoke(behaviors, 'close', args);
      },
  
      bindUIElements: function(bindUIElements, behaviors) {
        bindUIElements.apply(this);
        _.invoke(behaviors, bindUIElements);
      },
  
      unbindUIElements: function(unbindUIElements, behaviors) {
        unbindUIElements.apply(this);
        _.invoke(behaviors, unbindUIElements);
      },
  
      triggerMethod: function(triggerMethod, behaviors) {
        var args = _.tail(arguments, 2);
        triggerMethod.apply(this, args);
  
        _.each(behaviors, function(b) {
          triggerMethod.apply(b, args);
        });
      },
  
      delegateEvents: function(delegateEvents, behaviors) {
        var args = _.tail(arguments, 2);
        delegateEvents.apply(this, args);
  
        _.each(behaviors, function(b){
          Marionette.bindEntityEvents(b, this.model, Marionette.getOption(b, "modelEvents"));
          Marionette.bindEntityEvents(b, this.collection, Marionette.getOption(b, "collectionEvents"));
        }, this);
      },
  
      undelegateEvents: function(undelegateEvents, behaviors) {
        var args = _.tail(arguments, 2);
        undelegateEvents.apply(this, args);
  
        _.each(behaviors, function(b) {
          Marionette.unbindEntityEvents(b, this.model, Marionette.getOption(b, "modelEvents"));
          Marionette.unbindEntityEvents(b, this.collection, Marionette.getOption(b, "collectionEvents"));
        }, this);
      },
  
      behaviorEvents: function(behaviorEvents, behaviors) {
        var _behaviorsEvents = {};
        var viewUI = _.result(this, 'ui');
  
        _.each(behaviors, function(b, i) {
          var _events = {};
          var behaviorEvents = _.clone(_.result(b, 'events')) || {};
          var behaviorUI = _.result(b, 'ui');
  
          // Construct an internal UI hash first using
          // the views UI hash and then the behaviors UI hash.
          // This allows the user to use UI hash elements
          // defined in the parent view as well as those
          // defined in the given behavior.
          var ui = _.extend({}, viewUI, behaviorUI);
  
          // Normalize behavior events hash to allow
          // a user to use the @ui. syntax.
          behaviorEvents = Marionette.normalizeUIKeys(behaviorEvents, ui);
  
          _.each(_.keys(behaviorEvents), function(key) {
            // append white-space at the end of each key to prevent behavior key collisions
            // this is relying on the fact backbone events considers "click .foo" the same  "click .foo "
            // starts with an array of two so the first behavior has one space
  
            // +2 is uses becauce new Array(1) or 0 is "" and not " "
            var whitespace = (new Array(i+2)).join(" ");
            var eventKey   = key + whitespace;
            var handler    = _.isFunction(behaviorEvents[key]) ? behaviorEvents[key] : b[behaviorEvents[key]];
  
            _events[eventKey] = _.bind(handler, b);
          });
  
          _behaviorsEvents = _.extend(_behaviorsEvents, _events);
        });
  
        return _behaviorsEvents;
      }
    };
  
    _.extend(Behaviors, {
  
      // placeholder method to be extended by the user
      // should define the object that stores the behaviors
      // i.e.
      //
      // Marionette.Behaviors.behaviorsLookup: function() {
      //   return App.Behaviors
      // }
      behaviorsLookup: function() {
        throw new Error("You must define where your behaviors are stored. See https://github.com/marionettejs/backbone.marionette/blob/master/docs/marionette.behaviors.md#behaviorslookup");
      },
  
      // Takes care of getting the behavior class
      // given options and a key.
      // If a user passes in options.behaviorClass
      // default to using that. Otherwise delegate
      // the lookup to the users behaviorsLookup implementation.
      getBehaviorClass: function(options, key) {
        if (options.behaviorClass) {
          return options.behaviorClass;
        }
  
        // Get behavior class can be either a flat object or a method
        return _.isFunction(Behaviors.behaviorsLookup) ? Behaviors.behaviorsLookup.apply(this, arguments)[key] : Behaviors.behaviorsLookup[key];
      },
  
      // Maps over a view's behaviors. Performing
      // a lookup on each behavior and the instantiating
      // said behavior passing its options and view.
      parseBehaviors: function(view, behaviors){
        return _.map(behaviors, function(options, key){
          var BehaviorClass = Behaviors.getBehaviorClass(options, key);
          return new BehaviorClass(options, view);
        });
      },
  
      // wrap view internal methods so that they delegate to behaviors.
      // For example, onClose should trigger close on all of the behaviors and then close itself.
      // i.e.
      //
      // view.delegateEvents = _.partial(methods.delegateEvents, view.delegateEvents, behaviors);
      wrap: function(view, behaviors, methodNames) {
        _.each(methodNames, function(methodName) {
          view[methodName] = _.partial(methods[methodName], view[methodName], behaviors);
        });
      }
    });
  
    return Behaviors;
  
  })(Marionette, _);
  
  
  // AppRouter
  // ---------
  
  // Reduce the boilerplate code of handling route events
  // and then calling a single method on another object.
  // Have your routers configured to call the method on
  // your object, directly.
  //
  // Configure an AppRouter with `appRoutes`.
  //
  // App routers can only take one `controller` object.
  // It is recommended that you divide your controller
  // objects in to smaller pieces of related functionality
  // and have multiple routers / controllers, instead of
  // just one giant router and controller.
  //
  // You can also add standard routes to an AppRouter.
  
  Marionette.AppRouter = Backbone.Router.extend({
  
    constructor: function(options){
      Backbone.Router.prototype.constructor.apply(this, arguments);
  
      this.options = options || {};
  
      var appRoutes = Marionette.getOption(this, "appRoutes");
      var controller = this._getController();
      this.processAppRoutes(controller, appRoutes);
      this.on("route", this._processOnRoute, this);
    },
  
    // Similar to route method on a Backbone Router but
    // method is called on the controller
    appRoute: function(route, methodName) {
      var controller = this._getController();
      this._addAppRoute(controller, route, methodName);
    },
  
    // process the route event and trigger the onRoute
    // method call, if it exists
    _processOnRoute: function(routeName, routeArgs){
      // find the path that matched
      var routePath = _.invert(this.appRoutes)[routeName];
  
      // make sure an onRoute is there, and call it
      if (_.isFunction(this.onRoute)){
        this.onRoute(routeName, routePath, routeArgs);
      }
    },
  
    // Internal method to process the `appRoutes` for the
    // router, and turn them in to routes that trigger the
    // specified method on the specified `controller`.
    processAppRoutes: function(controller, appRoutes) {
      if (!appRoutes){ return; }
  
      var routeNames = _.keys(appRoutes).reverse(); // Backbone requires reverted order of routes
  
      _.each(routeNames, function(route) {
        this._addAppRoute(controller, route, appRoutes[route]);
      }, this);
    },
  
    _getController: function(){
      return Marionette.getOption(this, "controller");
    },
  
    _addAppRoute: function(controller, route, methodName){
      var method = controller[methodName];
  
      if (!method) {
        throwError("Method '" + methodName + "' was not found on the controller");
      }
  
      this.route(route, methodName, _.bind(method, controller));
    }
  });
  
  // Application
  // -----------
  
  // Contain and manage the composite application as a whole.
  // Stores and starts up `Region` objects, includes an
  // event aggregator as `app.vent`
  Marionette.Application = function(options){
    this._initRegionManager();
    this._initCallbacks = new Marionette.Callbacks();
    this.vent = new Backbone.Wreqr.EventAggregator();
    this.commands = new Backbone.Wreqr.Commands();
    this.reqres = new Backbone.Wreqr.RequestResponse();
    this.submodules = {};
  
    _.extend(this, options);
  
    this.triggerMethod = Marionette.triggerMethod;
  };
  
  _.extend(Marionette.Application.prototype, Backbone.Events, {
    // Command execution, facilitated by Backbone.Wreqr.Commands
    execute: function(){
      this.commands.execute.apply(this.commands, arguments);
    },
  
    // Request/response, facilitated by Backbone.Wreqr.RequestResponse
    request: function(){
      return this.reqres.request.apply(this.reqres, arguments);
    },
  
    // Add an initializer that is either run at when the `start`
    // method is called, or run immediately if added after `start`
    // has already been called.
    addInitializer: function(initializer){
      this._initCallbacks.add(initializer);
    },
  
    // kick off all of the application's processes.
    // initializes all of the regions that have been added
    // to the app, and runs all of the initializer functions
    start: function(options){
      this.triggerMethod("initialize:before", options);
      this._initCallbacks.run(options, this);
      this.triggerMethod("initialize:after", options);
  
      this.triggerMethod("start", options);
    },
  
    // Add regions to your app.
    // Accepts a hash of named strings or Region objects
    // addRegions({something: "#someRegion"})
    // addRegions({something: Region.extend({el: "#someRegion"}) });
    addRegions: function(regions){
      return this._regionManager.addRegions(regions);
    },
  
    // Close all regions in the app, without removing them
    closeRegions: function(){
      this._regionManager.closeRegions();
    },
  
    // Removes a region from your app, by name
    // Accepts the regions name
    // removeRegion('myRegion')
    removeRegion: function(region) {
      this._regionManager.removeRegion(region);
    },
  
    // Provides alternative access to regions
    // Accepts the region name
    // getRegion('main')
    getRegion: function(region) {
      return this._regionManager.get(region);
    },
  
    // Create a module, attached to the application
    module: function(moduleNames, moduleDefinition){
  
      // Overwrite the module class if the user specifies one
      var ModuleClass = Marionette.Module.getClass(moduleDefinition);
  
      // slice the args, and add this application object as the
      // first argument of the array
      var args = slice.call(arguments);
      args.unshift(this);
  
      // see the Marionette.Module object for more information
      return ModuleClass.create.apply(ModuleClass, args);
    },
  
    // Internal method to set up the region manager
    _initRegionManager: function(){
      this._regionManager = new Marionette.RegionManager();
  
      this.listenTo(this._regionManager, "region:add", function(name, region){
        this[name] = region;
      });
  
      this.listenTo(this._regionManager, "region:remove", function(name, region){
        delete this[name];
      });
    }
  });
  
  // Copy the `extend` function used by Backbone's classes
  Marionette.Application.extend = Marionette.extend;
  
  // Module
  // ------
  
  // A simple module system, used to create privacy and encapsulation in
  // Marionette applications
  Marionette.Module = function(moduleName, app, options){
    this.moduleName = moduleName;
    this.options = _.extend({}, this.options, options);
    // Allow for a user to overide the initialize
    // for a given module instance.
    this.initialize = options.initialize || this.initialize;
  
    // Set up an internal store for sub-modules.
    this.submodules = {};
  
    this._setupInitializersAndFinalizers();
  
    // Set an internal reference to the app
    // within a module.
    this.app = app;
  
    // By default modules start with their parents.
    this.startWithParent = true;
  
    // Setup a proxy to the trigger method implementation.
    this.triggerMethod = Marionette.triggerMethod;
  
    if (_.isFunction(this.initialize)){
      this.initialize(this.options, moduleName, app);
    }
  };
  
  Marionette.Module.extend = Marionette.extend;
  
  // Extend the Module prototype with events / listenTo, so that the module
  // can be used as an event aggregator or pub/sub.
  _.extend(Marionette.Module.prototype, Backbone.Events, {
  
    // Initialize is an empty function by default. Override it with your own
    // initialization logic when extending Marionette.Module.
    initialize: function(){},
  
    // Initializer for a specific module. Initializers are run when the
    // module's `start` method is called.
    addInitializer: function(callback){
      this._initializerCallbacks.add(callback);
    },
  
    // Finalizers are run when a module is stopped. They are used to teardown
    // and finalize any variables, references, events and other code that the
    // module had set up.
    addFinalizer: function(callback){
      this._finalizerCallbacks.add(callback);
    },
  
    // Start the module, and run all of its initializers
    start: function(options){
      // Prevent re-starting a module that is already started
      if (this._isInitialized){ return; }
  
      // start the sub-modules (depth-first hierarchy)
      _.each(this.submodules, function(mod){
        // check to see if we should start the sub-module with this parent
        if (mod.startWithParent){
          mod.start(options);
        }
      });
  
      // run the callbacks to "start" the current module
      this.triggerMethod("before:start", options);
  
      this._initializerCallbacks.run(options, this);
      this._isInitialized = true;
  
      this.triggerMethod("start", options);
    },
  
    // Stop this module by running its finalizers and then stop all of
    // the sub-modules for this module
    stop: function(){
      // if we are not initialized, don't bother finalizing
      if (!this._isInitialized){ return; }
      this._isInitialized = false;
  
      Marionette.triggerMethod.call(this, "before:stop");
  
      // stop the sub-modules; depth-first, to make sure the
      // sub-modules are stopped / finalized before parents
      _.each(this.submodules, function(mod){ mod.stop(); });
  
      // run the finalizers
      this._finalizerCallbacks.run(undefined,this);
  
      // reset the initializers and finalizers
      this._initializerCallbacks.reset();
      this._finalizerCallbacks.reset();
  
      Marionette.triggerMethod.call(this, "stop");
    },
  
    // Configure the module with a definition function and any custom args
    // that are to be passed in to the definition function
    addDefinition: function(moduleDefinition, customArgs){
      this._runModuleDefinition(moduleDefinition, customArgs);
    },
  
    // Internal method: run the module definition function with the correct
    // arguments
    _runModuleDefinition: function(definition, customArgs){
      // If there is no definition short circut the method.
      if (!definition){ return; }
  
      // build the correct list of arguments for the module definition
      var args = _.flatten([
        this,
        this.app,
        Backbone,
        Marionette,
        Marionette.$, _,
        customArgs
      ]);
  
      definition.apply(this, args);
    },
  
    // Internal method: set up new copies of initializers and finalizers.
    // Calling this method will wipe out all existing initializers and
    // finalizers.
    _setupInitializersAndFinalizers: function(){
      this._initializerCallbacks = new Marionette.Callbacks();
      this._finalizerCallbacks = new Marionette.Callbacks();
    }
  });
  
  // Type methods to create modules
  _.extend(Marionette.Module, {
  
    // Create a module, hanging off the app parameter as the parent object.
    create: function(app, moduleNames, moduleDefinition){
      var module = app;
  
      // get the custom args passed in after the module definition and
      // get rid of the module name and definition function
      var customArgs = slice.call(arguments);
      customArgs.splice(0, 3);
  
      // Split the module names and get the number of submodules.
      // i.e. an example module name of `Doge.Wow.Amaze` would
      // then have the potential for 3 module definitions.
      moduleNames = moduleNames.split(".");
      var length = moduleNames.length;
  
      // store the module definition for the last module in the chain
      var moduleDefinitions = [];
      moduleDefinitions[length-1] = moduleDefinition;
  
      // Loop through all the parts of the module definition
      _.each(moduleNames, function(moduleName, i){
        var parentModule = module;
        module = this._getModule(parentModule, moduleName, app, moduleDefinition);
        this._addModuleDefinition(parentModule, module, moduleDefinitions[i], customArgs);
      }, this);
  
      // Return the last module in the definition chain
      return module;
    },
  
    _getModule: function(parentModule, moduleName, app, def, args){
      var options = _.extend({}, def);
      var ModuleClass = this.getClass(def);
  
      // Get an existing module of this name if we have one
      var module = parentModule[moduleName];
  
      if (!module){
        // Create a new module if we don't have one
        module = new ModuleClass(moduleName, app, options);
        parentModule[moduleName] = module;
        // store the module on the parent
        parentModule.submodules[moduleName] = module;
      }
  
      return module;
    },
  
    // ## Module Classes
    //
    // Module classes can be used as an alternative to the define pattern.
    // The extend function of a Module is identical to the extend functions
    // on other Backbone and Marionette classes.
    // This allows module lifecyle events like `onStart` and `onStop` to be called directly.
    getClass: function(moduleDefinition) {
      var ModuleClass = Marionette.Module;
  
      if (!moduleDefinition) {
        return ModuleClass;
      }
  
      // If all of the module's functionality is defined inside its class,
      // then the class can be passed in directly. `MyApp.module("Foo", FooModule)`.
      if (moduleDefinition.prototype instanceof ModuleClass) {
        return moduleDefinition;
      }
  
      return moduleDefinition.moduleClass || ModuleClass;
    },
  
    // Add the module definition and add a startWithParent initializer function.
    // This is complicated because module definitions are heavily overloaded
    // and support an anonymous function, module class, or options object
    _addModuleDefinition: function(parentModule, module, def, args){
      var fn = this._getDefine(def);
      var startWithParent = this._getStartWithParent(def, module);
  
      if (fn){
        module.addDefinition(fn, args);
      }
  
      this._addStartWithParent(parentModule, module, startWithParent);
    },
  
    _getStartWithParent: function(def, module) {
      var swp;
  
      if (_.isFunction(def) && (def.prototype instanceof Marionette.Module)) {
        swp = module.constructor.prototype.startWithParent;
        return _.isUndefined(swp) ? true : swp;
      }
  
      if (_.isObject(def)){
        swp = def.startWithParent;
        return _.isUndefined(swp) ? true : swp;
      }
  
      return true;
    },
  
    _getDefine: function(def) {
      if (_.isFunction(def) && !(def.prototype instanceof Marionette.Module)) {
        return def;
      }
  
      if (_.isObject(def)){
        return def.define;
      }
  
      return null;
    },
  
    _addStartWithParent: function(parentModule, module, startWithParent) {
      module.startWithParent = module.startWithParent && startWithParent;
  
      if (!module.startWithParent || !!module.startWithParentIsConfigured){
        return;
      }
  
      module.startWithParentIsConfigured = true;
  
      parentModule.addInitializer(function(options){
        if (module.startWithParent){
          module.start(options);
        }
      });
    }
  });
  
  
    return Marionette;
  })(this, Backbone, _);
  
  return Backbone.Marionette;

}));

//noinspection JSHint
define(
    'configuration',["underscore"],
    function (_) {
      "use strict";

      var data = "./data/";

      window.configuration = window.config || {};

      return _.defaults(window.configuration, {
        configuration: data + "master.json",
        debug: false,
        develop: true,
        editable: false,
        language: data + "idiomas.json",
        library: "https://es.santillanacloud.com/bibliotecas/get_json_biblioteca",
        links: "https://es.santillanacloud.com/index.php/publicaciones/get_json_publicaciones_test",
        root: "./",
        rootData: "./",
        source: data + "source/",
        pageSizes: data + "source/sizes.data",
        thumbnails: data + "PAGINAS/",
        version: "4.5.0",
        sgoReadOnlyObjectsFile:  data + "readOnlyObjects.json",
        apiav:  "http://aulavirtual.santillana.es/index.php/apis/"
      });
    }
);
//noinspection JSHint
define(
    'app',["marionette", "configuration"],
    function (Marionette, configuration) {
      "use strict";

      var App = new Marionette.Application();

      if (configuration.develop) {
        window.App = App;
      }

      return App;
    }
);
//noinspection JSHint
define(
    'regions/replace',[
      "underscore",
      "marionette"
    ],
    function (_, Marionette) {
      "use strict";

      return Marionette.Region.extend({
        onClose: function () {
          this.$el = this.getEl(this.el);
          this.$el.empty().addClass("norender no-render");
        },

        onShow: function () {
          if (!this.currentView || !this.$el || this.$el.size() !== 1 ||
              this.$el.children(this.el.tagName) === 0) {
            return this;
          }

          this.$el.children(this.$el.get(0).tagName).unwrap();
          this.$el = this.getEl(this.el);
          var $el = this.$el;

          /* We replaced this.$el layout by this.$el view and we must avoid
           remove this.$el layout at onClose event view */
          this.currentView.onBeforeClose = function () {
            return false;
          };
        }
      });
    }
);
//noinspection JSHint
define(
    'models/page',["underscore", "backbone"],
    function (_, Backbone) {
      "use strict";

      var STATUS = {
        DRAWING: 1,
        NONE: 0
      };

      return Backbone.Model.extend({
        STATUS: STATUS,

        defaults: {
          // basic attributes
          id: 0,
          number: 0,
          title: 0,
          // index page on chapter
          index: 0,
          code: "default",

          // thumbnails
          thumbnail: null,

          // custom drawings
          custom: [],
          drawing: null,
          // pattern use as default background
          type: "blank",

          // status
          status: STATUS.DRAWING
        },

        initialize: function() {
          this.set("status", this.defaults.status);
        },

        save: function(savePage, name) {
          // we must to avoid save status
          var status = this.get("status");
          this.unset("status");
          
          if (savePage) {
            var id = this.get("id");
            this.set("id", this.get("code") + "-" + this.get("number"));
            if (name) {
                this.set("savename", name);
            }
          }
          Backbone.Model.prototype.save.apply(this, arguments);
          if (savePage) {
            this.set("id", id);     
          }

          this.set("status", status);
        },

        status: function (flag) {
          var status = this.get("status"),
              abs = Math.abs(flag);
          //noinspection JSHint
          status = (flag < 0) ? status & ~abs : status | abs;

          this.set("status", status);
          return status;
        }
      });
    }
);
// noinspection JSHint
define(
    'views/modal',["marionette"],
    function (Marionette) {
      "use strict";

      return Marionette.View.extend({
        className: "modal",
        tagName: "aside",
        triggers: {
          "tap": "quit"
        }
      });
    }
);
/*

Copyright (C) 2011 by Yehuda Katz

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

// lib/handlebars/browser-prefix.js
var Handlebars = {};

(function(Handlebars, undefined) {
;
// lib/handlebars/base.js

Handlebars.VERSION = "1.0.0";
Handlebars.COMPILER_REVISION = 4;

Handlebars.REVISION_CHANGES = {
  1: '<= 1.0.rc.2', // 1.0.rc.2 is actually rev2 but doesn't report it
  2: '== 1.0.0-rc.3',
  3: '== 1.0.0-rc.4',
  4: '>= 1.0.0'
};

Handlebars.helpers  = {};
Handlebars.partials = {};

var toString = Object.prototype.toString,
    functionType = '[object Function]',
    objectType = '[object Object]';

Handlebars.registerHelper = function(name, fn, inverse) {
  if (toString.call(name) === objectType) {
    if (inverse || fn) { throw new Handlebars.Exception('Arg not supported with multiple helpers'); }
    Handlebars.Utils.extend(this.helpers, name);
  } else {
    if (inverse) { fn.not = inverse; }
    this.helpers[name] = fn;
  }
};

Handlebars.registerPartial = function(name, str) {
  if (toString.call(name) === objectType) {
    Handlebars.Utils.extend(this.partials,  name);
  } else {
    this.partials[name] = str;
  }
};

Handlebars.registerHelper('helperMissing', function(arg) {
  if(arguments.length === 2) {
    return undefined;
  } else {
    throw new Error("Missing helper: '" + arg + "'");
  }
});

Handlebars.registerHelper('blockHelperMissing', function(context, options) {
  var inverse = options.inverse || function() {}, fn = options.fn;

  var type = toString.call(context);

  if(type === functionType) { context = context.call(this); }

  if(context === true) {
    return fn(this);
  } else if(context === false || context == null) {
    return inverse(this);
  } else if(type === "[object Array]") {
    if(context.length > 0) {
      return Handlebars.helpers.each(context, options);
    } else {
      return inverse(this);
    }
  } else {
    return fn(context);
  }
});

Handlebars.K = function() {};

Handlebars.createFrame = Object.create || function(object) {
  Handlebars.K.prototype = object;
  var obj = new Handlebars.K();
  Handlebars.K.prototype = null;
  return obj;
};

Handlebars.logger = {
  DEBUG: 0, INFO: 1, WARN: 2, ERROR: 3, level: 3,

  methodMap: {0: 'debug', 1: 'info', 2: 'warn', 3: 'error'},

  // can be overridden in the host environment
  log: function(level, obj) {
    if (Handlebars.logger.level <= level) {
      var method = Handlebars.logger.methodMap[level];
      if (typeof console !== 'undefined' && console[method]) {
        console[method].call(console, obj);
      }
    }
  }
};

Handlebars.log = function(level, obj) { Handlebars.logger.log(level, obj); };

Handlebars.registerHelper('each', function(context, options) {
  var fn = options.fn, inverse = options.inverse;
  var i = 0, ret = "", data;

  var type = toString.call(context);
  if(type === functionType) { context = context.call(this); }

  if (options.data) {
    data = Handlebars.createFrame(options.data);
  }

  if(context && typeof context === 'object') {
    if(context instanceof Array){
      for(var j = context.length; i<j; i++) {
        if (data) { data.index = i; }
        ret = ret + fn(context[i], { data: data });
      }
    } else {
      for(var key in context) {
        if(context.hasOwnProperty(key)) {
          if(data) { data.key = key; }
          ret = ret + fn(context[key], {data: data});
          i++;
        }
      }
    }
  }

  if(i === 0){
    ret = inverse(this);
  }

  return ret;
});

Handlebars.registerHelper('if', function(conditional, options) {
  var type = toString.call(conditional);
  if(type === functionType) { conditional = conditional.call(this); }

  if(!conditional || Handlebars.Utils.isEmpty(conditional)) {
    return options.inverse(this);
  } else {
    return options.fn(this);
  }
});

Handlebars.registerHelper('unless', function(conditional, options) {
  return Handlebars.helpers['if'].call(this, conditional, {fn: options.inverse, inverse: options.fn});
});

Handlebars.registerHelper('with', function(context, options) {
  var type = toString.call(context);
  if(type === functionType) { context = context.call(this); }

  if (!Handlebars.Utils.isEmpty(context)) return options.fn(context);
});

Handlebars.registerHelper('log', function(context, options) {
  var level = options.data && options.data.level != null ? parseInt(options.data.level, 10) : 1;
  Handlebars.log(level, context);
});
;
// lib/handlebars/compiler/parser.js
/* Jison generated parser */
var handlebars = (function(){
var parser = {trace: function trace() { },
yy: {},
symbols_: {"error":2,"root":3,"program":4,"EOF":5,"simpleInverse":6,"statements":7,"statement":8,"openInverse":9,"closeBlock":10,"openBlock":11,"mustache":12,"partial":13,"CONTENT":14,"COMMENT":15,"OPEN_BLOCK":16,"inMustache":17,"CLOSE":18,"OPEN_INVERSE":19,"OPEN_ENDBLOCK":20,"path":21,"OPEN":22,"OPEN_UNESCAPED":23,"CLOSE_UNESCAPED":24,"OPEN_PARTIAL":25,"partialName":26,"params":27,"hash":28,"dataName":29,"param":30,"STRING":31,"INTEGER":32,"BOOLEAN":33,"hashSegments":34,"hashSegment":35,"ID":36,"EQUALS":37,"DATA":38,"pathSegments":39,"SEP":40,"$accept":0,"$end":1},
terminals_: {2:"error",5:"EOF",14:"CONTENT",15:"COMMENT",16:"OPEN_BLOCK",18:"CLOSE",19:"OPEN_INVERSE",20:"OPEN_ENDBLOCK",22:"OPEN",23:"OPEN_UNESCAPED",24:"CLOSE_UNESCAPED",25:"OPEN_PARTIAL",31:"STRING",32:"INTEGER",33:"BOOLEAN",36:"ID",37:"EQUALS",38:"DATA",40:"SEP"},
productions_: [0,[3,2],[4,2],[4,3],[4,2],[4,1],[4,1],[4,0],[7,1],[7,2],[8,3],[8,3],[8,1],[8,1],[8,1],[8,1],[11,3],[9,3],[10,3],[12,3],[12,3],[13,3],[13,4],[6,2],[17,3],[17,2],[17,2],[17,1],[17,1],[27,2],[27,1],[30,1],[30,1],[30,1],[30,1],[30,1],[28,1],[34,2],[34,1],[35,3],[35,3],[35,3],[35,3],[35,3],[26,1],[26,1],[26,1],[29,2],[21,1],[39,3],[39,1]],
performAction: function anonymous(yytext,yyleng,yylineno,yy,yystate,$$,_$) {

var $0 = $$.length - 1;
switch (yystate) {
case 1: return $$[$0-1]; 
break;
case 2: this.$ = new yy.ProgramNode([], $$[$0]); 
break;
case 3: this.$ = new yy.ProgramNode($$[$0-2], $$[$0]); 
break;
case 4: this.$ = new yy.ProgramNode($$[$0-1], []); 
break;
case 5: this.$ = new yy.ProgramNode($$[$0]); 
break;
case 6: this.$ = new yy.ProgramNode([], []); 
break;
case 7: this.$ = new yy.ProgramNode([]); 
break;
case 8: this.$ = [$$[$0]]; 
break;
case 9: $$[$0-1].push($$[$0]); this.$ = $$[$0-1]; 
break;
case 10: this.$ = new yy.BlockNode($$[$0-2], $$[$0-1].inverse, $$[$0-1], $$[$0]); 
break;
case 11: this.$ = new yy.BlockNode($$[$0-2], $$[$0-1], $$[$0-1].inverse, $$[$0]); 
break;
case 12: this.$ = $$[$0]; 
break;
case 13: this.$ = $$[$0]; 
break;
case 14: this.$ = new yy.ContentNode($$[$0]); 
break;
case 15: this.$ = new yy.CommentNode($$[$0]); 
break;
case 16: this.$ = new yy.MustacheNode($$[$0-1][0], $$[$0-1][1]); 
break;
case 17: this.$ = new yy.MustacheNode($$[$0-1][0], $$[$0-1][1]); 
break;
case 18: this.$ = $$[$0-1]; 
break;
case 19:
    // Parsing out the '&' escape token at this level saves ~500 bytes after min due to the removal of one parser node.
    this.$ = new yy.MustacheNode($$[$0-1][0], $$[$0-1][1], $$[$0-2][2] === '&');
  
break;
case 20: this.$ = new yy.MustacheNode($$[$0-1][0], $$[$0-1][1], true); 
break;
case 21: this.$ = new yy.PartialNode($$[$0-1]); 
break;
case 22: this.$ = new yy.PartialNode($$[$0-2], $$[$0-1]); 
break;
case 23: 
break;
case 24: this.$ = [[$$[$0-2]].concat($$[$0-1]), $$[$0]]; 
break;
case 25: this.$ = [[$$[$0-1]].concat($$[$0]), null]; 
break;
case 26: this.$ = [[$$[$0-1]], $$[$0]]; 
break;
case 27: this.$ = [[$$[$0]], null]; 
break;
case 28: this.$ = [[$$[$0]], null]; 
break;
case 29: $$[$0-1].push($$[$0]); this.$ = $$[$0-1]; 
break;
case 30: this.$ = [$$[$0]]; 
break;
case 31: this.$ = $$[$0]; 
break;
case 32: this.$ = new yy.StringNode($$[$0]); 
break;
case 33: this.$ = new yy.IntegerNode($$[$0]); 
break;
case 34: this.$ = new yy.BooleanNode($$[$0]); 
break;
case 35: this.$ = $$[$0]; 
break;
case 36: this.$ = new yy.HashNode($$[$0]); 
break;
case 37: $$[$0-1].push($$[$0]); this.$ = $$[$0-1]; 
break;
case 38: this.$ = [$$[$0]]; 
break;
case 39: this.$ = [$$[$0-2], $$[$0]]; 
break;
case 40: this.$ = [$$[$0-2], new yy.StringNode($$[$0])]; 
break;
case 41: this.$ = [$$[$0-2], new yy.IntegerNode($$[$0])]; 
break;
case 42: this.$ = [$$[$0-2], new yy.BooleanNode($$[$0])]; 
break;
case 43: this.$ = [$$[$0-2], $$[$0]]; 
break;
case 44: this.$ = new yy.PartialNameNode($$[$0]); 
break;
case 45: this.$ = new yy.PartialNameNode(new yy.StringNode($$[$0])); 
break;
case 46: this.$ = new yy.PartialNameNode(new yy.IntegerNode($$[$0])); 
break;
case 47: this.$ = new yy.DataNode($$[$0]); 
break;
case 48: this.$ = new yy.IdNode($$[$0]); 
break;
case 49: $$[$0-2].push({part: $$[$0], separator: $$[$0-1]}); this.$ = $$[$0-2]; 
break;
case 50: this.$ = [{part: $$[$0]}]; 
break;
}
},
table: [{3:1,4:2,5:[2,7],6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],22:[1,14],23:[1,15],25:[1,16]},{1:[3]},{5:[1,17]},{5:[2,6],7:18,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,6],22:[1,14],23:[1,15],25:[1,16]},{5:[2,5],6:20,8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,5],22:[1,14],23:[1,15],25:[1,16]},{17:23,18:[1,22],21:24,29:25,36:[1,28],38:[1,27],39:26},{5:[2,8],14:[2,8],15:[2,8],16:[2,8],19:[2,8],20:[2,8],22:[2,8],23:[2,8],25:[2,8]},{4:29,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],25:[1,16]},{4:30,6:3,7:4,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,5],20:[2,7],22:[1,14],23:[1,15],25:[1,16]},{5:[2,12],14:[2,12],15:[2,12],16:[2,12],19:[2,12],20:[2,12],22:[2,12],23:[2,12],25:[2,12]},{5:[2,13],14:[2,13],15:[2,13],16:[2,13],19:[2,13],20:[2,13],22:[2,13],23:[2,13],25:[2,13]},{5:[2,14],14:[2,14],15:[2,14],16:[2,14],19:[2,14],20:[2,14],22:[2,14],23:[2,14],25:[2,14]},{5:[2,15],14:[2,15],15:[2,15],16:[2,15],19:[2,15],20:[2,15],22:[2,15],23:[2,15],25:[2,15]},{17:31,21:24,29:25,36:[1,28],38:[1,27],39:26},{17:32,21:24,29:25,36:[1,28],38:[1,27],39:26},{17:33,21:24,29:25,36:[1,28],38:[1,27],39:26},{21:35,26:34,31:[1,36],32:[1,37],36:[1,28],39:26},{1:[2,1]},{5:[2,2],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,2],22:[1,14],23:[1,15],25:[1,16]},{17:23,21:24,29:25,36:[1,28],38:[1,27],39:26},{5:[2,4],7:38,8:6,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,4],22:[1,14],23:[1,15],25:[1,16]},{5:[2,9],14:[2,9],15:[2,9],16:[2,9],19:[2,9],20:[2,9],22:[2,9],23:[2,9],25:[2,9]},{5:[2,23],14:[2,23],15:[2,23],16:[2,23],19:[2,23],20:[2,23],22:[2,23],23:[2,23],25:[2,23]},{18:[1,39]},{18:[2,27],21:44,24:[2,27],27:40,28:41,29:48,30:42,31:[1,45],32:[1,46],33:[1,47],34:43,35:49,36:[1,50],38:[1,27],39:26},{18:[2,28],24:[2,28]},{18:[2,48],24:[2,48],31:[2,48],32:[2,48],33:[2,48],36:[2,48],38:[2,48],40:[1,51]},{21:52,36:[1,28],39:26},{18:[2,50],24:[2,50],31:[2,50],32:[2,50],33:[2,50],36:[2,50],38:[2,50],40:[2,50]},{10:53,20:[1,54]},{10:55,20:[1,54]},{18:[1,56]},{18:[1,57]},{24:[1,58]},{18:[1,59],21:60,36:[1,28],39:26},{18:[2,44],36:[2,44]},{18:[2,45],36:[2,45]},{18:[2,46],36:[2,46]},{5:[2,3],8:21,9:7,11:8,12:9,13:10,14:[1,11],15:[1,12],16:[1,13],19:[1,19],20:[2,3],22:[1,14],23:[1,15],25:[1,16]},{14:[2,17],15:[2,17],16:[2,17],19:[2,17],20:[2,17],22:[2,17],23:[2,17],25:[2,17]},{18:[2,25],21:44,24:[2,25],28:61,29:48,30:62,31:[1,45],32:[1,46],33:[1,47],34:43,35:49,36:[1,50],38:[1,27],39:26},{18:[2,26],24:[2,26]},{18:[2,30],24:[2,30],31:[2,30],32:[2,30],33:[2,30],36:[2,30],38:[2,30]},{18:[2,36],24:[2,36],35:63,36:[1,64]},{18:[2,31],24:[2,31],31:[2,31],32:[2,31],33:[2,31],36:[2,31],38:[2,31]},{18:[2,32],24:[2,32],31:[2,32],32:[2,32],33:[2,32],36:[2,32],38:[2,32]},{18:[2,33],24:[2,33],31:[2,33],32:[2,33],33:[2,33],36:[2,33],38:[2,33]},{18:[2,34],24:[2,34],31:[2,34],32:[2,34],33:[2,34],36:[2,34],38:[2,34]},{18:[2,35],24:[2,35],31:[2,35],32:[2,35],33:[2,35],36:[2,35],38:[2,35]},{18:[2,38],24:[2,38],36:[2,38]},{18:[2,50],24:[2,50],31:[2,50],32:[2,50],33:[2,50],36:[2,50],37:[1,65],38:[2,50],40:[2,50]},{36:[1,66]},{18:[2,47],24:[2,47],31:[2,47],32:[2,47],33:[2,47],36:[2,47],38:[2,47]},{5:[2,10],14:[2,10],15:[2,10],16:[2,10],19:[2,10],20:[2,10],22:[2,10],23:[2,10],25:[2,10]},{21:67,36:[1,28],39:26},{5:[2,11],14:[2,11],15:[2,11],16:[2,11],19:[2,11],20:[2,11],22:[2,11],23:[2,11],25:[2,11]},{14:[2,16],15:[2,16],16:[2,16],19:[2,16],20:[2,16],22:[2,16],23:[2,16],25:[2,16]},{5:[2,19],14:[2,19],15:[2,19],16:[2,19],19:[2,19],20:[2,19],22:[2,19],23:[2,19],25:[2,19]},{5:[2,20],14:[2,20],15:[2,20],16:[2,20],19:[2,20],20:[2,20],22:[2,20],23:[2,20],25:[2,20]},{5:[2,21],14:[2,21],15:[2,21],16:[2,21],19:[2,21],20:[2,21],22:[2,21],23:[2,21],25:[2,21]},{18:[1,68]},{18:[2,24],24:[2,24]},{18:[2,29],24:[2,29],31:[2,29],32:[2,29],33:[2,29],36:[2,29],38:[2,29]},{18:[2,37],24:[2,37],36:[2,37]},{37:[1,65]},{21:69,29:73,31:[1,70],32:[1,71],33:[1,72],36:[1,28],38:[1,27],39:26},{18:[2,49],24:[2,49],31:[2,49],32:[2,49],33:[2,49],36:[2,49],38:[2,49],40:[2,49]},{18:[1,74]},{5:[2,22],14:[2,22],15:[2,22],16:[2,22],19:[2,22],20:[2,22],22:[2,22],23:[2,22],25:[2,22]},{18:[2,39],24:[2,39],36:[2,39]},{18:[2,40],24:[2,40],36:[2,40]},{18:[2,41],24:[2,41],36:[2,41]},{18:[2,42],24:[2,42],36:[2,42]},{18:[2,43],24:[2,43],36:[2,43]},{5:[2,18],14:[2,18],15:[2,18],16:[2,18],19:[2,18],20:[2,18],22:[2,18],23:[2,18],25:[2,18]}],
defaultActions: {17:[2,1]},
parseError: function parseError(str, hash) {
    throw new Error(str);
},
parse: function parse(input) {
    var self = this, stack = [0], vstack = [null], lstack = [], table = this.table, yytext = "", yylineno = 0, yyleng = 0, recovering = 0, TERROR = 2, EOF = 1;
    this.lexer.setInput(input);
    this.lexer.yy = this.yy;
    this.yy.lexer = this.lexer;
    this.yy.parser = this;
    if (typeof this.lexer.yylloc == "undefined")
        this.lexer.yylloc = {};
    var yyloc = this.lexer.yylloc;
    lstack.push(yyloc);
    var ranges = this.lexer.options && this.lexer.options.ranges;
    if (typeof this.yy.parseError === "function")
        this.parseError = this.yy.parseError;
    function popStack(n) {
        stack.length = stack.length - 2 * n;
        vstack.length = vstack.length - n;
        lstack.length = lstack.length - n;
    }
    function lex() {
        var token;
        token = self.lexer.lex() || 1;
        if (typeof token !== "number") {
            token = self.symbols_[token] || token;
        }
        return token;
    }
    var symbol, preErrorSymbol, state, action, a, r, yyval = {}, p, len, newState, expected;
    while (true) {
        state = stack[stack.length - 1];
        if (this.defaultActions[state]) {
            action = this.defaultActions[state];
        } else {
            if (symbol === null || typeof symbol == "undefined") {
                symbol = lex();
            }
            action = table[state] && table[state][symbol];
        }
        if (typeof action === "undefined" || !action.length || !action[0]) {
            var errStr = "";
            if (!recovering) {
                expected = [];
                for (p in table[state])
                    if (this.terminals_[p] && p > 2) {
                        expected.push("'" + this.terminals_[p] + "'");
                    }
                if (this.lexer.showPosition) {
                    errStr = "Parse error on line " + (yylineno + 1) + ":\n" + this.lexer.showPosition() + "\nExpecting " + expected.join(", ") + ", got '" + (this.terminals_[symbol] || symbol) + "'";
                } else {
                    errStr = "Parse error on line " + (yylineno + 1) + ": Unexpected " + (symbol == 1?"end of input":"'" + (this.terminals_[symbol] || symbol) + "'");
                }
                this.parseError(errStr, {text: this.lexer.match, token: this.terminals_[symbol] || symbol, line: this.lexer.yylineno, loc: yyloc, expected: expected});
            }
        }
        if (action[0] instanceof Array && action.length > 1) {
            throw new Error("Parse Error: multiple actions possible at state: " + state + ", token: " + symbol);
        }
        switch (action[0]) {
        case 1:
            stack.push(symbol);
            vstack.push(this.lexer.yytext);
            lstack.push(this.lexer.yylloc);
            stack.push(action[1]);
            symbol = null;
            if (!preErrorSymbol) {
                yyleng = this.lexer.yyleng;
                yytext = this.lexer.yytext;
                yylineno = this.lexer.yylineno;
                yyloc = this.lexer.yylloc;
                if (recovering > 0)
                    recovering--;
            } else {
                symbol = preErrorSymbol;
                preErrorSymbol = null;
            }
            break;
        case 2:
            len = this.productions_[action[1]][1];
            yyval.$ = vstack[vstack.length - len];
            yyval._$ = {first_line: lstack[lstack.length - (len || 1)].first_line, last_line: lstack[lstack.length - 1].last_line, first_column: lstack[lstack.length - (len || 1)].first_column, last_column: lstack[lstack.length - 1].last_column};
            if (ranges) {
                yyval._$.range = [lstack[lstack.length - (len || 1)].range[0], lstack[lstack.length - 1].range[1]];
            }
            r = this.performAction.call(yyval, yytext, yyleng, yylineno, this.yy, action[1], vstack, lstack);
            if (typeof r !== "undefined") {
                return r;
            }
            if (len) {
                stack = stack.slice(0, -1 * len * 2);
                vstack = vstack.slice(0, -1 * len);
                lstack = lstack.slice(0, -1 * len);
            }
            stack.push(this.productions_[action[1]][0]);
            vstack.push(yyval.$);
            lstack.push(yyval._$);
            newState = table[stack[stack.length - 2]][stack[stack.length - 1]];
            stack.push(newState);
            break;
        case 3:
            return true;
        }
    }
    return true;
}
};
/* Jison generated lexer */
var lexer = (function(){
var lexer = ({EOF:1,
parseError:function parseError(str, hash) {
        if (this.yy.parser) {
            this.yy.parser.parseError(str, hash);
        } else {
            throw new Error(str);
        }
    },
setInput:function (input) {
        this._input = input;
        this._more = this._less = this.done = false;
        this.yylineno = this.yyleng = 0;
        this.yytext = this.matched = this.match = '';
        this.conditionStack = ['INITIAL'];
        this.yylloc = {first_line:1,first_column:0,last_line:1,last_column:0};
        if (this.options.ranges) this.yylloc.range = [0,0];
        this.offset = 0;
        return this;
    },
input:function () {
        var ch = this._input[0];
        this.yytext += ch;
        this.yyleng++;
        this.offset++;
        this.match += ch;
        this.matched += ch;
        var lines = ch.match(/(?:\r\n?|\n).*/g);
        if (lines) {
            this.yylineno++;
            this.yylloc.last_line++;
        } else {
            this.yylloc.last_column++;
        }
        if (this.options.ranges) this.yylloc.range[1]++;

        this._input = this._input.slice(1);
        return ch;
    },
unput:function (ch) {
        var len = ch.length;
        var lines = ch.split(/(?:\r\n?|\n)/g);

        this._input = ch + this._input;
        this.yytext = this.yytext.substr(0, this.yytext.length-len-1);
        //this.yyleng -= len;
        this.offset -= len;
        var oldLines = this.match.split(/(?:\r\n?|\n)/g);
        this.match = this.match.substr(0, this.match.length-1);
        this.matched = this.matched.substr(0, this.matched.length-1);

        if (lines.length-1) this.yylineno -= lines.length-1;
        var r = this.yylloc.range;

        this.yylloc = {first_line: this.yylloc.first_line,
          last_line: this.yylineno+1,
          first_column: this.yylloc.first_column,
          last_column: lines ?
              (lines.length === oldLines.length ? this.yylloc.first_column : 0) + oldLines[oldLines.length - lines.length].length - lines[0].length:
              this.yylloc.first_column - len
          };

        if (this.options.ranges) {
            this.yylloc.range = [r[0], r[0] + this.yyleng - len];
        }
        return this;
    },
more:function () {
        this._more = true;
        return this;
    },
less:function (n) {
        this.unput(this.match.slice(n));
    },
pastInput:function () {
        var past = this.matched.substr(0, this.matched.length - this.match.length);
        return (past.length > 20 ? '...':'') + past.substr(-20).replace(/\n/g, "");
    },
upcomingInput:function () {
        var next = this.match;
        if (next.length < 20) {
            next += this._input.substr(0, 20-next.length);
        }
        return (next.substr(0,20)+(next.length > 20 ? '...':'')).replace(/\n/g, "");
    },
showPosition:function () {
        var pre = this.pastInput();
        var c = new Array(pre.length + 1).join("-");
        return pre + this.upcomingInput() + "\n" + c+"^";
    },
next:function () {
        if (this.done) {
            return this.EOF;
        }
        if (!this._input) this.done = true;

        var token,
            match,
            tempMatch,
            index,
            col,
            lines;
        if (!this._more) {
            this.yytext = '';
            this.match = '';
        }
        var rules = this._currentRules();
        for (var i=0;i < rules.length; i++) {
            tempMatch = this._input.match(this.rules[rules[i]]);
            if (tempMatch && (!match || tempMatch[0].length > match[0].length)) {
                match = tempMatch;
                index = i;
                if (!this.options.flex) break;
            }
        }
        if (match) {
            lines = match[0].match(/(?:\r\n?|\n).*/g);
            if (lines) this.yylineno += lines.length;
            this.yylloc = {first_line: this.yylloc.last_line,
                           last_line: this.yylineno+1,
                           first_column: this.yylloc.last_column,
                           last_column: lines ? lines[lines.length-1].length-lines[lines.length-1].match(/\r?\n?/)[0].length : this.yylloc.last_column + match[0].length};
            this.yytext += match[0];
            this.match += match[0];
            this.matches = match;
            this.yyleng = this.yytext.length;
            if (this.options.ranges) {
                this.yylloc.range = [this.offset, this.offset += this.yyleng];
            }
            this._more = false;
            this._input = this._input.slice(match[0].length);
            this.matched += match[0];
            token = this.performAction.call(this, this.yy, this, rules[index],this.conditionStack[this.conditionStack.length-1]);
            if (this.done && this._input) this.done = false;
            if (token) return token;
            else return;
        }
        if (this._input === "") {
            return this.EOF;
        } else {
            return this.parseError('Lexical error on line '+(this.yylineno+1)+'. Unrecognized text.\n'+this.showPosition(),
                    {text: "", token: null, line: this.yylineno});
        }
    },
lex:function lex() {
        var r = this.next();
        if (typeof r !== 'undefined') {
            return r;
        } else {
            return this.lex();
        }
    },
begin:function begin(condition) {
        this.conditionStack.push(condition);
    },
popState:function popState() {
        return this.conditionStack.pop();
    },
_currentRules:function _currentRules() {
        return this.conditions[this.conditionStack[this.conditionStack.length-1]].rules;
    },
topState:function () {
        return this.conditionStack[this.conditionStack.length-2];
    },
pushState:function begin(condition) {
        this.begin(condition);
    }});
lexer.options = {};
lexer.performAction = function anonymous(yy,yy_,$avoiding_name_collisions,YY_START) {

var YYSTATE=YY_START
switch($avoiding_name_collisions) {
case 0: yy_.yytext = "\\"; return 14; 
break;
case 1:
                                   if(yy_.yytext.slice(-1) !== "\\") this.begin("mu");
                                   if(yy_.yytext.slice(-1) === "\\") yy_.yytext = yy_.yytext.substr(0,yy_.yyleng-1), this.begin("emu");
                                   if(yy_.yytext) return 14;
                                 
break;
case 2: return 14; 
break;
case 3:
                                   if(yy_.yytext.slice(-1) !== "\\") this.popState();
                                   if(yy_.yytext.slice(-1) === "\\") yy_.yytext = yy_.yytext.substr(0,yy_.yyleng-1);
                                   return 14;
                                 
break;
case 4: yy_.yytext = yy_.yytext.substr(0, yy_.yyleng-4); this.popState(); return 15; 
break;
case 5: return 25; 
break;
case 6: return 16; 
break;
case 7: return 20; 
break;
case 8: return 19; 
break;
case 9: return 19; 
break;
case 10: return 23; 
break;
case 11: return 22; 
break;
case 12: this.popState(); this.begin('com'); 
break;
case 13: yy_.yytext = yy_.yytext.substr(3,yy_.yyleng-5); this.popState(); return 15; 
break;
case 14: return 22; 
break;
case 15: return 37; 
break;
case 16: return 36; 
break;
case 17: return 36; 
break;
case 18: return 40; 
break;
case 19: /*ignore whitespace*/ 
break;
case 20: this.popState(); return 24; 
break;
case 21: this.popState(); return 18; 
break;
case 22: yy_.yytext = yy_.yytext.substr(1,yy_.yyleng-2).replace(/\\"/g,'"'); return 31; 
break;
case 23: yy_.yytext = yy_.yytext.substr(1,yy_.yyleng-2).replace(/\\'/g,"'"); return 31; 
break;
case 24: return 38; 
break;
case 25: return 33; 
break;
case 26: return 33; 
break;
case 27: return 32; 
break;
case 28: return 36; 
break;
case 29: yy_.yytext = yy_.yytext.substr(1, yy_.yyleng-2); return 36; 
break;
case 30: return 'INVALID'; 
break;
case 31: return 5; 
break;
}
};
lexer.rules = [/^(?:\\\\(?=(\{\{)))/,/^(?:[^\x00]*?(?=(\{\{)))/,/^(?:[^\x00]+)/,/^(?:[^\x00]{2,}?(?=(\{\{|$)))/,/^(?:[\s\S]*?--\}\})/,/^(?:\{\{>)/,/^(?:\{\{#)/,/^(?:\{\{\/)/,/^(?:\{\{\^)/,/^(?:\{\{\s*else\b)/,/^(?:\{\{\{)/,/^(?:\{\{&)/,/^(?:\{\{!--)/,/^(?:\{\{![\s\S]*?\}\})/,/^(?:\{\{)/,/^(?:=)/,/^(?:\.(?=[}\/ ]))/,/^(?:\.\.)/,/^(?:[\/.])/,/^(?:\s+)/,/^(?:\}\}\})/,/^(?:\}\})/,/^(?:"(\\["]|[^"])*")/,/^(?:'(\\[']|[^'])*')/,/^(?:@)/,/^(?:true(?=[}\s]))/,/^(?:false(?=[}\s]))/,/^(?:-?[0-9]+(?=[}\s]))/,/^(?:[^\s!"#%-,\.\/;->@\[-\^`\{-~]+(?=[=}\s\/.]))/,/^(?:\[[^\]]*\])/,/^(?:.)/,/^(?:$)/];
lexer.conditions = {"mu":{"rules":[5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31],"inclusive":false},"emu":{"rules":[3],"inclusive":false},"com":{"rules":[4],"inclusive":false},"INITIAL":{"rules":[0,1,2,31],"inclusive":true}};
return lexer;})()
parser.lexer = lexer;
function Parser () { this.yy = {}; }Parser.prototype = parser;parser.Parser = Parser;
return new Parser;
})();;
// lib/handlebars/compiler/base.js

Handlebars.Parser = handlebars;

Handlebars.parse = function(input) {

  // Just return if an already-compile AST was passed in.
  if(input.constructor === Handlebars.AST.ProgramNode) { return input; }

  Handlebars.Parser.yy = Handlebars.AST;
  return Handlebars.Parser.parse(input);
};
;
// lib/handlebars/compiler/ast.js
Handlebars.AST = {};

Handlebars.AST.ProgramNode = function(statements, inverse) {
  this.type = "program";
  this.statements = statements;
  if(inverse) { this.inverse = new Handlebars.AST.ProgramNode(inverse); }
};

Handlebars.AST.MustacheNode = function(rawParams, hash, unescaped) {
  this.type = "mustache";
  this.escaped = !unescaped;
  this.hash = hash;

  var id = this.id = rawParams[0];
  var params = this.params = rawParams.slice(1);

  // a mustache is an eligible helper if:
  // * its id is simple (a single part, not `this` or `..`)
  var eligibleHelper = this.eligibleHelper = id.isSimple;

  // a mustache is definitely a helper if:
  // * it is an eligible helper, and
  // * it has at least one parameter or hash segment
  this.isHelper = eligibleHelper && (params.length || hash);

  // if a mustache is an eligible helper but not a definite
  // helper, it is ambiguous, and will be resolved in a later
  // pass or at runtime.
};

Handlebars.AST.PartialNode = function(partialName, context) {
  this.type         = "partial";
  this.partialName  = partialName;
  this.context      = context;
};

Handlebars.AST.BlockNode = function(mustache, program, inverse, close) {
  var verifyMatch = function(open, close) {
    if(open.original !== close.original) {
      throw new Handlebars.Exception(open.original + " doesn't match " + close.original);
    }
  };

  verifyMatch(mustache.id, close);
  this.type = "block";
  this.mustache = mustache;
  this.program  = program;
  this.inverse  = inverse;

  if (this.inverse && !this.program) {
    this.isInverse = true;
  }
};

Handlebars.AST.ContentNode = function(string) {
  this.type = "content";
  this.string = string;
};

Handlebars.AST.HashNode = function(pairs) {
  this.type = "hash";
  this.pairs = pairs;
};

Handlebars.AST.IdNode = function(parts) {
  this.type = "ID";

  var original = "",
      dig = [],
      depth = 0;

  for(var i=0,l=parts.length; i<l; i++) {
    var part = parts[i].part;
    original += (parts[i].separator || '') + part;

    if (part === ".." || part === "." || part === "this") {
      if (dig.length > 0) { throw new Handlebars.Exception("Invalid path: " + original); }
      else if (part === "..") { depth++; }
      else { this.isScoped = true; }
    }
    else { dig.push(part); }
  }

  this.original = original;
  this.parts    = dig;
  this.string   = dig.join('.');
  this.depth    = depth;

  // an ID is simple if it only has one part, and that part is not
  // `..` or `this`.
  this.isSimple = parts.length === 1 && !this.isScoped && depth === 0;

  this.stringModeValue = this.string;
};

Handlebars.AST.PartialNameNode = function(name) {
  this.type = "PARTIAL_NAME";
  this.name = name.original;
};

Handlebars.AST.DataNode = function(id) {
  this.type = "DATA";
  this.id = id;
};

Handlebars.AST.StringNode = function(string) {
  this.type = "STRING";
  this.original =
    this.string =
    this.stringModeValue = string;
};

Handlebars.AST.IntegerNode = function(integer) {
  this.type = "INTEGER";
  this.original =
    this.integer = integer;
  this.stringModeValue = Number(integer);
};

Handlebars.AST.BooleanNode = function(bool) {
  this.type = "BOOLEAN";
  this.bool = bool;
  this.stringModeValue = bool === "true";
};

Handlebars.AST.CommentNode = function(comment) {
  this.type = "comment";
  this.comment = comment;
};
;
// lib/handlebars/utils.js

var errorProps = ['description', 'fileName', 'lineNumber', 'message', 'name', 'number', 'stack'];

Handlebars.Exception = function(message) {
  var tmp = Error.prototype.constructor.apply(this, arguments);

  // Unfortunately errors are not enumerable in Chrome (at least), so `for prop in tmp` doesn't work.
  for (var idx = 0; idx < errorProps.length; idx++) {
    this[errorProps[idx]] = tmp[errorProps[idx]];
  }
};
Handlebars.Exception.prototype = new Error();

// Build out our basic SafeString type
Handlebars.SafeString = function(string) {
  this.string = string;
};
Handlebars.SafeString.prototype.toString = function() {
  return this.string.toString();
};

var escape = {
  "&": "&amp;",
  "<": "&lt;",
  ">": "&gt;",
  '"': "&quot;",
  "'": "&#x27;",
  "`": "&#x60;"
};

var badChars = /[&<>"'`]/g;
var possible = /[&<>"'`]/;

var escapeChar = function(chr) {
  return escape[chr] || "&amp;";
};

Handlebars.Utils = {
  extend: function(obj, value) {
    for(var key in value) {
      if(value.hasOwnProperty(key)) {
        obj[key] = value[key];
      }
    }
  },

  escapeExpression: function(string) {
    // don't escape SafeStrings, since they're already safe
    if (string instanceof Handlebars.SafeString) {
      return string.toString();
    } else if (string == null || string === false) {
      return "";
    }

    // Force a string conversion as this will be done by the append regardless and
    // the regex test will do this transparently behind the scenes, causing issues if
    // an object's to string has escaped characters in it.
    string = string.toString();

    if(!possible.test(string)) { return string; }
    return string.replace(badChars, escapeChar);
  },

  isEmpty: function(value) {
    if (!value && value !== 0) {
      return true;
    } else if(toString.call(value) === "[object Array]" && value.length === 0) {
      return true;
    } else {
      return false;
    }
  }
};
;
// lib/handlebars/compiler/compiler.js

/*jshint eqnull:true*/
var Compiler = Handlebars.Compiler = function() {};
var JavaScriptCompiler = Handlebars.JavaScriptCompiler = function() {};

// the foundHelper register will disambiguate helper lookup from finding a
// function in a context. This is necessary for mustache compatibility, which
// requires that context functions in blocks are evaluated by blockHelperMissing,
// and then proceed as if the resulting value was provided to blockHelperMissing.

Compiler.prototype = {
  compiler: Compiler,

  disassemble: function() {
    var opcodes = this.opcodes, opcode, out = [], params, param;

    for (var i=0, l=opcodes.length; i<l; i++) {
      opcode = opcodes[i];

      if (opcode.opcode === 'DECLARE') {
        out.push("DECLARE " + opcode.name + "=" + opcode.value);
      } else {
        params = [];
        for (var j=0; j<opcode.args.length; j++) {
          param = opcode.args[j];
          if (typeof param === "string") {
            param = "\"" + param.replace("\n", "\\n") + "\"";
          }
          params.push(param);
        }
        out.push(opcode.opcode + " " + params.join(" "));
      }
    }

    return out.join("\n");
  },
  equals: function(other) {
    var len = this.opcodes.length;
    if (other.opcodes.length !== len) {
      return false;
    }

    for (var i = 0; i < len; i++) {
      var opcode = this.opcodes[i],
          otherOpcode = other.opcodes[i];
      if (opcode.opcode !== otherOpcode.opcode || opcode.args.length !== otherOpcode.args.length) {
        return false;
      }
      for (var j = 0; j < opcode.args.length; j++) {
        if (opcode.args[j] !== otherOpcode.args[j]) {
          return false;
        }
      }
    }

    len = this.children.length;
    if (other.children.length !== len) {
      return false;
    }
    for (i = 0; i < len; i++) {
      if (!this.children[i].equals(other.children[i])) {
        return false;
      }
    }

    return true;
  },

  guid: 0,

  compile: function(program, options) {
    this.children = [];
    this.depths = {list: []};
    this.options = options;

    // These changes will propagate to the other compiler components
    var knownHelpers = this.options.knownHelpers;
    this.options.knownHelpers = {
      'helperMissing': true,
      'blockHelperMissing': true,
      'each': true,
      'if': true,
      'unless': true,
      'with': true,
      'log': true
    };
    if (knownHelpers) {
      for (var name in knownHelpers) {
        this.options.knownHelpers[name] = knownHelpers[name];
      }
    }

    return this.program(program);
  },

  accept: function(node) {
    return this[node.type](node);
  },

  program: function(program) {
    var statements = program.statements, statement;
    this.opcodes = [];

    for(var i=0, l=statements.length; i<l; i++) {
      statement = statements[i];
      this[statement.type](statement);
    }
    this.isSimple = l === 1;

    this.depths.list = this.depths.list.sort(function(a, b) {
      return a - b;
    });

    return this;
  },

  compileProgram: function(program) {
    var result = new this.compiler().compile(program, this.options);
    var guid = this.guid++, depth;

    this.usePartial = this.usePartial || result.usePartial;

    this.children[guid] = result;

    for(var i=0, l=result.depths.list.length; i<l; i++) {
      depth = result.depths.list[i];

      if(depth < 2) { continue; }
      else { this.addDepth(depth - 1); }
    }

    return guid;
  },

  block: function(block) {
    var mustache = block.mustache,
        program = block.program,
        inverse = block.inverse;

    if (program) {
      program = this.compileProgram(program);
    }

    if (inverse) {
      inverse = this.compileProgram(inverse);
    }

    var type = this.classifyMustache(mustache);

    if (type === "helper") {
      this.helperMustache(mustache, program, inverse);
    } else if (type === "simple") {
      this.simpleMustache(mustache);

      // now that the simple mustache is resolved, we need to
      // evaluate it by executing `blockHelperMissing`
      this.opcode('pushProgram', program);
      this.opcode('pushProgram', inverse);
      this.opcode('emptyHash');
      this.opcode('blockValue');
    } else {
      this.ambiguousMustache(mustache, program, inverse);

      // now that the simple mustache is resolved, we need to
      // evaluate it by executing `blockHelperMissing`
      this.opcode('pushProgram', program);
      this.opcode('pushProgram', inverse);
      this.opcode('emptyHash');
      this.opcode('ambiguousBlockValue');
    }

    this.opcode('append');
  },

  hash: function(hash) {
    var pairs = hash.pairs, pair, val;

    this.opcode('pushHash');

    for(var i=0, l=pairs.length; i<l; i++) {
      pair = pairs[i];
      val  = pair[1];

      if (this.options.stringParams) {
        if(val.depth) {
          this.addDepth(val.depth);
        }
        this.opcode('getContext', val.depth || 0);
        this.opcode('pushStringParam', val.stringModeValue, val.type);
      } else {
        this.accept(val);
      }

      this.opcode('assignToHash', pair[0]);
    }
    this.opcode('popHash');
  },

  partial: function(partial) {
    var partialName = partial.partialName;
    this.usePartial = true;

    if(partial.context) {
      this.ID(partial.context);
    } else {
      this.opcode('push', 'depth0');
    }

    this.opcode('invokePartial', partialName.name);
    this.opcode('append');
  },

  content: function(content) {
    this.opcode('appendContent', content.string);
  },

  mustache: function(mustache) {
    var options = this.options;
    var type = this.classifyMustache(mustache);

    if (type === "simple") {
      this.simpleMustache(mustache);
    } else if (type === "helper") {
      this.helperMustache(mustache);
    } else {
      this.ambiguousMustache(mustache);
    }

    if(mustache.escaped && !options.noEscape) {
      this.opcode('appendEscaped');
    } else {
      this.opcode('append');
    }
  },

  ambiguousMustache: function(mustache, program, inverse) {
    var id = mustache.id,
        name = id.parts[0],
        isBlock = program != null || inverse != null;

    this.opcode('getContext', id.depth);

    this.opcode('pushProgram', program);
    this.opcode('pushProgram', inverse);

    this.opcode('invokeAmbiguous', name, isBlock);
  },

  simpleMustache: function(mustache) {
    var id = mustache.id;

    if (id.type === 'DATA') {
      this.DATA(id);
    } else if (id.parts.length) {
      this.ID(id);
    } else {
      // Simplified ID for `this`
      this.addDepth(id.depth);
      this.opcode('getContext', id.depth);
      this.opcode('pushContext');
    }

    this.opcode('resolvePossibleLambda');
  },

  helperMustache: function(mustache, program, inverse) {
    var params = this.setupFullMustacheParams(mustache, program, inverse),
        name = mustache.id.parts[0];

    if (this.options.knownHelpers[name]) {
      this.opcode('invokeKnownHelper', params.length, name);
    } else if (this.options.knownHelpersOnly) {
      throw new Error("You specified knownHelpersOnly, but used the unknown helper " + name);
    } else {
      this.opcode('invokeHelper', params.length, name);
    }
  },

  ID: function(id) {
    this.addDepth(id.depth);
    this.opcode('getContext', id.depth);

    var name = id.parts[0];
    if (!name) {
      this.opcode('pushContext');
    } else {
      this.opcode('lookupOnContext', id.parts[0]);
    }

    for(var i=1, l=id.parts.length; i<l; i++) {
      this.opcode('lookup', id.parts[i]);
    }
  },

  DATA: function(data) {
    this.options.data = true;
    if (data.id.isScoped || data.id.depth) {
      throw new Handlebars.Exception('Scoped data references are not supported: ' + data.original);
    }

    this.opcode('lookupData');
    var parts = data.id.parts;
    for(var i=0, l=parts.length; i<l; i++) {
      this.opcode('lookup', parts[i]);
    }
  },

  STRING: function(string) {
    this.opcode('pushString', string.string);
  },

  INTEGER: function(integer) {
    this.opcode('pushLiteral', integer.integer);
  },

  BOOLEAN: function(bool) {
    this.opcode('pushLiteral', bool.bool);
  },

  comment: function() {},

  // HELPERS
  opcode: function(name) {
    this.opcodes.push({ opcode: name, args: [].slice.call(arguments, 1) });
  },

  declare: function(name, value) {
    this.opcodes.push({ opcode: 'DECLARE', name: name, value: value });
  },

  addDepth: function(depth) {
    if(isNaN(depth)) { throw new Error("EWOT"); }
    if(depth === 0) { return; }

    if(!this.depths[depth]) {
      this.depths[depth] = true;
      this.depths.list.push(depth);
    }
  },

  classifyMustache: function(mustache) {
    var isHelper   = mustache.isHelper;
    var isEligible = mustache.eligibleHelper;
    var options    = this.options;

    // if ambiguous, we can possibly resolve the ambiguity now
    if (isEligible && !isHelper) {
      var name = mustache.id.parts[0];

      if (options.knownHelpers[name]) {
        isHelper = true;
      } else if (options.knownHelpersOnly) {
        isEligible = false;
      }
    }

    if (isHelper) { return "helper"; }
    else if (isEligible) { return "ambiguous"; }
    else { return "simple"; }
  },

  pushParams: function(params) {
    var i = params.length, param;

    while(i--) {
      param = params[i];

      if(this.options.stringParams) {
        if(param.depth) {
          this.addDepth(param.depth);
        }

        this.opcode('getContext', param.depth || 0);
        this.opcode('pushStringParam', param.stringModeValue, param.type);
      } else {
        this[param.type](param);
      }
    }
  },

  setupMustacheParams: function(mustache) {
    var params = mustache.params;
    this.pushParams(params);

    if(mustache.hash) {
      this.hash(mustache.hash);
    } else {
      this.opcode('emptyHash');
    }

    return params;
  },

  // this will replace setupMustacheParams when we're done
  setupFullMustacheParams: function(mustache, program, inverse) {
    var params = mustache.params;
    this.pushParams(params);

    this.opcode('pushProgram', program);
    this.opcode('pushProgram', inverse);

    if(mustache.hash) {
      this.hash(mustache.hash);
    } else {
      this.opcode('emptyHash');
    }

    return params;
  }
};

var Literal = function(value) {
  this.value = value;
};

JavaScriptCompiler.prototype = {
  // PUBLIC API: You can override these methods in a subclass to provide
  // alternative compiled forms for name lookup and buffering semantics
  nameLookup: function(parent, name /* , type*/) {
    if (/^[0-9]+$/.test(name)) {
      return parent + "[" + name + "]";
    } else if (JavaScriptCompiler.isValidJavaScriptVariableName(name)) {
      return parent + "." + name;
    }
    else {
      return parent + "['" + name + "']";
    }
  },

  appendToBuffer: function(string) {
    if (this.environment.isSimple) {
      return "return " + string + ";";
    } else {
      return {
        appendToBuffer: true,
        content: string,
        toString: function() { return "buffer += " + string + ";"; }
      };
    }
  },

  initializeBuffer: function() {
    return this.quotedString("");
  },

  namespace: "Handlebars",
  // END PUBLIC API

  compile: function(environment, options, context, asObject) {
    this.environment = environment;
    this.options = options || {};

    Handlebars.log(Handlebars.logger.DEBUG, this.environment.disassemble() + "\n\n");

    this.name = this.environment.name;
    this.isChild = !!context;
    this.context = context || {
      programs: [],
      environments: [],
      aliases: { }
    };

    this.preamble();

    this.stackSlot = 0;
    this.stackVars = [];
    this.registers = { list: [] };
    this.compileStack = [];
    this.inlineStack = [];

    this.compileChildren(environment, options);

    var opcodes = environment.opcodes, opcode;

    this.i = 0;

    for(l=opcodes.length; this.i<l; this.i++) {
      opcode = opcodes[this.i];

      if(opcode.opcode === 'DECLARE') {
        this[opcode.name] = opcode.value;
      } else {
        this[opcode.opcode].apply(this, opcode.args);
      }
    }

    return this.createFunctionContext(asObject);
  },

  nextOpcode: function() {
    var opcodes = this.environment.opcodes;
    return opcodes[this.i + 1];
  },

  eat: function() {
    this.i = this.i + 1;
  },

  preamble: function() {
    var out = [];

    if (!this.isChild) {
      var namespace = this.namespace;

      var copies = "helpers = this.merge(helpers, " + namespace + ".helpers);";
      if (this.environment.usePartial) { copies = copies + " partials = this.merge(partials, " + namespace + ".partials);"; }
      if (this.options.data) { copies = copies + " data = data || {};"; }
      out.push(copies);
    } else {
      out.push('');
    }

    if (!this.environment.isSimple) {
      out.push(", buffer = " + this.initializeBuffer());
    } else {
      out.push("");
    }

    // track the last context pushed into place to allow skipping the
    // getContext opcode when it would be a noop
    this.lastContext = 0;
    this.source = out;
  },

  createFunctionContext: function(asObject) {
    var locals = this.stackVars.concat(this.registers.list);

    if(locals.length > 0) {
      this.source[1] = this.source[1] + ", " + locals.join(", ");
    }

    // Generate minimizer alias mappings
    if (!this.isChild) {
      for (var alias in this.context.aliases) {
        if (this.context.aliases.hasOwnProperty(alias)) {
          this.source[1] = this.source[1] + ', ' + alias + '=' + this.context.aliases[alias];
        }
      }
    }

    if (this.source[1]) {
      this.source[1] = "var " + this.source[1].substring(2) + ";";
    }

    // Merge children
    if (!this.isChild) {
      this.source[1] += '\n' + this.context.programs.join('\n') + '\n';
    }

    if (!this.environment.isSimple) {
      this.source.push("return buffer;");
    }

    var params = this.isChild ? ["depth0", "data"] : ["Handlebars", "depth0", "helpers", "partials", "data"];

    for(var i=0, l=this.environment.depths.list.length; i<l; i++) {
      params.push("depth" + this.environment.depths.list[i]);
    }

    // Perform a second pass over the output to merge content when possible
    var source = this.mergeSource();

    if (!this.isChild) {
      var revision = Handlebars.COMPILER_REVISION,
          versions = Handlebars.REVISION_CHANGES[revision];
      source = "this.compilerInfo = ["+revision+",'"+versions+"'];\n"+source;
    }

    if (asObject) {
      params.push(source);

      return Function.apply(this, params);
    } else {
      var functionSource = 'function ' + (this.name || '') + '(' + params.join(',') + ') {\n  ' + source + '}';
      Handlebars.log(Handlebars.logger.DEBUG, functionSource + "\n\n");
      return functionSource;
    }
  },
  mergeSource: function() {
    // WARN: We are not handling the case where buffer is still populated as the source should
    // not have buffer append operations as their final action.
    var source = '',
        buffer;
    for (var i = 0, len = this.source.length; i < len; i++) {
      var line = this.source[i];
      if (line.appendToBuffer) {
        if (buffer) {
          buffer = buffer + '\n    + ' + line.content;
        } else {
          buffer = line.content;
        }
      } else {
        if (buffer) {
          source += 'buffer += ' + buffer + ';\n  ';
          buffer = undefined;
        }
        source += line + '\n  ';
      }
    }
    return source;
  },

  // [blockValue]
  //
  // On stack, before: hash, inverse, program, value
  // On stack, after: return value of blockHelperMissing
  //
  // The purpose of this opcode is to take a block of the form
  // `{{#foo}}...{{/foo}}`, resolve the value of `foo`, and
  // replace it on the stack with the result of properly
  // invoking blockHelperMissing.
  blockValue: function() {
    this.context.aliases.blockHelperMissing = 'helpers.blockHelperMissing';

    var params = ["depth0"];
    this.setupParams(0, params);

    this.replaceStack(function(current) {
      params.splice(1, 0, current);
      return "blockHelperMissing.call(" + params.join(", ") + ")";
    });
  },

  // [ambiguousBlockValue]
  //
  // On stack, before: hash, inverse, program, value
  // Compiler value, before: lastHelper=value of last found helper, if any
  // On stack, after, if no lastHelper: same as [blockValue]
  // On stack, after, if lastHelper: value
  ambiguousBlockValue: function() {
    this.context.aliases.blockHelperMissing = 'helpers.blockHelperMissing';

    var params = ["depth0"];
    this.setupParams(0, params);

    var current = this.topStack();
    params.splice(1, 0, current);

    // Use the options value generated from the invocation
    params[params.length-1] = 'options';

    this.source.push("if (!" + this.lastHelper + ") { " + current + " = blockHelperMissing.call(" + params.join(", ") + "); }");
  },

  // [appendContent]
  //
  // On stack, before: ...
  // On stack, after: ...
  //
  // Appends the string value of `content` to the current buffer
  appendContent: function(content) {
    this.source.push(this.appendToBuffer(this.quotedString(content)));
  },

  // [append]
  //
  // On stack, before: value, ...
  // On stack, after: ...
  //
  // Coerces `value` to a String and appends it to the current buffer.
  //
  // If `value` is truthy, or 0, it is coerced into a string and appended
  // Otherwise, the empty string is appended
  append: function() {
    // Force anything that is inlined onto the stack so we don't have duplication
    // when we examine local
    this.flushInline();
    var local = this.popStack();
    this.source.push("if(" + local + " || " + local + " === 0) { " + this.appendToBuffer(local) + " }");
    if (this.environment.isSimple) {
      this.source.push("else { " + this.appendToBuffer("''") + " }");
    }
  },

  // [appendEscaped]
  //
  // On stack, before: value, ...
  // On stack, after: ...
  //
  // Escape `value` and append it to the buffer
  appendEscaped: function() {
    this.context.aliases.escapeExpression = 'this.escapeExpression';

    this.source.push(this.appendToBuffer("escapeExpression(" + this.popStack() + ")"));
  },

  // [getContext]
  //
  // On stack, before: ...
  // On stack, after: ...
  // Compiler value, after: lastContext=depth
  //
  // Set the value of the `lastContext` compiler value to the depth
  getContext: function(depth) {
    if(this.lastContext !== depth) {
      this.lastContext = depth;
    }
  },

  // [lookupOnContext]
  //
  // On stack, before: ...
  // On stack, after: currentContext[name], ...
  //
  // Looks up the value of `name` on the current context and pushes
  // it onto the stack.
  lookupOnContext: function(name) {
    this.push(this.nameLookup('depth' + this.lastContext, name, 'context'));
  },

  // [pushContext]
  //
  // On stack, before: ...
  // On stack, after: currentContext, ...
  //
  // Pushes the value of the current context onto the stack.
  pushContext: function() {
    this.pushStackLiteral('depth' + this.lastContext);
  },

  // [resolvePossibleLambda]
  //
  // On stack, before: value, ...
  // On stack, after: resolved value, ...
  //
  // If the `value` is a lambda, replace it on the stack by
  // the return value of the lambda
  resolvePossibleLambda: function() {
    this.context.aliases.functionType = '"function"';

    this.replaceStack(function(current) {
      return "typeof " + current + " === functionType ? " + current + ".apply(depth0) : " + current;
    });
  },

  // [lookup]
  //
  // On stack, before: value, ...
  // On stack, after: value[name], ...
  //
  // Replace the value on the stack with the result of looking
  // up `name` on `value`
  lookup: function(name) {
    this.replaceStack(function(current) {
      return current + " == null || " + current + " === false ? " + current + " : " + this.nameLookup(current, name, 'context');
    });
  },

  // [lookupData]
  //
  // On stack, before: ...
  // On stack, after: data[id], ...
  //
  // Push the result of looking up `id` on the current data
  lookupData: function(id) {
    this.push('data');
  },

  // [pushStringParam]
  //
  // On stack, before: ...
  // On stack, after: string, currentContext, ...
  //
  // This opcode is designed for use in string mode, which
  // provides the string value of a parameter along with its
  // depth rather than resolving it immediately.
  pushStringParam: function(string, type) {
    this.pushStackLiteral('depth' + this.lastContext);

    this.pushString(type);

    if (typeof string === 'string') {
      this.pushString(string);
    } else {
      this.pushStackLiteral(string);
    }
  },

  emptyHash: function() {
    this.pushStackLiteral('{}');

    if (this.options.stringParams) {
      this.register('hashTypes', '{}');
      this.register('hashContexts', '{}');
    }
  },
  pushHash: function() {
    this.hash = {values: [], types: [], contexts: []};
  },
  popHash: function() {
    var hash = this.hash;
    this.hash = undefined;

    if (this.options.stringParams) {
      this.register('hashContexts', '{' + hash.contexts.join(',') + '}');
      this.register('hashTypes', '{' + hash.types.join(',') + '}');
    }
    this.push('{\n    ' + hash.values.join(',\n    ') + '\n  }');
  },

  // [pushString]
  //
  // On stack, before: ...
  // On stack, after: quotedString(string), ...
  //
  // Push a quoted version of `string` onto the stack
  pushString: function(string) {
    this.pushStackLiteral(this.quotedString(string));
  },

  // [push]
  //
  // On stack, before: ...
  // On stack, after: expr, ...
  //
  // Push an expression onto the stack
  push: function(expr) {
    this.inlineStack.push(expr);
    return expr;
  },

  // [pushLiteral]
  //
  // On stack, before: ...
  // On stack, after: value, ...
  //
  // Pushes a value onto the stack. This operation prevents
  // the compiler from creating a temporary variable to hold
  // it.
  pushLiteral: function(value) {
    this.pushStackLiteral(value);
  },

  // [pushProgram]
  //
  // On stack, before: ...
  // On stack, after: program(guid), ...
  //
  // Push a program expression onto the stack. This takes
  // a compile-time guid and converts it into a runtime-accessible
  // expression.
  pushProgram: function(guid) {
    if (guid != null) {
      this.pushStackLiteral(this.programExpression(guid));
    } else {
      this.pushStackLiteral(null);
    }
  },

  // [invokeHelper]
  //
  // On stack, before: hash, inverse, program, params..., ...
  // On stack, after: result of helper invocation
  //
  // Pops off the helper's parameters, invokes the helper,
  // and pushes the helper's return value onto the stack.
  //
  // If the helper is not found, `helperMissing` is called.
  invokeHelper: function(paramSize, name) {
    this.context.aliases.helperMissing = 'helpers.helperMissing';

    var helper = this.lastHelper = this.setupHelper(paramSize, name, true);
    var nonHelper = this.nameLookup('depth' + this.lastContext, name, 'context');

    this.push(helper.name + ' || ' + nonHelper);
    this.replaceStack(function(name) {
      return name + ' ? ' + name + '.call(' +
          helper.callParams + ") " + ": helperMissing.call(" +
          helper.helperMissingParams + ")";
    });
  },

  // [invokeKnownHelper]
  //
  // On stack, before: hash, inverse, program, params..., ...
  // On stack, after: result of helper invocation
  //
  // This operation is used when the helper is known to exist,
  // so a `helperMissing` fallback is not required.
  invokeKnownHelper: function(paramSize, name) {
    var helper = this.setupHelper(paramSize, name);
    this.push(helper.name + ".call(" + helper.callParams + ")");
  },

  // [invokeAmbiguous]
  //
  // On stack, before: hash, inverse, program, params..., ...
  // On stack, after: result of disambiguation
  //
  // This operation is used when an expression like `{{foo}}`
  // is provided, but we don't know at compile-time whether it
  // is a helper or a path.
  //
  // This operation emits more code than the other options,
  // and can be avoided by passing the `knownHelpers` and
  // `knownHelpersOnly` flags at compile-time.
  invokeAmbiguous: function(name, helperCall) {
    this.context.aliases.functionType = '"function"';

    this.pushStackLiteral('{}');    // Hash value
    var helper = this.setupHelper(0, name, helperCall);

    var helperName = this.lastHelper = this.nameLookup('helpers', name, 'helper');

    var nonHelper = this.nameLookup('depth' + this.lastContext, name, 'context');
    var nextStack = this.nextStack();

    this.source.push('if (' + nextStack + ' = ' + helperName + ') { ' + nextStack + ' = ' + nextStack + '.call(' + helper.callParams + '); }');
    this.source.push('else { ' + nextStack + ' = ' + nonHelper + '; ' + nextStack + ' = typeof ' + nextStack + ' === functionType ? ' + nextStack + '.apply(depth0) : ' + nextStack + '; }');
  },

  // [invokePartial]
  //
  // On stack, before: context, ...
  // On stack after: result of partial invocation
  //
  // This operation pops off a context, invokes a partial with that context,
  // and pushes the result of the invocation back.
  invokePartial: function(name) {
    var params = [this.nameLookup('partials', name, 'partial'), "'" + name + "'", this.popStack(), "helpers", "partials"];

    if (this.options.data) {
      params.push("data");
    }

    this.context.aliases.self = "this";
    this.push("self.invokePartial(" + params.join(", ") + ")");
  },

  // [assignToHash]
  //
  // On stack, before: value, hash, ...
  // On stack, after: hash, ...
  //
  // Pops a value and hash off the stack, assigns `hash[key] = value`
  // and pushes the hash back onto the stack.
  assignToHash: function(key) {
    var value = this.popStack(),
        context,
        type;

    if (this.options.stringParams) {
      type = this.popStack();
      context = this.popStack();
    }

    var hash = this.hash;
    if (context) {
      hash.contexts.push("'" + key + "': " + context);
    }
    if (type) {
      hash.types.push("'" + key + "': " + type);
    }
    hash.values.push("'" + key + "': (" + value + ")");
  },

  // HELPERS

  compiler: JavaScriptCompiler,

  compileChildren: function(environment, options) {
    var children = environment.children, child, compiler;

    for(var i=0, l=children.length; i<l; i++) {
      child = children[i];
      compiler = new this.compiler();

      var index = this.matchExistingProgram(child);

      if (index == null) {
        this.context.programs.push('');     // Placeholder to prevent name conflicts for nested children
        index = this.context.programs.length;
        child.index = index;
        child.name = 'program' + index;
        this.context.programs[index] = compiler.compile(child, options, this.context);
        this.context.environments[index] = child;
      } else {
        child.index = index;
        child.name = 'program' + index;
      }
    }
  },
  matchExistingProgram: function(child) {
    for (var i = 0, len = this.context.environments.length; i < len; i++) {
      var environment = this.context.environments[i];
      if (environment && environment.equals(child)) {
        return i;
      }
    }
  },

  programExpression: function(guid) {
    this.context.aliases.self = "this";

    if(guid == null) {
      return "self.noop";
    }

    var child = this.environment.children[guid],
        depths = child.depths.list, depth;

    var programParams = [child.index, child.name, "data"];

    for(var i=0, l = depths.length; i<l; i++) {
      depth = depths[i];

      if(depth === 1) { programParams.push("depth0"); }
      else { programParams.push("depth" + (depth - 1)); }
    }

    return (depths.length === 0 ? "self.program(" : "self.programWithDepth(") + programParams.join(", ") + ")";
  },

  register: function(name, val) {
    this.useRegister(name);
    this.source.push(name + " = " + val + ";");
  },

  useRegister: function(name) {
    if(!this.registers[name]) {
      this.registers[name] = true;
      this.registers.list.push(name);
    }
  },

  pushStackLiteral: function(item) {
    return this.push(new Literal(item));
  },

  pushStack: function(item) {
    this.flushInline();

    var stack = this.incrStack();
    if (item) {
      this.source.push(stack + " = " + item + ";");
    }
    this.compileStack.push(stack);
    return stack;
  },

  replaceStack: function(callback) {
    var prefix = '',
        inline = this.isInline(),
        stack;

    // If we are currently inline then we want to merge the inline statement into the
    // replacement statement via ','
    if (inline) {
      var top = this.popStack(true);

      if (top instanceof Literal) {
        // Literals do not need to be inlined
        stack = top.value;
      } else {
        // Get or create the current stack name for use by the inline
        var name = this.stackSlot ? this.topStackName() : this.incrStack();

        prefix = '(' + this.push(name) + ' = ' + top + '),';
        stack = this.topStack();
      }
    } else {
      stack = this.topStack();
    }

    var item = callback.call(this, stack);

    if (inline) {
      if (this.inlineStack.length || this.compileStack.length) {
        this.popStack();
      }
      this.push('(' + prefix + item + ')');
    } else {
      // Prevent modification of the context depth variable. Through replaceStack
      if (!/^stack/.test(stack)) {
        stack = this.nextStack();
      }

      this.source.push(stack + " = (" + prefix + item + ");");
    }
    return stack;
  },

  nextStack: function() {
    return this.pushStack();
  },

  incrStack: function() {
    this.stackSlot++;
    if(this.stackSlot > this.stackVars.length) { this.stackVars.push("stack" + this.stackSlot); }
    return this.topStackName();
  },
  topStackName: function() {
    return "stack" + this.stackSlot;
  },
  flushInline: function() {
    var inlineStack = this.inlineStack;
    if (inlineStack.length) {
      this.inlineStack = [];
      for (var i = 0, len = inlineStack.length; i < len; i++) {
        var entry = inlineStack[i];
        if (entry instanceof Literal) {
          this.compileStack.push(entry);
        } else {
          this.pushStack(entry);
        }
      }
    }
  },
  isInline: function() {
    return this.inlineStack.length;
  },

  popStack: function(wrapped) {
    var inline = this.isInline(),
        item = (inline ? this.inlineStack : this.compileStack).pop();

    if (!wrapped && (item instanceof Literal)) {
      return item.value;
    } else {
      if (!inline) {
        this.stackSlot--;
      }
      return item;
    }
  },

  topStack: function(wrapped) {
    var stack = (this.isInline() ? this.inlineStack : this.compileStack),
        item = stack[stack.length - 1];

    if (!wrapped && (item instanceof Literal)) {
      return item.value;
    } else {
      return item;
    }
  },

  quotedString: function(str) {
    return '"' + str
      .replace(/\\/g, '\\\\')
      .replace(/"/g, '\\"')
      .replace(/\n/g, '\\n')
      .replace(/\r/g, '\\r')
      .replace(/\u2028/g, '\\u2028')   // Per Ecma-262 7.3 + 7.8.4
      .replace(/\u2029/g, '\\u2029') + '"';
  },

  setupHelper: function(paramSize, name, missingParams) {
    var params = [];
    this.setupParams(paramSize, params, missingParams);
    var foundHelper = this.nameLookup('helpers', name, 'helper');

    return {
      params: params,
      name: foundHelper,
      callParams: ["depth0"].concat(params).join(", "),
      helperMissingParams: missingParams && ["depth0", this.quotedString(name)].concat(params).join(", ")
    };
  },

  // the params and contexts arguments are passed in arrays
  // to fill in
  setupParams: function(paramSize, params, useRegister) {
    var options = [], contexts = [], types = [], param, inverse, program;

    options.push("hash:" + this.popStack());

    inverse = this.popStack();
    program = this.popStack();

    // Avoid setting fn and inverse if neither are set. This allows
    // helpers to do a check for `if (options.fn)`
    if (program || inverse) {
      if (!program) {
        this.context.aliases.self = "this";
        program = "self.noop";
      }

      if (!inverse) {
       this.context.aliases.self = "this";
        inverse = "self.noop";
      }

      options.push("inverse:" + inverse);
      options.push("fn:" + program);
    }

    for(var i=0; i<paramSize; i++) {
      param = this.popStack();
      params.push(param);

      if(this.options.stringParams) {
        types.push(this.popStack());
        contexts.push(this.popStack());
      }
    }

    if (this.options.stringParams) {
      options.push("contexts:[" + contexts.join(",") + "]");
      options.push("types:[" + types.join(",") + "]");
      options.push("hashContexts:hashContexts");
      options.push("hashTypes:hashTypes");
    }

    if(this.options.data) {
      options.push("data:data");
    }

    options = "{" + options.join(",") + "}";
    if (useRegister) {
      this.register('options', options);
      params.push('options');
    } else {
      params.push(options);
    }
    return params.join(", ");
  }
};

var reservedWords = (
  "break else new var" +
  " case finally return void" +
  " catch for switch while" +
  " continue function this with" +
  " default if throw" +
  " delete in try" +
  " do instanceof typeof" +
  " abstract enum int short" +
  " boolean export interface static" +
  " byte extends long super" +
  " char final native synchronized" +
  " class float package throws" +
  " const goto private transient" +
  " debugger implements protected volatile" +
  " double import public let yield"
).split(" ");

var compilerWords = JavaScriptCompiler.RESERVED_WORDS = {};

for(var i=0, l=reservedWords.length; i<l; i++) {
  compilerWords[reservedWords[i]] = true;
}

JavaScriptCompiler.isValidJavaScriptVariableName = function(name) {
  if(!JavaScriptCompiler.RESERVED_WORDS[name] && /^[a-zA-Z_$][0-9a-zA-Z_$]+$/.test(name)) {
    return true;
  }
  return false;
};

Handlebars.precompile = function(input, options) {
  if (input == null || (typeof input !== 'string' && input.constructor !== Handlebars.AST.ProgramNode)) {
    throw new Handlebars.Exception("You must pass a string or Handlebars AST to Handlebars.precompile. You passed " + input);
  }

  options = options || {};
  if (!('data' in options)) {
    options.data = true;
  }
  var ast = Handlebars.parse(input);
  var environment = new Compiler().compile(ast, options);
  return new JavaScriptCompiler().compile(environment, options);
};

Handlebars.compile = function(input, options) {
  if (input == null || (typeof input !== 'string' && input.constructor !== Handlebars.AST.ProgramNode)) {
    throw new Handlebars.Exception("You must pass a string or Handlebars AST to Handlebars.compile. You passed " + input);
  }

  options = options || {};
  if (!('data' in options)) {
    options.data = true;
  }
  var compiled;
  function compile() {
    var ast = Handlebars.parse(input);
    var environment = new Compiler().compile(ast, options);
    var templateSpec = new JavaScriptCompiler().compile(environment, options, undefined, true);
    return Handlebars.template(templateSpec);
  }

  // Template is only compiled on first use and cached after that point.
  return function(context, options) {
    if (!compiled) {
      compiled = compile();
    }
    return compiled.call(this, context, options);
  };
};

;
// lib/handlebars/runtime.js

Handlebars.VM = {
  template: function(templateSpec) {
    // Just add water
    var container = {
      escapeExpression: Handlebars.Utils.escapeExpression,
      invokePartial: Handlebars.VM.invokePartial,
      programs: [],
      program: function(i, fn, data) {
        var programWrapper = this.programs[i];
        if(data) {
          programWrapper = Handlebars.VM.program(i, fn, data);
        } else if (!programWrapper) {
          programWrapper = this.programs[i] = Handlebars.VM.program(i, fn);
        }
        return programWrapper;
      },
      merge: function(param, common) {
        var ret = param || common;

        if (param && common) {
          ret = {};
          Handlebars.Utils.extend(ret, common);
          Handlebars.Utils.extend(ret, param);
        }
        return ret;
      },
      programWithDepth: Handlebars.VM.programWithDepth,
      noop: Handlebars.VM.noop,
      compilerInfo: null
    };

    return function(context, options) {
      options = options || {};
      var result = templateSpec.call(container, Handlebars, context, options.helpers, options.partials, options.data);

      var compilerInfo = container.compilerInfo || [],
          compilerRevision = compilerInfo[0] || 1,
          currentRevision = Handlebars.COMPILER_REVISION;

      if (compilerRevision !== currentRevision) {
        if (compilerRevision < currentRevision) {
          var runtimeVersions = Handlebars.REVISION_CHANGES[currentRevision],
              compilerVersions = Handlebars.REVISION_CHANGES[compilerRevision];
          throw "Template was precompiled with an older version of Handlebars than the current runtime. "+
                "Please update your precompiler to a newer version ("+runtimeVersions+") or downgrade your runtime to an older version ("+compilerVersions+").";
        } else {
          // Use the embedded version info since the runtime doesn't know about this revision yet
          throw "Template was precompiled with a newer version of Handlebars than the current runtime. "+
                "Please update your runtime to a newer version ("+compilerInfo[1]+").";
        }
      }

      return result;
    };
  },

  programWithDepth: function(i, fn, data /*, $depth */) {
    var args = Array.prototype.slice.call(arguments, 3);

    var program = function(context, options) {
      options = options || {};

      return fn.apply(this, [context, options.data || data].concat(args));
    };
    program.program = i;
    program.depth = args.length;
    return program;
  },
  program: function(i, fn, data) {
    var program = function(context, options) {
      options = options || {};

      return fn(context, options.data || data);
    };
    program.program = i;
    program.depth = 0;
    return program;
  },
  noop: function() { return ""; },
  invokePartial: function(partial, name, context, helpers, partials, data) {
    var options = { helpers: helpers, partials: partials, data: data };

    if(partial === undefined) {
      throw new Handlebars.Exception("The partial " + name + " could not be found");
    } else if(partial instanceof Function) {
      return partial(context, options);
    } else if (!Handlebars.compile) {
      throw new Handlebars.Exception("The partial " + name + " could not be compiled when running in runtime-only mode");
    } else {
      partials[name] = Handlebars.compile(partial, {data: data !== undefined});
      return partials[name](context, options);
    }
  }
};

Handlebars.template = Handlebars.VM.template;
;
// lib/handlebars/browser-suffix.js
})(Handlebars);
;

define("handlebars", (function (global) {
    return function () {
        var ret, fn;
        return ret || global.Handlebars;
    };
}(this)));

define('template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["credits"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<i class=\"icon-copyright\"></i>";
  });

this["JST"]["dialog"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression, self=this;

function program1(depth0,data) {
  
  var stack1;
  if (stack1 = helpers.color) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.color); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  return escapeExpression(stack1);
  }

function program3(depth0,data) {
  
  
  return "8";
  }

  buffer += "<header class=\"bgcolor-";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.color), {hash:{},inverse:self.program(3, program3, data),fn:self.program(1, program1, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\" >\n    <i class=\"icon-close\"></i>\n    <h1>";
  if (stack1 = helpers.title) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.title); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "</h1>\n</header>\n<main>\n</main>";
  return buffer;
  });

this["JST"]["dropdown.composite"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression, self=this;

function program1(depth0,data) {
  
  var buffer = "", stack1;
  buffer += " class=\"bgcolor-";
  if (stack1 = helpers.color) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.color); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\"";
  return buffer;
  }

  buffer += "<a>\n    <span>";
  if (stack1 = helpers.name) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.name); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "</span>\n    <button";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.color), {hash:{},inverse:self.noop,fn:self.program(1, program1, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "><i class=\"arrow\"></i></button>\n</a>\n<ul>\n</ul>";
  return buffer;
  });

this["JST"]["dropdown"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression, self=this;

function program1(depth0,data) {
  
  var buffer = "", stack1;
  buffer += " class=\"bgcolor-";
  if (stack1 = helpers.color) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.color); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\"";
  return buffer;
  }

function program3(depth0,data) {
  
  var buffer = "", stack1;
  buffer += "\n    <li>\n        <a href=\"#\" title=\"";
  if (stack1 = helpers.name) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.name); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\"";
  stack1 = helpers.each.call(depth0, (depth0 && depth0.data), {hash:{},inverse:self.noop,fn:self.program(4, program4, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += ">\n            ";
  if (stack1 = helpers.name) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.name); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\n        </a>\n    </li>\n    ";
  return buffer;
  }
function program4(depth0,data) {
  
  var buffer = "", stack1;
  buffer += " data-"
    + escapeExpression(((stack1 = ((stack1 = data),stack1 == null || stack1 === false ? stack1 : stack1.key)),typeof stack1 === functionType ? stack1.apply(depth0) : stack1))
    + "=\""
    + escapeExpression((typeof depth0 === functionType ? depth0.apply(depth0) : depth0))
    + "\"";
  return buffer;
  }

  buffer += "<a>\n    <span>";
  if (stack1 = helpers.title) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.title); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "</span>\n    <button";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.color), {hash:{},inverse:self.noop,fn:self.program(1, program1, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "><i class=\"arrow\"></i></button>\n</a>\n<ul>\n    ";
  stack1 = helpers.each.call(depth0, (depth0 && depth0.options), {hash:{},inverse:self.noop,fn:self.program(3, program3, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\n</ul>";
  return buffer;
  });

this["JST"]["dropdown.option"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression;


  buffer += "<a href=\"#\">\n    ";
  if (stack1 = helpers.name) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.name); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\n</a>";
  return buffer;
  });

this["JST"]["html"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var stack1, functionType="function";


  if (stack1 = helpers.text) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.text); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  if(stack1 || stack1 === 0) { return stack1; }
  else { return ''; }
  });

this["JST"]["icon"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression, self=this;

function program1(depth0,data) {
  
  var buffer = "", stack1;
  buffer += " color-";
  if (stack1 = helpers.color) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.color); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1);
  return buffer;
  }

function program3(depth0,data) {
  
  
  return " inactive";
  }

function program5(depth0,data) {
  
  var stack1;
  if (stack1 = helpers.label) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.label); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  return escapeExpression(stack1);
  }

  buffer += "<i class=\"icon-";
  if (stack1 = helpers.type) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.type); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1);
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.color), {hash:{},inverse:self.noop,fn:self.program(1, program1, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.inactive), {hash:{},inverse:self.noop,fn:self.program(3, program3, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\">";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.label), {hash:{},inverse:self.noop,fn:self.program(5, program5, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "</i>";
  return buffer;
  });

this["JST"]["loader"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<input />";
  });

this["JST"]["media"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, options, functionType="function", escapeExpression=this.escapeExpression, helperMissing=helpers.helperMissing;


  if (stack1 = helpers.tag) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.tag); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + " ";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "tagNotSupported", options) : helperMissing.call(depth0, "language", "tagNotSupported", options)));
  return buffer;
  });

this["JST"]["notification"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<main>\n</main>";
  });

this["JST"]["scrollbar"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression, self=this;

function program1(depth0,data) {
  
  var stack1;
  if (stack1 = helpers.color) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.color); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  return escapeExpression(stack1);
  }

function program3(depth0,data) {
  
  
  return "8";
  }

  buffer += "<div class=\"bar bgcolor-";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.color), {hash:{},inverse:self.program(3, program3, data),fn:self.program(1, program1, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\">\n    <div class=\"handler bgcolor-";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.color), {hash:{},inverse:self.program(3, program3, data),fn:self.program(1, program1, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\" />\n</div>";
  return buffer;
  });

this["JST"]["style"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var stack1, functionType="function", escapeExpression=this.escapeExpression, self=this;

function program1(depth0,data) {
  
  var buffer = "", stack1, stack2;
  buffer += "\n"
    + escapeExpression(((stack1 = ((stack1 = data),stack1 == null || stack1 === false ? stack1 : stack1.key)),typeof stack1 === functionType ? stack1.apply(depth0) : stack1))
    + " {\n    ";
  stack2 = helpers.each.call(depth0, depth0, {hash:{},inverse:self.noop,fn:self.program(2, program2, data),data:data});
  if(stack2 || stack2 === 0) { buffer += stack2; }
  buffer += "\n}\n";
  return buffer;
  }
function program2(depth0,data) {
  
  var buffer = "", stack1;
  buffer += "\n    "
    + escapeExpression(((stack1 = ((stack1 = data),stack1 == null || stack1 === false ? stack1 : stack1.key)),typeof stack1 === functionType ? stack1.apply(depth0) : stack1))
    + ": "
    + escapeExpression((typeof depth0 === functionType ? depth0.apply(depth0) : depth0))
    + ";\n    ";
  return buffer;
  }

  stack1 = helpers.each.call(depth0, depth0, {hash:{},inverse:self.noop,fn:self.program(1, program1, data),data:data});
  if(stack1 || stack1 === 0) { return stack1; }
  else { return ''; }
  });

this["JST"]["text.editor"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, stack2, options, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function";


  buffer += "<textarea placeholder=\"";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "text", options) : helperMissing.call(depth0, "language", "text", options)))
    + "\">";
  if (stack2 = helpers.text) { stack2 = stack2.call(depth0, {hash:{},data:data}); }
  else { stack2 = (depth0 && depth0.text); stack2 = typeof stack2 === functionType ? stack2.call(depth0, {hash:{},data:data}) : stack2; }
  buffer += escapeExpression(stack2)
    + "</textarea>";
  return buffer;
  });

this["JST"]["thumbnail"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression, self=this;

function program1(depth0,data) {
  
  
  return " class=\"current\"";
  }

function program3(depth0,data,depth1) {
  
  var buffer = "", stack1, stack2;
  buffer += escapeExpression(((stack1 = (depth1 && depth1.path)),typeof stack1 === functionType ? stack1.apply(depth0) : stack1));
  if (stack2 = helpers.thumbnail) { stack2 = stack2.call(depth0, {hash:{},data:data}); }
  else { stack2 = (depth0 && depth0.thumbnail); stack2 = typeof stack2 === functionType ? stack2.call(depth0, {hash:{},data:data}) : stack2; }
  buffer += escapeExpression(stack2);
  return buffer;
  }

function program5(depth0,data) {
  
  var stack1;
  if (stack1 = helpers.blank) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.blank); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  return escapeExpression(stack1);
  }

  buffer += "<i class=\"icon-close\"></i>\n<img alt=\"";
  if (stack1 = helpers.title) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.title); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\"";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.current), {hash:{},inverse:self.noop,fn:self.program(1, program1, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += " src=\"";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.thumbnail), {hash:{},inverse:self.program(5, program5, data),fn:self.programWithDepth(3, program3, data, depth0),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\" title=\"";
  if (stack1 = helpers.title) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.title); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\" />\n<span>";
  if (stack1 = helpers.title) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.title); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "</span>\n<aside></aside>";
  return buffer;
  });

return this["JST"];

});
// noinspection JSHint
define(
    'views/popup',["underscore", "marionette", "regions/replace", "template"],
    function (_, Marionette, ReplaceRegion, JST) {
      "use strict";

      return Marionette.Layout.extend({
        className: "dialog",
        events: {
          "click  @ui.exit": "quit",

          "drag @ui.title": function (event) {
            event = event.originalEvent || event;

            event.preventDefault();
            event.stopPropagation();
            event.gesture.preventDefault();

            var offset = {
              x: this.offset.x + event.gesture.deltaX,
              y: this.offset.y + event.gesture.deltaY
            };

            this.translate(offset.x, offset.y);
          },

          "dragend @ui.title": function (event) {
            event = event.originalEvent || event;

            event.preventDefault();
            event.stopPropagation();
            event.gesture.preventDefault();

            this.offset = {
              x: this.offset.x + event.gesture.deltaX,
              y: this.offset.y + event.gesture.deltaY
            };
          }
        },
        tagName: "section",

        regions: {
          content: "main"
        },
        regionType: ReplaceRegion,
        template: JST.dialog,
        ui: {
          exit: ".icon-close",
          title: "header"
        },

        onShow: function () {
          var height = Marionette.getOption(this, "height"),
              width = Marionette.getOption(this, "width"),
              top = Marionette.getOption(this, "top"),
              bottom = Marionette.getOption(this, "bottom"),
              left = Marionette.getOption(this, "left"),
              right = Marionette.getOption(this, "right");

          if (width) {
            this.$el.width(width);
          }
          width = this.$el.width();
          this.$el.css("margin-left", -Math.floor(width / 2));

          if (height) {
            this.$el
                .height(height)
                .css("bottom", "auto");
          }

          if (top) {
            this.$el.css("top", top);
          }
          if (bottom) {
            this.$el
                .css("top", "auto")
                .css("bottom", bottom);
          }

          if (left) {
            this.$el
                .css("margin-left", 0)
                .css("left", left);
          }
          if (right) {
            this.$el
                .css("left", "auto")
                .css("right", right);
          }
        },

        offset: {
          x: 0,
          y: 0
        },
        outEvents: true,

        quit: function () {
          this.content.trigger("close");
          this.trigger("quit");
          this.close();
        },

        show: function (view) {
          view.on("all", function (eventName) {
            if (_.contains(["render", "show"], eventName)) {
              return false;
            }

            if (this[eventName]) {
              this[eventName].apply(
                  this,
                  Array.prototype.slice.call(arguments).slice(1)
              );
            }

            return true;
          }, this);

          this.content.show(view);
        },

        title: function (title) {
          this.model.set("title", title);
        },

        translate: function (x, y) {
          var translate = "translate3d(" + x + "px, " + y + "px, 0)";

          //noinspection JSHint
          Marionette.getOption(this, "draggable") &&
          this.$el
              .css("transform", translate)
              .css("-webkit-transform", translate);
        },
        resize: function (width, height) {
          var top = Marionette.getOption(this, "top"),
              bottom = Marionette.getOption(this, "bottom"),
              left = Marionette.getOption(this, "left"),
              right = Marionette.getOption(this, "right");

          if (width) {
            this.$el.width(width);
          }
          width = this.$el.width();
          this.$el.css("margin-left", -Math.floor(width / 2));

          if (height) {
            this.$el
                .height(height)
                .css("bottom", "auto");
          }

          if (top) {
            this.$el.css("top", top);
          }
          if (bottom) {
            this.$el
                .css("top", "auto")
                .css("bottom", bottom);
          }

          if (left) {
            this.$el
                .css("margin-left", 0)
                .css("left", left * 2);
          }
          if (right) {
            this.$el
                .css("left", "auto")
                .css("right", right);
          }
        }
      });
    }
);
//noinspection JSHint
define(
    'regions/prepend',["marionette"],
    function(Marionette) {
      "use strict";

      return Marionette.Region.extend({
        open: function(view) {
          this.$el.prepend(view.el);
        }
      });
    }
);
//noinspection JSHint
define(
    'views/dialog',[
      "jquery",
      "underscore",
      "backbone",
      "marionette",
      "views/modal",
      "views/popup",
      "regions/prepend"
    ],
    function ($, _, Backbone, Marionette, ModalView, PopupView, PrependRegion) {
      "use strict";

      return PrependRegion.extend({
        el: document.body,

        initialize: function (options) {
          _.defaults(options || (options = {}), {
            draggable: true,
            modal: true
          });

          if (options.modal) {
            options.modal = new ModalView();

            this.on("close", function () {
              options.modal.close();
            });
          }
        },

        show: function (view) {
          var popupView = new PopupView(_.extend({
                model: new Backbone.Model(this.options)
              }, this.options)),
              modal = Marionette.getOption(this, "modal"),
              response;

          if (modal) {
            Marionette.Region.prototype.show.call(this, modal);

//            modal.on("quit", function() {
//              popupView.quit();
//            });
          }

          response = Marionette.Region.prototype.show.call(
              this,
              popupView,
              {preventClose: true}
          );

          popupView.on("all", function () {
            this.trigger.apply(this, Array.prototype.slice.call(arguments));
          }, this);

          $(window).on("resize", _.bind(function () {
            this.$el.css("margin-left", -Math.floor(this.$el.width() / 2));
          }, popupView));

          popupView.show(view);

          return response;
        }
      });
    }
);
// noinspection JSHint
define(
    'views/iframe',["marionette"],
    function (Marionette) {
      "use strict";

      return Marionette.View.extend({
        attributes: function() {
          return {
            src: this.model.get("src")
          };
        },
        tagName: "iframe"
      });
    }
);
// noinspection JSHint
define(
    'views/html',["underscore", "backbone", "marionette", "template"],
    function (_, Backbone, Marionette, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        template: JST.html,

        initialize: function (options) {
          _.defaults(options || (options = {}), {
            className: "",
            tagName: "main"
          });

          // extend object
          _.extend(this, options);
          this.model = new Backbone.Model({text: options.message});

          // regenerate el
          delete this.el;
          this._ensureElement();
        }
      });
    }
);
/*!jQuery Knob*/
/**
 * Downward compatible, touchable dial
 *
 * Version: 1.2.12
 * Requires: jQuery v1.7+
 *
 * Copyright (c) 2012 Anthony Terrien
 * Under MIT License (http://www.opensource.org/licenses/mit-license.php)
 *
 * Thanks to vor, eskimoblood, spiffistan, FabrizioC
 */
(function (factory) {
    if (typeof exports === 'object') {
        // CommonJS
        module.exports = factory(require('jquery'));
    } else if (typeof define === 'function' && define.amd) {
        // AMD. Register as an anonymous module.
        define('knob',['jquery'], factory);
    } else {
        // Browser globals
        factory(jQuery);
    }
}(function ($) {

    /**
     * Kontrol library
     */
    "use strict";

    /**
     * Definition of globals and core
     */
    var k = {}, // kontrol
        max = Math.max,
        min = Math.min;

    k.c = {};
    k.c.d = $(document);
    k.c.t = function (e) {
        return e.originalEvent.touches.length - 1;
    };

    /**
     * Kontrol Object
     *
     * Definition of an abstract UI control
     *
     * Each concrete component must call this one.
     * <code>
     * k.o.call(this);
     * </code>
     */
    k.o = function () {
        var s = this;

        this.o = null; // array of options
        this.$ = null; // jQuery wrapped element
        this.i = null; // mixed HTMLInputElement or array of HTMLInputElement
        this.g = null; // deprecated 2D graphics context for 'pre-rendering'
        this.v = null; // value ; mixed array or integer
        this.cv = null; // change value ; not commited value
        this.x = 0; // canvas x position
        this.y = 0; // canvas y position
        this.w = 0; // canvas width
        this.h = 0; // canvas height
        this.$c = null; // jQuery canvas element
        this.c = null; // rendered canvas context
        this.t = 0; // touches index
        this.isInit = false;
        this.fgColor = null; // main color
        this.pColor = null; // previous color
        this.dH = null; // draw hook
        this.cH = null; // change hook
        this.eH = null; // cancel hook
        this.rH = null; // release hook
        this.scale = 1; // scale factor
        this.relative = false;
        this.relativeWidth = false;
        this.relativeHeight = false;
        this.$div = null; // component div

        this.run = function () {
            var cf = function (e, conf) {
                var k;
                for (k in conf) {
                    s.o[k] = conf[k];
                }
                s._carve().init();
                s._configure()
                 ._draw();
            };

            if (this.$.data('kontroled')) return;
            this.$.data('kontroled', true);

            this.extend();
            this.o = $.extend({
                    // Config
                    min: this.$.data('min') !== undefined ? this.$.data('min') : 0,
                    max: this.$.data('max') !== undefined ? this.$.data('max') : 100,
                    stopper: true,
                    readOnly: this.$.data('readonly') || (this.$.attr('readonly') === 'readonly'),

                    // UI
                    cursor: this.$.data('cursor') === true && 30
                            || this.$.data('cursor') || 0,
                    thickness: this.$.data('thickness')
                               && Math.max(Math.min(this.$.data('thickness'), 1), 0.01)
                               || 0.35,
                    lineCap: this.$.data('linecap') || 'butt',
                    width: this.$.data('width') || 200,
                    height: this.$.data('height') || 200,
                    displayInput: this.$.data('displayinput') == null || this.$.data('displayinput'),
                    displayPrevious: this.$.data('displayprevious'),
                    fgColor: this.$.data('fgcolor') || '#87CEEB',
                    inputColor: this.$.data('inputcolor'),
                    font: this.$.data('font') || 'Arial',
                    fontWeight: this.$.data('font-weight') || 'bold',
                    inline: false,
                    step: this.$.data('step') || 1,
                    rotation: this.$.data('rotation'),

                    // Hooks
                    draw: null, // function () {}
                    change: null, // function (value) {}
                    cancel: null, // function () {}
                    release: null, // function (value) {}

                    // Output formatting, allows to add unit: %, ms ...
                    format: function(v) {
                        return v;
                    },
                    parse: function (v) {
                        return parseFloat(v);
                    }
                }, this.o
            );

            // finalize options
            this.o.flip = this.o.rotation === 'anticlockwise' || this.o.rotation === 'acw';
            if (!this.o.inputColor) {
                this.o.inputColor = this.o.fgColor;
            }

            // routing value
            if (this.$.is('fieldset')) {

                // fieldset = array of integer
                this.v = {};
                this.i = this.$.find('input');
                this.i.each(function(k) {
                    var $this = $(this);
                    s.i[k] = $this;
                    s.v[k] = s.o.parse($this.val());

                    $this.bind(
                        'change blur',
                        function () {
                            var val = {};
                            val[k] = $this.val();
                            s.val(s._validate(val));
                        }
                    );
                });
                this.$.find('legend').remove();
            } else {

                // input = integer
                this.i = this.$;
                this.v = this.o.parse(this.$.val());
                this.v === '' && (this.v = this.o.min);
                this.$.bind(
                    'change blur',
                    function () {
                        s.val(s._validate(s.o.parse(s.$.val())));
                    }
                );

            }

            !this.o.displayInput && this.$.hide();

            // adds needed DOM elements (canvas, div)
            this.$c = $(document.createElement('canvas')).attr({
                width: this.o.width,
                height: this.o.height
            });

            // wraps all elements in a div
            // add to DOM before Canvas init is triggered
            this.$div = $('<div style="'
                + (this.o.inline ? 'display:inline;' : '')
                + 'width:' + this.o.width + 'px;height:' + this.o.height + 'px;'
                + '"></div>');

            this.$.wrap(this.$div).before(this.$c);
            this.$div = this.$.parent();

            if (typeof G_vmlCanvasManager !== 'undefined') {
                G_vmlCanvasManager.initElement(this.$c[0]);
            }

            this.c = this.$c[0].getContext ? this.$c[0].getContext('2d') : null;

            if (!this.c) {
                throw {
                    name:        "CanvasNotSupportedException",
                    message:     "Canvas not supported. Please use excanvas on IE8.0.",
                    toString:    function(){return this.name + ": " + this.message}
                }
            }

            // hdpi support
            this.scale = (window.devicePixelRatio || 1) / (
                            this.c.webkitBackingStorePixelRatio ||
                            this.c.mozBackingStorePixelRatio ||
                            this.c.msBackingStorePixelRatio ||
                            this.c.oBackingStorePixelRatio ||
                            this.c.backingStorePixelRatio || 1
                         );

            // detects relative width / height
            this.relativeWidth =  this.o.width % 1 !== 0
                                  && this.o.width.indexOf('%');
            this.relativeHeight = this.o.height % 1 !== 0
                                  && this.o.height.indexOf('%');
            this.relative = this.relativeWidth || this.relativeHeight;

            // computes size and carves the component
            this._carve();

            // prepares props for transaction
            if (this.v instanceof Object) {
                this.cv = {};
                this.copy(this.v, this.cv);
            } else {
                this.cv = this.v;
            }

            // binds configure event
            this.$
                .bind("configure", cf)
                .parent()
                .bind("configure", cf);

            // finalize init
            this._listen()
                ._configure()
                ._xy()
                .init();

            this.isInit = true;

            this.$.val(this.o.format(this.v));
            this._draw();

            return this;
        };

        this._carve = function() {
            if (this.relative) {
                var w = this.relativeWidth ?
                        this.$div.parent().width() *
                        parseInt(this.o.width) / 100
                        : this.$div.parent().width(),
                    h = this.relativeHeight ?
                        this.$div.parent().height() *
                        parseInt(this.o.height) / 100
                        : this.$div.parent().height();

                // apply relative
                this.w = this.h = Math.min(w, h);
            } else {
                this.w = this.o.width;
                this.h = this.o.height;
            }

            // finalize div
            this.$div.css({
                'width': this.w + 'px',
                'height': this.h + 'px'
            });

            // finalize canvas with computed width
            this.$c.attr({
                width: this.w,
                height: this.h
            });

            // scaling
            if (this.scale !== 1) {
                this.$c[0].width = this.$c[0].width * this.scale;
                this.$c[0].height = this.$c[0].height * this.scale;
                this.$c.width(this.w);
                this.$c.height(this.h);
            }

            return this;
        };

        this._draw = function () {

            // canvas pre-rendering
            var d = true;

            s.g = s.c;

            s.clear();

            s.dH && (d = s.dH());

            d !== false && s.draw();
        };

        this._touch = function (e) {
            var touchMove = function (e) {
                var v = s.xy2val(
                            e.originalEvent.touches[s.t].pageX,
                            e.originalEvent.touches[s.t].pageY
                        );

                if (v == s.cv) return;

                if (s.cH && s.cH(v) === false) return;

                s.change(s._validate(v));
                s._draw();
            };

            // get touches index
            this.t = k.c.t(e);

            // First touch
            touchMove(e);

            // Touch events listeners
            k.c.d
                .bind("touchmove.k", touchMove)
                .bind(
                    "touchend.k",
                    function () {
                        k.c.d.unbind('touchmove.k touchend.k');
                        s.val(s.cv);
                    }
                );

            return this;
        };

        this._mouse = function (e) {
            var mouseMove = function (e) {
                var v = s.xy2val(e.pageX, e.pageY);

                if (v == s.cv) return;

                if (s.cH && (s.cH(v) === false)) return;

                s.change(s._validate(v));
                s._draw();
            };

            // First click
            mouseMove(e);

            // Mouse events listeners
            k.c.d
                .bind("mousemove.k", mouseMove)
                .bind(
                    // Escape key cancel current change
                    "keyup.k",
                    function (e) {
                        if (e.keyCode === 27) {
                            k.c.d.unbind("mouseup.k mousemove.k keyup.k");

                            if (s.eH && s.eH() === false)
                                return;

                            s.cancel();
                        }
                    }
                )
                .bind(
                    "mouseup.k",
                    function (e) {
                        k.c.d.unbind('mousemove.k mouseup.k keyup.k');
                        s.val(s.cv);
                    }
                );

            return this;
        };

        this._xy = function () {
            var o = this.$c.offset();
            this.x = o.left;
            this.y = o.top;

            return this;
        };

        this._listen = function () {
            if (!this.o.readOnly) {
                this.$c
                    .bind(
                        "mousedown",
                        function (e) {
                            e.preventDefault();
                            s._xy()._mouse(e);
                        }
                    )
                    .bind(
                        "touchstart",
                        function (e) {
                            e.preventDefault();
                            s._xy()._touch(e);
                        }
                    );

                this.listen();
            } else {
                this.$.attr('readonly', 'readonly');
            }

            if (this.relative) {
                $(window).resize(function() {
                    s._carve().init();
                    s._draw();
                });
            }

            return this;
        };

        this._configure = function () {

            // Hooks
            if (this.o.draw) this.dH = this.o.draw;
            if (this.o.change) this.cH = this.o.change;
            if (this.o.cancel) this.eH = this.o.cancel;
            if (this.o.release) this.rH = this.o.release;

            if (this.o.displayPrevious) {
                this.pColor = this.h2rgba(this.o.fgColor, "0.4");
                this.fgColor = this.h2rgba(this.o.fgColor, "0.6");
            } else {
                this.fgColor = this.o.fgColor;
            }

            return this;
        };

        this._clear = function () {
            this.$c[0].width = this.$c[0].width;
        };

        this._validate = function (v) {
            var val = (~~ (((v < 0) ? -0.5 : 0.5) + (v/this.o.step))) * this.o.step;
            return Math.round(val * 100) / 100;
        };

        // Abstract methods
        this.listen = function () {}; // on start, one time
        this.extend = function () {}; // each time configure triggered
        this.init = function () {}; // each time configure triggered
        this.change = function (v) {}; // on change
        this.val = function (v) {}; // on release
        this.xy2val = function (x, y) {}; //
        this.draw = function () {}; // on change / on release
        this.clear = function () { this._clear(); };

        // Utils
        this.h2rgba = function (h, a) {
            var rgb;
            h = h.substring(1,7);
            rgb = [
                parseInt(h.substring(0,2), 16),
                parseInt(h.substring(2,4), 16),
                parseInt(h.substring(4,6), 16)
            ];

            return "rgba(" + rgb[0] + "," + rgb[1] + "," + rgb[2] + "," + a + ")";
        };

        this.copy = function (f, t) {
            for (var i in f) {
                t[i] = f[i];
            }
        };
    };


    /**
     * k.Dial
     */
    k.Dial = function () {
        k.o.call(this);

        this.startAngle = null;
        this.xy = null;
        this.radius = null;
        this.lineWidth = null;
        this.cursorExt = null;
        this.w2 = null;
        this.PI2 = 2*Math.PI;

        this.extend = function () {
            this.o = $.extend({
                bgColor: this.$.data('bgcolor') || '#EEEEEE',
                angleOffset: this.$.data('angleoffset') || 0,
                angleArc: this.$.data('anglearc') || 360,
                inline: true
            }, this.o);
        };

        this.val = function (v, triggerRelease) {
            if (null != v) {

                // reverse format
                v = this.o.parse(v);

                if (triggerRelease !== false
                    && v != this.v
                    && this.rH
                    && this.rH(v) === false) { return; }

                this.cv = this.o.stopper ? max(min(v, this.o.max), this.o.min) : v;
                this.v = this.cv;
                this.$.val(this.o.format(this.v));
                this._draw();
            } else {
                return this.v;
            }
        };

        this.xy2val = function (x, y) {
            var a, ret;

            a = Math.atan2(
                        x - (this.x + this.w2),
                        - (y - this.y - this.w2)
                    ) - this.angleOffset;

            if (this.o.flip) {
                a = this.angleArc - a - this.PI2;
            }

            if (this.angleArc != this.PI2 && (a < 0) && (a > -0.5)) {

                // if isset angleArc option, set to min if .5 under min
                a = 0;
            } else if (a < 0) {
                a += this.PI2;
            }

            ret = (a * (this.o.max - this.o.min) / this.angleArc) + this.o.min;

            this.o.stopper && (ret = max(min(ret, this.o.max), this.o.min));

            return ret;
        };

        this.listen = function () {

            // bind MouseWheel
            var s = this, mwTimerStop,
                mwTimerRelease,
                mw = function (e) {
                    e.preventDefault();

                    var ori = e.originalEvent,
                        deltaX = ori.detail || ori.wheelDeltaX,
                        deltaY = ori.detail || ori.wheelDeltaY,
                        v = s._validate(s.o.parse(s.$.val()))
                            + (
                                deltaX > 0 || deltaY > 0
                                ? s.o.step
                                : deltaX < 0 || deltaY < 0 ? -s.o.step : 0
                              );

                    v = max(min(v, s.o.max), s.o.min);

                    s.val(v, false);

                    if (s.rH) {
                        // Handle mousewheel stop
                        clearTimeout(mwTimerStop);
                        mwTimerStop = setTimeout(function () {
                            s.rH(v);
                            mwTimerStop = null;
                        }, 100);

                        // Handle mousewheel releases
                        if (!mwTimerRelease) {
                            mwTimerRelease = setTimeout(function () {
                                if (mwTimerStop)
                                    s.rH(v);
                                mwTimerRelease = null;
                            }, 200);
                        }
                    }
                },
                kval,
                to,
                m = 1,
                kv = {
                    37: -s.o.step,
                    38: s.o.step,
                    39: s.o.step,
                    40: -s.o.step
                };

            this.$
                .bind(
                    "keydown",
                    function (e) {
                        var kc = e.keyCode;

                        // numpad support
                        if (kc >= 96 && kc <= 105) {
                            kc = e.keyCode = kc - 48;
                        }

                        kval = parseInt(String.fromCharCode(kc));

                        if (isNaN(kval)) {
                            (kc !== 13)                     // enter
                            && kc !== 8                     // bs
                            && kc !== 9                     // tab
                            && kc !== 189                   // -
                            && (kc !== 190
                                || s.$.val().match(/\./))   // . allowed once
                            && e.preventDefault();

                            // arrows
                            if ($.inArray(kc,[37,38,39,40]) > -1) {
                                e.preventDefault();

                                var v = s.o.parse(s.$.val()) + kv[kc] * m;
                                s.o.stopper && (v = max(min(v, s.o.max), s.o.min));

                                s.change(s._validate(v));
                                s._draw();

                                // long time keydown speed-up
                                to = window.setTimeout(function () {
                                    m *= 2;
                                }, 30);
                            }
                        }
                    }
                )
                .bind(
                    "keyup",
                    function (e) {
                        if (isNaN(kval)) {
                            if (to) {
                                window.clearTimeout(to);
                                to = null;
                                m = 1;
                                s.val(s.$.val());
                            }
                        } else {
                            // kval postcond
                            (s.$.val() > s.o.max && s.$.val(s.o.max))
                            || (s.$.val() < s.o.min && s.$.val(s.o.min));
                        }
                    }
                );

            this.$c.bind("mousewheel DOMMouseScroll", mw);
            this.$.bind("mousewheel DOMMouseScroll", mw);
        };

        this.init = function () {
            if (this.v < this.o.min
                || this.v > this.o.max) { this.v = this.o.min; }

            this.$.val(this.v);
            this.w2 = this.w / 2;
            this.cursorExt = this.o.cursor / 100;
            this.xy = this.w2 * this.scale;
            this.lineWidth = this.xy * this.o.thickness;
            this.lineCap = this.o.lineCap;
            this.radius = this.xy - this.lineWidth / 2;

            this.o.angleOffset
            && (this.o.angleOffset = isNaN(this.o.angleOffset) ? 0 : this.o.angleOffset);

            this.o.angleArc
            && (this.o.angleArc = isNaN(this.o.angleArc) ? this.PI2 : this.o.angleArc);

            // deg to rad
            this.angleOffset = this.o.angleOffset * Math.PI / 180;
            this.angleArc = this.o.angleArc * Math.PI / 180;

            // compute start and end angles
            this.startAngle = 1.5 * Math.PI + this.angleOffset;
            this.endAngle = 1.5 * Math.PI + this.angleOffset + this.angleArc;

            var s = max(
                String(Math.abs(this.o.max)).length,
                String(Math.abs(this.o.min)).length,
                2
            ) + 2;

            this.o.displayInput
                && this.i.css({
                        'width' : ((this.w / 2 + 4) >> 0) + 'px',
                        'height' : ((this.w / 3) >> 0) + 'px',
                        'position' : 'absolute',
                        'vertical-align' : 'middle',
                        'margin-top' : ((this.w / 3) >> 0) + 'px',
                        'margin-left' : '-' + ((this.w * 3 / 4 + 2) >> 0) + 'px',
                        'border' : 0,
                        'background' : 'none',
                        'font' : this.o.fontWeight + ' ' + ((this.w / s) >> 0) + 'px ' + this.o.font,
                        'text-align' : 'center',
                        'color' : this.o.inputColor || this.o.fgColor,
                        'padding' : '0px',
                        '-webkit-appearance': 'none'
                        }) || this.i.css({
                            'width': '0px',
                            'visibility': 'hidden'
                        });
        };

        this.change = function (v) {
            this.cv = v;
            this.$.val(this.o.format(v));
        };

        this.angle = function (v) {
            return (v - this.o.min) * this.angleArc / (this.o.max - this.o.min);
        };

        this.arc = function (v) {
          var sa, ea;
          v = this.angle(v);
          if (this.o.flip) {
              sa = this.endAngle + 0.00001;
              ea = sa - v - 0.00001;
          } else {
              sa = this.startAngle - 0.00001;
              ea = sa + v + 0.00001;
          }
          this.o.cursor
              && (sa = ea - this.cursorExt)
              && (ea = ea + this.cursorExt);

          return {
              s: sa,
              e: ea,
              d: this.o.flip && !this.o.cursor
          };
        };

        this.draw = function () {
            var c = this.g,                 // context
                a = this.arc(this.cv),      // Arc
                pa,                         // Previous arc
                r = 1;

            c.lineWidth = this.lineWidth;
            c.lineCap = this.lineCap;

            if (this.o.bgColor !== "none") {
                c.beginPath();
                    c.strokeStyle = this.o.bgColor;
                    c.arc(this.xy, this.xy, this.radius, this.endAngle - 0.00001, this.startAngle + 0.00001, true);
                c.stroke();
            }

            if (this.o.displayPrevious) {
                pa = this.arc(this.v);
                c.beginPath();
                c.strokeStyle = this.pColor;
                c.arc(this.xy, this.xy, this.radius, pa.s, pa.e, pa.d);
                c.stroke();
                r = this.cv == this.v;
            }

            c.beginPath();
            c.strokeStyle = r ? this.o.fgColor : this.fgColor ;
            c.arc(this.xy, this.xy, this.radius, a.s, a.e, a.d);
            c.stroke();
        };

        this.cancel = function () {
            this.val(this.v);
        };
    };

    $.fn.dial = $.fn.knob = function (o) {
        return this.each(
            function () {
                var d = new k.Dial();
                d.o = o;
                d.$ = $(this);
                d.run();
            }
        ).parent();
    };

}));

// noinspection JSHint
define(
    'views/loader',["underscore", "marionette", "regions/prepend", "template", "knob"],
    function (_, Marionette, PrependRegion, JST) {
      "use strict";

      var LoaderView = Marionette.ItemView.extend({
            /* attributes */
            // backbone & marionette
            className: "loader",
            tagName: "article",
            template: JST.loader,
            ui: {
              input: "input"
            },

            // custom
            timeout: null,

            /* methods */
            // backbone & marionette
            initialize: function (options) {
              this.options = _.defaults(options || (options = {}), {
                infinite: false,
                interval: 5,
                velocity: 40
              });
            },

            onClose: function () {
              clearInterval(this.timeout);
            },

            onRender: function () {
              var options = _.defaults(this.options, {
                    readOnly: true,

                    bgColor: "#FFFFFF",
                    fgColor: "#56B8A7",
                    font: "Signika Negative",
                    fontWeight: 300,
                    inputColor: "#747474",
                    lineCap: "round",
                    thickness: 0.15,
                    width: 50
                  }),
                  margin = -options.width / 2;

              if (options.infinite) {
                _.extend(options, {
                  stopper: false,

                  cursor: 90,
                  thickness: options.thickness * 1.25,
                  displayInput: false
                });

                this.timeout = setInterval(_.bind(function (velocity) {
                  this.value(new Date().getTime() / 1000 * velocity % 100);
                }, this), options.interval, options.velocity);
              }

              this.$el
                  .css("margin-top", margin)
                  .css("margin-left", margin);

              if (options.top) {
                this.$el.css("top", options.top);
              }

              if (options.left) {
                this.$el.css("left", options.left);
              }

              this.ui.input.knob(options);
            },

            // custom
            value: function (value) {
              this.ui.input.val(parseFloat(value.toFixed(2))).trigger("change");
            }
          }),
          ModalView = Marionette.ItemView.extend({
            className: "modal",
            tagName: "aside",
            template: _.template("")
          });

      return PrependRegion.extend({
        el: document.body,

        initialize: function (options) {
          _.defaults(options || (options = {}), {
            modal: false
          });

          if (options.modal) {
            options.modal = new ModalView();

            this.on("close", function() {
              options.modal.close();
            });
            this.show(options.modal);
          }

          this.show(new LoaderView(options), {preventClose: true});
        },

        value: function () {
          return this.currentView.value.apply(this.currentView, arguments);
        }
      });
    }
);
// noinspection JSHint
define(
    'views/popup.mini',["underscore", "marionette", "views/popup", "regions/replace", "template"],
    function (_, Marionette, PopupView, ReplaceRegion, JST) {
      "use strict";

      return PopupView.extend({
        className: "notification norender",
        events: {
          "click": "quit",

          "drag": function (event) {
            event = event.originalEvent || event;

            event.preventDefault();
            event.stopPropagation();
            event.gesture.preventDefault();

            var offset = {
              x: this.offset.x + event.gesture.deltaX,
              y: this.offset.y + event.gesture.deltaY
            };

            this.translate(offset.x, offset.y);
          },

          "dragend": function (event) {
            event = event.originalEvent || event;

            event.preventDefault();
            event.stopPropagation();
            event.gesture.preventDefault();

            this.offset = {
              x: this.offset.x + event.gesture.deltaX,
              y: this.offset.y + event.gesture.deltaY
            };
          }
        },
        template: JST.notification,

        onShow: function () {
          if (this.options.color) {
            this.$el.addClass("bgcolor-" + this.options.color);
          }
          _.chain(function() {
            this.$el.removeClass("norender");
          })
              .bind(this)
              .defer()
              .value();

          PopupView.prototype.onShow.apply(this, arguments);

          _.chain(function() {
            if (this.isClosed) {
              return false;
            }
            this.$el.addClass("norender");
            _.chain(function() {
              if (!this.isClosed) {
                this.quit();
              }
            })
                .bind(this)
                .delay(500)
                .value();

            return true;
          })
              .bind(this)
              .delay(3500)
              .value();
        }
      });
    }
);
//noinspection JSHint
define(
    'views/notification',[
      "jquery",
      "underscore",
      "backbone",
      "marionette",
      "views/popup.mini",
      "regions/prepend"
    ],
    function ($, _, Backbone, Marionette, PopupView, PrependRegion) {
      "use strict";

      return PrependRegion.extend({
        el: document.body,

        initialize: function (options) {
          _.defaults(options || (options = {}), {
            draggable: true
          });
        },

        show: function (view) {
          var popupView = new PopupView(_.extend({
                model: new Backbone.Model(this.options)
              }, this.options)),
              response;

          response = Marionette.Region.prototype.show.call(
              this,
              popupView,
              {preventClose: true}
          );

          popupView.on("all", function () {
            this.trigger.apply(this, Array.prototype.slice.call(arguments));
          }, this);

          $(window).on("resize", _.bind(function () {
            this.$el.css("margin-left", -Math.floor(this.$el.width() / 2));
          }, popupView));

          popupView.show(view);

          return response;
        }
      });
    }
);
define('encrypt',{
    encode64: function(input) {
        input = escape(input);
        var keyStr = "ABCDEFGHIJKLMNOP" +
            "QRSTUVWXYZabcdef" +
            "ghijklmnopqrstuv" +
            "wxyz0123456789+/" +
            "=";
        var output = "";
        var chr1, chr2, chr3 = "";
        var enc1, enc2, enc3, enc4 = "";
        var i = 0;

        do {
            chr1 = input.charCodeAt(i++);
            chr2 = input.charCodeAt(i++);
            chr3 = input.charCodeAt(i++);

            enc1 = chr1 >> 2;
            enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
            enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
            enc4 = chr3 & 63;

            if (isNaN(chr2)) {
                enc3 = enc4 = 64;
            } else if (isNaN(chr3)) {
                enc4 = 64;
            }

            output = output +
                keyStr.charAt(enc1) +
                keyStr.charAt(enc2) +
                keyStr.charAt(enc3) +
                keyStr.charAt(enc4);
            chr1 = chr2 = chr3 = "";
            enc1 = enc2 = enc3 = enc4 = "";
        } while (i < input.length);

        return output;
    },
    decode64: function(input){
        var keyStr = "ABCDEFGHIJKLMNOP" +
            "QRSTUVWXYZabcdef" +
            "ghijklmnopqrstuv" +
            "wxyz0123456789+/" +
            "=";
        var output = "";
        var chr1, chr2, chr3 = "";
        var enc1, enc2, enc3, enc4 = "";
        var i = 0;

        // remove all characters that are not A-Z, a-z, 0-9, +, /, or =
        var base64test = /[^A-Za-z0-9\+\/\=]/g;
        if (base64test.exec(input)) {
            return null;
            alert("There were invalid base64 characters in the input text.\n" +
                "Valid base64 characters are A-Z, a-z, 0-9, '+', '/',and '='\n" +
                "Expect errors in decoding.");
        }
        input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

        do {
            enc1 = keyStr.indexOf(input.charAt(i++));
            enc2 = keyStr.indexOf(input.charAt(i++));
            enc3 = keyStr.indexOf(input.charAt(i++));
            enc4 = keyStr.indexOf(input.charAt(i++));

            chr1 = (enc1 << 2) | (enc2 >> 4);
            chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
            chr3 = ((enc3 & 3) << 6) | enc4;

            output = output + String.fromCharCode(chr1);

            if (enc3 != 64) {
                output = output + String.fromCharCode(chr2);
            }
            if (enc4 != 64) {
                output = output + String.fromCharCode(chr3);
            }

            chr1 = chr2 = chr3 = "";
            enc1 = enc2 = enc3 = enc4 = "";

        } while (i < input.length);

        return unescape(output);
    },
    decrypt: function(cadena){
        var key="1cff42dabb60beaf1e3b57988af787246c63613ef60435a05c9c79b98a9b41c8";
        var result = '';

        var mcadena = this.decode64(cadena);

        if (mcadena == null){
            return cadena;
        }else{
            cadena = mcadena;
        }

        var mychar = "";
        var mycharres = "";
        var keychar = "";
        var ordChar = "";
        var ordKeychar = "";
        var sum = 0;

        for(var i=0; i<cadena.length; i++) {
            mychar = cadena.substr(i,1);
            keychar = key.substr((i % key.length)-1, 1);
            ordChar = mychar.charCodeAt(0);
            ordKeychar = keychar.charCodeAt(0);
            sum = parseInt(ordChar,10) - parseInt(ordKeychar,10);
            mycharres = String.fromCharCode(sum);
            //console.log("mychar:" + mychar + " keychar:" + keychar + " ordChar:" + ordChar + " ordKeychar:" + ordKeychar + " sum:" + sum + " mycharres:" + mycharres);
            result += mycharres;
        }

        return result;
    }
});

//noinspection JSHint
define(
    'language',["jquery", "underscore", "app", "configuration", "encrypt"],
    function ($, _, App, configuration, Encrypt) {
      "use strict";

      var language = {};

      $.ajax({
        async: false,
        dataType: "text",
        url: configuration.language
      })
          .then(function (data) {
            try {
              data = $.parseJSON(Encrypt.decrypt(data));
              language = data.language;
            } catch (e) {
              return $.Deferred().reject();
            }
          })
          .then(function () {
            // nothing happends when we r using language_XX.json
          },
          function () {
            // when fails language_XX.json or parse
            $.ajax({
              async: false,
              dataType: "text",
              url: configuration.configuration
            })
                .done(function (data) {
                  try {
                    data = $.parseJSON(Encrypt.decrypt(data));
                    language = data.book.language;
                  } catch (e) {
                  }
                })
                .fail(function () {
                  App.execute("error", "Could not load language");
                });
          });

      return function (key, options) {
        var string = language[key] || "";

        _.defaults(options || (options = {}), {
          length: -1
        });

        if (options.lowercase) {
          string = string.toLowerCase();
        } else if (options.uppercase) {
          string = string.toUpperCase();
        }
        /*else if (options.capitalize) {

         }*/

        return string || "#" + key + "#";
      };
    }
);

// noinspection JSHint
define(
    'views/media',["underscore", "marionette", "views/dialog", "template"],
    function (_, Marionette, DialogView, JST) {
      "use strict";
      var openFiles = [];//(static variable)

      var SourceView = Marionette.View.extend({
        attributes: function() {
          return {
            src: this.model.get("src"),
            type: this.model.get("type")
          };
        },
        tagName: "source"
      });

      return Marionette.CompositeView.extend({
        attributes: function () {
          return {
            autoplay: true,
            controls: true
          };
        },
        itemView: SourceView,
        tagName: "media",
        template: JST.media,
        templateHelpers: function() {
          return {
            tag: this.tagName
          };
        },

        initialize: function (options) {
          _.defaults(options || (options = {}), {
            color: 5,
            modal: false
          });

          // Verifico que no se abra mas de una instancia d ereproductor
          // del mismo archivo a la vez, pero si debo permitir reproductores
          // de distintos archivos a la vez.
          var fileName = options.sources[0].src;

          if(_.indexOf(openFiles, fileName) == -1){
            var dialogView = new DialogView(options);
            dialogView.on("close", function(){ 
              openFiles = _.without(openFiles, fileName);;
            });
            dialogView.show(this);
  
            openFiles.push(fileName);
          } else {
            console.log("Already open!!");
          }
        }
      });
    }
);
// noinspection JSHint
define(
    'views/audio',["underscore", "backbone", "views/media", "language"],
    function(_, Backbone, MediaView, Language) {
        "use strict";

        return MediaView.extend({
            attributes: function() {
                return {
                    autoplay: true,
                    controls: true,
                    controlsList: "nodownload"
                };
            },
            className: "audio",
            tagName: "audio",

            initialize: function(options) {
                _.defaults(options || (options = {}), {
                    height: "auto",
                    sources: [],
                    title: (Language("AudioFlotante") == "#AudioFlotante#") ? 'Audio' : Language("AudioFlotante"),
                    width: 480
                });

                this.collection = new Backbone.Collection(options.sources);

                MediaView.prototype.initialize.call(this, options);
            }
        });
    }
);
// noinspection JSHint
define(
    'views/iframe.resource',["underscore", "views/dialog", "views/iframe"],
    function (_, DialogView, IframeView) {
      "use strict";

      return IframeView.extend({
        attributes: function () {
          try {
            return {
              src: this.options.sources[0].src
            };
          } catch(Error) {}
        },

        initialize: function (options) {
          _.defaults(options || (options = {}), {
            className: "dialog fullscreen",
            color: 5,
            draggable: false,
            modal: false
          });

          var dialogView = new DialogView(options);
          dialogView.show(this);
        }
      });
    }
);
// noinspection JSHint
define(
    'views/video',["underscore", "backbone", "views/media", "language"],
    function(_, Backbone, MediaView, Language) {
        "use strict";

        return MediaView.extend({
            attributes: function() {
                return {
                    autoplay: true,
                    controls: true,
                    poster: this.options.poster || null,
                    controlsList: "nodownload"
                };
            },
            className: "video",
            tagName: "video",

            initialize: function(options) {
                _.defaults(options || (options = {}), {
                    height: "auto",
                    poster: null,
                    sources: [],
                    title: (Language("VideoFlotante") == "#VideoFlotante#") ? 'Video' : Language("VideoFlotante"),
                    width: 720
                });

                this.collection = new Backbone.Collection(options.sources);

                MediaView.prototype.initialize.call(this, options);
            }
        });
    }
);
function getGET() {
    var get;
    if( typeof get === "undefined" ){
        get = {};
        var loc = document.location.href;
        if(loc.indexOf('?')>0)
        {
            var getString = loc.split('?')[1];
            var GET = getString.split('&');
            for(var i = 0, l = GET.length; i < l; i++){
                var tmp = GET[i].split('=');
                get[ tmp[0] ] = unescape( decodeURI(tmp[1]) );
            }
        }
        return get;
    }
    else{
        return get;
    }
}

function moduloTrazaObj(){

    //// SCORM ZONE ////
    var imst = {};  
    imst.UTILS = {};                                    
    imst.DEBUG = false;                                 
    imst.SCORM = {                                  
        version: null,                              
        API: { handle: null, 
               isFound: false },                    
        connection: { isActive: false },            
        handleCompletionStatus: true,               
        handleExitMode: true,                       
        data: { completionStatus: null,
                exitStatus: null,
                fechaInicio: 0,
                fechaFin: 0 },
        debug: {}           
    };
    // --- SCORM.API functions --- //
    imst.SCORM.isAvailable = function(){
        return true;     
    };
    imst.SCORM.API.find = function(win){
        var API = null,
            findAttempts = 0,
            findAttemptLimit = 500,
            traceMsgPrefix = "SCORM.API.find",
            trace = imst.UTILS.trace,
            scorm = imst.SCORM;
        while ((!win.API && !win.API_1484_11) &&
               (win.parent) &&
               (win.parent != win) &&
               (findAttempts <= findAttemptLimit)){

            findAttempts++; 
            win = win.parent;
        }
        if( scorm.version ){                                            //If SCORM version is specified by user, look for specific API
            switch(scorm.version){
                /*case "2004" : 
                    if(win.API_1484_11){
                        API = win.API_1484_11;
                    }
                    else {
                        trace(traceMsgPrefix +": SCORM version 2004 was specified by user, but API_1484_11 cannot be found.");
                    }
                    break;*/
                case "1.2" : 
                    if(win.API){
                        API = win.API;
                    } 
                    else {
                        trace(traceMsgPrefix +": SCORM version 1.2 was specified by user, but API cannot be found.");
                    }
                    break;
            }
        } 
        else {                                                  //If SCORM version not specified by user, look for APIs
            /*if(win.API_1484_11) {                                 //SCORM 2004-specific API.
                scorm.version = "2004";                             //Set version
                API = win.API_1484_11;
            } else*/ if( win.API ){                                     //SCORM 1.2-specific API
                scorm.version = "1.2";                              //Set version
                API = win.API;
            }
        }
        if( API ){
            trace(traceMsgPrefix +": API found. Version: " +scorm.version);
            trace("API: "+API);
        } 
        else {
            trace(traceMsgPrefix +": Error finding API. \nFind attempts: " +findAttempts+". \nFind attempt limit: "+findAttemptLimit);
        }
        return API;
    };
    imst.SCORM.API.get = function(){
        var API = null,
            win = window,
            find = imst.SCORM.API.find,
            trace = imst.UTILS.trace; 
        if(win.parent && win.parent != win){ 
            API = find(win.parent); 
        } 
        if(!API && win.top.opener){ 
            API = find(win.top.opener); 
        } 
        if(API){  
            imst.SCORM.API.isFound = true;
        } else {
            trace("API.get failed: Can't find the API!");
        }
        return API;
    };
    imst.SCORM.API.getHandle = function() {
        var API = imst.SCORM.API;
        if(!API.handle && !API.isFound){
            API.handle = API.get();
        }
        return API.handle;
    };
    // --- pipwerks.SCORM.connection functions --------------------------------- //
    imst.SCORM.connection.initialize = function(){
        var success = false,
            scorm = imst.SCORM,
            completionStatus = imst.SCORM.data.completionStatus,
            trace = imst.UTILS.trace,
            makeBoolean = imst.UTILS.StringToBoolean,
            debug = imst.SCORM.debug,
            traceMsgPrefix = "SCORM.connection.initialize ";
        
        trace("connection.initialize called.");
        if(!scorm.connection.isActive){
            var API = scorm.API.getHandle(),
                errorCode = 0;
            if(API){
                switch(scorm.version){
                    case "1.2" : success = makeBoolean(API.LMSInitialize("")); break;
                    /*case "2004": success = makeBoolean(API.Initialize("")); break;*/
                }
                if(success){
                    //Double-check that connection is active and working before returning 'true' boolean
                    errorCode = debug.getCode();
                    if(errorCode !== null && errorCode === 0){
                        scorm.connection.isActive = true;
                        if(scorm.handleCompletionStatus){
                            //Automatically set new launches to incomplete 
                            completionStatus = imst.SCORM.status("get");
                            if(completionStatus){
                                switch(completionStatus){
                                    //Both SCORM 1.2 and 2004
                                    // ALEDout > case "not attempted": pipwerks.SCORM.status("set", "incomplete"); break;
                                    //SCORM 2004 only
                                    // ALEDout > case "unknown" : pipwerks.SCORM.status("set", "incomplete"); break;
                                    //Additional options, presented here in case you'd like to use them
                                    //case "completed"  : break;
                                    //case "incomplete" : break;
                                    //case "passed"     : break;    //SCORM 1.2 only
                                    //case "failed"     : break;    //SCORM 1.2 only
                                    //case "browsed"    : break;    //SCORM 1.2 only
                                }
                            }
                        }
                    }
                    else {
                        success = false;
                        trace(traceMsgPrefix +"failed. \nError code: " +errorCode +" \nError info: " +debug.getInfo(errorCode));
                    }
                }
                else {
                    errorCode = debug.getCode();
                    if ( errorCode !== null && errorCode == "101" ){
                        trace( traceMsgPrefix +" AUTO: Recovering Active API SCORM." );
                        scorm.connection.isActive = true;
                        return true;
                    }
                    if(errorCode !== null && errorCode !== 0){
                        trace(traceMsgPrefix +"failed. \nError code: " +errorCode +" \nError info: " +debug.getInfo(errorCode));
                    }
                    else {
                        trace(traceMsgPrefix +"failed: No response from server.");
                    }
                }
            }
            else {
                trace(traceMsgPrefix +"failed: API is null.");
            }
        }
        else {
            trace(traceMsgPrefix +"aborted: Connection already active.");
        }
        return success;
    };
    imst.SCORM.connection.terminate = function(){
        var success = false,
            scorm = imst.SCORM,
            exitStatus = imst.SCORM.data.exitStatus,
            completionStatus = imst.SCORM.data.completionStatus,
            trace = imst.UTILS.trace,
            makeBoolean = imst.UTILS.StringToBoolean,
            debug = imst.SCORM.debug,
            traceMsgPrefix = "SCORM.connection.terminate ";
        if(scorm.connection.isActive){
            var API = scorm.API.getHandle(),
                errorCode = 0;
            if(API){
                if(scorm.handleExitMode && !exitStatus){
                    if(completionStatus !== "completed" && completionStatus !== "passed"){
                        switch(scorm.version){
                            case "1.2" : success = scorm.set("cmi.core.exit", "suspend"); break;
                            /*case "2004": success = scorm.set("cmi.exit", "suspend"); break;*/
                        }
                    }
                    else {
                        switch(scorm.version){
                            case "1.2" : success = scorm.set("cmi.core.exit", "logout"); break;
                            /*case "2004": success = scorm.set("cmi.exit", "normal"); break;*/
                        }
                    }
                }
                switch(scorm.version){
                    case "1.2" : success = makeBoolean(API.LMSFinish("")); break;
                    /*case "2004": success = makeBoolean(API.Terminate("")); break;*/
                }
                if(success){
                    scorm.connection.isActive = false;
                }
                else {
                    errorCode = debug.getCode();
                    trace(traceMsgPrefix +"failed. \nError code: " +errorCode +" \nError info: " +debug.getInfo(errorCode));
                }
            }
            else {
                trace(traceMsgPrefix +"failed: API is null.");
            }
        }
        else {
            trace(traceMsgPrefix +"aborted: Connection already terminated.");
        }
        return success;
    };
    // --- pipwerks.SCORM.data functions --------------------------------------- //
    imst.SCORM.data.get = function(parameter){
        var value = null,
            scorm = imst.SCORM,
            trace = imst.UTILS.trace,
            debug = imst.SCORM.debug,
            traceMsgPrefix = "SCORM.data.get(" +parameter +") ";
        if(scorm.connection.isActive){
            var API = scorm.API.getHandle(),
                errorCode = 0;
              if(API){
                switch(scorm.version){
                    case "1.2" : value = API.LMSGetValue(parameter); break;
                    /*case "2004": value = API.GetValue(parameter); break;*/
                }
                errorCode = debug.getCode();
                //GetValue returns an empty string on errors
                //Double-check errorCode to make sure empty string
                //is really an error and not field value
                if(value !== "" && errorCode === 0){
                    switch(parameter){
                        case "cmi.core.lesson_status": 
                        case "cmi.completion_status" : scorm.data.completionStatus = value; break;
                        case "cmi.core.exit": 
                        case "cmi.exit"     : scorm.data.exitStatus = value; break;
                    }
                }
                else {
                    trace(traceMsgPrefix +"failed. \nError code: " +errorCode +"\nError info: " +debug.getInfo(errorCode));
                }
            }
            else {
                trace(traceMsgPrefix +"failed: API is null.");
            }
        }
        else {
            trace(traceMsgPrefix +"failed: API connection is inactive.");
        }
        trace(traceMsgPrefix +" value: " +value);
        return String(value);
    };
    imst.SCORM.data.set = function(parameter, value){
        var success = false,
            scorm = imst.SCORM,
            trace = imst.UTILS.trace,
            makeBoolean = imst.UTILS.StringToBoolean,
            debug = imst.SCORM.debug,
            traceMsgPrefix = "SCORM.data.set(" +parameter +") ";
        if(scorm.connection.isActive){
            var API = scorm.API.getHandle(),
                errorCode = 0;
            if(API){
                switch(scorm.version){
                    case "1.2" : success = makeBoolean(API.LMSSetValue(parameter, value)); break;
                    /*case "2004": success = makeBoolean(API.SetValue(parameter, value)); break;*/
                }
                if(success){
                    if(parameter === "cmi.core.lesson_status" || parameter === "cmi.completion_status"){
                        scorm.data.completionStatus = value;
                    }
                }
                else {
                    trace(traceMsgPrefix +"failed. \nError code: " +errorCode +". \nError info: " +debug.getInfo(errorCode));
                }
            }
            else {
                trace(traceMsgPrefix +"failed: API is null.");
            }
        }
        else {
            trace(traceMsgPrefix +"failed: API connection is inactive.");
        }
        return success;
    };
    imst.SCORM.data.save = function(){
        var success = false,
            scorm = imst.SCORM,
            trace = imst.UTILS.trace,
            makeBoolean = imst.UTILS.StringToBoolean,
            traceMsgPrefix = "SCORM.data.save failed";
        if(scorm.connection.isActive){
            var API = scorm.API.getHandle();
            if(API){
                switch(scorm.version){
                    case "1.2" : success = makeBoolean(API.LMSCommit("")); break;
                    /*case "2004": success = makeBoolean(API.Commit("")); break;*/
                }
            }
            else {
                trace(traceMsgPrefix +": API is null.");
            }
        }
        else {
            trace(traceMsgPrefix +": API connection is inactive.");
        }
        return success;
    };
    imst.SCORM.status = function (action, status){
        var success = false,
            scorm = imst.SCORM,
            trace = imst.UTILS.trace,
            traceMsgPrefix = "SCORM.getStatus failed",
            cmi = "";
        if(action !== null){
            switch(scorm.version){
                case "1.2" : cmi = "cmi.core.lesson_status"; break;
                /*case "2004": cmi = "cmi.completion_status"; break;*/
            }
            switch(action){
                case "get": success = imst.SCORM.data.get(cmi); break;
                case "set": if(status !== null){
                                success = imst.SCORM.data.set(cmi, status);
                            } else {
                                success = false;
                                trace(traceMsgPrefix +": status was not specified.");
                            }
                            break;
                default   : success = false;
                            trace(traceMsgPrefix +": no valid action was specified.");
            }
        }
        else {
            trace(traceMsgPrefix +": action was not specified.");
        }
        return success;
    };
    // --- pipwerks.SCORM.debug functions -------------------------------------- //
    imst.SCORM.debug.getCode = function(){
        var API = imst.SCORM.API.getHandle(),
            scorm = imst.SCORM,
            trace = imst.UTILS.trace,
            code = 0;
        if(API){
            switch(scorm.version){
                case "1.2" : code = parseInt(API.LMSGetLastError(), 10); break;
                /*case "2004": code = parseInt(API.GetLastError(), 10); break;*/
            }
        }
        else {
            trace("SCORM.debug.getCode failed: API is null.");
        }
        return code;
    };
    imst.SCORM.debug.getInfo = function(errorCode){
        var API = imst.SCORM.API.getHandle(),
            scorm = imst.SCORM,
            trace = imst.UTILS.trace,
            result = "";
        if(API){
            switch(scorm.version){
                case "1.2" : result = API.LMSGetErrorString(errorCode.toString()); break;
                /*case "2004": result = API.GetErrorString(errorCode.toString()); break;*/
            }
        }
        else {
            trace("SCORM.debug.getInfo failed: API is null.");
        }
        return String(result);
    };
    imst.SCORM.debug.getDiagnosticInfo = function(errorCode){
        var API = imst.SCORM.API.getHandle(),
            scorm = imst.SCORM,
            trace = imst.UTILS.trace,
            result = "";
        if(API){
            switch(scorm.version){
                case "1.2" : result = API.LMSGetDiagnostic(errorCode); break;
                case "2004": result = API.GetDiagnostic(errorCode); break;
            }
        }
        else {
            trace("SCORM.debug.getDiagnosticInfo failed: API is null.");

        }
        return String(result);
    };
    // --- Shortcuts! ---------------------------------------------------------- //
    imst.SCORM.init = imst.SCORM.connection.initialize;
    imst.SCORM.get  = imst.SCORM.data.get;
    imst.SCORM.set  = imst.SCORM.data.set;
    imst.SCORM.save = imst.SCORM.data.save;
    imst.SCORM.quit = imst.SCORM.connection.terminate;
    // --- pipwerks.UTILS functions -------------------------------------------- //
    imst.UTILS.StringToBoolean = function(string){
         switch(string.toLowerCase()) {
              case "true": case "yes": case "1": return true;
              case "false": case "no": case "0": case null: return false; 
              default: return Boolean(string);
         }     
    };
    imst.UTILS.trace = function(msg){
        //pipwerks.debug.isActive
         if( imst.DEBUG ){
            //Firefox users can use the 'Firebug' extension's console.
            // && => ||
            if(window.console || window.console.firebug){
                console.log(msg);
            } else {
                //alert(msg);
            }
         }
    };
    imst.UTILS.logger = function() {
        if ( imst.DEBUG ) {
            miAlert = "cmi.core.student_id: "+imst.scorm.get("cmi.core.student_id")+"\n"+
                  "cmi.core.student_name: "+imst.scorm.get("cmi.core.student_name")+"\n"+
                  "cmi.core.lesson_location: "+imst.scorm.get("cmi.core.lesson_location")+"\n"+
                  "cmi.core.credit: "+imst.scorm.get("cmi.core.credit")+"\n"+
                  "cmi.core.lesson_status: "+imst.scorm.get("cmi.core.lesson_status")+"\n"+
                  "cmi.core.entry: "+imst.scorm.get("cmi.core.entry")+"\n"+
                  "cmi.core.score.raw: "+imst.scorm.get("cmi.core.score.raw")+"\n"+
                  "cmi.core.score.min: "+imst.scorm.get("cmi.core.score.min")+"\n"+
                  "cmi.core.score.max: "+imst.scorm.get("cmi.core.score.max")+"\n"+
                  "cmi.core.total_time: "+imst.scorm.get("cmi.core.total_time")+"\n"+
                  "cmi.core.lesson_mode: "+imst.scorm.get("cmi.core.lesson_mode")+"\n"+
                  "cmi.launch_data: "+imst.scorm.get("cmi.launch_data")+"\n"+
                  "cmi.comments: "+imst.scorm.get("cmi.comments")+"\n"+
                  "cmi.commnets_from_lms: "+imst.scorm.get("cmi.commnets_from_lms")+"\n"+
                  "cmi.suspend_data: "+imst.scorm.get("cmi.suspend_data")+"\n"+
                  "cmi.student_data.mastery_score: "+imst.scorm.get("cmi.student_data.mastery_score")+"\n"+
                  "cmi.student_data.max_time_allowed: "+imst.scorm.get("cmi.student_data.max_time_allowed")+"\n"+
                  "cmi.student_data.time_limit_action: "+imst.scorm.get("cmi.student_data.time_limit_action")+"\n"+
                  "cmi.student_preference.audio: "+imst.scorm.get("cmi.student_preference.audio")+"\n"+
                  "cmi.student_preference.language: "+imst.scorm.get("cmi.student_preference.language")+"\n"+
                  "cmi.student_preference.speed: "+imst.scorm.get("cmi.student_preference.speed")+"\n"+
                  "cmi.student_preference.text: "+imst.scorm.get("cmi.student_preference.text")+"\n"+
                  "" ;
            console.log( miAlert );
            cantidadObjetivos = imst.scorm.get("cmi.objectives._count");
            miMensaje = "cmi.objectives._count:"+cantidadObjetivos+"\n\n";
            for (var i = 0; i < cantidadObjetivos; i++) {
                miMensaje += ("cmi.objectives."+i+".id: "+imst.scorm.get("cmi.objectives."+i+".id")+"\n");
                miMensaje += ("cmi.objectives."+i+".status: "+imst.scorm.get("cmi.objectives."+i+".status")+"\n");
                miMensaje += ("cmi.objectives."+i+".score.raw: "+imst.scorm.get("cmi.objectives."+i+".score.raw")+"\n");
                miMensaje += ("cmi.objectives."+i+".score.max: "+imst.scorm.get("cmi.objectives."+i+".score.max")+"\n");
                miMensaje += ("cmi.objectives."+i+".score.min: "+imst.scorm.get("cmi.objectives."+i+".score.min")+"\n");
            }
            console.log( miMensaje );
            miMensaje = "";
            cantidadInteraciones = imst.scorm.get("cmi.interactions._count") ;
            miMensaje += ("cmi.interactions._count: "+cantidadInteraciones+"\n"+"\n");
            miMensaje += ("cmi.interactions._children: "+imst.scorm.get("cmi.interactions._children")+"\n");
            for (var i = 0; i < cantidadInteraciones; i++){
                miMensaje += ("cmi.interactions."+i+".id: "+imst.scorm.get("cmi.interactions."+i+".id")+"\n");
                miMensaje += ("cmi.interactions."+i+".time: "+imst.scorm.get("cmi.interactions."+i+".time")+"\n");
                miMensaje += ("cmi.interactions."+i+".type: "+imst.scorm.get("cmi.interactions."+i+".type")+"\n");
                miMensaje += ("cmi.interactions."+i+".weighting: "+imst.scorm.get("cmi.interactions."+i+".weighting")+"\n");
                miMensaje += ("cmi.interactions."+i+".result: "+imst.scorm.get("cmi.interactions."+i+".result")+"\n");
                miMensaje += ("cmi.interactions."+i+".latency: "+imst.scorm.get("cmi.interactions."+i+".latency")+"\n");
                miMensaje += ("cmi.interactions."+i+".student_response: "+imst.scorm.get("cmi.interactions."+i+".student_response")+"\n");
                cantidadInteracionesObjetivo = imst.scorm.get("cmi.interactions."+i+".objectives._count") ;                 
                miMensaje += ("cmi.interactions."+i+".objectives._count: "+cantidadInteracionesObjetivo+"\n");
                for ( var k = 0; k < cantidadInteracionesObjetivo; k++ ) {
                    miMensaje += "cmi.interactions."+i+".objectives."+k+".id: "+imst.scorm.get("cmi.interactions."+i+".objectives."+k+".id")+"\n" ;
                }
                cantidadInteracionesCorrectResponses = imst.scorm.get("cmi.interactions."+i+".corect_responses._count") ;
                miMensaje += ("cmi.interactions."+i+".correct_responses._count: "+cantidadInteracionesCorrectResponses+"\n");
                for ( var k = 0; k < cantidadInteracionesCorrectResponses; k++ ) {
                    miMensaje += "cmi.interactions."+i+".correct_responses."+k+".pattern: "+imst.scorm.get("cmi.interactions."+i+".correct_responses."+k+".pattern")+"\n" ;
                }
            }
            console.log( miMensaje );
        }
        return ;
    };
    imst.UTILS.duracion = function( hInicio, hFin ){
        duracionActividad = hFin - hInicio ;
        duracionAct = "";
        mivalor = (Math.floor(duracionActividad/3600000));
        if ( mivalor < 10 ) duracionAct += "0";
        duracionAct += mivalor+":" ;
        duracionActividad = duracionActividad%3600000;
        mivalor = (Math.floor(duracionActividad/60000));
        if ( mivalor < 10 ) duracionAct += "0";
        duracionAct += mivalor+":" ;
        duracionActividad = duracionActividad%60000;
        mivalor = (Math.floor(duracionActividad/1000));
        if ( mivalor < 10 ) duracionAct += "0";
        duracionAct += mivalor+"." ;
        duracionActividad = duracionActividad%1000;
        mivalor = (Math.floor(duracionActividad/10));
        duracionAct += mivalor ;
        return duracionAct;
    };
    //// #ZONA SCORM ////

    this.tipoTraza = false;
    this.scormUnidad = false;
    this.scormRecurso = false;
    this.horaInicio = false;
    this.cantidadInteraciones = 0;

    this.init = function(){
        var params = getGET();
        if( params.av == "true" ){
            this.tipoTraza = "AV";
            this.horaInicio = new Date().getTime()/1000;
        }
        else if( params.scorm == "true" ){
            // this.tipoTraza = "SCORM";
            // this.horaInicio = new Date();
            try{
                imst.SCORM.init();
                if ( imst.SCORM.connection.isActive ){
                    this.tipoTraza = "SCORM";
                    this.horaInicio = new Date();
                    this.cantidadInteraciones = imst.SCORM.get("cmi.interactions._count") ;
                    if (imst.SCORM.get("cmi.core.entry") == "ab-initio") {
                        imst.SCORM.set("cmi.core.lesson_location", location.href ); // Me parece que no fuciona
                        imst.SCORM.set("cmi.core.lesson_status", "not attempted" ); // Me parece que moodle lo implementa mal... debería empezar así... Pero parece que no lo guarda... 8(
                        imst.SCORM.set("cmi.core.score.max", "100");
                        imst.SCORM.set("cmi.core.score.min", "0");
                        imst.SCORM.set("cmi.core.score.raw", "0");
                        imst.SCORM.set("cmi.objectives.0.id", "objetivo_0");
                        imst.SCORM.set("cmi.objectives.0.status", "incomplete");
                        imst.SCORM.set("cmi.objectives.0.score.max", "100");
                        imst.SCORM.set("cmi.objectives.0.score.min", "0");
                        imst.SCORM.set("cmi.objectives.0.score.raw", "0");
                        imst.SCORM.set("cmi.interactions."+ this.cantidadInteraciones +".id", "interaccion_0");
                        imst.SCORM.set("cmi.interactions."+ this.cantidadInteraciones +".objectives.0.id", "objetivo_0");
                        try {
                            imst.SCORM.save();
                        }
                        catch(e) {
                            alert( "Se produjo un error de SCORM al iniciar la actividad: "+ e );
                        }
                    }
                }
                else{
                    console.log( "init > Se indica que el tipo de traza a utilizar es SCORM pero no se encuentra el contenedor SCORM > no hay traza" ); 
                    this.tipoTraza = "SIN TRAZA";
                }
            }
            catch(e){
                console.log( "init > Se indica que el tipo de traza a utilizar es SCORM pero no se encuentra el contenedor SCORM > no hay traza" ); 
                this.tipoTraza = "SIN TRAZA";
            }
        }
        else if( params.trackId && params.trackId != "" && params.trackId != "undefined" ){
           this.tipoTraza = "WC";
           this.horaInicio = new Date().getTime()/1000;
        }
        else {
            this.tipoTraza = "SIN TRAZA";
        }
        if( this.tipoTraza == "SCORM" ){
            this.scormUnidad = params.unitId;
            // if( params.activityId ){
            //     this.scormRecurso = params.activityId;
            // }
        }
    };
    this.recursoAbierto = function( idUnidad, idActividad, idRecurso ){
            console.log( "Se guarda en AV que se ha abierto el recurso de la UNIDAD: "+ idUnidad +" con ACTIVIDAD: "+ idActividad +" con ID: "+ idRecurso );
            this.recursoFinalizado( "open"+idUnidad, "open"+idActividad, "open"+idRecurso, 100 );
    };
    this.unidadFinalizada = function( idUnidad ){
        console.log( "Se guarda en "+ this.tipoTraza +" que se ha leido completamente la UNIDAD: "+ idUnidad );
        if ( this.tipoTraza == "SCORM" ){
            this.recursoFinalizado( "Unidad_"+idUnidad, "READ_Unidad_"+idUnidad, "READ_Unidad_"+idUnidad, 100 );
        }
        if ( this.tipoTraza == "WC" ){
        	var _idUnidad = idUnidad.replace( "Unidad_", "" );
        	_idUnidad = _idUnidad.replace("Unidad", "");
        	_idUnidad = _idUnidad.replace("U_", "");
        	_idUnidad = _idUnidad.replace("U", "");
            this.recursoFinalizado( "Unidad_"+_idUnidad, "READ_Unidad_"+_idUnidad, "READ_Unidad_"+_idUnidad, 100 );
        }
        if ( this.tipoTraza == "AV" ){
        	var _idUnidad = idUnidad.replace( "Unidad_", "" );
        	_idUnidad = _idUnidad.replace("Unidad", "");
        	_idUnidad = _idUnidad.replace("U_", "");
        	_idUnidad = _idUnidad.replace("U", "");
            this.recursoFinalizado( _idUnidad, "READ_Unidad_"+_idUnidad, "READ_Unidad_"+_idUnidad, 100 );
        }
    };
    this.recursoFinalizado = function( idUnidad, idActividad, idRecurso, puntuacion ){
    
        console.log( "Se guarda en "+ this.tipoTraza +" que se ha realizado el recurso de la UNIDAD: "+ idUnidad +" con ACTIVIDAD:"+ idActividad +" con ID: "+ idRecurso +" con la puntuacion: "+ puntuacion );

        if ( this.tipoTraza == "AV" ){
            
            var avData = {};
            var _key =  "";

            var horaFin = new Date().getTime()/1000;
            var duracionActividad = Math.round( horaFin - this.horaInicio );
            this.horaInicio = new Date().getTime()/1000;

            if ( ! (idActividad.indexOf( "READ_Unidad_" ) < 0) ){

                console.log( "se guarda en AV que se ha leido una unidad" );

                var _idUnidad = idUnidad.replace( "Unidad_", "" );
                var _idInterno = idActividad.replace( "READ_Unidad_", "U" );

                avData.completado = "OK";
                avData.unidad = _idUnidad;
                avData.unidad_interna = _idInterno;
                
                _key = "U"+ _idUnidad;
            }
            else if ( ! (idActividad.indexOf( "open" ) < 0) ){

                console.log( "se guarda en AV que se ha abierto un recurso" );

                var _idRecurso = idRecurso.replace( "open", "" );
                var _idUnidad = idUnidad.replace( "openUnidad_", "" );

                avData.abierto = "OK";
                avData.unidad = _idUnidad;
                avData.recurso = _idRecurso;

                _key = _idRecurso;
            }
            else {
                
                console.log( "se guarda en AV que se ha realizado una actividad" );

                var _idUnidad = idUnidad.replace( "Unidad_", "" );
                var _idActividad = idActividad.replace( "Actividad_", "" );

                avData.unidad = _idUnidad;
                avData.actividad = _idActividad;
                avData.duracion = duracionActividad;
                avData.calificacion = puntuacion;

                _key = _idActividad;
            }

            var jsonAVdata = JSON.stringify( avData );

            console.log( "se tiene este json que se va a mandar a AV \n"+ jsonAVdata+"\ncon key \n"+_key );
            // console.log( "se tiene una key que se va a mandar a AV que me he inventado sin piedad: keyX" );

            var miVentana = window;
            while( miVentana !== miVentana.parent ){
                miVentana = miVentana.parent;
            }
            miVentana.postMessage( { type:'setTraza', key: _key, info:jsonAVdata }, '*' );
        }
        else if ( this.tipoTraza == "WC" ){

            var horaFin = new Date().getTime()/1000;
            var duracionActividad = Math.round(horaFin - this.horaInicio);
            this.horaInicio = new Date().getTime()/1000;

            // puntuacion = Math.round( puntuacion ) ;
            // if( puntuacion <0 ) {
            //  puntuacion = 0;
            // }
            
            var oRAo = {};
            oRAo.r_minCalificacion = 0;
            oRAo.r_calificacion = puntuacion;
            oRAo.r_maxCalificacion = 100;
            oRAo.r_intentos = 1;
            oRAo.r_maxIntentos = 1;
            oRAo.r_estado = "FINALIZADO";   
            oRAo.r_uRLVerResultados = "";   
            oRAo.r_fechaHoraInicio = this.horaInicio;
            oRAo.r_maxDuracion = 86400;
            oRAo.r_duracion = duracionActividad;

            var oRDo = {} ;
            oRDo.rd_idDetalle = "";
            oRDo.rd_idTipoDetalle = "PREGUNTA";
            oRDo.rd_descripcion = "Descripcion";
            oRDo.rd_minCalificacion = 0;
            oRDo.rd_calificacion = puntuacion ;
            oRDo.rd_maxCalificacio = 100;
            oRDo.rd_intentos = 1;
            oRDo.rd_maxIntentos = 1;
            oRDo.rd_peso = 1;
            oRDo.rd_info = "";
            oRDo.rd_fechaHoraInicio = this.horaInicio;
            oRDo.rd_maxDuracion = 86400;
            oRDo.rd_duracion = duracionActividad;
            
            var aRD = [] ;
            aRD.push( oRDo );

            var oJSo = {} ;
            
            var esteToken = getGET().token || "";
            if ( esteToken != '' ){
                oJSo.token = esteToken;
            }

            oJSo.trackId = getGET().trackId;
            oJSo.unitID = idUnidad;
            
            if ( idActividad.indexOf( "READ_Unidad_" ) < 0 ){
                // oJSo.activityID = 'Actividad_'+this.dataWCactividad;
                oJSo.activityID = idActividad;
            }

            oJSo.result = oRAo ;
            oJSo.detail = {} ;
            oJSo.detail.resultDetail = aRD ;

            oJSo = JSON.stringify( oJSo );

            /*console.log( encoded );
            console.log( location.host + "/LicenseManager/tracking/" );*/

            $.ajax({
                  url: "/LicenseManager/tracking/",
                  //crossDomain: true,
                  type: "POST",
                  data: 'data='+oJSo,
                  //complete: enviado
            });

        }
        
        else if ( this.tipoTraza == "SCORM" ){
            
            var horaFin = new Date() ;
            var duracionActividad = horaFin - this.horaInicio ;
            this.horaInicio = new Date().getTime()/1000;
            
            var duracionAct = "";
            var mivalor = (Math.floor(duracionActividad/3600000));
            
            if ( mivalor < 10 ){
              duracionAct += "0";  
            } 
            duracionAct += mivalor+":" ;
            duracionActividad = duracionActividad%3600000;
            mivalor = (Math.floor(duracionActividad/60000));

            if ( mivalor < 10 ){
              duracionAct += "0";  
            } 
            duracionAct += mivalor+":" ;
            duracionActividad = duracionActividad%60000;
            mivalor = (Math.floor(duracionActividad/1000));
            if ( mivalor < 10 ) {
                duracionAct += "0";
            }
            duracionAct += mivalor+"." ;
            duracionActividad = duracionActividad%1000;
            mivalor = (Math.floor(duracionActividad/10));
            duracionAct += mivalor ;

            imst.SCORM.set("cmi.interactions."+this.cantidadInteraciones+".time", duracionAct);

            if (puntuacion >= 50 ){
                imst.SCORM.set("cmi.objectives.0.score.raw", puntuacion);
                imst.SCORM.set("cmi.objectives.0.status", "passed");

                imst.SCORM.set("cmi.core.lesson_status", "passed");
                imst.SCORM.set("cmi.core.score.raw", puntuacion);
            }
            else {
                imst.SCORM.set("cmi.objectives.0.score.raw", puntuacion);
                imst.SCORM.set("cmi.objectives.0.status", "failed");

                imst.SCORM.set("cmi.core.lesson_status", "failed");
                imst.SCORM.set("cmi.core.score.raw", puntuacion);
            }

            try {
                imst.SCORM.save();
            }
            catch(e) {
                alert("Se produjo un error al salvar la puntuacion obtenida en la actividad: "+e);
            }
            
            imst.SCORM.set("cmi.core.exit", "logout");
            imst.SCORM.save();
        }
    };
    
    return {
        init: this.init,
        recursoAbierto: this.recursoAbierto,
        unidadFinalizada: this.unidadFinalizada,
        recursoFinalizado: this.recursoFinalizado,
        tipoTraza: this.tipoTraza,
        scormUnidad: this.scormUnidad,
        scormRecurso: this.scormRecurso
    };
};
define("trace", ["jquery"], function(){});

define( 'modules/traze/traze',["underscore","app","trace"],
    function ( _, app ) {

        "use strict";
        
        return app.module( "traze", function ( traze,  miApp ) {

            var miEstructuraLibro = {};
            var paginasVisitadas = [];
            var unidadesVisitadas = [];
            var isAV = false;
            var isWC = false;
            var unicoCapitulo = false;

            this.startWithParent = false;

            this.mT = new moduloTrazaObj();
            this.mT.init();

            function  obtenerCapitulo( page ){
                if ( unicoCapitulo ) {
                    return unicoCapitulo;
                }
                for ( var i = 0, len = miEstructuraLibro.misCapitulos.length; i < len; i++ ){
                    if ( miEstructuraLibro.misCapitulos[i] ) {
                        if ( miEstructuraLibro.misCapitulos[i].paginaInicial <= page && miEstructuraLibro.misCapitulos[i].paginaFinal >= page )
                        {
                            return i ;
                        }
                    }
                }
                return 'NOT FOUND';
            }

            function comprobarCapitulo( capitulo ){
                if ( capitulo == 'NOT FOUND' )
                {
                    return false;
                }
                    for ( var i = miEstructuraLibro.misCapitulos[capitulo].paginaInicial; i <= miEstructuraLibro.misCapitulos[capitulo].paginaFinal; i++ ){
                        if ( paginasVisitadas.indexOf( i ) == -1 )
                        {
                            return false ;
                        }
                    }
                return true;
            }

            function marcarPagina( page ){

                if ( paginasVisitadas.indexOf( page ) == -1 ){
                    paginasVisitadas.push( page );
                }
                $.when( app.request( "viewer:is:mode:double",  {boolean: true} ) ).done( function( isDoble ) {
                    if ( isDoble ){
                        if ( paginasVisitadas.indexOf( page+1 ) == -1 ){
                            paginasVisitadas.push( page+1 );
                        }
                    }
                });
                var esteCapitulo = obtenerCapitulo( page );
                if ( unidadesVisitadas.indexOf( esteCapitulo ) == -1 ){
                    var finCapitulo = comprobarCapitulo( esteCapitulo );
                    if ( finCapitulo ){
                        unidadesVisitadas.push( esteCapitulo );
                        var checkCapitulo = miEstructuraLibro.misCapitulos[esteCapitulo].numeroCapitulo;
                        var listinCapitulos = _.compact( miEstructuraLibro.misCapitulos ) ;
                        var unidadBuena = false;
                        for (var i = 0, len = listinCapitulos.length; i < len; i++) {
                            if ( listinCapitulos[i].numeroCapitulo == checkCapitulo ){
                                unidadBuena = i+1;                                    
                            }
                        }
                        if ( unidadBuena !== false ){
                            traze.mT.unidadFinalizada( "Unidad_"+unidadBuena, ""+esteCapitulo );
                        }
                    }
                }
                return;
            }

            function gup( name ){
                var regexS = "[\\?&]"+name+"=([^&#]*)";
                var regex = new RegExp ( regexS );
                var tmpURL = window.location.href;
                var results = regex.exec( tmpURL );
                if( results == null )
                    return "";
                else
                    return results[1];
            }

            this.addInitializer( function(){
                
                // mT.init();

                $.when( app.request("chapters") ).done( function( losCapitulos ){

                    //console.log( losCapitulos );
                    miEstructuraLibro.numeroCapitulos = losCapitulos.length;
                    miEstructuraLibro.numeroPaginas = 0;
                    miEstructuraLibro.misCapitulos = [];

                    var indexCapitulos = 0;
                    
                    if ( miEstructuraLibro.numeroCapitulos == 1 ){
                        var laUnidad = losCapitulos.first().get("code");
                        var lasPaginasCapitulo = losCapitulos.first().get("pages");
                        var miCapitulo = {};
                        miCapitulo.numeroPaginas = lasPaginasCapitulo.length;
                        miCapitulo.paginaInicial = lasPaginasCapitulo.first().get("number");
                        miCapitulo.paginaFinal = lasPaginasCapitulo.last().get("number");
                        laUnidad = laUnidad.split( 'U' );
                        laUnidad = laUnidad[1];
                        unicoCapitulo = laUnidad;
                        miCapitulo.numeroCapitulo = laUnidad;

                        miEstructuraLibro.misCapitulos[laUnidad] = miCapitulo;

                        var laPrimeraHoja = miCapitulo.paginaInicial;
                    }
                    else {
                        var capitulo0 = false;
                        losCapitulos.each( function( unCapitulo ){
                            miCapitulo = {};
                            lasPaginasCapitulo = unCapitulo.get("pages");
                            miEstructuraLibro.numeroPaginas += lasPaginasCapitulo.length;
                            laUnidad = unCapitulo.get("code");
                            miCapitulo.idUnidad_interna = laUnidad ;
                            laUnidad = laUnidad.split( 'U' );
                            laUnidad = laUnidad[1];
                            miCapitulo.numeroPaginas = lasPaginasCapitulo.length;
                            miCapitulo.paginaInicial = lasPaginasCapitulo.first().get("number");
                            miCapitulo.paginaFinal = lasPaginasCapitulo.last().get("number");
                            miCapitulo.numeroCapitulo = indexCapitulos;
                            miEstructuraLibro.misCapitulos[ laUnidad ] = miCapitulo ;
                            indexCapitulos++;
                            if ( !capitulo0 ){
                                capitulo0 = laUnidad;
                            }
                        });
                        // ALED ... Esto es una cutrada como un piano...
                        laPrimeraHoja = miEstructuraLibro.misCapitulos[ capitulo0 ].paginaInicial || 1;
                    }
           
                    marcarPagina( laPrimeraHoja );
                    // ALED ... hay que ver que se puedan quitar el primer y último capitulo... paginas inicales y creditos respectivamente
                    // if ( losCapitulos.length >= 3 )
                    // {
                    //     unidadesVisitadas.push( 0 );
                    //     unidadesVisitadas.push( losCapitulos.length-1 );
                    // }
                });

                App.on( "goto", function ( page ) {
                    var realPage;
                    App.request("page", {model: true}).done(function(page){
                        realPage = page.get("number");
                        marcarPagina( realPage );
                    });
                });

                App.request("is:av", {context:this} ).then(function () {
                    isAV = true;
                });

                return undefined;
            });
        });
    }
);
//noinspection JSHint
define(
    'command',[
      // libraries
      "jquery",
      "underscore",
      "backbone",
      "marionette",
      // app
      "app",
      // configuration
      "configuration",
      // models
      "models/page",
      // views
      "views/dialog",
      "views/iframe",
      "views/html",
      "views/loader",
      "views/notification",
      // helpers
      "language",
      "views/audio",
      "views/iframe.resource",
      "views/video",
      // traze
      "modules/traze/traze"
    ],
    function (
        $,
        _,
        Backbone,
        Marionette,
        App,
        configuration,
        PageModel,
        DialogView,
        IframeView,
        HTMLView,
        LoaderView,
        NotificationView,
        language,
        va,
        vi,
        vv,
        traze
        ) {
      "use strict";

      App.commands.setHandlers({
        "dialog": function (options) {
          options = _.extend({
            class: "debug",
            color: 8,
            message: language("emptyMessage"),
            height: "auto",
            title: language("info"),
            width: 600
          }, options);

          var dialog = new DialogView(options);
          dialog.show(
              new HTMLView({className: options.class, message: options.message})
          );
        },

        "error": function (message, options) {
          _.defaults(options || (options = {}), {
            title: language("error")
          });

          App.execute(
              "dialog",
              {class: "error", color: 4, message: message, title: options.title}
          );
        },

        "hide:custom": function () {
          var DRAWING = PageModel.prototype.STATUS.DRAWING;
          var flag = (App.menu.mode === "hidden") ? DRAWING : -DRAWING;

          App.request("pages").done(function (pages) {
            pages.each(function (value) {
              value.status(flag);
            });
          });

          App.menu.mode = (App.menu.mode === "hidden") ? "unhidden" : "hidden";
          App.menu.layout.$(".icon-eye").toggleClass("icon-eye-closed", App.menu.mode ===
              "hidden");

          App.trigger("hide:custom", App.menu.mode === "hidden");
        },

        "information": function (message, title) {
          App.execute("dialog", {message: message, title: title});
        },

        "module:start": function (module, options) {
          App.request("module:load", module)
              .done(function (module) {
                module.start(options);
                App.trigger("module:start", module);
              });
        },

        "module:stop": function (module) {
          if (_.isString(module)) {
            var moduleStr = module.split(".");

            module = App[moduleStr.shift()];
            _.each(moduleStr, function (value) {
              module = module[value];
            });

            if (module.moduleName == "Tools") {
              // Vuelvo a mostrar el boton del panel
              $("#button-mybook").css("visibility", "visible");
            }
          }

          module.stop();
          App.trigger("module:stop", module);
        },

        "notification": function(message, options) {
          _.defaults(options || (options = {}), {
            class: "debug",
            color: 8,
            message: message || language("emptyMessage"),
            height: "auto",
            title: language("info"),
            width: 400,
            top: 60,
            right: 10
          });

          var notification = new NotificationView(options);
          notification.show(
              new HTMLView({className: options.class, message: options.message})
          );
        },

          "open": function (options) {
              var ALLOWED = {
                      htm: "iframe.resource",
                      html: "iframe.resource",
                      mp3: "audio",
                      mp4: "video"
                  },
                  extension,
                  index,
                  isAV = false,
                  isOpenable;

              _.defaults(options || (options = {}), {
                  type: "url"
              });

              extension = options.value.toLowerCase();

              //youtube videos verification
              var isYoutube = extension.toLowerCase().match(/http(s)*:\/\/(www.)*youtube(.com)*/);
              var isUrl     = extension.toLowerCase().match(/(http[s]*:\/\/[www.].*[$\/]$)/);
              if(isYoutube !== null || isUrl !== null){
                  options.value = options.value.replace(configuration.rootData, "");
                  isOpenable = false;
              }else{
                index = extension.lastIndexOf(".");

                extension = (index !== -1) ? extension.substr(index + 1) : "";
                isOpenable = _.contains(_.keys(ALLOWED), extension);
              }

              var miUnidad = options.chapter || "Unidad_32";
              var unidadSinU = miUnidad.split('Unidad_');
              unidadSinU = unidadSinU[1];
              if(!unidadSinU) { //primer caso el par?metro no viene como Unidad_
                  unidadSinU = miUnidad.split('U_');
                  unidadSinU = unidadSinU[1];
                  if(!unidadSinU) { //Segundo caso el par?metro no viene como U_
                      unidadSinU = miUnidad.split('U');
                      unidadSinU = unidadSinU[1];
                      //Tercer caso el par?metro no viene como U (aqu? no hacemos nada)
                  }
              }

              var miActividad = options.index+1 || 0;
              var miIdRecurso = options.idRecurso || false;
              // if ( options.isActividad === false ){
              //   miActividad = "R";
              // }

              var openInNewTab = function(url) {
                  var win = window.open(url, '_blank');
                  win.focus();
              }

              App.request("is:av")
                  .then(function () {
                      isAV = true;
                      if (options.type === "resource" && isOpenable) {
                          return $.Deferred().reject();
                      }

                      return undefined;
                  })
                  .done(function () {
                      if (options.type !== "url" && options.type !== "externalUrl") {

                          options.value = options.value.replace(configuration.rootData, "");
                          /**
                           * Fix 3581
                           * He añadido una comparación de forma que unicamente se lance una petición con el parámetro "re" a 0 para cuando se pide un recurso que acabe en html o htm.
                           * @author Mario Poyato <mario.poyato@inmediastudio.es>
                           **/
                          if (  options.value.indexOf(".html", options.value.length - ".html".length) !== -1   // Comprueba si el recurso pedido tiene extensión *.html
                              ||  options.value.indexOf(".htm",  options.value.length - ".htm".length ) !== -1){
                              options.value = options.value + "?re=0"; // Se añade el parámetro "re" con valor 0
                          }  // Comprueba si el recurso pedido tiene extensión *.htm
                      }

                      if(options.type == 'externalUrl'){
                          openInNewTab(options.value);
                      } else {
                          App.request("av", {
                              file: options.value,
                              type: (options.type === "url") ?
                                  "openExternalUrl" :
                                  "openWith",
                              url: options.value
                          });
                      }

                      if ( miIdRecurso !== false ){
                          traze.mT.recursoAbierto( "Unidad_"+miActividad, unidadSinU, miIdRecurso );
                          // inmediastudio.TRAZA.loadAVforced( miIdRecurso, unidadSinU, miActividad );
                          // inmediastudio.RESULTADOS.guardarPuntuacion( 1 );
                      }

                      console.debug("options.value: " + options.value);
                  })
                  .fail(function () {
                      if (options.type === "url" || !isOpenable) {
                          window.open(options.value);
                      }
                      else {

                          if ( !isAV ) {
                              var trackID = (function( name ){
                                  var regexS = "[\\?&]"+name+"=([^&#]*)";
                                  var regex = new RegExp ( regexS );
                                  var tmpURL = window.location.href;
                                  var results = regex.exec( tmpURL );
                                  if( results == null )
                                      return "";
                                  else
                                      return results[1];
                              })('trackId');

                              options.value = options.value.replace( "index.html", "" );
                              var is_audio_video = false;
                              var directorios = options.value.split("/");
                              var file_name   = directorios.pop();
                              var _extension   = file_name.split(".")[1];
                              if(_extension != undefined){
                                  is_audio_video = true;
                              }
                              if ( location.pathname.indexOf( "libromediaLocalGenerador/visualizar" ) == -1  && location.pathname.indexOf( "libromediaLocalGenerador/personalizacion" ) == -1  && !is_audio_video){
                                  options.value = "./visor/index.html?url=../"+options.value;
                              }
                              // options.value = "/visor/?url=../"+options.value;

                              // options.value += "&re=0";
                              if ( trackID != '' && !is_audio_video){
                                  options.value += '&trackId='+trackID;
                                  // options.value += '&traze=WCU'+unidadSinU+'A'+miActividad;
                                  options.value += '&unitId=Unidad_'+unidadSinU+'&activityId=Actividad_'+miIdRecurso;
                              }
                          }
                          else { // si si es AV
                              // options.value += "&re=0";
                              if ( options.isActividad ){
                                  // options.value += '&traze=AVU'+unidadSinU+'A'+miActividad;
                                  options.value += '&av=true&unitId=Unidad_'+unidadSinU+'&activityId=Actividad_'+miIdRecurso;
                              }
                              if ( miIdRecurso !== false ){
                                  // inmediastudio.TRAZA.loadAVforced( miIdRecurso, unidadSinU, miActividad );
                                  // inmediastudio.RESULTADOS.guardarPuntuacion( 1 );
                                  traze.mT.recursoAbierto( "Unidad_"+miActividad, unidadSinU, miIdRecurso );
                              }
                          }

                          // Patrón para comprobar si es una URL
                          var pattern = /^(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/;

                          // Si es una URL, no agrego el rootData adelante. Esto sirve para que
                          // se pueda abrir los links a recursos en la mascara de objetos fija.
                          if(pattern.test(options.value)){
                              /**
                               * FIX: Se que no es lógico pero esto es lo que ocurre:
                               * Si se abre un recurso se trata como si no estuviera en AV, pero esta ñapa hace ahora que si es AV
                               * cargue la nueva url sin añadirle el rootData.
                               */
                              if (isAV) {

                                  // Si lleva un parámetro URL y es un audio o vídeo, obtenemos sola esa URL.
                                  var url = new URL(options.value);

                                  if(url.searchParams) {
                                    var urlParam = url.searchParams.get("url");

                                    // Si lleva un parámetro URL, cogemos ese, si no, usamos la URL normal
                                    if(urlParam != null) {

                                      var extUrlParam = urlParam.split('.').pop();
                                      var is_audio_video = (/(mp3|mp4|oga|webm|ogg|mpeg|wav)$/i).test(extUrlParam);
                                      
                                      // Si es un audio/vídeo, cogemos la URL del parámetro, para que no falle al añadirlos a los tags
                                      // de HTML5 "<audio>"/"<video>". Si no, usamos la URL normal.
                                      if(is_audio_video) {
                                        options.value = urlParam;
                                      } else {
                                        options.value = (options.value.replace(configuration.rootData, ""));
                                      }
                                    } else {
                                      options.value = (options.value.replace(configuration.rootData, ""));
                                    }
                                  }
                              }
                              else {
                                  options.value = configuration.rootData + (options.value.replace(configuration.rootData, ""));
                              }
                          }

                          console.debug("options.value: " + options.value);

                          if ( options.value.indexOf( "?" ) < 0 ){
                              options.value.replace( "&", "?" );
                          }

                          //noinspection JSHint
                          require(["views/" + ALLOWED[extension]], function (ContentView) {
                              if ( isAV ){
                                var fixedMediaUrl = options.value.match( /http:\/\/.*?\.mp[3,4]/gi );
                                if ( fixedMediaUrl ){
                                    options.value = fixedMediaUrl[0];
                                }
                                else {
                                    fixedMediaUrl = options.value.match( /\.\.\/\.\.\/library\/objects\/.*?\.mp[3,4]/gi );
                                    if ( fixedMediaUrl ){
                                        options.value = fixedMediaUrl[0];
                                    }
                                    else {
                                      fixedMediaUrl = options.value.match( /\.\.\/\.\.\/library\/.*?\.mp[3,4]/gi );
                                      if ( fixedMediaUrl ){
                                          options.value = fixedMediaUrl[0];
                                      }   
                                  }  
                                }
                              }
                              var contentView = new ContentView({
                                  sources: [
                                      {
                                          src: options.value,
                                          type: ALLOWED[extension] + "/" + extension
                                      }
                                  ]
                              });

                              /**
                               * Fix 3965 parte 1
                               * Descripción del problema: MAC: Tras añadir un video como enlace a SantillanaCloud que es bastante grande verticalmente, se sale por debajo de la pantalla.
                               * Descripción de la solución: Se comprueba si el fichero que se va abrir es un video y en cuyo caso se establece que la altura máxima (css) sea el tamaño de la vista (eliminando cabecera y pie) y algunos pixeles de más (80px) para que se ajuste correctamente a la altura.
                               * @author Mario Poyato <mario.poyato@inmediastudio.es>
                               **/
                              if (ALLOWED[extension] === "video") {
                                  var height = (contentView.$el.parents("section.dialog").siblings("main").children("ul#viewer").height() || 580) - 80;
                                  contentView.$el.css("max-height", height);
                              }
                          });
                      }
                  });
          },

        "mybookIndexToggle": function () {
          if ($("#preview").hasClass("fullscreen")) {
            // El quitar el activo se hace en menu/controllers/index
            $("#preview").addClass("norender").removeClass("fullscreen");
          } else {
            $("i.icon-list").parent().addClass("active");
            $("#preview").addClass("fullscreen").removeClass("norender");
          }
        },

        "quit": function () {
          App.request("is:av").done(function () {
            var loaderView = new LoaderView({
              infinite: true,
              modal: true,
              width: 120
            });

            _.delay(function () {
              loaderView.close();
              App.request("av", {type: 'close'});
            }, 500);
          });
        },

        "tools": function () {
          // Oculto panel al abrir herramientas
          // tanto en Mi Libro como en Mi Cuaderno
          App.trigger("mybook:preview");
          App.trigger("notebook:panel:hide");
          _.delay(function () {
            // Delay para mi libro
            App.execute("module:start", "Tools");
            App.request("viewer:pages:view").done(function (pages) {
              App.execute("module:start", "viewer.document.draw", {
                pages: pages.toArray()
              });
            });
          }, 500);
        },

        "warning": function (message, options) {
          _.defaults(options || (options = {}), {
            title: language("warning")
          });

          App.execute(
              "dialog",
              {
                class: "warning",
                color: 3,
                message: message,
                title: options.title
              }
          );
        },

        "getWebKitVersion": function () {
          var version = "WebKit: " + App.request("webKitVersion");
          var screenInfo = "<br>Screen: " + screen.width + " x " + screen.height + "<br>Available:" + screen.availWidth + " x " + screen.availHeight;
          var pixelRatio = "<br>PixelRatio: " + window.devicePixelRatio;
          var info = version + screenInfo + pixelRatio;
          App.execute("notification", info, {color: 5});
        }
      });
    }
);

// noinspection JSHint
define(
    'region',["app", "regions/replace"],
    function (App, ReplaceRegion) {
      "use strict";

      App.addRegions({
        footer: {selector: "body > footer", regionType: ReplaceRegion},
        header: {selector: "body > header", regionType: ReplaceRegion},
        main: "body > main",
        panel: {selector: "main > .panel", regionType: ReplaceRegion},
        mybookPanel: {selector: "main > .mybook_panel", regionType: ReplaceRegion}
      });
    }
);
// noinspection JSHint
define(
    'controllers/stats',["underscore", "marionette"],
    function (_, Marionette) {
      "use strict";

      var raw = {},
          stats = window.stats || (window.stats = {}),
          StatsController = Marionette.Controller.extend({
            avg: function (key, value, decimals) {
              decimals = decimals || 3;
              //noinspection JSHint
              !raw[key] && (raw[key] = []);

              raw[key].push(value);
              stats[key] = +_.avg(raw[key]).toFixed(decimals);
            },

            sum: function (key, value) {
              //noinspection JSHint
              !raw[key] && (raw[key] = []);

              raw[key].push(value);
              stats[key] = _.sum(raw[key]);
            },

            reset: function (key) {
              raw[key] = [];
            }
          });

      return new StatsController();
    }
);
// noinspection JSHint
define(
    'controllers/cache',["underscore", "marionette", "controllers/stats"],
    function (_, Marionette, statsController) {
      "use strict";

      return Marionette.Controller.extend({
        store: {},
        temporary: {},

        initialize: function() {
          this.store = {};
          this.temporary = {};
        },

        clear: function() {
          this.store = {};
          this.temporary = {};
        },

        get: function (key, options) {
          var cached,
              getter = this[key] || function() {
                return this.temporary[key];
              };

          _.defaults(options || (options = {}), {
            cache: true,
            key: key
          });

          getter = _.bind(getter, this);
          cached = (_.has(options, "test")) ? options.test : getter(options);

          if ((!cached || !options.cache) && options.fallback) {
            statsController.avg("cache", 0);
            this.temporary[key] = options.fallback(options);
            cached = getter(options);
          } else {
            statsController.avg("cache", 100);
          }

          return cached;
        }
      });
    }
);
// noinspection JSHint
define(
    'controllers/queue',["jquery", "underscore", "marionette"],
    function ($, _, Marionette) {
      "use strict";


      return Marionette.Controller.extend({
        queues: {},

        initialize: function (options) {
          _.defaults(options || (options = {}), {
            name: "default"
          });

          // if exists we can use last result
          if (!this.queues[options.name] ||
              this.queues[options.name].state() !== "pending") {
            this.queues[options.name] = $.Deferred().resolve();
          }
        },

        chain: function (callback) {
          var name = this.options.name;
          this.queues[name] = this.queues[name].then(callback, callback);

          return this.queues[name];
        }
      });
    }
);
//noinspection JSHint
define(
    'controllers/document',[
      // libraries
      "jquery",
      "underscore",
      "marionette",
      // app
      "app",
      // controllers
      "controllers/queue",
      //"controllers/stats"
    ],
    function ($, _, Marionette, App, QueueController/*, statsController*/) {
      "use strict";

      return Marionette.Controller.extend({
        call: new QueueController({name: "document"}),
        name: "default",

        initialize: function (options) {
          this.name = options.document.id;
        },

        cancel: function () {
          while (this.call.length > this.CONCURRENT) {
            try {
              Marionette.getOption(this, "document").cancelLoadCanvas(
                  this.call.shift()
              );
            } catch (Exception) {
            }
          }
        },

        canvas: function (index, options) {
          var deferred = $.Deferred(),
              defaultPixelRatio = window.devicePixelRatio;

          App.request("is:device")
            .done(function() {
              window.devicePixelRatio = 1;
            });

          _.defaults(options || (options = {}), {
            context: deferred,
            document: this.name,
            fallback: _.bind(function (options) {
              var chained = function () {
                var deferred = $.Deferred(),
                    timer = new Date().getTime(),
                    doc = Marionette.getOption(this, "document");
                    options.zoom = App.request("zoomProportion", options);

                var id = doc.loadCanvasAsync(
                    options.index,
                    options.zoom,
                    options.rotation,
                    function (canvas) {
                      var time = new Date().getTime() - timer;
                      App.request("is:device")
                        .done(function() {
                          window.devicePixelRatio = defaultPixelRatio;
                        });
                      // Se comprueba si existe la función porque esta función solo existe en versiones nuevas de PDFTron
                      if (_.isFunction(doc.unloadCanvasResources)) {
                        doc.unloadCanvasResources(id);
                      }
                      else {
                        console.warn("unloadCanvasResources doesnt exists","PDFTRON is not updated");
                      }
                      //statsController.avg("canvas", time);
                      deferred.resolve($(canvas).removeAttr("style").get(0));
                    },
                    function (canvas) {
                      $(canvas).addClass('bookpage');
                      deferred.notify(canvas);
                    },
                    options.number
                );

                doc.iteraciones = doc.iteraciones == undefined ? 0 : doc.iteraciones+1;
                // Realizar la liberación del Document (para que no se llene la memoria de la aplicación)
                //if (doc.iteraciones > 6) {
                //  doc.unloadResources();
                //}

                return deferred.promise();
              };

              return this.call.chain(chained.bind(this));
            }, this),
            index: index || 0,
            name: this.name,
            number: 1,
            origin: {x: 0, y: 0},
            rotation: 0,
            zoom: 1
          });

          /**
           * devicePixelRatio more than 1 makes weird effects with canvas.
           * This statement avoid that canvas hides.
           * This statement could cause an error in any moment.
           */
          // Forzamos el zoom a 1 solo para iOS por el retina. En Android trae problemas
          if (window.devicePixelRatio && window.devicePixelRatio > 1 && !!navigator.userAgent.match(/iPhone|iPad|iPod/i)) {
            options.zoom = 1;
          }

          return options.fallback(options).promise();

        },

        text: function (index, options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            context: deferred
          });

          Marionette.getOption(this, "document").loadPageText(index, function (text) {
            deferred.resolveWith(options.context, [text]);
          });

          return deferred.promise();
        },

        text_position: function (index, options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            context: deferred
          });

          Marionette.getOption(this, "document")
              .getTextPosition(index, options.from, options.to, function (position) {
                deferred.resolveWith(options.context, [position]);
              });

          return deferred.promise();
        }
      });
    }
);
//noinspection JSHint
define(
    'models/page.collection',["backbone", "models/page"],
    function (Backbone, PageModel) {
      "use strict";

      return Backbone.Collection.extend({
        comparator: "id",
        model: PageModel
      });
    }
);
define(
    'models/chapter',["backbone", "models/page.collection"],
    function (Backbone, PageCollectionModel) {
      "use strict";

      return Backbone.Model.extend({
        defaults: {
          id: 0,

          status: true,

          code: "",
          title: "title", // TODO i18next.t("title")

          pages: new PageCollectionModel()
        }
      });
    }
);
define(
    'models/book',["backbone", "models/chapter", "models/page.collection"],
    function (Backbone, ChapterModel, PageCollection) {
      "use strict";

      return Backbone.Collection.extend({
        comparator: "id",
        model: ChapterModel,

        pages: function () {
          return new PageCollection(
              this.reduce(function (result, value) {
                return result.concat(value.get("pages").toArray());
              }, [])
          );
        }
      });
    }
);
// noinspection JSHint
define(
    'modules/favorites/models/page.collection',["models/page.collection"],
    function (PageCollectionModel) {
      "use strict";

      return PageCollectionModel.extend({
        url: "favorites"
      });
    }
);
define(
    'models/resource',["underscore", "backbone", "configuration"],
    function (_, Backbone, configuration) {
      "use strict";

      return Backbone.Model.extend({
        defaults: {
          link: "",
          srcImage: configuration.root + "assets/images/no-image.jpg"
        },

        initialize: function(){
          this.on("change:link", this.getIconByType);
          this.getIdExt();
          this.getIconByType();
          this.getStatus();
        },

        get: function(attribute) {
          if (_.contains(["link", "srcImage"], attribute) && this.attributes[attribute]) {
            return this.attributes[attribute]
                .replace("//", "/")
                .replace("http:/", "http://")
                .replace("https:/", "https://");
          }

          return this.attributes[attribute];
        },
        getIdExt: function(){
          var url = this.get("link");
          var id  = "";

          try{
            id = url.match(/\/(ES\w+)\//)[1];
          } catch(e){
            id = false;
          }
          this.set("idExt", id);
          return id;
        },
        getStatus: function(){
          var status  = false;
          var self    = this;

          self.set("status", status);
          self.set("quality", false);

          $.when(App.request("is:quality", {context: this})).done(function () {
              var urlControl = configuration.apiav+"av_utils/getStatusByIdExt/";
              self.set("quality", true);
              setTimeout(function() {
                $.post(urlControl, {
                  idExt : self.getIdExt()
                }, function(respuesta) {
                  console.log("res",respuesta);
                  console.log("idExt",self.getIdExt());
                  status = "estado_"+respuesta;
                  self.set("status", status);
                });
              },100);
          })
        },
        getIconByType: function(){
          var url = this.get("link");
          var id  = this.getIdExt();
          var _prefix = "";
          var _icon = "";
          try{
            var _prefix;
            var pos = id.indexOf("ESR3D");
            if(pos != -1){
              _prefix = "ESR3D"; 
            }else{
              _prefix = id.replace(/[0-9]/g, '');
            }
          } catch(e){}
          if (url != "") {
            switch (_prefix) {

              case "ESAD":
                _icon = "ICONO_1";
                break;
              case "ESAQ":
                _icon = "ICONO_1";
                break;

              case "ESAM":
                _icon = "ICONO_2";
                break;
              case "ESPM":
                _icon = "ICONO_3";
                break;

              case "ESPR":
                _icon = "ICONO_4";
                break;

              case "ESPA":
                _icon = "ICONO_4";
                break;

              case "ESAT":
                _icon = "ICONO_5";
                break;

              case "ESBDD":
                _icon = "ICONO_6";
                break;

              case "ESBI":
                _icon = "ICONO_7";
                break;

              case "ESRUP":
                _icon = "ICONO_8";
                break;


              case "ESCCR":
                _icon = "ICONO_9";
                break;

              case "ESVP":
                _icon = "ICONO_9";
                break;


              case "ESCL":
                _icon = "ICONO_10";
                break;

              case "ESCV":
                _icon = "ICONO_10";
                break;

              case "ESCVE":
                _icon = "ICONO_10";
                break;


              case "ESDI":
                _icon = "ICONO_11";
                break;

              case "ESDO":
                _icon = "ICONO_11";
                break;


              case "ESEH":
                _icon = "ICONO_12";
                break;

              case "ESEYO":
                _icon = "ICONO_13";
                break;


              case "ESGM":
                _icon = "ICONO_16";
                break;

              case "ESGE":
                _icon = "ICONO_16";
                break;

              case "ESGO":
                _icon = "ICONO_16";
                break;

              case "ESGU":
                _icon = "ICONO_16";
                break;

              case "ESGV":
                _icon = "ICONO_16";
                break;

              case "ESGAR":
                _icon = "ICONO_16";
                break;


              case "ESGA":
                _icon = "ICONO_17";
                break;

              case "ESAR":
                _icon = "ICONO_18";
                break;

              case "ESTL":
                _icon = "ICONO_19";
                break;


              case "ESRO":
                _icon = "ICONO_21";
                break;

              case "ESRD":
                _icon = "ICONO_21";
                break;

              case "ESMA":
                _icon = "ICONO_21";
                break;


              case "ESGL":
                _icon = "ICONO_22";
                break;


              case "ESSLM":
                _icon = "ICONO_23";
                break;

              case "ESSL":
                _icon = "ICONO_23";
                break;


              case "ESTM":
              case "ESTMP":
                _icon = "ICONO_24";
                break;

              case "ESTS":
                _icon = "ICONO_24";
                break;

              case "ESTT":
                _icon = "ICONO_24";
                break;

              case "ESRA":
                _icon = "ICONO_24";
                break;

              case "ESPS":
                _icon = "ICONO_24";
                break;

              case "ESCOV":
                _icon = "ICONO_24";
                break;
              case "ESTS":
                _icon = "ICONO_25";
                break;
              case "ESFL":
                _icon = "ICONO_27";
                break;
              case "ESLLP":
                _icon = "ICONO_28";
                break;

              case "ESCG":
                _icon = "ICONO_29";
                break;

              case "ESLT":
                _icon = "ICONO_30";
                break;

              case "ESKM":
              case "ESTC":
                _icon = "ICONO_31";
                break;

              case "ESOC":
                _icon = "ICONO_32";
                break;


              case "ESOL":
                _icon = "ICONO_33";
                break;

              case "ESFP":
                _icon = "ICONO_33";
                break;


              case "ESOP":
                _icon = "ICONO_34";
                break;

              case "ESDP":
                _icon = "ICONO_37";
                break;

              /*case "ESJD":
                _icon = "ICONO_38";
                break;*/

              case "ESR3D":
                _icon = "ICONO_recursos-3d";
                break;

              case "ESPP":
                _icon = "ICONO_pasoapaso";
                break;

              case "ESSI":
                _icon = "ICONO_slider";
                break;

              case "ESJD":
                _icon = "ICONO_recursos-ab";
                break;

              case "ESHS":
                _icon = "ICONO_recursos-hotspot";
                break;

              //CASOS DISTINTOS
              case "ESARC":
                //miramos la extension de la URL
                var _ext = url.split('.').pop().toLowerCase();

                switch (_ext) {
                  case "pdf":
                    _icon = "ICONO_26";
                    break;
                  case "doc":
                    _icon = "ICONO_35";
                    break;
                  case "docx":
                    _icon = "ICONO_35";
                    break;
                  case "mp3":
                    _icon = "ICONO_36";
                    break;
                  case "wav":
                    _icon = "ICONO_36";
                    break;
                  case "ogg":
                    _icon = "ICONO_36";
                    break;
                  case "webm":
                    _icon = "ICONO_36";
                    break;
                  case "aac":
                    _icon = "ICONO_36";
                    break;
                  case "html":
                    _icon = "ICONO_14";
                    break;
                  case "htm":
                    _icon = "ICONO_14";
                    break;
                  case "mp4":
                    _icon = "ICONO_recursos-video";
                    break;
                  case "mov":
                    _icon = "ICONO_recursos-video";
                    break;
                  case "avi":
                    _icon = "ICONO_recursos-video";
                    break;
                  case "zip":
                    _icon = "ICONO_recursos-cloud";
                    break;
                  case "xls":
                  case "xlsx":
                    _icon =  "ICONO_20";
                    break;
                  case "pps":
                  case "ppt":
                  case "pptx":
                    _icon =  "ICONO_recursos_ppt";
                    break;
                  default:
                    _icon = "ICONO_15";
                }

                break;



              default:
                _icon = "ICONO_15";
                if(!id){
                  _icon = "ICONO_recursos_02_web";
                }
            }

            this.set("resourceIcon", _icon);
          }
          else
            this.set("resourceIcon", "");
        }
      });
    }
);

define(
    'models/resource.collection',["backbone", "models/resource"],
    function (Backbone, ResourceModel) {
      "use strict";

      return Backbone.Collection.extend({
        comparator: "order",
        model: ResourceModel
      });
    }
);
//noinspection JSHint
define(
    'request',[
      // libraries
      "jquery",
      "underscore",
      "marionette",
      // configuration
      "configuration",
      // app
      "app",
      "encrypt",
      "controllers/cache",
      "controllers/document",
      "controllers/queue",
      "controllers/stats",
      "models/book",
      "models/chapter",
      "modules/favorites/models/page.collection",
      "models/page.collection",
      "models/page",
      "models/resource.collection",
      // helpers
      "language"
    ],
    function (
        $,
        _,
        Marionette,
        configuration,
        App,
        Encrypt,
        CacheController,
        DocumentController,
        QueueController,
        statsController,
        BookModel,
        ChapterModel,
        FavoritesCollectionModel,
        PageCollectionModel,
        PageModel,
        ResourceCollectionModel,
        language
        ) {
      "use strict";

      /* Cache */
      var JSON = window.master || (window.master = {}),
          cache,
          modifier = 1;

      CacheController = CacheController.extend({
        store: {
          document: {},
          pages: {},
          temporary: {}
        },

        initialize: function() {
          window.store = this.store;
          window.documents = this.store.document;
        },

        chapters: function (options) {
          _.defaults(options || (options = {}), {
            key: "chapters"
          });

          if (this.temporary[options.key]) {
            this.store[options.key] = this.temporary[options.key];
            this.temporary[options.key] = null;
          }

          return this.store[options.key];
        },

        document: function (options) {
          if (!(options && options.code)) {
            // Return a failure
            return $.Deferred().reject();
          }

          _.defaults(options || (options = {}), {
            key: "document"
          });

          if (this.temporary[options.key]) {
            this.store.temporary[options.code] = this.temporary[options.key];
            delete this.temporary[options.key];
            return this.store.temporary[options.code];
          }

          if (this.store[options.key].code === options.code) {
            return this.store[options.key].value;
          } else if (this.store.temporary[options.code]) {
            return this.store.temporary[options.code];
          }

          return null;
        },

        pages: (function () {
          var last;

          return function (options) {
            var pages;

            _.defaults(options || (options = {}), {
              // default key
              draw: last || "default",
              key: "pages"
            });
            last = options.draw;

            if (this.temporary[options.key]) {
              this.store[options.key][options.draw] =
                  this.temporary[options.key];
              this.temporary[options.key] = null;

              this.store.chapters = null;
            }

            return this.store[options.key][options.draw];
          };
        })()
      });

      cache = new CacheController();


      var getReplacedText = function (texto) {

        if(!texto){
          return '';
        }

        var patt = new RegExp('<(.|\n)*?>', 'g');
        var result = texto.match(patt);
        var textoFinal = texto;


        var cumpleCond = function (aux) {
          if (aux == '<b>' || aux == '</b>' || aux == '<strong>' ||
              aux == '</strong>' || aux == '<i>' || aux == '</i>' ||
              aux == '<italic>' || aux == '</italic>' ||
              aux == '<sup>' || aux == '</sup>' ||
              aux == '<sub>' || aux == '</sub>' ||
              aux.indexOf('<font color') > -1 || aux == '</font>') {
            return true;
          }
          else {
            return false;
          }
        }

        var cumpleCondFont = function (aux) {
          if (aux.indexOf('<font color') > -1) {

            return 1;
          } else {
            if (aux == '</font>') {
              return 2;
            }
            return false;
          }
        }

        var aux = '';
        if (result) {
          for (var x = 0; x < result.length; x++) {
            aux = result[x];
            if (!cumpleCond(aux)) {
              textoFinal = textoFinal.replace(result[x], '');
            }

            var num = cumpleCondFont(aux);

            if (num == 1) {
              textoFinal =
                  textoFinal.replace('<font color="', '<span style="color:');
              textoFinal =
                  textoFinal.replace("<font color='", "<span style='color:");
            }
            if (num == 2) {
              textoFinal = textoFinal.replace(result[x], '</span>');
            }
          }
        }


        return textoFinal;
      }

      /* Request */
      App.reqres.setHandlers({
        "av": (function () {
          var listeners = {};

          $(window).on("message", function (event) {
            var listener;

            event = event.originalEvent || event;

            listener = listeners[event.data.identifier];
            //noinspection JSHint
            listener && listener.resolve(event.data);
          });

          return function (options) {
            var deferred = $.Deferred(),
                timeout = _.delay(function () {
                  deferred.reject("timeout");
                }, 15000);

            _.defaults(options || (options = {}), {
              context: deferred,
              identifier: options.type + new Date().getTime(),
              timeout: true
            });

            if (!options.type) {
              return deferred.resolve();
            }

            if (!options.timeout) {
              clearTimeout(timeout);
            }

            App.request("is:av")
                .done(function () {
                  var local = $.Deferred();

                  listeners[options.identifier] = local;
                  local.done(function (data) {
                    clearTimeout(timeout);
                    delete listeners[options.identifier];
                    deferred.resolveWith(options.context, [data]);
                  });

                  window.parent.postMessage(
                      _.pick(options, "identifier", "key", "type", "action", "dataJSON", "file", "url", "content", "clipboard"),
                      "*"
                  );
                })
                .fail(function () {
                  deferred.reject("Is not AV");
                });

            return deferred.promise();
          };
        })(),

        "chapter": function (options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            clone: false,
            context: deferred,
            page: null
          });

          if (_.has(options, "test")) {
            return deferred.resolveWith(options.context, [options.test]);
          }

          $.when(
              App.request("page", {model: true, number: options.page}),
              App.request("chapters")
          )
              .done(function (page, chapters) {
                var chapter = chapters.findWhere({code: "*"}) ||
                    chapters.findWhere({code: page && page.get("code")});

                deferred.resolveWith(
                    options.context,
                    [(options.clone && chapter) ? chapter.clone() : chapter]
                );
              })
              .fail(function () {
                deferred.rejectWith(options.context);
              });

          return deferred.promise();
        },

        "chapters": function (options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            clone: false,
            context: deferred,
            fallback: function () {
              var deferred = $.Deferred();

              App.request("pages")
                  .done(function (pages) {
                    var chapters = [],
                        k = 1,
                        origin;

                    try {
                      origin = JSON.book.chapters;
                    } catch (e) {
                      origin = [];
                    }

                    _.each(origin, function (value) {
                      var filtered = (value.idUnidad !== "*") ?
                          pages.where({code: value.idUnidad}) : pages.toArray();

                      if (!filtered.length && pages.length) {
                        return true;
                      }

                      chapters.push(
                          new ChapterModel({
                            id: k++,

                            status: value.active,

                            code: value.idUnidad,
                            title: value.title,

                            pages: new PageCollectionModel(filtered)
                          })
                      );

                      return true;
                    });

                    deferred.resolve(new BookModel(chapters));
                  })
                  .fail(function () {
                    deferred.reject();
                  });

              return deferred.promise();
            }
          });

          cache.get("chapters", options)
              .done(function (chapters) {
                deferred.resolveWith(
                    options.context,
                    [(options.clone && chapters) ? chapters.clone() : chapters]
                );
              })
              .fail(function () {
                deferred.rejectWith(options.context);
              });

          return deferred.promise();
        },

        "credits": function (options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            context: deferred,
            fallback: function () {
              try {
                return JSON.book.credits || "";
              }
              catch (e) {
                return "";
              }
            }
          });

          deferred.resolveWith(options.context, [cache.get("credits", options)]);

          return deferred.promise();
        },

          "document": function (options) {

              var deferred = $.Deferred();

              deferred.resolve(document);

              return deferred.promise();
          },

        "draw:custom": function (options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            context: deferred
          });

          // Mockup request
          return deferred.resolveWith(options.context);
        },
        "draw:exit": function (options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            context: deferred
          });

          // Mockup request
          return deferred.resolveWith(options.context);
        },

        "favorites": function (options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            context: deferred,
            fallback: function () {
              var collection = new FavoritesCollectionModel(),
                  deferred = $.Deferred();

              collection.fetch()
                  .done(function () {
                    deferred.resolve(collection);
                  })
                  .fail(function () {
                    deferred.reject();
                  });

              return deferred.promise();
            }
          });

          cache.get("favorites", options)
              .done(function (collection) {
                deferred.resolveWith(options.context, [collection]);
              })
              .fail(function () {
                deferred.rejectWith(options.context);
              });

          return deferred.promise();
        },

        "GEDI": function (options) {
          var deferred = $.Deferred();

          options = _.extend({
            context: deferred,
            fallback: function () {
              try {
                return JSON.book.master.GEDI;
              } catch (e) {
                return null;
              }
            }
          }, options);

          deferred.resolveWith(options.context, [cache.get("GEDI", options)]);

          return deferred.promise();
        },

        // Obtiene el info.json
        "info": function (options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            // TODO check after this
            url: configuration.info || configuration.rootData + "info.json"
          });

          $.ajax({
            dataType: "json",
            url: options.url
          })
              .done(function (data) {
                deferred.resolve({
                  language: data.idioma,
                  name: data.nombre,
                  type: data.tipo_producto
                });
              })
              .fail(function (err) {
                deferred.resolve({});
              });

          return deferred.promise();
        },

        "is:av": function (options) {
          var deferred = $.Deferred(),
              is;

          _.defaults(options || (options = {}), {
            boolean: false,
            context: deferred,
            fallback: function (options) {
              return !!~window.location.search.toLowerCase().indexOf(options.keyword);
            },
            keyword: "av",
            not: false
          });

          is = cache.get("is:av", options);
          //noinspection JSHint
          options.not && (is = !is);

          if (options.boolean || is) {
            deferred.resolveWith(options.context, [is]);
          } else {
            deferred.rejectWith(options.context);
          }

          return deferred.promise();
        },

        "is:quality": function (options) {
          var deferred = $.Deferred(),
              is;
          _.defaults(options || (options = {}), {
            boolean: false,
            context: deferred,
            fallback: function (options) {
              return !!~window.location.search.toLowerCase().indexOf(options.keyword);
            },
            keyword: "quality",
            not: false
          });

          is = cache.get("is:quality", options);
          //noinspection JSHint
          options.not && (is = !is);
          if (options.boolean || is) {
            deferred.resolveWith(options.context, [is]);
          } else {
            deferred.rejectWith(options.context);
          }

          return deferred.promise();
        },

        "is:custom": function (options) {
          var deferred = $.Deferred(),
              is;

          _.defaults(options || (options = {}), {
            boolean: false,
            context: deferred,
            fallback: function (options) {
              return !!~window.location.search.toLowerCase().indexOf(options.keyword);
            },
            keyword: "mybook",
            not: false
          });

          is = cache.get("is:custom", options);
          //noinspection JSHint
          options.not && (is = !is);

          if (options.boolean || is) {
            deferred.resolveWith(options.context, [is]);
          } else {
            deferred.rejectWith(options.context);
          }

          return deferred.promise();
        },

        "is:notebook": function (options) {
          var deferred = $.Deferred(),
              is;

          _.defaults(options || (options = {}), {
            boolean: false,
            context: deferred,
            fallback: function (options) {
              return !!~window.location.search.toLowerCase().indexOf(options.keyword);
            },
            keyword: "notebook",
            not: false
          });

          is = cache.get("is:notebook", options);
          //noinspection JSHint
          options.not && (is = !is);

          if (options.boolean || is) {
            deferred.resolveWith(options.context, [is]);
          } else {
            deferred.rejectWith(options.context);
          }

          return deferred.promise();
        },

        "is:external": function (options) {
          var deferred = $.Deferred(),
              is;

          _.defaults(options || (options = {}), {
            boolean: false,
            context: deferred,
            fallback: function (options) {
              return !!~window.location.search.toLowerCase().indexOf(options.keyword);
            },
            keyword: "external",
            not: false
          });

          is = cache.get("is:external", options);
          //noinspection JSHint
          options.not && (is = !is);

          if (options.boolean || is) {
            deferred.resolveWith(options.context, [is]);
          } else {
            deferred.rejectWith(options.context);
          }

          return deferred.promise();
        },

        "is:device": function (options) {
          var deferred = $.Deferred(),
              is,
              isMobile = {
                Android: function () {
                  return navigator.userAgent.match(/Android/i);
                },
                BlackBerry: function () {
                  return navigator.userAgent.match(/BlackBerry/i);
                },
                iOS: function () {
                  return navigator.userAgent.match(/iPhone|iPad|iPod/i);
                },
                Opera: function () {
                  return navigator.userAgent.match(/Opera Mini/i);
                },
                Windows: function () {
                  return navigator.userAgent.match(/IEMobile/i);
                },
                WindowsTouch: function () {
                  return navigator.platform.match("Win") && ('ontouchstart' in window || navigator.msMaxTouchPoints) && false;
                },
                SantillanaUserAgent: function () {
                  return navigator.userAgent.match(/SantillanaUserAgent/i);
                },
                any: function () {
                  return (isMobile.Android() || isMobile.BlackBerry() ||
                      isMobile.iOS() || isMobile.Opera() || isMobile.Windows() || isMobile.WindowsTouch() || isMobile.SantillanaUserAgent());
                }
              };

          _.defaults(options || (options = {}), {
            boolean: false,
            context: deferred,
            fallback: function () {
              return isMobile.any();
            },
            not: false
          });

          is = cache.get("is:device", options);
          //noinspection JSHint
          options.not && (is = !is);

          if (options.boolean || is) {
            deferred.resolveWith(options.context, [is]);
          } else {
            deferred.rejectWith(options.context);
          }

          return deferred.promise();
        },

        "is:device:landscape": function (options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            boolean: false,
            context: deferred
          });

          App.request("is:device:portrait", options)
              .done(function (isDevicePortrait) {
                if (options.boolean) {
                  isDevicePortrait =
                      (_.has(options, "test")) ?
                          !options.test :
                          isDevicePortrait;

                  deferred.resolveWith(options.context, [!isDevicePortrait]);
                } else {
                  deferred.rejectWith(options.context);
                }
              })
              .fail(function () {
                deferred.resolveWith(options.context);
              });

          return deferred.promise();
        },

        "is:device:portrait": function (options) {
          var deferred = $.Deferred(),
              $window = $(window),
              is;

          options = _.extend({
            boolean: false,
            context: deferred
          }, options);

          is = (_.has(options, "test")) ? options.test :
              $window.width() < $window.height();

          App.request("is:device", options)
              .done(function (isDevice) {
                if (options.boolean || is) {
                  deferred.resolveWith(options.context, [isDevice && is]);
                } else {
                  deferred.rejectWith(options.context);
                }
              })
              .fail(function () {
                deferred.rejectWith(options.context);
              });

          return deferred.promise();
        },

        "is:document:landscape": function (options) {
          var deferred = $.Deferred();

          options = _.extend({
            boolean: false,
            context: deferred,
            fallback: function () {
              var deferred = $.Deferred();

              App.request("is:document:portrait", {boolean: true})
                  .done(function (isDocumentPortrait) {
                    deferred.resolve(!isDocumentPortrait);
                  });

              return deferred.promise();
            }
          }, options);

          cache.get("is:document:landscape", options)
              .done(function (isDocumentLandscape) {
                if (options.boolean || isDocumentLandscape) {
                  deferred.resolveWith(options.context, [isDocumentLandscape]);
                } else {
                  deferred.rejectWith(options.context);
                }
              });

          return deferred.promise();
        },

        "is:document:portrait": function (options) {
          var deferred = $.Deferred(),
              is;

          options = _.extend({
            boolean: false,
            context: deferred,
            fallback: function () {
              var deferred = $.Deferred();

              App.request("pages").done(function (pages) {
                var page = pages.find(function (model) {
                  return !!model.get("thumbnail");
                });

                if (!page) {
                  var size = configuration.size;

                  return deferred.resolve(size.width < size.height);
                }

                var thumbnail = configuration.thumbnails +
                    page.get("thumbnail");

                $("<img />", {src: thumbnail}).on("load", function () {
                  deferred.resolve(this.width < this.height);
                });
              });

              return deferred.promise();
            }
          }, options);

          cache.get("is:document:portrait", options)
              .done(function (is) {
                if (options.boolean || is) {
                  deferred.resolveWith(options.context, [is]);
                } else {
                  deferred.rejectWith(options.context);
                }
              });

          return deferred.promise();
        },

        "is:editable": function (options) {
          var deferred = $.Deferred(),
              is;

          _.defaults(options || (options = {}), {
            boolean: false,
            context: deferred,
            fallback: function (options) {
              return !!~window.location.search.toLowerCase().indexOf(options.keyword);
            },
            keyword: "edit",
            not: false
          });

          is = cache.get("is:editable", options);
          //noinspection JSHint
          options.not && (is = !is);

          var sgo = !!configuration.sgoReadOnlyObjectsURL;

          if (options.boolean || is || sgo) {
            deferred.resolveWith(options.context, [is]);
          } else {
            deferred.rejectWith(options.context);
          }

          return deferred.promise();
        },

        "module:load": function (module) {
          var deferred = $.Deferred(),
              path = module.split(".").shift();

          try {
            //noinspection JSHint
            require(
                [("modules/" + path + "/" + module).toLowerCase()],
                function (module) {
                  deferred.resolve(module);
                  App.trigger("module:load", module);
                  App.trigger("module:load:" + module.moduleName);
                }
            );
          }
          catch (Exception) {
            deferred.reject();
          }

          return deferred.promise();
        },

        "page": function (options) {
          var deferred = $.Deferred(),
              page;

          _.defaults(options || (options = {}), {
            clone: false,
            context: deferred,
            model: false,
            /**
             * @deprecated options.page
             */
            number: options.page || null
          });
          options.model = !!options.number || options.model;

          // Return earliest we can
          if (options.test) {
            return deferred.resolveWith(options.context, [options.test]);
          }

          App.request("viewer:page")
              .done(function (current) {
                page = current;
              })
              .fail(function () {
                page = 1;
              })
              .always(function () {
                page = options.number || page;

                if (options.model) {
                  App.request("pages")
                      .done(function (pages) {
                        page = pages.get(page);

                        if (!page) {
                          return $.Deferred().reject();
                        }

                        deferred.resolveWith(
                            options.context,
                            [(options.clone) ? page.clone() : page]
                        );
                      })
                      .fail(function () {
                        deferred.rejectWith(options.context);
                      });
                } else {
                  deferred.resolveWith(options.context, [page]);
                }
              });

          return deferred.promise();
        },

        "pages": function (options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            clone: false,
            context: deferred,
            extension: ".jpg",
            fallback: function (options) {
              var DrawCollectionModel = PageCollectionModel.extend({
                    url: "book/" + options.draw
                  }),
                  overwrite = function (index) {
                    var page = pages.at(index),
                        value = origin[index];

                    if (!value) {
                      return false;
                    }

                    //noinspection JSHint
                    value.readOnlyObjects || (value.readOnlyObjects = []);
                    App.request("is:editable")
                        .done(function () {
                          if (configuration.sgoReadOnlyObjectsURL) {
                            page.set("drawing", value.readOnlyObjects[0]);
                            page.set("custom", null);
                          } else {
                            page.set("drawing", value.readOnlyObjects[1]);
                            page.set("custom", [value.readOnlyObjects[0]]);
                          }
                        })
                        .fail(function () {
                          page.set("custom", value.readOnlyObjects);
                        });
                  },
                  deferred = $.Deferred(),
                  drawCollectionModel = new DrawCollectionModel(),
                  origin,
                  pages = [];

              try {
                origin = JSON.book.pages;
              } catch (e) {
                origin = [];
              }

              _.each(origin, function (value, index) {
                var number = parseInt(value.number),
                    page = new PageModel({
                      id: index + 1,
                      number: number,
                      title: (value.name) ? value.name : index + 1,

                      index: value.index,
                      code: value.idUnidad,

                      thumbnail: (!value.blank) ?
                          number + options.extension :
                          null,

                      type: value.tipo,

                      first: index === 0,
                      last: origin.length === index + 1,
                      order: value.order
                    });

                //noinspection JSHint
                value.readOnlyObjects || (value.readOnlyObjects = []);
                App.request("is:editable")
                    .done(function () {
                      if (configuration.sgoReadOnlyObjectsURL) {
                        page.set("drawing", value.readOnlyObjects[0]);
                        page.set("custom", null);
                      } else {
                        page.set("drawing", value.readOnlyObjects[1]);
                        page.set("custom", [value.readOnlyObjects[0]]);
                      }
                    })
                    .fail(function () {
                      page.set("custom", value.readOnlyObjects);
                    });

                pages.push(page);
              });

              pages = new PageCollectionModel(pages);

              drawCollectionModel.fetch()
                  .done(function () {
                    drawCollectionModel.each(function (value) {
                      var page = pages.get(value.get("id"));

                      if (!page) {
                        return true;
                      }

                      page.set(value.toJSON());
                      overwrite(value.get("id") - 1);
                    });

                    App.trigger("hide:custom");
                  });

              if (pages && pages.filter(options.filter)) {
                pages =
                    new PageCollectionModel(pages.filter(options.filter));
              }
              deferred.resolve(pages);

              return deferred.promise();
            },
            filter: function (model) {
              var filter = window.location.search.match(/scorm=true&unitId=Unidad_([^&]+)/i);

              return !filter || model.get("code") === "U"+filter[1];
            }
          });

          cache.get("pages", options)
              .done(function (pages) {
                deferred.resolveWith(
                    options.context,
                    [(options.clone && pages) ? pages.clone() : pages]
                );
              })
              .fail(function () {
                deferred.rejectWith(options.context);
              });

          return deferred.promise();
        },

        "viewer:is:mode:double": function () {
          // Mockup request
          return $.Deferred().resolve();
        },

        "viewer:is:mode:single": function () {
          // Mockup request
          return $.Deferred().reject();
        },

        "viewer:page": function () {
          // Mockup request
          return $.Deferred().reject();
        },

        /*
         Temporary, it is can be substitute when use a model on worktools
         */
        "is:worktools:available": function () {
          var buttons,
              deferred = $.Deferred();

          try {
            buttons = JSON.book.buttons;
          } catch (e) {
            buttons = [];
          }

          deferred.resolve(
              _.find(buttons, function (num) {
                return num.path.length > 0;
              })
          );

          return deferred.promise();
        },

        "is:plataforma": function () {
          var plataforma,
              deferred = $.Deferred();

          try {
            plataforma = JSON.book.plataforma;
          } catch (e) {
            plataforma = false;
          }

          deferred.resolve(
              plataforma
          );

          return deferred.promise();
        },

        // Obtiene el info.json
        "searchIndex": function (options) {
          var deferred = $.Deferred();

          _.defaults(options || (options = {}), {
            url: configuration.source + "search.data"
          });

          $.ajax({
            dataType: "text",
            url: options.url
          })
              .done(function (data) {

                data = $.parseJSON(Encrypt.decrypt(data))

                if (data == "null" || !data) {
                  deferred.reject();
                } else {
                  // Decodifico caracteres Unicode y paso a lowerCase
                  _.each(data, function (item) {
                    item = item.toLowerCase();
                  });

                  deferred.resolve(data);
                }
              })
              .fail(function (err) {
                deferred.reject();
              });

          return deferred.promise();
        },

        "url:worktools": function (options) {
          var deferred = $.Deferred();

          options = _.extend({
            context: deferred,
            type: "activities",
            url: ""
          }, options);

          if (options.type === "activities") {
            options.url = configuration.rootData + options.url;
          }

          App.request("worktools:" +
              options.type, {context: this, url: options.url})
              .done(function (resources, filters) {
                deferred.resolve([
                  new ResourceCollectionModel(resources),
                      filters || ""
                ]);
              });

          return deferred.promise();
        },

        "worktools:activities": function (options) {
          var deferred = $.Deferred(),
              k = 1;

          $.ajax({
            url: options.url,
            dataType: "json",
            success: function (data) {
              var resource = [];
              var i = 0;

              _.each(data, function (item, index) {
                _.each(item, function (item1) {
                  var el = {
                    id: index + (item1.idActividad || k++),
                    order: ++i, //item1.orden,
                    chapter: index,
                    tittle: getReplacedText(item1.name || "")
                    //icon: getReplacedText(item1.icon || "")
                  };

                  if (item1.url) {
                    el.link = configuration.rootData + item1.url;
                    //comprobamos que sea un enlace para que no añada el rootData
                    if(typeof(item1.idActividad) == "undefined" && typeof(item1.type) != "undefined" && item1.type == "link")
                    {
                      el.link = item1.url;
                    }
                  }

                  if (item1.imagen) {
                    el.srcImage = configuration.rootData + item1.imagen;
                    el.showThumbnail = false;
                  }

                  resource.push(el);
                });
              });

              deferred.resolve(resource);
            }
          });


          return deferred.promise();
        },

        "worktools:library": function (options) {

          var deferred = $.Deferred();

          $.ajax({
            url: configuration.library,
            data: {
              id_biblioteca: options.url
            },
            dataType: "jsonp",
            crossDomain: true,
            cache: false,
            success: function (json) {
              var library = [];

              try {
                _.each(json[1].elementos, function (item1, index1) {
                  var el = {
                    id: index1,
                    tittle: getReplacedText(item1.name || ""),
                    filter: item1.opciones.substring(1, item1.opciones.length -
                        1).split(")("),
                    link: item1.link,
                    srcImage: item1.image,
                    showThumbnail: true
                  };

                  library.push(el);
                });

                var filter = [];

                _.each(json[0].elementos, function (item, index) {
                  filter.push({
                    title: item.name || "",
                    options: _.map(item.elementos, function (value) {
                      return {name: value.nombre, data: {id: value.id}}
                    })
                  });

                  // Añadida ñapa para meter el elemento "padre", que sería como el "seleccionar todos".
                  filter[filter.length-1].options.splice(0,0, {name: item.name || "", data : { id: 0 }});
                });

                deferred.resolve(library, filter);
              } catch (e) {
                deferred.resolve([], [null, null]); // Pasando 'null, null' hace que no se vean mal los combos
              }
            },
            error: function(){
              if(!navigator.onLine){
                App.execute("notification", language("noInternet"), {color: 10});
              }
              deferred.resolve([], []);
            }
          });

          return deferred.promise();
        },

        "worktools:links": function (options) {
          var deferred = $.Deferred();

          $.ajax({
            url: configuration.links,
            data: {
              gedi: options.url
            },
            dataType: "jsonp",
            crossDomain: true,
            cache: false,
            success: function (data) {
              if (!data) {
                deferred.reject();
              } else {
                var categorias = [];
                var subCategorias = [];
                var filter = [];
                var itemList = [];
                var linkCount = 0;
                var unitCount = 0;
                var identificador = "";

                // Recorro todas las categorías
                _.each(data, function (categoria, key, list) {
                  // Obtengo los nombres de las unidades dentro de cada categoría
                  Object.keys(categoria.elementos).forEach(function (unidad) {
                    /**
                     * FIX: LIBROMEDIA: Cajón de recursos - (Gestor de publicaciones V2): Eliminar del cajón los elementos ENLACES  que estén en el gestor SIN ASIGNAR cuando sean por unidades = NO en el gestor (Modificar la select que construye los enlaces para no meter SIN ASIGNAR) - GEDI Ejemplo: 679316
                     * No estoy muy de acuerdo con esta solución ya que debería ser manejado a nivel de backend pero bueno :(
                     * @author Mario Poyato <mario.poyato@inmediastudio.es>
                     */
                    if (unidad == 'Sin asignar') {
                        return ;
                    }
                    subCategorias.push({
                      name: unidad,
                      data: {
                        code: unidad
                      }
                    });

                    // Si hay una sola categoría, no antepongo el nombre a la unidad
                    if(list.length > 1){
                      identificador = categoria.nombre + " - " + unidad;
                    } else {
                      identificador = unidad;
                    }

                    categorias[unitCount] = {
                      name: identificador,
                      data: {
                        code: identificador
                      }
                    };
                    unitCount++;
                  });

                  // Agrego opcion "Todas las Unidades"
                  categorias[unitCount] = {
                      name: language("links.allFilters"),
                      page: "All",
                      data: {
                        code: "All",
                        status: "active"
                      },
                      code: "All"
                    };

                  // Armo el listado de links completo
                  _.each(categoria.elementos, function(unidades, index) {
                    _.each(unidades, function(link){
                      itemList.push(_.compactObject({
                        code: (list.length > 1) ? categoria.nombre + " - " + index : index, // Si hay una sola categoría, no antepongo el nombre a la unidad
                        id: linkCount,
                        link: link.url,
                        srcImage: link.thumbnail,
                        tittle: getReplacedText(link.nombre),
                        showThumbnail: true,
                        order: linkCount + 1
                      }));
                      linkCount++;
                    });
                  });
                });

                // Creo el filtro
                filter.push({
                  title: language("links.allFilters"),
                  options: categorias
                });

                // Devuelvo listado de links y el filtro
                deferred.resolve(itemList, filter);
              }
            },
            error: function(){
              if(!navigator.onLine){
                App.execute("notification", language("noInternet"), {color: 10});
              }
              deferred.resolve([], []);
            }
          });

          return deferred.promise();
        },

        /*
         Return buttons
         */

        "buttons": function () {
          var deferred = $.Deferred();
          var _buttons = JSON.book.buttons;

          if (_buttons === undefined) {
            App.execute("error", "No se ha encontrado un nodo de botones");
            _buttons = JSON.book.buttons = {};
          }


          deferred.resolve(_buttons);

          return deferred.promise();
        },

        /*
         Return tool
         */

        "tools": function () {
          var deferred = $.Deferred();
          var _tools = JSON.book.tools;

          if (_tools === undefined) {
//            App.execute("error", "No se ha encontrado un nodo de herramientas");
            _tools = JSON.book.tools = {};
          }


          deferred.resolve(_tools);

          return deferred.promise();
        },

        "webKitVersion": function() {
          var result = /AppleWebKit\/([\d.]+)/.exec(navigator.userAgent);
          if (result) {
              return parseFloat(result[1]);
          }
          return null;
        },

        "zoomProportion": function(options) {
          /**
           * Fix 3719
           * Reparada la función del cálculo de la proporción del zoom que antes calculaba mal la proporción para versiónes de android <= 4.2.2.
           * Para ello, se comprueba si es un dispositivo móvil y en el caso de que se trate de un dispositivo móvil se mantiene el cálculo anterior del zoom.
           * @author Mario Poyato <mario.poyato@inmediastudio.es>
           **/
          var prop;
          // Forzamos el zoom a 1 solo para iOS por el retina. En Android trae problemas
          if (window.devicePixelRatio && window.devicePixelRatio > 1 && !!navigator.userAgent.match(/iPhone|iPad|iPod/i)) {
            prop = 1;
          }
          else {
            App.request("is:device", {not: false})
              .done(function () {
                prop = options.zoom;
              })
              .fail(function () {
                var altoPage = $(".page").css("height").replace("px", "");
                var altoTotal = window.innerHeight;

                if (options.scale <= 1)
                  prop = altoPage / altoTotal;
                else
                  prop = (altoPage / altoTotal) * options.scale;
              });
          }
          return prop;
        }
      });
    }
);

//noinspection JSHint
require(
    [
      "jquery",
      "underscore",
      "backbone"
    ],
    function ($, _, Backbone) {
      "use strict";

      var delegateEventSplitter = /^(\S+)\s*(.*)$/;

      Backbone.View.prototype.delegateEvents = function (events) {
        if (!(events || (events = _.result(this, 'events')))) {
          return this;
        }

        this.undelegateEvents();

        var k = 1,
            self = this;

        _.each(events, function (value, key) {
          if (!_.isFunction(value)) {
            value = self[value];
          }

          if (!value) {
            return true;
          }

          var match = key.match(delegateEventSplitter),
              out = _.contains(self._outEvents, match[1]),
              eventName = match[1],
              selector = match[2];
          value = _.bind(value, self);
          eventName += '.delegateEvents' + self.cid;

          if (self.outEvents && out) {
            self.addOutEvent(eventName, selector, value, k);
            k *= 2;
          } else if (selector === '') {
            self.$el.on(eventName, value);
          } else {
            self.$el.on(eventName, selector, value);
          }

          return true;
        });

        return this;
      };

      (function (undelegateEvents) {
        Backbone.View.prototype.undelegateEvents = function () {
          if (this.outEvents) {
            $(document.body).off(".delegateEvents" + this.cid);
          }

          return undelegateEvents.apply(this, Array.prototype.slice.call(arguments));
        };
      })(Backbone.View.prototype.undelegateEvents);

      Backbone.View.prototype._outEventsStatus = 0;
      Backbone.View.prototype._outEvents = [
        "drag",
        "dragstart",
        "dragend",
        "dragup",
        "dragdown",
        "dragleft",
        "dragright",
        "pinch",
        "pinchin",
        "pinchout",
        "release",
        "swipe",
        "swipeup",
        "swipedown",
        "swipeleft",
        "swiperight"
      ];

      Backbone.View.prototype.addOutEvent =
          function (eventName, selector, method, flag) {
            if (selector === "") {
              this.$el.on(
                  "touch",
                  _.bind(
                      function () {
                        this._outEventsStatus |= flag;
                      },
                      this
                  )
              );
            } else {
              this.$el.on(
                  "touch",
                  selector,
                  _.bind(
                      function (event) {
                        event.stopPropagation();
                        this._outEventsStatus |= flag;
                      },
                      this
                  )
              );
            }

            $(document.body).on(
                "release mouseleave",
                _.bind(
                    function (event) {
                      var self = this;
                      _.defer(function () {
                        self._outEventsStatus &= ~flag;
                      });
                    },
                    this
                )
            );

            $(document.body).on(
                eventName,
                _.bind(
                    function (event) {
                      if (this._outEventsStatus & flag) {
                        _.bind(method, this)(event);
                      }
                    },
                    this
                )
            );
          };
    }
);
define("controllers/out", function(){});

define(
    'controllers/scrollbar',[
      "jquery",
      "underscore",
      "backbone",
      "marionette",
      "template",
      "app",
      "controllers/out"
    ],
    function ($, _, Backbone, Marionette, JST) {
      "use strict";

      var Scrollbar = Marionette.ItemView.extend({
        tagName: "div",
        className: "scrollbarWrapper animate",

        events: {
          "drag @ui.handler": function (event) {
            event.preventDefault();
            event.stopPropagation();

            event = event.originalEvent || event;

            event.gesture.preventDefault();

            var orientation = Marionette.getOption(this, "orientation"),
                prop = (orientation === "horizontal") ? "width" : "height",
                size = this.ui.scrollbar[prop](),
                delta = (orientation === "horizontal") ? event.gesture.deltaX :
                    event.gesture.deltaY,
                progress = this.progress() + delta / size * 100;

            this.progress(progress);
          },

          "dragend @ui.handler": function (event) {
            event.stopPropagation();

            event = event.originalEvent || event;

            event.gesture.preventDefault();

            var orientation = Marionette.getOption(this, "orientation"),
                prop = (orientation === "horizontal") ? "width" : "height",
                size = this.ui.scrollbar[prop](),
                delta = (orientation === "horizontal") ? event.gesture.deltaX :
                    event.gesture.deltaY,
                progress = this.progress() + delta / size * 100;

            this.percentage = this.progress(progress);
          },

          "release @ui.handler": function () {
            this.$el.addClass("animate");
            this.progress(this.percentage);
          },

          "touch @ui.handler": function () {
            this.ensureSize();
            this.$el.removeClass("animate");
          },

          "tap @ui.scrollbar": function (event) {
            event.stopPropagation();

            var center = event.originalEvent.gesture.center,
                $el = this.ui.scrollbar,
                offset = $el.offset(),
                orientation = Marionette.getOption(this, "orientation"),
                percentage = (orientation === "horizontal") ?
                    (center.pageX - offset.left) / $el.width() :
                    (center.pageY - offset.top) / $el.height();

            this.percentage = this.progress(percentage * 100);
          },

          "drag": function (event) {
            event.preventDefault();

            event = event.originalEvent || event;

            event.gesture.preventDefault();

            var orientation = Marionette.getOption(this, "orientation"),
                size = Marionette.getOption(this, "size"),
                delta = (orientation === "horizontal") ? event.gesture.deltaX :
                    event.gesture.deltaY,
                progress = this.progress() - delta / size * 100;

            this.progress(progress);
          },

          "dragend": function () {
            var orientation = Marionette.getOption(this, "orientation"),
                size = Marionette.getOption(this, "size"),
                delta = (orientation === "horizontal") ? event.gesture.deltaX :
                    event.gesture.deltaY;

            this.percentage = this.progress(this.progress() - delta / size * 100);
          },

          "mousewheel": function (event) {
            event = event.originalEvent || event;

            this.ensureSize();

            var orientation = Marionette.getOption(this, "orientation"),
                size = Marionette.getOption(this, "size"),
                progress = this.progress() + event.deltaY * 1.75 / size * 100;

            this.$el.removeClass("animate");
            this.percentage = this.progress(progress);
            this.$el.addClass("animate");
          },

          "release": function () {
            this.$el.addClass("animate");
            this.progress(this.percentage);
          },

//          "swipe": function () {
//            var orientation = Marionette.getOption(this, "orientation"),
//                size = Marionette.getOption(this, "size"),
//                delta = (orientation === "horizontal") ? event.gesture.deltaX :
//                    event.gesture.deltaY,
//                force = (orientation === "horizontal") ? event.gesture.velocityX :
//                    event.gesture.velocityY;
//            force = Math.abs(force + 1);
//
//            this.$el.addClass("animate");
//
//            this.percentage =
//                this.progress(this.progress() - delta * force / size * 100);
//          },

          "touch": function () {
            this.ensureSize();
            this.$el.removeClass("animate");
          }
        },

        ui: {
          bar: ".bar",
          content: ".content",
          handler: ".handler",
          scrollbar: ".scrollbar"
        },
        template: JST.scrollbar,

        onRender: function () {
          var position = Marionette.getOption(this, "position");

          switch (position) {
            case "top":
            case "right":
            case "bottom":
              this.ui.scrollbar.css(position, 0);
              break;

            default:
              this.ui.scrollbar.css("left", 0);
              break;
          }
          this.ui.scrollbar.addClass(Marionette.getOption(this, "orientation"));
          this.ui.content.html(Marionette.getOption(this, "content"));

          this.progress(0);
        },

        initialize: function (options) {
          this.options = _.extend({
            content: options.$parent.children(),
            orientation: (_.contains(["top", "bottom"], options.position)) ?
                "horizontal" : "vertical",
            position: "right"
          }, options);

          this.model = new Backbone.Model(this.options);

          this.on("dom:refresh", this.display);
        },

        percentage: 0,
        outEvents: true,

        display: function () {
          var $last = this.ui.content.children(":last"),
              $parent = this.$el,
              position = Marionette.getOption(this, "position"),
              properties = (_.contains(["top", "bottom"], position)) ?
                  ["left", "width"] : ["top", "height"],
              offset = parseInt($parent.offset()[properties[0]], 10),
              size = parseInt($parent[properties[1]](), 10),
              lastOffset = ($last.size()) ?
                  parseInt($last.offset()[properties[0]], 10) : 0,
              lastSize = ($last.size()) ? parseInt($last[properties[1]](), 10) : 0;

          if (!$last.size() || offset + size >= lastOffset + lastSize) {
            this.ui.scrollbar.hide();
          } else {
            this.ui.scrollbar.show();
          }

          this.$el.trigger("release");
        },

        progress: function (percentage) {
          var steps = Marionette.getOption(this, "steps");
          this.ensureSize();

          percentage = Math.min(Math.max(percentage, 0), 100);

          if (steps) {
            percentage = Math.round(percentage / (100 / steps)) * (100 / steps);
          }

          if (_.isNaN(percentage)) {
            return this.percentage;
          }

          var orientation = Marionette.getOption(this, "orientation"),
              size = Marionette.getOption(this, "size"),
              barProp = (orientation === "horizontal") ? "width" : "height",
              contentProp = (orientation === "horizontal") ? "margin-left" :
                  "margin-top";
          percentage = +Math.max(Math.min(percentage, 100), 0).toFixed(2);

          if (size < 0) {
            return false;
          }

          this.ui.content.css(
              contentProp,
              parseInt(-size * percentage / 100, 10) + "px"
          );
          this.ui.bar.css(barProp, percentage + "%");

          this.trigger("progress", percentage);
          return percentage;
        },

        ensureSize: function () {
          var orientation = Marionette.getOption(this, "orientation"),
              prop = (orientation === "horizontal") ? "width" : "height";
          this.size = parseInt(this.ui.content[prop]() - this.$el[prop](), 10);
        }
      });

      Marionette.View.prototype.isScrollbar = function (el) {
        return $(el).hasClass("scrollbarWrapper");
      };

      Marionette.View.prototype.renderScrollbar = function () {
        if (this.scrollbar) {
          if (_.isString(this.scrollbar)) {
            this.scrollbar = {parent: this.scrollbar};
          }

          this.scrollbar = $.extend({
            parent: this.$el,
            position: "right"
          }, this.scrollbar);

          var $parent = this.$(this.scrollbar.parent);
          $parent = ($parent.length) ? $parent : this.scrollbar.parent;

          this.scrollbar = _.extend(this.scrollbar, {
            $parent: $parent,
            tagName: $parent.get(0).tagName
          });

          this.on("show", function () {
            var scrollbar = new Scrollbar(this.scrollbar).render(),
                self = this;
            this.scrollbar.$parent.replaceWith(scrollbar.$el);

            this.on("dom:refresh", function () {
              scrollbar.triggerMethod("dom:refresh");
            });

            scrollbar.display();
            scrollbar.on("progress", function (progress) {
              self.trigger("progress", progress);
            });
          });
        }

        return null;
      };

      (function (render) {
        Marionette.ItemView = Marionette.ItemView.extend({
          render: function () {
            this.on("render", this.renderScrollbar);

            return render.apply(this, Array.prototype.slice.call(arguments));
          }
        });
      })(Marionette.ItemView.prototype.render);

      (function (render) {
        Marionette.CollectionView = Marionette.CollectionView.extend({
          render: function () {
            this.on("render", this.renderScrollbar);

            return render.apply(this, Array.prototype.slice.call(arguments));
          }
        });
      })(Marionette.CollectionView.prototype.render);

      (function (getItemViewContainer, render) {
        Marionette.CompositeView = Marionette.CompositeView.extend({
          getItemViewContainer: function (containerView) {
            if (this.scrollbar && containerView.$(".scrollbar") &&
                containerView.itemViewContainer === containerView.scrollbar.parent) {

              return $(this.scrollbar.parent + " .content");
            }

            return getItemViewContainer.apply(
                this,
                Array.prototype.slice.call(arguments)
            );
          },
          render: function () {
            this.on("render", this.renderScrollbar);

            return render.apply(this, Array.prototype.slice.call(arguments));
          }
        });
      })(
              Marionette.CompositeView.prototype.getItemViewContainer,
              Marionette.CompositeView.prototype.render
          );
    }
);
// noinspection JSHint
define(
    'helpers/backbone.sync',["jquery", "underscore", "backbone", "marionette", "app"],
    function ($, _, Backbone, Marionette, App) {
      "use strict";

      Backbone.Model.prototype.parse = function (data) {
        return _.isString(data) ? JSON.parse(data) : data;
      };

      var records = {},
          OnlineStorage = Marionette.Controller.extend({
            receiver: $(window),
            store: window.parent,

            get: function (key) {
              /* Timeout 10s */
              var deferred = $.Deferred(),
                  receiver = this.receiver,
                  timeout = _.delay(function () {
                    deferred.reject("timeout");
                  }, 10000);

              receiver.on("message." + key, function (event) {
                event = event.originalEvent || event;

//                console.debug("READ", event, key, event.data.info);

                if (event.data.key === key) {
                  deferred.resolve(event.data.info);
                  try {
                    receiver.off("." + key);
                  } catch (err) {
                    console.info("catch small error");
                  }
                  clearTimeout(timeout);
                }
                  });

              this.query(key);

              return deferred.promise();
            },

            query: function (key, value) {
              if (_.isUndefined(value)) {
                this.store.postMessage(
                    {type: "getProductData", key: key},
                    "*"
                );
              } else {

                this.store.postMessage(
                    {type: "setProductData", key: key, info: value},
                    "*"
                );
              }
            },

            set: function (key, value) {
//              console.debug("WRITE", key, value);

              this.query(key, value);
            },

            unset: function (key) {
              return this.set(key, null);
            }
          }),
          OfflineStorage = Marionette.Controller.extend({
            advised: false,
            persistent: false,
            store: null,

            initialize: function () {
              if (!window.localStorage && !this.advised) {
                this.advised = true;
                App.execute("warning", "Este navegador no soporta localStorage");
              }

              this.persistent = !!window.localStorage;
              this.store = window.localStorage || window.Storage;
            },

            get: function (key) {
              var deferred = $.Deferred(),
                  prefix = Marionette.getOption(this, "prefix") + "/";
              key = prefix + key;

//              console.log(
//                  "READ",
//                  key,
//                  (this.persistent) ? this.store.getItem(key) : this.store[key]
//              );

              deferred.resolve(
                  (this.persistent) ? this.store.getItem(key) : this.store[key]
              );

              return deferred.promise();
            },

            set: function (key, value) {
              var prefix = Marionette.getOption(this, "prefix") + "/";
              key = prefix + key;

//              console.log("WRITE", key, value);

              if (this.persistent) {
                this.store.setItem(key, value);
              } else {
                this.store[key] = value;
              }
            },

            unset: function (key) {
              var prefix = Marionette.getOption(this, "prefix") + "/";
              key = prefix + key;

              if (this.persistent) {
                this.store.removeItem(key);
              } else {
                delete this.store[key];
              }
            }
          }),
          MyStorage = Marionette.Controller.extend({
            store: null,

            initialize: function (options) {
              _.defaults(options || (options = {}), {
                local: undefined
              });

              App.request("is:av", {context: this, test: options.local})
                  .done(function () {
                    this.store = new OnlineStorage();
                  })
                  .fail(function () {
                    App.request("GEDI", {context: this}).done(function (GEDI) {
                      this.store = new OfflineStorage({prefix: GEDI});
                    });
                  });
            },

            get: function (key) {
              return this.store.get(key);
            },

            set: function (key, value) {
              return this.store.set(key, value);
            },

            unset: function (key) {
              return this.store.unset(key);
            }
          });

      Backbone.sync = function (method, model, options) {
        var collection, deferred,
            local = _.result(model, "local"),
            path = _.result(model, "url"),
            response,
            store = new MyStorage({local: local});

        if (!path) {
          throw new Error("A \"url\" property or function must be specified");
        }

        _.defaults(options || (options = {}), {
        });

        try {
          switch (method) {
            case "read":
              response = store.get(path);

              if (_.isUndefined(model.id)) {
                // init a deferred resolved
                collection = $.Deferred().resolve();

                response = response.then(function (data) {
                  if (!data) {
                    records[path] = [];
                    return this;
                  }

                  data = data.split(",");


                  // ==================================================================
                  // === Fix temporal para Orden de Trabajo #2718 en la que         ===
                  // === no se cargaban los dibujos guardados en la pagina derecha. ===
                  // === Se debe a que AV solo me devuelve un numero pagina, en     ===
                  // === vez de los dos. Lo agrego a mano...                        ===
                  // ==================================================================
                  var allNumeric = true;
                  var tmp = _.clone(data);
                  var newData = [];

                  _.each(tmp, function(pageNum){
                    if(isNaN(pageNum)){
                      allNumeric = false;
                    }
                  });

                  if(allNumeric){
                    _.each(tmp, function(pageNum){
                      newData.push(parseInt(pageNum), parseInt(pageNum) + 1);
                    });
                  }

                  if(!!newData.length){
                    newData = _.uniq(newData);
                    data = newData;
                  }
                  // ==================================================================
                  // ==================================================================


                  records[path] = data.slice(0);

                  // We used collection for start a chain of deferred.
                  _.each(data, function (value, key, list) {
                    collection = collection.then(function () {
                      return store.get(path + "/" + value)
                          .done(function (data) {
                            list[key] = data;
                          });
                    }, function () {
                      return store.get(path + "/" + value);
                    });
                  });

                  // Once it's done, we resolve it with the list.
                  collection = collection.then(function () {
                    return $.Deferred().resolve(_.compact(data));
                  }, function () {
                    return $.Deferred().resolve(_.compact(data));
                  });

                  // And return a promise. (Same as deferred)
                  return collection;
                });
              }
              break;

            case "create":
              // setting id
              model.set("id", Math.round(Math.random() * 100));
              path = _.result(model, "url");

              store.set(path, JSON.stringify(model.toJSON()));
              var response;
              setTimeout(function () {
                  response = store.get(path);
              }, 100);
              break;

            case "update":
              store.set(path, JSON.stringify(model.toJSON()));
              var response;
              setTimeout(function () {
                  response = store.get(path);
              }, 100);
              break;

            case "delete":
              // In this case, we simulate an order because it has no response.
              deferred = $.Deferred();

              store.unset(path);
              deferred.resolve(model);

              response = deferred.promise();
              break;
          }

          if (model.collection) {
            collection = model.collection;
            path = _.result(collection, "url");

            if (!path) {
              throw new Error("A \"url\" property or function must be specified");
            }

            if (method !== "delete") {
              var id = parseInt(model.id, 10) || model.id;
              //noinspection JSHint
              !records[path] && (records[path] = []);
              records[path].push(id);
              records[path] = _.uniq(records[path]);
            } else {
              records[path] = _.without(records[path], model.id);
            }

            store.set(path, records[path].join(","));
          }

          response.done(options.success);
        } catch (Exception) {
          options.error();
        }

        if (_.isFunction(options.complete)) {
          options.complete(response);
        }

        return response;
      };
    }
);
// noinspection JSHint
require(
    ["marionette"],
    function (Marionette) {
      "use strict";

      window.Behaviors = {};

      Marionette.Behaviors.behaviorsLookup = function() {
        return window.Behaviors;
      };
    }
);
define("helpers/behaviors", function(){});

// noinspection JSHint
define(
    'helpers/console',["jquery", "underscore", "app", "configuration"],
    function ($, _, App, configuration) {
      "use strict";

      var message = function () {
        return _.map(arguments, function (value) {
          if (_.isObject(value)) {
            value = JSON.stringify(value, function() { return value.toString(); });
          }

          return value;
        })
            .join(" ");
      };

      if (!configuration.develop) {
        $(window).on("error", function () {
          window.console.error.apply(window.console, arguments);
        });

        _.extend(window.console, {
          debug: function () {
            // removing message
          },

          error: function () {
            App.execute("error", message.apply(this, arguments));
          },

          log: function () {
            // removing every message
          },

          warn: function () {
            App.execute("warning", message.apply(this, arguments));
          }
        });
      }
    }
);
// noinspection JSHint
define(
    'helpers/cordova',["jquery", "../app"],
    function ($, App) {
      "use strict";

      $(document)
          .on("backbutton", function () {
            App.execute("quit");
          });
    }
);
//noinspection JSHint
define(
    'helpers/handlebars',["handlebars", "language"],
    function (Handlebars, language) {
      "use strict";

      Handlebars.registerHelper('language', function (cadena) {
        return language(cadena);
      });
    }
);
// noinspection JSHint
define(
    'helpers/scrollbar',[
      "underscore",
      "backbone",
      "marionette",
      "regions/prepend",
      "template",
      "controllers/out"
    ],
    function (_, Backbone, Marionette, PrependRegion, JST) {
      "use strict";

      // Debo agregar un delay porque este archivo  se ejecuta antes
      // que la App y no puedo hacer request para saber si es una tablet
      _.delay(function(){

        var isTablet = false;
        App.request("is:device")
          .done(function(){ isTablet = true; });

        var BarView = Marionette.ItemView.extend({
          /* attributes */
          // backbone & marionette
          
          className: isTablet ? "scrollbar" : "scrollbar animate",
          events: {
            "dragg": function (event) {
              event = event.originalEvent || event;

              event.preventDefault();
              event.stopPropagation();

              var $el = this.$el,
                property = this.model.get("property"),
                data = event.data.touches || event.data;

              this.offset = this.scroll(
                ($(data).attr(property.page) -
                $el.offset()[property.offset] - 25) /
                ($el[property.size]()) * 100
              );
              this.trigger("scroll", this.offset);
            },

            "release @ui.handler": function () {
              if(!isTablet){
                this.$el.addClass("animate");
              } 

              this.scroll(this.offset);
            },

            "touch @ui.handler": function () {
              if(!isTablet){
                this.$el.removeClass("animate");
              }
            },

            "tap": function (event) {
              event = event.originalEvent || event;

              event.stopPropagation();
              event.preventDefault();

              var $el = this.$el,
                  property = this.model.get("property");

              this.offset = this.scroll(
                  ($(event).attr(property.page) -
                      $el.offset()[property.offset] - 25) /
                      ($el[property.size]()) * 100
              );
              this.trigger("scroll", this.offset);
            }
          },
          tagName: "aside",
          template: JST.scrollbar,
          ui: {
            "bar": ".bar",
            "handler": ".handler"
          },

          // custom
          offset: 0,
          outEvents: true,

          /* methods */
          // backbone & marionette
          onRender: function () {
            this.$el.addClass(this.model.get("position"));
            this.el.children[0].children[0].className = this.el.children[0].children[0].className + " drag";
          },

          // custom
          scroll: function (percentage) {
            var property = this.model.get("property"),
                steps = this.model.get("steps");

            percentage = Math.min(Math.max(percentage, 0), 100);
            if (steps) {
              percentage = Math.round(percentage / (100 / steps)) * (100 / steps);
            }
            percentage = Math.round(percentage);

            if (percentage === this.offset) {
              return this.offset;
            }

            this.ui.bar.css(property.size, percentage + "%");

            return percentage;
          }
        });

        window.Behaviors.Scrollbar = Marionette.Behavior.extend({
          defaults: {
            color: 8,
            object: null,
            offset: 0,
            position: "right",
            steps: null
          },
          events: {
            "mousewheel": function (event) {
              var container = this.options.container,
                  size = this.options.size;

              event = event.originalEvent || event;

              event.stopPropagation();

              if (this.size() < 0 ||
                  (!container.find(event.target).length &&
                      container !== event.target)) {
                return false;
              }

              event.preventDefault();

              !isTablet && this.options.object && this.options.object.$el.removeClass("animate");
              clearTimeout(this.timeout);
              this.view.offset = this.scroll(
                  (this.view.offset + event.wheelDeltaY / size * -100)
              );
              this.view.trigger("scroll", this.view.offset);
              this.timeout = _.chain(function () {
                !isTablet && this.options.object && this.options.object.$el.addClass("animate");
              })
                  .bind(this)
                  .delay(5)
                  .value();

              return true;
            }
          },

          initialize: function () {
            var isVertical = _.contains(["left", "right"], this.options.position),
                properties = {
                  horizontal: {
                    delta: "deltaX",
                    offset: "left",
                    page: "pageX",
                    size: "width",
                    scroll: "margin-left",
                    scrollSize: "scrollWidth"
                  },
                  vertical: {
                    delta: "deltaY",
                    offset: "top",
                    page: "pageY",
                    scroll: "margin-top",
                    scrollSize: "scrollHeight",
                    size: "height"
                  }
                };

            _.defaults(this.options || (this.options = {}), {
              orientation: (isVertical) ? "vertical" : "horizontal",
              property: properties[(isVertical) ? "vertical" : "horizontal"]
            });

            this.view
                .on("dom:refresh", this.scrollbar, this)
                .on("show", this.scrollbar, this)
                .on("itemview:show", this.scrollbar, this)
                .on("item:removed", this.scrollbar, this);
          },

          onRender: function () {
            var container = this.options.container;

            _.extend(this.options, {
              container: (this.$(this.options.container).length) ?
                  this.$(this.options.container) : this.$el
            });
          },

          scroll: function (percentage) {
            // plus operator is used to casting a number
            percentage = +Math.min(Math.max(percentage, 0), 100).toFixed(2);

            if (percentage === this.view.offset) {
              return this.view.offset;
            }

            this.options.container.children(":not(.scrollbar)").first().css(
                this.options.property.scroll,
                -Math.round(this.options.size * percentage / 100)
            );

            return percentage;
          },

          scrollbar: function () {
            var options = this.options,
                container = options.container,
                properties = options.property;

            if (!this.options.object && this.size() > 0) {
              _.defaults(this.options || (this.options = {}), {
                size: this.size()
              });

              var region = new PrependRegion({el: container}),
                  view = this.options.object =
                      new BarView({model: new Backbone.Model(options)});
              this.view.offset = 0;

              region.show(view);
              view.scroll(this.options.offset);

              this.view.scroll = _.bind(function(offset) {
                this.view.offset = this.scroll(offset);
                view.offset = view.scroll(offset);
              }, this);

              view.on("scroll", function (offset) {
                this.view.offset = this.scroll(offset);
                this.view.trigger("scroll:after", offset);
              }, this);
              this.view
                  .on("scroll", _.debounce(function (offset) {
                    var foo = this.scroll(offset);
                    if(foo){
                      this.offset = foo;
                    }
                  }, 25), view)
                  .on("scroll", function(offset) {
                    this.view.trigger("scroll:after", offset);
                  }, this);
            } else if (this.size() > 0) {
              this.options.size = this.size();
            } else if (this.options.object) {
              this.options.object.close();
              this.options.object = null;
            }
          },

          size: function () {
            var container = this.options.container,
                properties = this.options.property;

            return container.get(0)[properties.scrollSize] -
                container[properties.size]();
          }
        });
      }, 50);
    }
);
// noinspection JSHint
define(
    'helpers/underscore',["underscore"],
    function (_) {
      "use strict";

      _.mixin({
        avg: function(a) {
          return _.sum(a) / a.length;
        },

        capitalize: function(s) {
          return s.charAt(0).toUpperCase() + s.slice(1).toLowerCase();
        },

        compactObject: function (o) {
          _.each(o, function (v, k) {
            if (!v) {
              delete o[k];
            }
          });
          return o;
        },

        encodeUnicode: function(text) {
          var unicodeString = '';
          for (var i=0; i < text.length; i++) {
            var theUnicode = text.charCodeAt(i).toString(16).toUpperCase();
            while (theUnicode.length < 4) {
              theUnicode = '0' + theUnicode;
            }
            theUnicode = '\\u' + theUnicode;
            unicodeString += theUnicode;
          }
          return unicodeString;
        },

        decodeUnicode: function (text){
          var r = /\\u([\d\w]{4})/gi;
          text = text.replace(r, function (match, grp) {
              return String.fromCharCode(parseInt(grp, 16)); } );
          text = unescape(text);
          return(text);
        },

        sum: function(a) {
          return _.reduce(a, function(r, v) {
            return r + v;
          }, 0);
        }
      });
    }
);
//noinspection JSHint
require(
    [
      // libraries
      "jquery",
      "underscore",
      "backbone",
      // app
      "app",
      // options
      "configuration",
      // helpers
      "regions/replace",
      // commands & requests
      "command",
      "region",
      "request",
      // more helpers
      "controllers/out",
      "controllers/scrollbar",
      "helpers/backbone.sync",
      "helpers/behaviors",
      "helpers/console",
      "helpers/cordova",
      "helpers/handlebars",
      "helpers/scrollbar",
      "helpers/underscore"
    ],
    function ($, _, Backbone, App, configuration) {
      "use strict";

    var pointer = function(event) {
      try {
      var canvas = document.getElementsByTagName("canvas")[0],
        rect = canvas.getBoundingClientRect();
      return {
        x: (event.pageX-rect.left)/(rect.right-rect.left)*canvas.width,
        y: (event.pageY-rect.top)/(rect.bottom-rect.top)*canvas.height
      };
      } catch (err) {
        return {x:0,y:0};
      }
    };

    var updateScrollPos = function(e) {
      var viewer = document.querySelector("ul#viewer"),
        $viewer = $(viewer),
        point = pointer(e),
        diffY = click.y - point.y,
        diffX = click.x - point.x;
      if ($viewer.hasClass("draggable")) {
        _.each(viewer.childNodes, function(pag) {
          if (!$viewer.hasClass("fit")) {
            pag.style.left = parseInt(pag.style.left||0) - diffX + "px";
          }
          pag.style.top = parseInt(pag.style.top||0) - diffY + "px";
        });
      }
    };

    var clicked = false, click;
    $(document).on({
      'mousemove': function(e) {
        if (!$(document.body).hasClass("kinetic")) {
          clicked && updateScrollPos(e);
        }
      },
      'mousedown': function(e) {
        if (!$(document.body).hasClass("kinetic")) {
          clicked = true;
          click = pointer(e);
          $('html').css('cursor', 'pointer');
        }
      },
      'mouseup mouseout': function(e) {
        if (!$(document.body).hasClass("kinetic")) {
          clicked = false;
          $('html').css('cursor', 'auto');
        }
      }
    });

      /**
       * Añade los eventos de swipe hipernativos.
       * · swl -> swipe from right to left
       * · swr -> swipe from left to right
       * · swu -> swipe from down to up
       * · swd -> swipe from up to down
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
    var
      _dragElement,
      ce=function(e,n,target){
          var a=document.createEvent("CustomEvent");
          target = target || e.target;
          a.initCustomEvent(n,true,true,target);
          a.data = e;
          return target.dispatchEvent(a);
        },
        nm=true,sp={x:0,y:0},dblsp={x:0,y:0},
        time = 0,
        start=function(e){

            e = e || window.event;
          nm=true;
          sp.x=e.touches[0].pageX;
          sp.y=e.touches[0].pageY;
          // -----------------------
          // Drag
          var target = e.target != null ? e.target : e.srcElement;
          _dragElement = target;
          // -----------------------
        },
        startd=function(e){
          e = e || window.event;
          nm=true;
          sp.x=e.pageX;
          sp.y=e.pageY;
          // -----------------------
          // Drag
          var target = e.target != null ? e.target : e.srcElement;
          _dragElement = target;
          // -----------------------

        },
        move=function(e){
          e = e || window.event;
          nm=false;
          // -----------------------
          // Drag
          var target = e.target != null ? e.target : e.srcElement;
          if (_dragElement) {
            ce(e,"dragg",_dragElement);
            if (navigator.userAgent.match(/Android/i) && (target.className.indexOf("drag")!=-1 || target.className.indexOf("scrollbar")!=-1 || target.className.indexOf("bar")!=-1 || target.className.indexOf("handler")!=-1)) {
              e.preventDefault();
            }
          }
        },
        end=function(e){
          var x=e.changedTouches[0].pageX-sp.x,xr=Math.abs(x),
            y=e.changedTouches[0].pageY-sp.y,yr=Math.abs(y);
          if(nm){
            if (time && Date.now() - time < 300 && Math.abs(e.changedTouches[0].pageX-dblsp.x) < 50 && Math.abs(e.changedTouches[0].pageY-dblsp.y) < 50) {
              ce(e,'dbltouch');
            }
            else {
              ce(e,'tap');
              e.stopPropagation();
              ce(e,'touch');
              dblsp.x=e.changedTouches[0].pageX;
              dblsp.y=e.changedTouches[0].pageY;
            }
          }
          else{
            //if(Math.max(xr,yr)>window.innerWidth*0.12){
              ce(e,(xr>yr?(x<0?'swipeleft':'swiperight'):(y<0?'swipeup':'swipedown')));
            //}
          };
          time = Date.now();
          nm=true;
          // -----------------------
          // Drag
          if (e.type != "touchcancel") {
            _dragElement = null;
            e.stopPropagation();
          }
          // -----------------------
        },
        endd=function(e){

          if (e.type == "click") {
            _dragElement = null;
            ce(e,'tap');
            e.stopPropagation();
          }
          else {
            var x=e.pageX-sp.x,xr=Math.abs(x),
              y=e.pageY-sp.y,yr=Math.abs(y);
            if(nm){
              ce(e,'touch');
              ce(e,'tap');
              /*if (time && Date.now() - time < 300) {
               //ce(e,'dblclick'); this triggers automatically!!
               }
               else {
               ce(e,'tap');
               }*/
              _dragElement = null;
            }
            else if(Math.max(xr,yr)>window.innerWidth*0.12){
              ce(e,(xr>yr?(x<0?'swipeleft':'swiperight'):(y<0?'swipeup':'swipedown')));
            };
            nm=true;
            time = Date.now();
            // -----------------------
            // Drag
            _dragElement = null;
            e.stopPropagation();
            // -----------------------
          }
        };

    App.request("is:device")
      .done(function(isDevice){
        document.addEventListener("touchstart",start,false);
        document.addEventListener("touchmove",move,false);
        document.addEventListener("touchend",end,false);
        if (isDevice.indexOf("Android") > -1) {
          document.addEventListener("touchcancel",end,false);
        }
        else {
          document.addEventListener("touchcancel",move,false);
        }
      })
      .fail(function(){
        document.addEventListener("mousedown",startd,false);
        document.addEventListener("mousemove",move,false);
        document.addEventListener("mouseup",endd,false);
        document.addEventListener("click",endd,false);
      })
      .always(function(){
        startd = start = move = end = endd = null;
      });

    $(function () {
      $(window).on("resize", _.debounce(function () {
        /**
         * FIX: En dispositivos android no se sencesita el resize de las páginas y con esto se arregla el problema de que los cuadros de texto no se puedan editar
         * @author Mario Poyato <mario.poyato@inmediastudio.es>
         */
        App.request("is:device")
          .done(function (type) {
            if(type[0] != "Android"){
              App.trigger("resize");
            }
          })
          .fail(function(type) {
            App.trigger("resize");
          });
      }, 800));

      // Detectar cambio de orientacion en móviles
      window.addEventListener('orientationchange', function() {
        window.orientation_changed = true;
      });

      $(document.body).on("keydown", function (event) {
        var targetType = event.target.tagName.toLowerCase(),
          textFields = ["input", "textarea"];

        event = event.originalEvent || event;

        if (event.keyCode === 8 && !_.contains(textFields, targetType)) {
          event.preventDefault();
        }

        if (event.keyCode === 27) {
          App.execute("quit");
        }
      });

      App.start(configuration);
      App.execute("module:start", "loader");
    });
  }
);
define("index", function(){});

define('modules/favorites/template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["edit"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, options, functionType="function", escapeExpression=this.escapeExpression, helperMissing=helpers.helperMissing;


  buffer += "<input type=\"text\" value=\"";
  if (stack1 = helpers.title) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.title); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\" />\n<button class=\"bgcolor-3\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "accept", options) : helperMissing.call(depth0, "language", "accept", options)))
    + "</button>";
  return buffer;
  });

this["JST"]["panel"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, options, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;


  buffer += "<button>";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "edit", options) : helperMissing.call(depth0, "language", "edit", options)))
    + "</button>\n<section>\n    <a class=\"thumbnail\">\n        <div class=\"wrapper\">\n            <i class=\"icon-star color-3\"></i>\n        </div>\n        <span>";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "add", options) : helperMissing.call(depth0, "language", "add", options)))
    + "</span>\n    </a>\n    <nav>\n    </nav>\n</section>\n";
  return buffer;
  });

return this["JST"];

});
// noinspection JSHint
define(
    'modules/favorites/views/edit',["marionette", "modules/favorites/template"],
    function (Marionette, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        className: "favorites",
        events: {
          "keydown @ui.input": function(event) {
            event = event.originalEvent || event;
            event.stopPropagation();

            if (event.keyCode === 13) {
              this.ui.button.trigger("tap");
            }
          },

          "tap button": function(event) {
            event = event.originalEvent || event;

            event.preventDefault();
            event.stopPropagation();

            this.model.set("title", this.ui.input.val());
            this.model.save();

            this.trigger("quit");
          }
        },
        modelEvents: {
          'change': 'refresh'
        },
        refresh: function() {
          this.render();
        },
        tagName: "main",
        template: JST.edit,
        ui: {
          "button": "button",
          "input": "input"
        }
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/helpers/marionette.view',["underscore", "marionette"],
    function (_, Marionette) {
      "use strict";

      Marionette.Controller.prototype.delay =
          Marionette.View.prototype.delay = function (callback, time) {
            return _.chain(callback)
                .bind(this)
                .delay(time)
                .value();
          };
    }
);
//noinspection JSHint
define(
    'modules/viewer/viewer',["app", "regions/replace", "modules/viewer/helpers/marionette.view"],
    function (App, ReplaceRegion) {
      "use strict";

      return App.module(
          "viewer",
          function (module, App, Backbone, Marionette, $, _) {
            /* Presets module */
            this.startWithParent = false;

            /* Public  */

            // Layout
            this.layout = (function () {
              var RegionManager = new Marionette.RegionManager();
              RegionManager.addRegions({
                bookmark: "#index",
                main: "#viewer",
                controls: App.footer.el
              }, {regionType: ReplaceRegion});

              return RegionManager;
            })();

            // Constructor
            this.addInitializer(function (options) {
              this.options = _.defaults(options || (options = {}), {
                bookmark: true,
                controls: true,
                module: {}
              });

              // Events
              this.on("load:after", function (page) {
                _.each(
                    _.compactObject(_.pick(options, "bookmark", "controls")),
                    function (value, key) {
                      App.execute(
                          "module:start",
                          "viewer." + key,
                          _.extend({
                            page: page
                          }, options.module[key])
                      );
                    }
                );

                App.trigger("menu:show");
              });

              // initialize
              App.request("is:device", {boolean: true, context: this})
                  .done(function (isDevice) {
                    // this === module
                    this.options.controls = options.controls = !isDevice || (navigator.platform.match("Win") && ('ontouchstart' in window || navigator.msMaxTouchPoints));
                  })
                  .always(function () {
                    App.execute(
                        "module:start",
                        "viewer.document",
                        options.module.document
                    );
                  });
            });
          }
      );
    }
);
//noinspection JSHint
define(
    'views/thumbnail',[
      "marionette",
      "configuration",
      "modules/viewer/viewer",
      "template"
    ],
    function (Marionette, configuration, viewer, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        className: "thumbnail",
        events: {
          "tap": "goto"
        },
        tagName: "a",
        template: JST.thumbnail,
        templateHelpers: {
          blank: configuration.root + "assets/images/blank.png",
          path: configuration.thumbnails
        },

        onRender: function () {
          this.$("img").css(
              "background",
              "url(" + configuration.root + "assets/images/custom/" +
                  this.model.get("type") + ".png)"
          );
        },

        goto: function () {
          viewer.trigger("goto", parseInt(this.model.get("id"), 10));
        }
      });

    }
);
// noinspection JSHint
define(
    'modules/favorites/views/thumbnail',[
      // app
      "views/dialog",
      "modules/favorites/views/edit",
      "views/thumbnail",
      // helpers
      "language"
    ],
    function (DialogView, EditView, ThumbnailView, language) {
      "use strict";

      return ThumbnailView.extend({
        className: "thumbnail norender",
        events: {
          "tap aside": function (event) {
            event.stopPropagation();

            var dialog = new DialogView({
              title: language("favorites.changeName"),
              color: 3,
              width: "400px",
              height: "auto"
            });
            dialog.show(new EditView({model: this.model}));
          },

          "tap": "goto"
        },
        modelEvents: {
          "change": "render"
        },
        triggers: {
          "tap i": {
            event: "remove",
            preventDefault: true,
            stopPropagation: true
          }
        },

        onShow: function() {
          _.chain(function() {
            this.$el.removeClass("norender");
          })
              .bind(this)
              .defer()
              .value();
        }
      });
    }
);
// noinspection JSHint
define(
    'modules/favorites/views/index',[
      // libraries
      "backbone",
      "marionette",
      // app
      "modules/favorites/views/thumbnail",
      "modules/favorites/template",
      // helpers
      "language"
    ],
    function (Backbone, Marionette, ThumbnailView, JST, language) {
      "use strict";

      return Marionette.CompositeView.extend({
        className: "panel norender",
        events: {
          "touch @ui.button": function () {
            var editable;

            this.model.set("editable", !this.model.get("editable"));
            editable = this.model.get("editable");

            this.$el.toggleClass("editable", editable);
            this.ui.button
                .text((editable) ? language("close") : language("edit"))
                .toggleClass("active");

            // Fix para tablets low-cost
            if(!navigator.userAgent.match(/iPhone|iPad|iPod/i)) {
                // Fix para tablets low-cost puesto en cuarentena
                  $('#favorites .thumbnail').hide();
                  _.delay(function(){
                  $('#favorites .thumbnail').show();
                  }, 1);
              }
          }
        },
        id: "favorites",
        tagName: "section",
        template: JST.panel,

        itemView: ThumbnailView,
        itemViewContainer: "nav",
        triggers: {
          "touch @ui.add": "add"
        },
        ui: {
          "add": "section > a",
          "button": "button"
        },

        appendHtml: function (collectionView, itemView, index) {
          var childrenContainer = collectionView.$(collectionView.itemViewContainer),
              children = childrenContainer.children();

          if (children.size() <= index) {
            childrenContainer.append(itemView.el);
          } else {
            children.eq(index).before(itemView.el);
          }
        },

        initialize: function () {
          this.model = new Backbone.Model({
            editable: false
          });
        }
      });
    }
);
//noinspection JSHint
define(
    'modules/favorites/favorites',[
      // app
      "app",
      "modules/favorites/views/index",
      "modules/favorites/models/page.collection"
    ],
    function (App, IndexView, FavoritesCollectionModel) {
      "use strict";

      return App.module(
          "favorites",
          function (module, App, Backbone, Marionette, $, _) {
            /* Presets */
            this.startWithParent = false;

            /* Private */
            var favoritesCollection;

            /* Public */

            // Layout
            this.layout = App.panel;

            // Constructor
            this.addInitializer(function () {
              App.request("favorites", {context: this, cache: false})
                  .done(function (collection) {
                    // Fix para AV Online en el que se nos entrega un modelo con id=0
                    var borrar = collection.where({id: 0});
                    collection.remove(borrar);

                    // Fix para paginas repetidas al agregar favoritos.
                    var fixedCollection = new FavoritesCollectionModel(collection.models);

                    var view = new IndexView({
                      collection: fixedCollection
                    });
                    favoritesCollection = fixedCollection;

                    view
                        .on("add", this.add)
                        .on("itemview:remove", function (view) {
                          this.remove.call(
                              this,
                              {page: view.model.get("id")}
                          );
                        }, this);

                    this.layout.show(view);

                    // init
                    _.chain(function () {
                      this.layout.$el.removeClass("norender");
                    })
                        .bind(this)
                        .delay(150)
                        .value();
                  });
            });

            // Destructor
            this.addFinalizer(function () {
              this.layout.close();
            });

            // Custom
            this.add = function (options) {
              var deferred = $.Deferred();

              _.defaults(options || (options = {}), {
                context: deferred
              });

              App.request(
                  "page",
                  {context: favoritesCollection, model: true, number: options.page}
              )
                  .done(function (page) {
                    deferred.resolveWith(
                        options.context,
                        [this.create(page.toJSON())]
                    );

                    // Fix para tablets low-cost
                    $('#favorites button').hide();
                    _.delay(function(){
                      $('#favorites button').show();
                    }, 1);
                  })
                  .fail(function() {
                    deferred.rejectWith(options.context);
                  });

              return deferred.promise();
            };

            this.remove = function (options) {
              var deferred = $.Deferred();

              _.defaults(options || (options = {}), {
                context: deferred
              });

              App.request(
                  "page",
                  {context: favoritesCollection, model: true, number: options.page}
              )
                  .done(function (page) {
                    this.get(page.get("id")).destroy();
                    deferred.resolveWith(options.context);
                  })
                  .fail(function() {
                    deferred.rejectWith(options.context);
                  });

              return deferred.promise();
            };

            // Requests
//            App.reqres.removeHandler("favorites:add");
//            App.reqres.removeHandler("favorites:remove");
            App.reqres.setHandlers({
              "favorites:add": this.add,
              "favorites:remove": this.remove
            });
          }
      );
    }
)
;
define('modules/fullscreen_worktools/template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["activity"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression, self=this;

function program1(depth0,data) {
  
  var buffer = "", stack1;
  buffer += "\n        	<img alt=\"\" src=\"";
  if (stack1 = helpers.srcImage) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.srcImage); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\" onerror=\"this.src='';\">\n        ";
  return buffer;
  }

function program3(depth0,data) {
  
  var buffer = "", stack1;
  buffer += "\n        	<div class=\"";
  if (stack1 = helpers.resourceIcon) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.resourceIcon); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\"></div>\n        ";
  return buffer;
  }

  buffer += "<a class=\"item\" href=\"#\" name=\"";
  if (stack1 = helpers.link) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.link); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\">\n    <div class=\"item-pic\">\n    	";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.showThumbnail), {hash:{},inverse:self.program(3, program3, data),fn:self.program(1, program1, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\n    </div>\n    <p class=\"item-name\">";
  if (stack1 = helpers.tittle) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.tittle); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "</p>\n</a>";
  return buffer;
  });

this["JST"]["filters"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression;


  buffer += "    <div id=\"filtercombos\"></div>\n<div id=\"filtersearch\" class=\"filter_all\">\n    <input class=\"buscar\" placeholder=\"";
  if (stack1 = helpers.txt0) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.txt0); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\" type=\"search\" />\n    <button class=\"icon-magnifier bgcolor-5 buscar\"></button>\n</div>";
  return buffer;
  });

this["JST"]["fs-layout"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<nav class=\"buttons\"></nav>\n<div id=\"wrapper\">\n	<div id=\"filters\">\n	</div>\n</div>	\n<div class=\"elements\"></div>";
  });

this["JST"]["fs-modalcontent"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression;


  buffer += "<iframe id=\"modalContent\" src=";
  if (stack1 = helpers.link) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.link); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + " width=\"100%\" height=\"100%\" style=\"border:none;background-color:white; position:relative\"></iframe>";
  return buffer;
  });

this["JST"]["icon-worktool"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression;


  buffer += "<a class=";
  if (stack1 = helpers.iconClass) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.iconClass); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + " href=\"#\">\n    <i class=\"icon-";
  if (stack1 = helpers['class']) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0['class']); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\"></i>\n</a>";
  return buffer;
  });

return this["JST"];

});
//noinspection JSHint
define(
    'views/dropdown',[
      "jquery",
      "underscore",
      "marionette",
      "template"
    ],
    function ($, _, Marionette, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        behaviors: function() {
          return {
            Scrollbar: {
              color: this.options && this.options.color || 8,
              container: "ul",
              position: "right"
            }
          };
        },
        className: "dropdown",
        events: {
          "tap a": "toggle",
          "tap li a": "select"
        },

        modelEvents: {
          "change:title": function () {
            this.ui.title.text(this.model.get("title"));
          }
        },
        ui: {
          title: "> a span:first"
        },
        template: JST.dropdown,

        onRender: function() {
          var self = this;
          this.$("a:first").one("tap", function() {
            _.defer(function() {
              self.triggerMethod("dom:refresh");
            });
          });
        },

        select: function (event) {
            console.log("aqui 2");
          event.stopPropagation();

          this.model.set("title", $(event.target).attr("title"));
          this.trigger("select", $(event.target));
        },

        toggle: function (event) {
          event.stopPropagation();
          this.throttledToggle(event);
        },

          throttledToggle: _.throttle(function (event) {

              var $el = this.$el,
                  $ul = this.$("ul");

              $ul.toggle();
              $el.toggleClass("on");
              this.trigger("toggle");

              if (this.$("ul:visible").length) {
                  _.defer(function () {
                      $(document.body).bind("tap.dropdown", function () {
                          /**
                           * Fix para el dropdown por el drag del scroll para hacer que al hacerse un tap no se cierre la ventana
                           */
                          if (!window.event || $ul.has(window.event.target).length == 0) {
                              $ul.hide();
                              $el.removeClass("on");
                          }
                      });
                  });
              }

          }, 100, {trailing: false})
      });
    }
);
//noinspection JSHint
define(
    'views/icon',[
      "marionette",
      "template"
    ],
    function (Marionette, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        events: {
          "click": function(event) {
            event = event.originalEvent || event;

            event.preventDefault();
            event.stopPropagation();
            try {
              // Esto se debe al lanzamiento de mentira del evento.
              event.gesture.stopDetect();
            } catch(e) {}

            if (!this.isDisabled()) {
              this.trigger("touch");
            }
          }
        },
        tagName: "a",
        template: JST.icon,

        modelEvents: {
          "change": "render"
        },

        /* custom */
        active: function () {
          this.$el.addClass("active");
        },

        disable: function () {
          this.$el.toggleClass("disabled");
        },

        isActived: function () {
          return this.$el.hasClass("active");
        },

        isDisabled: function () {
          return this.$("i").hasClass("inactive");
        }
      });
    }
);
//noinspection JSHint
define(
    'views/icon.collection',[
        "marionette",
        "views/icon"
    ],
    function (Marionette, IconView) {
        "use strict";

        return Marionette.CollectionView.extend({
            tagName: "nav",

            itemView: IconView
        });
    }
);
//noinspection JSHint
define(
    'modules/fullscreen_worktools/controllers/fullscreen_worktool',["underscore", "marionette", "app", "language","configuration", "models/resource.collection"],
    function (_, Marionette, App, Language, configuration, ResourceCollection) {
      "use strict";

      return Marionette.Controller.extend({

        initialize : function (options){

          var activeModel = "";
          var activeList = "";
          var filterList = "";
          var typeButtonActive = "";

          this.typeButtonActive = options.view.collection.at(0).get("iconClass");

          // Carga la primera lista de recursos cuando se abre el popup
          this.loadActivities(options.view.collection.at(0)).done(function(lista){

            this.activeModel = options.view.collection.at(0);
            options.resource.collection.set(lista);
            options.resource.render();
            options.resource.$el.parents(".dialog").find("header").find("h1").text(options.view.collection.at(0).get('name'));

            if(this.typeButtonActive == "library"){
                // options.resource.$el.find(".item").css('width','150px');
                // options.resource.$el.find(".item-pic").css('height','205px');
                options.resource.$el.find(".item-pic").css('display','inline-block');
            }else{
              options.resource.$el.find(".item").css('width','250px');
              options.resource.$el.find(".item-pic").css('height','134px');
            }

            if(!options.view.collection.at(0).get('porunidad')){
              options.dropdown.$el.find("#filtercombos").hide();
              options.dropdown.$el.find("#filtersearch").removeClass('filter_all');
              options.dropdown.$el.find("#filtersearch").addClass('filter_unique');
            }else{
              options.dropdown.$el.find("#filtercombos").show();
              // options.dropdown.$el.find("#filtersearch").removeClass('filter_unique');
              // options.dropdown.$el.find("#filtersearch").addClass('filter_all');
            }
        },this);


          // Cuando se clikea un icono se cambia la vista

          options.view.on("itemclick",function(model){

            var loader = new options.loader({
              auto: true,
              modal: true,
              parent: "#listItems"
            });

            loader.render();

            this.activeModel = model;
            this.typeButtonActive = model.get("iconClass");
            var self = this;

            this.loadActivities(model).done(function(lista){

              options.resource.collection.set(lista);
              options.resource.render();
              options.view.$el.children().removeClass('active');
              options.resource.$el.parents(".dialog").find("header").find("h1").text(model.get('name'));
              options.view.$el.find("#button_" + model.get("buttonid")).addClass("active");

              if(self.typeButtonActive == "library"){
                // options.resource.$el.find(".item").css('width','150px');
                // options.resource.$el.find(".item-pic").css('height','205px');
                options.resource.$el.find(".item-pic").css('display','inline-block');
              }else{
                options.resource.$el.find(".item").css('width','250px');
                options.resource.$el.find(".item-pic").css('height','134px');
              }


              if(!model.get('porunidad')){
                options.dropdown.$el.find("#filtercombos").hide();
                options.dropdown.$el.find("#filtersearch").removeClass('filter_all');
                options.dropdown.$el.find("#filtersearch").addClass('filter_unique');
              }else{
                if(self.typeButtonActive == "library"){
                  options.dropdown.$el.find("#filtercombos").show();
                  options.dropdown.$el.find("#filtersearch").removeClass('filter_all');
                  options.dropdown.$el.find("#filtersearch").addClass('filter_unique');
                }else{
                  options.dropdown.$el.find("#filtercombos").show();
                  options.dropdown.$el.find("#filtersearch").removeClass('filter_unique');
                  options.dropdown.$el.find("#filtersearch").addClass('filter_all');
                }
              }

              // Fix para iPad. Se queda el foco en la busqueda al abrir combos.
              $('#filtercombos .dropdown').bind('touchstart', function(e) {
                e.preventDefault();
                $('input.buscar').blur();
              });

              loader.close();

            });

          },this);


          //Evento del select
          options.dropdown.on("select",function(){

            var self = this;
            var list = new ResourceCollection(self.activeList);
            var filter = list;

            $.each(options.dropdown.collection.models,function(i,v){
              $.each(v.get('options'),function(i1,v1){

                if(v.get("title") == v1.name ){

                  if(self.typeButtonActive == "activities"){

                    if(v1.data.code == 'All'){
                      var unicos = new ResourceCollection();
                      var cantidad = 0;

                      _.each(self.activeList, function(model, index) {
                        cantidad = unicos.where({tittle: model.get("tittle")}).length;
                        if(cantidad == 0){
                          unicos.add(model);
                        }
                      });

                      filter = unicos;
                      // filter = list;
                    }else{
                        filter = new Backbone.Collection(filter.where({chapter:v1.data.code}));
                    }
                    self.selectChapter = v1.data.code;

                  }else{

                    if(self.typeButtonActive == "library"){
                       // Apaño para que salga la opción de "CONTINGUTS" o el seleccionar todas.
                       if (v1.data.id.toString() != "0")
                       {
                         filter = new Backbone.Collection(filter.filter(function(item){
                           return _.contains(item.get('filter'), v1.data.id.toString());
                         }) );
                       }
                    }else{

                      if(v1.data.code == 'All'){
                        var unicos = new Backbone.Collection();
                        var cantidad = 0;

                        _.each(self.activeList, function(model, index) {
                          cantidad = unicos.where({tittle: model.get("tittle")}).length;
                          if(cantidad == 0){
                            unicos.add(model);
                          }
                        });

                        filter = unicos;
                        // filter = list;
                      }else{
                          filter = new Backbone.Collection(filter.where({code:v1.data.code}));
                      }
                    }

                  }
                }
              });
            });

            $("#filtersearch").find("input[class=buscar]").val("");
            this.filterList = filter.toArray();
            options.resource.collection.set(filter.toArray());

            if(this.typeButtonActive == "library"){
                // options.resource.$el.find(".item").css('width','150px');
                // options.resource.$el.find(".item-pic").css('height','205px');
                options.resource.$el.find(".item-pic").css('display','inline-block');
              }else{
                options.resource.$el.find(".item").css('width','250px');
                options.resource.$el.find(".item-pic").css('height','134px');
              }

          },this);

          // Buscador
          options.dropdown.on("listSearch",function(val){

            var self = this;

            var list = new Backbone.Collection(self.activeList);
            var filter = list;

            if(options.dropdown.$el.find("#filtersearch").hasClass("filter_unique")){
              $.each(options.dropdown.collection.models,function(i,v){
                $.each(v.get('options'),function(i1,v1){

                  if(v.get("title") == v1.name ){

                    if(self.typeButtonActive == "activities"){

                      if(v1.data.code == 'All'){
                        var unicos = new Backbone.Collection();
                        var cantidad = 0;

                        _.each(self.activeList, function(model, index) {
                          cantidad = unicos.where({tittle: model.get("tittle")}).length;
                          if(cantidad == 0){
                            unicos.add(model);
                          }
                        });

                        filter = unicos;
                        // filter = list;
                      }else{
                        filter = new Backbone.Collection(filter.where({chapter:v1.data.code}));
                      }
                      self.selectChapter = v1.data.code;

                    }else{

                      if(self.typeButtonActive == "library"){

                        filter = new Backbone.Collection(filter.filter(function(item){
                          return _.contains(item.get('filter'), v1.data.id.toString());
                        }) );

                      }else{
                        filter = list;
                        if(v1.data.code == 'All'){
                          var unicos = new Backbone.Collection();
                          var cantidad = 0;

                          _.each(self.activeList, function(model, index) {
                            cantidad = unicos.where({tittle: model.get("tittle")}).length;
                            if(cantidad == 0){
                              unicos.add(model);
                            }
                          });

                          filter = unicos;
                          // filter = list;
                        }else{
                          filter = new Backbone.Collection(filter.where({code:v1.data.code}));
                        }
                      }

                    }
                  }
                });
              });
            }

            // Limpiar buscador

            options.dropdown.on("cleanSearch",function(){
              $("#filtersearch").find("input[class=buscar]").val("");
              options.resource.collection.set(this.filterList);
            },this);

            this.search(filter.toArray(),val).done(function(list){

              options.resource.collection.set(list);

              if(self.typeButtonActive == "library"){
                // options.resource.$el.find(".item").css('width','150px');
                // options.resource.$el.find(".item-pic").css('height','205px');
                options.resource.$el.find(".item-pic").css('display','inline-block');
              }else{
                options.resource.$el.find(".item").css('width','250px');
                options.resource.$el.find(".item-pic").css('height','134px');
              }

            });

          },this);

          // Abrir recursos

          options.resource.on( "openResource", function(model){

            var _idRecurso = false;
            var elLink = model.get("link") ;
            if ( elLink.indexOf( "/data/RECURSOS/" ) != -1  ){
              elLink = elLink.split("/data/RECURSOS/");
              elLink = elLink[1];
              elLink = elLink.split("/");
              _idRecurso = elLink[0];
            }
            /* else{
              _idRecurso = model.get("tittle") || "noName";
            }*/
            var _isActividad = false;
            var nameActividad = model.get("name") || "";
            if ( nameActividad.indexOf( "ctividad" ) != -1 ){
              _isActividad = true;
            }

            // Comprobacion de AV online
            var tipo = (this.typeButtonActive === 'links' || this.typeButtonActive === 'library') ? "url" : "resource";
            if(!model.get("idExt")){
                tipo = "url";
            }
            // Si is:external, abrimos todos los recursos como URLs
            App.request("is:external")
              .done(function () {
                if (tipo != "resource") {
                  tipo = 'externalUrl';
                }
              })
              .always(function(){
                
                var value = model.get("link");
                                
                var open = function () {
                    App.execute(
                        "open",
                        {
                          color: 5,
                          idRecurso: _idRecurso,
                          isActividad: _isActividad,
                          chapter: model.get("chapter"),
                          index: model.get("order") - 1 || _.indexOf(model.collection.where({chapter: model.get("chapter")}), model),
                          type: tipo,
                          value: value
                        }
                    );
                }
                
                App.request("is:av")
                    .done(function () {
                    
                        if (tipo == "resource" && _idRecurso) {
                            
                            var _parent = window.parent;
                            
                            var _identifier = "openResource";
                            
                            var ALLOWED = {
                              htm: "iframe.resource",
                              html: "iframe.resource"
                            },
                            extension,
                            index,
                            isAV = false,
                            isOpenable;

                            _.defaults(options || (options = {}), {
                              type: "url"
                            });

                            extension = model.get("link").toLowerCase();
                            index = extension.lastIndexOf(".");
                            extension = (index !== -1) ? extension.substr(index + 1) : "";
                            
                            isOpenable = _.contains(_.keys(ALLOWED), extension);
                            
                            _parent.postMessage(
                                {
                                    type: 'getResourceURL',
                                    idResource: _idRecurso,
                                    resourcesPath: 'data/RECURSOS/',
                                    filePath: model.get("link").split("/"+_idRecurso+"/").pop(),
                                    viewer: (isOpenable) ? true : false,
                                    identifier: _identifier
                                },
                                '*'
                            );
                            var listener = function(event) {
                                console.log('Event data URL answered: '+event.data.url);
                                if (typeof event.data != 'undefined' && typeof event.data.url != 'undefined') {
                                    value = event.data.url;
                                    open();
                                }
                                if (window.addEventListener) {
                                    window.removeEventListener("message", listener, false);
                                    listener = null;
                                } else {
                                    window.detachEvent("onmessage", listener);
                                    listener = null;
                                }
                            }
                            if (window.addEventListener) {
                                window.addEventListener("message", listener, false);
                            } else {
                                window.attachEvent("onmessage", listener);
                            }
                            _parent = null;

                        } else {
                            open();
                        }
                    
                    })
                    .fail(function() {
                        open();
                    });
                                
              });

          },this);
          
        },

        loadActivities: function(model) {


          var deferred = $.Deferred();

          $.when(
            App.request("url:worktools", {context: this, url: model.get('path'),
              type: model.get('iconClass')}),
            App.request("chapters", { clone: true, context: this, collection: true})
          )
            .done(function(resource, chapters) {
                var chapter = chapters.first();

              // Elimino la unidad "Paginas Iniciales" sólo para libroclick
              chapters.models = _.reject(chapters.models, function(chapter){
                if(chapter.get("code") == "U32"){
                  return true;
                }
              });

              // Selecciono la primer unidad disponible
              if(!this[1].selectChapter){
                this[1].selectChapter = chapters.models[0].get("code");
              }

              if (resource[1] == "") {

                chapters.each(function(value, key, list) {
                  list[key] = {
                    name: value.get("title"),
                    data: {
                      code: value.get("code"),
                      page: value.get("pages").length && value.get("pages").first().get("id")
                    }
                  };

                  if (key == list.length - 1) {
                    list[list.length] = {
                      name: Language("allUnits"),
                      data: {
                        title: Language("allUnits"),
                        page: "All",
                        code: "All",
                        status: "active"
                      }
                    };
                  }

                });

                chapters = chapters.toArray();

                if (this[1].selectChapter == "" || typeof this[1].selectChapter == 'undefined') {
                  Marionette.getOption(this[1], "dropdown").collection.set({
                    title: chapters[0].name, //chapter.get("title"),
                    options: chapters
                  });
                } else {
                  var selectChapterAux = this[1].selectChapter;
                  var title = "";

                  $.each(chapters, function(i, v) {
                    if (v.data.code == selectChapterAux) {
                      title = v.name;
                    }
                  });

                  Marionette.getOption(this[1], "dropdown").collection.set({
                    title: title,
                    options: chapters
                  });
                }

                Marionette.getOption(this[1], "dropdown").$el.find(".dropdown").removeClass('biblioteca');
                Marionette.getOption(this[1], "dropdown").$el.find("#filtercombos").css('width', "47%");
                Marionette.getOption(this[1], "dropdown").$el.find("#filtersearch").addClass('filter_all');
                Marionette.getOption(this[1], "dropdown").$el.find("#filtersearch").removeClass('filter_unique');
                Marionette.getOption(this[1], "dropdown").$el.find("#filtercombos").removeClass('clearfix');

                $("#filtersearch").find("input[class=buscar]").val("");

                // === Para desactivar unidades que no tienen recursos ===
                var that = this;
                var chaptersWithResources = [];

                // Obtengo los capítulos que tienen recursos
                _.each(resource[0].models, function(res) {
                  res.set("name", model.get( "name") );
                  chaptersWithResources.push(res.get("chapter"));
                });

                // Filtro repetidos
                chaptersWithResources = _.uniq(chaptersWithResources);

                // Recorro los capítulos. Si no tiene recursos, lo desactivo.
                _.each(chapters, function(chapter, index) {
                  var code = chapter.data.code;
                  if (code == "All" || chaptersWithResources.indexOf(code) !== -1) {
                  } else {
                    // Desactivo las opciones
                    Marionette.getOption(that[1], "dropdown")
                      .$el.find('#filtercombos li a')
                      .eq(index).addClass('inactive disabled');
                  }
                });
                // ======

                this[1].activeList = resource[0].models;

                if (!model.get('porunidad')) {
                  this[1].filterList = resource[0].models;
                  deferred.resolve(resource[0].models);
                } else {
                  if (chapter && (this[1].selectChapter == "" || typeof this[1].selectChapter == 'undefined')) {
                    this[1].filterList = resource[0].where({
                      chapter: chapter.get("code")
                    })
                    deferred.resolve(resource[0].where({
                      chapter: chapter.get("code")
                    }));
                  } else {
                    if (this[1].selectChapter == "All") {
                      this[1].filterList = resource[0].models;
                      deferred.resolve(resource[0].models);
                    } else {
                      this[1].filterList = resource[0].where({
                        chapter: this[1].selectChapter
                      });
                      deferred.resolve(resource[0].where({
                        chapter: this[1].selectChapter
                      }));
                    }
                  }
                }

              } else {

                Marionette.getOption(this[1], "dropdown").collection.set(resource[1]);

                if (this[1].typeButtonActive == "links") {
                  Marionette.getOption(this[1], "dropdown").$el.find(".dropdown").removeClass('biblioteca');
                  Marionette.getOption(this[1], "dropdown").$el.find("#filtercombos").css('width', "47%");
                  Marionette.getOption(this[1], "dropdown").$el.find("#filtersearch").addClass('filter_all');
                  Marionette.getOption(this[1], "dropdown").$el.find("#filtersearch").removeClass('filter_unique');
                  Marionette.getOption(this[1], "dropdown").$el.find("#filtercombos").removeClass('clearfix');
                } else {
                  Marionette.getOption(this[1], "dropdown").$el.find(".dropdown").addClass('biblioteca');
                  Marionette.getOption(this[1], "dropdown").$el.find("#filtercombos").css('width', "auto");
                  Marionette.getOption(this[1], "dropdown").$el.find("#filtercombos").addClass('clearfix');
                  Marionette.getOption(this[1], "dropdown").$el.find("#filtersearch").removeClass('filter_all');
                  Marionette.getOption(this[1], "dropdown").$el.find("#filtersearch").addClass('filter_unique');
                }

                $("#filtersearch").find("input[class=buscar]").val("");

                this[1].activeList = resource[0].models;
                this[1].filterList = resource[0].models;
                deferred.resolve(resource[0].models);
              }

            });

          return deferred.promise();
        },

        search: function (list, q){
          var  self = this;
          var deferred = $.Deferred();

          if (q) {
            deferred.resolve(list.filter(function(element) {
              var text = element.get("tittle").toLowerCase() || "";
              text = self.htmlentitiesJS(text);
              text = self.removeHTML(text);

              return (~text.indexOf(q.toLowerCase()));
            }));
          }else{
            deferred.resolve(list);
          }

          return deferred.promise();
        },

       htmlentitiesJS: function(cadena) {
          var caracteresReemplazados = ['À', 'à', 'Á', 'á', 'Â', 'â', 'Ã', 'ã', 'Ä', 'ä', 'Å', 'Æ', 'æ', 'Ç', 'ç', 'Ð', 'ð', 'È', 'è', 'É', 'é', 'Ê','ê', 'Ë', 'ë', 'Ì', 'ì', 'Í', 'í', 'Î', 'î', 'Ï', 'ï', 'Ñ', 'ñ', 'Ò', 'ò', 'Ó', 'ó', 'Ô', 'ô', 'Õ', 'õ', 'Ö', 'ö', 'Ø', 'ø', 'Œ', 'œ', 'ß', 'Þ', 'þ', 'Ù', 'ù', 'Ú', 'ú', 'Û', 'û', 'Ü', 'ü', 'Ý', 'ý', 'Ÿ', 'ÿ', '©',  '®',  '€', '<', '>', '"'];
          var caracteresOriginales= ['&Agrave;', '&agrave;', '&Aacute;', '&aacute;', '&Acirc;', '&acirc;', '&Atilde;', '&atilde;', '&Auml;', '&auml;', '&Aring;',' &AElig;', '&aelig;', '&Ccedil;', '&ccedil;', '&ETH;', '&eth;', '&Egrave;', '&egrave;', '&Eacute;', '&eacute;', '&Ecirc;', '&ecirc;', '&Euml;', '&euml;', '&Igrave', '&igrave;', '&Iacute;', '&iacute;', '&Icirc;', '&icirc;', '&Iuml;', '&iuml;', '&Ntilde;', '&ntilde;', '&Ograve;', '&ograve;', '&Oacute;', '&oacute;', '&Ocirc;', '&ocirc;', '&Otilde;', '&otilde;', '&Ouml;', '&ouml;', '&Oslash;', '&oslash;', '&OElig;', '&oelig;', '&szlig;', '&THORN;', '&thorn;', '&Ugrave;', '&ugrave;', '&Uacute;', '&uacute;', '&Ucirc;', '&ucirc;', '&Uuml;', '&uuml;', '&Yacute;', '&yacute;', '&Yuml;', '&yuml;', '&copy;',  '&reg;',  '&euro;', '&lt;', '&gt;', '&quot;'];
          var longOriginales = caracteresOriginales.length;

          for ( var i=0; i < longOriginales; i++ ) {
            var patron = new RegExp(caracteresOriginales[i],"g");
            cadena = cadena.replace(patron, caracteresReemplazados[i]);

          }

          return cadena;
        },

        removeHTML: function (html) {
          var tmp = document.createElement("DIV");
          tmp.innerHTML = html;
          return tmp.textContent || tmp.innerText || "";
        }

      });

    }
);

define(
    'controllers/loader',[
      "jquery",
      "underscore",
      "marionette",
      "template",
      "knob"
    ],
    function ($, _, Marionette, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        className: "loader",
        ui: {
          input: "input"
        },
        tagName: "article",
        template: JST.loader,

        onClose: function () {
          this.$el.remove();

          if (Marionette.getOption(this, "modal")) {
            this.modalEl.remove();
          }

          if (Marionette.getOption(this, "mini")) {
            this.miniEl.remove();
          }

          if (Marionette.getOption(this, "auto")) {
            this.timeout = clearTimeout(this.timeout);
          }
        },
        onRender: function () {
          var $parent = $(Marionette.getOption(this, "parent")),
              ratio = Math.min(
                  $parent.width() / $(window).width(),
                  $parent.height() / $(window).height(),
                  1
              ),
              step = Marionette.getOption(this, "step"),
              cursor = 90,
              thickness = 0.15,
              width = Math.max(parseInt(ratio * 120), 40),
              options = {
                readOnly: true,

                bgColor: "#FFFFFF",
                fgColor: "#56B8A7",
                font: "Signika Negative",
                fontWeight: 300,
                inputColor: "#747474",
                lineCap: "round",
                thickness: thickness,
                width: width
              };

          this.$el
              .css("margin-top", -parseInt(width / 2, 10))
              .css("margin-left", -parseInt(width / 2, 10));

          if (Marionette.getOption(this, "auto")) {
            options = _.extend(options, {
              stopper: false,
              cursor: cursor,
              thickness: thickness * 1.25,
              displayInput: false
            });

            var self = this;

            if (!this.timeout) {
              this.timeout = setInterval(function () {
                self.value(new Date().getTime() / step % 100);
              }, step);
            }

            this.value(
                new Date().getTime() / Marionette.getOption(this, "step") % 100
            );
          }

          if (Marionette.getOption(this, "mini")) {
            options = _.extend(options, {
              displayInput: false,
              thickness: 0.25,
              width: 30
            });
          }

          options = _.extend(
              options,
              _.pick(this.options, "bgColor", "fgColor", "thickness", "width")
          );

          this.ui.input.knob(options);

          if (Marionette.getOption(this, "modal")) {
            if (!this.modalEl) {
              this.modalEl = $("<div />", {class: "modal"});
            }

            $parent.prepend(this.modalEl);
          }

          $parent.prepend(this.$el);
        },

        initialize: function (options) {
          this.options = options = _.extend({
            auto: false,
            modal: false,
            mini: false,
            parent: "body",
            step: 40
          }, options);

          if (options.mini) {
            if (!this.miniEl) {
              this.miniEl = $(
                  "<div />",
                  {
                    class: "mini",
                    style: "position:absolute; " +
                        "margin-right:0; " +
                        "margin-top:0; " +
                        "width:50px; " +
                        "height:50px;"
                  }
              ).prependTo("main");
            }

            this.options.parent = options.parent = this.miniEl;
          }
        },

        value: function (percentage) {
          this.ui.input.val(parseInt(percentage, 10)).trigger('change');
        }
      });
    }
);
define(
    'modules/fullscreen_worktools/fullscreen_worktools',[
        "app",
        "language",
        "modules/fullscreen_worktools/template",
        "controllers/scrollbar",
        "views/dropdown",
        "configuration",
        "encrypt",
        "language",
        "views/dialog",
        "views/icon.collection",
        "views/icon",
        "regions/replace",
        "modules/fullscreen_worktools/controllers/fullscreen_worktool",
        "models/resource.collection",
        "controllers/loader",
        "modules/traze/traze"
    ],
    function (App, language, JST, Scrollbar, DropdownView, Config, Encrypt,Language,  DialogView, IconCollection, Icon, ReplaceRegion,Controller, ResourceCollation, Loader, traze) {
        "use strict";

        return App.module("Fullscreen_worktools", function (Fullscreen_worktools, App, Backbone, Marionette, $, _) {

            this.startWithParent = false;


            // Views
            var Layout = Backbone.Marionette.Layout.extend({
                template: JST["fs-layout"],

                regions: {
                    buttons: ".buttons",
                    filters: "#filters",
                    elements: ".elements"
                },

                onRender: function(){
                    this.$el.attr('id', 'worktools');
                    this.$el.addClass('white');
                },

                regionType: ReplaceRegion,
            }); 

            Icon = Icon.extend({
                triggers:{
                    "touch" : "itemclick"
                }
            });
            
            IconCollection = IconCollection.extend({
                className: "buttons",
                itemView: Icon,
                itemEvents:{
                    "itemclick" : function(name,view, object){
                        if(object.model.get('active')){
                            // Para botón de créditos
                            if(object.model.get('type') == "copyright"){
                                App.request("credits")
                                  .done(function(message){
                                    App.execute("dialog", {
                                      color: 5,
                                      message: message,
                                      height: "90%",
                                      title: language("credits"),
                                      width: "90%",
                                      id: "creditosLC"
                                    });
                                  });
                            } else {
                                // Resto de botones
                                this.trigger("itemclick",object.model);
                            }
                        }  
                    }
                },
                itemViewOptions: function(model,index){
                    var active = (index == 0);
                    var inactive = model.get('active');
                    return{
                        className: model.get('iconClass') + " " + ((active) ? 'active': '') + " " + ((!inactive) ? 'inactive': ''),
                        id: "button_" + model.get('buttonid')

                    }
                }
            });

            DropdownView = DropdownView.extend({
                className: "my dropdown",
                templateHelpers: {
                    color: 5
                }   

            });

            // filters
            var filtersContainer = Marionette.CompositeView.extend({
                className: "filters col-form",
                template: JST["filters"],
                itemViewContainer: "#filtercombos",
                itemView: DropdownView,
                itemViewOptions: {
                  color: 5
                },
                itemEvents:{

                  "select": function(name,view,object){
                    // Actualizo el título con la unidad seleccionada
                    // $('h1#index').css('padding-top', 20).text(object[0].title);
                    
                    this.trigger("select");
                  }
                },

                events: {
                  "keyup input.buscar": "search",
                  "touch button.buscar": "search",
                  "focus input.buscar": "cleanSearch"
                },

                search: function (event) {
                 if(event.type ==  "touch"){
                   this.trigger("listSearch",$("#filtersearch").find("input[class=buscar]").val());
                 }else{
                   this.trigger("listSearch",$("#filtersearch").find("input[class=buscar]").val());
                 }
                },
              cleanSearch: function (event) {
                this.trigger("cleanSearch");
              }

            });

          // Resources

            var ListElementView = Marionette.ItemView.extend({
                template: JST["activity"],
                className: "elements",
                modelEvents: {
                    "change": "render"
                },
                triggers:{
                  "touch" : "openResource"
                }
            });


            var ListView = Marionette.CollectionView.extend({
                className: "clearfix",
                id: "listItems",
                itemView: ListElementView,
                itemEvents:{
                  "openResource" : function(name,view, object){
                    if(object.model.get('link') != ""){
                        this.trigger("openResource",object.model);
                    }
                  }
                },
                appendHtml: function(collectionView, itemView, index){
                    var childrenContainer = collectionView.itemViewContainer ? collectionView.$(collectionView.itemViewContainer) : collectionView.$el;
                    var children = childrenContainer.children();
                    if (children.size() <= index) {
                      childrenContainer.append(itemView.el);
                    } else {
                      children.eq(index).before(itemView.el);
                    }
                  },

                itemViewOptions: function(model,index){
                    var inactive = model.get('link');
                    return{
                        className: "elements " + ((model.get('link') == "") ? 'inactive': ''),

                    }
                }  
            });


          // Dialog open resource

          var ResourceContentView = new DialogView({
            color: 5,
            width: '100%',
            height: '100%',
            className: "dialog full-screen"
          });


          var ContentDialog = Marionette.ItemView.extend({
            template: JST["fs-modalcontent"],
            tagName: "main"
          });

           

            this.addInitializer(function () {
                var self = this;

                var el = new Layout({el: "body > main"});

                $.when(
                    App.request("buttons"), 
                    App.request("chapters", {
                        collection: true
                    }),
                    App.request("GEDI"),
                    App.request("info")
                    ).done(function (buttons, chapters, gedi, info) {
                      var current = chapters.first();
                    el.on("render", function () {
                        var iconsetup = [];
                        _.each(buttons, function (item, index) {

                            if(item.class == "library"){
                                iconsetup.push({
                                    type: item.icon,
                                    iconClass: item.class,
                                    path: item.path,
                                    buttonid: index,
                                    active: item.active,
                                    name: Language("teachersLibrary"),
                                    porunidad: item.porunidad,
                                    iframe: item.iframe
                                });
                            }else{
                                if(item.class == "links"){
                                    iconsetup.push({
                                        type: item.icon,
                                        iconClass: item.class,
                                        path: item.path,
                                        buttonid: index,
                                        active: item.active,
                                        name: Language("links"),
                                        porunidad: item.porunidad,
                                        iframe: item.iframe
                                    });
                                }else{
                                    iconsetup.push({
                                        type: item.icon,
                                        iconClass: item.class,
                                        path: item.path,
                                        buttonid: index,
                                        active: item.active,
                                        name: item.name,
                                        porunidad: item.porunidad,
                                        iframe: item.iframe
                                    });
                                }
                            }

                            
                        });

                        // Si tengo créditos, muestro boton extra
                        App.request("credits")
                          .done(function(message){
                            if(message){
                                iconsetup.push({
                                    type: "copyright",
                                    iconClass: "activities",
                                    path: "",
                                    buttonid: "credits",
                                    active: true,
                                    name: "credits",
                                    porunidad: false
                                });
                            }
                          });
                        

                        var a = new IconCollection({
                            collection: new Backbone.Collection(iconsetup)
                        });


                        el.buttons.show(a);

                        var resourceView = new ListView({
                            collection: new ResourceCollation()
                        });



                        // Data Mapper
                        

                        var b = new filtersContainer({
                            collection: new Backbone.Collection()
                        });
                        
                        el.filters.show(b);

                    var contentDialog = new ContentDialog ({model: new Backbone.Model()});

                    this.controller = new Controller({view:a,resource:resourceView, dropdown: b,dialog: ResourceContentView, contentDialog: contentDialog, loader:Loader});

                    el.elements.show(resourceView);

                    });

                    // Título del LibroClick
                    var titulo = gedi;
                    // Si tengo el nombre del libro, lo uso. Si no, uso el GEDI
                    if(info && info.name){
                        titulo = info.name;
                    }

                    $('header h1#index').css('padding-top', 20).text(titulo);
                      App.trigger("menu:show");

                    el.render();

                });

                if( traze.mT.tipoTraza == 'SCORM' ){
                    $(".icon-check").parent().addClass('inactive disabled');
                }

            });

        });
    }
);
//noinspection JSHint
define(
    'modules/help/help',[
      // app
      "app",
      // language
      "language",
      // views
      "views/dialog",
      "views/iframe"
    ],

    function (App, language, DialogView, IframeView) {
      "use strict";

      return App.module("help", function (module, App, Backbone, Marionette, $, _) {
        /* Module presets */
        this.startWithParent = false;

        /* Private */

        /* Public */

        // Constructor
        this.addInitializer(function () {
          var dialog = new DialogView({
                color: 5,
                title: language("help.LMHelp")
              }),
              languageHelp = "es-ES";

          App.request("info")
              .done(function (data) {
                if (data.language) {
                  languageHelp = data.language;
                }
              })
              .always(function () {
                if(navigator.onLine){
                  dialog.show(new IframeView({
                    model: new Backbone.Model({
                      src: "http://aulavirtual.santillana.es/ayuda/ayuda/preguntas/" +
                          languageHelp + "/1/lm"
                    })
                  }));
                  

                  /*

                    var w = $('.dialog > main').width();

                    var isiOS = navigator.userAgent.match(/(iPad)|(iPhone)|(iPod)/i) != null;

                    if(isiOS){
                      $( ".dialog > main > iframe" ).contents().find( "html" ).css( "max-width", w);
                      $( ".dialog > main" ).css( "overflow-y", "auto");
                      $( ".dialog > main" ).css( "-webkit-overflow-scrolling", "touch");

                      if(!navigator.userAgent.match('OS 8_')){
                          $( ".dialog > main > iframe" ).height(($(".dialog > main > iframe").contents().height()+50) + "px");
                          $(this).contents().find("body").on('click', function(event) {
                            console.log("clicked");
                          window.setTimeout(function(){
                            console.log($(".dialog > main > iframe").contents().height());
                            $( ".dialog > main > iframe" ).height(($(".dialog > main > iframe").contents().height()+50) + "px");
                          },500)
                      });
                      }else{
                        console.log("ios mayor de 8");
                        $( ".dialog > main > iframe" ).prop("height","100%");
                        //$( ".dialog > main > iframe" ).css("overflow-y","scroll");
                      }
                    }else{
                      $( "#visualizer").contents().find('.dialog > main').height($( "#visualizer").contents().find('.dialog > main').height() - $( "#visualizer").contents().find('.dialog > header').height());

                    }
                  });*/

                $('.dialog > main > iframe').css('display','none');
                  $('.dialog > main > iframe').load(function(){
                    $('.dialog > main > iframe').css('display','block');
                    $('.dialog > main').height($('.dialog > main').height() - $('.dialog > header').height());
                    $( window ).resize(function() {
                         $('.dialog > main').height("100%");
                         $('.dialog > main').height($('.dialog > main').height() - $('.dialog > header').height());
                        });

                        var isiOS = navigator.userAgent.match(/(iPad)|(iPhone)|(iPod)/i) != null;
                        
                        if(isiOS){
                          var w = $('.dialog > main').width();
                          $( ".dialog > main > iframe" ).contents().find( "html" ).css( "max-width", w);
                          $( ".dialog > main" ).css( "overflow-y", "auto");
                          $( ".dialog > main" ).css( "-webkit-overflow-scrolling", "touch");
                        }
                        
                });

                } else {
                  App.execute("notification", language("noInternet"), {color: 10});
                  App.execute("module:stop", "help");
                }
                dialog.on("quit", function () {
                  App.execute("module:stop", "help");
                });
              });
        });
      });
    }
);
//noinspection JSHint
define(
    'views/chapters.dropdown',["jquery", "underscore", "backbone", "marionette", "app", "views/dropdown"],
    function ($, _, Backbone, Marionette, App, DropdownView) {
      "use strict";

      var toggle = _.constant(DropdownView.prototype.toggle)();

      DropdownView = DropdownView.extend({
        select: function (event) {
          event.stopPropagation();

          if ($(event.target).attr("data-status") !== "inactive") {
            this.model.set("title", $(event.target).attr("title"));
            this.trigger("select", $(event.target));
          }
        },

        toggle: function (event) {
          event.stopPropagation();

          if ($(event.target).attr("data-status") !== "inactive") {
            _.bind(toggle, this)(event);
          }
        }
      });

      return function () {
        var deferred = $.Deferred();

        $.when(App.request("chapters", {clone: true}))
            .done(function (chapters) {
              var first = chapters.first();

              // Data Mapper
              chapters.each(function (value, key, list) {
                list[key] = {
                  name: value.get("title"),
                  data: {
                    code: value.get("code"),
                    page: value.get("pages").first().get("id"),
                    status: (value.get("status")) ? "active" : "inactive"
                  }
                };
              });
              chapters = chapters.toArray();

              deferred.resolve(
                  DropdownView.extend({
                    model: new Backbone.Model({
                      title: first && first.get("title"),
                      options: chapters
                    })
                  })
              );
            })
            .fail(function () {
              deferred.resolve(Marionette.ItemView);
            });

        return deferred.promise();
      };
    }
);
define('modules/index/template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["index.thumbnail"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression, self=this;

function program1(depth0,data,depth1) {
  
  var buffer = "", stack1;
  buffer += "\n    <img alt=\"";
  if (stack1 = helpers.title) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.title); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\"\n        ";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.current), {hash:{},inverse:self.noop,fn:self.program(2, program2, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\n        ";
  stack1 = helpers.each.call(depth0, (depth0 && depth0.data), {hash:{},inverse:self.noop,fn:self.program(4, program4, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\n        src=\"";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.thumbnail), {hash:{},inverse:self.programWithDepth(8, program8, data, depth1),fn:self.programWithDepth(6, program6, data, depth1),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\" />\n";
  return buffer;
  }
function program2(depth0,data) {
  
  
  return " class=\"current\"";
  }

function program4(depth0,data) {
  
  var buffer = "", stack1;
  buffer += " data-"
    + escapeExpression(((stack1 = ((stack1 = data),stack1 == null || stack1 === false ? stack1 : stack1.key)),typeof stack1 === functionType ? stack1.apply(depth0) : stack1))
    + "=\""
    + escapeExpression((typeof depth0 === functionType ? depth0.apply(depth0) : depth0))
    + "\"";
  return buffer;
  }

function program6(depth0,data,depth2) {
  
  var buffer = "", stack1, stack2;
  buffer += escapeExpression(((stack1 = (depth2 && depth2.path)),typeof stack1 === functionType ? stack1.apply(depth0) : stack1));
  if (stack2 = helpers.thumbnail) { stack2 = stack2.call(depth0, {hash:{},data:data}); }
  else { stack2 = (depth0 && depth0.thumbnail); stack2 = typeof stack2 === functionType ? stack2.call(depth0, {hash:{},data:data}) : stack2; }
  buffer += escapeExpression(stack2);
  return buffer;
  }

function program8(depth0,data,depth2) {
  
  var stack1;
  return escapeExpression(((stack1 = (depth2 && depth2.blank)),typeof stack1 === functionType ? stack1.apply(depth0) : stack1));
  }

  stack1 = helpers.each.call(depth0, (depth0 && depth0.pages), {hash:{},inverse:self.noop,fn:self.programWithDepth(1, program1, data, depth0),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\n<span>";
  if (stack1 = helpers.title) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.title); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "</span>";
  return buffer;
  });

this["JST"]["panel"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<aside></aside>\n<button class=\"icon-maximize\"></button>\n<section></section>\n<div id=\"buttons\"></div>\n\n\n";
  });

return this["JST"];

});
// noinspection JSHint
define(
    'modules/index/views/thumbnail',["jquery", "underscore", "configuration", "views/thumbnail", "modules/index/template"],
    function ($, _, configuration, ThumbnailView, JST) {
      "use strict";

      return ThumbnailView.extend({
        className: "thumbnail norender",
        events: {
          "tap img": "goto"
        },
        template: JST["index.thumbnail"],

        onRender: function () {
          this.$("img").each(_.bind(function(key, value) {
            $(value).css(
                "background",
                "url(" + configuration.root + "assets/images/custom/" +
                    this.model.get("pages")[key].type + ".png)"
            );
          }, this));
        },

        onShow: function() {
          _.chain(function() {
            this.$el.removeClass("norender");
          })
              .bind(this)
              .defer()
              .value();
        },

        goto: function (event) {
          this.trigger("goto", parseInt($(event.target).attr("data-page"), 10));
        }
      });
    }
);
// noinspection JSHint
define(
    'modules/index/views/index',["marionette", "modules/index/views/thumbnail"],
    function (Marionette, ThumbnailView) {
      "use strict";

      return Marionette.CollectionView.extend({
        tagName: "nav",

        itemEvents: {
          "goto": function(eventName, view, page) {
            // Elimino resultados de busqueda
            App.trigger("search:removeResults");
            
            this.trigger("goto", page);
          }
        },
        itemView: ThumbnailView,

        appendHtml: function (collectionView, itemView, index) {
          var childrenContainer = collectionView.itemViewContainer ?
              collectionView.$(collectionView.itemViewContainer) :
              collectionView.$el;
          var children = childrenContainer.children();
          if (children.size() <= index) {
            childrenContainer.append(itemView.el);
          } else {
            children.eq(index).before(itemView.el);
          }
        }//,

//        scrollbar: {
//          color: 2,
//          position: "bottom"
//        }
      });
    }
);
// noinspection JSHint
define(
    'modules/index/regions/index',["marionette", "modules/index/template"],
    function (Marionette, JST) {
      "use strict";

      return Marionette.Layout.extend({
        className: "panel norender",
        events: {
          "touch > button": function () {
            this.$el.toggleClass("fullscreen");
          }
        },
        id: "preview",
        regions: {
          filter: "aside",
          thumbnails: "section",
          buttons: "#buttons"
        },
        tagName: "section",
        template: JST.panel
      });
    }
);
//noinspection JSHint
define(
    'modules/index/index',[
      // app
      "app",
      "views/chapters.dropdown",
      "modules/viewer/viewer",
      "modules/index/views/index",
      "modules/index/regions/index",
      "modules/index/template",
      // helpers
      "language"
    ],
    function (App, ChaptersDropdownViewDeferred, viewer, IndexView, IndexLayout, JST, language) {
      "use strict";

      return App.module(
          "index",
          function (module, App, Backbone, Marionette, $, _) {
            /* Presets */
            this.startWithParent = false;

            /* Private */

            /* Public */

            // Layout is generated in constructor

            // Constructor
            this.addInitializer(function (options) {
              options = _.extend({
              }, options);

              this.layout = (function () {
                var layout = new IndexLayout(),
                    region = App.panel;
                region.show(layout);

                return layout;
              })();

              $.when(
                      App.request("chapter", {context: this}),
                      ChaptersDropdownViewDeferred()
                  )
                  .done(function (chapter, View) {
                    var filter,
                        view = new IndexView({
                          collection: new Backbone.Collection()
                        });

                    View = View.extend({
                      className: "my dropdown",
                      tagName: "h2",

                      initialize: function () {
                        this.model = this.model.clone();
                        this.model.set("title", chapter.get("title"));
                        this.model.set("color", 2);
                        this.model.set("options", this.model.get("options").slice(0));
                        this.model.get("options").push({
                          name: language("allUnits"),
                          data: {code: "*"}
                        });
                        
                        // ========== QUITAR ===========
                        //App.execute("getWebKitVersion");
                        // =============================
                      }
                    });

                    filter = new View({color: 2});

                    filter.on("select", function ($el) {
                      this.chapter($el.attr("data-code"));
                    }, module);

                    view.on("goto", function (page) {
                      viewer.trigger("goto", page);
                    });


                    module.layout.filter.show(filter);
                    module.layout.thumbnails.show(view);

//                    _.defer(function() {
                    module.layout.$el.removeClass("norender");
//                    });

                    module.chapter(chapter.get("code"));
                  })
                  .fail(function () {
                    App.execute("error", language("index.couldNotLoadChapters"));
                  });

            });

            // Destructor
            this.addFinalizer(function () {
              this.layout.close();
              this.layout.$el.addClass("norender").removeClass("fullscreen");
            });

            /* Custom */
            this.chapter = function (code) {
              $.when(
                      App.request("chapters", {context: this}),
                      App.request(
                          "is:document:portrait",
                          {boolean: true, context: this}
                      ),
                      App.request("page", {context: this})
                  )
                  .done(function (chapters, isDocumentPortrait, page) {
                    var pages = (code === "*") ? chapters.pages() :
                            chapters.findWhere({code: code}).get("pages"),
                        thumbnails = [];

                    pages = pages.groupBy(function (value) {
                      return parseInt(
                          value.get("id") / (isDocumentPortrait + 1),
                          10
                      );
                    });

                    _.each(pages, function (value) {
                      var collection = new Backbone.Collection(value);

                      thumbnails.push({
                        id: collection.first().get("id"),
                        title: collection.pluck("title").join("-"),

                        pages: value.map(function (rvalue) {
                          rvalue = rvalue
                              .clone()
                              .set("data", {page: rvalue.get("id")});

                          if (rvalue.get("id") === page) {
                            rvalue.set("current", true);
                          }

                          return rvalue.toJSON();
                        })
                      });
                    });

                    this[0].layout.thumbnails.currentView.collection.set(thumbnails);
                  });
            };
          }
      );
    }
);
// noinspection JSHint
define(
    'views/style',["backbone", "marionette", "template"],
    function (Backbone, Marionette, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        tagName: "style",
        template: JST.style,

        initialize: function(options) {
          this.model = new Backbone.Model(options);
        }
      });
    }
);
// noinspection JSHint
define(
    'modules/loader/views/style',["views/style"],
    function (StyleView) {
      "use strict";

      return StyleView.extend({
        id: "reseting"
      });
    }
);
// noinspection JSHint
define(
    'modules/loader/loader',[
        "app",
        "configuration",
        "encrypt",
        "language",
        "controllers/stats",
        "views/loader",
        "modules/loader/views/style"
    ],
    function (App, configuration, Encrypt, language, statsController, LoaderView, StyleView) {
        "use strict";

        return App.module(
            "loader",
            function (Worktools, App, Backbone, Marionette, $, _) {
                /* Presets module */
                this.startWithParent = false;

                /* Private */

                var MODE = {
                    CUSTOM: 4,
                    EDIT: 2,
                    NOTEBOOK: 5,
                    RESOURCE: 1
                };

                /* Public */

                // Constructor
                this.addInitializer(function (options) {
                    var application = $.Deferred(),
                        information,
                        modules = {},
                        option,
                        page,
                        search = window.location.search;

                    _.defaults(options || (options = {}), {
                        custom: (!!~search.toLowerCase().indexOf("mybook") ||
                        !!~search.toLowerCase().indexOf("mynotebook")) << 2,
                        edit: !!~search.toLowerCase().indexOf("edit") << 1,
                        resource: false
                    });

                    var timer = new Date().getTime();

                    $.ajax({

                        dataType: "text",
                        url: configuration.configuration

                    }).done(function (data) {

                        _.extend(
                            window.master || (window.master = {}),
                            $.parseJSON(Encrypt.decrypt(data))
                        );

                        console.debug(
                            (new Date().getTime() - timer) + "ms load configuration"
                        );

                    }).fail(function() {

                        window.master.book = {
                            chapters: [],
                            pages: []
                        };

                    }).then(function() {

                        $.ajax({

                            dataType: "text",
                            url: configuration.pageSizes

                        }).done(function (data) {

                            _.each($.parseJSON(Encrypt.decrypt(data)), function (pageSize, index) {

                                if (typeof(window.master.book.pages[index]) !== "undefined") {

                                    window.master.book.pages[index].size = {
                                        'height': pageSize[1],
                                        'width': pageSize[0]
                                    };

                                }

                            });

                        }).fail(function () {

                            console.log("Could not read page sizes information");

                        }).then(function () {

                            return $.ajax({

                                dataType: "json",
                                url: configuration.sgoReadOnlyObjectsFile

                            }).done(function (sgoReadOnlyObjects) {

                                _.each(sgoReadOnlyObjects.pages, function (page, index) {
                                    // Busco en window.master la pagina que coincida con la de
                                    // sgoReadOnlyObjects y guardo dentro las figuras obtenidas.
                                    _.each(window.master.book.pages, function (bookPage, index, list) {
                                        if (bookPage.number == page.number) {
                                            bookPage.readOnlyObjects = page.custom;
                                        }
                                    });
                                });

                            });
                        })
                        // extract pages
                            .then(function () {
                                return App.request("pages");
                            }, function () {
                                return App.request("pages");
                            })
                            .done(function (pages) {

                                options.resource = !pages.length << 0;

                                option = options.custom | options.edit | options.resource;

                                switch (true) {

                                    // =*=*=*=*= MI CUADERNO =*=*=*=*=
                                    case !!(option & MODE.NOTEBOOK && option & MODE.EDIT):
                                        window.mode = 'micuaderno';
                                        _.defaults(modules, {
                                            menu: {
                                                index: false,
                                                favorites: false,
                                                search: false,
                                                worktools: false,
                                                help: false,
                                                eye: false
                                            },
                                            viewer: {
                                                bookmark: false,
                                                module: {
                                                    viewer: {
                                                        mode: "single"
                                                    },
                                                    controls: {
                                                        // mode: false
                                                    }
                                                }
                                            },
                                            notebook: {}
                                        });
                                        break;

                                    // =*=*=*=*= PREVIEW DE MI LIBRO =*=*=*=*=
                                    case !!(option & MODE.CUSTOM):
                                        window.mode = 'milibro-preview';
                                        _.defaults(modules, {
                                            menu: {
                                                favorites: !!~search.toLowerCase().indexOf("tools"),
                                                search: false,
                                                worktools: false,
                                                tools: !!~search.toLowerCase().indexOf("tools"),
                                                eye: false
                                            },
                                            viewer: {
                                                bookmark: false,
                                                module: {
                                                    viewer: {
                                                        mode: "single"
                                                    },
                                                    controls: {
                                                        mode: false
                                                    }
                                                }
                                            }
                                        });
                                        break;

                                    // =*=*=*=*= EDICION DE MI LIBRO =*=*=*=*=
                                    case !!(option & MODE.EDIT):
                                        window.mode = 'milibro-edicion';
                                        _.defaults(modules, {
                                            menu: {
                                                favorites: false,
                                                index: false,
                                                mybook: true,
                                                search: false,
                                                worktools: false,
                                                eye: false
                                            },
                                            mybook: {},
                                            viewer: {}
                                        });
                                        break;

                                    // =*=*=*=*= LIBRO CLICK =*=*=*=*=
                                    case !!(option & MODE.RESOURCE):
                                        window.mode = 'libroclick';
                                        _.defaults(modules, {
                                            menu: {
                                                favorites: false,
                                                help: false,
                                                index: false,
                                                search: false,
                                                tools: false,
                                                worktools: false,
                                                eye: false
                                            },
                                            Fullscreen_worktools: {}
                                        });
                                        break;

                                    // =*=*=*=*= VISOR NORMAL LIBROMEDIA =*=*=*=*=
                                    default:
                                        window.mode = 'libromedia';
                                        _.defaults(modules, {
                                            menu: {},
                                            traze: {},
                                            viewer: {}
                                        });

                                        // Fix temporal para desactivar herramientas en tablets.
                                        /*
                                         App.request("is:device").done(function (type) {
                                         modules.menu.tools = false;
                                         var isWindowsTouch = (navigator.platform.match("Win") && ('ontouchstart' in window || navigator.msMaxTouchPoints)) || false;
                                         var isIpadRetina = (type[0].match("iPad") && (window.devicePixelRatio > 1)) || false;
                                         var androidVersion = navigator.userAgent.match(/Android\s([0-9\.]*)/) || false;
                                         var isNewAndroid = false;

                                         if(androidVersion && androidVersion[1]){
                                         var version = androidVersion[1].split('.');
                                         if(version[0] > 4){
                                         isNewAndroid = true;
                                         } else if(version[0] == 4){
                                         if(version[1] >= 4){
                                         isNewAndroid = true;
                                         }
                                         }
                                         }

                                         // Si no es Windows Touch, ni iPad no-retina, ni Android 4.4 o superior, anulo las herramientas.
                                         if(type[0].match("iPad")){
                                         if(isIpadRetina){
                                         modules.menu.tools = false;
                                         }
                                         } else {
                                         if(!isWindowsTouch && !isNewAndroid){
                                         modules.menu.tools = false;
                                         }
                                         }
                                         });
                                         //*/

                                        break;
                                }

                                if (configuration.debug) {
                                    _.defaults(modules, {
                                        debug: {}
                                    });
                                }
                            })
                            // setting css styles
                            .then(function (pages) {
                                console.debug((new Date().getTime() - timer) + "ms init style");

                                page = pages.find(function (unaPage) {
                                    return ( unaPage.get("code") && unaPage.get("thumbnail") );
                                });
                                var _isWC = ( getURLParameter("trackId") != "null" );
                                if (_isWC) {
                                    var _miUnidad = getURLParameter("unitId");
                                    if (_miUnidad != "null") {
                                        _miUnidad = _miUnidad.replace("Unidad_", "U");
                                        // console.log( "pages", pages );
                                        page = pages.find(function (unaPage) {
                                            // console.log( 'Checking values', unaPage, unaPage.get('code'), _miUnidad );
                                            return ( unaPage.get("code") == "" + _miUnidad && unaPage.get("thumbnail") );
                                        });
                                        // console.log( "page", page );
                                    }
                                }

                                // Si no estoy en AV, puedo simular iniciar en otra página
                                // page.set("index", 3);
                                // page.set("number", 3);
                                // page.set("thumbnail", "4.jpg");

                                return App.request("document", {code: page && page.get("code")});
                            })
                            .then(function (document) {

                                return _.clone(window.master.book.pages[page.get("index")].size);

                            })
                            .done(function (size) {
                                information = size;
                            })
                            .fail(function () {
                                information = {
                                    width: 1084,
                                    height: 831
                                };
                            })
                            .always(function () {
                                console.debug(
                                    (new Date().getTime() - timer) + "ms page size"
                                );

                                configuration.size = information;
                                var $main = $("body > main"),
                                    size = {
                                        height: $main.height() - 10,
                                        width: $main.width() - 10
                                    },
                                    ratio = Math.min(
                                        size.height / information.height,
                                        size.width / information.width
                                    ),
                                    style = {
                                        height: Math.floor(
                                            information.height * ratio
                                        ) + "px",
                                        width: Math.floor(
                                            information.width * ratio
                                        ) + "px"
                                    },
                                    width = Math.floor(
                                        (information.width * ratio) * 145 /
                                        (information.height * ratio)
                                    ),
                                    view = new StyleView({
                                        "#viewer .page canvas": style,
                                        "#viewer .page img": style,
                                        "#viewer .page span": {
                                            width: style.width
                                        },
                                        "#viewer .page .interactive": style,
                                        ".thumbnail img": {
                                            height: "145px",
                                            width: width + "px",
                                            "background-size": Math.floor(
                                                16 / width * 100
                                            ) + "% !important"
                                        }
                                    });

                                $(window.document.head).append(view.render().el);

                                //noinspection JSHint
                                if (!(option === MODE.RESOURCE)) {
                                    //noinspection JSHint
                                    /**
                                     * It tries to add size on viewer.document module
                                     */
                                    modules.viewer.module || (modules.viewer.module = {});
                                    //noinspection JSHint
                                    modules.viewer.module.document ||
                                    (modules.viewer.module.document = {});
                                    _.extend(modules.viewer.module.document, {
                                        size: information
                                    });
                                }
                            })
                            .always(function () {
                                // CSS Class for document.body
                                _.each(options, function (value, key) {
                                    if (key && options[key]) {
                                        $(document.body).addClass(key);
                                    }
                                });

                                App.request("is:device").done(function () {
                                    $(document.body).addClass("device");
                                });

                                App.request("is:av").done(function () {
                                    $(document.body).addClass("av");
                                });
                            })
                            // Recover custom book
                            .always(function () {
                                /**
                                 * When we use a custom book, we recover the information through
                                 * AV. Then we must wait for that information.
                                 */
                                if (option & options.custom) {
                                    // TODO replace for this
                                    App.request("av", {action: "get_data", type: "getToolData"})
                                        .done(function (data) {
                                            data = $.parseJSON(data.dataJSON);
                                            // FIX para doble encodig de JSON
                                            try {
                                                data = $.parseJSON(data);
                                            } catch (e) {
                                            }

                                            if (!data) {
                                                window.master.book = {
                                                    chapters: [
                                                        {
                                                            idUnidad: "*",
                                                            title: "Mi Cuaderno"
                                                        }
                                                    ],
                                                    pages: [
                                                        {
                                                            blank: true
                                                        }
                                                    ]
                                                };
                                            } else {
                                                _.each(data, function (value, key) {
                                                    _.extend(
                                                        window.master[key],
                                                        value
                                                    );
                                                });
                                            }

                                            App.request("pages", {cache: false}).always(function () {
                                                application.resolve();
                                            });
                                        })
                                        .fail(function () {
                                            App.execute(
                                                "warning",
                                                language("couldNotLoadDataFromAV")
                                            );
                                        });
                                } else {
                                    application.resolve();
                                }
                            })
                            // try to turn on modules
                            .always(function () {
                                // Avoiding first hard requests on the average timer.
                                statsController.reset("cache");

                                // Intialize
                                application.done(function () {
                                    _.each(modules, function (value, key) {
                                        App.execute("module:start", key, value);
                                    });
                                })
                                //;

                                    .then(function () {

                                        //AleD
                                        var _isWC = ( getURLParameter("trackId") != "null" );
                                        if (_isWC) {
                                            var _miUnidad = getURLParameter("unitId");
                                            console.log("estamos en WC", _miUnidad);
                                            if (_miUnidad != "null") {
                                                console.log("abrimos pagina inicial de " + _miUnidad + ", pagina " + page.get("order"));
                                                // console.log( "a ver que hay aqui", App.viewer.controls.options );
                                                App.trigger("goto", page.get("order"));
                                                App.viewer.controls.goto(page.get("order"));
                                            }
                                            // else {
                                            //     App.trigger( "goto", 1 );
                                            //     App.viewer.controls.goto( 1 );
                                            // }
                                        }
                                        //AleD
                                    })
                                    //;

                                    .then(function () {

                                        //AleD
                                        var _isWC = ( getURLParameter("trackId") != "null" );
                                        if (_isWC) {
                                            var _miUnidad = getURLParameter("unitId");
                                            var _miActividad = getURLParameter("activityId");
                                            if (_miUnidad != "null" && _miActividad != "null") {
                                                console.log("vamos a abrir la actividad " + _miActividad);
                                                var _idActividad = _miActividad.replace("Actividad_", "");
                                                var opcionesRecurso = {
                                                    color: 5,
                                                    idRecurso: _idActividad,
                                                    isActividad: true,
                                                    chapter: _miUnidad,
                                                    index: 0,
                                                    type: "resource",
                                                    value: "./data/RECURSOS/" + _idActividad + "/index.html"
                                                };
                                                App.execute("open", opcionesRecurso);
                                            }
                                        }
                                        //#AleD
                                    });

                                console.debug(
                                    (new Date().getTime() - timer) + "ms load modules"
                                );
                            });
                    });
                });
            }
        );
    }
);
//noinspection JSHint
define(
    'modules/menu/controllers/index',["underscore", "marionette", "app"],
    function (_, Marionette, App) {
      "use strict";

      return Marionette.Controller.extend({
        active: null,
        view: null,

        initialize: function (options) {
          options.view
              .on("itemview:touch", function (waste, options) {
                if (this.execute(options.command)) {
                  options.icon.active();
                }
              }, this);
        },

        close: function() {
          Marionette.getOption(this, "view").off("itemview:touch");
        },

        execute: function (command, options) {
          options = _.extend({
            command: (_.isString(command)) ? {
              name: "module:start",
              options: command
            } : command,
            modules: ["menu", "viewer"]
          }, options);

          var active = (!_.isEqual(this.active, options.command)) ?
              options.command : null;
              
          // Si el segundo parámetro de las opciones es "true"
          // ejecuto el comando cada vez que toco el boton del menu
          if(options.command[1] == true){
            active = options.command;
            this.options.view.collection.models[0].trigger("reset");
          }

          this.stop(this.active);
          this.active = active;
          this.start(this.active);

          return !!this.active;
        },

        reset: function () {
          Marionette.getOption(this, "view").reset();
        },

        start: function (command) {
          if (command){
            App.execute.apply(App, _.values(command));
          }

          return command;
        },

        stop: function (command) {
          if (command && command.name === "module:start") {
            App.execute.apply(App, ["module:stop", command.options]);
          }

          return command;
        }
      });
    }
);
//noinspection JSHint
define(
    'modules/menu/views/icon.collection',["views/icon.collection"],
    function (IconCollectionView) {
      "use strict";

      return IconCollectionView.extend({
        tagName: "section",

        itemEvents: {
          "touch": function (eventName, view) {
            var command = view.model.get("command");

            if (command) {
              this.trigger("touch", {command: command, icon: view});
            }
          }
        }
      });
    }
);
//noinspection JSHint
define(
    'modules/menu/views/index',["underscore", "marionette", "modules/menu/views/icon.collection"],
    function (_, Marionette, IconCollectionView) {
      "use strict";

      return Marionette.CollectionView.extend({
        /* attributes */
        // backbone & marionette
        events: {
          "tap": function () {
            this.step++;
            clearTimeout(this.timeout);
            this.timeout = _.chain(function() {
              this.step = 0;
            })
                .bind(this)
                .delay(2000)
                .value();

            if (this.step >= 15) {
              this.trigger((this.flag) ? "debug" : "version");

              this.step = 0;
              this.flag = true;

              _.chain(function() {
                this.flag = false;
              })
                  .bind(this)
                  .delay(10000)
                  .value();
            }
          }
        },
        id: "navigation",
        itemView: IconCollectionView,
        tagName: "nav",

        // custom
        flag: false,
        step: 0,
        timeout: null,

        /* methods */

        // backbone & marionette

        itemViewOptions: function (model, index) {
          return {
            collection: model.attributes,
            className: (index) ? "right" : "left"
          };
        },

        onRender: function () {
          this.delegateEvents(this.events);
        },

        // custom

        reset: function () {
          this.$("a").removeClass("active");
        }
      });
    }
);
define('modules/menu/template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["menu"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<nav id=\"navigation\"></nav>\n<h1 id=\"index\"></h1>\n<nav class=\"hidden\"></nav>";
  });

return this["JST"];

});
//noinspection JSHint
define(
    'modules/menu/regions/layout',["underscore", "marionette", "regions/replace", "modules/menu/template"],
    function (_, Marionette, ReplaceRegion, JST) {
      "use strict";

      return Marionette.Layout.extend({
        className: "norender",
        regionType: ReplaceRegion,
        regions: {
          navigation: "#navigation",
          tools: ".hidden"
        },
        tagName: "header",
        template: JST.menu,

        toggle: function () {
          this.navigation.$el.toggle();
          _.chain(function() {
            this.tools.$el.toggle();
          })
              .defer()
              .bind(this)
              .value();
        }
      });
    }
);
//noinspection JSHint
define(
    'modules/menu/menu',[
      "underscore",
      "app",
      "configuration",
      "modules/menu/controllers/index",
      "modules/menu/views/index",
      "modules/menu/regions/layout",
      "language"
    ],
    function (_, App, configuration, Controller, IndexView, IndexLayout, language) {
      "use strict";

      return App.module(
          "menu",
          function (menu, App, Backbone, Marionette, $, _) {
            /* Presets module */
            this.startWithParent = false;

            /* Private */

            // Data

            var buttons = {
              back: {type: "books", color: 1, command: ["quit"]},
              favorites: {type: "star", color: 3, command: "favorites"},
              help: {type: "question", color: 8, command: "help"},
              index: {type: "list", color: 2, command: "index"},
              search: {type: "magnifier", color: 4, command: "search"},
              tools: {type: "wrench", color: 6, command: ["tools"]},
              worktools: {type: "media", color: 5, command: "Worktools"},
              mybook: {type: "list", color: 2, command: ["mybookIndexToggle", true]},
              eye: {type: "eye", color: 6, command: ["hide:custom", true]}
            };

            /* Public */

            // Layout
            this.layout = (function () {
              var layout = new IndexLayout(),
                  region = App.header;
              region.show(layout);

              return layout;
            })();

            // Constructor
            this.addInitializer(function (options) {
              var view = new IndexView({
                collection: new Backbone.Collection()
              });

              this.options = _.defaults(options || (options = {}), {
                back: true,
                favorites: true,
                help: true,
                index: true,
                search: true,
                tools: true,
                worktools: true,
                mybook: false,
                eye: true
              });

              this.controller = new Controller({view: view});

              App.reqres.removeHandler("draw:custom");
              App.reqres.setHandler("draw:custom", function (options) {
                var deferred = $.Deferred();

                _.defaults(options || (options = {}), {
                  context: deferred
                });

                if (this.mode === "hidden") {
                  deferred.rejectWith(options.context);
                } else {
                  deferred.resolveWith(options.context);
                }

                App.trigger("draw:custom", this.mode === "hidden");

                return deferred.promise();
              }, this);

              $.when(
                      App.request("is:av", {boolean: true, context: this}),
                      App.request("is:worktools:available", {context: this})
                  )
                  .done(function (isAV, isWorkToolsAvailable) {
                    options = _.clone(options);

                    options.back = isAV;
                    buttons.worktools.inactive = isWorkToolsAvailable === undefined;

                    _.map(options, function (value, key, list) {
                      list[key] = (value) ? buttons[key] : null;
                    });

                    // Generating icons list
                    var left = new Backbone.Collection(
                            _.compact(
                                [
                                  options.back,
                                  options.index,
                                  options.favorites,
                                  options.search,
                                  options.mybook
                                ]
                            )
                        ),
                        right = new Backbone.Collection(
                            _.compact(
                                [
                                  options.worktools,
                                  options.eye,
                                  options.tools,
                                  options.help
                                ]
                            )
                        );

                    view.collection.set([left, right]);
                    this[0].layout.navigation.show(view);
                  });

              /* View events */

              view
                  .on("debug", function () {
                    App.execute("module:start", "debug");
                  })
                  .on("version", function () {
                    var version = "v" + configuration.version;

                    // Mostrar herramientas
                    App.execute("module:stop", "menu");
                    App.execute("module:start", "menu");
                    $('header').removeClass('norender');

                    App.request("is:av")
                        .done(function () {
                          version += "-av";
                        })
                        .always(function () {
                          version += "<br>WebKit: " + App.request("webKitVersion");
                          App.execute("information", version, language("version"));
                        });
                  });

              /* Events */

              App
                  .on("goto", this.goto, this)
                  .on("module:stop", this.moduleStop, this);

              /* Init */
              App.on("menu:show", function () {
                this.layout.$el.removeClass("norender");
              }, this);
            });

            this.addFinalizer(function () {
              this.layout.$el.addClass("norender");

              App
                  .off("goto", this.goto)
                  .off("menu:show")
                  .off("module:stop", this.moduleStop);
            });

            /* Methods */

            this.goto = function () {
              if (!Marionette.getOption(this, "nonstop")) {
                this.controller.stop(this.controller.active);
              }
            };

            this.moduleStop = function (module) {
              if (this.controller.active &&
                  _.contains(
                      [this.controller.active.options, "Tools"],
                      module.moduleName
                  )) {
                this.controller.reset();
                this.controller.active = null;
              }
            };
          }
      );
    }
);

// noinspection JSHint
define(
    'modules/search/views/thumbnail',[
      "views/thumbnail", 
      "template"
    ],
    function (ThumbnailView, JST) {
      "use strict";

      return ThumbnailView.extend({
        events: {
          "tap": "goto"
        },
        template: JST.thumbnail,

        goto: function (event) {
          this.trigger("highlight");
        }
      });
    }
);
define('modules/search/template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["browse"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, options, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;


  buffer += "<section>\n	<div id=\"units\"></div>\n	<div id=\"search_input\" class=\"filter_all\">\n	    <input id=\"search_query\" placeholder=\"";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "search", options) : helperMissing.call(depth0, "language", "search", options)))
    + "...\" type=\"search\" />\n	    <button class=\"icon-magnifier bgcolor-4\"></button>\n	</div>\n</section>\n<status>\n	<div id=\"status_label\"></div>\n</status>\n<output></output>";
  return buffer;
  });

this["JST"]["empty"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var stack1, options, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;


  options = {hash:{},data:data};
  return escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "noResults", options) : helperMissing.call(depth0, "language", "noResults", options)));
  });

return this["JST"];

});
// noinspection JSHint
define(
    'modules/search/regions/layout',["marionette", "modules/search/template"],
    function (Marionette, JST) {
      "use strict";

      return Marionette.Layout.extend({
        id: "search",
        regions: {
          units: "#units",
          search_input: "#search_input",
          filter: "input",
          thumbnails: "output",
          status: "status"
        },
        tagName: "section",
        template: JST.browse
      });
    }
);
define(
    'modules/search/search',[
      "app",
      "underscore",
      "views/dialog",
      "controllers/loader",
      // "views/thumbnail",
      "modules/search/views/thumbnail",
      "modules/search/regions/layout",
      "models/page.collection",
      "modules/search/template",
      "language",
      "views/chapters.dropdown"
    ],
    function (app, _, DialogView, Loader, ThumbnailView, SearchLayout, PageCollectionModel, JST, Language, ChaptersDropdownViewDeferred) {
      "use strict";

      return app.module("search", function (search, App, Backbone, Marionette, $, _) {
        /* Presets module */
        this.startWithParent = false;

        /* Private */

        var lastQuery = "";

        // Data
        var cache = [];

        // View
        var EmptyView = Marionette.ItemView.extend({
          template: JST.empty
        });

        var SearchView = Marionette.CompositeView.extend({
          events: {
            "touch @ui.button": "search",
            "keydown @ui.input": function (event) {
              if (event.keyCode === 13) {
                this.ui.input.blur();
                this.search();
              } else {
                // Evito que las flechas me cambien de página
                event.stopPropagation();
              }
            }
          },
          id: "search_box",
          tagName: "main",

          emptyView: EmptyView,
          itemView: ThumbnailView,
          itemViewContainer: "output",
          ui: {
            button: "section:first-of-type button",
            input: "input[type=search]",
            list: "output",
            units: "#units",
            status: "status"
          },
          template: JST.browse,

          initialize: function () {
            this.collection = new PageCollectionModel();
            $(this.el).on("touchstart", function(ev) {
              /**
               * Fix 3494 versión 2
               * La solución que adoptó Miguel debería haber funcionado en teoría el problema es que al ser una aplicación híbrida no se que entenderá exactamente el iPad por lo que lo he solucionado añadiendo un elemento al DOM para aplicarle el foco y posteriormente acto seguido lo elimino con lo que se consigue perder el foco para recuperarlo posteriormente de forma que se fuerza a los dispositivos que antes no perdían el foco.
               * De esta forma finalmente ya se puede volver a escribir sin problemas.
               * @author Mario Poyato <mario.poyato@inmediastudio.es>
               */
              App.request("is:device", {not: false})
                .done(function () {
                  /* En el caso de ser un dispositivo móvil de Apple hay que forzar el desenfoque añadiendo un input al DOM para posteriormente aplicar el foco a ese elemento y eliminandolo */
                  if (!!navigator.userAgent.match(/iPhone|iPad|iPod/i)) {
                    /* Añadimos el elemento en el DOM */
                    $("section#search div#search_input input#search_query[type=search]").after('<input type="checkbox" id="focusableAUX" style="height:0;margin-left:-200px;" />');
                    /* Colocamos el foco en el elemento auxiliar */
                    $('section#search div#search_input input#focusableAUX').focus();
                    /* Eliminamos el elemento del DOM */
                    $('section#search div#search_input input#focusableAUX').remove();
                  }
                  /* En otro caso simplemente desenfocar el input del campo de búsqueda para que el motor del navegador simplemente vuelva a aplicar el foco */
                  else {
                    $("section#search div#search_input input#search_query[type=search]").blur();
                  }
                })
                /* Al igual que en dispositivos moviles que no sean de Apple se desenfoca el input del campo de búsqueda */
                .fail(function () {
                  $("section#search div#search_input input#search_query[type=search]").blur();
                })
            });
          },

          onRender: function() {
            if (this.collection.size()) {
              this.ui.list.show();
            }
          },

          onShow: function() {
            // Restauro la ultima busqueda
            this.ui.input.val(lastQuery);
          },

          search: function () {
            var unit = search.unit[0].dataset.code;
            var query = this.ui.input.val().toLowerCase();
            
            lastQuery = query;
            
            // Elimino resultados anteriores
            App.trigger("search:removeResults");

            if(!!query){
              search.controller.fastSearch(query, unit);
              // search.controller.search(query, unit);
            }
          }
        });


        /* Public */

        // Controller
        var Controller = Marionette.Controller.extend({
          initialize: function (view) {
            this.view = view;

            // En la busqueda rápida no puedo buscar por unidad
            // Oculto el combo de seleccion de unidades.
            app.request("searchIndex")
              .done(function (data) {
              $("#search #units").hide();
            });

          },

          fastSearch: function(query, unit){
            var that = this;
            var view = this.view;

            this.view.ui.list.hide();
            this.view.collection.reset();

            app.request("searchIndex")
              .done(function (pagesPlainText) {

                var matches = [];

                _.each(pagesPlainText, function(text, index){
                  if(text.toLowerCase().indexOf(query.toLowerCase()) > -1){
                    matches.push(index);

                    App.request("page", {page: index, model: true})
                      .done(function(pagina) {
                        var ocurrences = [];
                        text = text.replace(/\t/g, " ").toLowerCase();
                        pagina = pagina.clone();
                        
                        var higlightOptions = {
                          index: pagina.get("index"), 
                          number: pagina.get("id"), 
                          chapter: pagina.get("code"), 
                          text: text
                        };

                        pagina.set("query", query);
                        pagina.set("search_fast", true);
                        pagina.set("highlight_page", higlightOptions);

                        view.collection.add(pagina);
                      })
                      .fail(function(err) {
                        console.log(err);
                      });

                  }
                });

                view.ui.list.show();
                
                if(!matches.length){
                  console.log("No hay resultados");
                }
              })
              .fail(function(err) {
                // No existe archivo JSON de busqueda. Busco con el algoritmo lento
                search.controller.search(query, unit);
              });
          },

          // Transforma las páginas a texto y las almacena
          // en cache.
          search: function (query, unit) {
            // Muestro selector de unidades
            $("#search #units").show();

            var loader = new Loader({
                  auto: true,
                  bgColor: "#efefef",
                  parent: "status"
                }),
                view = this.view;
            
            var that = this;

            this.view.ui.list.hide();
            this.view.collection.reset();
            loader.render();

            // Obtengo capítulos
            app.request("chapters")
              .done(function (chapters) {
                var cantPaginas = 0;
                var flag = 0; // Para saber cuando termine de cargar páginas

                // Si la busqueda no es completa, solo contar paginas de este capitulo
                if(unit != "*"){
                  var tmp_chapter = chapters.findWhere({code: unit});
                  chapters = new Backbone.Collection();
                  chapters.add(tmp_chapter); 
                  cantPaginas = chapters.models[0].get("pages").length; // Solo este capítulo
                } else {
                  cantPaginas = chapters.pages().length; // Todo el libro
                }

                // Recorro todas las páginas del capítulo y
                // voy llenando la caché con el texto
                chapters.each(function(chapter, index, list){
                  var code = chapter.get("code");
                  var pages = chapter.get("pages");
                  var haveIt = _.some(cache, function(elem){ 
                                              return elem.chapter == code
                                            });

                  // Si no tengo el capitulo en cache, lo cargo
                  if(!haveIt){
                    app.request("document", {code: code, delay: 0})
                      .done(function(document){
                        // Recorro todas las paginas del capitulo
                        pages.each(function(page, index){
                          var numPagina = page.get("id");
                          that.view.$('#status_label').html(Language("searching") + "...<br>" + Language("page") + " " + numPagina + " / " + cantPaginas);

                          document.text(index)
                            .done(function(text){
                              flag++;

                              // Almaceno el texto en cache
                              cache.push({index: index, number: numPagina, chapter: code, text: text});

                              // Necesito esperar a que termine de cargar
                              // todas las páginas antes de continuar
                              if(flag == cantPaginas){
                                that.find(query, unit, loader, that.view);
                              }
                            });
                        });
                      })
                      .fail(function(err) {
                        console.log(err);
                        // Si hay un error, descuento las páginas de ese chapter
                        // para continuar con la búsqueda.
                        cantPaginas -= pages.length;
                      });
                    } else {
                      // Ya lo tengo
                      // Rresto las paginas de la unidad actual
                      if(cantPaginas > 1){
                        cantPaginas -= pages.length;
                      } 

                      // Si cantPaginas <= 1, significa que ya tengo en cache todas las unidades
                      if(cantPaginas <= 1) {
                        // Paso directamente a la busqueda
                        that.find(query, unit, loader, that.view);
                      }
                    }
                });
              })
              .fail(function(err) {
                console.log(err);
              });
          },

          // Recorre la cache en busca de ocurrencias
          find: function(query, unit, loader, view){
            var that = this;
            var found = false;
            var tmp_cache = [];

            // Si es por capíulo, filtro por unidad, si no, busco e toda la cache.
            if(unit == "*"){
              tmp_cache = cache;
            } else {
              tmp_cache = _.where(cache, {chapter: unit});
            }

            // Recorro todos los capitulos de la cache
            _.each(tmp_cache, function(element, index, list){
              var page = list[index];
              var text = page.text.replace(/\t/g, " ").toLowerCase();
              query = query.toLowerCase();

              // Obtengo los indices de todas las ocurrencias
              var allResults = that.getAllResults(query, text, false);

              if(allResults.length){
                found = true;

                // Agrego thumbnail
                App.request("page", {page: page.number, model: true})
                  .done(function(pagina) {

                    pagina = pagina.clone();

                    pagina.set("query", query);
                    pagina.set("search_fast", true);
                    pagina.set("highlight_page", page);

                    view.collection.add(pagina);
                  })
                  .fail(function(err) {
                    console.log(err);
                  });
              }
            });

            if(!found){
              console.log("NO ENCONTRADO");
            }

            view.$('#status_label').html("");
            view.ui.list.show();
            loader.close();
            view.$('.loader').remove(); // Sin esto no se elimina el div del loader
          },

          // Obtiene todas las ocurrencias de una página
          getAllResults: function(searchStr, str, caseSensitive) {

            var startIndex = 0, searchStrLen = searchStr.length;
            var index, indices = [];
            if (!caseSensitive) {
              str = str.toLowerCase();
              searchStr = searchStr.toLowerCase();
            }

            while ((index = str.indexOf(searchStr, startIndex)) > -1) {
              indices.push(index);
              startIndex = index + searchStrLen;
            }

            return indices;
          },

          // Resalta los resultados sobre la página
          highlight: function(page, from, to) {

              console.log("deprecated");

          }
        });

        // Defino el dialogo
        this.layout = new DialogView({
          color: 4,
          height:"auto",
          title: Language("search"),
          width: 600
        });

        this.layout.on("quit", function() {
          App.execute("module:stop", "search");
        });

        // Constructor
        this.addInitializer(function (options) {
          var view = new SearchView();
          this.controller = new Controller(view);

          // Defino el layout interno
          var container = new SearchLayout();
          this.layout.show(container);

          var that = this;

            // Listener para hacer el marcado de resultados en el libro
            view.on("itemview:highlight", function (view, model) {

                var page = view.model.get("highlight_page");
                var query = view.model.get("query");

                // Ir a la página con el resultado
                App.viewer.trigger("goto", parseInt(view.model.get("id"), 10));

                // This flag is used to trigger the action on the next interval as it seems offset is not correct as soon as its set
                var flag = false;

                var tempInterval = setInterval(function() {

                    var pageDiv = $("#p" + page.number);

                    if (typeof(pageDiv.offset()) !== "undefined" && !flag) {

                        flag = true;

                    } else if (typeof(pageDiv.offset()) !== "undefined" && flag) {

                        clearInterval(tempInterval);

                        var pageScale = parseFloat(pageDiv.css('transform').split('(')[1].split(')')[0].split(',')[0]);
                        var viewerScale = parseFloat($("#viewer > li").css('transform').split('(')[1].split(')')[0].split(',')[0]);
                        var initialTop = pageDiv.offset().top;
                        var initialLeft = pageDiv.offset().left;
                        var resultSet = [];

                        pageDiv.find("div").each(function (index) {

                            var originalText = $(this)[0].innerHTML;

                            var pos;
                            var newText = "";
                            var temp = originalText;

                            while ((pos = temp.toLowerCase().indexOf(query.toLowerCase())) !== -1) {

                                var foundString = temp.substr(pos, query.length); // To obtain it case sensitive as the search was insensitive

                                newText += temp.substr(0, pos + query.length).replace(foundString, "<underline>" + foundString + "</underline>");
                                temp = temp.substr(pos + query.length, temp.length);

                            }

                            newText += temp;

                            $(this)[0].innerHTML = newText;

                            $("underline").each(function () {

                                resultSet.push({
                                    'page': page,
                                    'x': ($(this).offset().left - initialLeft) / pageScale / viewerScale * 2,
                                    'y': ($(this).offset().top - initialTop) / pageScale / viewerScale * 2,
                                    'width': $(this).width() / 2,
                                    'height': $(this).height() / 2
                                });

                            });

                            $(this)[0].innerHTML = originalText;

                        });

                        App.trigger("search:drawResults", resultSet);

                    }

                }, 100);


                /*
                App.request("document", {code: view.model.get("code"), delay: 0})
                    .done(function(document){

                        document.text(indice)
                            .done(function(text){
                                // Obtengo los indices de todas las ocurrencias
                                var allResults = that.controller.getAllResults(query, text, false);

                                // Obtengo texto anterior y posterior para titulo de thumbnails
                                var firstMatch = allResults[0];
                                var lastSpace = text.substr(0, firstMatch - 4).lastIndexOf(" ");
                                var nextSpace = text.substr(firstMatch + query.length + 3, text.length - firstMatch)
                                        .indexOf(" ") + firstMatch + query.length + 3;
                                var title = text.substr(lastSpace, firstMatch - lastSpace) +
                                    text.substr(firstMatch, query.length) +
                                    text.substr(firstMatch + query.length, nextSpace - firstMatch - query.length);
                                title = title.trim();
                                page.title = title;

                                // Recorro todas las ocurrencias y las agrego a un array
                                _.each(allResults, function(aMatch) {
                                    var from = aMatch;
                                    var to = from + query.length;

                                    // Marcar los resultados en el libro
                                    that.controller.highlight(page, from, to);
                                });
                            });
                    });


*/




                // Cierro el dialogo. Esto sirve para cuando el
                // resultado está en la misma página en la que estoy
                App.execute("module:stop", "search");
            });

          // Obtengo unidades y selector de unidades
          $.when(
            App.request("chapter", {
              context: this
            }),
            ChaptersDropdownViewDeferred()
          )
            .done(function(chapter, ChaptersSelectorView) {
              // Defino vista de selector de unidades
              ChaptersSelectorView = ChaptersSelectorView.extend({
                className: "my dropdown",
                tagName: "h2",

                initialize: function () {
                  this.model = this.model.clone();
                  this.model.set("title", Language("allUnits"));
                  this.model.set("color", 4);
                  this.model.set("options", this.model.get("options").slice(0));
                  this.model.get("options").push({
                    name: Language("allUnits"),
                    data: {code: "*"}
                  });
                },

                onShow: function() {
                  // Para buscar solo en la unidad que estoy viendo
                  /*var buscarEn = _.filter(this.model.get("options"), function(obj) {
                    if(obj.data.code == chapter.get("code"))
                      return obj;
                    }
                  );
                  buscarEn = buscarEn[0];*/

                  // Para empezar a buscar en "Todas las unidades" por default
                  var buscarEn = this.model.get("options");
                  buscarEn = buscarEn[buscarEn.length - 1];


                  var index = _.indexOf(
                    this.model.get("options"),
                    _.findWhere(
                      this.model.get("options"),
                      buscarEn
                    )
                  );

                  this.trigger(
                      "select",
                      this.$("ul a:nth(" + index +")")
                  );
                }
              });

              // Selector de unidades
              var selector = new ChaptersSelectorView({color: 4});
              selector.on("select", function(data){
                that.unit = data;
              });
              container.units.show(selector);

              // Input de busqueda
              container.search_input.show(view);

              // Evito problemas de teclado de iOS
              App.request("is:device")
                .fail(function() {
                  // Si no es device hago foco en el input
                  $('#search_query').focus();
                });
            })
            .fail(function(err) {
              console.log(err);
            });
        });

        this.addFinalizer(function () {
          this.layout.currentView && this.layout.currentView.close();
        });
      });
    }
);
define('modules/tools/template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["cancel"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<section class=\"left\"></section>\n<section class=\"right\"><a><i class=\"icon-close\"></i></a></section>";
  });

this["JST"]["closeDialog"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, options, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;


  buffer += "<p>";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "close_tools", options) : helperMissing.call(depth0, "language", "close_tools", options)))
    + "</p>\n<button class=\"bgcolor-6\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "accept", options) : helperMissing.call(depth0, "language", "accept", options)))
    + "</button>\n<button class=\"bgcolor-6\"><i class=\"icon-save\"></i></button>\n<button class=\"bgcolor-6\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "cancel", options) : helperMissing.call(depth0, "language", "cancel", options)))
    + "</button>";
  return buffer;
  });

this["JST"]["editHTMLText"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, stack2, options, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function";


  buffer += "<textarea placeholder=\"";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "text", options) : helperMissing.call(depth0, "language", "text", options)))
    + "\">";
  if (stack2 = helpers.text) { stack2 = stack2.call(depth0, {hash:{},data:data}); }
  else { stack2 = (depth0 && depth0.text); stack2 = typeof stack2 === functionType ? stack2.call(depth0, {hash:{},data:data}) : stack2; }
  buffer += escapeExpression(stack2)
    + "</textarea>";
  return buffer;
  });

this["JST"]["icon-tools"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression, self=this;

function program1(depth0,data) {
  
  var buffer = "", stack1;
  buffer += "class=\"icon-";
  if (stack1 = helpers['class']) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0['class']); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\"";
  return buffer;
  }

function program3(depth0,data) {
  
  
  return "class=\"label\"";
  }

function program5(depth0,data) {
  
  var stack1;
  if (stack1 = helpers.label) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.label); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  return escapeExpression(stack1);
  }

  buffer += "<i ";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0['class']), {hash:{},inverse:self.noop,fn:self.program(1, program1, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += " ";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.label), {hash:{},inverse:self.noop,fn:self.program(3, program3, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += ">";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.label), {hash:{},inverse:self.noop,fn:self.program(5, program5, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "</i>\n";
  return buffer;
  });

this["JST"]["kinetic"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "";


  return buffer;
  });

this["JST"]["layout-tool"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<nav id=\"tools_primary\">primary</nav>\n<nav id=\"tools_controls\">controls</nav>\n<nav id=\"tools_secondary\">secondary</nav>";
  });

this["JST"]["menu"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<nav id=\"tools_primary\">primary</nav>\n<nav id=\"tools_controls\">controls</nav>\n<nav id=\"tools_secondary\">secondary</nav>";
  });

this["JST"]["modal-body"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<div class=\"colors-catalog clearfix\">\n    <div class=\"col\">\n        <span class=\"color-btn colorcat-01\"></span>\n        <span class=\"color-btn colorcat-02\"></span>\n        <span class=\"color-btn colorcat-03\"></span>\n    </div>\n    <div class=\"col\">\n        <span class=\"color-btn colorcat-04\"></span>\n        <!-- Cuando se selecciona un color se añade la clase active para resaltarlo -->\n        <span class=\"color-btn colorcat-05 active\"></span>\n    </div>\n    <div class=\"col\">\n        <span class=\"color-btn colorcat-06\"></span>\n        <span class=\"color-btn colorcat-07\"></span>\n        <span class=\"color-btn colorcat-08\"></span>\n    </div>\n    <div class=\"col\">\n        <span class=\"color-btn colorcat-09\"></span>\n        <span class=\"color-btn colorcat-10\"></span>\n    </div>\n    <div class=\"col\">\n        <span class=\"color-btn colorcat-11\"></span>\n        <span class=\"color-btn colorcat-12\"></span>\n        <span class=\"color-btn colorcat-13\"></span>\n    </div>\n</div>\n";
  });

this["JST"]["modal-open"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression;


  if (stack1 = helpers.name) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.name); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\n";
  return buffer;
  });

this["JST"]["modal"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression;


  buffer += "<div class=\"modal-content\">\n    <div id=\"tools_modal_header\">\n        <button type=\"button\" class=\"close\"><i class=\"icon-close\"></i></button>\n        <h4 class=\"modal-title\">";
  if (stack1 = helpers.titleModal) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.titleModal); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "</h4>\n    </div>\n    <div id=\"tools_modal_body\" class=\"modal-body\">\n\n    </div>\n</div>\n\n\n";
  return buffer;
  });

this["JST"]["saveas"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, options, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;


  buffer += "<span>";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "tools.file_name", options) : helperMissing.call(depth0, "language", "tools.file_name", options)))
    + "</span>\n<input type=\"text\" value = \"\" />\n<button class=\"js-ok\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "accept", options) : helperMissing.call(depth0, "language", "accept", options)))
    + "</button>\n<button class=\"js-cancel\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "cancel", options) : helperMissing.call(depth0, "language", "cancel", options)))
    + "</button>";
  return buffer;
  });

return this["JST"];

});
define(
    'modules/tools/tools',[
      "app",
      "modules/tools/template",
      "views/icon.collection",
      "regions/replace",
      "language",
      "views/dialog"
    ],
    function (App, JST, IconCollection, ReplaceRegion, Language, DialogView) {
      "use strict";

      return App.module("Tools", function (Tools, App, Backbone, Marionette, $, _) {

        var self = this;
        this.startWithParent = false;
        this.selectedColor = "colorcat-07";
        

        /*

          ================================
          ========= FUNCIONAMIENTO =======
          ================================

          shape: {                      // Nombre del boton que luego se incluye en un grupo dentro de "buttonGroups"
            type: "shape",              // Tipo de herramienta -> El icono debe llamarse igual en icons.css
            menu: "shape_menu",         // Nombre del grupo de botones que usara para llenar el menu
            submenu: "shape_submenu",   // Nombre del grupo de botones que usara para llenar el submenu
            toggle: false,              // Si esta en "false", el boton hara una accion y no quedara seleccionado ".active"
            action: true                // Si esta en "true", este boton ejecuta una accion de la funcion this.execute()
          }

        */

        var buttons = {
          // === Generic ===
          back: {
                  type: "arrow-up",
                  toggle: false
                },
          undo: {
                  type: "undo",
                  toggle: false
                },
          redo: {
                  type: "redo",
                  toggle: false
                },
          save: {
                  type: "cloud",
                  menu: "save_menu",
                  toggle: false
                },
          load: {
                  type: "cloud-open-1",
                  toggle: false
                },
          close: {
                  type: "close",
                  toggle: false
                },

          // === menu ===
          cursor: {
                  type: "cursor",
                  menu: "cursor_menu"
                },
          shape: {
                  type: "shapes",
                  menu: "shape_menu",
                  submenu: "shape_submenu"
                },
          text: {
                  type: "type",
                  menu: "text_menu"
                },
          highlight: {
                  type: "underlining",
                  menu: "highlight_menu",
                  submenu: "shape_submenu"
                },
          picture: {
                  type: "scloud",
                  action: true
                },
          link: {
                  type: "clip-dot",
                  menu: "link_menu"
                },
          mask: {
                  type: "masking",
                  menu: "mask_menu",
                  submenu: "mask_submenu"
                },

          // === Primario ===
          // --- cursor
          cursor_sub: {
                  type: "cursor-clean",
                  action: true
                },
          text_select: {
                  type: "text-select",
                  action: true
                },
          copy: {
                  type: "copy",
                  toggle: false,
                  action: true
                },
          paste: {
                  type: "paste",
                  toggle: false,
                  action: true
                },
          trash: {
                  type: "trash",
                  toggle: false,
                  action: true
                },


          // --- shape
          square: {
                  type: "square",
                  submenu: "shape_submenu",
                  action: true
                },
          circle: {
                  type: "circle",
                  submenu: "shape_submenu",
                  action: true
                },
          triangle: {
                  type: "triangle",
                  submenu: "shape_submenu",
                  action: true
                },
          arrow: {
                  type: "arrow",
                  submenu: "shape_submenu",
                  action: true
                },
          line: {
                  type: "line",
                  submenu: "shape_submenu",
                  action: true
                },
          doddle: {
                  type: "doodle",
                  submenu: "shape_submenu",
                  action: true
                },
          stamp: {
                  type: "stamp",
                  submenu: "stamp_submenu"
                },

          // --- text
          text_sub: {
                  type: "character",
                  action: true
                },
          text_box: {
                  type: "edit",
                  action: true
                },
          text_arrow: {
                  type: "text-direction",
                  action: true
                },

          // --- highlight
          underline: {
                  type: "text-underline",
                  submenu: "shape_submenu",
                  action: true
                },
          highlight_sub: {
                  type: "text-highlight",
                  submenu: "shape_submenu",
                  action: true
                },

          // --- link
          link_area: {
                  type: "link-square",
                  action: true
                },
          link_sub: {
                  type: "clip",
                  action: true
                },

          // --- mask
          mask_regular: {
                  type: "mask",
                  submenu: "mask_submenu"
                },
          mask_inverse: {
                  type: "mask-inverse",
                  submenu: "mask_inverse_submenu"
                },

          // --- save
          save_sub: {
                  type: "save-text",
                  toggle: false,
                  action: true
                },
          save_as: {
                  type: "save-as-text",
                  toggle: false,
                  action: true
                },


          // === Secundario
          // --- Generic
          color_picker: {
                  type: "btn-color-select",
                  toggle: false
                },
                
          // --- shape -> stamp
          stamp_tick: {
                  type: "tick",
                  action: true
                },
          stamp_cross: {
                  type: "cross",
                  action: true
                },

          // --- mask -> mask_regular
          mask_square_hollow: {
                  type: "fill-square",
                  action: true
                },
          mask_circle_hollow: {
                  type: "fill-circle",
                  action: true
                },
          mask_square_filled: {
                  type: "transparent-square",
                  action: true
                },
          mask_circle_filled: {
                  type: "transparent-circle",
                  action: true
                },

          // --- mask -> mask_inverse
          mask_inverse_square_hollow: {
                  type: "fill-square-inverse",
                  action: true
                },
          mask_inverse_circle_hollow: {
                  type: "fill-circle-inverse",
                  action: true
                },
          mask_inverse_square_filled: {
                  type: "transparent-square-inverse",
                  action: true
                },
          mask_inverse_circle_filled: {
                  type: "transparent-circle-inverse",
                  action: true
                }

        };

        var buttonGroups = {
          // === Genericos ===
          main: [
                      buttons.cursor,
                      buttons.shape,
                      buttons.text,
                      buttons.highlight,
                      buttons.picture,
                      buttons.link,
                      buttons.mask
                    ],
          controls: [
                      buttons.undo,
                      buttons.redo,
                      buttons.save,
                      buttons.load,
                      buttons.close
                    ],
          // === Menus ===
          cursor_menu: [
                      buttons.back,
                      buttons.cursor_sub,
                      buttons.text_select,
                      buttons.copy,
                      buttons.paste,
                      buttons.trash
                    ],
          shape_menu: [
                      buttons.back,
                      buttons.square,
                      buttons.circle,
                      buttons.triangle,
                      buttons.arrow,
                      buttons.line,
                      buttons.doddle,
                      buttons.stamp
                    ],
          text_menu: [
                      buttons.back,
                      buttons.text_sub,
                      buttons.text_box,
                      buttons.text_arrow
                    ],
          highlight_menu: [
                      buttons.back,
                      buttons.underline,
                      buttons.highlight_sub
                    ],
          link_menu: [
                      buttons.back,
                      buttons.link_area,
                      buttons.link_sub
                    ],
          mask_menu: [
                      buttons.back,
                      buttons.mask_regular,
                      buttons.mask_inverse
                    ],
          save_menu: [
                      buttons.back,
                      buttons.save_sub,
                      buttons.save_as
                    ],
          // === Submenus ===
          shape_submenu: [
                      buttons.color_picker
                    ],
          stamp_submenu: [
                      buttons.stamp_tick,
                      buttons.stamp_cross
                    ],
          mask_submenu: [
                      buttons.mask_square_hollow,
                      buttons.mask_circle_hollow,
                      buttons.mask_square_filled,
                      buttons.mask_circle_filled
                    ],
          mask_inverse_submenu: [
                      buttons.mask_inverse_square_hollow,
                      buttons.mask_inverse_circle_hollow,
                      buttons.mask_inverse_square_filled,
                      buttons.mask_inverse_circle_filled
                    ]
        };

        // Extiendo IconCollection para recibir eventos
        IconCollection = IconCollection.extend({
          itemEvents: {
            "touch": function(eventname, view){
              self.handleClick(view);
            }
          }
        });

        // Defino grupos de herramientas
        // Grupo primario
        var toolsPrimaryCollection = new Backbone.Collection(buttonGroups.main);
        var toolsPrimaryView = new IconCollection({
          collection: toolsPrimaryCollection
        });

        // Grupo secundario
        var toolsSecondaryCollection = new Backbone.Collection([]);
        var toolsSecondaryView = new IconCollection({
          collection: toolsSecondaryCollection
        });

        // Grupo de controles
        var toolsControlsCollection = new Backbone.Collection(buttonGroups.controls);
        var toolsControlsView = new IconCollection({
          collection: toolsControlsCollection
        });

        // Defino el layout
        var Layout = Backbone.Marionette.Layout.extend({
          tagName: "main",
          template: JST["layout-tool"],
          id: "tools",

          regions: {
            primary: "#tools_primary",
            secondary: "#tools_secondary",
            controls: "#tools_controls"
          },

          regionType: ReplaceRegion
        });

        // Constructor
        this.addInitializer(function(){
          // Reseteo detector de cambios de orientacion
          window.orientation_changed = false;
          
          // Oculto botones que no quiero mostrar
          this.hideButtons();

          // Dibujo el layout
          var layout = new Layout();
          App.menu.layout.tools.show(layout);
          
          // Dibujo grupos en el layout
          layout.primary.show(toolsPrimaryView);
          layout.secondary.show(toolsSecondaryView);
          layout.controls.show(toolsControlsView);

          $("body > header > *").toggle();
        });

        // Manejador de clicks
        this.handleClick = function(view){
          var model = view.model;
          var type = model.get("type");
          var menu = model.get("menu") || null;
          var submenu = model.get("submenu") || null;
          var selected = model.get("selected") || null;
          var selected_sub = model.get("selected_sub") || null;
          var toggle = model.has("toggle") ? model.get("toggle") : null;
          var action = model.get("action") || null;

          // Elimino seleccion de copia de texto (siempre)
          App.trigger("draw:textDeselect", true);


          switch(type){
            // Casos especiales de herramientas
            case "close":
              App.request("draw:exit", {context: this})
                .done(function () {
                  this.exitMenu();
                })
                .fail(function(){
                  this.showUnsavedDialog();
                });
              break;

            case "arrow-up":
              // Quito sombra del boton
              $("#tools nav:nth-of-type(1) a:nth-of-type(1)").css("box-shadow", "");

              // VOLVEMOS A PERMITIR EL SCROLL
			        $('ul#viewer li').unbind('touchmove');

              // Quito herramienta actual
              App.trigger("draw:brush", null);
              App.trigger("draw:deselect", true);
              App.trigger("kinetic:select", false);
              
              $("#tools").removeClass("tools-submenu");
              // Asigno el espacio que ocupan los botones del NAV de controles
              var rightLength = toolsControlsCollection.length * 60;
              $("#tools nav:nth-of-type(1)").css("right", rightLength);

              toolsPrimaryCollection.reset();
              toolsPrimaryCollection.add(buttonGroups.main);
              toolsSecondaryCollection.reset();
              break;

            case "btn-color-select":
              this.showColorDialog();
              break;

            // Controles
            case "undo":
              App.trigger("draw:undo");
              break;

            case "redo":
              App.trigger("draw:redo");
              break;

            case "cloud-open-1":
              this.showOpenDialog();
              break;

            // Comportamiento generico de herramientas
            default:
              if(menu){
                // Cambio el fondo del menu a color naranja
                $("#tools").addClass("tools-submenu");

                // EVITAMOS EL TOUCHMOVE, PARA QUE NO DEJE HACER SCROLL
				        $('ul#viewer li').bind('touchmove', function(e){e.preventDefault()});

                toolsPrimaryCollection.reset();
                toolsPrimaryCollection.add(buttonGroups[menu]);

                // Agrego sombra al boton de volver
                $("#tools nav:nth-of-type(1) a:nth-of-type(1)").css("box-shadow", "4px 0 6px -2px rgba(0, 0, 0, .5)");

                // Para el menu de guardar, debo poner texto en vez de iconos
                // y no seleccionar el primero
                if(menu == "save_menu"){
                  $(".icon-save-text").parent().html('<p style="margin: 0;">' + Language("cloud") + '</p>');
                  $(".icon-save-as-text").parent().html('<p style="margin: 0;">' + Language("saveas") + '</p>');

                  // Tambien debo ocultar el submenu
                  toolsSecondaryCollection.reset();
                } else {
                  // Seleccionamos boton predefinido en menu primario
                  // Obtengo el boton dentro del DOM. Uso siempre el primero
                  var btnEl = toolsPrimaryView.el.children[1];
                  $(btnEl).click();
                }
              }

              if(submenu){
                toolsSecondaryCollection.reset();
                toolsSecondaryCollection.add(buttonGroups[submenu]);

                // Asigno el espacio que ocupan los botones del submenu
                // para ocultar detras los del menu. Sumo la cantidad de
                // botones del NAV de controles, mas los del submenu.
                var rightLength = (toolsControlsCollection.length + toolsSecondaryCollection.length) * 60;
                $("#tools nav:nth-of-type(1)").css("right", rightLength);

                // Seleccionamos boton predefinido en menu secundario
                // Obtengo el boton dentro del DOM. Uso siempre el primero
                var btnEl = toolsSecondaryView.el.children[0];
                $(btnEl).addClass("active");

                // Para el caso del selector de colores, no quiero que
                // quede seleccionado y debo asignarle un color actual
                if(submenu == "shape_submenu"){
                  $(btnEl).removeClass("active");
                  this.changeColor(this.selectedColor);
                } else {
                  $(btnEl).click();
                }
              }
          }

          // Oculto botones que no quiero mostrar
          this.hideButtons();

          // Ponemos el boton como activo
          if(toggle !== false){
            view.$el.parent().find(".active").removeClass("active");
            view.active();
          }

          // Ejecutamos la accion solicitada
          if(action){
            this.execute(view);
          }
        };

        // Control de acciones
        this.execute = function(view){
          var model = view.model;
          var type = model.get("type");

          switch(type){
            // Casos especiales de acciones
            case "cursor-clean":
              App.trigger("draw:brush", null);
              App.trigger("draw:select", true);
              App.trigger("kinetic:select", true);
              break;

            case "copy":
              App.trigger("draw:copy");
              App.trigger("draw:brush", null);
              App.trigger("draw:select", true);
              App.trigger("kinetic:select", true);
              view.$el.parent().find(".active").removeClass("active");
              view.$el.parent().find(".icon-cursor-clean").parent().addClass("active");
              break;

            case "paste":
              App.trigger("draw:paste");
              App.trigger("draw:brush", null);
              App.trigger("draw:select", true);
              App.trigger("kinetic:select", true);
              view.$el.parent().find(".active").removeClass("active");
              view.$el.parent().find(".icon-cursor-clean").parent().addClass("active");
              break;

            case "trash":
              App.trigger("draw:trash");
              App.trigger("draw:brush", null);
              App.trigger("draw:select", true);
              App.trigger("kinetic:select", true);
              view.$el.parent().find(".active").removeClass("active");
              view.$el.parent().find(".icon-cursor-clean").parent().addClass("active");
              break;

            case "save-text":
              App.trigger("kinetic:select", true);
              App.trigger("draw:save");
              break;

            case "save-as-text":
              App.trigger("kinetic:select", true);
              this.showSaveAsDialog();
              break;

            // Acciones por defecto
            default:
              App.trigger("kinetic:select", false);
              App.trigger("draw:brush", type);
          }
        };

        // Cambia el color en todo el scope de las herramientas
        this.changeColor = function(color){
          this.selectedColor = color;
          $("i.icon-btn-color-select")
            .removeClass()
            .addClass("icon-btn-color-select")
            .addClass(this.selectedColor);

          App.trigger("draw:color", this.getHexColor());
        };

        this.showColorDialog = function(){
          var colorDialog = new DialogView({
            color: 6,
            width: 365,
            height: 265,
            className: "dialog",
            title: Language('color')
          });

          var ContentDialog = Marionette.ItemView.extend({
            template: JST["modal-body"],
            events: {
              "keyup textarea#area": function (e) {
                this.model.set("text", e.currentTarget.value);
              },

              "click span": function (e) {
                var _color = e.currentTarget.className.split(" ")[1];
                this.selectedColor = _color;
                this.trigger("colorChanged", _color);
              }
            },
            onRender: function () {
              this.$el.find('.col').find('span').removeClass('active');
              this.$el.find('.col').find('.' + self.selectedColor).addClass('active');
            }
          });

          var contentDialog = new ContentDialog();
          contentDialog.on("colorChanged", function(color){
            this.changeColor(color);
            
            colorDialog.close();
          },this);

          colorDialog.show(contentDialog);
        };

        this.showOpenDialog = function(){
          var dibujos = new (Backbone.Collection.extend({url:'draw'}))();
          dibujos.fetch().done(function(){
            if (!dibujos.findWhere({name: "default"})) {
              dibujos.add({name: "default"});
            }

            var dialogOpen = new DialogView({
              color: 6,
              title: Language("open"),
              width: 400,
              height: 'auto'
            });

            var listOpen = Marionette.ItemView.extend({
              template: JST["modal-open"],
              tagName: 'li',
              className: 'open_li',
              events: {
                "click": function (e) {
                  window.configuration.lastSaved = this.model.get("id"); // Mantengo nombre en variable global para guardar correctamente luego.
                  window.configuration.lastSavedName = this.model.get("name");
                  this.trigger("quit");
                  App.trigger("draw:load", this.model.get("id"));
                }
              }
            });

            var ContentDialogOpen = Marionette.CollectionView.extend({
              tagName: 'ul',
              itemView: listOpen,
              className: 'open',
              itemEvents: {
                'quit': function () {
                  this.trigger('quit');
                }
              }
            });

            var contentDialogOpen = new ContentDialogOpen ({collection:dibujos});
            dialogOpen.show(contentDialogOpen);
          });
        };

        this.showSaveAsDialog = function(){
          var dialogSaveAs = new DialogView({
            color: 6,
            title: Language("saveas"),
            width: 365,
            height: 200
          });

          dialogSaveAs.show(new Marionette.ItemView({
            template: JST["saveas"],
            className: "wrapper",
            model: new Backbone.Model({}),
            events: {
              "click .js-ok": function (ev) {
                window.configuration.lastSaved = this.$("input").val(); // Mantengo nombre en variable global para guardar correctamente luego.
                App.trigger("draw:save", this.$("input").val());
                this.trigger("quit");
              },
              "click .js-cancel": function (e) {
                this.trigger("quit");
              },
              "keydown": function(event) {
                event = event.originalEvent || event;
                event.stopPropagation();
              }
            }
          }));
        };

        this.showUnsavedDialog = function () {
          var ContentView = Marionette.ItemView.extend({
            className: "warning close",
            events: {
              "tap button:first": function () {
                self.exitMenu();
              },

              "tap button:nth-of-type(2)": function () {
                App.trigger("draw:save");
                self.exitMenu();
              },

              "tap button": function () {
                this.trigger("quit");
              }
            },
            tagName: "main",
            template: JST.closeDialog
          });

          var dialogView = new DialogView({
            color: 6,
            height: "auto",
            title: Language("warning"),
            width: 400
          });

          dialogView.show(new ContentView());
        };

        this.exitMenu = function(){
          App.execute("module:stop", "viewer.document.draw");
          App.execute("module:stop", "Tools");
        };

        // Oculta botones especificos en casos especiales
        this.hideButtons = function() {

            App.request("is:custom", {context: this})
                .fail(function () {
                    App.request("is:notebook", {context: this}) // ---> Mi Cuaderno
                        .done(function () {
                            // Si es Mi Cuaderno, oculto subrayado y resaltado
                            var eliminar = toolsPrimaryCollection.where({type: "underlining"});
                            toolsPrimaryCollection.remove(eliminar);

                            // Oculto tambien la herramienta de seleccion de texto
                            var eliminar = toolsPrimaryCollection.where({type: "text-select"});
                            toolsPrimaryCollection.remove(eliminar);

                            // Quito el boton de "Guardar como..."
                            var eliminar = toolsPrimaryCollection.where({type: "save-as-text"});
                            toolsPrimaryCollection.remove(eliminar);
                        })
                        .fail(function () {
                            App.request("is:editable", {context: this}) // ---> Mi Libro
                                .done(function () {
                                    // Si es Mi Libro, quito el boton de "Guardar como..."
                                    var eliminar = toolsPrimaryCollection.where({type: "save-as-text"});
                                    toolsPrimaryCollection.remove(eliminar);
                                })
                                .fail(function () {
                                    // Si no es MiLibro ni MiCuaderno, oculto el boton de recurso de imagen
                                    var eliminar = toolsPrimaryCollection.where({type: "scloud"});
                                    toolsPrimaryCollection.remove(eliminar);
                                });
                        });
                });

          App.request("is:plataforma")
              .done(function (isPlat) {
                //botón de notas
                console.log("isPlat",isPlat);
                if(isPlat){
                  var eliminar = toolsPrimaryCollection.where({type: "text-select"});
                  toolsPrimaryCollection.remove(eliminar);

                  // Quito el boton de "Guardar como..."
                  var eliminar = toolsPrimaryCollection.where({type: "edit"});
                  toolsPrimaryCollection.remove(eliminar);
                }
          });
            /*
            App.request("is:device", {context: this})
                .done(function () {
                    // -------------------------------------------------------------------------------------------------------
                    //  This code removes functions in mobile devices that breaks mobile app because it uses much memory
                    //  @author Mario Poyato
                    //  Start patch [tools.js:limit.app.crashes]
                    // -------------------------------------------------------------------------------------------------------
                    // Hide primary controls
                    //toolsPrimaryCollection.remove(toolsPrimaryCollection.where({type: "cursor"}));
                    //toolsPrimaryCollection.remove(toolsPrimaryCollection.where({type: "shapes"}));
                    //toolsPrimaryCollection.remove(toolsPrimaryCollection.where({type: "type"}));
                    //toolsPrimaryCollection.remove(toolsPrimaryCollection.where({type: "scloud"}));
                    //toolsPrimaryCollection.remove(toolsPrimaryCollection.where({type: "clip-dot"}));
                    //toolsPrimaryCollection.remove(toolsPrimaryCollection.where({type: "masking"}));
                    // Hide secondary controls
                    //toolsControlsCollection.remove(toolsControlsCollection.where({type: "undo"}));
                    //toolsControlsCollection.remove(toolsControlsCollection.where({type: "redo"}));
                    //toolsControlsCollection.remove(toolsControlsCollection.where({type: "save"}));
                    //toolsControlsCollection.remove(toolsControlsCollection.where({type: "load"}));
                    ///toolsControlsCollection.remove(toolsControlsCollection.where({type: "cloud"}));
                    ///toolsControlsCollection.remove(toolsControlsCollection.where({type: "cloud-open-1"}));
                    //toolsControlsCollection.remove(toolsControlsCollection.where({type: "close"}));
                    // Hide secondary cursor controls
                    //toolsPrimaryCollection.remove(toolsPrimaryCollection.where({type: "copy"}));
                    //toolsPrimaryCollection.remove(toolsPrimaryCollection.where({type: "paste"}));
                    //toolsPrimaryCollection.remove(toolsPrimaryCollection.where({type: "text-select"}));
                    // -------------------------------------------------------------------------------------------------------
                    // End patch [tools.js:limit.app.crashes]
                    // -------------------------------------------------------------------------------------------------------
            });
            //*/

          // Si estoy editando desde SantillanaGo quito el boton de "Guardar como..."
          if(configuration.sgoReadOnlyObjectsURL){
            var eliminar = toolsPrimaryCollection.where({type: "save-as-text"});
            toolsPrimaryCollection.remove(eliminar);
          }
          
        };

        this.getHexColor = function () {
          var _c = null;
          switch (this.selectedColor) {
            case "colorcat-01":
              _c = "#22FF06";
              break;
            case "colorcat-02":
              _c = "#128E3A";
              break;
            case "colorcat-03":
              _c = "#13938F";
              break;
            case "colorcat-04":
              _c = "#FFFF09";
              break;
            case "colorcat-05":
              _c = "#FBC508";
              break;
            case "colorcat-06":
              _c = "#21FFFF";
              break;
            case "colorcat-07":
              _c = "#1396DD";
              break;
            case "colorcat-08":
              _c = "#6AC9EC";
              break;
            case "colorcat-09":
              _c = "#DC0016";
              break;
            case "colorcat-10":
              _c = "#E88407";
              break;
            case "colorcat-11":
              _c = "#FB00FF";
              break;
            case "colorcat-12":
              _c = "#DD0074";
              break;
            case "colorcat-13":
              _c = "#000";
              break;
            default:
              _c = "#FF0000";
              break;
          }
          return _c;
        };

        this.addFinalizer(function () {
          App.menu.layout.tools.$el.hide();

          // Vuelvo el menu al estado inicial
          toolsPrimaryCollection.reset();
          toolsPrimaryCollection.add(buttonGroups.main);
          toolsSecondaryCollection.reset();

          $("body > header > *").toggle();
          $("body > header > *:last").toggle();
        });

      });
    }
);

define('modules/mybook/template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["mybook_addpages"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, options, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;


  buffer += "<main class=\"debug\">\n    <span class=\"nPages\">\n	    <span class=\"subtitle\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "mybook.tipo", options) : helperMissing.call(depth0, "language", "mybook.tipo", options)))
    + "</span>\n	    <nav class=\"circles-nav\">\n	        <a class=\"blank active\" tipo=\"blank\" href=\"#\" title=\"\"></a>\n	        <a class=\"rowed\" tipo=\"rowed\" href=\"#\" title=\"\"></a>\n	        <a class=\"checkered\" tipo=\"checkered\" href=\"#\" title=\"\"></a>\n	    </nav>\n	    <p>";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "mybook.numPages", options) : helperMissing.call(depth0, "language", "mybook.numPages", options)))
    + "</p>\n	    <div class=\"centerContent\">\n		    <div class=\"number-selector\">\n			    <span class=\"color-10\" id=\"less\">-</span>\n			    <input type=\"text\" id=\"numPages\" value=\"1\" placeholder=\"\">\n			    <span class=\"color-10\" id=\"more\">+</span>\n		    </div>\n		</div>\n    </span>\n\n    <p class=\"centerContent\">\n        <button id=\"add\" class=\"bgcolor-10\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "accept", options) : helperMissing.call(depth0, "language", "accept", options)))
    + "</button>\n    </p>\n</main>";
  return buffer;
  });

this["JST"]["mybook_index"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<aside></aside>\n<section></section>";
  });

this["JST"]["mybook_panel"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<button id=\"button-mybook\" class=\"bgcolor-10\">\n    <i class=\"icon-arrow-left\"></i>\n</button>\n\n<section id=\"mybook\" class=\"panel-book pages\">\n    <nav class=\"navigation bgcolor-10\">\n        <a><i class=\"icon-page-new\"></i></a>\n        <a><i class=\"icon-view\"></i></a>\n        <a><i class=\"icon-save\"></i></a>\n        <a><i class=\"icon-edit-circle\"></i></a>\n        <a><i class=\"icon-arrow-right\"></i></a>\n    </nav>\n    \n    <section id=\"pages-my-book\"></section>\n</section>\n";
  });

return this["JST"];

});
define(
  'modules/mybook/views/index',[
    "marionette",
    "modules/index/views/index",
    "modules/mybook/template",
    "configuration"
  ],
  function(Marionette, IndexView, JST, configuration) {
    "use strict";

    Marionette.SortableItemView = Marionette.ItemView.extend({
      tagName: "li",
      className: "thumbnail",

      attributes: {
        "draggable": true
      },

      dragEvents: {
        "dragstart img": "start"
      },

      // template: JST.mybook_index,
      template: JST["thumbnail"],
      templateHelpers: {
        path: configuration.thumbnails
      },

      initialize: function(options) {
        _.extend(this, options);
      },

      // Adds the drag events
      delegateEvents: function(events) {
        var ev = _.extend({}, events, this.dragEvents);
        Marionette.View.prototype.delegateEvents.call(this, ev);
      },

      start: function(e) {      
        // Envio el modelo para que lo reciba el panel
        window.transfer = this.model;        

        e.stopPropagation();
        this.parent.draggedModel = this.model;
        if (e.originalEvent) e = e.originalEvent;

        try{
          e.dataTransfer.effectAllowed = "move";
          e.dataTransfer.dropEffect = "move";
          e.dataTransfer.setData('text', "Drag");
        } catch (err) {
          // console.log(err);
        }
      }

    });

    return Marionette.CollectionView.extend({

      tagName: 'ul',

      itemView: Marionette.SortableItemView,

      overClass: 'over',

      buildItemView: function(item, ItemViewType, itemViewOptions){
        var options = _.extend({ model: item,
                                 overClass: this.overClass,
                                 parent: this
                               },
                               itemViewOptions);

        return new ItemViewType(options);
      },

      appendHtml: function(collectionView, itemView, index){
        var childrenContainer = collectionView.itemViewContainer ? collectionView.$(collectionView.itemViewContainer) : collectionView.$el;
        var children = childrenContainer.children();
        if (children.size() <= index) {
          childrenContainer.append(itemView.el);
        } else {
          childrenContainer.children().eq(index).before(itemView.el);
        }
      }

    });

  }
);
define(
  'modules/mybook/views/panel',[
    "marionette",
    "modules/mybook/template",
    "configuration"
  ],
  function(Marionette, JST, configuration) {
    "use strict";

    Marionette.SortableItemView = Marionette.ItemView.extend({
        tagName: "li",

        ui: {
          erase: ".erase"
        },

        attributes: {
          "draggable": true
        },
        
        events: {
          "dragstart img": "start",
          "dragenter": "enter",
          "dragleave": "leave",
          "dragend": "leave",
          "dragover": "over",
          "drop": "drop",
          "touch @ui.erase" : "erase"
        },

        template: _.template('<img src="<%=ubicacion%><%=thumbnail%>" alt=""/><div class="erase"></div>'),
        templateHelpers: function(){
          var path;
          // La ubicacion de los thumbnails cambia
          // para las paginas en blanco
          if(this.model.get("blank")){
            path = configuration.root + 'assets/images/mybook/' + this.model.get("tipo") + '.png';
          } else {
            path = configuration.thumbnails;
          }

          return  { ubicacion: path }
        },

        className: 'box shadow padder',

        initialize: function(options) {
          _.extend(this, options);
        },

        // Drag & drop events
        start: function(e) { 
          e.stopPropagation();

          // Vacío el transfer porque quiero reordenar, no agregar.
          window.transfer = null;
          // Si estoy en modo edicion, me salgo
          this.parent.$(".erase").hide();
          this.parent.draggedModel = this.model;

          if (e.originalEvent) e = e.originalEvent;

          try{
            e.dataTransfer.effectAllowed = "move";
            e.dataTransfer.dropEffect = "move";
            e.dataTransfer.setData('text', "Drag");

          } catch (err) {
            // console.log(err);
          }
        },

        enter: function(e) {
          e.preventDefault();
          this.$el.addClass(this.overClass);
        },

        leave: function(e) {
          e.preventDefault();
          this.$el.removeClass(this.overClass);
        },

        over: function(e) {
          e.preventDefault();
          // this.$el.addClass(this.overClass);
          return false;
        },

        drop: function(e) {
          e.preventDefault();
          e.stopPropagation();
          this.leave(e);
          this.parent.$(".erase").hide();
          var collection = this.model.collection,
          parent = this.parent,
          currentIndex = this.$el.index();

          // Si el drop viene desde el indice
          if(window.transfer){
            collection.add(window.transfer.toJSON(), { at: currentIndex });
            window.transfer = null;
          } else {
            // El drop viene desde el propio panel, para reordenar
            collection.remove(parent.draggedModel);
            collection.add(parent.draggedModel, { at: currentIndex });
          }

          this.trigger('drop', this.parent.draggedModel);
        },

        erase: function() {
          this.parent.collection.remove(this.model);
        }

    });

    return Marionette.CollectionView.extend({

      tagName: 'ul',
      template: JST.mybook_panel,
      className: 'sortable-view',
      itemView: Marionette.SortableItemView,
      overClass: 'over',

      events: {
        "dragover": "over",
        "drop": "panelDrop"
      },

      buildItemView: function(item, ItemViewType, itemViewOptions){
        var options = _.extend({ model: item,
                                 overClass: this.overClass,
                                 parent: this
                               },
                               itemViewOptions);

        return new ItemViewType(options);
      },

      appendHtml: function(collectionView, itemView, index){
        var childrenContainer = collectionView.itemViewContainer ? collectionView.$(collectionView.itemViewContainer) : collectionView.$el;
        var children = childrenContainer.children();
        if (children.size() <= index) {
          childrenContainer.append(itemView.el);
        } else {
          childrenContainer.children().eq(index).before(itemView.el);
        }
      },

      over: function(e){
        e.preventDefault();
      },

      // Si se suelta el thumbnail en el panel, 
      // se agrega al final de la coleccion
      panelDrop: function(e){
        e.preventDefault();
        e.originalEvent.stopPropagation();
        e.originalEvent.preventDefault();

        if(window.transfer){
          this.$(".erase").hide();
          var collection = this.collection;
          collection.add(window.transfer.toJSON());
          window.transfer = null;
        }
      }

    });    

  });
define(
    'modules/mybook/regions/index',[
      "modules/index/regions/index",
      "modules/mybook/template"
    ],
    function (Region, JST) {
      "use strict";

      return Region.extend({
        className: "panel norender fullscreen mybook_index",
        template: JST.mybook_index
        
      });
    }
);
// noinspection JSHint
define(
    'modules/mybook/regions/addPages',[
      "app",
      "marionette", 
      "backbone",
      "models/page",
      "modules/mybook/template"
    ],
    function (App, Marionette, Backbone, PageModel, JST) {
      "use strict";

      return Marionette.Layout.extend({
        id: "addPages",
        ui: {
          blank: ".blank",
          rowed: ".rowed",
          checkered: ".checkered",
          more: "#more",
          less: "#less",
          number: "#numPages",
          add: "#add"
        },
        tagName: "section",
        template: JST.mybook_addpages,

        events: {
          "touch @ui.blank": "blank",
          "touch @ui.rowed": "rowed",
          "touch @ui.checkered": "checkered",
          "touch @ui.more": "more",
          "touch @ui.less": "less",
          "touch @ui.add": "add"
        },

        blank: function(){
          this.ui.rowed.removeClass("active");
          this.ui.checkered.removeClass("active");

          this.ui.blank.addClass("active");
        },

        rowed: function(){
          this.ui.blank.removeClass("active");
          this.ui.checkered.removeClass("active");

          this.ui.rowed.addClass("active");
        },

        checkered: function(){
          this.ui.rowed.removeClass("active");
          this.ui.blank.removeClass("active");

          this.ui.checkered.addClass("active");
        },

        more: function(){
          if(this.ui.number.val() < 99){
            this.ui.number.val(
              parseInt(this.ui.number.val()) + 1
            );
          }
        },

        less: function(){
          if(this.ui.number.val() > 1){
            this.ui.number.val(
              parseInt(this.ui.number.val()) - 1
            );
          }
        },

        add: function(){
          var cantidad = parseInt(this.ui.number.val());
          var tipo = this.$(".active").attr("tipo");
          
          // Obtengo la coleccion de Mi Cuaderno a lo macho
          var collection = App.mybookPanel.currentView.pages.currentView.collection;
          
          for(var i = 0; i < cantidad; i++){
            var page = new PageModel({
              "id": this.generateGUID(),
              "idUnidad": "U32",
              "name": "",
              "blank": true,
              "tipo": tipo, 
              "type": tipo
            });

            collection.add(page);

            // Tiene que haber una mejor manera de cerrar el dialogo
            // Adri?
            $("header .icon-close").click();
          }
        },

        generateGUID: function() {
          function s4() {
            return Math.floor((1 + Math.random()) * 0x10000)
              .toString(16)
              .substring(1);
          }
          
          return s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4();
        }

      });
    }
);
define(
    'modules/mybook/regions/panel',[ 
      "underscore",
      "marionette",
      "language",
      "views/dialog",
      "views/html",
      "modules/mybook/regions/addPages",
      "modules/mybook/template"
    ],
    function (_, Marionette, language, DialogView, HTMLView, AddPageLayout, JST) {
      "use strict";

      return Marionette.Layout.extend({
        regions: {
          pages: "#pages-my-book"
        },
        
        template: JST.mybook_panel,

        ui: {
          add: ".icon-page-new",
          preview: ".icon-view",
          save: ".icon-save",
          edit: ".icon-edit-circle",
          hide: ".icon-arrow-right",
          button: "#button-mybook",
          erase: ".erase"
        },

        events: {
          "touch @ui.add": "add",
          "touch @ui.preview": "preview",
          "touch @ui.save": "save",
          "touch @ui.edit": "erase",
          "touch @ui.hide": "hide",
          "touch @ui.button": "show", 
        },

        initialize: function() {
          App.on("mybook:preview", function(){
            if(this.previewState == 0){
              this.preview();
            }
          }, this);
        },

        add: function() {
          this.$(".erase").hide();

          var dialog = new DialogView({
            title: language("mybook.addPages"),
            color: 10,
            width: "400px",
            height: "auto"
          });

          var container = new AddPageLayout();
          dialog.show(container);
        },

        originalCollection: null,
        previewState: 0, // 0: libro original, 1: mi libro

        preview: function(){
          this.$(".erase").hide();

          var collection = this.pages.currentView.collection.toArray();

          // Si no hay paginas en el panel, no hago preview.
          if(!collection.length){
            App.execute("notification", language("mybook.noPages"), {color: 10, top: 60});
            // Cierro el panel de herramientas por si 
            // se llamó a esta funcion desde el menu.
            // El delay de 600 es porque la apertura de las herramientas
            // tiene un delay de 500 para alcanzar a abrir los modulos.
            _.delay(function(){
              App.execute("module:stop", "viewer.document.draw");
              App.execute("module:stop", "Tools");
            }, 600);

            return;
          }

          if(this.previewState == 0){
            this.previewState = 1;

            $("#navigation .icon-list").parent().hide();

            _.each(collection, function(value, index) {
                value.set("id", index + 1);
                value.set("title", index + 1); // Para ver numero correcto en precarga al pasar de pagina.
            });

            collection = new Backbone.Collection(collection);

            // Si mi libro no tiene paginas, no muestro preview
            if(collection.length == 0) return false;

            // Guardo la estructura
            this.save();
            
            this.originalCollection = App.viewer.document.controller.options.book;

            App.viewer.document.controller.options.book = collection;
            App.viewer.document.controller.attributes.page = -1;
            App.trigger("goto", 1, true);

            // Cambio a modo pagina simple
            App.viewer.trigger("mode", "single");

            // Oculto control de modo
            $("footer nav a").first().hide();

            // Mostrar herramientas
            // $(".icon-wrench.color-6").show();

            // Oculto selector de unidades
            $("header h1 a").hide();

            // Oculto preview
            $("#preview").addClass("norender").removeClass("fullscreen");
            $("i.icon-list").parent().removeClass("active");

            // Oculto panel
            this.hide();

          } else {
            this.previewState = 0;

            $("#navigation .icon-list").parent().show();

            // Recupero los ids originales
            _.each(collection, function(value, index) {
              value.set("id", value.get("idOriginal"));
            });

            App.viewer.document.controller.options.book = this.originalCollection;
            App.viewer.document.controller.attributes.page = -1;
            App.trigger("goto", 1, true);

            // Cambio a modo pagina doble
            App.viewer.trigger("mode", "double");

            // Muestro control de modo
            $("footer nav a").first().show();
          }
        },

        save: function(){
          this.$(".erase").hide();

          // Creo un libro falso
          var myBookJson = {"book":{ "chapters":[{"idUnidad":"*","title":"Mi libro", "active":true}] }};

          // Mapeo la coleccion actual a las paginas del libro. La primera vez, obtengo los
          // datos del libro, por lo que las propiedades se llaman "code" y "type". A partir
          // de la segunda, los nombres ya son iguales.
          myBookJson.book.pages = this.pages.currentView.collection.map(function(valor){
            var custom = valor.get("custom") ? valor.get("custom").slice(0) : "";
            custom[1] = valor.get("drawing");

            if(valor.get("blank")){
              // Es una página en blanco
               return {
                "id": valor.get("id"),
                "idOriginal": valor.get("idOriginal"),
                "idUnidad": valor.get("code"),
                "number": valor.get("number"),
                "index": valor.get("index"),
                "blank": true,
                "type": valor.get("tipo"),
                "tipo": valor.get("tipo"),
                "readOnlyObjects": custom
               }
            } else {
              // Es una pagina de libro
              return {
                "id": valor.get("id"),
                "idOriginal": valor.get("idOriginal"),
                "idUnidad": valor.get("code"),
                "number": valor.get("number"),
                "index": valor.get("index"),
                "blank": false,
                "type": valor.get("type"),
                "readOnlyObjects": custom
              }
            }
          });

          App.request("av", {
            type: 'setToolData', 
            action: 'set_data',
            dataJSON: JSON.stringify(myBookJson),
            key: ''
          });

          // Mostrar dialogo solo viendo el libro original
          if(this.previewState == 0){
            // El "set_data" no devuelve confimacion, por tanto no me sirve poner un 
            // listener. Damos por entendido que guardamos ok.
            App.execute("notification", language("mybook.saved"), {color: 10, top: 112});
          }
        },

        hide: function() {
          this.$(".erase").hide();
          // $("#mybook").addClass("norender");
          $("#mybook").css("width", 0);
        },

        show: function() {
          // Mostrar panel
          // $("#mybook").removeClass("norender");
          $("#mybook").css("width", "320px");

          if(this.previewState == 1){
            // Ocultar herramientas
            // $(".icon-wrench.color-6").hide();

            // Mostrar selector de unidades
            $("header h1 a").show();

            // Mostrar preview
            $("i.icon-list").parent().addClass("active");
            $("#preview").addClass("fullscreen").removeClass("norender");

            // Volver a libro original
            this.preview();
          }
        },

        erase: function(){
          this.$(".erase").toggle();
        }

      });
    }
);
define(
  'modules/mybook/mybook',[
    "app",
    "views/chapters.dropdown",
    "models/page",
    "models/page.collection",
    "modules/viewer/viewer",
    "modules/mybook/views/index",
    "modules/mybook/views/panel",
    "modules/mybook/regions/index",
    "modules/mybook/regions/panel",
    "language",
    "regions/replace"
  ],
  function(App, ChaptersDropdownViewDeferred, PageModel, PageCollection, viewer, IndexView, PanelView, IndexLayout, PanelLayout, language, ReplaceRegion) {
    "use strict";

    return App.module(
      "mybook",
      function(module, App, Backbone, Marionette, $, _) {
        this.startWithParent = false;

        this.layout = App.panel;

        // Constructor
        this.addInitializer(function(options) {
          options = _.extend({}, options);

          this.layout = (function() {
            var RegionManager = new Marionette.RegionManager();

            var indexLayout = new IndexLayout();
            var indexRegion = App.panel;
            App.request("is:notebook").fail(function() {
              indexRegion.show(indexLayout);
            });

            var panelLayout = new PanelLayout();
            var panelRegion = App.mybookPanel;
            panelRegion.show(panelLayout);

            RegionManager.addRegions({
              index: indexRegion,
              panel: panelRegion
            }, {
              regionType: ReplaceRegion
            });

            return RegionManager;
          })();

          $.when(
            App.request("chapter", {
              context: this
            }),
            ChaptersDropdownViewDeferred()
          )
            .done(function(chapter, View) {
              var filter;
              var view = new IndexView({
                collection: new Backbone.Collection()
              });

              View = View.extend({
                className: "my dropdown",
                tagName: "h2",

                initialize: function() {
                  this.model = this.model.clone();
                  this.model.set("title", chapter.get("title"));
                  this.model.set("color", 2);
                  this.model.set("options", this.model.get("options").slice(0));
                }
              });

              filter = new View({color: 2});

              filter.on("select", function($el) {
                this.getIndexSingle($el.attr("data-code"));
              }, module);

              // Indice
                if (module.layout.get("index").currentView) {
                  module.layout.get("index").currentView.filter.show(filter);
                  module.layout.get("index").currentView.thumbnails.show(view);
                  module.layout.get("index").$el.removeClass("norender");
                }

              // Panel

              module.drawPages();


              module.getIndexSingle(chapter.get("code"));
            })
            .fail(function() {
              App.execute("error", language("index.couldNotLoadChapters"));
            });
            $("i.icon-list").parent().addClass("active");
          App.trigger("menu:show");
        });

        // Destructor
        this.addFinalizer(function() {
          this.layout.get("index").$el.addClass("norender").removeClass("fullscreen");
        });


        // Dibuja paginas simples
        this.getIndexSingle = function(code) {
          $.when(
            App.request("chapters", {context: this}),
            App.request("is:document:portrait", {boolean: true, context: this}),
            App.request("page", {context: this})
          )
            .done(function(chapters, isDocumentPortrait, page) {
              var pages = (code === "*") ? chapters.pages() :
                chapters.findWhere({code: code}).get("pages"),
                thumbnails = [];

              pages.each(function(value) {
                value.set("idOriginal", value.get("id"));
                thumbnails.push(value.toJSON());
              });

              this[0].layout.get("index").currentView.thumbnails.currentView.collection.set(thumbnails);
            });
        };


        // Dibuja paginas dobles
        this.getIndexDouble = function(code) {
          $.when(
            App.request("chapters", {context: this}),
            App.request("is:document:portrait", {boolean: true, context: this}),
            App.request("page", {context: this})
          )
            .done(function(chapters, isDocumentPortrait, page) {
              var pages = (code === "*") ? chapters.pages() :
                chapters.findWhere({code: code}).get("pages"),
                thumbnails = [];

              pages = pages.groupBy(function(value) {
                return parseInt(
                  value.get("id") / (isDocumentPortrait + 1),
                  10
                );
              });

              _.each(pages, function(value) {
                var collection = new Backbone.Collection(value);

                thumbnails.push({
                  id: collection.first().get("id"),
                  title: collection.pluck("title").join("-"),

                  pages: value.map(function(rvalue) {
                    rvalue = rvalue
                      .clone()
                      .set("data", {page: rvalue.get("id")});

                    if (rvalue.get("id") === page) {
                      rvalue.set("current", true);
                    }

                    return rvalue.toJSON();
                  })
                });
              });

              this[0].layout.get("index").currentView.thumbnails.currentView.collection.set(thumbnails);
            });
        };



        this.drawPages = function() {
          $.when(
            App.request("chapters", {context: this})
          )
            .done(function(chapters) {
              // Defino un listener para cuando reciba los datos de AV
              App.request("av", {type: 'getToolData', action: 'get_data'})
                .done(function(data){
                  var temp_fix = "";
                  try {
                      temp_fix = $.parseJSON($.parseJSON(data.dataJSON));
                  } catch (e) {
                      temp_fix = (data.dataJSON && $.parseJSON(data.dataJSON)) || "";
                  }

                  if(temp_fix){
                    _.each(temp_fix.book.pages, function(item, index, list) {
                      list[index] = new PageModel({
                        id: item.id,
                        idOriginal: item.idOriginal,
                        number: item.number,
                        index: item.index,
                        code: item.idUnidad,
                        thumbnail: (!item.blank) ? item.number + ".jpg" : null,
                        type: item.type,
                        tipo: item.tipo,
                        blank: item.blank
                      });

                      list[index].set("drawing", item.readOnlyObjects[1]);
                      list[index].set("custom", [item.readOnlyObjects[0]]);
                    });

                    temp_fix = temp_fix.book.pages;
                  }

                  var collection = new (Backbone.Collection.extend({
                    model: PageModel
                  }))(temp_fix);

                  var panelPagesView = new PanelView({
                    className:'sortable-view list-unstyled',
                    collection: collection
                  });

                  module.layout.get("panel").currentView.pages.show(panelPagesView);

                  // Agregamos item "fantasma"
                  $('ul.sortable-view').append('<li class="over" draggable="false"></li>');
                });
            });
        };
        
      });
  }
);
define('modules/notebook/template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["mybook_addpages"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, options, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;


  buffer += "<main class=\"debug\">\n    <span class=\"nPages\">\n	    <span class=\"subtitle\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "mybook.tipo", options) : helperMissing.call(depth0, "language", "mybook.tipo", options)))
    + "</span>\n	    <nav class=\"circles-nav\">\n	        <a class=\"blank active\" tipo=\"blank\" href=\"#\" title=\"\"></a>\n	        <a class=\"rowed\" tipo=\"rowed\" href=\"#\" title=\"\"></a>\n	        <a class=\"checkered\" tipo=\"checkered\" href=\"#\" title=\"\"></a>\n	    </nav>\n	    <p>";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "mybook.numPages", options) : helperMissing.call(depth0, "language", "mybook.numPages", options)))
    + "</p>\n	    <div class=\"centerContent\">\n		    <div class=\"number-selector\">\n			    <span class=\"color-10\" id=\"less\">-</span>\n			    <input type=\"text\" id=\"numPages\" value=\"1\" placeholder=\"\">\n			    <span class=\"color-10\" id=\"more\">+</span>\n		    </div>\n		</div>\n    </span>\n\n    <p class=\"centerContent\">\n        <button id=\"add\" class=\"bgcolor-10\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "accept", options) : helperMissing.call(depth0, "language", "accept", options)))
    + "</button>\n    </p>\n</main>";
  return buffer;
  });

this["JST"]["mybook_index"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<aside></aside>\n<section></section>";
  });

this["JST"]["mybook_panel"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<button id=\"button-mybook\" class=\"bgcolor-10\">\n    <i class=\"icon-arrow-left\"></i>\n</button>\n\n<section id=\"mybook\" class=\"panel-book pages\">\n    <nav class=\"navigation bgcolor-10\">\n        <a><i class=\"icon-page-new\"></i></a>\n        <a><i class=\"icon-save\"></i></a>\n        <a><i class=\"icon-edit-circle\"></i></a>\n        <a><i class=\"icon-arrow-right\"></i></a>\n    </nav>\n    \n    <section id=\"pages-my-book\"></section>\n</section>";
  });

return this["JST"];

});
define(
  'modules/notebook/views/panel',[
    "marionette",
    "modules/notebook/template",
    "configuration"
  ],
  function(Marionette, JST, configuration) {
    "use strict";

    Marionette.SortableItemView = Marionette.ItemView.extend({
        tagName: "li",

        ui: {
          erase: ".erase"
        },

        attributes: {
          "draggable": true
        },
        
        events: {
          "tap img": "tap",
          "dragstart img": "start",
          "dragenter": "enter",
          "dragleave": "leave",
          "dragend": "leave",
          "dragover": "over",
          "drop": "drop",
          "touch @ui.erase" : "erase"
        },

        template: _.template('<img src="<%=ubicacion%><%=thumbnail%>" alt=""/><div class="erase"></div>'),
        templateHelpers: function(){
          var path;
          // La ubicacion de los thumbnails cambia
          // para las paginas en blanco
          if(this.model.get("blank")){
            path = configuration.root + 'assets/images/mybook/' + this.model.get("tipo") + '.png';
          } else {
            path = configuration.thumbnails;
          }

          return  { ubicacion: path }
        },

        className: 'box shadow padder',

        initialize: function(options) {
          _.extend(this, options);
        },

        tap: function(){
          var self = this;

          App.request("page").done(function(page){
            // Si hago click en una pagina distinta a la actual
            if(self.model.get("number") != page){
              // Voy a la pagina clickeada
              App.viewer.document.controller.goto(self.model.get("number"), true);
            }
          });
        },

        // Drag & drop events
        start: function(e) {
          e.stopPropagation();

          // Vacío el transfer porque quiero reordenar, no agregar.
          window.transfer = null;
          // Si estoy en modo edicion, me salgo
          this.parent.$(".erase").hide();
          this.parent.draggedModel = this.model;

          if (e.originalEvent) e = e.originalEvent;

          try{
            e.dataTransfer.effectAllowed = "move";
            e.dataTransfer.dropEffect = "move";
            e.dataTransfer.setData('text', "Drag");

          } catch (err) {
            // console.log(err);
          }
        },

        enter: function(e) {
          e.preventDefault();
          this.$el.addClass(this.overClass);
        },

        leave: function(e) {
          e.preventDefault();
          this.$el.removeClass(this.overClass);
        },

        over: function(e) {
          e.preventDefault();
          // this.$el.addClass(this.overClass);
          return false;
        },

        drop: function(e) {
          e.preventDefault();
          e.stopPropagation();
          this.leave(e);
          this.parent.$(".erase").hide();
          var collection = App.notebook.notebookPageCollection,
          parent = this.parent,
          currentIndex = this.$el.index();

          // Si el drop viene desde el indice
          if(window.transfer){
            collection.add(window.transfer.toJSON(), { at: currentIndex });
            window.transfer = null;
          } else {
            // El drop viene desde el propio panel, para reordenar
            collection.remove(parent.draggedModel);
            collection.add(parent.draggedModel, { at: currentIndex });

            App.trigger("notebook:refresh");
          }

          this.trigger('drop', this.parent.draggedModel);
        },

        erase: function() {
          this.parent.collection.remove(this.model);
          App.trigger("notebook:refresh");
        }

    });

    return Marionette.CollectionView.extend({

      tagName: 'ul',
      template: JST.mybook_panel,
      className: 'sortable-view',
      itemView: Marionette.SortableItemView,
      overClass: 'over',

      events: {
        "dragover": "over",
        "drop": "panelDrop"
      },

      buildItemView: function(item, ItemViewType, itemViewOptions){
        var options = _.extend({ model: item,
                                 overClass: this.overClass,
                                 parent: this
                               },
                               itemViewOptions);

        return new ItemViewType(options);
      },

      appendHtml: function(collectionView, itemView, index){
        var childrenContainer = collectionView.itemViewContainer ? collectionView.$(collectionView.itemViewContainer) : collectionView.$el;
        var children = childrenContainer.children();
        if (children.size() <= index) {
          childrenContainer.append(itemView.el);
        } else {
          childrenContainer.children().eq(index).before(itemView.el);
        }
      },

      over: function(e){
        e.preventDefault();
      },

      // Si se suelta el thumbnail en el panel, 
      // se agrega al final de la coleccion
      panelDrop: function(e){
        e.preventDefault();
        e.originalEvent.stopPropagation();
        e.originalEvent.preventDefault();

        if(window.transfer){
          this.$(".erase").hide();
          var collection = this.collection;
          collection.add(window.transfer.toJSON());
          window.transfer = null;
        }
      }

    });    

  });
// noinspection JSHint
define(
    'modules/notebook/regions/addPages',[
      "app",
      "marionette", 
      "backbone",
      "models/page",
      "modules/notebook/template"
    ],
    function (App, Marionette, Backbone, PageModel, JST) {
      "use strict";

      return Marionette.Layout.extend({
        id: "addPages",
        ui: {
          blank: ".blank",
          rowed: ".rowed",
          checkered: ".checkered",
          more: "#more",
          less: "#less",
          number: "#numPages",
          add: "#add"
        },
        tagName: "section",
        template: JST.mybook_addpages,

        events: {
          "touch @ui.blank": "blank",
          "touch @ui.rowed": "rowed",
          "touch @ui.checkered": "checkered",
          "touch @ui.more": "more",
          "touch @ui.less": "less",
          "touch @ui.add": "add"
        },

        blank: function(){
          this.ui.rowed.removeClass("active");
          this.ui.checkered.removeClass("active");

          this.ui.blank.addClass("active");
        },

        rowed: function(){
          this.ui.blank.removeClass("active");
          this.ui.checkered.removeClass("active");

          this.ui.rowed.addClass("active");
        },

        checkered: function(){
          this.ui.rowed.removeClass("active");
          this.ui.blank.removeClass("active");

          this.ui.checkered.addClass("active");
        },

        more: function(){
          if(this.ui.number.val() < 99){
            this.ui.number.val(
              parseInt(this.ui.number.val()) + 1
            );
          }
        },

        less: function(){
          if(this.ui.number.val() > 1){
            this.ui.number.val(
              parseInt(this.ui.number.val()) - 1
            );
          }
        },

        add: function(){
          var cantidad = parseInt(this.ui.number.val());
          var tipo = this.$(".active").attr("tipo");
          
          for(var i = 0; i < cantidad; i++){
            var page = new PageModel({
              "id": this.generateGUID(),
              "idUnidad": "*",
              "name": "",
              "blank": true,
              "tipo": tipo, 
              "type": tipo
            });

            // Accedo directamente a la coleccion principal (malo, malo)
            App.notebook.notebookPageCollection.add(page);

            // Actualizo la coleccion principal
            App.trigger("notebook:refresh");
            
            // Tiene que haber una mejor manera de cerrar el dialogo
            // Adri?
            $("header .icon-close").click();
          }
        },

        generateGUID: function() {
          function s4() {
            return Math.floor((1 + Math.random()) * 0x10000)
              .toString(16)
              .substring(1);
          }
          
          return s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4();
        }

      });
    }
);
define(
    'modules/notebook/regions/panel',[ 
      "marionette",
      "language",
      "modules/notebook/regions/addPages",
      "modules/notebook/template",
      "views/dialog"
    ],
    function (Marionette, language, AddPageLayout, JST, DialogView) {
      "use strict";

      return Marionette.Layout.extend({
        regions: {
          pages: "#pages-my-book"
        },
        
        template: JST.mybook_panel,

        // Esto no funciona en algunas tablets
        // ui: {
        //   add: ".icon-page-new",
        //   save: ".icon-save",
        //   edit: ".icon-edit-circle",
        //   hide: ".icon-arrow-right",
        //   button: "#button-mybook",
        //   erase: ".erase"
        // },

        ui: {
          add: 'nav.navigation a:nth(0)',
          save: 'nav.navigation a:nth(1)',
          edit: 'nav.navigation a:nth(2)',
          hide: 'nav.navigation a:nth(3)',
          button: "#button-mybook",
          erase: ".erase"
        },

        events: {
          "touch @ui.add": "add",
          "touch @ui.save": "save",
          "touch @ui.edit": "erase",
          "touch @ui.hide": "hide",
          "touch @ui.button": "show", 
        },

        initialize: function() {
          App.on("notebook:panel:hide", function(){
            // Oculto el boton para que no se pueda abrir el panel
            // mientras tengo las heramientas abiertas
            $("#button-mybook").css("visibility", "hidden");
            this.hide();
          }, this);
        },

        add: function() {
          this.$(".erase").hide();

          var dialog = new DialogView({
            title: language("mybook.addPages"),
            color: 10,
            width: "400px",
            height: "auto"
          });

          var container = new AddPageLayout();
          dialog.show(container);
        },

        save: function(event){
          this.$(".erase").hide();

          // Paso el evento para saber si se llama desde
          // el panel o desde las herramientas.
          App.notebook.save(event);
        },

        hide: function() {
          this.$(".erase").hide();
          // $("#mybook").addClass("norender");
          $("#mybook").css("width", 0);
        },

        show: function() {
          // $("#mybook").removeClass("norender");
          $("#mybook").css("width", "320px");
        },

        erase: function(){
          if($(".erase").is(":visible")){
            $(".erase").hide();
            
            // Actualizo la coleccion principal
            App.notebook.refreshBook();
          } else {
            $(".erase").show();
            
          }
        }

      });
    }
);
define(
  'modules/notebook/notebook',[
    "app",
    "models/page",
    "models/page.collection",
    "modules/viewer/viewer",
    "modules/notebook/views/panel",
    "modules/notebook/regions/panel",
    "language",
    "regions/replace",
    "views/dialog",
    "views/html"
  ],
  function(App, PageModel, PageCollection, viewer, PanelView, PanelLayout, language, ReplaceRegion, DialogView, HTMLView) {
    "use strict";

    return App.module(
      "notebook",
      function(module, App, Backbone, Marionette, $, _) {
        var self = this;
        this.startWithParent = false;

        this.layout = App.panel;
        this.notebookPageCollection = new Backbone.Collection();

        // Constructor
        this.addInitializer(function(options) {
          options = _.extend({}, options);

          App.on("notebook:refresh", function(){
            this.refreshBook();
          }, this);

          this.layout = (function() {
            var RegionManager = new Marionette.RegionManager();
            var panelLayout = new PanelLayout();
            var panelRegion = App.mybookPanel;
            panelRegion.show(panelLayout);

            RegionManager.addRegions({
              panel: panelRegion
            }, {
              regionType: ReplaceRegion
            });

            return RegionManager;
          })();

          module.drawPages();
        });

        this.refreshBook = function(){
          if(this.notebookPageCollection){
            // Creo una colección temporal para ordenar las
            // páginas sin repetir los ID's
            var tmpCollection = new Backbone.Collection();

            // Recorro la coleccion del panel
            this.notebookPageCollection.each(function(page, index){
              var tmpPage = page.clone();

              tmpPage.set("id", index + 1);
              tmpPage.set("index", index + 1);
              tmpPage.set("number", index + 1);
              tmpPage.set("title", index + 1);

              tmpCollection.add(tmpPage);
            });

            // Asigno la coleccion temporal a la principal
            this.notebookPageCollection.reset();
            this.notebookPageCollection.add(tmpCollection.toArray());
            
            // Actualizo el master para reflejar cambios al agregar / quitar paginas
            window.master.book.pages = this.formatPages();

            App.request("pages", {cache: false});

            // Mantengo la vista en la página actual
            App.request("page").done(function(page){
              App.viewer.document.controller.goto(page, true);
            });
          }
        };

        // Adapta las paginas para que sean compatibles con libros
        this.formatPages = function(){
          var formatedPages = this.notebookPageCollection.map(function(valor){
            var custom = valor.get("custom") ? valor.get("custom").slice(0) : "";
            custom[1] = valor.get("drawing");

            return {
              "idUnidad": "*",
              "number": valor.get("number"),
              "index": valor.get("index"),
              "blank": true,
              "type": valor.get("tipo"),
              "tipo": valor.get("tipo"),
              "readOnlyObjects": custom
            }
          });

          return formatedPages;
        };

        this.save = function(event){
          // Creo un libro falso
          var myBookJson = {"book":{ "chapters":[{"idUnidad":"*","title":"Mi cuaderno", "active":true}] }};
          myBookJson.book.pages = this.formatPages();

          App.request("av", {
            type: 'setToolData', 
            action: 'set_data',
            dataJSON: JSON.stringify(myBookJson),
            key: ''
          });

          // Si guardo desde el panel muestro confirmacion
          if(event && event.type == "touch"){
            // El "set_data" no devuelve confimacion, por tanto no me sirve poner un 
            // listener. Damos por entendido que guardamos ok.
            var options = {
              class: "debug",
              color: 10,
              message: language("savedChanges"),
              height: "auto",
              title: language("info"),
              width: 600
            };

            var dialog = new DialogView(options);
            dialog.show(
                new HTMLView({className: options.class, message: options.message})
            );
          }
        };

        this.drawPages = function() {
          // Pido datos a AV
          App.request("av", {type: 'getToolData', action: 'get_data'})
            .done(function(data){
              // Parseo la información obtenida.
              // Aglunas veces viene doblemente codificada, por lo que intento parsear dos veces
              var decoded = "";
              try {
                  decoded = $.parseJSON($.parseJSON(data.dataJSON));
              } catch (e) {
                  decoded = (data.dataJSON && $.parseJSON(data.dataJSON)) || "";
              }

              // Armo la coleccion de paginas con los datos obtenidos
              if(decoded){
                _.each(decoded.book.pages, function(item, pos, list) {
                  list[pos] = new PageModel({
                    id: pos + 1,
                    number: item.number,
                    index: item.index,
                    code: item.idUnidad,
                    thumbnail: (!item.blank) ? item.number + ".jpg" : null,
                    type: item.type,
                    tipo: item.tipo || "blank",
                    blank: item.blank,
                    drawing: item.readOnlyObjects ? item.readOnlyObjects[1] : null
                  });
                });

                decoded = decoded.book.pages;
              }

              // Agrego las paginas a la coleccion principal
              self.notebookPageCollection.add(decoded);

              // Creo la vista del panel y le asigno la coleccion principal
              var panelPagesView = new PanelView({
                className:'sortable-view list-unstyled',
                collection: self.notebookPageCollection
              });

              module.layout.get("panel").currentView.pages.show(panelPagesView);
              module.layout.get("panel").$el.show();

              // Asigno la coleccion principal al viewer
              App.viewer.document.controller.options.book = self.notebookPageCollection;

              // Actualizo la coleccion
              self.refreshBook();
            })
            .fail(function(err){
              console.log(err);
            });

        };
        
    });
  }
);
// noinspection JSHint
/**
 * Clase de la que extienden todas las figuras que se dibujan con las herramientas
 * @class Shape
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape',["underscore"],
    function (_) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Shape = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "Shape",
          x: 0,
          y: 0,
          width: 0,
          height: 0,
          order: 1 // Figura general
        });
        if ("undefined" !== typeof(options))
        {
          /**
           * Escala del canvas.
           *
           * @property scale
           * @type {Number}
           * @default 1
           */
          this.scale = scale || 1;
          /**
           * Identificador de la figura.
           *
           * @property id
           * @type {Number}
           * @default 1
           */
          this.id = options.id;
          /**
           * Clase de la figura.
           *
           * @property className
           * @type {String}
           */
          this.className = options.className;
          /**
           * Canvas en el que se pinta la figura.
           *
           * @property canvas
           * @type {Object}
           */
          this.canvas = canvas || $("aside canvas.mainTools").get(0);
          /**
           * Posición de en las abcisas de la esquina superior izquierda de la figura.
           *
           * @property x
           * @type {Number}
           */
          this.x = options.x * this.scale;
          /**
           * Posición de en las ordenadas la esquina superior izquierda de la figura.
           *
           * @property y
           * @type {Number}
           */
          this.y = options.y * this.scale;
          /**
           * Ancho en pixeles de la figura.
           *
           * @property width
           * @type {Integer}
           */
          this.width = options.width * this.scale;
          /**
           * Alto en pixeles de la figura.
           *
           * @property height
           * @type {Integer}
           */
          this.height = options.height * this.scale;
          /**
           * Categoria de la figura.
           * · Grupo: 0
           * · Figuras en general: 1
           * · Máscaras: 2
           * · Cuadros de texto: 3
           *
           * @property order
           * @type {Integer}
           */
          this.order = options.order;
        }
        this.anchors = [];
      };

      Shape.prototype = {
        /**
         * Anclas de la figura para cuando se selecciona la figura.
         *
         * @property anchors
         * @type {Array}
         * @default []
         */
        anchors: [],
        /**
         * Variable que almacena el estado de la figura durante los lanzamientos de los eventos (eventTriggered)
         * Siendo 0 si no hay cambios
         *        1 si hay cambios pero no se ha lanzado el evento de almacenar en el historial los cambios (caso de que se sigue modificando la figura)
         *        2 si hay cambios y se ha finalizado de modificar por lo que se almacenará el estado actual en el historial de las herramientas para esa página y el modo de edición actual
         * @property changed
         * @type {Boolean}
         * @default false
         */
        changed: false,
        /**
         * Coordenada del ancla para utilizarla de referencia para poder realizar el escalado de la figura
         * @property referenceAnchorPoint
         * @type {Object}
         * @default null
         */
        referenceAnchorPoint: null,

        /**
         * Posición del primer click para la referencia del desplazamiento de la figura
         * @property firstPosition
         * @type {Object}
         * @default null
         */
        firstPosition: null,

        /**
         * Función que añade un ancla a la figura
         * @param {Object} options
         * @method anchor
         */
        anchor: function (options) {
          _.defaults(options || (options = {}), {
            draggable: true,
            dragOnTop: false,
            fill: "#FFFFFF",
            listening: true,
            name: "anchor",
            size: 25,
            stroke: "#3399FF",
            x: 0,
            y: 0,
            scale: 1
          });

          var square = require("modules/viewer/models/shape/square");

          var anchor = new square(_.extend(options, {
            strokeWidth: Math.max(options.size / 5, 1),
            height: options.size,
            width: options.size,
            x: options.x / options.scale - options.size / 2,
            y: options.y / options.scale - options.size / 2,
            opacity: 1
          }), options.scale);

          anchor.pos = options.pos;

          square = null;

          App.request("is:device")
              .done(function () {
                anchor.x -= anchor.width / 2;
                anchor.y -= anchor.height / 2;
                anchor.height = anchor.height * 2;
                anchor.width = anchor.width * 2;
              });

          this.anchors.push(anchor);
        },

        /**
         * Función que realiza la deselección de una figura
         * @method deselect
         */
        deselect: function () {
          while (this.anchors.pop()) {
            //anchor.destroy();
          }
        },

        /**
         * Función que realiza el proceso relacionado con los lanzamientos de eventos sobre la figura de toques, clicks, desplazamientos y levantamientos de click y toques.
         * @param string type tipo del evento
         * @param object center posición del puntero o el toque en el evento
         * @param bool recolocar es un parámetro opcional o booleano que indica si los anclas se recolocan de forma que cuando pasa el inferior a estar arriba se cambian los nombres o no de esta forma si existe una recolocación la figura siempre aparece depie y si no existe la figura puede darse la vuelta como ocurre con el triangulo
         * @returns integer con valor 0 si no hay cambios, 1 si hay cambios, 2 si hay cambios y además hay que hacer una captura para el historial de las herramientas
         * @method eventTriggered
         */
        eventTriggered: function(type, center, recolocar) {
          var changed = 0;
          switch (type) {
            case "mouseover": {
              document.body.style.cursor = "pointer";
              break;
            }
            case "mouseout": {
              document.body.style.cursor = "default";
              break;
            }
          /**
           * Cuando se comienza a presionar la pantalla o se ejecuta un click
           */
            case "touchstart":
            case "mousedown": {
              this.diffX = 0;
              this.diffY = 0;
              this.hold = false;
              // Por ser el comienzo de un toque o click no ha podido ser posible que haya habido cambios por lo que se pone a null por seguridad
              this.changed = false;
              //Se comprueba que se pase las coordenadas del evento lanzado
              if (center) {
                // Obtener el ancla que se ha pinchado (si no se ha pinchado un ancla se obtendrá null)
                var anchor = _.find(this.anchors,function(anchor){return center.x<=anchor.x+anchor.width&&center.x>=anchor.x&&center.y<=anchor.y+anchor.height&& center.y>=anchor.y;});
                // En el caso de haberse seleccionado un ancla se guardará el ancla seleccionado
                if (anchor && anchor.draggable) {
                  this.referenceAnchorPoint = anchor.pos;   // Guardar el ancla seleccionado
                }
                // En caso de no haberse pinchado un ancla se comprueba si se ha pinchado una figura y en tal caso se almacena la posición del click o toque
                else {
                  this.referenceAnchorPoint = null;         // No se va a realizar un escalado (sino que será desplazamiento) por lo que se pone a null por seguridad
                }
                this.firstPosition = center;              // Guardar la posición del toque o del click
              }
              break;
            }
            /*
             * Se ejecutan las acciones pertinentes cuando se detecta una acción de desplazamiento (tanto de ratón como de toque mantenido)
             */
            case "touchmove":
            case "mousemove": {
              //Se comprueba que se pase las coordenadas del evento lanzado
              if (center && !this.hold) {
                // En el caso de desplazamiento (se ha tomado la posición del puntero) y además (no se ha presionado un ancla con el que escalar)
                if (this.firstPosition && !this.referenceAnchorPoint) {
                  this.diffX = center.x - this.firstPosition.x;
                  this.diffY = center.y - this.firstPosition.y;
                  this.x += this.diffX;                         // Obtener la nueva posición de abcisas (teniendo como referencia el punto almacenado en el primer toque)
                  this.y += this.diffY;                         // Obtener la nueva posición de ordenadas (teniendo como referencia el punto almacenado en el primer toque)
                  this.firstPosition = center;                  // Almacenar la nueva posición y tenerla como referencia para el (posible) siguiente desplazamiento
                  //this.anchors = [];                          // Borrar las anclas de la figura
                  if (this.bound) {
                    this.bound.x = this.x;
                    this.bound.y = this.y;
                  }
                  if (this.bound2) {
                    this.bound2.x = this.x;
                    this.bound2.y = this.y;
                  }
                  this.select();                              // Volver a calcular la posiciones de los anclas de la figura
                  this.changed = true;                        // Indicar en la variable de clase que se ha cambiado el estado de la figura de forma no definitiva (pueden haber más modificaciones: 1)
                  changed = 1;
                }
                // Si lo que se quiere hacer es escalar la figura
                else if (this.referenceAnchorPoint) {
                  var oldx = this.x,
                      oldy = this.y,
                      oldw = this.width,
                      oldh = this.height;
                  // En función del ancla que fue seleccionado
                  switch (this.referenceAnchorPoint) {
                    // Ancla superior izquierda (left-top)
                    case "lt": {
                      this.diffX = center.x - this.x;
                      this.diffY = center.y - this.y;
                      // Obtener la nueva posición de abcisas (sería como un "desplazamiento" de la esquina)
                      this.width += this.x - center.x;
                      this.x = center.x;
                      // Obtener la nueva posición de ordenadas (sería como un "desplazamiento" de la esquina)
                      this.height += this.y - center.y;
                      this.y = center.y;
                      break;
                    }
                    // Ancla superior derecha (right-top)
                    case "rt": { // Ancla superior izquierda (left-top)
                      this.diffX = 0;
                      this.diffY = center.y - this.y;
                      // Obtener la nueva posición de abcisas (sería como un "desplazamiento" de la esquina)
                      this.width += center.x - this.firstPosition.x;
                      // Obtener la nueva posición de ordenadas (sería como un "desplazamiento" de la esquina)
                      this.y     += center.y - this.firstPosition.y;
                      this.height -= center.y - this.firstPosition.y;
                      this.y = center.y;
                      break;
                    }
                    // Ancla inferior derecha (right-bottom)
                    case "rb": {
                      this.diffX = 0;
                      this.diffY = 0;
                      // Obtener la nueva posición de abcisas (sería como un "desplazamiento" de la esquina)
                      this.width += center.x - this.firstPosition.x;
                      // Obtener la nueva posición de ordenadas (sería como un "desplazamiento" de la esquina)
                      this.height += center.y - this.firstPosition.y;
                      break;
                    }
                    // Ancla inferior izquierda (left-bottom)
                    case "lb":
                    default: {
                      this.diffX = center.x - this.firstPosition.x;
                      this.diffY = 0;
                      // Obtener la nueva posición de abcisas (sería como un "desplazamiento" de la esquina)
                      this.x     += center.x - this.firstPosition.x;
                      this.width -= center.x - this.firstPosition.x;
                      // Obtener la nueva posición de ordenadas (sería como un "desplazamiento" de la esquina)
                      this.height = center.y - this.y;
                      break;
                    }
                  }
                  // Almacenar la penultima posición
                  this.firstPosition = center;
                  // En caso de que se produzcan coordenadas negativas para el ancho ponerlas positivas y ajustar el rectángulo correctamente
                  // \-> En caso de que cambien los anclas de posición siendo los de izquierda ahora los de derecha ajustar los nuevos valores
                  if (recolocar && this.width < 0) {
                    this.width = -this.width;
                    this.diffX -= this.width;
                    this.x -= this.width;
                    this.referenceAnchorPoint = (this.referenceAnchorPoint[0]=="l"?"r":"l") + this.referenceAnchorPoint[1];
                  }
                  // En caso de que se produzcan coordenadas negativas para el alto ponerlas positivas y ajustar el rectángulo correctamente
                  // \-> En caso de que cambien los anclas de posición siendo los de abajo ahora los de arriba ajustar los nuevos valores
                  if (recolocar && this.height < 0) {
                    this.height = -this.height;
                    this.diffY -= this.height;
                    this.y -= this.height;
                    this.referenceAnchorPoint = this.referenceAnchorPoint[0] + (this.referenceAnchorPoint[1]=="t"?"b":"t");
                  }
                  //this.anchors = [];
                  this.resize(this.width, this.height);
                  this.select();
                  this.changed = true;
                  changed = 1;
                }
              }
              break;
            }
            /*
             * Se ejecutan las acciones pertinentes cuando se detecta una acción de levantamiento de toque o click
             */
            case "mouseup":
            case "touchend": {
              // Si hay un cuadro translucido de como queda el texto
              if (this.changed && (this.referenceAnchorPoint || this.firstPosition)) {
                changed = 2;
                this.changed = false;
              }
              this.firstPosition = null;
              this.referenceAnchorPoint = null;
            }
            default: {
              // do nothing
              break;
            }
          }
          return changed;
        },

        /**
         * Función que indica si hay que limpiar el canvas para la figura.
         * @method unDraw
         * @returns {boolean}
         */
        unDraw: function() { return true; /* Hay que limpiar el canvas */ },

        /**
         * Función que abre una ventana o popup en el caso de algunos elementos. Además se bloquea la figura.
         * @method openWindow
         */
        openWindow: function () {
          // Bloquear el elemento sobre el que se hace doble click
          this.hold = true;
        },

        staticDraw: false,
        /**
         * Indica si la figura esta dentro del canvas (por defecto esta a falso para obligar la implementación de la función)
         * @method isInsideCanvas
         * @returns {boolean}
         */
        isInsideCanvas: function(x, w, canvasWidth) {
          x = x || this.x;
          w = w || this.width;
          var xMax = Math.max(x,x+w),
            xMin = Math.min(x,x+w),
            canvasWidth;
          if (this.canvas || canvasWidth) {
            canvasWidth = canvasWidth || this.canvas.width;
            /**
             * FIX: Problema en escalas debido a que la escala aplicada cuando las herramientas no estan abiertas es distinta a cuando estan cerradas.
             * El número mágico 1662 no se saca de ningun lado visible tirando para atrás (son 821px por página).
             * @author Mario Poyato <mario.poyato@inmediastudio.es>
             */
            if (this.staticDraw) {
              xMax /= this.scale;
              xMin /= this.scale;
              canvasWidth = 1662; // Magic number! :P
              /**
                       _\,,
                       "-=\~     _
                       \\~___( ~
                       _|/---\\_
                       \        \
               */

            }
            return (w != 0 || _.isArray(this.points) && _.without(this.points, 0).length > 0) && ( (xMin >= 0 && xMax <= canvasWidth) || (xMin <= 0 && xMax >= 0) || (xMin < canvasWidth && xMax >= canvasWidth) );
            // La figura esta complentamente dentro del canvas
            /**
              ____________________
             |     Canvas         |
             |      ____________  |
             |     |            | |
             |     |   Square   | |
             |     |____________| |
             |____________________|

                   /------------/
                 xMin         xMax
             /--------------------/
             0               canvasWidth

             La figura tiene parte de la derecha dentro del canvas
              ____________________
             |                    |
           ____________   Canvas  |
          |            |          |
          |   Square   |          |
          |____________|          |
             |____________________|

          /------------/
        xMin         xMax
             /--------------------/
             0               canvasWidth

             La figura tiene parte de la izquierda dentro del canvas

              ____________________
             |                    |
             |   Canvas   ____________
             |           |            |
             |           |   Square   |
             |           |____________|
             |____________________|

                         /------------/
                       xMin         xMax
             /--------------------/
             0               canvasWidth
             **/
          }
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @returns {{attrs: {id: *, x: number, y: number, height: number, width: number}, className: *, children: Array}}
         * @method compress
         */
        compress: function () {
          return {
            attrs: {
              id: this.id,
              x: this.x / this.scale,
              y: this.y / this.scale,
              height: this.height / this.scale,
              width: this.width / this.scale
            },
            className: this.className,
            children: []
          };
        },

        /**
         * Copia los datos del objeto actual a la figura que se pasa como parámetro
         * @param shape objeto al que copiar los datos
         * @param boolean flagPosicion si es true o undefined se copia la posición con offset en caso de false no se copia
         * @method copyData
         */
        copyData: function(shape, flagPosicion) {
          if (flagPosicion === undefined || flagPosicion === true) {
            shape.x += this.diffX;
            shape.y += this.diffY;
          }
          shape.resize(this.width, this.height);
        }
      };

      return Shape;
    }
);
// noinspection JSHint
/**
 * Figura de la linea
 * @class Line
 * @extends Shape
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/line',["underscore", "modules/viewer/models/shape"],
    function (_, Shape) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Linee = function (options, scale, canvas) {
        // Default properties of KineticJS
        _.defaults(options || (options = {}), {
          className: "Linee",
          lineCap: 'round',
          lineJoin: 'round',
          points: [0, 0, 0, 0],
          strokeWidth: 10
        });
        Shape.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          this.width = this.width || (options.points[2] * this.scale);
          this.height = this.height || (options.points[3] * this.scale);
          /**
           * Color de la linea.
           *
           * @property color
           * @type {String}
           */
          this.color = options.color;
          /**
           * Puntos origen y destino de la flecha.
           *
           * @property strokeWidth
           * @type {Number}
           */
          this.strokeWidth = options.strokeWidth;
          /**
           * Puntos origen y destino de la linea.
           *
           * @property points
           * @type {Number}
           */
          this.points = options.points || []
        }
      };

      Linee.prototype = _.extend({}, Shape.prototype, {
        /**
         * Array de puntos de los que consta la linea (origen y destino) por lo que tiene 4 posiciones.
         *
         * @property points
         * @type {Array}
         * @default []
         */
        points: [],

        /**
         * Camino a seguir para el dibujado.
         * @method drawPath
         * @param {Object} context
         */
        drawPath: function(context) {
          context.beginPath();
          context.moveTo(this.x,this.y);
          context.lineTo((this.x + this.width), (this.y + this.height));
          context.lineWidth = this.strokeWidth * this.scale;
          context.lineJoin = "round";
          context.lineCap = "round";
        },

        /**
         * Función que realiza el dibujado.
         * @method draw
         * @param {Object} context
         */
        draw: function (context) {
          this.drawPath(context);
          context.strokeStyle = this.color;
          context.stroke();
          _.each(this.anchors, function(anchor) {
            anchor.draw(context);
          });
        },

        /**
         * Función que realiza la redimensión.
         * @method resize
         * @param {Integer} width
         * @param {Integer} height
         */
        resize: function (width, height) {
          this.width = width;
          this.height = height;
        },

        /**
         * Distance from a point to a line or segment.
         *
         * @param {number} x point's x coord
         * @param {number} y point's y coord
         * @param {number} x0 x coord of the line's A point
         * @param {number} y0 y coord of the line's A point
         * @param {number} x1 x coord of the line's B point
         * @param {number} y1 y coord of the line's B point
         * @param {boolean} overLine specifies if the distance should respect the limits
         * of the segment (overLine = true) or if it should consider the segment as an
         * infinite line (overLine = false), if false returns the distance from the point to
         * the line, otherwise the distance from the point to the segment.
         */
        dotLineLength: function(x, y, x0, y0, x1, y1, o) {
          function lineLength(x, y, x0, y0){
            return Math.sqrt((x -= x0) * x + (y -= y0) * y);
          }
          if(o && !(o = function(x, y, x0, y0, x1, y1){
              if(!(x1 - x0)) return {x: x0, y: y};
              else if(!(y1 - y0)) return {x: x, y: y0};
              var left, tg = -1 / ((y1 - y0) / (x1 - x0));
              return {x: left = (x1 * (x * tg - y + y0) + x0 * (x * - tg + y - y1)) / (tg * (x1 - x0) + y0 - y1), y: tg * left - tg * x + y};
            }(x, y, x0, y0, x1, y1), o.x >= Math.min(x0, x1) && o.x <= Math.max(x0, x1) && o.y >= Math.min(y0, y1) && o.y <= Math.max(y0, y1))){
            var l1 = lineLength(x, y, x0, y0), l2 = lineLength(x, y, x1, y1);
            return l1 > l2 ? l2 : l1;
          }
          else {
            var a = y0 - y1, b = x1 - x0, c = x0 * y1 - y0 * x1;
            return Math.abs(a * x + b * y + c) / Math.sqrt(a * a + b * b);
          }
        },

        /**
         * Función que indica si el objeto ha sido seleccionado.
         * @method isSelected
         * @param {Object} context
         * @param {Object} point
         */
        isSelected: function(context, point) {
          this.drawPath(context);
          var pis = false;
          if (_.isFunction(context.isPointInStroke)) {
            pis = context.isPointInStroke(point.x, point.y);
          }
          else {
            pis = this.dotLineLength(point.x, point.y, this.x, this.y, this.x+this.width, this.y+this.height, true) < this.strokeWidth*0.5;
          }
          return pis || _.find(this.anchors, function(anchor) { return anchor.isSelected(context, point, true); });
        },

        /**
         * Función que realiza la especialización de la selección para la flecha del cuadro de texto.
         * @method select
         * @param numtiradores
         */
        select: function (numtiradores) {
          numtiradores = numtiradores || 2; // Por defecto, pintar los dos tiradores

          this.anchors = [];

          if (numtiradores > 1) {
            if (this.anchors.length > 0) {
              // Todos los anclas tienen el mismo tamaño
              var offset = this.anchors[0].width / 2,
                x = this.x - offset,
                y = this.y - offset;
              this.anchors[0].x = x;
              this.anchors[0].y = y;
              this.anchors[1].x = x + this.width;
              this.anchors[1].y = y + this.height;
            }
            else {
              this.anchor({
                pos: "lt",
                x: this.x,
                y: this.y,
                color: "#009ee0",
                scale: this.scale
              });
              this.anchor({
                pos: "rb",
                x: ( this.x+this.width ),
                y: ( this.y+this.height ),
                color: "#009ee0",
                scale: this.scale
              });
            }
          }
          else {
            if (this.anchors.length > 0) {
              // Todos los anclas tienen el mismo tamaño
              var offset = this.anchors[0].width / 2;
              this.anchors[0].x = this.x - offset + this.width;
              this.anchors[0].y = this.y - offset + this.height;
            }
            else {
              this.anchor({
                pos: "rb",
                x: ( this.x + this.width ),
                y: ( this.y + this.height ),
                color: "#009ee0",
                scale: this.scale
              });
            }
          }
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Shape.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            color: this.color,
            strokeWidth: this.strokeWidth,
            lineCap: "round",
            lineJoin: "round",
            points: [0, 0, this.width / this.scale, this.height / this.scale]
          });

          return comprimido;
        },

        /**
         * Función que procesa los eventos que llegan al objeto
         * @param {String} type campo tipo del evento (touchstart, mousedown...)
         * @param {Object} center punto donde se realiza el evento
         * @method evento
         * @returns {integer}
         */
        evento: function(type, center) {
          return this.eventTriggered(type,center,false);
        }
      });

      return Linee;
    }
);
// noinspection JSHint
/**
 * Figura de la flecha
 * @class Arrow
 * @extends Line
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/arrow',["underscore", "modules/viewer/models/shape/line"],
    function (_, Line) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Arrow = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "Arrow"
        });
        Line.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          this.width = this.width || (options.points[2] * this.scale);
          this.height = this.height || (options.points[3] * this.scale);
          this.strokeWidth = options.strokeWidth;
        }
      };

      Arrow.prototype = _.extend({}, Line.prototype, {
        /**
         * Función que dibuja el camino que se realiza en el objeto
         * @param context contexto del canvas
         * @method drawPath
         */
        drawPath: function(context) {
          var fromX = this.x,
              fromY = this.y,
              toX   = this.x + this.width,
              toY   = this.y + this.height,
              headlen = 10,
              angle = Math.atan2(toY - fromY, toX - fromX);
          context.lineWidth = this.strokeWidth * this.scale;
          context.beginPath();
          context.moveTo(fromX, fromY);
          context.lineTo(toX, toY);
          context.moveTo(toX, toY);
          context.lineTo(
            toX - headlen * Math.cos(angle - Math.PI / 6),
            toY - headlen * Math.sin(angle - Math.PI / 6)
          );
          context.moveTo(toX, toY);
          context.lineTo(
            toX - headlen * Math.cos(angle + Math.PI / 6),
            toY - headlen * Math.sin(angle + Math.PI / 6)
          );
          context.lineCap = "round";
          context.lineWidth = this.strokeWidth * this.scale;
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Line.prototype.compress.call(this),
            // Obtener la inclinación del segmento
            angle = Math.atan2(this.height, this.width),
            fst = this.width / this.scale,
            snd = this.height / this.scale,
            thr = fst - 20 * Math.cos(angle-Math.PI/6),
            fnd = snd - 20 * Math.cos(angle-Math.PI/6);

          // Aplicar los atributos propios del objeto
          // Es importante que si se añaden parámetros aqui actualizar los parámetros de la especialización
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            points: [0, 0, fst, snd, thr, fnd, fst, snd, thr, fnd]
          });

          return comprimido;
        },

        /**
         * Copia los datos del objeto actual a la figura que se pasa como parámetro
         * @param shape objeto al que copiar los datos
         * @param boolean flagPosicion si es true o undefined se copia la posición con offset en caso de false no se copia
         * @method copyData
         * @author Mario Poyato <mario.poyato@inmediastudio.es>
         */
        copyData: function(shape, flagPosicion) {
          // -----------------------------------------------------------------------
          // Se toman los parámetros de la herencia
          Line.prototype.copyData.call(this,shape,flagPosicion);
          // -----------------------------------------------------------------------
          // -----------------------------------------------------------------------
          // Se toman los parámetros de la especialización
          shape.points = this.points;
          // -----------------------------------------------------------------------
        },

        /**
         * Función que procesa los eventos que llegan al objeto
         * @param type campo tipo del evento (touchstart, mousedown...)
         * @param center punto donde se realiza el evento
         * @returns {integer}
         */
        evento: function(type, center) {
          return this.eventTriggered(type,center,false);
        }
      });

      return Arrow;
    }
);
// noinspection JSHint
/**
 * Figura de la rectángulo con enlace
 * @class Square
 * @extends Shape
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/square',["underscore", "modules/viewer/models/shape"],
    function ( _, Shape) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Square = function (options, scale, canvas) {
        // Default properties of KineticJS
        _.defaults(options || (options = {}), {
          className: "Square",
          strokeWidth: 10,
          hasBorder: true,
          fill: "transparent",
          opacity: 0
        });
        Shape.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          this.fill = options.fill;
          this.strokeWidth = options.strokeWidth;
          this.opacity = options.opacity;
          this.color = options.color || options.fill /* Si no hay color definido para el borde se deja el color del relleno */;
          this.draggable = options.draggable; // Sirve para los anclas de selección
          this.hasBorder = options.hasBorder; // Indica si se dibuja el borde
        }
      };

      Square.prototype = _.extend({}, Shape.prototype, {
        /**
         * Función que realiza el dibujado.
         * @method draw
         * @param {Object} context
         */
        draw: function(context) {
          if (this.width != 0 && this.height != 0) {
            var alpha = context.globalAlpha;
            // Relleno del rectángulo
            if (this.opacity > 0 && this.fill != "transparent") {
              // Cuadro de dentro
              context.beginPath();
              context.rect(
                this.x,
                this.y,
                this.width,
                this.height
              );
              context.globalAlpha = this.opacity;
              context.fillStyle = this.fill;
              context.lineWidth = this.strokeWidth * this.scale;
              context.fill();
            }
            if (this.hasBorder && this.color != "transparent") {
              // Recuadro de fuera
              context.beginPath();
              context.strokeStyle = this.color;
              context.lineWidth = this.strokeWidth * this.scale;
              context.strokeRect(
                this.x,
                this.y,
                this.width,
                this.height
              );
            }
            else if (this.anchors.length > 0 && this.color != "transparent") {
              // Recuadro de fuera
              context.beginPath();
              context.globalAlpha = this.strokeOpacity;
              context.strokeStyle = this.color;
              context.lineWidth = this.strokeWidth * this.scale;
              context.strokeRect(
                this.x,
                this.y,
                this.width,
                this.height
              );
            }
            context.globalAlpha = alpha;
            _.each(this.anchors, function(anchor) {
              anchor.draw(context);
            });
          }
        }, // end draw

        /**
         * Función que realiza la redimensión.
         * @method resize
         * @param {Integer} width
         * @param {Integer} height
         */
        resize: function (width, height) {
          this.width = width;
          this.height = height;
        },

        /**
         * Función que indica si el objeto ha sido seleccionado.
         * @method isSelected
         * @param {Object} context
         * @param {Object} point
         * @return {boolean}
         */
        isSelected: function(context, point, busquedaExhaustiva) {
          var minX = this.x, minY = this.y, maxX = this.x+this.width, maxY = this.y+this.height;
          if (minX > maxX) { minX = maxX; maxX = this.x; }
          if (minY > maxY) { minY = maxY; maxY = this.y; }
          // En otro caso mirar si se ha seleccionado el rectángulo (búsqueda exhaustiva)
          if (busquedaExhaustiva) {
            if (point.y >= minY && point.y <= maxY && point.x >= minX && point.x <= maxX) {
              return true;
            }
          }
          else {
            if (point.y >= minY && point.y <= maxY && (point.x >= minX && point.x <= minX + this.strokeWidth ||  point.x >= maxX && point.x <= maxX + this.strokeWidth)) {
              return true;
            }
            if (point.x >= minX && point.x <= maxX && (point.y >= minY && point.y <= minY + this.strokeWidth ||  point.y >= maxY && point.y <= maxY + this.strokeWidth)) {
              return true;
            }
          }
          return _.find(this.anchors, function(anchor) { return anchor.isSelected(context, point, true); }) ? true: false;
        },

        /**
         * Función que realiza la especialización de la selección para la flecha del cuadro de texto.
         * @method select
         */
        select: function () {
          var width = this.width,
              height = this.height;
          if (this.element) {
            width  += this.border; // Se le suma el padding y el borde
            height += this.border; // Se le suma el padding y el borde
          }
          if (this.anchors.length > 0) {
            // Todos los anclas tienen el mismo tamaño
            var offset = this.anchors[0].width / 2,
                x = this.x - offset,
                y = this.y - offset;
            this.anchors[0].x = this.anchors[3].x = x;
            this.anchors[0].y = this.anchors[1].y = y;
            this.anchors[1].x = this.anchors[2].x = x + width;
            this.anchors[2].y = this.anchors[3].y = y + height;
          }
          else {
            this.anchor({
              x: this.x,
              y: this.y,
              pos: "lt",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              x: this.x + width,
              y: this.y,
              pos: "rt",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              x: this.x + width,
              y: this.y + height,
              pos: "rb",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              x: this.x,
              y: this.y + height,
              pos: "lb",
              color: "#009ee0",
              scale: this.scale
            });
          }
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Shape.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            color: this.color,
            strokeWidth: this.strokeWidth
          });

          return comprimido;
        },

        /**
         * Función que procesa los eventos que llegan al objeto
         * @param {String} type campo tipo del evento (touchstart, mousedown...)
         * @param {Object} center punto donde se realiza el evento
         * @method evento
         * @returns {integer}
         */
        evento: function(type, center) {
          var ret = this.eventTriggered(type,center,true);
          // Comprobar que se haya añadido una figura visible
          if (ret > 0 && this.height != 0 && this.width != 0) {
            return ret;
          }
          return 0;
        }
      });

      return Square;
    }
);
// noinspection JSHint
define(
    'views/text.editor',["marionette", "template"],
    function (Marionette, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        className: "textEditor",
        events: {
          "keydown @ui.input": function(event) {
            event = event.originalEvent || event;

            event.stopPropagation();
          },

          "keyup @ui.input": function () {
            this.model.set({
              raw: this.ui.input.val(),
              text: this.ui.input.val()
            });
          }
        },
        tagName: "main",
        template: JST["text.editor"],
        ui: {
          input: "textarea"
        },

        onShow: function() {
          var value = this.ui.input.val();

          this.ui.input.focus().val("").val(value);
        }
      });
    }
);
// noinspection JSHint
/**
 * Figura de del cuadro de texto.
 * @class Character
 * @extends Square
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/character',[
      // libraries
      "underscore",
      "backbone",
      "marionette",
      "app",
      // configuration
      "configuration",
      // language
      "language",
      // models
      "modules/viewer/models/shape/square",
      // views
      "views/dialog",
      "views/text.editor"

    ],
    function (_, Backbone, Marionette, App, configuration, language, Square, DialogView, TextEditorView) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @param elem Elemento interactivo donde opera el div de la ventana de texto
       * @constructor
       */
      var Character = function (options, scale, canvas, elem, oddPage) {
        _.defaults(options || (options = {}), {
          align: "left",
          className: "Character",
          fill: "#000000",
          fontFamily: 'Signika Negative',
          fontSize: 30,
          padding: 20,
          border: 6,
          width: 200,
          height: 200,
          order: 3 // Cuadros de texto
        });
        Square.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          this.scale = scale || 1;
          /**
           * Indica si el elemento interactivo se muestra o no (el div). Se oculta y se muestra porque en phonegap da problemas de rendimiento operar con el DOM constantemente.
           * @property interactivoMostrado
           * @type {boolean}
           * @default false
           */
          this.interactivoMostrado = false;
          /**
           * Borde de la figura.
           * @property border
           * @type {Integer}
           * @default 0
           */
          this.border = options.border * this.scale;
          /**
           * Padding de la figura.
           * @property padding
           * @type {Integer}
           * @default 20
           */
          this.padding = options.padding * this.scale;
          /**
           * Indica si la figura esta en la página par o impar.
           * @property oddPage
           * @type {boolean}
           */
          this.oddPage = oddPage || options.oddPage;
          /**
           * Relleno del cuadro de texto.
           * @property fill
           * @type {String}
           * @default #FFF (blanco)
           */
          this.fill = options.fill;
          /**
           * Color del borde del cuadro de texto.
           * @property color
           * @type {String}
           */
          this.color = options.color;
          /**
           * Texto en bruto del cuadro de texto.
           * @property raw
           * @type {String}
           */
          this.raw = options.raw;
          /**
           * Texto parseado (sin código html).
           * @property text
           * @type {String}
           */
          this.text = options.text;
          /**
           * Texto parseado (sin código html).
           * @property element
           * @type {Object} Jquery(Aside) o elemento interactivo del DOM.
           */
          this.element = $(document.createElement('aside'));
          $(elem).append(this.element);
          this.element
            .addClass("text")
            .addClass("editable")
            .css("padding", options.padding )
            .css("transform", "scale3d("+this.scale+","+this.scale+",1)")
            .css("-webkit-transform", "scale3d("+this.scale+","+this.scale+",1)")
            .css("display", "block")
            .css("margin-top", -3*this.scale)
            .css("margin-left", -3*this.scale)
            .css("zIndex", "999")
            .hide();
          this.element.html(this.raw);
          /**
           * Frontera del cuadro de texto. Se pinta en el canvas por problemas de rendimiento cuando se mueve en phonegap.
           * @property bound
           * @type {Square}
           */
          this.bound = new Square(_.extend(options, {
            color: "#F29431",
            fill: "#FFFFFF",
            strokeWidth: options.border,
            opacity: 1,
            width: options.width + options.border,
            height: options.height + options.border
          }), this.scale);
        }
        this.anchors = [];
      };

      Character.prototype = _.extend({}, Square.prototype, {
        /**
         * Función que indica si la figura se ve en el otro canvas a la vez
         */
        isInsideOtherCanvas: function() {
          var xMax = Math.max(this.x, this.x+this.width),
            xMin = Math.min(this.x, this.x+this.width);
          // Si estamos en el canvas de la izquierda (comprobar que se ve en el canvas de la derecha)
          // Si estamos en el canvas de la derecha   (comprobar que se ve en el canvas de la izquierda)
          return (!this.oddPage && xMax > this.canvas.width) || (this.oddPage && xMin < 0);
        },

        /**
         * Función que realiza el dibujado.
         * @method draw
         * @param {Object} context
         * @param {boolean} definitive indica si el cuadro de texto dibujado se mantiene así definitivamente o no
         */
        draw: function(context, definitive) {
          if (this.width != 0 && this.height != 0) {
            var shadowColor = context.shadowColor;
            // =========================================
            // Pintar la sombra
            /* //Poner una barra "/" al principio de esta línea para añadir el sombreado al cuadro de texto
            context.shadowColor = '#999';
            context.shadowBlur = 10;
            context.shadowOffsetX = 10;
            context.shadowOffsetY = 10;
             //*/
            // =========================================
            if (definitive) {
              this.resize(this.width, this.height, true);
              this.bound2 = null;
              this.drawElementSelection();
              if (!this.interactivoMostrado) {
                var numPaginas = $("article.page").length;
                /**------------------------------------
                 FIX para cuadros de texto que al aparecer dos aside con algo de desfase en la primera página se ve borroso
                 · Para ello se comprueba si es un cuadro de texto (order = 3)
                 · Se comprueba además si el cuadro es visible en ambos canvas
                 */
                // Hacer aparecer el aside únicamente cuando haya 1 página o si el número de páginas es mayor (2)
                //this.element.show();
                if (this.isInsideCanvas())
                  this.element.show();
                this.interactivoMostrado = true;
              }
            }
            else {
              if (this.bound2 && (this.width < 100 || this.height < 50)) {
                this.bound2.draw(context);
              }
              if (this.interactivoMostrado) {
                this.element.hide();
                this.interactivoMostrado = false;
              }
            }
            // Fix para ajuste del recuadro
            this.bound.width += this.border;
            this.bound.height += this.border;
            this.bound.draw(context);
            this.bound.width -= this.border;
            this.bound.height -= this.border;
            _.each(this.anchors, function(anchor) {
              anchor.draw(context);
            });
            // =========================================
            // Restaurar la sombra
            /* //Poner una barra "/" al principio de esta línea para añadir el sombreado al cuadro de texto
            context.shadowColor = shadowColor;
             //*/
            // =========================================
          }
        }, // end draw

        /**
         * Función que indica si el aside se debe dibujar o no (se tiene en cuenta si esta seleccionado o no, en caso de que si se  se oculta y en caso de que no se muestra.
         * @method drawPath
         * @param {Object} context
         */
        drawElementSelection: function() {
          // Si esta seleccionado
          if (this.anchors.length > 0) {
            this.element
              .css("border", "0px solid transparent")
              .css("left", this.x + this.border)
              .css("top", this.y + this.border)
              .height( (this.height - this.padding * 2) / this.scale - this.border )
              .width( (this.width - this.padding * 2) / this.scale - this.border );
            $("aside.text.editable").css("background-color","transparent");
            this.element.css("background-color","");
          }
          // Si no esta seleccionado
          else {
            this.element
              .css("border", "")
              .css("left", this.x)
              .css("top", this.y)
              .height( (this.height - this.padding * 2) / this.scale )
              .width( (this.width - this.padding * 2) / this.scale );
            $("aside.text.editable").css("background-color","");
            this.element.css("background-color","");
          }
        },

        /**
         * Función que realiza la especialización de la selección para la flecha del cuadro de texto.
         * @method select
         */
        select: function () {
          // En el caso de que no estuviera seleccionado añadir al final de los elementos
          if (this.anchors.length == 0) {
            var parent = this.element[0].parentNode;
            parent.appendChild(parent.removeChild(this.element[0]));
          }
          Square.prototype.select.call(this);
        },

        /**
         * Función que abre una ventana o popup en el caso de algunos elementos. Además se bloquea la figura.
         * @method openWindow
         */
        openWindow: function (fn, view) {
          var that = this;

          this.bound2 = null;
          Square.prototype.openWindow.call(this);
          //AV
          App.request("is:av", {context: this})
            .done(function(){
              App.request("av", {type: 'openTextEditor', content: that.raw, timeout: false})
                .done(function(data){
                  // ========================================================================
                  // === Esta mini-funcion inserta saltos de linea para que se vean en    ===
                  // === durante el modo de edición. No podemos mostrar colores, fuentes, ===
                  // === negritas, etc., pero al menos los saltos de linea se ven ok.     ===
                  // ========================================================================
                  var fake = data._html.replace(new RegExp("<div>", 'g'), "\n").replace(new RegExp("</div>", 'g'), "");
                  var tmp = document.createElement("DIV");
                  tmp.innerHTML = fake;
                  fake = tmp.textContent || tmp.innerText || "";
                  // ========================================================================¡
                  that.raw = data._html;
                  that.text = fake;
                  that.element.html(that.raw);
                  // Llamar a la función que se pasa como parámetro para que guarde en el historial el estado
                  App.request("is:device")
                    .fail(function() {
                      fn.call(view);
                    });
                });
            })
            .fail(function(){
              var dv = new DialogView({
                color: 6,
                title: language("editText"),
                width: 365,
                height: 280
              });

              dv.show(new TextEditorView({
                model: new Backbone.Model({
                  raw: that.raw || "",
                  text: that.text || ""
                }),
                modelEvents: {
                  "change": function () {
                    that.raw = this.model.get("raw");
                    that.text = this.model.get("text");
                    that.element.html(that.raw);
                    // Llamar a la función que se pasa como parámetro para que guarde en el historial el estado
                    App.request("is:device")
                      .fail(function() {
                        fn.call(view);
                      });
                  }
                }
              }));
            });
        },

        /**
         * Función que realiza la redimensión.
         * En caso de que se realice el arreglo de tamaño simplemente se ajusta el tamaño del rectángulo dibujado en el canvas y el aside.
         * En caso de que no se realice el arreglo se añade el trazo de la "segunda frontera" que consiste en un cuadro semitransparente que indica como quedaría el tamaño.
         * @method resize
         * @param {Integer} width
         * @param {Integer} height
         * @param {boolean} fix indica si hay que realizar el arreglo de tamaño minimo o no
         */
        resize: function (width, height, fix) {
          if (fix) {
            this.bound.width = this.width  = Math.max(100,width);
            this.bound.height = this.height = Math.max(50,height);
            if (this.anchors && this.anchors.length > 0) {
              this.select();
            }
          }
          else {
            if (width < 100 || height < 50) {
              if (!this.bound2) {
                this.bound2 = new Square(_.extend(this, {
                  color: "#FF9C34",
                  fill: "#FFFFFF",
                  strokeWidth: 2,
                  opacity: 0.55
                }), this.scale);
              }
              this.bound2.x = this.x;
              this.bound2.y = this.y;
              this.bound2.width = Math.max(100,width) + this.border * this.scale;
              this.bound2.height = Math.max(50,height) + this.border * this.scale;
            }
            else {
              this.bound2 = null;
            }
            this.width = width;
            this.height = height;
            this.bound.x = this.x;
            this.bound.y = this.y;
            this.bound.resize(this.width, this.height);
          }
        },

        /**
         * Función que procesa los eventos que llegan al objeto
         * @param {String} type campo tipo del evento (touchstart, mousedown...)
         * @param {Object} center punto donde se realiza el evento
         * @method evento
         * @returns {integer}
         */
        evento: function(type, center) {
          return this.eventTriggered(type,center,false);
        },

        /**
         * Función que indica si hay que limpiar el canvas para la figura.
         * @method unDraw
         * @returns {boolean}
         */
        unDraw: function() {
          if (this.element)
            this.element.remove();
          return true; // No hay que limpiar el canvas por ser totalmente interactivo
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Square.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            align: this.align,
            fill: this.fill,
            fontFamily: this.fontFamily,
            fontSize: this.fontSize,
            padding: this.padding / this.scale,
            color: "#f2942e",
            raw: this.raw,
            text: this.text,
            oddPage: this.oddPage
          });

          return comprimido;
        },

        /**
         * Copia los datos del objeto actual a la figura que se pasa como parámetro
         * @param shape objeto al que copiar los datos
         * @param boolean flagPosicion si es true o undefined se copia la posición con offset en caso de false no se copia
         * @method copyData
         */
        copyData: function(shape, flagPosicion) {
          // -----------------------------------------------------------------------
          // Se toman los parámetros de la herencia
          Square.prototype.copyData.call(this,shape,flagPosicion);
          // -----------------------------------------------------------------------
          // -----------------------------------------------------------------------
          // Se toman los parámetros de la especialización
          shape.align = this.align;
          shape.fill = this.fill;
          shape.fontFamily = this.fontFamily;
          shape.fontSize = this.fontSize;
          shape.padding = this.padding;
          shape.color = "#f2942e";
          shape.raw = this.raw;
          shape.text = this.text;
          // -----------------------------------------------------------------------
        }
      });

      return Character;
    }
);
// noinspection JSHint
/**
 * Clase de la figura óvalo
 * @class Cicle
 * @extends Shape
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/circle',["underscore", "modules/viewer/models/shape"],
    function (_, Shape) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Circle = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "Circle",
          strokeWidth: 10,
          hasBorder: true,
          opacity: 1,
          fill: "transparent"
        });
        Shape.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          /**
           * Relleno de la figura.
           *
           * @property fill
           * @type {String}
           */
          this.fill = options.fill;
          /**
           * Color de la figura.
           *
           * @property color
           * @type {String}
           */
          this.color = options.color || options.fill;
          /**
           * Ancho del borde del óvalo.
           *
           * @property strokeWidth
           * @type {Integer}
           */
          this.strokeWidth = options.strokeWidth;
          /**
           * Indica si tiene borde.
           *
           * @property hasBorder
           * @type {Integer}
           * @default true
           */
          this.hasBorder = options.hasBorder;
          /**
           * Opacidad del óvalo.
           *
           * @property opacity
           * @type {Number}
           * @default 1
           */
          this.opacity = options.opacity;
        }
      };

      Circle.prototype = _.extend({}, Shape.prototype, {
        /**
         * Función que realiza el dibujado.
         * @method draw
         * @param {Object} context
         */
        draw: function(context) {
          if (this.width != 0 && this.height != 0) {
            var alpha = context.globalAlpha;
            context.save();
            context.translate(this.x + this.width / 2, this.y + this.height / 2);
            context.scale(1, this.height / this.width);
            context.beginPath();
            context.arc(0, 0, (Math.abs(this.width) / 2), 0, 2 * Math.PI);
            context.restore();
            context.lineWidth = this.strokeWidth * this.scale;

            if (this.fill != "transparent") {
              context.globalAlpha = this.opacity;
              context.fillStyle = this.fill;
              context.fill();
            }
            if (this.hasBorder) {
              // Recuadro de fuera
              context.strokeStyle = this.color;
              context.stroke();
            }
            else if (this.anchors.length > 0) {
              var alpha = context.globalAlpha;
              context.globalAlpha = this.strokeOpacity;
              context.strokeStyle = this.color;
              context.stroke();
              context.globalAlpha = alpha;
            }

            context.globalAlpha = alpha;
            // Dibujar los anclas
            _.each(this.anchors, function(anchor) {
              anchor.draw(context);
            });
          }
        }, // end draw

        /**
         * Función que indica si el objeto ha sido seleccionado.
         * @method isSelected
         * @param {Object} context
         * @param {Object} point
         * @return {boolean}
         */
        isSelected: function(context, point, busquedaExhaustiva) {
          var touched = true;
          context.save();
          context.translate(this.x + this.width / 2, this.y + this.height / 2);
          context.scale(1, this.height / this.width);
          context.beginPath();
          context.arc(0, 0, (Math.abs(this.width) / 2), 0, 2 * Math.PI);
          touched = context.isPointInPath(point.x, point.y);
          // En otro caso mirar si se ha seleccionado el rectángulo (búsqueda exhaustiva)
          if (!busquedaExhaustiva && Math.abs(this.width) / 2 > this.strokeWidth) {
            context.beginPath();
            context.arc(0, 0, Math.abs(this.width) / 2 - this.strokeWidth, 0, 2 * Math.PI);
            touched = touched && !context.isPointInPath(point.x, point.y);
          }
          context.restore();
          return (touched && context.isPointInPath(point.x, point.y)) || _.find(this.anchors, function(anchor) { return anchor.isSelected(context, point, true); });
        },

        /**
         * Función que realiza la especialización de la selección para la flecha del cuadro de texto.
         * @method select
         */
        select: function () {
          if (this.anchors.length > 0) {
            // Todos los anclas tienen el mismo tamaño
            var offset = this.anchors[0].width / 2,
              x = this.x - offset,
              y = this.y - offset;
            this.anchors[0].x = this.anchors[3].x = x;
            this.anchors[0].y = this.anchors[1].y = y;
            this.anchors[1].x = this.anchors[2].x = x + this.width;
            this.anchors[2].y = this.anchors[3].y = y + this.height;
          }
          else {
            this.anchor({
              x: this.x,
              y: this.y,
              pos: "lt",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              x: this.x + this.width,
              y: this.y,
              pos: "rt",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              x: this.x + this.width,
              y: this.y + this.height,
              pos: "rb",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              x: this.x,
              y: this.y + this.height,
              pos: "lb",
              color: "#009ee0",
              scale: this.scale
            });
          }
        },

        /**
         * Función que realiza la redimensión.
         * @method resize
         * @param {Integer} width
         * @param {Integer} height
         */
        resize: function (width, height) {
          this.width = width;
          this.height = height;
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Shape.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            color: this.color,
            fill: this.fill,
            strokeWidth: this.strokeWidth
          });

          return comprimido;
        },

        /**
         * Función que procesa los eventos que llegan al objeto
         * @param {String} type campo tipo del evento (touchstart, mousedown...)
         * @param {Object} center punto donde se realiza el evento
         * @method evento
         * @returns {integer}
         */
        evento: function(type, center) {
          return this.eventTriggered(type,center,true);
        }
      });

      return Circle;
    }
);

define('modules/tools/link-forms/template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["form-list-link"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<div class=\"text_box\"></div>\n<div class=\"js-content\">\n</div>";
  });

this["JST"]["form-page-link"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, options, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;


  buffer += "<button class=\"js-ok\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "accept", options) : helperMissing.call(depth0, "language", "accept", options)))
    + "</button>\n<button class=\"js-cancel\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "cancel", options) : helperMissing.call(depth0, "language", "cancel", options)))
    + "</button>";
  return buffer;
  });

this["JST"]["form-web-link"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, stack2, options, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function";


  buffer += "<label>";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "tools.input_url", options) : helperMissing.call(depth0, "language", "tools.input_url", options)))
    + "</label>\n<input name=\"url\" value=\"";
  if (stack2 = helpers.url) { stack2 = stack2.call(depth0, {hash:{},data:data}); }
  else { stack2 = (depth0 && depth0.url); stack2 = typeof stack2 === functionType ? stack2.call(depth0, {hash:{},data:data}) : stack2; }
  buffer += escapeExpression(stack2)
    + "\"/>\n<label>";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "tools.input_name", options) : helperMissing.call(depth0, "language", "tools.input_name", options)))
    + "</label>\n<input name=\"tooltip\" value=\"";
  if (stack2 = helpers.tooltip) { stack2 = stack2.call(depth0, {hash:{},data:data}); }
  else { stack2 = (depth0 && depth0.tooltip); stack2 = typeof stack2 === functionType ? stack2.call(depth0, {hash:{},data:data}) : stack2; }
  buffer += escapeExpression(stack2)
    + "\"/>                                                                                                                                         \n<button class=\"js-ok\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "accept", options) : helperMissing.call(depth0, "language", "accept", options)))
    + "</button>\n<button class=\"js-cancel\">";
  options = {hash:{},data:data};
  buffer += escapeExpression(((stack1 = helpers.language || (depth0 && depth0.language)),stack1 ? stack1.call(depth0, "cancel", options) : helperMissing.call(depth0, "language", "cancel", options)))
    + "</button>";
  return buffer;
  });

return this["JST"];

});
define('modules/worktools/template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["activity"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression, self=this;

function program1(depth0,data) {
  
  
  return "\n        <div class=\"item-pic item-thumbnail\">\n    ";
  }

function program3(depth0,data) {
  
  
  return "\n        <div class=\"item-pic\">\n    ";
  }

function program5(depth0,data) {
  
  var buffer = "", stack1;
  buffer += "\n        	<img alt=\"\" src=\"";
  if (stack1 = helpers.srcImage) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.srcImage); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\" onerror=\"this.src='';\">\n        ";
  return buffer;
  }

function program7(depth0,data) {
  
  var buffer = "", stack1;
  buffer += "\n        	<div class=\"";
  if (stack1 = helpers.resourceIcon) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.resourceIcon); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\"></div>\n        ";
  return buffer;
  }

function program9(depth0,data) {
  
  var buffer = "", stack1;
  buffer += "\n        	<span alt=\"";
  if (stack1 = helpers.status) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.status); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\" title=\"";
  if (stack1 = helpers.status) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.status); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\" class=\"estado ";
  if (stack1 = helpers.status) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.status); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\"></span>\n        ";
  return buffer;
  }

function program11(depth0,data) {
  
  var buffer = "", stack1;
  buffer += "\n        ";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.idExt), {hash:{},inverse:self.noop,fn:self.program(12, program12, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "  \n    ";
  return buffer;
  }
function program12(depth0,data) {
  
  var buffer = "", stack1;
  buffer += "\n            <p class=\"item-name item-idExt\">ID:";
  if (stack1 = helpers.idExt) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.idExt); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "</p>\n        ";
  return buffer;
  }

  buffer += "<a class=\"item\" href=\"#\" name=\"";
  if (stack1 = helpers.link) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.link); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\">\n\n    ";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.showThumbnail), {hash:{},inverse:self.program(3, program3, data),fn:self.program(1, program1, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\n    	";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.showThumbnail), {hash:{},inverse:self.program(7, program7, data),fn:self.program(5, program5, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\n\n        ";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.status), {hash:{},inverse:self.noop,fn:self.program(9, program9, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\n    </div>\n    <p class=\"item-name\">";
  if (stack1 = helpers.tittle) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.tittle); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "</p>\n    ";
  stack1 = helpers['if'].call(depth0, (depth0 && depth0.quality), {hash:{},inverse:self.noop,fn:self.program(11, program11, data),data:data});
  if(stack1 || stack1 === 0) { buffer += stack1; }
  buffer += "\n</a>";
  return buffer;
  });

this["JST"]["filters"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression;


  buffer += "<div class=\"icos-rejilla\">\n                \n    <div class=\"icon active\">\n        <span class=\"ICONO_grid-ico\"></span>\n    </div>\n    \n    <div class=\"icon\">\n        <span class=\"ICONO_listado-ico\"></span>\n    </div>\n\n</div>\n<div id=\"filtercombos\"></div>\n<div id=\"filtersearch\" class=\"filter_all\">\n    <input class=\"buscar\" placeholder=\"";
  if (stack1 = helpers.txt0) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.txt0); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\" type=\"search\" />\n    <button class=\"icon-magnifier bgcolor-5 buscar\"></button>\n</div>";
  return buffer;
  });

this["JST"]["icon-worktool"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression;


  buffer += "<a class=";
  if (stack1 = helpers.iconClass) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.iconClass); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + " href=\"#\">\n    <i class=\"icon-";
  if (stack1 = helpers['class']) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0['class']); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\"></i>\n</a>";
  return buffer;
  });

this["JST"]["layout"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<nav class=\"buttons\"></nav>\n<div id=\"wrapper\">\n	<div id=\"filters\">\n	</div>\n</div>	\n<div class=\"elements\"></div>";
  });

this["JST"]["modalcontent"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression;


  buffer += "<iframe id=\"modalContent\" src=\"";
  if (stack1 = helpers.link) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.link); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\" width=\"100%\" height=\"100%\" style=\"border:none;background-color:white; position:relative\"></iframe>";
  return buffer;
  });

return this["JST"];

});
// noinspection JSHint
define(
    'modules/worktools/regions/index',["marionette", "modules/worktools/template","regions/replace"],
    function (Marionette, JST,ReplaceRegion) {
      "use strict";

      return Marionette.Layout.extend({
        tagName: "main",
        template: JST["layout"],
        id: "worktools",
        regions: {
            buttons: ".buttons",
            filters: "#filters",
            elements: ".elements"
        },
        regionType: ReplaceRegion,
      });
    }
);
//noinspection JSHint
define(
    'modules/worktools/controllers/worktool',["underscore","backbone", "marionette", "app", "language", "configuration", "models/resource.collection"],
    function (_, Backbone, Marionette, App, Language, configuration, ResourceCollection) {
      "use strict";

      return Marionette.Controller.extend({

        initialize : function (options){

          var activeModel = "";
          var activeList = "";
          var filterList = "";
          var typeButtonActive = "";

          this.selectChapter = (options.unidad) ? options.unidad : '';

          this.typeButtonActive = options.view.collection.at(0).get("iconClass");

          // Carga la primera lista de recursos cuando se abre el popup
          this.loadActivities(options.view.collection.at(0)).done(function(lista){

            this.activeModel = options.view.collection.at(0);
            options.resource.collection.set(lista);
            options.resource.render();
            options.resource.$el.parents(".dialog").find("header").find("h1").text(options.view.collection.at(0).get('name'));

            if(this.typeButtonActive == "library"){
                // options.resource.$el.find(".item").css('width','150px');
                // options.resource.$el.find(".item-pic").css('height','205px');
                options.resource.$el.find(".item-pic").css('display','inline-block');
            }else{
              options.resource.$el.find(".item").css('width','250px');
              options.resource.$el.find(".item-pic").css('height','134px');
            }

            if(!options.view.collection.at(0).get('porunidad')){
              options.dropdown.$el.find("#filtercombos").hide();
              options.dropdown.$el.find("#filtersearch").removeClass('filter_all');
              options.dropdown.$el.find("#filtersearch").addClass('filter_unique');

            }else{
              options.dropdown.$el.find("#filtercombos").show();
              options.dropdown.$el.find("#filtersearch").removeClass('filter_unique');
              options.dropdown.$el.find("#filtersearch").addClass('filter_all');
            }
        },this);


          // Cuando se clikea un icono se cambia la vista

          options.view.on("itemclick",function(model){

            var loader = new options.loader({
              auto: true,
              modal: true,
              parent: "#listItems"
            });

            loader.render();

            this.activeModel = model;
            this.typeButtonActive = model.get("iconClass");
            var self = this;

            this.loadActivities(model).done(function(lista){

              if(model.get('iframe')){
                options.resource.$el.html('<iframe iframeonload src="'+model.get('iframe')+'"></iframe>');
                options.resource.$el.find("iframe").css("width","100%");
                options.resource.$el.find("iframe").css("min-height","100%");
                options.resource.$el.find("iframe").css("border","0px");
              }else{
                options.resource.$el.html('');
              }
              options.resource.collection.set(lista);
              options.resource.render();
              options.view.$el.children().removeClass('active');
              options.resource.$el.parents(".dialog").find("header").find("h1").text(model.get('name'));
              options.view.$el.find("#button_" + model.get("buttonid")).addClass("active");

              if(self.typeButtonActive == "library"){
                // options.resource.$el.find(".item").css('width','150px');
                // options.resource.$el.find(".item-pic").css('height','205px');
                options.resource.$el.find(".item-pic").css('display','inline-block');
              }else{
                options.resource.$el.find(".item").css('width','250px');
                options.resource.$el.find(".item-pic").css('height','134px');
              }


              if(!model.get('porunidad')){
                options.dropdown.$el.find("#filtercombos").hide();
                options.dropdown.$el.find("#filtersearch").show();
                options.dropdown.$el.find("#filtersearch").removeClass('filter_all');
                options.dropdown.$el.find("#filtersearch").addClass('filter_unique');

              }else{
                if(self.typeButtonActive == "library"){
                    options.dropdown.$el.find("#filtercombos").show();
                    options.dropdown.$el.find("#filtersearch").show();
                    options.dropdown.$el.find("#filtersearch").removeClass('filter_all');
                    options.dropdown.$el.find("#filtersearch").addClass('filter_unique');
                }else if(model.get('iframe')){
                    options.dropdown.$el.find("#filtercombos").hide();
                    options.dropdown.$el.find("#filtersearch").hide();
                }else{
                    options.dropdown.$el.find("#filtercombos").show();
                    options.dropdown.$el.find("#filtersearch").show();
                    options.dropdown.$el.find("#filtersearch").removeClass('filter_unique');
                    options.dropdown.$el.find("#filtersearch").addClass('filter_all');
                }
              }

              // Fix para iPad. Se queda el foco en la busqueda al abrir combos.
              $('#filtercombos .dropdown').bind('touchstart', function(e) {
                e.preventDefault();
                $('input.buscar').blur();
              });

              loader.close();

              // Si tenia todas las unidades seleccioandas,
              // disparo el evento de seleccionar para volver
              // a filtrar los elementos repetidos.
              if(self.selectChapter == "All"){
                options.dropdown.trigger("select");
              }
            });

          },this);

          /**
           * FIX: Búsqueda que suma restricciones
           * @author Mario Poyato <mario.poyato@inmediastudio.es>
           */
          this.searchFunction = function(keepSearch){

            var self = this;
            var list = new ResourceCollection(self.activeList);
            var filter = list;

            $.each(options.dropdown.collection.models,function(i,v){
              $.each(v.get('options'),function(i1,v1){
                
                if(v.get("title") == v1.name ){

                  if(self.typeButtonActive == "activities"){

                    if(v1.data.code == 'All'){
                      var unicos = new ResourceCollection();
                      var cantidad = 0;

                      _.each(self.activeList, function(model, index) {
                        //cantidad = unicos.where({tittle: model.get("tittle")}).length;
                        if(model.get("idExt")){
                            cantidad = unicos.where({idExt: model.get("idExt")}).length;
                        }
                        if(cantidad == 0){
                          unicos.add(model);
                        }
                      });

                      filter = unicos;
                      // filter = list;
                    }else{
                      filter = new Backbone.Collection(filter.where({chapter:v1.data.code}));
                    }
                    self.selectChapter = v1.data.code;

                  }else{

                    if(self.typeButtonActive == "library"){
                      // Apaño para que salga la opción de "CONTINGUTS" o el seleccionar todas.
                      if (v1.data.id.toString() != "0")
                      {
                        filter = new Backbone.Collection(filter.filter(function(item){
                          return _.contains(item.get('filter'), v1.data.id.toString());
                        }) );
                      }
                    }else{

                      if(v1.data.code == 'All'){
                        var unicos = new Backbone.Collection();
                        var cantidad = 0;

                        _.each(self.activeList, function(model, index) {

                          cantidad = unicos.where({tittle: model.get("tittle")}).length;
                          if(model.get("idExt")){
                            cantidad = unicos.where({idExt: model.get("idExt")}).length;
                          }

                          if(cantidad == 0){
                            unicos.add(model);
                          }
                        });

                        filter = unicos;
                        // filter = list;
                      }else{
                        filter = new Backbone.Collection(filter.where({code:v1.data.code}));
                      }
                    }

                  }
                }
              });
            });

            if (keepSearch !== true) {
              $("#filtersearch").find("input[class=buscar]").val("");
            }
            this.filterList = filter.toArray();
            options.resource.collection.reset(); // Sin esto se desordenan todos los items
            options.resource.collection.set(filter.toArray());

            if(this.typeButtonActive == "library"){
              // options.resource.$el.find(".item").css('width','150px');
              // options.resource.$el.find(".item-pic").css('height','205px');
              options.resource.$el.find(".item-pic").css('display','inline-block');
            }else{
              options.resource.$el.find(".item").css('width','250px');
              options.resource.$el.find(".item-pic").css('height','134px');
            }

            return filter;
          };

          //Evento del select
          options.dropdown.on("select", this.searchFunction,this);

          // Buscador
          options.dropdown.on("listSearch",function(val){

            var self = this;

            var list = new Backbone.Collection(self.activeList);
            var filter = list;

            filter = this.searchFunction(true);

            if(!options.dropdown.$el.find("#filtersearch").hasClass("filter_unique")){
              $.each(options.dropdown.collection.models,function(i,v){
                $.each(v.get('options'),function(i1,v1){

                  if(v.get("title") == v1.name ){

                    if(self.typeButtonActive == "activities"){

                      if(v1.data.code == 'All'){
                        var unicos = new Backbone.Collection();
                        var cantidad = 0;

                        _.each(self.activeList, function(model, index) {
                          //cantidad = unicos.where({tittle: model.get("tittle")}).length;
                            if(model.get("idExt")){
                                cantidad = unicos.where({idExt: model.get("idExt")}).length;
                            }
                          if(cantidad == 0){
                            unicos.add(model);
                          }
                        });

                        filter = unicos;
                        // filter = list;
                      }else{
                        filter = new Backbone.Collection(filter.where({chapter:v1.data.code}));
                      }
                      self.selectChapter = v1.data.code;

                    }else{

                      if(self.typeButtonActive == "library"){

                        filter = new Backbone.Collection(filter.filter(function(item){
                          return _.contains(item.get('filter'), v1.data.id.toString());
                        }) );

                      }else{
                        filter = list;
                        if(v1.data.code == 'All'){
                          var unicos = new Backbone.Collection();
                          var cantidad = 0;

                          _.each(self.activeList, function(model, index) {

                            cantidad = unicos.where({tittle: model.get("tittle")}).length;
                            if(model.get("idExt")){
                              cantidad = unicos.where({idExt: model.get("idExt")}).length;
                            }

                            if(cantidad == 0){
                              unicos.add(model);
                            }
                          });

                          filter = unicos;
                          // filter = list;
                        }else{
                          filter = new Backbone.Collection(filter.where({code:v1.data.code}));
                        }
                      }

                    }
                  }
                });
              });
            }

            // Limpiar buscador

            options.dropdown.on("cleanSearch",function(){
              $("#filtersearch").find("input[class=buscar]").val("");
              options.resource.collection.set(this.filterList);
            },this);

            this.search(filter.toArray(),val).done(function(list){

              options.resource.collection.set(list);

              if(self.typeButtonActive == "library"){
                // options.resource.$el.find(".item").css('width','150px');
                // options.resource.$el.find(".item-pic").css('height','205px');
                options.resource.$el.find(".item-pic").css('display','inline-block');
              }else{
                options.resource.$el.find(".item").css('width','250px');
                options.resource.$el.find(".item-pic").css('height','134px');
              }

            });

          },this);

          // Abrir recursos
            
          options.resource.on("openResource",function(model){
            var _idRecurso = false;
            var elLink = model.get("link") ;
            if ( elLink.indexOf( "/data/RECURSOS/" ) != -1  ){
              elLink = elLink.split("/data/RECURSOS/");
              elLink = elLink[1];
              elLink = elLink.split("/");
              _idRecurso = elLink[0];
            }
            /* else{
              _idRecurso = model.get("tittle") || "noName";
            }*/
            var _isActividad = false;
            var nameActividad = model.get("name") || "";
            if ( /[A,a]ctivi\w+/.test( nameActividad ) || /[J,j]ardu\w+/.test( nameActividad ) ) {
              _isActividad = true;
            }

            // Comprobacion de AV online
            var tipo = (this.typeButtonActive === 'links' || this.typeButtonActive === 'library') ? "url" : "resource";
            if(!model.get("idExt")){
                tipo = "url";
            }
            // Si is:external, abrimos todos los recursos como URLs
            App.request("is:external")
              .done(function () {
                tipo = 'externalUrl';
              })
              .always(function(){
                
                var value = model.get("link");
                                
                var open = function () {
                    App.execute(
                        "open",
                        {
                          color: 5,
                          idRecurso: _idRecurso,
                          isActividad: _isActividad,
                          chapter: model.get("chapter"),
                          index: model.get("order") - 1 || _.indexOf(model.collection.where({chapter: model.get("chapter")}), model),
                          type: tipo,
                          value: value
                        }
                    );
                }
                
                App.request("is:av")
                    .done(function () {
                    
                        if (tipo == "resource" && _idRecurso) {
                            
                            var _parent = window.parent;
                            
                            var _identifier = "openResource";
                            
                            var ALLOWED = {
                              htm: "iframe.resource",
                              html: "iframe.resource"
                            },
                            extension,
                            index,
                            isAV = false,
                            isOpenable;

                            _.defaults(options || (options = {}), {
                              type: "url"
                            });

                            extension = model.get("link").toLowerCase();
                            index = extension.lastIndexOf(".");
                            extension = (index !== -1) ? extension.substr(index + 1) : "";
                            
                            isOpenable = _.contains(_.keys(ALLOWED), extension);
                            
                            _parent.postMessage(
                                {
                                    type: 'getResourceURL',
                                    idResource: _idRecurso,
                                    resourcesPath: 'data/RECURSOS/',
                                    filePath: model.get("link").split("/"+_idRecurso+"/").pop(),
                                    viewer: (isOpenable) ? true : false,
                                    identifier: _identifier
                                },
                                '*'
                            );
                            var listener = function(event) {
                                console.log('Event data URL answered: '+event.data.url);
                                if (typeof event.data != 'undefined' && typeof event.data.url != 'undefined') {
                                    value = event.data.url;
                                    open();
                                }
                                if (window.addEventListener) {
                                    window.removeEventListener("message", listener, false);
                                    listener = null;
                                } else {
                                    window.detachEvent("onmessage", listener);
                                    listener = null;
                                }
                            }
                            if (window.addEventListener) {
                                window.addEventListener("message", listener, false);
                            } else {
                                window.attachEvent("onmessage", listener);
                            }
                            _parent = null;

                        } else {
                            open();
                        }
                    
                    })
                    .fail(function() {
                        open();
                    });
                                
              });

          },this);
            
        },

        loadActivities : function (model){
          var deferred = $.Deferred();

            $.when (
              App.request("url:worktools",{context: this,url: model.get('path'), type:model.get('iconClass')}),
              App.request("chapter",{context: this}),
              App.request("chapters",{clone:true,context: this,collection: true})
              )
                .done(function (resource, chapter,chapters) {

                  if(resource[1] == ""){
                    chapters.each(function (value, key, list) {
                            list[key] = {
                                name: value.get("title"),
                                data: {
                                    code: value.get("code"),
                                    page: value.get("pages").first().get("id")
                                }
                            };

                          if(key == list.length - 1){
                            list[list.length] = {
                              name: Language("allUnits"),
                              data: {
                                title: Language("allUnits"),
                                page: "All",
                                code: "All",
                                status: "active"
                              }
                            };
                          }

                        });

                        chapters = chapters.toArray();

                        if(this[2].selectChapter == "" || typeof this[2].selectChapter == 'undefined'){
                          Marionette.getOption(this[2],"dropdown").collection.set({
                              title: chapter.get("title"),
                              options: chapters
                          });
                        }else{

                          var selectChapterAux = this[2].selectChapter;
                          var title = "";

                          $.each(chapters,function(i,v){
                            if(v.data.code == selectChapterAux){
                              title = v.name;
                            }
                          });

                          Marionette.getOption(this[2],"dropdown").collection.set({
                              title: title,
                              options: chapters
                          });
                        }

                        Marionette.getOption(this[2],"dropdown").$el.find(".dropdown").removeClass('biblioteca');
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtercombos").css('width', "47%");
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtersearch").addClass('filter_all');
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtersearch").removeClass('filter_unique');
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtercombos").removeClass('clearfix');

                        $("#filtersearch").find("input[class=buscar]").val("");

                        // === Para desactivar unidades que no tienen recursos ===
                        var that = this;
                        var chaptersWithResources = [];

                        // Obtengo los capítulos que tienen recursos
                        _.each(resource[0].models, function(res) {
                          res.set("name", model.get( "name") );
                          chaptersWithResources.push(res.get("chapter"));
                        });

                        // Filtro repetidos
                        chaptersWithResources = _.uniq(chaptersWithResources);

                        // Recorro los capítulos. Si no tiene recursos, lo desactivo.
                        _.each(chapters, function(chapter, index) {
                          var code = chapter.data.code;
                          if (code == "All" || chaptersWithResources.indexOf(code) !== -1) {
                          } else {
                            // Desactivo las opciones
                            Marionette.getOption(that[2], "dropdown")
                              .$el.find('#filtercombos li a')
                              .eq(index).addClass('inactive disabled');
                          }
                        });
                        // ======

                    this[2].activeList = resource[0].models;

                    if(!model.get('porunidad')){
                      this[2].filterList = resource[0].models;
                      deferred.resolve(resource[0].models);
                    }else{
                      if(this[2].selectChapter == "" || typeof this[2].selectChapter == 'undefined'){
                        this[2].filterList = resource[0].where({chapter:chapter.get("code")})
                        deferred.resolve(resource[0].where({chapter:chapter.get("code")}));
                      }else{
                        if(this[2].selectChapter == "All"){
                          this[2].filterList = resource[0].models;
                          deferred.resolve(resource[0].models);
                        }else{
                          this[2].filterList = resource[0].where({chapter:this[2].selectChapter});
                          deferred.resolve(resource[0].where({chapter:this[2].selectChapter}));
                        }

                      }
                    }

                  }else{

                      Marionette.getOption(this[2],"dropdown").collection.set(resource[1]);

                      if(this[2].typeButtonActive == "links"){
                        Marionette.getOption(this[2],"dropdown").$el.find(".dropdown").removeClass('biblioteca');
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtercombos").css('width', "47%");
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtersearch").addClass('filter_all');
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtersearch").removeClass('filter_unique');
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtercombos").removeClass('clearfix');
                      }else{
                        Marionette.getOption(this[2],"dropdown").$el.find(".dropdown").addClass('biblioteca');
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtercombos").css('width', "auto");
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtersearch").removeClass('filter_all');
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtersearch").addClass('filter_unique');
                        Marionette.getOption(this[2],"dropdown").$el.find("#filtercombos").addClass('clearfix');
                      }

                    $("#filtersearch").find("input[class=buscar]").val("");

                      this[2].activeList = resource[0].models;
                      this[2].filterList = resource[0].models;
                      deferred.resolve(resource[0].models);
                  }

                });

          return deferred.promise();
        },

        search: function (list, q){
          var  self = this;
          var deferred = $.Deferred();

          if (q) {
            deferred.resolve(list.filter(function(element) {
              var text = element.get("tittle").toLowerCase() || "";
              text = self.htmlentitiesJS(text);
              text = self.removeHTML(text);

              return (~text.indexOf(q.toLowerCase()));
            }));
          }else{
            deferred.resolve(list);
          }

          return deferred.promise();
        },

       htmlentitiesJS: function(cadena) {
          var caracteresReemplazados = ['À', 'à', 'Á', 'á', 'Â', 'â', 'Ã', 'ã', 'Ä', 'ä', 'Å', 'Æ', 'æ', 'Ç', 'ç', 'Ð', 'ð', 'È', 'è', 'É', 'é', 'Ê','ê', 'Ë', 'ë', 'Ì', 'ì', 'Í', 'í', 'Î', 'î', 'Ï', 'ï', 'Ñ', 'ñ', 'Ò', 'ò', 'Ó', 'ó', 'Ô', 'ô', 'Õ', 'õ', 'Ö', 'ö', 'Ø', 'ø', 'Œ', 'œ', 'ß', 'Þ', 'þ', 'Ù', 'ù', 'Ú', 'ú', 'Û', 'û', 'Ü', 'ü', 'Ý', 'ý', 'Ÿ', 'ÿ', '©',  '®',  '€', '<', '>', '"'];
          var caracteresOriginales= ['&Agrave;', '&agrave;', '&Aacute;', '&aacute;', '&Acirc;', '&acirc;', '&Atilde;', '&atilde;', '&Auml;', '&auml;', '&Aring;',' &AElig;', '&aelig;', '&Ccedil;', '&ccedil;', '&ETH;', '&eth;', '&Egrave;', '&egrave;', '&Eacute;', '&eacute;', '&Ecirc;', '&ecirc;', '&Euml;', '&euml;', '&Igrave', '&igrave;', '&Iacute;', '&iacute;', '&Icirc;', '&icirc;', '&Iuml;', '&iuml;', '&Ntilde;', '&ntilde;', '&Ograve;', '&ograve;', '&Oacute;', '&oacute;', '&Ocirc;', '&ocirc;', '&Otilde;', '&otilde;', '&Ouml;', '&ouml;', '&Oslash;', '&oslash;', '&OElig;', '&oelig;', '&szlig;', '&THORN;', '&thorn;', '&Ugrave;', '&ugrave;', '&Uacute;', '&uacute;', '&Ucirc;', '&ucirc;', '&Uuml;', '&uuml;', '&Yacute;', '&yacute;', '&Yuml;', '&yuml;', '&copy;',  '&reg;',  '&euro;', '&lt;', '&gt;', '&quot;'];
          var longOriginales = caracteresOriginales.length;

          for ( var i=0; i < longOriginales; i++ ) {
            var patron = new RegExp(caracteresOriginales[i],"g");
            cadena = cadena.replace(patron, caracteresReemplazados[i]);
          }

          return cadena;
        },

        removeHTML: function (html) {
          var tmp = document.createElement("DIV");
          tmp.innerHTML = html;
          return tmp.textContent || tmp.innerText || "";
        }

      });

    }
);

//noinspection JSHint
define(
    'modules/worktools/views/listElementView',[
      "marionette",
      "configuration",
      "modules/worktools/template"
    ],
    function (Marionette, configuration, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        template: JST["activity"],
        className: "elements",
        modelEvents: {
            "change": "render"
        },
        triggers:{
          "touch" : "openResource"
        },
        templateHelpers:{
          blankImg: configuration.root + "assets/images/blank.png"
        }
      });

    }
);
// noinspection JSHint
define(
    'modules/worktools/views/listView',["marionette", "modules/worktools/views/listElementView"],
    function (Marionette, ListElementView) {
      "use strict";

      return Marionette.CollectionView.extend({
        className: "clearfix",
        id: "listItems",
        itemView: ListElementView,
        itemEvents:{
          "openResource" : function(name,view, object){
            if(object.model.get('link') != ""){
                this.trigger("openResource",object.model);
            }
          }
        },
        appendHtml: function(collectionView, itemView, index){
            var childrenContainer = collectionView.itemViewContainer ? collectionView.$(collectionView.itemViewContainer) : collectionView.$el;
            var children = childrenContainer.children();
            if (children.size() <= index) {
              childrenContainer.append(itemView.el);
            } else {
              children.eq(index).before(itemView.el);
            }
          },

        itemViewOptions: function(model,index){
            var inactive = model.get('link');
            return{
                className: "elements " + ((model.get('link') == "") ? 'inactive': ''),

            }
        }  
      });
    }
);
//noinspection JSHint
define(
    'modules/worktools/views/listFiltersContainer',[
      "marionette",
      "configuration",
      "views/dropdown"
    ],
    function (Marionette, configuration,DropdownView) {
      "use strict";

      return DropdownView.extend({
        className: "my dropdown",
        templateHelpers: {
            color: 5
        }   
      });

    }
);
// noinspection JSHint
define(
    'modules/worktools/views/filtersContainer',["marionette", "modules/worktools/template", "modules/worktools/views/listFiltersContainer"],
    function (Marionette, JST ,DropdownView) {
      "use strict";

      return Marionette.CompositeView.extend({
        className: "filters col-form",
        template: JST["filters"],
        itemViewContainer: "#filtercombos",
        itemView: DropdownView,
        itemEvents:{
            
          "select": function(name,view,object){
            this.trigger("select");
          }
        },
        itemViewOptions: {
          color: 5
        },

        events: {
          "keydown .buscar": function(event) {
            event = event.originalEvent || event;

            event.stopPropagation();
          },
          "keyup input.buscar": "search",
          "touch button.buscar": "search",
          "focus input.buscar": "cleanSearch",
          "touch div.icon": "changeView"
        },

        search: function (event) {
         if(event.type ==  "touch"){
           this.trigger("listSearch",$("#filtersearch").find("input[class=buscar]").val());
         }else{
           this.trigger("listSearch",$("#filtersearch").find("input[class=buscar]").val());
         }
        },
      cleanSearch: function (event) {
        this.trigger("cleanSearch");
      },
      changeView: function (event) {
        $(".icon").removeClass("active");
        $(event.currentTarget).addClass("active");
        if($(event.currentTarget).find("span").hasClass("ICONO_grid-ico")){
          $("#listItems").removeClass("listado");
        }else{
          $("#listItems").addClass("listado");
        }
      }
      });
    }
);
//noinspection JSHint
define(
    'modules/worktools/views/listIcon',[
      "marionette",
      "configuration",
      "views/icon"
    ],
    function (Marionette, configuration, Icon) {
      "use strict";

      return Icon.extend({
        triggers:{
            "touch" : "itemclick"
        }
      });

    }
);
// noinspection JSHint
define(
    'modules/worktools/views/listIconView',[ "marionette", "modules/worktools/template", "views/icon.collection", "modules/worktools/views/listIcon" ],
    function ( Marionette, JST, IconCollection, Icon ){
      "use strict";

      return IconCollection.extend({
        className: "buttons",
        itemView: Icon,
        itemEvents: {
            "itemclick" : function( name, view, object ){ 
                if( object.model.get( 'active' ) ){
                    this.trigger( 'itemclick', object.model );
                }  
            }
        },
        itemViewOptions: function( model, index ){
            var active = ( index == 0 );
            var inactive = model.get( 'active' );
            return {
                className: model.get( 'iconClass' ) +" "+ ( (active) ? 'active': '' ) +" "+ ( (!inactive) ? 'inactive': '' ),
                id: "button_" + model.get( 'buttonid' )
            }
        }
      });
    }
);
//noinspection JSHint
define(
    'modules/worktools/views/contentDialog',[
      "marionette",
      "configuration",
      "modules/worktools/template"
    ],
    function (Marionette, configuration, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        template: JST["modalcontent"],
        tagName: "main"
      });

    }
);
define(
    'modules/tools/link-forms/EditLink',[
        "app",
        "modules/tools/link-forms/template",
        "underscore",
        "backbone",
        "marionette",
        "configuration",
        "language",
        "models/page.collection",
        "modules/index/views/index",
        "modules/index/views/thumbnail",
        "modules/index/regions/index",
        "views/chapters.dropdown",
        "views/dialog",
        "modules/worktools/regions/index",
        "modules/worktools/controllers/worktool",
        "modules/worktools/views/listView",
        "modules/worktools/views/filtersContainer",
        "modules/worktools/views/listIconView",
        "modules/worktools/views/contentDialog",
        "modules/worktools/views/listElementView",
        "models/resource.collection",
        "controllers/loader",
    ],
    function (App, JST, _, Backbone, Marionette, configuration, Language, PageCollection, IndexView, ThumbnailsView, IndexLayout, ChaptersDropdownViewDeferred, DialogView, IndexLayoutResource, Controller, ListView, FiltersContainer, ListIconView, ContentDialog, ListElementView, ResourceCollation, Loader) {
        "use strict";
        
        var item = null;
        var model = new Backbone.Model({
            action : "",
            url: "",
            page: "",
            tooltip: "",
            link: "",
            button: "",
            dropdown: ""
        });

        /*
         * TMP: Modelo de datos de los tipos de links
         */
        var collection = new Backbone.Collection([
            {
                title: Language("webLink"),
                action: "web"
            },
            {
                title: Language("pageLink"),
                action: "page"
            }
        ]);

        // Si NO es Mi Cuaderno agrego enlace a recursos
        App.request("is:notebook")
          .fail(function () {
            collection.add([
             {
                title: Language("resourceLink"),
                action: "resource"
             }
            ]);
        });

        App.request("is:av", {context: this})
            .done(function(){
                collection.add([
                   {
                       title: Language("cloudLink"),
                       action: "cloudFile"
                   }
                ]);
        });

        // ItemView del listado de links
        var my_template_html = '<a href="#"><%= title %></a>'
        var iv = Marionette.ItemView.extend({
            tagName: "li",
            template: _.template(my_template_html),
            triggers: {
                "click a": "changeView"
            }
        });

        /*
         * Vista de link tipo web
         */
        var webView = Marionette.ItemView.extend({
            template: JST["form-web-link"],
            className: "wrapper",
            ui: {
                "url": "input[name=url]",
                "tooltip": "input[name=tooltip]"
            },
            onShow: function(){
                if(this.model.get("url") == undefined || this.model.get("url").length == 0)
                    this.ui.url.val("http://");
            },
            events: {
                "click .js-ok": function (ev) {
                    this.model.set("action", this.model.get("action"));
                    this.model.set("url", this.ui.url.val());
                    this.model.set("tooltip", this.ui.tooltip.val());
                    
                    this.trigger("updateItemLink", this.model);
                    this.trigger("quit");
                },
                "click .js-cancel": function (e) {
                    this.trigger("quit");
                },
                "keydown": function(event) {
                  event = event.originalEvent || event;
                  event.stopPropagation();
                }
            }
        });
        
        /*
         * Metodo que inicializa las vistas
         */
        var init = function(){
            var self = this;
            item = Marionette.getOption(this, "item"); // Objeto seleccionado

            // CollectionView del listado de tipos de links
            var cv = (function () {
                var x = Marionette.CollectionView.extend({
                    tagName: "ul",
                    className: "link-list",
                    collection: collection,
                    itemView: iv
                });
                return new x();
            })();
            
            showView(item);
            
            function showView(item){
                
                switch(item.get("action")){
                case "web":
                    
                    model.set("action", item.get("action"));
                    model.set("url", item.get("url"));
                    model.set("tooltip", item.get("tooltip"));
                    
                    var wv = new webView({model:model});
                    if(typeof (self.text_area.$el) != 'undefined'){
                          self.text_area.$el.hide();
                   }     
                    self.content.show(wv);
                    self.trigger('resize',"365px");
                break;
                case "page":
                    model.set("action", item.get("action"));
                    model.set("page", item.get("page"));

                    // Tapamos la pregunta de texto 
                    $("#editLinkLayout").find('.text_box').hide();    

                    // Intetamos meter las miniaturas en el dialog

                    var layout = new IndexLayout(),
                        region = self.content;
                        region.show(layout);

                    $.when(
                        App.request("chapter", {context: this,page:model.get('page')}),
                        ChaptersDropdownViewDeferred()
                    )
                  .done(function (chapter, View) {
                    var filter,
                        thumbnailsView = ThumbnailsView.extend({
                            events: {
                              "tap img": "select"
                            },
                            select: function (page) {
                                var img = $(event.target);

                                img.parents('nav:first').find("img").removeClass('active');

                                if(img.hasClass('active')){
                                    model.set("action", 'page');
                                    model.set("page", '');
                                }else{
                                    img.addClass('active');
                                    model.set("action", 'page');
                                    // En el caso de Mi Libro obtengo el numero de pagina en base al "alt". Es lo que hay...
                                    model.set("page", parseInt($(event.target).attr("data-page"), 10) || parseInt(page.target.alt, 10));
                                }
                                
                                self.trigger("updateItemLink",model);
                                self.trigger("quit");
                            },

                            onRender:function(){
                                _.each(this.model.get('pages'),function(value,index){
                                    if(value.data.page === model.get('page')){
                                        this.$el.find('img').eq(index).addClass('active');
                                    }
                                },this);
                            }
                        }),
                        view = new IndexView({
                            collection: new Backbone.Collection(),
                            itemView: thumbnailsView 
                        });

                    // Para Mi Libro, debo reemplazar la coleccion del libro original por las paginas creadas por mi
                    if(App.mybookPanel.currentView && App.mybookPanel.currentView.pages.currentView.collection){
                      var mybookCollection = App.mybookPanel.currentView.pages.currentView.collection;

                      var pageList = new PageCollection();

                      mybookCollection.each(function(item, index){
                        var pagina = {
                          id: item.get("id"),
                          pages: [{
                            code: item.get("code"),
                            data: {},
                            id: item.get("id"),
                            index: 0,
                            number: item.get("id"),
                            thumbnail: item.get("thumbnail") || (item.get("type") + '.png'),
                            title: item.get("id"), // Este es el mas importante, dado que se traduce en el "alt", el cual se usa como pagina de destino luego.
                            type: item.get("thumbnail") ? null : item.get("type")
                          }],
                          title: item.get("id")
                        };

                        pageList.add(pagina);
                      });

                      var tmpView = thumbnailsView.extend({
                        templateHelpers: function(){
                          var path;
                          // La ubicacion de los thumbnails cambia
                          // para las paginas en blanco
                          if(this.model.get("pages")[0].type){
                            path = configuration.root + 'assets/images/mybook/';
                          } else {
                            path = configuration.thumbnails;
                          }

                          return {
                            path: path
                          };
                        },
                      });

                      var mybookview = new IndexView({
                                              collection: pageList,
                                              itemView: tmpView 
                                          });
                      layout.thumbnails.show(mybookview);
                    } else {
                      // Fuera de Mi Libro muestro libro completo y el selector de unidades
                      View = View.extend({
                        className: "my dropdown",
                        tagName: "h2",

                        initialize: function () {
                          this.model = this.model.clone();
                          this.model.set("title", chapter.get("title"));
                          this.model.set("color", 6);
                          this.model.set("options", this.model.get("options").slice(0));
                          this.model.get("options").push({
                            name: Language("allUnits"),
                            data: {code: "*"}
                          });
                        }
                      });

                      filter = new View({color: 6});

                      filter.on("select", function ($el) {
                        chapterL($el.attr("data-code"),layout);
                      });

                      layout.filter.show(filter);
                      layout.thumbnails.show(view);
                      //layout.buttons.show(new pageView());
                      chapterL(chapter.get("code"),layout); // Esto junta las paginas de a dos
                    }

                    $('#preview a:hover').css('color','#F2942E');
                    layout.$el.css('position','relative');
                    layout.$el.css('margin','0');
                    layout.$el.removeClass("norender");
                    layout.$el.find("button[class='icon-maximize']").remove();
                  })
                  .fail(function () {
                    App.execute("error", language("index.couldNotLoadChapters"));
                  });

                  self.trigger('resize',"90%");
     
                break;
                case 'resource':

                    model.set("action", item.get("action"));
                    model.set("link", item.get("link"));
                    model.set("button", item.get("button"));
                    model.set("dropdown", item.get("dropdown"));

                    // Tapamos la pregunta de texto 
                    $("#editLinkLayout").find('.text_box').hide();    

                    // Intetamos meter las miniaturas en el dialog

                    var ResourceContentView = new DialogView({
                        color: 6,
                        width: '100%',
                        height: '100%',
                        className: "dialog fullscreen",
                        draggable: false
                    }); 

                    var layout = new IndexLayoutResource();

                    $.when(App.request("buttons"), App.request("chapter"), App.request("chapters", {
                        collection: true
                     })).done(function (buttons, current, chapters) {
                        layout.on("render", function () {

                            var iconsetup = [];
                            _.each(buttons, function (item, index) {

                                if(item.class == "library"){
                                    // iconsetup.push({
                                    //     type: item.icon,
                                    //     iconClass: item.class,
                                    //     path: item.path,
                                    //     buttonid: index,
                                    //     active: item.active,
                                    //     name: Language("teachersLibrary"),
                                    //     porunidad: item.porunidad
                                    // });
                                }else{
                                    if(item.class == "links"){
                                        iconsetup.push({
                                            type: item.icon,
                                            iconClass: item.class,
                                            path: item.path,
                                            buttonid: index,
                                            active: item.active,
                                            name: Language("links"),
                                            porunidad: item.porunidad,
                                            iframe: item.iframe
                                        });
                                    }else{
                                        iconsetup.push({
                                            type: item.icon,
                                            iconClass: item.class,
                                            path: item.path,
                                            buttonid: index,
                                            active: item.active,
                                            name: item.name,
                                            porunidad: item.porunidad,
                                            iframe: item.iframe
                                        });
                                    }
                                }


                            });

                            var a = new ListIconView({
                                collection: new Backbone.Collection(iconsetup)
                            });


                            layout.buttons.show(a);

                            var listElementView = ListElementView.extend({
                                events:{
                                  "touch" : "selectResource"
                                },
                                triggers:{

                                },
                                selectResource: function () {

                                    var resource = $(event.target);

                                    if(resource.parent().hasClass('active')){
                                        $(".item-pic").removeClass('active')
                                        model.set("action", 'resource');
                                        model.set("link", '');
                                    }else{
                                       $(".item-pic").removeClass('active')
                                        resource.parent().addClass('active');
                                        
                                        model.set("action", 'resource');
                                        model.set("link", resource.parents('a:first').attr('name'));
                                        model.set("tooltip", this.model.get("tittle"));
                                        model.set("button",$("#editLinkLayout").find('.buttons').find('.active').attr("id"));

                                        var title = $("#editLinkLayout").find('.dropdown').find('a:first').find('span:first').text();
                                        _.each($("#filtercombos a"),function(value,index){
                                            if($(value).attr('title') == title){
                                                model.set("dropdown",$(value).attr('data-code'));
                                            }
                                        }); 
                                    }

                                    self.trigger("updateItemLink",model);
                                    self.trigger("quit");
                                },
                                onRender:function(){
                                    if(this.model.get('link') === model.get('link')){
                                        this.$el.find("a[name='"+this.model.get('link')+"']").find("div:first").addClass('active'); 
                                    }
                                    
                                }
                            });


                            var resourceView = new ListView({
                                collection: new ResourceCollation(),
                                itemView: listElementView
                            });

                            var b = new FiltersContainer({
                                collection: new Backbone.Collection(),
                                color: 6
                            });

                            layout.filters.show(b);

                            var contentDialog = new ContentDialog ({model: new Backbone.Model()});

                            this.controller = new Controller({view:a,resource:resourceView, dropdown: b,dialog: ResourceContentView, contentDialog: contentDialog, loader:Loader, unidad:model.get('dropdown')});

                            layout.elements.show(resourceView);
                            
                        }); 

                        this.layout = (function () {
                            self.content.show(layout);
                            return layout;
                        })();

                        $(".dialog").css('bottom','65px');
                        self.content.$el.css('height','100%');
                        self.$el.css('height','100%');
                        $('#wrapper').find('.bgcolor-5').removeClass("bgcolor-5").addClass("bgcolor-6");

                        if(!$("#" + model.get('button')).hasClass('active')){
                            $("#" + model.get('button')).trigger('touch');
                        }

                        _.each($("#filtercombos a"),function(value,index){
                            if(value.attr('title') == model.get('dropdown')){
                                if(value.hasClass('inactive')){
                                    value.trigger('tap');
                                }
                            }
                        }); 

                    });
                    self.trigger('resize',"90%");

                    break;
                case "cloudFile":

                    model.set("action", item.get("action"));
                    model.set("link", item.get("link"));
                    self.trigger("quit");

                    App.request("av", {type: 'filesystem', _selected: "image", timeout: false})
                        .done(function(data){
                            model.set("action", "cloudFile");
                            // model.set("link", (configuration.rootData + data.image).replace("//", "/"));
                            model.set("link", data.image.replace("//", "/")); // Guardo solo la ruta relativa.
                            self.trigger("updateItemLink", model);
                        });

                    break;                
                default:
                    // Validamos para no duplicar la vista
                    if(self.content.currentView === cv)
                        return;
                    
                        
                    // Escuchamos la elección del tipo de link que se quiere configurar
                    cv.on("itemview:changeView", function (vista, modelo, coleccion) {
                        item.set("action", modelo.model.get("action"));
                        showView(item);
                    });
                    self.content.show(cv); 
                    self.text_area.show(new Marionette.ItemView({template:_.template(Language('linkToolsQuestion'))}));
                    self.trigger('resize',"365px");
                    break;
                }
            }
        };


        //** function paginas **/ 

        var chapterL = function (code,layout) {
              $.when(
                      App.request("chapters", {context: this}),
                      App.request(
                          "is:document:portrait",
                          {boolean: true, context: this}
                      ),
                      App.request("page", {context: this})
                  )
                  .done(function (chapters, isDocumentPortrait, page) {
                    var pages = (code === "*") ? chapters.pages() :
                            chapters.findWhere({code: code}).get("pages"),
                        thumbnails = [];

                    pages = pages.groupBy(function (value) {
                      return parseInt(
                          value.get("id") / (isDocumentPortrait + 1),
                          10
                      );
                    });

                    _.each(pages, function (value) {
                      var collection = new Backbone.Collection(value);

                      thumbnails.push({
                        id: collection.first().get("id"),
                        title: collection.pluck("title").join("-"),

                        pages: value.map(function (rvalue) {
                          rvalue = rvalue
                              .clone()
                              .set("data", {page: rvalue.get("id")});

                          if (rvalue.get("id") === page) {
                            rvalue.set("current", true);
                          }

                          return rvalue.toJSON();
                        })
                      });
                    });

                    layout.thumbnails.currentView.collection.set(thumbnails);
                    

                  });
            };
          
        
        
        /*
         * Contenedor de todas mis vistas de EditLink
         */
        var layout = Backbone.Marionette.Layout.extend({
            template: JST["form-list-link"],
            id: "editLinkLayout",
            initialize: function(){
                var self = this;
                this.content.on("show", function(){
                    this.currentView.on("quit", function(){
                        self.trigger("quit");
                    });
                    this.currentView.on("updateItemLink", function(_model){
                        self.trigger("updateItemLink", _model);
                    });
                });  
            },
            regions: {
                text_area: ".text_box",
                content: ".js-content"
            },
            onShow: function(){
                _.bind(init, this)();
            }
        });
        
        return layout;

    }
);
// noinspection JSHint
/**
 * Figura de la imagen
 * @class Tampon
 * @extends Shape
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/tampon',[
      // libraries
      "underscore",
      "backbone",
      "marionette",
      // configuration
      "configuration",
      // language
      "language",
      // models
      "modules/viewer/models/shape",
      // views
      "views/dialog",
      // wtf is that below?
      "modules/tools/link-forms/EditLink"
    ],
    function (_, Backbone, Marionette, configuration, language, Shape, DialogView, LinkEditor) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Tampon = function (options, scale, canvas) {
        // Default properties of KineticJS
        _.defaults(options || (options = {}), {
          className: "Tampon",
          width: 100,
          height: 100,
          innerX: 50,
          innerY: 50,
          dimensionsMiddle: true,
          anchorsDraggable: false
        });
        Shape.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          this.innerX = options.innerX;
          this.innerY = options.innerY;
          // Indica si la posición X e Y son la posición central o no
          this.dimensionsMiddle = options.dimensionsMiddle;
          this.anchorsDraggable = options.anchorsDraggable;
        }
      };

      Tampon.prototype = _.extend({}, Shape.prototype, {
        /**
         * Función que realiza la redimensión (por defecto no hacer nada).
         * @method resize
         */
        resize: function () {},

        /**
         * Función que realiza el dibujado.
         * @method draw
         * @param {Object} context
         * @param v
         * @param end
         */
        draw: function (context, v, end) {
          var that = this,
            view = v;
          if (this.src) {
            if (!this.img) {
              var img = this.img = new Image();
              this.img.src = configuration.root + this.src;
              this.img.onload = function() {
                if (that.dimensionsMiddle) {
                  context.drawImage(
                    img, // Imagen
                    that.x - that.width / 2, // Posicion X
                    that.y - that.height / 2, // Posicion Y
                    that.width, // Ancho
                    that.height // Alto
                  );
                }
                else {
                  context.drawImage(
                    img, // Imagen
                    that.x, // Posicion X
                    that.y, // Posicion Y
                    that.width, // Ancho
                    that.height // Alto
                  );
                }
                img = null;
                _.each(that.anchors, function(anchor) {
                  anchor.draw(context);
                });
                // Indicar que se ha acabado
                if (_.isFunction(end)) {
                  end(view);
                }
              }
            }
            else {
              if (this.dimensionsMiddle) {
                context.drawImage(
                  this.img, // Imagen
                  that.x - that.width / 2, // Posicion X
                  that.y - that.height / 2, // Posicion Y
                  that.width, // Ancho
                  that.height // Alto
                );
              }
              else {
                context.drawImage(
                  this.img, // Imagen
                  that.x, // Posicion X
                  that.y, // Posicion Y
                  that.width, // Ancho
                  that.height // Alto
                );
              }
              _.each(that.anchors, function(anchor) {
                anchor.draw(context);
              });
            }
          }
        },

        /**
         * Función que indica si el objeto ha sido seleccionado.
         * @method isSelected
         * @param {Object} context
         * @param {Object} point
         * @return {boolean}
         */
        isSelected: function(context, point, busquedaExhaustiva) {
          var minX, maxX, minY, maxY;
          if (this.dimensionsMiddle) {
            var width = this.width / 2,
              height = this.height / 2;
            minX = Math.min(this.x-width, this.x+width);
            maxX = Math.max(this.x-width, this.x+width);
            minY = Math.min(this.y-height, this.y+height);
            maxY = Math.max(this.y-height, this.y+height);
          }
          else {
            minX = Math.min(this.x, this.x+this.width);
            maxX = Math.max(this.x, this.x+this.width);
            minY = Math.min(this.y, this.y+this.height);
            maxY = Math.max(this.y, this.y+this.height);
          }
          return (busquedaExhaustiva && point.x >= minX && point.x <= maxX && point.y >= minY && point.y <= maxY)
            || _.find(this.anchors, function(anchor) { return anchor.isSelected(context, point, true); });
        },

        /**
         * Función que realiza la especialización de la selección para la flecha del cuadro de texto.
         * @method select
         */
        select: function () {
          var minX, maxX, minY, maxY;
          if (this.dimensionsMiddle) {
            var width = this.width / 2,
              height = this.height / 2;
            minX = Math.min(this.x-width, this.x+width);
            maxX = Math.max(this.x-width, this.x+width);
            minY = Math.min(this.y-height, this.y+height);
            maxY = Math.max(this.y-height, this.y+height);
          }
          else {
            minX = Math.min(this.x, this.x+this.width);
            maxX = Math.max(this.x, this.x+this.width);
            minY = Math.min(this.y, this.y+this.height);
            maxY = Math.max(this.y, this.y+this.height);
          }
          if (this.anchors.length > 0) {
            // Todos los anclas tienen el mismo tamaño
            var offset = this.anchors[0].width / 2;
            this.anchors[0].x = this.anchors[3].x = minX - offset;
            this.anchors[0].y = this.anchors[1].y = minY - offset;
            this.anchors[1].x = this.anchors[2].x = maxX - offset;
            this.anchors[2].y = this.anchors[3].y = maxY - offset;
          }
          else {
            this.anchor({
              draggable: this.anchorsDraggable,
              pos: "lt",
              x: minX,
              y: minY,
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              draggable: this.anchorsDraggable,
              pos: "rt",
              x: maxX,
              y: minY,
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              draggable: this.anchorsDraggable,
              pos: "rb",
              x: maxX,
              y: maxY,
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              draggable: this.anchorsDraggable,
              pos: "lb",
              x: minX,
              y: maxY,
              color: "#009ee0",
              scale: this.scale
            });
          }
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Shape.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            innerX: this.innerX, // this.object.position().x
            innerY: this.innerY, // this.object.position().y
            color: this.fill
          });

          return comprimido;
        },

        /**
         * Función que procesa los eventos que llegan al objeto
         * @param {String} type campo tipo del evento (touchstart, mousedown...)
         * @param {Object} center punto donde se realiza el evento
         * @method evento
         * @returns {integer}
         */
        evento: function(type, center) {
          return this.eventTriggered(type,center,false);
        }
      });


      return Tampon;
    }
);
// noinspection JSHint
/**
 * Clase de la figura Clip.
 * @class Clip
 * @extends Tampon
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/clip',[
      // libraries
      "underscore",
      "backbone",
      "marionette",
      // configuration
      "configuration",
      // language
      "language",
      // models
      "modules/viewer/models/shape/tampon",
      // views
      "views/dialog",
      // wtf is that below?
      "modules/tools/link-forms/EditLink"
    ],
    function (_, Backbone, Marionette, configuration, language, Tampon, DialogView, LinkEditor) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Clip = function (options, scale, canvas) {
        var that = this;
        this.scale = scale || 1;
        _.defaults(options || (options = {}), {
          className: "Clip",
          src: "assets/images/draw/link.svg",
          width: 100,
          height: 100
        });
        Tampon.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          /**
           * Enlace relativo a la imagen.
           *
           * @property src
           * @type {String}
           */
          this.src = options.src;
          this.copy(this, options);
        }
      };

      Clip.prototype = _.extend({}, Tampon.prototype, {
        /**
         * Función que abre una ventana o popup en el caso de algunos elementos. Además se bloquea la figura.
         * @method openWindow
         */
        openWindow: function (fn, view) {
          // Vista modal para mostrar los formularios
          var self = this;

          // Vista modal para mostrar los formularios
          var dv = new DialogView({
            color: 6,
            title: language("createLink"),
            width: '90%',
            height: "auto"
          });

          var objModel = new Backbone.Model({
            action: this.action,
            url: this.url,
            tooltip: this.tooltip,
            page: this.page,
            link: this.link,
            button: this.button,
            dropdown: this.dropdown
          });

          var le = new LinkEditor({
            item: objModel
          });

          le.on("updateItemLink", function (_model) {

            switch (_model.get("action")) {
              case "web":
                self.action = _model.get("action");
                self.url = _model.get("url");
                self.tooltip = _model.get("tooltip");
                break;

              case "page":
                self.action = _model.get("action");
                self.page = _model.get("page");
                var token= ": ",
                    str = _model.get("tooltip"),
                    pos = str.lastIndexOf(": "),
                    tooltip = language("pageLink");
                if (pos > 0) {
                  tooltip += + str.substr(str.lastIndexOf(": "));
                }
                self.tooltip = tooltip;
                break;

              case "resource":
                self.action = _model.get("action");
                self.link = _model.get("link").replace(configuration.rootData,""); // Fix Ale
                self.button = _model.get("button");
                self.dropdown = _model.get("dropdown");
                self.tooltip = _model.get("tooltip");
                break;

              case "cloudFile":
                self.action = _model.get("action");
                self.link = _model.get("link");
                break;
            }
            fn.call(view);
          });

          dv.show(le);
        },

        /**
         * Función que copia los datos del enlace del obj2 al obj1
         * @param obj1 - source object
         * @param obj2 - destination object
         * @method copy
         */
        copy: function(obj1, obj2) {
          switch (obj2.action) {
            case "web": {
              obj1.action = obj2.action;
              obj1.url = obj2.url;
              obj1.tooltip = obj2.tooltip;
              break;
            }
            case "page": {
              obj1.action = obj2.action;
              obj1.page = obj2.page;
              obj1.tooltip = obj2.tooltip;
              break;
            }
            case "resource": {
              obj1.action = obj2.action;
              obj1.link = obj2.link;
              obj1.button = obj2.button;
              obj1.dropdown = obj2.dropdown;
              obj1.tooltip = obj2.tooltip;
              break;
            }
            case "cloudFile": {
              obj1.action = obj2.action;
              obj1.link = obj2.link;
              break;
            }
          }
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Tampon.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          this.copy(comprimido.attrs, this);

          return comprimido;
        },

        /**
         * Copia los datos del objeto actual a la figura que se pasa como parámetro
         * @param shape objeto al que copiar los datos
         * @param boolean flagPosicion si es true o undefined se copia la posición con offset en caso de false no se copia
         * @method copyData
         */
        copyData: function(shape, flagPosicion) {
          // Se copian las dimesiones
          Tampon.prototype.copyData.call(this,shape,flagPosicion);
          this.copy(shape,this);
        }
      });

      return Clip;
    }
);
// noinspection JSHint
/**
 * Clase para la figura de la estampa de la cruce.
 * @class Cross
 * @extends Tampon
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
  'modules/viewer/models/shape/cross',[
    // libraries
    "underscore",
    // configuration
    "configuration",
    // models
    "modules/viewer/models/shape/tampon"
  ],
  function (_, configuration, Tampon) {
    "use strict";

    /**
     * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
     * @param options
     * @param scale
     * @param canvas
     * @constructor
     */
    var Cross = function (options, scale, canvas) {
      var that = this;
      this.scale = scale || 1;
      _.defaults(options || (options = {}), {
        className: "Cross",
        src: "assets/images/draw/cross.svg",
        width: 100,
        height: 100
      });
      // En el caso de dispositivos móviles no hace falta aplicar una escala
      App.request("is:device").done(function () {
        that.height = that.width;
      });
      Tampon.call(this, options, scale, canvas);
      if ("undefined" !== typeof(options))
      {
        /**
         * Enlace relativo a la imagen.
         *
         * @property src
         * @type {String}
         */
        this.src = options.src;
      }
    };

    Cross.prototype = _.extend({}, Tampon.prototype, {});

    return Cross;
  }
);
// noinspection JSHint
/**
 * Clase de para el trazo libre
 * @class Doodle
 * @extends Line
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/doodle',["underscore", "modules/viewer/models/shape/line"],
    function (_, Line) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Doodle = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "Doodle"
        });
        Line.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          /**
           * Relleno de la figura.
           *
           * @property fill
           * @type {String}
           */
          this.fill = options.color || options.fill;
          /**
           * Ancho del borde del óvalo.
           *
           * @property strokeWidth
           * @type {Integer}
           */
          this.strokeWidth = options.strokeWidth;
          /**
           * Puntos de cada uno de los puntos de trazo.
           *
           * @property points
           * @type {Arrat}
           */
          this.points = _.map(options.points || [], function(point){return point * this.scale;}, this);
          /**
           * La coordenada de las abcisas mínima.
           * @property xMin
           * @type {Number}
           * @default 0
           */
          /**
           * La coordenada de las abcisas máxima.
           * @property xMax
           * @type {Number}
           * @default 0
           */
          /**
           * La coordenada de las ordenadas mínima.
           * @property yMin
           * @type {Number}
           * @default 0
           */
          /**
           * La coordenada de las ordenadas máxima.
           * @property yMax
           * @type {Number}
           * @default 0
           */
          this.xMin = this.xMax = this.yMin = this.yMax = 0; // Se inician las casillas al punto de referencia (0,0)
          /**
           *
           *    xMin           xMax
           *      /--------------/
           * yMin -   # ------------ #
           *      |   |              |
           *      |   |              |
           *      |   |        · <---|--- (0,0) = (this.x, this.y)
           *      |   |              |
           * yMax -   # ------------ #
           *
           */
        }
      };

      Doodle.prototype = _.extend({}, Line.prototype, {
        /**
         * Función que realiza el camino del dibujado.
         * @method drawPath
         * @param {Object} context
         */
        drawPath: function(context) {
          context.beginPath();
          context.moveTo(this.x, this.y);
          for(var i = 0; i < this.points.length; i += 2) {
            context.lineTo(
              this.points[i] + this.x,
              this.points[i+1] + this.y
            );
          }
          context.lineWidth = this.strokeWidth * this.scale;
        },

        /**
         * Función que realiza la redimensión.
         * @method resize
         * @param {Integer} width
         * @param {Integer} height
         */
        resize: function (width, height) {
          this.xMin = Math.min(this.xMin, width);
          this.xMax = Math.max(this.xMax, width);
          this.yMin = Math.min(this.yMin, height);
          this.yMax = Math.max(this.yMax, height);
          /**
           *
           *    xMin           xMax
           *      /--------------/
           * yMin -   # ------------ #
           *      |   |              |
           *      |   |              |
           *      |   |        · <---|--- (0,0) = (this.x, this.y)
           *      |   |              |
           * yMax -   # ------------ #
           *
           */
          // En caso de no estar seleccionado
          if (!this.anchors.length) {
            this.points.push(width);
            this.points.push(height);
          }
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Line.prototype.compress.call(this);

          comprimido.attrs.points = _.map(this.points, function(point){
            return point / this.scale;
          }, this);

          return comprimido;
        },

        /**
         * Función que indica si el objeto ha sido seleccionado.
         * @method isSelected
         * @param {Object} context
         * @param {Object} point
         * @return {boolean}
         */
        isSelected: function(context, point, busquedaExhaustiva) {
          var touch = false;
          if (!this.xMin && !this.yMin && !this.xMax && !this.yMax) {
            _.each(this.points, function (e, i) {
              if ((i % 2) === 0) {
                this.xMin = Math.min(this.xMin, e);
                this.xMax = Math.max(this.xMax, e);
              } else {
                this.yMin = Math.min(this.yMin, e);
                this.yMax = Math.max(this.yMax, e);
              }
            }, this);
          }
          if (busquedaExhaustiva) {
            touch = point.x >= this.x + this.xMin && point.x <= this.x + this.xMax && point.y >= this.y + this.yMin && point.y <= this.y + this.yMax;
          }
          return touch || _.find(this.anchors, function(anchor) { return anchor.isSelected(context, point, true); });
        },

        /**
         * Función que realiza la especialización de la selección para la flecha del cuadro de texto.
         * @method select
         */
        select: function () {
          if (this.anchors.length > 0) {
            // Todos los anclas tienen el mismo ancho y alto
            var offset = this.anchors[0].width / 2;
            this.anchors[0].x = this.anchors[3].x = this.x + this.xMin - offset;
            this.anchors[0].y = this.anchors[1].y = this.y + this.yMin - offset;
            this.anchors[1].x = this.anchors[2].x = this.x + this.xMax - offset;
            this.anchors[2].y = this.anchors[3].y = this.y + this.yMax - offset;
          }
          else {
            this.anchor({
              draggable: false,
              x: this.x + this.xMin,
              y: this.y + this.yMin,
              pos: "lt",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              draggable: false,
              x: this.x + this.xMax,
              y: this.y + this.yMin,
              pos: "rt",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              draggable: false,
              x: this.x + this.xMax,
              y: this.y + this.yMax,
              pos: "rb",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              draggable: false,
              x: this.x + this.xMin,
              y: this.y + this.yMax,
              pos: "lb",
              color: "#009ee0",
              scale: this.scale
            });
          }
        }
      });

      return Doodle;
    }
);
// noinspection JSHint
/**
 * Clase para la nota
 * @class Edit
 * @extends Tampon
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/edit',[
      // libraries
      "underscore",
      "backbone",
      "marionette",
      // configuration
      "configuration",
        // language
      "language",
      // models
      "modules/viewer/models/shape/tampon",
        // views
      "views/dialog",
      "views/text.editor"
    ],
    function (_, Backbone, Marionette, configuration, language, Tampon, DialogView, TextEditorView) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Note = function (options, scale, canvas) {
        this.scale = scale || 1;
        _.defaults(options || (options = {}), {
          className: "Note",
          src: "assets/images/draw/note.svg",
          width: 100,
          height: 100 /* * this.scale */
        });
        Tampon.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          /**
           * Enlace relativo a la imagen.
           *
           * @property src
           * @type {String}
           */
          this.src = options.src;
          /**
           * Texto en bruto del cuadro de texto.
           * @property raw
           * @type {String}
           */
          this.raw = options.raw;
          /**
           * Texto parseado (sin código html).
           * @property text
           * @type {String}
           */
          this.text = options.text;
        }
      };

      Note.prototype = _.extend({}, Tampon.prototype, {
        /**
         * Función que abre una ventana o popup en el caso de algunos elementos. Además se bloquea la figura.
         * @method openWindow
         */
        openWindow: function (fn, view) {
          var that = this;

          Tampon.prototype.openWindow.call(this);

          //AV
          App.request("is:av", {context: this})
            .done(function(){
              App.request("av", {type: 'openTextEditor', content: that.raw, timeout: false})
                .done(function(data){
                  var regex = /(<([^>]+)>)/ig;
                  that.text = data._html.replace(regex, "").replace("&nbsp;", "");
                  that.raw = data._html;
                  // Llamar a la función que se pasa como parámetro para que guarde en el historial el estado
                  App.request("is:device")
                    .fail(function() {
                      fn.call(view);
                    });
                });
            })
            .fail(function(){
              var dv = new DialogView({
                color: 6,
                title: language("editText"),
                width: 365,
                height: 280
              });

              dv.show(new TextEditorView({
                model: new Backbone.Model({
                  raw: that.raw,
                  text: that.text
                }),
                modelEvents: {
                  "change": function () {
                    that.raw = this.model.get("raw");
                    that.text = this.model.get("text");
                    // Llamar a la función que se pasa como parámetro para que guarde en el historial el estado
                    App.request("is:device")
                      .fail(function() {
                        fn.call(view);
                      });
                  }
                }
              }));
            });
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Tampon.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            raw: this.raw,
            text: this.text
          });

          return comprimido;
        },

        /**
         * Copia los datos del objeto actual a la figura que se pasa como parámetro
         * @param shape objeto al que copiar los datos
         * @param boolean flagPosicion si es true o undefined se copia la posición con offset en caso de false no se copia
         * @method copyData
         */
        copyData: function(shape, flagPosicion) {
          // Se copian las dimesiones
          Tampon.prototype.copyData.call(this,shape,flagPosicion);
          // Se copia el texto
          shape.raw = this.raw;
          shape.text = this.text;
          shape.src = this.src;
        }
      });

      return Note;
    }
);
// noinspection JSHint
/**
 * Clase para el óvalo inverso relleno
 * @class FillCircleInverse
 * @extends Circle
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/fill-circle-inverse',["underscore", "modules/viewer/models/shape/circle"],
    function (_, Circle) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var FillCircleInverse = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "FillCircleInverse",
          fill: "#FFF",
          strokeOpacity: 0.15,
          opacity: 1,
          order: 2 // Máscaras
        });
        options.color = "#000000";
        this.strokeOpacity = options.strokeOpacity;
        Circle.call(this, options, scale, canvas);
      };

      FillCircleInverse.prototype = _.extend({}, Circle.prototype, {
        /**
         * Función que realiza el dibujado.
         * @method draw
         * @param {Object} context
         */
        draw: function (context) {
          if (this.isInsideCanvas()) {
            var x = this.x,
              y = this.y,
              width = this.width,
              height = this.height;
            if (width < 0) {
              x = this.x + this.width;
              width = -this.width;
            }
            if (height < 0) {
              y = this.y + this.height;
              height = -this.height;
            }
            context.save();
            context.beginPath();
            context.translate(x + width / 2, y + height / 2);
            context.scale(1, height / width);
            context.arc(0, 0, (Math.abs(width) / 2), 0, 2 * Math.PI);
            context.restore();
            if (this.anchors.length > 0) {
              var alpha = context.globalAlpha;
              context.globalAlpha = this.strokeOpacity;
              context.strokeStyle = this.color;
              context.lineWidth = this.strokeWidth * this.scale;
              context.stroke();
              context.globalAlpha = alpha;
            }
            if (this.opacity == 1) {
              context.globalAlpha = this.opacity;
            }
            else {
              context.globalAlpha = this.opacity * this.scale;
            }
            // Pintar un rectángulo en sentido antihorario (lo que hace que al pintar figuras en sentido horario en su interior anule el pintado)
            /**
             * FIX: Se añade un tamaño extra debido a que se aplica una escala aveces incorrecta (x100).
             */
            context.moveTo(0,0);
            context.lineTo(0,this.canvas.height*100);
            context.lineTo(this.canvas.width*100,this.canvas.height*100);
            context.lineTo(this.canvas.width*100,0);
            context.fillStyle = this.fill;
            context.fill();
            context.closePath();
            _.each(this.anchors, function(anchor) {
              anchor.draw(context);
            });
          }
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Circle.prototype.compress.call(this),
              object = _.clone(comprimido.attrs); // Se duplica el objeto para que no haya recursión en la definición

          // ===========================================================================
          // Se hace esto para mantener retrocompatibilidad con la versión con KineticJS
          object.x += comprimido.attrs.width / 2;
          object.y += comprimido.attrs.height / 2;
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            circle: {
              className: "Circle",
              attrs: object,
              children: []
            }
          });
          // ===========================================================================

          return comprimido;
        }
      });

      return FillCircleInverse;
    }
);
// noinspection JSHint
/**
 * Figura de la óvalo inverso transparente
 * @class EmptyCircleInverse
 * @extends FillCircleInverse
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/transparent-circle-inverse',["underscore", "modules/viewer/models/shape/fill-circle-inverse"],
    function (_, FillCircleInverse) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var EmptyCircleInverse = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          // ======================
          // Lo único que se añade es la transparencia y el nombre
          className: "EmptyCircleInverse",
          opacity: 0.7
          // ======================
        });
        FillCircleInverse.call(this,options,scale,canvas);
      };

      EmptyCircleInverse.prototype = _.extend({}, FillCircleInverse.prototype, {});

      return EmptyCircleInverse;
    }
);
// noinspection JSHint
/**
 * Figura del óvalo transparente
 * @class EmptyCircle
 * @extends Circle
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/transparent-circle',["underscore","modules/viewer/models/shape/circle"],
    function (_, Circle) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var EmptyCircle = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "EmptyCircle",
          fill : "#FFF",
          stroke: 0,
          hasBorder: false,
          opacity: 0.7,
          strokeOpacity: 0.15,
          order: 2 // Máscaras
        });
        // Se pone este color para cuando se selecciona la máscara al cual luego se le aplica una pesada transparencia
        options.color = "#000000";
        this.strokeOpacity = options.strokeOpacity;
        Circle.call(this,options,scale,canvas);
      };

      EmptyCircle.prototype = _.extend({}, Circle.prototype, {});

      return EmptyCircle;
    }
);
// noinspection JSHint
/**
 * Clase para el rectángulo inverso relleno
 * @class FillSquareInverse
 * @extends Square
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/fill-square-inverse',["underscore", "modules/viewer/models/shape/square"],
    function (_, Square) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var FillSquareInverse = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "FillSquareInverse",
          fill: "#FFF",
          strokeOpacity: 0.15,
          opacity: 1,
          order: 2 // Máscaras
        });
        options.color = "#000000";
        this.strokeOpacity = options.strokeOpacity;
        Square.call(this, options, scale, canvas);
      };

      FillSquareInverse.prototype = _.extend({}, Square.prototype, {
        /**
         * Función que realiza el dibujado.
         * @method draw
         * @param {Object} context
         */
        draw: function (context) {
          if (this.isInsideCanvas()) {
            var alpha = context.globalAlpha,
                x = this.x,
                y = this.y,
                width = this.width,
                height = this.height;
            if (width < 0) {
              x = this.x + this.width;
              width = -this.width;
            }
            if (height < 0) {
              y = this.y + this.height;
              height = -this.height;
            }
            context.beginPath();
            if (this.opacity == 1) {
              context.globalAlpha = this.opacity;
            }
            else {
              context.globalAlpha = this.opacity * this.scale;
            }
            // Pintar un rectángulo en sentido antihorario (lo que hace que al pintar figuras en sentido horario en su interior anule el pintado)
            /**
             * FIX: Se añade un tamaño extra debido a que se aplica una escala aveces incorrecta (x100).
             */
            context.moveTo(0,0);
            context.lineTo(0,this.canvas.height*100);
            context.lineTo(this.canvas.width*100,this.canvas.height*100);
            context.lineTo(this.canvas.width*100,0);
            context.rect(x,y,width,height);
            context.lineWidth = 0;
            context.fillStyle = this.fill;
            context.fill();
            context.closePath();
            if (this.anchors.length > 0) {
              // Recuadro de fuera
              context.globalAlpha = this.strokeOpacity;
              context.strokeStyle = this.color;
              context.lineWidth = this.strokeWidth * this.scale;
              context.strokeRect(x,y,width,height);
            }
            context.globalAlpha = alpha;
            _.each(this.anchors, function(anchor) {
              anchor.draw(context);
            });
          }
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Square.prototype.compress.call(this);

          // ===========================================================================
          // Se hace esto para mantener retrocompatibilidad con la versión con KineticJS
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            square: {
              className: "Square",
              attrs: _.clone(comprimido.attrs), // Se duplica el objeto para que no haya recursión en la definición
              children: []
            }
          });
          // ===========================================================================

          return comprimido;
        }
      });

      return FillSquareInverse;
    }
);
// noinspection JSHint
/**
 * Figura de la rectángulo inverso transparente
 * @class EmptySquareInverse
 * @extends FillSquareInverse
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/transparent-square-inverse',["underscore", "modules/viewer/models/shape/fill-square-inverse"],
    function (_, FillSquareInverse) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var EmptySquareInverse = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          // ======================
          // Lo único que se añade es la transparencia y el nombre
          className: "EmptySquareInverse",
          opacity: 0.7
          // ======================
        });
        FillSquareInverse.call(this,options,scale,canvas);
      };

      EmptySquareInverse.prototype = _.extend({}, FillSquareInverse.prototype, {});

      return EmptySquareInverse;
    }
);
// noinspection JSHint
/**
 * Figura de la rectángulo inverso transparente
 * @class EmptySquare
 * @extends Square
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/transparent-square',["underscore","modules/viewer/models/shape/square"],
    function (_, Square) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var EmptySquare = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "EmptySquare",
          fill : "#FFF",
          stroke: 0,
          hasBorder: false,
          opacity: 0.7,
          strokeOpacity: 0.15,
          order: 2 // Máscaras
        });
        // Se pone este color para cuando se selecciona la máscara al cual luego se le aplica una pesada transparencia
        options.color = "#000000";
        this.strokeOpacity = options.strokeOpacity;
        Square.call(this,options,scale,canvas);
      };

      EmptySquare.prototype = _.extend({}, Square.prototype, {});

      return EmptySquare;
    }
);
// noinspection JSHint
/**
 * Clase para el óvalo relleno
 * @class FillCircle
 * @extends Circle
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/fill-circle',["underscore","modules/viewer/models/shape/circle"],
    function (_, Circle) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      function FillCircle(options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "FillCircle",
          fill: "#FFFFFF",
          hasBorder: false,
          opacity: 1,
          strokeOpacity: 0.15,
          order: 2 // Máscaras
        });
        // Se pone este color para cuando se selecciona la máscara al cual luego se le aplica una pesada transparencia
        options.color = "#000000";
        this.strokeOpacity = options.strokeOpacity;
        Circle.call(this, options, scale, canvas);
      };

      FillCircle.prototype = _.extend({}, Circle.prototype, {});

      return FillCircle;
    }
);
// noinspection JSHint
/**
 * Clase para el rectángulo rellenado
 * @class FillCircleInverse
 * @extends Square
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/fill-square',["underscore","modules/viewer/models/shape/square"],
    function (_, Square) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      function FillSquare(options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "FillSquare",
          fill: "#FFFFFF",
          hasBorder: false,
          opacity: 1,
          strokeOpacity: 0.15,
          order: 2 // Máscaras
        });
        // Se pone este color para cuando se selecciona la máscara al cual luego se le aplica una pesada transparencia
        options.color = "#000000";
        this.strokeOpacity = options.strokeOpacity;
        Square.call(this, options, scale, canvas);
      };

      FillSquare.prototype = _.extend({}, Square.prototype, {});

      return FillSquare;
    }
);
// noinspection JSHint
/**
 * Clase para el resaltado
 * @class Highlight
 * @extends Square
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/highlight',["marionette", "underscore", "modules/viewer/models/shape/square"],
    function (Marionette, _, Square) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Highlight = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "Highlight",
          strokeWidth: 0,
          opacity: 0.7,
          order: 0 // Grupo
        });
        Square.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          this.scale = scale || 1;
          // Copia del color del rectángulo porque al seleccionar cambia de color (y si se deselecciona así se puede restaurar)
          this.fill2 = this.fill = options.color;
          this.opacity = options.opacity;
          this.strokeWidth = options.strokeWidth;
        }
      };

      Highlight.prototype = _.extend({}, Square.prototype, {
        /**
         * Función que realiza el dibujado.
         * @method draw
         * @param {Object} context
         */
        draw: function(context) {
          if (this.opacity > 0) {
            // convierto Hex a RGB
            var color = parseInt(this.fill.substring(1), 16); // extraigo en un array (R G B)
            context.beginPath();
            context.rect(
              this.x,
              this.y,
              this.width,
              this.height
            );
            context.fillStyle = "rgba(" + ((color >> 16) & 255) + ", " + ((color >> 8) & 255) + ", " + (color & 255) + ", " + this.opacity * this.scale + ")";
            context.lineWidth = this.strokeWidth * this.scale;
            context.fill();
          }
        },

        /**
         * Función de seleccionado de la figura.
         * @method select
         */
        select: function() {
          this.fill = "#32B6FF";
        },

        /**
         * Función de deseleccionado de la figura. Que simplemente restaura el color del resaltado.
         * @method deselect
         */
        deselect: function() {
          this.fill = this.fill2;
        },

        /**
         * Función que indica si el objeto ha sido seleccionado.
         * @method isSelected
         * @param {Object} context
         * @param {Object} point
         * @return {boolean}
         */
        isSelected: function(context, point) {
          var minX = this.x, minY = this.y, maxX = this.x+this.width, maxY = this.y+this.height;
          if (minX > maxX) { minX = maxX; maxX = this.x; }
          if (minY > maxY) { minY = maxY; maxY = this.y; }
          return point.x >= minX &&  point.x <= maxX &&  point.y >= minY &&  point.y <= maxY;
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function() {
          // Obtener el esqueleto de la compresión
          var comprimido = Square.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            opacity: this.opacity,
            color: this.fill
          });

          return comprimido;
        },

        /**
         * Función que procesa los eventos que llegan al objeto
         * @param {String} type campo tipo del evento (touchstart, mousedown...)
         * @param {Object} center punto donde se realiza el evento
         * @method evento
         * @returns {integer}
         */
        evento: function(type, center) {
          switch (type) {
            case "mouseevent": {

              break;
            }
            default: {
              // do nothing
              break;
            }
          }
        },

        /**
         * Función que indica si hay que limpiar el canvas para la figura.
         * @method unDraw
         * @returns {boolean}
         */
        unDraw: function() { return true; /* Hay que limpiar el canvas */ }
      });

      return Highlight;
    }
);
// noinspection JSHint
/**
 * Figura de la rectángulo con enlace
 * @class LinkSquare
 * @extends Square
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/link-square',[
      // libraries
      "underscore",
      "backbone",
      "marionette",
      // language
      "language",
      // models
      "modules/viewer/models/shape/square",
      // views
      "views/dialog",
      "modules/tools/link-forms/EditLink"
    ],
    function (_, Backbone, Marionette, language, Square, DialogView, LinkEditor) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var LinkSquare = function (options, scale, canvas, nofill) {
        _.defaults(options || (options = {}), {
          className: "LinkSquare",
          fill: "#F2942E",
          opacity: 0.5,
          strokeWidth: 0
        });
        Square.call(this, options, scale, canvas);
        if (nofill === true) {
            this.color = "transparent";
            this.fill = "transparent";
        }
        if (typeof(options) != "undefined") {
          this.copy(this, options);
        }
      };

      LinkSquare.prototype = _.extend({}, Square.prototype, {
        /**
         * Función que abre una ventana o popup en el caso de algunos elementos. Además se bloquea la figura.
         * @method openWindow
         */
        openWindow: function (fn, view) {
          // Vista modal para mostrar los formularios
          var self = this;

          // Vista modal para mostrar los formularios
          var dv = new DialogView({
            color: 6,
            title: language("createLink"),
            width: '90%',
            height: "auto"
          });

          var objModel = new Backbone.Model({
            action: this.action,
            url: this.url,
            tooltip: this.tooltip,
            page: this.page,
            link: this.link,
            button: this.button,
            dropdown: this.dropdown
          });

          var le = new LinkEditor({
            item: objModel
          });

          le.on("updateItemLink", function (_model) {

            switch (_model.get("action")) {
              case "web":
                self.action = _model.get("action");
                self.url = _model.get("url");
                self.tooltip = _model.get("tooltip");
                break;

              case "page":
                self.action = _model.get("action");
                self.page = _model.get("page");
                self.tooltip = _model.get("tooltip");
                break;

              case "resource":
                self.action = _model.get("action");
                self.link = _model.get("link").replace(configuration.rootData,""); // Fix Ale
                self.button = _model.get("button");
                self.dropdown = _model.get("dropdown");
                self.tooltip = _model.get("tooltip");
                break;

              case "cloudFile":
                self.action = _model.get("action");
                self.link = _model.get("link");
                break;
            }
            fn.call(view);
          });

          dv.show(le);
        },

        /**
         * Función que copia los datos del enlace del obj2 al obj1
         * @param obj1 - source object
         * @param obj2 - destination object
         * @method copy
         */
        copy: function(obj1, obj2) {
          switch (obj2.action) {
            case "web": {
              obj1.action = obj2.action;
              obj1.url = obj2.url;
              obj1.tooltip = obj2.tooltip;
              break;
            }
            case "page": {
              obj1.action = obj2.action;
              obj1.page = obj2.page;
              obj1.tooltip = obj2.tooltip;
              break;
            }
            case "resource": {
              obj1.action = obj2.action;
              obj1.link = obj2.link;
              obj1.button = obj2.button;
              obj1.dropdown = obj2.dropdown;
              obj1.tooltip = obj2.tooltip;
              break;
            }
            case "cloudFile": {
              obj1.action = obj2.action;
              obj1.link = obj2.link;
              break;
            }
          }
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Square.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          this.copy(comprimido.attrs, this);

          return comprimido;
        },

        /**
         * Copia los datos del objeto actual a la figura que se pasa como parámetro
         * @param shape objeto al que copiar los datos
         * @param boolean flagPosicion si es true o undefined se copia la posición con offset en caso de false no se copia
         * @method copyData
         */
        copyData: function(shape, flagPosicion) {
          // Se copian las dimesiones
          Square.prototype.copyData.call(this,shape,flagPosicion);
          this.copy(shape,this);
        }
      });

      return LinkSquare;
    }
);
// noinspection JSHint
/**
 * Figura de la flecha (para el cuadro de texto).
 * @class TextArrow
 * @extends Arrow
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
  */
define(
    'modules/viewer/models/shape/arrow.text',["underscore", "modules/viewer/models/shape/arrow"],
    function (_, Arrow) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var TextArrow = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "TextArrow",
          lineCap: 'round',
          lineJoin: 'round',
          points: [0, 0, 0, 0],
          strokeWidth: 10,
          draggable: false
        });
        Arrow.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          this.scale = scale || 1;
          /**
           * Color del relleno de la flecha.
           *
           * @property fill
           * @type {String}
           */
          this.fill = options.fill;
          /**
           * Puntos origen y destino de la flecha.
           *
           * @property strokeWidth
           * @type {Number}
           */
          this.points = options.points;
        }
      };

      TextArrow.prototype = _.extend({}, Arrow.prototype, {
        /**
         * Función que realiza la especialización de la selección para la flecha del cuadro de texto
         * @method select
         * @param numtiradores
         */
        select: function(numtiradores) {
          // En este caso vale con la selección de la flecha pero especificando en este caso que solo hay un tirador
          numtiradores = 1;
          Arrow.prototype.select.call(this,numtiradores);
        }
      });

      return TextArrow;
    }
);
// noinspection JSHint
/**
 * Figura del cuadro de texto con la flecha
 * @class TextDirection
 * @extends Character
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/text-direction',[
      // libraries
      "underscore",
      // language
      "language",
      "modules/viewer/models/shape/arrow.text",
      "modules/viewer/models/shape/character"
    ],
    function (_, language, TextArrow, Character) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var TextDirection = function (options, scale, canvas, elem) {
        _.defaults(options || (options = {}), {
          className: "TextDirection",
          fontFamily: 'Arial',
          border: 6
        });
        Character.call(this, options, scale, canvas, elem);
        if ("undefined" !== typeof(options))
        {
          var points = (options.arrow && options.arrow.points && options.arrow.points.length >= 4) ? options.arrow.points : [
            0,
            0,
            150,
            0,
            150 - 20 * Math.cos(Math.atan2(0, 150) - Math.PI / 6),
            0 - 20 * Math.sin(Math.atan2(0, 150) - Math.PI / 6),
            150,
            0,
            150 - 20 * Math.cos(Math.atan2(0, 150) + Math.PI / 6),
            0 - 20 * Math.sin(Math.atan2(0, 150) + Math.PI / 6)
          ];
          this.pointer = new TextArrow({
            x: this.bound.x + this.bound.width,
            y: this.bound.y + this.bound.height / 2,
            strokeWidth: 6,
            color: "#F2942E",
            points: points
          }, this.scale, canvas);
        }
        this.anchors = [];
      };

      TextDirection.prototype = _.extend({}, Character.prototype, {
        draw: function(context, definitive) {
          if (this.width != 0 && this.height != 0) {
            // llamar al pintado del cuadro de texto
            Character.prototype.draw.call(this,context, definitive);

            var bound = this.bound2 || this.bound;
            this.pointer.x = bound.x + bound.width + this.border;
            this.pointer.y = bound.y + bound.height / 2;

            // Realizar el pintado de la flecha
            this.pointer.draw(context);
          }
        }, // end draw

        resize: function (width, height, fix) {
          Character.prototype.resize.call(this, width, height, fix);
          this.pointer.x = this.x + this.width + this.border;
          this.pointer.y = this.y + this.height / 2;
        },

        seleccionadoTiradorEspecial: false,

        isSelected: function(context, point, busquedaExhaustiva) {
          this.seleccionadoTiradorEspecial = this.pointer.isSelected(context, point, busquedaExhaustiva) // Si ha sido seleccionada la flecha
          return this.seleccionadoTiradorEspecial ||  Character.prototype.isSelected.call(this,context,point,busquedaExhaustiva); // o ha sido seleccionada la caja
        },

        select: function () {
          Character.prototype.select.call(this);
          this.pointer.select(); // Poner solo el tirador de la flecha
        },

        deselect: function () {
          Character.prototype.deselect.call(this);
          this.pointer.deselect();
        },

        compress: function (scale) {
          // Obtener el esqueleto de la compresión
          var comprimido = Character.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            arrow: this.pointer.compress().attrs
          });

          return comprimido;
        },

        evento: function(type, center) {
          var elem = ((this.seleccionadoTiradorEspecial) ? this.pointer : this);
          if (type == "touchend" || type == "mouseup")
            this.seleccionadoTiradorEspecial = false;
          return Character.prototype.evento.call(elem, type, center);
        },

        /**
         * Copia los datos del objeto actual a la figura que se pasa como parámetro
         * @param shape objeto al que copiar los datos
         * @param boolean flagPosicion si es true o undefined se copia la posición con offset en caso de false no se copia
         * @author Mario Poyato <mario.poyato@inmediastudio.es>
         */
        copyData: function(shape, flagPosicion) {
          // Se copian las dimesiones
          Character.prototype.copyData.call(this,shape,flagPosicion);
          // Se copia el texto
          shape.arrow = this.arrow;
        }
      });

      return TextDirection;
    }
);
// noinspection JSHint
/**
 * Figura del tick (sello de OK)
 * @class Check
 * @extends Tampon
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/tick',[
      // libraries
      "underscore",
      // configuration
      "configuration",
      // models
      "modules/viewer/models/shape/tampon"
    ],
    function (_, configuration, Tampon) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Check = function (options, scale, canvas) {
        var that = this;
        this.scale = scale || 1;
        _.defaults(options || (options = {}), {
          className: "Check",
          src: "assets/images/draw/check.svg",
          width: 100,
          height: 100
        });
        // En el caso de dispositivos móviles no hace falta aplicar una escala
        App.request("is:device").done(function () {
          that.height = that.width;
        });
        Tampon.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          this.src = options.src;
        }
      };

      Check.prototype = _.extend({}, Tampon.prototype, {});

      return Check;
    }
);
// noinspection JSHint
/**
 * Figura del triangulo
 * @class Triangle
 * @extends Shape
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/triangle',[
      // libraries
      "underscore",
      // models
      "modules/viewer/models/shape"
    ],
    function (_, Shape) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Triangle = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "Triangle",
          strokeWidth: 10,
          fill: "transparent",
          opacity: 0
        });
        Shape.call(this,options,scale,canvas);
        if ("undefined" !== typeof(options))
        {
          this.fill = options.color || options.fill;
          this.strokeWidth = options.strokeWidth;
        }
        this.anchors = [];
      };

      Triangle.prototype = _.extend({}, Shape.prototype, {
        /**
         * Función que realiza el camino del dibujado.
         * @method drawPath
         * @param {Object} context
         */
        drawPath: function(context, inner) {
          var diff = inner ? this.strokeWidth : 0;
          context.lineWidth = this.strokeWidth * this.scale;
          context.beginPath();
          context.moveTo(this.x + this.width / 2, this.y - diff); // Top Corner
          context.lineTo(this.width + this.x - diff/2, this.height + this.y - diff/2); // Bottom Right
          context.lineTo(this.x - diff/2, this.height + this.y - diff/2); // Bottom Left
          context.closePath();
        },

        draw: function(context) {
          if (this.width != 0 && this.height != 0) {
            this.drawPath(context);
            context.strokeStyle = this.fill;
            context.stroke();
            _.each(this.anchors, function(anchor) {
              anchor.draw(context);
            });
          }
        }, // end draw

        /**
         * Función que realiza la redimensión.
         * @method resize
         * @param {Integer} width
         * @param {Integer} height
         */
        resize: function (width, height) {
          this.width = width;
          this.height = height;
        },

        /**
         * Función que indica si el objeto ha sido seleccionado.
         * @method isSelected
         * @param {Object} context
         * @param {Object} point
         * @return {boolean}
         */
        isSelected: function(context, point, busquedaExhaustiva) {
          var touch;
          // En otro caso mirar si se ha seleccionado el rectángulo (búsqueda exhaustiva)
          if (busquedaExhaustiva) {
            this.drawPath(context,true);
            touch = context.isPointInPath(point.x, point.y);
          }
          this.drawPath(context);
          return (touch && context.isPointInPath(point.x, point.y)) || _.find(this.anchors, function(anchor) { return anchor.isSelected(context, point, true); });
        },

        /**
         * Función que realiza la especialización de la selección.
         * @method select
         */
        select: function () {
          if (this.anchors.length > 0) {
            // Todos los anclas tienen el mismo tamaño
            var offset = this.anchors[0].width / 2,
              x = this.x - offset,
              y = this.y - offset;
            this.anchors[0].x = this.anchors[3].x = x;
            this.anchors[0].y = this.anchors[1].y = y;
            this.anchors[1].x = this.anchors[2].x = x + this.width;
            this.anchors[2].y = this.anchors[3].y = y + this.height;
          }
          else {
            this.anchor({
              x: this.x,
              y: this.y,
              pos: "lt",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              x: this.x + this.width,
              y: this.y,
              pos: "rt",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              x: this.x + this.width,
              y: this.y + this.height,
              pos: "rb",
              color: "#009ee0",
              scale: this.scale
            });
            this.anchor({
              x: this.x,
              y: this.y + this.height,
              pos: "lb",
              color: "#009ee0",
              scale: this.scale
            });
          }
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Shape.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            color: this.fill,
            strokeWidth: this.strokeWidth
          });

          return comprimido;
        },

        /**
         * Función que procesa los eventos que llegan al objeto
         * @param type campo tipo del evento (touchstart, mousedown...)
         * @param center punto donde se realiza el evento
         * @returns {integer}
         */
        evento: function(type, center) {
          return this.eventTriggered(type,center,false);
        }
      });

      return Triangle;
    }
);
// noinspection JSHint
/**
 * Figura del subrayado
 * @class Underline
 * @extends Line
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/underline',["underscore", "modules/viewer/models/shape/line"],
    function (_, Line) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Underline = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "Underline",
          opacity: 0.9,
          stroke: "#FF0000",
          order: 0 // Grupo
        });
        Line.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          this.scale = scale || 1;
          this.points = (scale === 1) ? options.points : _.map(options.points, function(point){return point * this.scale}, this);
          this.fill2 = this.fill = this.color = options.color;
          this.strokeWidth = options.strokeWidth;
        }
      };

      Underline.prototype = _.extend({}, Line.prototype, {
        /**
         * Función que realiza la especialización de la selección para la flecha del cuadro de texto.
         * @method select
         */
        select: function() {
          this.fill = "#32B6FF";
        },

        /**
         * Función de deseleccionado de la figura. Que simplemente restaura el color del resaltado.
         * @method deselect
         */
        deselect: function() {
          this.fill = this.fill2;
        },

        /**
         * Función que indica si el objeto ha sido seleccionado.
         * @method isSelected
         * @param {Object} context
         * @param {Object} point
         * @return {boolean}
         */
        isSelected: function(context, point) {
          var minX = this.points[0],
            minY = this.points[1],
            maxX = this.points[2],
            maxY = this.points[3];
          if (minY > maxY) {
            minY = this.points[3];
            maxY = this.points[1];
          }
          if (minX > maxX) {
            minX = this.points[2];
            maxX = this.points[0];
          }
          return point.x >= minX &&  point.x <= maxX &&  point.y >= minY &&  point.y <= maxY;
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function() {
          return {
            attrs: {
                color: this.color
              , lineCap: "round"
              , lineJoin: "round"
              , points: _.map(this.points, function(point){ return point / this.scale; }, this) // Aplicar la escala al array de puntos
              , stroke: "#FF0000"
              , strokeWidth: this.strokeWidth
            },
            className: "Underline",
            children: []
          };
        },

        /**
         * Función que realiza el dibujado.
         * @method draw
         * @param {Object} context
         */
        draw: function(context) {
          if (this.width != 0 && this.height != 0) {
            context.beginPath();
            context.moveTo(this.points[0], this.points[1]);
            context.lineTo(this.points[2], this.points[3]);
            context.lineWidth = this.strokeWidth * this.scale;
            context.strokeStyle = this.fill;
            context.stroke();
          }
        },

        /**
         * Función que procesa los eventos que llegan al objeto
         * @param {String} type campo tipo del evento (touchstart, mousedown...)
         * @param {Object} center punto donde se realiza el evento
         * @method evento
         * @returns {integer}
         */
        evento: function(type, center) {
          switch (type) {
            case "mouseevent": {
              break;
            }
            default: {
              // do nothing
              break;
            }
          }
        },

        /**
         * Función que indica si hay que limpiar el canvas para la figura.
         * @method unDraw
         * @returns {boolean}
         */
        unDraw: function() { return true; /* Hay que limpiar el canvas */ }
      });

      return Underline;
    }
);
// noinspection JSHint
/**
 * Figura de la rectángulo con enlace
 * @class Scloud
 * @extends Tampon
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/scloud',[
      // libraries
      "jquery",
      "underscore",
      "backbone",
      "marionette",
      // configuration
      "configuration",
      // language
      "language",
      // models
      "modules/viewer/models/shape/tampon",
      // views
      "views/dialog"
    ],
    function ($, _, Backbone, Marionette, configuration, language, Tampon, DialogView) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var Scloud = function (options, scale, canvas) {
        var width = $("aside.interactive").width() / scale * 0.6;
        // Default properties of KineticJS
        _.defaults(options || (options = {}), {
          type: "Image",
          className: "Scloud",
          innerX: 0,
          innerY: 0,
          color: "#1396DD",
          width: width,
          height: width * 0.7,
          dimensionsMiddle: false,
          anchorsDraggable: true
        });
        Tampon.call(this, options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          this.type = options.type;
          this.src = options.img;
          if (this.src) {
            this.img = new Image();
            this.img.onload = function(){
              // Llamar a la función que se pasa como parámetro para que guarde en el historial el estado
              if (typeof(fn) != "undefined") {
                App.request("is:device")
                  .fail(function() {
                    fn.call(view);
                  });
              }
            }.bind(this);
            this.img.onerror = function(){
              // No existe
              App.execute("notification", language("imageError"), {color: 6});
              console.log("Error al agregar imagen");
            };
            this.img.src = this.src;
          }
        }
      };

      Scloud.prototype = _.extend({}, Tampon.prototype, {
        /**
         * Función que realiza la redimensión.
         * @method resize
         * @param {Integer} width
         * @param {Integer} height
         * @param {boolean} fix indica si hay que realizar el arreglo de tamaño minimo o no
         */
        resize: function (width, height, fix) {
          if (fix) {
            this.width  = Math.max(100,width);
            this.height = Math.max(50,height);
            if (this.anchors && this.anchors.length > 0) {
              this.select();
            }
          }
          else {
            this.width = width;
            this.height = height;
          }
        },

        /**
         * Función que abre una ventana o popup en el caso de algunos elementos. Además se bloquea la figura.
         * @method openWindow
         */
        openWindow: function (fn, view) {
          var that = this;

          App.request("av", {type: 'filesystem', identifier: 'scloud', timeout: false, _selected: "image"})
            .done(function (data) {
              if (_.contains([
                  "gif", "jpg", "jpeg", "png"
                ], data.image.substr(data.image.lastIndexOf(".")).substr(1).toLowerCase())) {
                // Verificamos si la imagen existe
                var imageUrl = data.image.replace("//","/");
                /*
                if (data.identifier == "scloud") {
                  imageUrl = imageUrl.replace("/files/","/");
                }
                //*/
                that.img = new Image();
                that.img.onload = function(){
                  // Si existe
                  that.src = imageUrl; // Ruta completa para mostrar imagen correctamente
                  // Llamar a la función que se pasa como parámetro para que guarde en el historial el estado
                  if (typeof(fn) != "undefined") {
                    App.request("is:device")
                      .fail(function() {
                        fn.call(view);
                      });
                  }
                }.bind(that);
                that.img.onerror = function(){
                  // No existe
                  App.execute("notification", language("imageError"), {color: 6});
                  console.log("Error al agregar imagen");
                };
                that.img.src = imageUrl;
              } else {
                App.execute("notification", language("imagesOnly"), {color: 6});
              }
            });
        },

        /**
         * Función que devuelve los datos del objeto comprimidos
         * @method compress
         * @returns {Object}
         */
        compress: function () {
          // Obtener el esqueleto de la compresión
          var comprimido = Tampon.prototype.compress.call(this);

          // Aplicar los atributos propios del objeto
          _.defaults(comprimido.attrs || (comprimido.attrs = {}), {
            type: this.type,
            img: this.src
          });

          return comprimido;
        }
      });


      return Scloud;
    }
);
// noinspection JSHint
define(
  'modules/viewer/views/kinetic',[
    // libraries
    "jquery",
    "underscore",
    "marionette",
    // figures
    "modules/viewer/models/shape/arrow",
    "modules/viewer/models/shape/character",
    "modules/viewer/models/shape/circle",
    "modules/viewer/models/shape/clip",
    "modules/viewer/models/shape/cross",
    "modules/viewer/models/shape/doodle",
    "modules/viewer/models/shape/edit",
    "modules/viewer/models/shape/transparent-circle-inverse",
    "modules/viewer/models/shape/transparent-circle",
    "modules/viewer/models/shape/transparent-square-inverse",
    "modules/viewer/models/shape/transparent-square",
    "modules/viewer/models/shape/fill-circle-inverse",
    "modules/viewer/models/shape/fill-circle",
    "modules/viewer/models/shape/fill-square-inverse",
    "modules/viewer/models/shape/fill-square",
    "modules/viewer/models/shape/highlight",
    "modules/viewer/models/shape/line",
    "modules/viewer/models/shape/link-square",
    "modules/viewer/models/shape/square",
    "modules/viewer/models/shape/text-direction",
    "modules/viewer/models/shape/tick",
    "modules/viewer/models/shape/triangle",
    "modules/viewer/models/shape/underline",
    "modules/viewer/models/shape/scloud",
  ],
  function ($, _, Marionette, Arrow, Character, Circle, Clip, Cross, Doodle, Note, EmptyCircleInverse, EmptyCircle, EmptySquareInverse, EmptySquare, FillCircleInverse, FillCircle, FillSquareInverse, FillSquare, Highlight, Line, LinkSquare, Square, TextDirection, Check, Triangle, Underline, Scloud) {
    "use strict";

    var STATUS = {
      IDLE: 0,
      RUN: 1
    };

    return Marionette.ItemView.extend({
      /* constants */
      STATUS: STATUS,

      /* attributes */
      // backbone & marionette
      events: {
        //"doubletap": "cancelAlways",
        "drag": "cancel",
        "dragend": "cancel",
        "mouseout": function() {
          //$(this.canvas).trigger("mouseup");
        },
        "swipeleft": "cancelAlways",
        "swiperight": "cancelAlways",
        "release": function () {
          $(this.canvas).trigger("mouseup");
        },
        "touch": "cancel"
      },
      options: {
        sel: false,
        layer: {
          scale: 1,
          size: {
            height: 0,
            width: 0
          }
        },
        position: {
          left: 0,
          top: 0
        },
        scale: 1,
        size: {
          height: 0,
          width: 0
        }
      },
      tagName: "aside",
      template: _.template(""),

      // custom
      draggable: true,
      status: STATUS.IDLE,

      canvas: null,
      ctx: null,
      frontcanvas: null,
      fctx: null,

      snapshot: null, // Save the snapshot of the layer in JSON format
      shape: [],
      selectedAux: null,
      oldSelected: null,
      selected: null,

      numsel: -1,

      changed: false, // Indica si la vista ha cambiado

      interactiveTexts: null,

      center: null,
      primeraSeleccion: false,
      lastTime: null,

      /* methods */
      // backbone & marionette
      initialize: function () {
        if (this.options.size.width !== this.options.layer.size.width) {
          this.options.layer.scale = +(this.options.size.width / this.options.layer.size.width).toFixed(3);
        } else {
          this.options.layer.scale = this.options.scale;
        }
        App.on("kinetic:select", this.select, this);
      },

      /**
       * Función de limpieza de memoria del objeto
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      onClose: function () {
        // Limpiar el array de las figuras
        _.each(this.snapshot.shapes.current, function (elem) { if (elem.element) { elem.element.remove(); } });
        this.snapshot.shapes.current.length = 0;
        this.snapshot = null;
        _.each(this.shape, function (elem) { if (elem&&elem.element) { elem.element.remove(); } });
        this.shape = null;
        // Canvas
        this.canvas = null;
        this.ctx = null;
        this.frontcanvas = null;
        this.fctx = null;
        // Selecciones
        if (this.selected && this.selected.element) { this.selected.element.remove(); };
        this.selected = null;
        this.selectedAux = null;
        this.oldSelected = null;
        this.interactiveTexts = null;
        // Limpiar eventos
        this.unbind();
      },

      /**
       * Función de renderizado de canvas
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      onRender: function () {
        var drawing = "{}";
        if (!$(this.canvas).data("data-tools-drawed"))
          drawing = this.model.get("drawing");

        // Inicializar la figura de dibujado auxiliar a null (no hay figura)
        this.shape = null;
        // Inicializar la parte interactiva de la vista a la establecida por el modelo
        this.interactiveTexts = this.model.get("interactive");
        this.model.unset("interactive", { silent: true }); // Limpiar el modelo
        // En caso de que no esten los canvas se toman como canvas el del DOM (generado por LoadCanvasAsync de la librería PDFTron)
        if (!this.canvas || !this.frontcanvas) {
          var element = $("ul#viewer > li > article.page > aside.interactive");
          this.canvas = document.createElement("canvas");
          this.ctx = this.canvas.getContext('2d');
          this.canvas.className = "mainTools";
          this.canvas.height = element.height();
          this.canvas.width = element.width();
          element = null;
          this.canvas.style.backgroundColor = "transparent";
          this.canvas.style.top = "0px";
          this.canvas.style.left = "0px";
          this.canvas.style.border = "0px";
          this.canvas.style.margin = "0px";
          this.canvas.style.padding = "0px";
          this.canvas.style.position = "absolute";
          this.el.appendChild(this.canvas);
          this.shape = [];
          this.snapshot = {
            shapes: {
              current: [],
              left: null,
              right: null
            },
            nextId: 0, // Valor del siguiente identificador del elemento que se introducirá en la lista
            sortFunction: function (a, b) { return a.order - b.order; }
          };
        }
        // En caso de que se hayan añadido los canvas al KineticView se añaden al DOM y se opera con ellos
        else {
          // Añadir a la vista el canvas principal, la parte interactiva y el canvas frontal
          this.el.appendChild(this.canvas);
          this.el.appendChild(this.frontcanvas);
        }
        // Realizar el dibujado del canvas
        if ("undefined" !== typeof(drawing)) {
          var drawingJSON = JSON.parse(drawing);
          if (drawingJSON && drawingJSON.children) {
            // Cargar las figuras guardadas y además dibujarlas en el canvas fantasma
            this.snapshot.shapes.current = this.convertObjectSnapshot(drawingJSON.children);
            this.mainDraw();
          }
          console.debug("[DEBUG] Current page loaded with "+this.snapshot.shapes.current.length+" elements");
          drawingJSON = null;
        }

        this.$el
          .css("top", this.options.position.top)
          .css("left", this.options.position.left);

        this.bind();

        // Obtener la versión de Android
        this.androidVersion = navigator.userAgent.match(/Android\s([0-9\.]*)/i);
        if (this.androidVersion && this.androidVersion.length > 1) {
          this.androidVersion = parseFloat(this.androidVersion[1]);
        }
        else {
          this.androidVersion = null;
        }
      },

      /**
       * Pasado un evento cualquiera de ratón o táctil obtiene la posición de este
       * en las X e Y respecto al canvas.
       * @param {Object} event El evento disparado.
       * @returns {Object} con el parámetro X e Y del evento disparado.
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      pointer: function(event) {
        // <-----------------------( Declaración de variables )----------------------->
        var center = null;
        event = event || window.event;
        if (!this.canvas) {
          var canvasString = "canvas.mainTools"
          // Si es par se coge el canvas de la derecha (o último)
          if ((this.model.attributes.id-1) % 2 == 0) {
            this.canvas = $(canvasString+":last")[0];
          }
          else {
            this.canvas = $(canvasString+":first")[0];
          }
        }
        // <-------------------------( Cuerpo de la función )------------------------->
        if (typeof(event) !== "undefined" && typeof(event.target) !== "undefined") {
          var outerScale = App.viewer.document.controller.attributes.zoom.scale;
          if (event.touches) {
            var touch = event.touches[0],
              target = $(event.target);
            if (touch) {
              var x = touch.pageX+(target==this.$el)?0:target.offset().left,
                  y = touch.pageY+(target==this.$el)?0:target.offset().top;
              center = {
                x: (touch.pageX - this.$el.offset().left) / outerScale,
                y: (touch.pageY - this.$el.offset().top) / outerScale
              };
            }
          }
          if (center === null) {
            var x = event.pageX,
                y = event.pageY,
                rect = this.canvas.getBoundingClientRect();
            if (!x && event.touches && event.touches.length > 0) {
              x = event.touches[0].clientX;
            }
            if (!x && event.changedTouches && event.changedTouches.length > 0) {
              x = event.changedTouches[0].clientX;
            }
            if (!y && event.touches && event.touches.length > 0) {
              y = event.touches[0].clientY;
            }
            if (!y && event.changedTouches && event.changedTouches.length > 0) {
              y = event.changedTouches[0].clientY;
            }
            center = {
              x: (x-rect.left)/(rect.right-rect.left)*this.canvas.width,
              y: (y-rect.top)/(rect.bottom-rect.top)*this.canvas.height
            };
          }
        }
        // <------------------------( Retorno de la función )------------------------->
        return center;
      },

      /**
       * Limpia la parte interactiva de la vista (como pueden ser el html de los cuadros
       * de texto).
       * @param {Object} - elemento que se quiere limpiar (en caso de limpiar únicamente solo ese elemento)
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      clearInteractive: function (elem) {
        // <-----------------------( Declaración de variables )----------------------->
        var clearCanvas = false;
        // <-------------------------( Cuerpo de la función )------------------------->
        // En el caso de limpiar la parte interactiva de un solo elemento
        if (elem) {
          if (elem.unDraw) {
            clearCanvas = elem.unDraw();
          }
        }
        else {
          // Para cada una de las figuras del canvas
          _.each(this.snapshot.shapes.current, function(elem) {
            if (elem.unDraw) {
              clearCanvas = elem.unDraw() || clearCanvas;
            }
          });
        }
        // <------------------------( Retorno de la función )------------------------->
        return clearCanvas;
      },

      /**
       * Añade una nueva figura o representación al conjunto de figuras del canvas.
       * Teniendo en cuenta el orden de las figuras siendo:
       *  · 1) Grupo (subrayado, resaltado...).
       *  · 2) Figuras en general.
       *  · 3) Máscaras.
       *  · 4) Cuadros de texto. (con o sin flecha)
       * @param {Object} snapshot Nueva figura a añadir.
       * @param {array} shapes Array con las figuras (si no se especifica se toma el array de la página actual).
       * @returns {Object} con el parámetro X e Y del evento disparado.
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      addSnapshot: function(snapshot, shapes) {
        // En caso de no especificarse el array de datos se toma el array de datos de la página actual
        shapes = shapes || this.snapshot.shapes.current;
        // <-------------------------( Cuerpo de la función )------------------------->
        if (typeof(snapshot) != "undefined") {

          snapshot.id = shapes.length;
          // Añadir la figura
          shapes.push(snapshot);
          // Ordenar el array de figuras
          shapes.sort(this.snapshot.sortFunction);
          /*
          shapes = this.snapshot.shapes.left || this.snapshot.shapes.right;
          if (shapes && !_.find(shapes, function(shape){return shape.id == snapshot.id})) {
            // Añadir la figura
            shapes.push(snapshot);
            // Ordenar el array de figuras
            shapes.sort(this.snapshot.sortFunction);
          }
          //*/
        }
        // <------------------------( Retorno de la función )------------------------->
        return this.snapshot;
      },

      /**
       * Función que limpia el canvas al que pertenece el contexto del argumento.
       * @param {Object} context Contexto del canvas a limpiar.
       * @param {boolean} flag Indica si se debe necesariamente forzar la redimensión de canvas para su limpiado.
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      clear: function (context, flag) {
        /**
         * FIX: En dispositivos con versión menor a 4.4 (estricto) de android no funciona la limpieza de canvas con la función clearRect.
         * Por lo que se fuerza al limpiado cambiando el ancho de forma que se eliminan todas las figuras.
         * @author Mario Poyato <mario.poyato@inmediastudio.es>
         */
        if (flag && this.androidVersion && this.androidVersion < 4.4) {
          context.canvas.width = context.canvas.width;
        }
        else if (context) {
          context.clearRect(0, 0, context.canvas.width, context.canvas.height);
        }
      },

      /**
       * Generar un JSON de las figuras representadas en el canvas.
       * @param {Boolean} currentPage - bandera que indica si se devuelve la información de la página actual o la otra
       * @returns {Object} JSON de las figuras representadas.
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      layerToJSON: function(currentPage) {
        // <-----------------------( Declaración de variables )----------------------->
        var algoritmoCompresion = function(elem) {
          // Devolver la versión comprimida
          return elem.compress(this.options.layer.scale);
        };
        var elems = (currentPage === false) ? ( this.snapshot.shapes.left || this.snapshot.shapes.right ) : this.snapshot.shapes.current;
        // <------------------------( Retorno de la función )------------------------->
        // Devolver la clase principal que representa la capa
        return {
          attrs: {
            height: this.options.layer.size.height,
            width: this.options.layer.size.width
          },
          className: "Layer",
          children: _.map(elems, function(elem) {
            if (elem) {
              // En el caso de tener un grupo (o conjunto) de elementos
              // [p.ej. caso de subrayado que tiene varias lineas]
              if (elem.className === "Group") {
                // Duplicar el elemento porque se cambiará el campo children (así no cambiar el original)
                elem = _.clone(elem);
                elem.children = _.map(elem.children, algoritmoCompresion, this);
                return elem;
              } else {
                // Obtener la versión reducida del elemento tratado
                return algoritmoCompresion.call(this, elem);
              }
            }
            else
              return {};
          }, this)
        };
      },

      /**
       * Elimina la figura auxiliar y devuelve esta figura elminada.
       * @returns {Object} figura eliminada.
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      getLastShape: function() {
        //var ret = this.shape;
        //this.shape = null;
        //return ret;
        return this.shape;
      },

      /**
       * Pinta el canvas frontal de la vista, en concreto la figura seleccionada y la figura auxiliar.
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      frontDraw: function () {
        //console.debug("frontDraw executed");
        // ===============================================================================================================
        // ¡Importante! esto se hace porque los elementos interactivos no aplican bien la escala en todos los dispositivos
        // Comprobar si las herramientas interactivas estan ya dibujadas por separado de las no editables
        if (this.interactiveTexts && this.interactiveTexts.parentNode != this.canvas.parentNode) {
          var interactiveTexts = this.interactiveTexts.cloneNode(false);
          // En cuyo caso generar
          this.el.insertBefore(interactiveTexts, this.el.childNodes[0]/*this.frontcanvas*/);
          // Eliminar los elementos "editable" del div
          var elements = this.interactiveTexts.getElementsByClassName("editable");
          while (elements.length > 0) {
            interactiveTexts.appendChild(elements[0]);
          }
          this.interactiveTexts = interactiveTexts;
        }
        // ===============================================================================================================
        // Limpiar el canvas frontal (antes de dibujar)
        this.clear(this.fctx, false);
        // Pintar el elemento seleccionado
        if (this.selected) {
          if (this.selected.className === "Group") {
            _.each(this.selected.children, function (elem) {
              elem.draw(this.fctx, false);
            }, this);
          }
          else {
            this.selected.draw(this.fctx, false);
          }
        }
        // Pintar la figura que se esta dibujando actualmente
        if (this.shape) {
          if (this.shape.children) {
            _.each(this.shape.children, function (shape) {
              shape.draw(this.fctx, false);
            }, this);
          }
          else {
            this.shape.draw(this.fctx, false);
          }
        }
      },

      /**
       * Pinta el canvas principal de la vista.
       * @param {Boolean} release - indica si el objeto seleccionado ha lanzado un evento "release"
       * @param {Object} ctxDibujado - contexto del canvas donde dibujar
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      mainDraw: function (release, ctxDibujado) {
        //console.debug("mainDraw executed");
        var ctx = ctxDibujado || this.ctx;
        // Limpiar el canvas fantasma
        if (!ctxDibujado)
          this.clear(ctx, true);
        // Dibujar todos los elementos en el canvas
        if (this.snapshot.shapes.current) {
          _.each(this.snapshot.shapes.current, function (elem) {
            if (elem != this.selected) {
              if (elem.className == "Group" && elem.children) {
                _.each(elem.children, function (elem) {
                  if (elem) elem.draw(ctx, true);
                }, this);
              }
              else {
                //console.log("Dibujado "+elem.className);
                elem.draw(ctx, true);
              }
            }
          }, this);
        }
        // Pintar el elemento seleccionado solo si se hizo relase (para que aparezca completo)
        if (this.selected && release && (this.selected.className == "TextDirection" || this.selected.className == "Character")) {
          if (!ctxDibujado)
            this.clear(this.fctx, false);
          ctx = ctxDibujado || this.fctx;
          this.selected.draw(ctx, true);
        }
      },

      /**
       * Función que elimina el texto seleccionado si hubo texto seleccionado pendiente de copiado o edición.
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      removeSelectedText: function() {
        var group = _.find(this.snapshot.shapes.current, function (group) {
          if (group.className == "Group") {
            var res = _.find(group.children, function (select) {
              if (select.className == "TextCopy") {
                return true;
              }
            }, this);
            if (res)
              return group;
            return null;
          }
        }, this);
        if (group)
          this.snapshot.shapes.current = _.filter(this.snapshot.shapes.current,function(elem){return (typeof elem != "undefined" && elem != null)&&elem!==group;});
      },

      /**
       * Función que realiza la selección.
       * @param force elemento.
       */
      select: function (force) {
        this.options.sel = force;
        if (!force && this.selected) {
          this.selected.deselect();
          this.selected = null;
          this.mainDraw();
          this.frontDraw();
        }
      },

      /**
       * Función que realiza la deselección del elemento seleccionado.
       * @param {Boolean} sel - true si se quiere seleccionar y false si no se quiere seleccionar
       * @param {Object} elemento - elemento a seleccionar o deseleccionar (si no se recibe se toma el elemento que ya estaba seleccionado)
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      selectElement: function (sel, elemento) {
        // En caso de que haya un elemento seleccionado
        if (this.selected) {
          if (this.selected.className == "Group") {
            if (sel) {
              if (elemento)
                this.selected = elemento;
              _.each(this.selected.children, function (groupElem) {
                groupElem.select();
              }, this);
            }
            else {
              _.each(this.selected.children, function (groupElem) {
                groupElem.deselect();
              }, this);
              this.selected = null;
            }
          }
          else {
            if (sel) {
              this.selected.select();
            }
            else {
              this.selected.deselect();
              this.selected = null;
            }
          }
        }
      },

      /**
       * Función que obtiene el elemento seleccionado dado un punto donde se lanza un evento.
       * Es importante indicar que el elemento no es seleccionado, solo se indica cual sería seleccionado.
       * @param center - última posición del puntero o del evento táctil
       * @param changeItem - bandera que indica si es es necesario cambiar de elemento, en cuyo caso solo se selecciona el mismo elemento si no hay
       *                     otra posibilidad.
       * @returns {Object} {val, sel} - donde "sel" es elemento seleccionado y "val" es el valor de retorno con estos significados:
       *
       *    -----------------------------------------------------------------------------------------------------------------------------------------
       *   | Valor \ Momento |   Antes   |   Ahora   | Descripción                                                                                   |
       *   |        0        |     NO    |     NO    | No habia una figura seleccionada y no se selecciona ninguna.                                  |
       *   |        1        |     SI    |     NO    | Había una figura seleccionada y después se deselecciona.                                      |
       *   |        2        |     NO    |     SI    | Se selecciona una figura nueva (no habia una figura seleccionada antes).                      |
       *   |        3        |     SI    |     SI    | Se selecciona una figura diferente a la que se selecciono antes.                              |
       *   |        4        |     SI    |     SI    | Se selecciona la misma figura pero se ha indicado de no seleccionar otra si se puede repetir. |
       *   |        5        |     SI    |     SI    | Se reselecciona la misma figura pero porque solo se podía seleccionar esa.                    |
       *    -----------------------------------------------------------------------------------------------------------------------------------------
       *
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      selectItem: function (center, changeItem) {
        // <-----------------------( Declaración de variables )----------------------->
        var retorno,
            selected;
        // <-------------------------( Cuerpo de la función )------------------------->
        // ¡Importante! se le da la vuelta porque en el dibujado aparece en el fondo el primero (por ser el primero en ser dibujado) y el último encima del resto. De esta forma se da más prioridad al que esta por encima dibujado
        this.snapshot.shapes.current.reverse();
        // Obtener el elemento seleccionado de la lista de elementos
        selected = _.find(this.snapshot.shapes.current, function(group) {
          if (group.className == "Group") {
            if (_.find(group.children, function(elem) { return elem.isSelected(this.ctx, center); }, this)) {
              return group.children;
            }
          }
          else {
            return group.isSelected(this.ctx, center) ? group : null;
          }
        }, this);
        if (selected) {
          selected = [selected];
        }
        // Si no ha habido ningún elemento seleccionado realizar una búsqueda más exhaustiva para por ejemplo se seleccionen la parte interior de las figuras
        else {
          selected = _.filter(this.snapshot.shapes.current, function (group) {
            return (
              // La búsqueda se hace sobre aquellos elementos que no sean grupos (selección de texto, subrayado o resaltado)
            group.className != "Group"
              // Se mira que cumpliendo con lo anterior además este seleccionado por ese punto pero añadiendo el parámetro de búsqueda exhaustiva
            && group.isSelected(this.ctx, center, true)
            );
          }, this);
        }
        // Devolver el listado de figuras al mismo orden
        this.snapshot.shapes.current.reverse();

        // Si no se ha seleccionado ninguna figura
        if (!selected) {
          // comprobar si habia una seleccionada antes (estado 1) o si no habia ninguna seleccionada (estado 0)
          retorno = (this.selected) ? 1 : 0;
        }
        else {
          // Comprobar si se realiza repetición de la figura
          var seleccionadoSeReselecciona = _.find(selected,function(sel){return sel==(this.selected||this.selectedAux);},this);
          // Si se indico de no cambiar el elemento seleccionado y en la lista se encuentra el mismo que estaba seleccionado
          if (!changeItem && seleccionadoSeReselecciona) {
            retorno = 4; // Indicar que se reselecciona el mismo elemento
            selected = this.selected;
          }
          else {
            // En caso de que no haya ningún objeto seleccionado colocar el indice en la primera posición para obtener el de la posición de más afuera
            if (!this.selected) {
              this.numsel = -1;
            }
            var sel = selected[(++this.numsel)%selected.length];
            if (sel && sel == this.selected)
              sel = selected[(++this.numsel)%selected.length];
            selected = sel;
            if (this.selected) {
              if (selected == this.selected) {
                retorno = 4;
              }
              else {
                retorno = (seleccionadoSeReselecciona) ? 3 : 5;
              }
            }
            else {
              retorno = (sel == this.selectedAux) ? 4 : 2;
            }
          }
        }
        // <------------------------( Retorno de la función )------------------------->
        return {
          val: retorno,
          sel: selected
        };
      },

      /**
       * Función que envia el evento especifico a la figura especificada.
       * @param {Object} elem - Elemento al que enviar el evento.
       * @param {Object} event - Evento que hay que mandar.
       * @returns {Integer} - 0: si no hay cambios, 1: si cambia el canvas frontal, 2: si cambia el canvas frontal y principal.
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      sendEvent: function (elem, event) {
        // <-----------------------( Declaración de variables )----------------------->
        var retorno = 0;
        // <-------------------------( Cuerpo de la función )------------------------->
        if (elem) switch (elem.className) {
          case "Group": {
            retorno = _.some(elem.children, function(groupElem) {
              groupElem.evento(event.type, this.center);
            }, this);
            break;
          }
          default: {
            retorno = elem.evento(event.type, this.center);
            break;
          }
        };
        // <------------------------( Retorno de la función )------------------------->
        return retorno;
      },

      // custom
      bind: function () {
        var elemento = $(document.getElementsByClassName("interactive")),
            start = "ontouchstart" in elemento[0] ? "touchstart" : "mousedown",
            move = "ontouchmove" in elemento[0] ? "touchmove" : "mousemove",
            end = "ontouchend" in elemento[0] ? "touchend touchleave" : "mouseup mouseout";

        // Se asignan los eventos al elemento padre del canvas y el cuadro de elementos interactivos
        elemento
          .css("zIndex", "100")

        // =========================================================================================
        // START
        // =========================================================================================
          .on(start, _.bind(function (event) {
            if (!this.options.sel) {
              event.preventDefault();
              event.stopPropagation();
            }
            // Obtener la posición del evento
            this.center = this.pointer(event.originalEvent) || this.center || {x: 0, y: 0};
            // Hacer lo correspondiente a ese evento
            this.trigger("touch", this.center);
            this.status |= STATUS.RUN;
            /* ----------------------------------------------------------------------------------------------------------------------------
             * Obtener los elementos seleccionados. Se hace caso a las situaciones:
             *  · (1) Se selecciona un elemento por primera vez.
             *  · (2) Se deselecciona un elemento.
             *  · (4) Se selecciona de nuevo el mismo elemento.
             *  · (5) Se selecciona el mismo elemento que estaba seleccionado pero solo porque se podia seleccionar ese.
             */
            this.changed = false;
            var numsel = this.numsel,
                ret = this.selectItem(this.center, true /* Un resultado */, true /* Mantener si es el mismo resultado */),
                time = Date.now(),
                invalidateDraw = false;
            switch (ret.val) {
              // Se selecciona un elemento por primera vez.
              case 1: {
                // Solo hacer caso cuando esta activada la herramienta de selección
                if (this.options.sel) {
                  this.primeraSeleccion = false;
                  this.selectElement(false);
                  // Send event
                  this.sendEvent(this.selected, event);
                  this.selected = null;
                  invalidateDraw = true;
                  this.selectedAux = null;
                }
                break;
              }
              case 2: {
                // Solo hacer caso cuando esta activada la herramienta de selección
                if (this.options.sel) {
                  this.primeraSeleccion = true;
                  this.selected = ret.sel;
                  this.selectElement(true, ret.sel);
                  this.snapshot.shapes.current = _.reject(this.snapshot.shapes.current, function (elem) {
                    return !(typeof elem != "undefined" && elem != null) || elem == this.selected;
                  }, this);
                  if (typeof(this.selected) != "undefined") {
                    this.addSnapshot(this.selected);
                  }
                  invalidateDraw = true;
                }
                this.selectedAux = ret.sel;
                break;
              }
              case 4: {
                // Manejar el evento de dobleclick
                var sel = (this.selected||this.selectedAux),
                    pages = document.getElementsByClassName("page");
                if (this.lastTime && time - this.lastTime < 400 && sel.className != "Group" && (pages.length == 1 || pages.length == 2 && this.model.attributes.id % 2 != 0)) {
                  sel.openWindow(function () {
                    this.changed = true;
                    this.trigger("release");
                  }, this);
                }
                this.selectedAux = null;
                break;
              }
              case 5: {
                // Solo hacer caso cuando esta activada la herramienta de selección
                if (this.options.sel) {
                  this.primeraSeleccion = false;
                  invalidateDraw = true;
                  // Deseleccionar el anterior
                  if (this.selected && this.selected != ret.sel) {
                    this.selectElement(false);
                  }
                  this.selected = ret.sel;
                  // No se para aqui porque se selecciona igual que cuando no habia seleccionado
                  this.selectElement(true, ret.sel);
                  this.snapshot.shapes.current = _.reject(this.snapshot.shapes.current, function (elem) {
                    return !(typeof elem != "undefined" && elem != null) || elem == this.selected;
                  }, this);
                  if (typeof(this.selected) != "undefined") {
                    this.addSnapshot(this.selected);
                  }
                  this.selectedAux = null;
                }
                break;
              }
              default: {
                this.numsel = numsel;
                this.selectedAux = null;
              }
            }
            if (this.options.sel) {
              if (this.selected) {
                this.sendEvent(this.selected, event);
                this.lastTime = time;
              }
              else {
                this.lastTime = null;
              }
              if (invalidateDraw) {
                this.frontDraw();
                this.mainDraw(true);
              }
            }
          }, this))
          // =========================================================================================
          // RUN
          // =========================================================================================
          .on(move, _.bind(function (event) {
            // Obtener la posición del evento
            this.center = this.pointer(event.originalEvent) || this.center || {x: 0, y: 0};
            // Hacer lo correspondiente a ese evento
            if (this.status & STATUS.RUN) {
              this.trigger("drag", this.center);
              /* ----------------------------------------------------------------------------------------------------------------------------
               * Modificar los elementos seleccionados.
               */
              if (this.selected && this.options.sel) {
                event.preventDefault();
                event.stopPropagation();
                // ----------------------------------------------------------------------------
                // Si se ha seleccionado algún elemento o hay algún elemento implicado no
                // permitir que haya más eventos para que no haya scroll o cambios de página
                // ----------------------------------------------------------------------------
                this.changed = this.sendEvent(this.selected, event);
                if (this.changed) {
                  _.each(this.snapshot.shapes.left || this.snapshot.shapes.right, function(shape) {
                    if (shape.className == "Group") {
                      _.each(shape.children, function(shape) {
                        if (shape.id == this.selected.id) {
                          this.selected.copyData(shape);
                        }
                      },this);
                    }
                    else {
                      if (shape.id == this.selected.id) {
                        this.selected.copyData(shape);
                      }
                    }
                  }, this);
                  this.frontDraw();
                }
                // ----------------------------------------------------------------------------
              }
              else if (!this.options.sel) {
                event.preventDefault();
                event.stopPropagation();
                if (this.shape) {
                  _.each(this.snapshot.shapes.left || this.snapshot.shapes.right, function(shape) {
                    if (shape.className == "Group") {
                      _.each(shape.children, function(shape) {
                        if (shape.id == this.shape.id) {
                          this.shape.copyData(shape,false);
                        }
                      },this);
                    }
                    else {
                      if (shape.id == this.shape.id) {
                        this.shape.copyData(shape,false);
                      }
                    }
                  }, this);
                }
              }
            }
          }, this))
          // =========================================================================================
          // END
          // =========================================================================================
          .on(end, _.bind(function (event) {
            // En caso de que al sacar el puntero o ratón de un canvas se comprueba si el siguiente elemento es el otro canvas
            if ((event.type == "mouseout" || event.type == "touchleave") && (event.originalEvent.toElement && (event.originalEvent.toElement.className.indexOf("interactive")>-1 || event.originalEvent.toElement.tagName.toLowerCase() == "canvas"))) {
              // En caso de no estar en ninguno de los canvas se cierra el evento
              return true;
            }
            event.preventDefault();
            event.stopPropagation();
            // Hacer lo correspondiente a ese evento
            if (this.status & STATUS.RUN) {
              _.chain(function () {
                this.trigger("release");
              })
                .bind(this)
                .defer()
                .value();
            }
            //noinspection JSHint
            this.status &= STATUS.IDLE;
            /* ----------------------------------------------------------------------------------------------------------------------------
             * Libera los elementos seleccionados. Se hace caso a las situaciones:
             *  · (3) Se selecciona otro elemento distinto al seleccionado.
             */
            if (this.options.sel) {
              // Obtener la posición del evento
              this.center = this.pointer(event.originalEvent) || this.center || {x: 0, y: 0};
              var invalidateDraw = false,
                  ret = this.selectItem(this.center, true /* Un resultado */, false /* Mantener si es el mismo resultado */);
              switch (ret.val) {
                // Antes seleccionado && Ahora seleccionado (distinto)
                case 3: {
                  if (!this.primeraSeleccion && !this.changed) {
                    invalidateDraw = true;
                    // Deseleccionar el anterior
                    if (this.selected) {
                      this.selectElement(false);
                    }
                    this.selected = ret.sel;
                    this.sendEvent(this.selected, event);
                    // No se para aqui porque se selecciona igual que cuando no habia seleccionado
                    this.selectElement(true);
                    this.snapshot.shapes.current = _.reject(this.snapshot.shapes.current, function(elem){return !(typeof elem != "undefined" && elem != null) || elem==this.selected;}, this);
                    if (typeof this.selected != "undefined" && this.selected != null) {
                      this.addSnapshot(this.selected);
                    }
                    if (invalidateDraw) {
                      this.frontDraw();
                      this.mainDraw(true);
                    }
                  }
                  this.primeraSeleccion = false;
                  break;
                }
              }
              if (this.changed) {
                this.changed = true;
                if (this.selected) {
                  this.sendEvent(this.selected, event);
                }
                this.trigger("release"); // El release ya hace el repintado del main y front
              }
              this.primeraSeleccion = false;
            }
          }, this))
      },

      /**
       * Libera los eventos.
       */
      unbind: function () {
        $(document.getElementsByClassName("interactive"))
          .css("zIndex", "")
          .off() /* Remove all events */
          .unbind() /* Remove all events */;
      },

      /**
       * Cancela un evento.
       * @param {Object} event - evento a cancelar
       */
      cancel: function (event) {
        event = event.originalEvent || event;

        if (!this.draggable) {
          this.cancelAlways(event);
        }
      },

      /**
       * Cancela un evento en cualquier caso.
       * @param {Object} event - evento a cancelar
       */
      cancelAlways: function (event) {
        event = event.originalEvent || event;

        event.preventDefault();
        event.stopPropagation();
        if (event.gesture)
          event.gesture.preventDefault();
      },

      /**
       * Realiza el dibujo de todos los elementos dado una lista como un string.
       * @param {String} drawing - Listado de los elementos en un string.
       * @param {Boolean} definitive - Indica si es el dibujado sobre el canvas definitivo o es sobre los canvas principal y frontal de dibujado
       */
      draw: function (drawing, definitive) {
        if (drawing && drawing.length) {
          this.snapshot.shapes.current = this.convertObjectSnapshot(JSON.parse(drawing).children);
          this.selected = null;
          this.mainDraw();
          if (definitive) {
            App.request("is:device", {boolean: true, context: this})
              .fail(function(isDevice) {
                var canvasDefinitivo = this.$el.siblings("canvas"); // Canvas sobre el que esta la página
                if (canvasDefinitivo && canvasDefinitivo.length > 0 && this.ctx.resetTransform) {
                  var context = canvasDefinitivo[0].getContext("2d");
                  this.ctx.save();
                  context.save();
                  this.ctx.resetTransform(); // reset transformation
                  context.resetTransform(); // reset transformation
                  context.drawImage(this.canvas, 0, 0);
                  context.restore();
                  this.ctx.restore();
                }
              });
          }
          else {
            this.frontDraw(); // Limpiar por si las flyes (no tiene un coste demasiado alto)
          }
        }
      },

      /**
       * Función que aplica una escala al elemento padre del canvas
       * @param {Integer} - scale escala a aplicar
       */
      scale: function (scale) {
        /*var transform = "scale3d(" + scale + ", " + scale + ", 1)";
        this.$el
          .css("transform", transform)
          .css("-webkit-transform", transform);*/
      },

      /**
       * Genera una figura a partir de su versión comprimida
       * @param figure - datos de la figura comprimidos
       * @returns figura sin comprimir
       * @author Mario Poyato <mario.poyato@inmediastudio.es>
       */
      generaFigura: function(figure) {
        var figureConverted,
            oddPage = this.model.attributes.id % 2 == 0; // Indicar si es una página par
        switch (figure.className) {
          case "TextDirection":
            figureConverted = new TextDirection(figure.attrs, this.options.layer.scale, this.canvas, this.interactiveTexts, oddPage);
            break;

          case "Arrow":
            figureConverted = new Arrow(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "Circle":
            figureConverted = new Circle(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "FillCircle":
            figureConverted = new FillCircle(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "EmptyCircle":
            figureConverted = new EmptyCircle(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "FillCircleInverse":
            figureConverted = new FillCircleInverse(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "EmptyCircleInverse":
            figureConverted = new EmptyCircleInverse(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "Doodle":
            figureConverted = new Doodle(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "Linee":
            figureConverted = new Line(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "Clip":
            figureConverted = new Clip(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "Note":
            figureConverted = new Note(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "Check":
            figureConverted = new Check(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "Cross":
            figureConverted = new Cross(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "Scloud":
            figureConverted = new Scloud(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "Square":
            figureConverted = new Square(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "FillSquare":
            figureConverted = new FillSquare(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "EmptySquare":
            figureConverted = new EmptySquare(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "FillSquareInverse":
            figureConverted = new FillSquareInverse(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "EmptySquareInverse":
            figureConverted = new EmptySquareInverse(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "Triangle":
            figureConverted = new Triangle(figure.attrs, this.options.layer.scale, this.canvas);
            break;

          case "Character":
            figureConverted = new Character(figure.attrs, this.options.layer.scale, this.canvas, this.interactiveTexts, oddPage);
            break;
          case "LinkSquare":
            figureConverted = new LinkSquare(figure.attrs, this.options.layer.scale, this.canvas, this.interactiveTexts, oddPage);
            break;
          case "Rect":
            break;

          case "Group": // Puede ser Subrayado o Resaltado
            figureConverted = {
              attrs: figure.attrs,
              className: "Group",
              children: _.map(figure.children, function (groupFigure) {
                var groupFigureConverted;
                switch (groupFigure.className) {
                  case "Highlight":
                    groupFigureConverted = new Highlight(groupFigure.attrs, this.options.layer.scale, this.canvas);
                    break;
                  case "Underline":
                    groupFigureConverted = new Underline(groupFigure.attrs, this.options.layer.scale, this.canvas);
                    break;
                }
                /*if (context && groupFigureConverted && groupFigureConverted.draw) {
                 groupFigureConverted.draw(context);
                 }*/
                return groupFigureConverted;
              }, this)
            };
            break;

          default:
            break;
        }
        return figureConverted;
      },

      /**
       * Genera un listado de figuras (con sus eventos) a partir de un listado con objetos comprimidos.
       * Además ordena estas figuras para un correcto pintado.
       * @param {Array} drawing - Array de figuras.
       * @param context - contexto del canvas sobre el que dibujar
       * @returns {*}
       */
      convertObjectSnapshot: function (drawing, context) {
        var shapes = _.map(drawing, this.generaFigura, this), max = 0;
        /**
         * Fix para cuando se generan figuras no mapeadas. (Que llegan valores invalidos).
         * @author Mario Poyato <mario.poyato@inmediastudio.es>
         */
        function cleanArray(actual){
            var newArray = new Array();
            for(var i = 0; i<actual.length; i++){
                if (actual[i]){
                  newArray.push(actual[i]);
              }
            }
            return newArray;
        };
        shapes = cleanArray(shapes);

        // Obtener el siguiente identificador global
        _.each(shapes, function(shape){
          var id = shape.id;
          if (!id && shape.attrs) { id = shape.attrs.id; };
          max = Math.max(max, !isNaN(id) ? 0 : id);
        });
        this.snapshot.nextId = Math.max(max + 1, this.snapshot.nextId);
        if (this.snapshot.shapes.left && this.snapshot.shapes.left.length) {
          _.each(this.snapshot.shapes.left, function(shape){
            var id = shape.id;
            if (!id && shape.attrs) { id = shape.attrs.id; };
            max = Math.max(max, isNaN(id) ? 0 : id);
          });
          this.snapshot.nextId = Math.max(max + 1, this.snapshot.nextId);
          this.snapshot.shapes.left = _.map(this.snapshot.shapes.left, this.generaFigura, this).filter(function(elem){ return typeof elem != "undefined" || elem != null });
        }
        if (this.snapshot.shapes.right && this.snapshot.shapes.right.length) {
          _.each(this.snapshot.shapes.right, function(shape){
            var id = shape.id;
            if (!id && shape.attrs) { id = shape.attrs.id; };
            max = Math.max(max, isNaN(id) ? 0 : id);
          });
          this.snapshot.nextId = Math.max(max + 1, this.snapshot.nextId);
          this.snapshot.shapes.right = _.map(this.snapshot.shapes.right, this.generaFigura, this).filter(function(elem){ return typeof elem != "undefined" || elem != null });
        }

        _.each(shapes, function(shape) {
          if (isNaN(shape.id)) {
            shape.id = this.snapshot.nextId++;
          }
        }, this);

        shapes.sort(this.snapshot.sortFunction);
        return shapes;
      }
    });
  }
);
// noinspection JSHint
define(
    'controllers/copy.queue',[
      // libraries
      "jquery",
      "underscore",
      "backbone",
      "marionette",
      // app
      "app",
        // configuration
      "configuration",
      // controllers
      "controllers/queue",
      // views
      "modules/viewer/views/kinetic"
    ],
    function ($, _, Backbone, Marionette, App, configuration, QueueController, KineticView) {
      "use strict";

      var TemporaryView = KineticView.extend({
            el: $("<aside />", {id: "temporary"}).appendTo("body")
          }),
          temporaryView;

      return QueueController.extend({
        initialize: function (options) {
          _.defaults(options || (options = {}), {
            layer: {
              scale: 1,
              size: {
                height: 0,
                width: 0
              }
            },
            model: new Backbone.Model(),
            name: "drawing",
            scale: 1,
            size: {
              height: 0,
              width: 0
            },
            code: "*"
          });

          this.code = options.code;

          if (!temporaryView) {
            App.request("is:device", {test: true}) // Esto DEBE ir con test = true para mejoras en performance
                .fail(function() {
                  _.extend(options, {
                    size: options.layer.size
                  });
                })
                .always(function() {
                  _.extend(options, {
                    scale: 1
                  });
                })
                .always(function () {
                  temporaryView = new TemporaryView(options);
                });
          }

          QueueController.prototype.initialize.apply(this, arguments);
        },

        chain: function (drawing, canvas, self) {
          var that = this;
          var paint = _.bind(function (drawing, canvas) {
            var deferred = $.Deferred();

            // Detecto si es una pagina en blanco o la de un libro
            // y cambio la escala en consecuencia.
            var fixedHeight = configuration.size.height / 2;
            var fixedWidth = configuration.size.width / 2;

                if (self) {
                    self.plainCanvasDraw(drawing, canvas);
                }
                else {
                    // Si es una página en blanco (no pertenece a una unidad)
                    if(that.code == '*' || that.code == 'default'){
                      if (window.mode == 'milibro-preview') {
                        fixedHeight = configuration.size.height;
                        fixedWidth = configuration.size.width;
                      }
                    }
                    // Si es una página de un libro y se realiza un ajuste
                    else if (window.mode == 'milibro-preview' && $("ul#viewer.fit").length>0) {
                        fixedHeight *= 2;
                        fixedWidth *= 2;
                    }
                    temporaryView.model.set("drawing", drawing);
                    temporaryView.render();
                    temporaryView.mainDraw();
                    canvas.getContext("2d").drawImage(
                                temporaryView.$("canvas:last").get(0),
                                0,
                                0,
                                fixedWidth,
                                fixedHeight
                    );
                }

            deferred.resolve(canvas);

            return deferred.promise();
          }, this);

          QueueController.prototype.chain.call(this, function () {
            return paint(drawing, canvas);
          });
        }
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/views/interactive',["underscore", "marionette", "models/page", "modules/viewer/models/shape/square"],
    function (_, Marionette, PageModel, Shape) {
      "use strict";

      return Marionette.ItemView.extend({
        /* attributes */
        // backbone & marionette
        modelEvents: {
          "change:status": function() {
            var DRAWING = PageModel.prototype.STATUS.DRAWING;

            if (this.model.get("isWritable")) {
              //noinspection JSHint
              this.$el.toggle(!!(this.model.get("status") & DRAWING));
            }
          }
        },
        tagName: "aside",
        template: _.template(""),

        // custom
        transforms: null,

        /* methods */

        // backbone & marionette

        initialize: function () {
          this.transforms = [];
          if(this.model.get("type") == "link"){
            if(this.model.get("width") < 0){
              // Esta dibujado de manera inversa en el eje X
              this.model.set("width", Math.abs(this.model.get("width")));
              this.model.set("x", this.model.get("x") - this.model.get("width"));
            }

            if(this.model.get("height") < 0){
              // Esta dibujado de manera inversa en el eje Y
              this.model.set("height", Math.abs(this.model.get("height")));
              this.model.set("y", this.model.get("y") - this.model.get("height"));
            }
          }

          this.model.set("x", Math.round(this.model.get("x")));
          this.model.set("y", Math.round(this.model.get("y")));
        },

        onShow: function () {

          // Fix. If element is not rendered we reload the page.
            if (!this.$el[0].parentElement && !window.__reloaded) {
                window.__reloaded = true;
                setTimeout(function() {
                    App.trigger("goto", window.current + 1);
                    App.trigger("goto", window.current);
                }, 2000);

            }

          this.$el
              .height(this.model.get("height"))
              .width(this.model.get("width"));

          this.translate({
            x: this.model.get("x"),
            y: this.model.get("y")
          });

          var DRAWING = PageModel.prototype.STATUS.DRAWING;

          if (this.model.get("isWritable")) {
            //noinspection JSHint
            this.$el.toggle(!!(this.model.get("status") & DRAWING));
          }
        },

        scale: function(scale) {
          var x = this.model.attributes.x,
            y = this.model.attributes.y,
            w = this.model.attributes.width,
            h = this.model.attributes.height;
          this.transforms[0] = "scale3d(" + scale + ", " + scale + ", 1)";

          this.transform();

          /**
           * FIX: Arreglo para los elementos interactivos que aparecen cuando son de otra página.
           * @author Mario Poyato <mario.poyato@inmediastudio.es>
           */
          var interactive = document.getElementsByClassName("interactive")[0];
          if (interactive) {
            if (!Shape.prototype.isInsideCanvas(x, w, ($(interactive).offset().left + $(interactive).width()) / scale)) {
              this.$el.remove();
            }
          }
        },

        transform: function() {
          this.$el
              .css("transform", this.transforms.join(" "))
              .css("-webkit-transform", this.transforms.join(" "));
        },

        translate: function (position) {
          this.transforms[1] =
              "translate3d(" + position.x + "px, " + position.y + "px, 0)";

          this.transform();
        }

      });
    }
);

define('modules/viewer/template',['handlebars'], function(Handlebars) {

this["JST"] = this["JST"] || {};

this["JST"]["controls.layout"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<nav></nav>\n<section></section>\n<article></article>\n";
  });

this["JST"]["document"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<li></li>";
  });

this["JST"]["interactive.text"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var stack1, functionType="function";


  if (stack1 = helpers.text) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.text); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  if(stack1 || stack1 === 0) { return stack1; }
  else { return ''; }
  });

this["JST"]["note"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var stack1, functionType="function";


  if (stack1 = helpers.text) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.text); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  if(stack1 || stack1 === 0) { return stack1; }
  else { return ''; }
  });

this["JST"]["page"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression;


  buffer += "<aside class=\"interactive\"></aside>\n<p><span>";
  if (stack1 = helpers.title) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.title); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "</span></p>";
  return buffer;
  });

this["JST"]["paginator"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  var buffer = "", stack1, functionType="function", escapeExpression=this.escapeExpression;


  buffer += "<a><i class=\"icon-prev\"></i></a>\n<input value=\"";
  if (stack1 = helpers.page) { stack1 = stack1.call(depth0, {hash:{},data:data}); }
  else { stack1 = (depth0 && depth0.page); stack1 = typeof stack1 === functionType ? stack1.call(depth0, {hash:{},data:data}) : stack1; }
  buffer += escapeExpression(stack1)
    + "\" />\n<a><i class=\"icon-next\"></i></a>";
  return buffer;
  });

this["JST"]["zoom"] = Handlebars.template(function (Handlebars,depth0,helpers,partials,data) {
  this.compilerInfo = [4,'>= 1.0.0'];
helpers = this.merge(helpers, Handlebars.helpers); data = data || {};
  


  return "<i class=\"icon-zoom-center\"></i> <i class=\"icon-zoom\"></i><aside><div></div></aside>";
  });

return this["JST"];

});
// noinspection JSHint
define(
    'modules/viewer/views/interactive/text',["jquery", "modules/viewer/views/interactive", "modules/viewer/template"],
    function ($, InteractiveView, JST) {
      "use strict";

      return InteractiveView.extend({
        className: "text",
        template: JST["interactive.text"],

        onShow: function() {
          InteractiveView.prototype.onShow.apply(this, arguments);

          this.$el
              .css("padding", 20);
        },

        scale: function(scale) {
          InteractiveView.prototype.scale.apply(this, arguments);

          this.$el
              .css("margin-top", -3 * scale)
              .css("margin-left", -3 * scale);
        }
      });
    }
);
//noinspection JSHint
define(
    'modules/viewer/views/interactive/credits',[
      "marionette",
      "template",
      "language",
      "modules/viewer/views/interactive"
    ],
    function (Marionette, JST, language, InteractiveView) {
      "use strict";

      return InteractiveView.extend({
        className: "credits bgcolor-9",
        events: {
          "touch": function(event) {
            event = event.originalEvent || event;

            event.preventDefault();
            event.stopPropagation();

            App.request("credits")
              .done(function(message){
                App.execute("dialog", {
                  class: "creditsDialog debug",
                  color: 9,
                  message: message,
                  title: language("credits"),
                  width: "90%",
                  height: "80%"
                });
              });
          }
        },
        template: JST.credits,
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/views/interactive/link',["modules/viewer/views/interactive"],
    function (InteractiveView) {
      "use strict";

      return InteractiveView.extend({
        attributes: function() {
          var toolTipText = this.model.get("tooltip");
          if(this.model.get("url")){
            toolTipText += ' - ' + this.model.get("url");
          } else if(this.model.get("page")){
            toolTipText += ': ' + this.model.get("page");
          }
          return {
            title: toolTipText
          };
        },
        className: "link",
          events: {
              "click": function (event) {
                  event = event.originalEvent || event;

                  event.preventDefault();
                  event.stopPropagation();

                  switch (this.model.get("linkType")) {
                      case "page":
                          this.trigger("goto", this.model.get("page"));
                          break;

                      case "resource":

                          var that = this;
                          var isAV = false;

                          App.request("is:av")
                              .then(function () {
                                  isAV = true;
                              })
                              .done(function () {

                                  var _open = function (link) {

                                      that.trigger(
                                          "open",
                                          {color: 6, type: "resource", value: link}
                                      );

                                  }

                                  if (isAV) {

                                      var _idRecurso = that.model.get("link").replace("data/RECURSOS/", "").split('/')[0];

                                      var _parent = window.parent;

                                      var _identifier = "openResource";

                                      var ALLOWED = {
                                          htm: "iframe.resource",
                                          html: "iframe.resource",
                                          mp3: "audio",
                                          mp4: "video"
                                      },
                                      extension,
                                      index,
                                      isViewer;

                                      var EXCLUDED = {
                                          mp3: "audio",
                                          mp4: "video"
                                      };

                                      //extension = options.value.toLowerCase();
                                      var _value = that.model.get("link");
                                      extension = _value.toLowerCase();

                                      //youtube videos verification
                                      var isYoutube = extension.toLowerCase().match(/http(s)*:\/\/(www.)*youtube(.com)*/);
                                      var isUrl     = extension.toLowerCase().match(/(http[s]*:\/\/[www.].*[$\/]$)/);
                                      if(isYoutube !== null || isUrl !== null){
                                          _value = _value.replace(configuration.rootData, "");
                                          that.model.set("link", _value);
                                          isViewer = false;
                                      }else{
                                        index = extension.lastIndexOf(".");

                                        extension = (index !== -1) ? extension.substr(index + 1) : "";
                                        isViewer = _.contains(_.keys(ALLOWED), extension);
                                      }

                                      _parent.postMessage(
                                          {
                                              type: 'getResourceURL',
                                              idResource: _idRecurso,
                                              resourcesPath: 'data/RECURSOS/',
                                              filePath: that.model.get("link").split("/"+_idRecurso+"/").pop(),
                                              viewer: (isViewer) ? true : false,
                                              identifier: _identifier
                                          },
                                          '*'
                                      );
                                      var listener = function(event) {
                                          console.log('Event data URL answered: '+event.data.url);
                                          if (typeof event.data != 'undefined' && typeof event.data.url != 'undefined') {
                                              _open(event.data.url);
                                          }
                                          if (window.addEventListener) {
                                              window.removeEventListener("message", listener, false);
                                              listener = null;
                                          } else {
                                              window.detachEvent("onmessage", listener);
                                              listener = null;
                                          }
                                      }
                                      if (window.addEventListener) {
                                          window.addEventListener("message", listener, false);
                                      } else {
                                          window.attachEvent("onmessage", listener);
                                      }
                                      _parent = null;

                                  } 
                                  // else {
                                  //     _open(that.model.get("link"));
                                  // }

                              })
                                .fail(function () {
                                    that.trigger( "open", {
                                        color: 6, 
                                        type: "resource", 
                                        value: that.model.get("link") 
                                    });
                                });
                          break;

                      case "cloudFile":

                          this.trigger(
                              "open",
                              {color: 6, type: "resource", value: this.model.get("link")}
                          );

                          break;

                      case "web":

                          this.trigger(
                              "open",
                              {color: 6, type: "url", value: this.model.get("url")}
                          );
                          break;

                  }
              }
          }
      });
    }
);

// noinspection JSHint
define(
    'modules/viewer/views/interactive/tampon',["underscore", "modules/viewer/views/interactive"],
    function (_, InteractiveView) {
      "use strict";

      return InteractiveView.extend({
      translate: function(position) {
        _.each(position, function(value, key, list) {
            list[key] = value - 50;
          });

          InteractiveView.prototype.translate.call(this, position);
        }
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/views/interactive/clip',[
      "modules/viewer/views/interactive/link",
      "modules/viewer/views/interactive/tampon"
    ],
    function (LinkView, TamponView) {
      "use strict";

      return LinkView.extend({
        translate: TamponView.prototype.translate
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/views/interactive/note',[
      "jquery",
      "language",
      "modules/viewer/views/interactive/tampon",
      "views/dialog",
      "views/html"
    ],
    function ($, language, TamponView, DialogView, HTMLView) {
      "use strict";
      var alreadyOpen = false;

      return TamponView.extend({
        className: "link",
        events: {
          "tap": function (event) {
            //var center;
            event = event.originalEvent || event;
            if(alreadyOpen){
              return;
            } else {
              alreadyOpen = true;
            }

            //center = event.gesture.center;

            event.preventDefault();
            event.stopPropagation();

            var dialogView = new DialogView({
              id: "note-dialog",
              color: 6,
              height: "auto",
              modal: false,
              title: language("note"),
              // top: Math.max(60, center.pageY - 125),
              width: 400
            });

            dialogView.on("close", function(){
              alreadyOpen = false;
            });

            dialogView.show(new HTMLView({
              className: "debug",
              message: this.model.get("message"),
              tagName: "main"
            }));
          }
        }
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/views/interactive/searchResult',["underscore", "modules/viewer/views/interactive"],
    function (_, InteractiveView) {
      "use strict";

      return InteractiveView.extend({
        className: "search_result"
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/views/style',["views/style"],
    function (StyleView) {
      "use strict";

      return StyleView.extend({
        id: "page",
      });
    }
);
//noinspection JSHint
define(
    'regions/append',["marionette"],
    function(Marionette) {
      "use strict";

      return Marionette.Region.extend({
        open: function(view) {
          this.$el.append(view.el);
        }
      });
    }
);
//noinspection JSHint
define(
    'modules/viewer/views/page',[
      // libraries
      "backbone",
      "jquery",
      "marionette",
      "underscore",
      // app
      "app",
      // configuration
      "configuration",
      // controllers
      "controllers/copy.queue",
      // models
      "models/page",
      // views
      "modules/viewer/views/interactive",
      "modules/viewer/views/interactive/text",
      "modules/viewer/views/interactive/credits",
      "modules/viewer/views/interactive/clip",
      "modules/viewer/views/interactive/link",
      "modules/viewer/views/interactive/note",
      "modules/viewer/views/interactive/text",
      "modules/viewer/views/interactive/searchResult",
      "modules/viewer/views/kinetic",
      "modules/viewer/views/style",
      // regions
      "regions/append",
      // templates
      "modules/viewer/template"
    ],
    function (
        Backbone,
        $,
        Marionette,
        _,
        App,
        configuration,
        CopyQueueController,
        PageModel,
        InteractiveView,
        CharView,
        CreditsView,
        ClipView,
        LinkView,
        NoteView,
        TextView,
        SearchresultView,
        KineticView,
        StyleView,
        AppendRegion,
        JST
        ) {
      "use strict";

      var ELEMENTS = [
        "Clip",
        "Character",
        "LinkSquare",
        "Note",
        "TextDirection"
      ];

      var copyController,
          scale = 1;

      App.on("resize", function () {
        scale = 1;
      });

      return Marionette.CompositeView.extend({
        /* constants */

        ELEMENTS: ELEMENTS,

        /* attributes */

        // backbone & marionette
        className: "page",
        itemEvents: {
          "show": function (eventName, view) {
            view.scale(+(1 / scale).toFixed(3));
          }
        },
        itemViewContainer: ".interactive",
        modelEvents: {
          "change:drawing": function () {
            this.paint({cache: false, scale: 1 / scale});
            this.analyze();
          },
          "change:status": function () {
            var STATUS = PageModel.prototype.STATUS;
            //noinspection JSHint
            var offset = Math.abs(
                    this.model.get("status") ^ this.model.previous("status")
            );

            //noinspection JSHint
            if (offset & STATUS.DRAWING) {
              this.paint({cache: false});
              this.collection.each(function (value) {
                value.set("status", this.model.get("status"));
              }, this);
            }
          }
        },
        options: {
          canvas: 250,
          size: {
            height: 0,
            width: 0
          },
          thumbnail: 125
        },
        tagName: "article",
        template: JST.page,
        templateHelpers: {
          src: configuration.root + "assets/images/blank.png"
        },
        triggers: {
          "doubletap": "fit"
        },
        ui: {
          "image": "img",
          "interactive": ".interactive",
          "number": "p"
        },

        // custom
        position: {
          top: 0,
          left: 0
        },

        /* methods */
        initialize: function () {
          // put a delay on thumbnail call
          this.thumbnail = _.chain(this.thumbnail)
              .bind(this)
              .debounce(this.options.thumbnail)
              .value();

          this.collection = new Backbone.Collection();
          this.analyze();

          // Events
          App.on("search:drawResults", this.drawSearchResults, this);
          App.on("search:removeResults", this.removeSearchResults, this);
        },

        getItemView: function (model) {
          var ItemView;

          try {
            //noinspection JSHint
            ItemView = eval(_.capitalize(model.get("type")) + "View");
          }
          catch (e) {
            window.console.debug(e);
            ItemView = InteractiveView;
          }

          return ItemView;
        },

        onClose: function () {
          clearTimeout(this.timeoutID);
        },

        onRender: function () {
          var odd = !this.model.get("id") % 2;
          if (this.model.get("first") || (this.model.get("last") && odd)) {
            this.cover();
          }

          // Si estoy en la última página
          if (this.model.get("last")) {
            this.credits();
          }

          // Fix para imagenes de scloud que no se pintan en Mi Libro ni Mi Cuaderno.
          // En Mi Libro los dibujos estan en 'custom[1]', y en Mi Cuaderno en 'drawing'
          $.when( // Verifico si estoy en Mi Libro o en Mi Cuaderno
              App.request("is:custom", {context: this, not: true}),
              App.request("is:notebook", {context: this, not: true})
          )
          .fail(function(){
            // Es Mi Libro o Mi Cuaderno
            var customDrawings = this.model.get("custom") && this.model.get("custom")[1] || ""; // Mi Libro
            customDrawings += this.model.get("drawing") || ""; // Mi Cuaderno

            if(customDrawings && customDrawings.indexOf("Scloud")){
              _.delay(function(){
                // Hago y deshago fit para mostrar las imágenes.
                // No es elegante, pero funciona.
                App.trigger("fit");
                App.trigger("fit");
              }, 500);
            }
          });
        },

        onShow: function () {
          this.thumbnail();

          var that = this;
//        if (!copyController) {
          var $element = this.$element(),
              size = this.options.size;

          copyController = new CopyQueueController({
            layer: {
              size: size
            },
            scale: $element.width() / size.width,
            size: {
              height: $element.height(),
              width: $element.width()
            },
            code: that.model.get("code") // Paso el codigo de la pagina para detectar si es en blanco o de libro
          });
//        }
        },

        // custom

        // private
        $element: function () {
          var image = this.$("img");

          return (image.length) ? image : this.$("> canvas");
        },

        analyze: function () {
          var _analyze = function (drawing, isWritable, status) {
            try {
              drawing = JSON.parse(drawing).children;
            } catch (Error) {
              drawing = [];
            }

            return _.chain(drawing)
                .map(function (value) {
                  if (!_.contains(ELEMENTS, value.className)) {
                    return false;
                  }

                  var attrs = value.attrs;
          
                  return {
                    message: attrs.raw,
                    height: attrs.height,
                    link: attrs.link,
                    linkType: attrs.action,
                    page: attrs.page,
                    isWritable: isWritable,
                    status: status,
                    text: attrs.raw,
                    tooltip: attrs.tooltip,
                    type: value.className.substr(0, 4).toLowerCase(),
                    url: attrs.url,
                    width: attrs.width,
                    x: attrs.x,
                    y: attrs.y
                  };
                })
                .compact()
                .value();
          };

          this.collection.reset();

          this.collection.add(
              _.reduce(
                  [this.model.get("drawing")].concat(this.model.get("custom")),
                  function (result, value, index) {
                    return result.concat(
                        _analyze(value, index === 0, this.model.get("status"))
                    );
                  },
                  [],
                  this
              )
          );
        },

        fit: function () {
          var FIT = PageModel.prototype.STATUS.FIT;
          //noinspection JSHint
          var fit = this.model.get("status") & FIT,
              scale,
              size = this.options.size;

          if (fit) {
            var height = this.$el.height();

            this.$el.height(
                Math.max(
                    Math.floor(this.$el.width() / size.width * size.height),
                        this.$el.parent().height() - 10
                )
            );
            scale = this.$el.height() / height;
          } else {
            this.$el.css("height", "");
            scale = 1;
          }

          this.thumbnail();
          this.trigger("zoom:after", {scale: scale});

          return true;
        },

        paint: function (options) {
          _.defaults(options || (options = {}), {
            cache: true
          });

          clearTimeout(this.timeoutID);
          this.timeoutID = _.chain(
              function () {
                this.trigger("canvas", options);
              }
          )
              .bind(this)
              .delay(this.options.canvas)
              .value();
        },

        // public
        canvas: function (canvas) {

          var $canvas = $(canvas);
          // We must control if view has been closed cause this method is async.
          if (this.isClosed) {
            return false;
          }

          // Para dibujar fondo de paginas en blanco
          $canvas.addClass(this.model.get("type"));

          this.$element().remove();
          this.$el.append(canvas);

          if (!$canvas.attr("data-drawed")) {
            _.each(_.compact(this.model.get("custom")), function (value) {
              copyController.chain(value, canvas, this);
            }, this);

            //noinspection JSHint
            if ((this.model.get("status") & PageModel.prototype.STATUS.DRAWING) && this.model.get("drawing")) {
              // Representa figuras usando Kinetic
              // copyController.chain(this.model.get("drawing"), canvas);

              // Representa figuras usando Canvas puro
              this.plainCanvasDraw(this.model.get("drawing"), canvas);
            }
            $canvas.attr("data-drawed", true);
          }



          return true;
        },

        cover: function () {
          this.$el.addClass("cover");
        },

        credits: function () {
          var that = this;

          // Verifico si tengo creditos
          App.request("credits")
              .done(
              function (message) {
                if (message) {
                  // We use id 99 for possible duplicates.
                  that.collection.add(
                      new Backbone.Model(
                          {
                            id: 99,
                            type: "credits",
                            width: 200,
                            height: 200,
                            x: -200,
                            y: 150}
                      )
                  );
                }
              }
          );
        },

        drawSearchResults: function (resultSet) {
          _.each(resultSet, function (result) {
            if (result.page.index === this.model.get("index")) {
              this.collection.add(
                  new Backbone.Model({
                    type: "searchResult",
                    isSearchResult: true,
                    width: result.width,
                    height: result.height,
                    x: result.x,
                    y: result.y})
              );
            }
          }, this);
        },

        plainCanvasDraw: function(drawing, canvas){
          var that = this;
          var context = canvas.getContext('2d');
          var drawing = JSON.parse(drawing).children;
          var myScale = 0.5; // No se bien por que... Probar en tablets retina.
          /**
           * Fix 3834
           * He añadido el ajuste de la escala a la unitaria tanto para mi cuaderno como mi libro cuando estan en modo de edición
           * @author Mario Poyato <mario.poyato@inmediastudio.es>
           **/
          // Si es un mi libro y es una página en blanco
          if (!$(canvas).hasClass("bookpage") && (window.mode == 'milibro-edicion' || (window.mode == 'milibro-preview'))) {
            myScale = 1;
          }
          /*
	  App.request("is:custom", {context: this})
            .fail(function(){
              App.request("is:notebook", {context: this}) // ---> Mi Cuaderno
                .done(function(){
                  //myScale = 1;
                })
                .fail(function(){
                  App.request("is:editable", {context: this}) // ---> Mi Libro
                    .done(function(){
                      // Si es un mi libro y es una página en blanco
                      if (!$(canvas).hasClass("bookpage") && (window.mode == 'milibro-edicion' || (window.mode == 'milibro-preview'))) {
                        myScale = 1;
                      }
                    });
                });
            });
	  //*/

          App.request("is:notebook", {context: this}) // ---> Mi Cuaderno
            .done(function(){
              // Si es Mi Cuaderno, cambio la escala
              myScale = 1;
            })
            .always(function(){
              var brush;
              _.each(drawing, function(figure){
                switch(figure.className){
                  case "TextDirection":
                    brush = "text-direction";
                    break;
                  case "Arrow":
                    brush = "arrow";
                    break;
                  case "Circle":
                    brush = "circle"
                    break;

                  case "Scloud":
                    brush = "scloud";
                    break;

                  case "FillCircle":
                    brush = "fill-circle";
                    break;

                  case "EmptyCircle":
                    brush = "transparent-circle";
                    break;

                  case "FillCircleInverse":
                    brush = "fill-circle-inverse";
                    break;

                  case "EmptyCircleInverse":
                    brush = "transparent-circle-inverse";
                    break;

                  case "Doodle":
                    brush = "doodle";
                    break;

                  case "Linee":
                    brush = "line";
                    break;

                  case "Clip":
                    brush = "clip";
                    break;

                  case "Note":
                    brush = "edit";
                    break;

                  case "Check":
                    brush = "tick";
                    break;

                  case "Cross":
                    brush = "cross";
                    break;

                  case "Square":
                    brush = "square";
                    break;
                    
                  case "FillSquare":
                    brush = "fill-square";
                    break;

                  case "EmptySquare":
                    brush = "transparent-square";
                    break;

                  case "FillSquareInverse":
                    brush = "fill-square-inverse";
                    break;

                  case "EmptySquareInverse":
                    brush = "transparent-square-inverse";
                    break;

                  case "Triangle":
                    brush = "triangle";
                    break;
                  /**
                   * FIX: LinkSquare no funcionaba porque no se creaba.
                   * @author Mario Poyato <mario.poyato@inmediastudio.es>
                   */
                  case "LinkSquare":
                    brush = "link-square";
                    break;

                  case "Character":
                  case "Rect":
                    // No hago nada.
                    brush = null;
                    break;

                  case "Group": // Puede ser Subrayado o Resaltado
                    _.each(figure.children, function(groupFigure){
                      if (groupFigure.className == "Highlight" || groupFigure.className =="Underline") {
                        require(
                          ["modules/viewer/models/shape/"+groupFigure.className.toLowerCase()],
                          function (Shape) {
                            var figure = new Shape(this.attrs, myScale, canvas);
                            figure.draw(context);
                            figure = null;
                          }.bind(groupFigure)
                        );
                      }
                    });
                    brush = null;
                    break;

                  default:
                    brush = null;
                    console.warn(figure, "Clase "+figure.className+" no reconocida");
                }
                if (brush) {
                  require(
                    ["modules/viewer/models/shape/" + brush],
                    function (Shape) {
                      var figure;
                      if (brush == 'link-square') {
                        figure = new Shape(this.attrs, myScale, canvas, true/*no rellenar*/);
                      }
                      else {
                        figure = new Shape(this.attrs, myScale, canvas);
                      }
                      figure.staticDraw = true;
                      figure.draw(context);
                      figure = null;
                    }.bind(figure)
                  );
                }
              });

            });
        },

        removeSearchResults: function () {
          // Filtramos la colleccion de objetos interacivos
          // quitando los resultados de busqueda
          var filteredCollection = this.collection.filter(
              function (item) {
                return !item.get("isSearchResult");
              }
          );

          this.collection.reset();
          this.collection.add(filteredCollection);
        },

        thumbnail: function () {
          // We must control if view has been closed cause this method is async.
          if (this.isClosed) {
            return false;
          }

          var size = this.options.size,
              offset = +Math.max(
                      size.height / this.$el.height(),
                      size.width / this.$el.width()
              ).toFixed(2);

          if (offset !== scale) {
            //noinspection JSHint
            var ratio = scale = +Math.max(
                        size.height / this.$el.height(),
                        size.width / this.$el.width()
                ).toFixed(2),
                style = {
                  height: Math.round(size.height / ratio) + "px",
                  width: Math.round(size.width / ratio) + "px"
                },
                view = new StyleView({
                  "#viewer .page canvas": style,
                  "#viewer .page img": style,
                  "#viewer .page span": {
                    width: style.width
                  },
                  "#viewer .page .interactive": style
                });

            // replace style of page
            $("#page").remove();
            $(window.document.head).append(view.render().el);
          }

          // recalculate scale of children forcing show event
          this.children.each(function (value) {
            value.trigger("show");
          });

          // wait to attach on dom the element, then we calculate the position
          _.chain(function () {
            // We must control if view has been closed cause this method is async.
            if (this.isClosed) {
              return false;
            }

            this.position = this.$element().position();

            if (typeof(this.position) !== "undefined") {

                this.ui.interactive
                    .css("top", this.position.top)
                    .css("left", this.position.left);

            }

            return true;
          })
          .bind(this)
          .defer()
          .value();

          if (this.model.get("thumbnail")) {
            var background = configuration.thumbnails +
                this.model.get("thumbnail");
            $("<img />", {src: background}).on("load", _.bind(function () {
              // We must control if view has been closed cause this method is async.
              if (!this.isClosed) {
                this.ui.image.css(
                    "background-image", "url(" + background + ")"
                );
                this.ui.number.remove();
              }
            }, this));
          } else {
            this.ui.number.remove();
          }

          this.paint({scale: 1 / scale});

          return true;
        }
      });
    }
);

//noinspection JSHint
define(
    'modules/viewer/views/document',[
      // libraries
      "underscore",
      "backbone",
      "marionette",
      // views
      "modules/viewer/views/page",
      // templates
      "modules/viewer/template"
    ],
    function (_, Backbone, Marionette, PageView, JST) {
      "use strict";

      var MODE = {
        DOUBLE: 1,
        SINGLE: 0
      };

      return Marionette.CompositeView.extend({
        /* constants */

        MODE: MODE,

        /* attributes */
        // backbone & marionette
        modelEvents: {
          "change:fit": "fit",
          "change:mode": function () {
            this.mode();
            this.fit();
          }
        },
        id: "viewer",
        itemEvents: {
          "show": function () {
            this.$el.scrollTop(0);
          }
        },
        itemView: PageView,
        itemViewContainer: "> li",
        itemViewOptions: function () {
          return {
            canvas: this.options.canvas,
            size: this.options.size,
            thumbnail: this.options.thumbnail
          };
        },
        options: {
          canvas: 250,
          size: {
            height: 0,
            width: 0
          },
          thumbnail: 125
        },
        tagName: "ul",
        template: JST.document,
        ui: {
          "container": "> li"
        },

        /* methods */
        // backbone & marionette
        initialize: function () {
          // Extending page size
          _.each(this.options.size, function (value, key, list) {
            list[key] = value * 2;
          });

          this.model = new Backbone.Model({
            fit: false,
            mode: MODE.DOUBLE
          });
        },

        onRender: function () {
          // Si es tablet, inicio con FIT = true
          App.request("is:device").done(function () {
            setTimeout(function(){
              App.trigger("tabletFit");
            }, 1000);
          });
        },

        appendHtml: function (collectionView, itemView, index) {
          var childrenContainer = collectionView.$(collectionView.itemViewContainer),
              children = childrenContainer.children();

          if (children.size() <= index) {
            childrenContainer.append(itemView.el);
          } else {
            children.eq(index).before(itemView.el);
          }
        },

        // custom

        fit: function () {
          var scale,
              size = this.options.size;

          if (this.model.get("fit")) {
            var elements = this.ui.container.children().size(),
                height = this.ui.container.height();

            this.ui.container.css(
                "height",
                Math.max(
                    Math.floor(
                            (this.ui.container.outerWidth() / elements) /
                            size.width * size.height
                    ),
                    this.$el.height()
                )
            );
            this.$el.css("overflow-y", "auto");

            scale = this.ui.container.height() / this.$el.height();
          } else {
            this.ui.container.css("height", "");
            this.$el.css("overflow-y", "hidden");
            scale = 1;
          }

          _.chain(function() {
            this.$el.scrollTop(0);
          })
              .bind(this)
              .defer()
              .value();
          this.children.call("thumbnail");
          this.trigger("zoom:after", {scale: scale});

          return true;
        },

        mode: function () {
          this.$el.removeClass("single double").addClass(
              (this.model.get("mode") === MODE.SINGLE) ? "single" : "double"
          );
        }
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/controllers/document',["app", "underscore", "marionette", "modules/viewer/views/document"],
    function (App, _, Marionette, DocumentView) {
      "use strict";

      var ZOOM_MAX = 3,
          ZOOM_MIN = 1;

      return Marionette.Controller.extend({
        /* constants */

        ZOOM_MAX: ZOOM_MAX,
        ZOOM_MIN: ZOOM_MIN,

        // attributes
        attributes: {
          limits: {
            scale: null,
            values: null
          },
          status: null,
          translate: {
            x: 0,
            y: 0
          },
          zoom: {
            initial: 1,
            min: 1,
            scale: 1
          }
        },
        options: {
          draw: "default",
          fit: false,
          mode: "double",
          page: null
        },
        timeout: null,

        // method
        initialize: function (options) {
          App.on("fit", this.fit, this);
          App.on("tabletFit", function(){
            this.mode("single");
            this.fit(true);
          }, this);
          // Añadir la función global de obtener el dibujado de una página para que todas las figuras sean accedidas desde cualquier parte del código
          App.reqres.setHandler("getDrawingOfPage", function(index){
            return this.getDrawing(index);
          }, this);

          options.view
            /* own events */
              .on("keydown", this.keydown, this)
              .on("fit", function () {
                this.fit();
              }, this)
              .on("next", function () {
                if (this.attributes.zoom.scale === 1) {
                  this.next();
                }
              }, this)
              .on("previous", function () {
                if (this.attributes.zoom.scale === 1) {
                  this.previous();
                }
              }, this)
              .on("mode", function () {
                if (this.attributes.zoom.scale === 1) {
                  this.mode();
                }
              }, this)
              .on("transform", function (options) {
                if (this.attributes.zoom.scale !== 1) {
                  this.transform({
                    translate: {
                      x: this.attributes.translate.x + options.deltaX,
                      y: this.attributes.translate.y + options.deltaY
                    }
                  });
                }
              }, this)
              .on("translate", function (options) {
                if (this.attributes.zoom.scale !== 1) {
                  this.translate(
                          this.attributes.translate.x + options.deltaX,
                          this.attributes.translate.y + options.deltaY
                  );
                }
              }, this)
              .on("zoom", this.zoom, this)
            /* itemview events */
              .on("zoom:after", _.bind(function (options) {
                _.defaults(options || (options = {}), {
                  scale: 1
                });

                this.trigger("zoom", options.scale);
              }, this))
//              .on("itemview:show", function () {
//                var status = this.attributes.status;
//                this.fit(this.options.fit);
//                this.attributes.status = status;
//              }, this)
//              .on("item:removed", function () {
//                this.$("li").scrollTop(0);
//              })
            /* interactive views events */
              .on("itemview:itemview:goto", function (parentView, view, page) {
                this.goto(page);
              }, this);
          options.view.$el.addClass(options.mode || "double");
        },

        onClose: function () {

        },

        // private

        offset: function (page) {
          try {
            var even,
              first = this.options.book.first().get("id"),
                last = this.options.book.last().get("id"),

            page = Math.min(Math.max(page, first), last);

            if (this.options.mode === "single") {
              return [page];
            }

            page = Math.max(Math.floor(page / 2) * 2, first);
            even = (page % 2);
            if ((page === first && even) || page === last) {
              return [page];
            }

            return [page, page + 1];
          }
          catch (err) {
            return null; // En caso de que este vacio
          }
        },

        // public

        draw: function (book) {
          this.options.book = book;
          this.goto(this.options.page, true);
        },

        fit: function (force) {
          if (this.options.fit === force) {
            return false;
          }

          _.each(viewer.childNodes, function(pag) {
            pag.style.top = pag.style.left = "0px";
          });

          force = (_.isBoolean(force)) ? force : !this.options.fit;

          // removing status
          this.attributes.status = null;

          if (force)
            $("ul#viewer").addClass("fit");
          else
            $("ul#viewer").removeClass("fit");

          // Changing zoom to 1 adapting zoom bar
          if (this.attributes.zoom.scale !== 1) {
            this.zoom(1);
          }

          this.options.fit = force;
          this.options.view.model.set("fit", force);
          this.trigger("fit", force);

          return true;
        },

        goto: function (page, force, page_search) {
          if (page_search) {

              var page_by_title;
              this.options.book.models.forEach(function(element) {
                  if(element.attributes.title == page){
                      page_by_title = element;
                  }
              });

              this.trigger( "goto", page_by_title ? page_by_title.attributes.id : (parseInt(page, 10) || this.options.page), force );
              return true;
          }

          if (this.options.page === page && !force) {
            return false;
          }

          var children,
              pages = this.offset(page);

          // removing limits before generated for new pages
          this.attributes.limits.scale = null;

          if (pages) {
            this.options.page = pages[0];

            if (!this.options.fit) {
              this.zoom(1);
//            this.trigger("zoom", ZOOM_MIN);
            }
            children = this.options.view.children.length;
            this.page(pages, force);

            if (children && this.options.view.children.length !== children &&
              this.options.fit) {
              this.options.view.fit();
            }

            this.trigger( "goto", pages[0] );
			  return true;

            setTimeout( function(){
                _.each( document.querySelector("ul#viewer").childNodes, function( pag ){
                    pag.style.top = pag.style.left = "0px";
                });
            }, 1000);

            return true;
          }
        },

        keydown: function (keyCode) {
          if (_.contains([37, 38], keyCode)) {
            this.previous();
          } else if (_.contains([39, 40], keyCode)) {
            this.next();
          } else if (keyCode === 65) {
            this.fit();
          } else if (keyCode === 77) {
            this.mode();
          }
        },

        limits: function (scale) {
          var attributes = this.attributes,
              height,
              limits = [],
              options = this.options,
              width = 0;

          scale = scale || attributes.zoom.scale;

          if (attributes.limits.scale === scale && attributes.limits.values) {
            return attributes.limits.values;
          }

          height = options.view.children.first().$element().height();
          options.view.children.each(function (value) {
            width += value.$element().width();
          });

          height += 10;
          width += 10;

          height = (height * scale - options.view.$el.height()) / 2;
          width = (width * scale - options.view.$el.width()) / 2;

          limits[0] = height;
          limits[1] = -width;
          limits[2] = -height;
          limits[3] = width;

          attributes.limits.values = limits;

          return limits;
        },

        mode: function (mode) {
          var that = this;

          // Verifico si estoy en Mi Libro o en Mi Cuaderno y no permito pagina doble
          $.when(
              App.request("is:custom", {context: this, not: true}),
              App.request("is:notebook", {context: this, not: true})
          )
          .fail(function(){
            mode = "single"; // Forzamos el modo simple porque es Mi Libro o Mi Cuaderno
          })
          .always(function(){
            var MODE = DocumentView.prototype.MODE;
            var pages;

            if (that.options.mode === mode) {
              return false;
            }

            mode =
                mode || ((that.options.mode === "single") ? "double" : "single");

            // removing status
            that.attributes.status = null;
            that.options.mode = (mode === "double") ? "double" : "single";

            pages = that.offset(that.options.page);

            if (pages) {
              that.options.page = pages[0];
              that.page(pages);
              that.options.view.model.set(
                "mode",
                (that.options.mode === "single") ? MODE.SINGLE : MODE.DOUBLE
              );


              if (that.attributes.zoom.scale !== 1) {
                that.zoom(1);
              }
              that.options.view.children.call("thumbnail");
              that.trigger("mode", that.options.mode);
            }

            return true;
          });
        },

        next: function () {
          // Si no tengo heramientas abiertas
          if(!$("#tools").is(":visible")){
            // Elimino resultados de busqueda
            App.trigger("search:removeResults");

            var options = this.options,
                double = options.mode === "double",
                first = options.page === 1,
                page = options.page;

            App.request('is:device')
              .fail(function() {
                _.each(document.querySelector("ul#viewer").childNodes, function(pag) {
                  pag.style.top = pag.style.left = "0px";
                });
              });

            this.goto(this.offset(page + 1 + (!first && double))[0]);
          }
        },

        getDrawing: function(index) {
          var drawing;
          try {
            drawing = this.options.book.get(index).get("drawing");
          }
          catch (err) {
            drawing = null;
          };
          return drawing;
        },

        page: function (pages, force) {
          if(force){
            this.options.view.collection.reset();
          }

          this.options.view.collection.set(_.map(pages, function (value) {
            //console.log("Obteniendo modelo de página: "+value);
            return this.options.book.get(value);
          }, this));
        },

        previous: function () {
          // Si no tengo heramientas abiertas
          if(!$("#tools").is(":visible")){
            // Elimino resultados de busqueda
            App.trigger("search:removeResults");

            var options = this.options,
                double = options.mode === "double",
                first = options.page === 1,
                page = options.page;

            this.goto(this.offset(page - 1 - (!first && double))[0]);
          }
        },

        resize: function () {
          this.zoom(1);
          this.options.view.fit(this.options.fit);
          this.options.view.children.call("thumbnail");
        },

        transform: function (options) {
          var attributes = this.attributes;

          _.defaults(options || (options = {}), {
            scale: attributes.zoom.scale,
            translate: attributes.translate
          });

          _.each(options.translate, function (value, key, list) {
            list[key] = value / options.scale;
          });

          var children = "translate3d(" + options.translate.x + "px, " +
                  options.translate.y + "px, 0)",
              parent = "scale3d(" + options.scale + ", " + options.scale +
                  ", 1)";

          this.options.view.ui.container
              .css("transform", parent)
              .css("-webkit-transform", parent)
              .find("> *")
              .css("transform", children)
              .css("-webkit-transform", children);
          
          // Si el zoom está a 1 (inicial), la posición de las páginas o página será centrada
          if(options.scale === 1) {
            this.options.view.ui.container
                .css("left", options.translate.x + "px")
                .css("top", options.translate.y + "px");
          }
        },

        translate: function (x, y, options) {
          var attributes = this.attributes,
              limits = this.limits(options && options.scale),
              size = this.size(),
              view = Marionette.getOption(this, "view");

          x = x || attributes.translate.x;
          y = y || attributes.translate.y;

          _.defaults(options || (options = {}), {
            x: (size.width < view.$el.width()) ? 0 :
                Math.min(Math.max(x, limits[1]), limits[3]),
            y: (size.height < view.$el.height()) ? 0 :
                Math.min(Math.max(y, limits[2]), limits[0])
          });

          this.transform({
            translate: {
              x: options.x,
              y: options.y
            }
          });
          attributes.translate = options;
        },

        size: function () {
          var attributes = this.attributes,
              height = 0,
              width = 0;

          Marionette.getOption(this, "view").children.each(function (value) {
            var $element = value.$element();

            if ($element) {
              if (!height) {
                height = $element.height();
              }

              width += $element.width();
            }
          });

          return {
            height: height * attributes.zoom.scale,
            width: width * attributes.zoom.scale
          };
        },

        zoom: function (scale, options) {
          if (this.options.fit) {
            this.fit(false);
          }

          if (this.attributes.zoom.scale === scale) {
            return false;
          }

          var view = this.options.view;

          _.defaults(options || (options = {}), {
            scale: +Math.min(Math.max(scale, ZOOM_MIN), ZOOM_MAX).toFixed(3)
          });

          // removing status
          this.attributes.status = null;

          if (options.scale === 1) {
            options.translate = {
              x: 0,
              y: 0
            };

            delete options.origin;
          }

          if (options.origin) {
            options.translate = {
              x: -(options.origin.pageX - view.$el.width() / 2) *
                  options.scale + this.attributes.translate.x,
              y: -(options.origin.pageY - view.$el.height() / 2) *
                  options.scale + this.attributes.translate.y
            };
          }

          this.attributes.zoom.scale = options.scale;
          this.trigger("zoom", options.scale);

          _.chain(function() {
            this.transform(options);

            if (options.scale !== 1) {
              this.translate(
                  options.translate.x,
                  options.translate.y,
                  {scale: options.scale}
              );
              $("ul#viewer").addClass("draggable");
            }
            else {
              $("ul#viewer").removeClass("draggable");
            }

            this.options.view.children.each(function (value) {
              value.paint({scale: options.scale});
            });
          })
              .bind(this)
              .delay(150)
              .value();

          return true;
        }
      });
    }
);

// noinspection JSHint
define(
    'modules/viewer/views/desktop.document',[
      // libraries
      "jquery",
      "underscore",
      "marionette",
      // views
      "modules/viewer/views/document"
    ],
    function ($, _, Marionette, DocumentView) {
      "use strict";

      return DocumentView.extend({
        className: "double animate",
        events: {
          "release": function() {
            this.flag = false;
            this.$el.addClass("animate");
          },

          "touch": function() {
            this.flag = true;
            this.$el.removeClass("animate");
          },

          "dblclick": function(event) {
            if (document.getElementsByClassName("kinetic").length == 0) {
              var page = (($("aside.interactive")[1] == event.target) ? this.options.collection.models[1] : this.options.collection.models[0]);
              if (!isNaN(page.attributes.id)) {
                page = page.attributes.id;
              }
              else {
                page = page.attributes.number;
              }
              this.trigger("mode");
              App.trigger("fit");
              App.trigger("goto",page);
            }
          }
        },
        triggers: {
          "swipeleft": "next",
          "swiperight": "previous"
        },

        delegateEvents: function () {
          //noinspection JSAccessibilityCheck
          Marionette.View.prototype.delegateEvents.apply(
              this,
              Array.prototype.slice.call(arguments)
          );

          $(document.body).on(
              "keydown.viewer",
              _.bind(function (event) {
                event = event.originalEvent || event;

                this.trigger("keydown", event.keyCode);
              }, this)
          );
        },

        undelegateEvents: function () {
          //noinspection JSAccessibilityCheck
          Marionette.View.prototype.undelegateEvents.apply(
              this,
              Array.prototype.slice.call(arguments)
          );

          $(document.body).off("keydown.viewer");
        }
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/views/device.document',[
      // libraries
      "underscore",
      "marionette",
      // views
      "modules/viewer/views/document"
    ],
    function (_, Marionette, DocumentView) {
      "use strict";

      var MINDRAG = 0.12;

      return DocumentView.extend({
        /* constants */
        MINDRAG: MINDRAG,

        /* attributes */
        // backbone & marionette
        events: {
          "swiperight": function(event) {
            event = event.originalEvent || event;
            event.preventDefault();
            event.stopPropagation();
            this.trigger("previous");
          },

          "swipeleft": function(event) {
            event = event.originalEvent || event;
            event.preventDefault();
            event.stopPropagation();
            this.trigger("next");
          },

          "dbltouch": function(event) {
            if (document.getElementsByClassName("kinetic").length == 0) {
              var page = (($("aside.interactive")[1] == event.target) ? this.options.collection.models[1] : this.options.collection.models[0]);
              if (!isNaN(page.attributes.id)) {
                page = page.attributes.id;
              }
              else {
                page = page.attributes.number;
              }
              this.trigger("mode");
              App.trigger("fit");
              App.trigger("goto",page);
            }
          }
        },
        options: {
          canvas: 500,
          thumbnail: 250
        },
        scale: 1
      });
    }
);
//noinspection JSHint
define(
    'modules/viewer/viewer.document',[
      // app
      "app",
      "encrypt",
      // parent
      "modules/viewer/viewer",
      // configuration
      "configuration",
      // models
      "models/page",
      "models/page.collection",
      // controllers
      "modules/viewer/controllers/document",
      "controllers/queue",
      // views
      "modules/viewer/views/desktop.document",
      "modules/viewer/views/device.document",
      "modules/viewer/views/kinetic",
      // helpers
      "language"
    ],
    function (
        App,
        Encrypt,
        parent,
        configuration,
        PageModel,
        PageCollectionModel,
        Controller,
        QueueController,
        DesktopView,
        DeviceView,
        KineticView,
        language
        ) {
      "use strict";

      return App.module(
          "viewer.document",
          function (module, App, Backbone, Marionette, $, _) {
            /* Presets module */
            this.startWithParent = false;

            /* Private */

            /* Public */

            // Layout
            this.layout = parent.layout.get("main");

            this.addInitializer(function (options) {

              this.options = _.defaults(options || (options = {}), {
                draw: "default",
                fit: false,
                mode: "double",
                page: 1,
                size: {
                  height: 0,
                  width: 0
                },
                zoom: 1
              });

              var ItemViewType = DesktopView,
                  documentView;

              App.request("is:device", {context: this})
                  .done(function () {
                    ItemViewType = DeviceView;
                  })
                // Then is like an always and other request.
                  .then(function () {
                    return App.request("chapters", {context: this});
                  }, function () {
                    return App.request("chapters", {context: this});
                  })
                  .done(function (chapters) {
                    // Creating controllers & views
                    var pages = chapters.pages(),
                        page;

                    documentView = new ItemViewType(_.extend({
                      collection: new PageCollectionModel()
                    }, options));

                    this.controller = new Controller(_.extend({
                      book: pages,
                      view: documentView
                    }, options));

                    // Setting view
                    this.layout.show(documentView);

                    // Substitute handler for good ones.
                    // Add handler to request
                    App.reqres.removeHandler("viewer:is:mode:double");
                    App.reqres.removeHandler("viewer:is:mode:single");
                    App.reqres.removeHandler("viewer:page");
                    App.reqres.removeHandler("viewer:pages");

                    App.reqres.setHandlers({
                      "viewer:is:mode:double": function (options) {
                        var deferred = $.Deferred();

                        _.defaults(options || (options = {}), {
                          boolean: false,
                          context: deferred
                        });

                        App.request("viewer:is:mode:single", options)
                            .done(function (isModeSingle) {
                              if (options.boolean) {
                                isModeSingle =
                                    (_.has(options, "test")) ? !options.test :
                                        isModeSingle;

                                deferred.resolveWith(
                                    options.context,
                                    [!isModeSingle]
                                );
                              } else {
                                deferred.rejectWith(options.context);
                              }
                            })
                            .fail(function () {
                              deferred.resolveWith(options.context);
                            });

                        return deferred.promise();
                      },

                      "viewer:is:mode:single": function (options) {
                        var deferred = $.Deferred(),
                            is;

                        _.defaults(options || (options = {}), {
                          boolean: false,
                          context: deferred
                        });

                        is = (_.has(options, "test")) ? options.test :
                            module.controller.options.mode === "single";

                        if (options.boolean || is) {
                          deferred.resolveWith(options.context, [is]);
                        } else {
                          deferred.rejectWith(options.context);
                        }

                        return deferred.promise();
                      },

                      "viewer:page": function () {
                        var deferred = $.Deferred();

                        deferred.resolve(module.controller.options.page);

                        return deferred.promise();
                      },

                      "viewer:pages:view": function (options) {
                        var deferred = $.Deferred();

                        _.defaults(options || (options = {}), {
                          context: deferred
                        });

                        deferred.resolveWith(
                            options.context,
                            [documentView.children]
                        );

                        return deferred.promise();
                      }
                    });

                    // Events & handlers
                    documentView
                        .on("itemview:canvas", (function () {
                          var last = 1;

                            return function (view, options) {

                                var page = false;

                                $.ajax({

                                    dataType: "text",
                                    url: configuration.source + view.model.get("number") + ".data"

                                }).done(function (data) {

                                    page = data;

                                }).fail(function () {

                                    console.log("Could not read page file");

                                }).always(function () {

                                    window.current = view.model.get("number");

                                    var interactive = view.$el.find('> .interactive');

                                    var viewerScale = parseFloat($("#viewer > li").css('transform').split('(')[1].split(')')[0].split(',')[0]);
                                    if (typeof(window.master.book.pages[view.model.get("number")-1]) !== "undefined") {
                                        var scale = interactive.height() / window.master.book.pages[view.model.get("number")-1].size.height;
                                    } else {
                                        var scale = 1;
                                    }

                                    page = (page)?Encrypt.decrypt(page):"";
                                    page = page.replace(/#PATH#/g, configuration.source);
                                    page = page.replace(/#SCALE#/g, scale);

                                    //view.$el.find('> img').remove();
                                    view.$el.find('> canvas').remove();
                                    view.$el.find('> .html-page').remove();

                                    view.canvas($("<canvas style='position:absolute; z-index: 1;'></canvas>").attr("width", view.options.size.width / 2).attr("height", view.options.size.height / 2).get(0));
                                    view.$el.append("<div class='html-page' style='width: " + interactive.width() + "px; height: " + interactive.height() + "px;'>" + page + "</div>");

                                    interactive.css('left', (view.$el.find('> .html-page').offset().left - view.$el.offset().left) / viewerScale);
                                    interactive.css('top', (view.$el.find('> .html-page').offset().top - view.$el.offset().top) / viewerScale);

                                    view.$el.find('> canvas').css('top', interactive.css('top'));

                                    last = options.scale;

                                });

                                this.render(view, options);

                            };

                        })(), this)
                        .on("itemview:fit", function (view) {
                          if ($(".kinetic").length) {
                            return false;
                          }

                          App.request("is:device", {context: this})
                              .fail(function () {
                                var controller = this.controller,
                                    attributes = controller.attributes,
                                    mode,
                                    status = attributes.status,
                                    search = {};

                                documentView.children.each(function (value) {
                                  search[value.model.get("id")] =
                                      value.collection.where({
                                        type: "searchResult"
                                      });
                                });

                                if (attributes.zoom.scale > 1) {
                                  controller.zoom(1);
                                } else {
                                  mode = controller.options.mode;
                                  controller.mode(status || "single");
                                  controller.fit();
                                  controller.goto(view.model.get("id"));
                                  documentView.children.each(function (value) {
                                    var searchId = search[value.model.get("id")];
                                    if (searchId) {
                                      value.collection.add(searchId);
                                    }
                                  });
                                  attributes.status = mode;
                                }
                              })
                              .then(function () {
                                return App.request(
                                    "is:device:landscape",
                                    {context: this}
                                );
                              })
                              .done(function () {
                                this.controller.fit();

                                if (!documentView.model.get("fit")) {
                                  this.controller.mode("double");
                                } else {
                                  this.controller.mode("single");
                                  this.controller.goto(view.model.get("id"));
                                }
                              });
                        }, this)
                        .on("itemview:itemview:open", function (
                            parentView, view, options
                            ) {
                          _.defaults(options || (options = {}), {
                            chapter: "U0",
                            index: 0
                          });
                          App.execute("open", options);
                        });

                    this.controller
                        .on("all", function () {
                          App.trigger.apply(
                              App,
                              Array.prototype.slice.apply(arguments)
                          );
                        });

                    parent
                        .on("all", function (eventName) {
                          //noinspection JSHint
                          this[eventName] &&
                          this[eventName].apply(
                              this,
                              Array.prototype.slice.apply(arguments).slice(1)
                          );
                        }, this.controller);

                    App
                        .on("draw:load", this.load, this)
                        .on("goto", this.controller.goto, this.controller)
                        .on("goto", this.save, this)
                        .on("resize", this.resize, this);

                    // Initialize
                    App.request("is:av", {context: this})
                        .fail(function () {
                          page = pages.first();
                        })
                        .then(function () {
                          PageModel = PageModel.extend({
                            url: "page"
                          });

                          // setting id to 1 for fallback (null page)
                          page = new PageModel({id: 1});

                          return page.fetch();
                        })
                        .always(function () {
                          module.controller.goto(page.get("id"), true);
                          module.resize();
                          parent.trigger("load:after", page.get("id"));

                        });
                  })
                  .fail(function () {
                    window.console.error("Impossible to load chapters");
                  });
            });

            this.addFinalizer(function () {
              App
                  .off("fit", this.controller.fit)
                  .off("goto", this.controller.goto)
                  .off("mode", this.controller.mode)
                  .off("resize", this.resize);
            });

            /* Events */

            this.load = function (name) {
              App.request("pages", {context: this, draw: name, cache: false})
                  .done(function (pages) {
                    App.execute("module:stop", "Tools");
                    App.execute("module:stop", "viewer.document.draw");

                    App.trigger("hide:custom");
                    this.controller.draw(pages);

                    App.execute("dialog", {
                      class: "debug",
                      color: 6,
                      message: language("tools.loadDraw"),
                      height: "auto",
                      title: language("info"),
                      width: 600
                    });
                  });
            };

            this.render = function (view, options) {
              var documentController,
                  model = view.model;

              _.defaults(options || (options = {}), {
                origin: {x: 0, y: 0},
                scale: 1
              });

              App.request("document", {code: model.get("code")})
                  .then(function (document) {
                    documentController = document;

                    return $.when(
                        App.request("is:device", {boolean: true})
                    );
                  })
                  .done(function (isDevice) {
                    isDevice = !!isDevice;

                    var information = window.master.book.pages[model.get("index")].size;

                    _.defaults(options || (options = {}), {
                      height: Math.round(information.height * options.scale),
                      width: Math.round(information.width * options.scale),
                      zoom: options.scale * ((options.scale < 1) ? 2 : 1)
                    });

                    if (!isDevice && options.zoom < 1) {
                      options.zoom = 1;
                    }

                    if (_.isFunction(options.callback)) {
                      options.callback(documentController, options);
                    }
                  })
                  .fail(function () {
                    App.execute("error", language("couldNotLoadFile"));
                  });
            };

            /**
             * Resize document view.
             */
            this.resize = function () {
              this.controller.resize();

              /* If is not mybook & is not landscape book then is a regular book */
              $.when(
                  App.request("is:custom", {context: this, not: true}),
                  App.request("is:notebook", {context: this, not: true}),
                  App.request("is:document:portrait", {context: this})
              )
                  .done(function () {
                    // Regular book, only modified when we are on device.
                    App.request("is:device", {context: this[0]})
                        .done(function () {

                          // =======================================================
                          // == Fix para guardar cambios antes de girar la        ==
                          // == pantalla en tablets, solo cuando las herramientas ==
                          // == estan abiertas. De esta forma solucionamos        ==
                          // == muchos problemas al dibujar.                      ==
                          // =======================================================

                          // Para evitar falsas alarmas al abrir el teclado, verifico
                          // también que se haya cambiado la orientación del dispositivo.
                          if($("#tools").is(":visible") && !!window.orientation_changed){
                            App.trigger("draw:save");
                            window.orientation_changed = false;

                            setTimeout(function(){
                              // Esto corrige los dibujos con escala y ubicacion erronea
                              // luego de guardar y cambiar orientacion.
                              if(App.viewer.document.draw){
                                App.execute("module:stop", "viewer.document.draw");
                              }
                            }, 1000);
                          }
                          // =======================================================

                          this.controller.fit(false);

                          App.request("is:device:portrait", {context: this})
                              .done(function () {
                                this.controller.mode("single");
                              })
                              .fail(function () {
                                this.controller.mode("double");
                              });
                        })
                        .fail(function () {
                          this.controller.zoom(1);
                        });
                  })
                  .fail(function () {
                    // Is custom or landscape book
                    this.controller.mode("single");
                  });
            };

            /**
             * Save current page on local/AV Storage.
             * @param page
             */
            this.save = function (page) {
              $.when(
                  App.request("page", {context: this, model: true, page: page}),
                  App.request("is:av", {context: this}),
                  App.request("is:custom", {context: this, not: true})
              )
                  .done(function (page) {
                    var cloned = page.clone();
                    cloned.clear();
                    cloned.set({id: page.get("id")});

                    cloned.url = "page";
                    cloned.save();
                  });
            };
          }
      );
    }
);
// noinspection JSHint
define(
    'controllers/builder',["marionette"],
    function (Marionette) {
      "use strict";

      return Marionette.Controller.extend({
        construct: function () {}
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/controllers/kinetic.builder',["underscore", "controllers/builder", "modules/viewer/views/kinetic"],
    function (_, Builder, KineticView) {
      "use strict";

      Builder = Builder.extend({
        construct: function (pageView, isDevice) {

          var interactive = pageView.$el.find('.interactive');

          var $element = pageView.$element(),
              position = pageView.position,
              size = pageView.options.size,
              options = {
                model: pageView.model,
                position: {
                  top: interactive.css('top'),
                  left: interactive.css('left') + 4
                },
                layer: {
                  size: size
                },
                scale: $element.width() / size.width,
                size: size
              };

          if (isDevice) {
            _.extend(options, {
              scale: 1,
              size: {
                height: $element.height(),
                width: $element.width()
              }
            });
          }

          var kineticView = new KineticView(options),
              pageInfo = kineticView.model.attributes,
              left = null,
              right = null;
          // -----------------------------------------------------------------------------------------------------------
          // - Reutilizar el mismo canvas -
          //kineticView.layer = $element.get(0);
          //$(kineticView.layer).data("data-tools-drawed",drawedTools);
          // -----------------------------------------------------------------------------------------------------------
          // -----------------------------------------------------------------------------------------------------------
          // - Utilizar otro canvas para las herramientas -
          
          // Canvas definitivo de las herramientas
          kineticView.canvas = document.createElement("canvas");
          kineticView.ctx = kineticView.canvas.getContext('2d');
          kineticView.canvas.className = "mainTools";
          kineticView.canvas.height = $element.height();
          kineticView.canvas.width = $element.width();
          kineticView.canvas.style.backgroundColor = "transparent";
          kineticView.canvas.style.top = "0px";
          kineticView.canvas.style.left = "0px";
          kineticView.canvas.style.border = "0px";
          kineticView.canvas.style.margin = "0px";
          kineticView.canvas.style.padding = "0px";
          kineticView.canvas.style.position = "absolute";

          // Canvas de dibujado frontal
          kineticView.frontcanvas = document.createElement("canvas");
          kineticView.frontcanvas.className = "frontTools";
          kineticView.fctx = kineticView.frontcanvas.getContext('2d');
          kineticView.frontcanvas.height = $element.height();
          kineticView.frontcanvas.width = $element.width();
          kineticView.frontcanvas.style.backgroundColor = "transparent";
          kineticView.frontcanvas.style.top = "0px";
          kineticView.frontcanvas.style.left = "0px";
          kineticView.frontcanvas.style.border = "0px";
          kineticView.frontcanvas.style.margin = "0px";
          kineticView.frontcanvas.style.padding = "0px";
          kineticView.frontcanvas.style.position = "absolute";
          kineticView.frontcanvas.style.zIndex = "3"; // Debe estar siempre por encima del canvas principal de herramientas y de las herramientas interactivas

          kineticView.shape = [];

          // Comprobar que hay 2 páginas
          if ($("article.page").length == 1) {
            if (!pageInfo.first && !pageInfo.last) {
              /**
               *     En caso de ser una página impar se debe cargar la información de la página posterior
               *      ______________  _____________
               *     |              V              |
               *     |              |              |
               *     |              |   #######    |
               *     |    CURRENT   |   # GET #    |
               *     |              |   #######    |
               *     |              |              |
               *     |_____________ ^______________|
               */
              if (!(pageInfo.id % 2)) {
                // Obtener la información de la página posterior
                right = App.reqres.request("getDrawingOfPage", pageInfo.code + "-" + (pageInfo.number + 1));
                // Se convierte el stringify del JSON a un objeto de javascript
                if (!right) {
                  right = [];
                }
                else {
                  right = JSON.parse(right);
                  if (right.children && right.className == "Layer") {
                    right = right.children;
                  }
                }
                console.debug("[DEBUG] Right page: "+pageInfo.id+" loaded  number "+pageInfo.code + "-" + (pageInfo.number + 1)+" with "+right.length+" elements");
              }
              /**
               *     En caso de ser una página par se debe cargar la información de la página anterior
               *      ______________  _____________
               *     |              V              |
               *     |              |              |
               *     |   #######    |              |
               *     |   # GET #    |    CURRENT   |
               *     |   #######    |              |
               *     |              |              |
               *     |_____________ ^______________|
               */
              else {
                // Obtener la información de la página anterior
                left = App.reqres.request("getDrawingOfPage", pageInfo.code + "-" + (pageInfo.number - 1));
                // Se convierte el stringify del JSON a un objeto de javascript
                if (!left) {
                  left = [];
                }
                else {
                  left = JSON.parse(left);
                  if (left.children && left.className == "Layer") {
                    left = left.children;
                  }
                }
                console.debug("[DEBUG] Left page: "+(pageInfo.id-1)+" loaded  number "+pageInfo.code + "-" + (pageInfo.number - 1)+" with "+left.length+" elements");
              }
            }
          }
          kineticView.snapshot = {
            shapes: {
              current: [],
              left: left,
              right: right
            },
            nextId: 0, // Valor del siguiente identificador del elemento que se introducirá en la lista
            sortFunction: function (a, b) { return a.order - b.order; }
          };

          $(kineticView.canvas).data("data-tools-drawed",false);
          // -----------------------------------------------------------------------------------------------------------
          return kineticView;
        }
      });

      return new Builder();
    }
);

// noinspection JSHint
define(
    'modules/viewer/models/draw.collection',["backbone"],
    function (Backbone) {
      "use strict";

      return Backbone.Collection.extend({
        url: "draw"
      });
    }
);
/**
 * Figura del texto a seleccionar
 * @class SelectionBox
 * @extends EmptySquare
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/text-select',[
    	"app",
    	"underscore",
    	"modules/viewer/models/shape/transparent-square"
    ],
    function (App, _, EmptySquare) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var SelectionBox = function (options, scale, canvas) {
        // Default properties of KineticJS
        _.defaults(options || (options = {}), {
          className: "SelectionBox",
          fill: "#888888",
          opacity: 0.5,
          strokeWidth: 0
        });
        EmptySquare.call(this,options,scale,canvas);
      };

      SelectionBox.prototype = _.extend({}, EmptySquare.prototype, {
        /**
         * Acción del objeto.
         *
         * @property action
         * @type {String}
         * @default "textSelect"
         */
        action: "textSelect" // Extender y cambiar para otras acciones
      });

      return SelectionBox;

    }
);
// noinspection JSHint
/**
 * Clase para la acción de copia.
 * @class Copy
 * @extends TextSelect
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/copy',["marionette", "underscore", "modules/viewer/models/shape/text-select"],
    function (Marionette, _, TextSelect) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var TextCopy = function (options, scale, canvas) {
        _.defaults(options || (options = {}), {
          className: "TextCopy",
          strokeWidth: 0,
          opacity: 0.7,
          fill: "#32B6FF"
        });
        TextSelect.call(this,options, scale, canvas);
        if ("undefined" !== typeof(options))
        {
          /**
           * Color del borde del cuadro de texto.
           * @property color
           * @type {String}
           */
          this.color = options.color;
          /**
           * Relleno del cuadro de texto.
           * @property fill
           * @type {String}
           * @default #FFF (blanco)
           */
          this.fill = options.fill;
          /**
           * Puntos origen y destino de la flecha.
           *
           * @property strokeWidth
           * @type {Number}
           */
          this.strokeWidth = options.strokeWidth;
          /**
           * Opacidad del óvalo.
           *
           * @property opacity
           * @type {Number}
           * @default 1
           */
          this.opacity = options.opacity * scale;
        }
      };

      TextCopy.prototype = _.extend({}, TextSelect.prototype, {
        /**
         * Acción del objeto.
         *
         * @property action
         * @type {String}
         * @default "textCopy"
         */
        action: "textCopy"
      });

      return TextCopy;
    }
);
// noinspection JSHint
define(
    'modules/viewer/controllers/kinetic',[
      "jquery",
      "underscore",
      "backbone",
      "marionette",
      "app",
      "language",
      "modules/viewer/models/draw.collection",
      "modules/viewer/models/shape/copy",
      "modules/viewer/models/shape/underline",
      "modules/viewer/models/shape/highlight",
      "modules/viewer/models/shape/square"
    ],
    function (
        $,
        _,
        Backbone,
        Marionette,
        App,
        language,
        DrawCollectionModel,
        TextCopy,
        UnderLine,
        Highlight,
        Square
        ) {
      "use strict";

      var STATUS = {
        IDLE: 0,
        RUN: 1
      };

      return Marionette.Controller.extend({
        /* constants */
        STATUS: STATUS,

        attributes: null,

        initialize: function (options) {
          _.defaults(options || (options = {}), {
            draw: null,
            view: this.options.view || null
          });

          this.attributes = {
            brush: null,
            center: {x: 0, y: 0},
            color: "#1396DD",
            object: null,
            select: false,
            status: 0
          };

          options.view
              .on("touch", this.touch, this)
              .on("drag", this.drag, this)
              .on("dragend", function() {
                //console.log("history triggered!");
                //this.trigger("history");
              }, this)
              .on("release", this.release, this)
              .on("select", function(options) {
                if (!options.shift) {
                  this.trigger("select");
                }
//                !options.shift && this.trigger("select");
                if (this.attributes.select) {
                  options.object.select();
                }

                this.options.view.canvas.draw();
              }, this)
              .on("text-underline text-highlight text-select", _.bind(function (options) {
                var lastShape = Marionette.getOption(this, "view").getLastShape();
                if (lastShape) {
                  this.drawSelection(this.attributes.brush, {
                      xMin: Math.floor(Math.min(lastShape.x,lastShape.x+lastShape.width))
                    , yMin: Math.floor(Math.min(lastShape.y,lastShape.y+lastShape.height))
                    , xMax: Math.ceil(Math.max(lastShape.x,lastShape.x+lastShape.width))
                    , yMax: Math.ceil(Math.max(lastShape.y,lastShape.y+lastShape.height))
                    , type: this.attributes.brush
                  });
                }
              }, this));
        },

        onClose: function () {
          this.options.view.draw(this.options.view.model.attributes.drawing, true /* Pintar sobre el canvas en el que aparece la página (definitivo) */);
          this.options.view.close();
        },

        deselect: function () {
          var canvas = Marionette.getOption(this, "view").canvas;
          if (canvas) _.each(canvas.children, function (value) {
            //noinspection JSHint
            value.deselect && value.deselect();
          });
        },

        drag: function (current) {
          var center = this.attributes.center,
              object = this.attributes.object;

          if (object) {
            object.resize(current.x - center.x, current.y - center.y);
            Marionette.getOption(this, "view").frontDraw();
          }
        },

        draw: function (drawing) {
          this.options.view.draw(drawing);
        },

        clearInteractive: function () {
          this.options.view.clearInteractive();
        },

        image: function (force) {
          var view = Marionette.getOption(this, "view"),
              scale = Marionette.getOption(this, "scale"),
              canvas = view.canvas;

          _.each(canvas.children, function (value) {
            //noinspection JSHint
            value.image && value.image(force);
          });

          //noinspection JSHint
          (force) ?
            canvas.scale({x: 1, y: 1}) :
            canvas.scale({x: scale, y: scale});
        },

        release: function () {
          //console.log("release!");
          var view = Marionette.getOption(this, "view"),
              canvas = $("article.page > canvas");
              //canvas = view.$el.siblings("canvas")[0];

          // Se comprueba si son el primer o segundo canvas en página doble
          //view.doblePagCanvas = canvasAct.length == 2  && (canvasAct[1] == canvas || canvasAct[0] == canvas);

          //if (canvas.length == 0 || canvas[1] == view.$el.siblings("canvas")[0]) {
            //console.log("RELEASE! "+relcanvasDoble);
            view.trigger(this.attributes.brush);
            // Guardo histórico para undo / redo (solo en el caso de que la vista haya cambiado)
            if (view.changed) {
              if ((typeof view.shape != "undefined" && view.shape != null) && "Group" != view.shape.className && "TextHighlight" != view.shape.className && "TextUnderline" != view.shape.className && "SelectionBox" != view.shape.className)
                view.addSnapshot(view.shape);
              view.changed = false;
              if ((view.shape||view.selected).isInsideCanvas()) {
                this.trigger("history");
              }
              if (view.shape) {
                // Solo hacer openWindow si es simple página o en caso de ser doble página el release se dispara para la primera vista
                if (view.shape.openWindow && (canvas.length == 1 || (canvas.length > 1 && canvas[1] == view.$el.siblings("canvas")[0]))) {
                  view.shape.openWindow(function () {
                    this.trigger("history");
                    this.options.view.frontDraw();
                    this.options.view.mainDraw(true); // Indicar que el objeto ha parado de modificarse
                  }, this); // Ya se encarga de hacer el guardado en el historial
                }
              }
              if (view.shape && view.shape.className != "Group")
                view.shape = null;
              view.frontDraw();
              view.mainDraw(true); // Indicar que el objeto ha parado de modificarse
            }
            // Se hace un pintado principal para el caso de que haya seleccionado un elemento interactivo (para que el div se repinte)
            else if (view.selected && view.selected.element) {
              view.shape = null;
              view.frontDraw();
              view.mainDraw(true);
            }
            else {
              view.shape = null;
              view.frontDraw();
              view.mainDraw(true); // Indicar que el objeto ha parado de modificarse
            }

            this.attributes.center = {x: 0, y: 0};
            this.attributes.object = null;

            view = null;
          //}
        },

        touch: function (center) {
          var attributes = this.attributes,
              brush = attributes.brush;

          this.deselect();

          if (brush) {
            var that = this,
                canvas = that.options.view.canvas;
            this.attributes.center = center;

            //noinspection JSHint
            require(
              ["modules/viewer/models/shape/" + brush],
              function (Shape) {
                var attrs = {
                  x: center.x / that.options.view.options.layer.scale,
                  y: center.y / that.options.view.options.layer.scale,
                  color: attributes.color,
                  id: that.options.view.snapshot.nextId++
                };

                // Se indica la escala del canvas, se manda el elemento de interactividad y se indica si es una página par
                attributes.object = new Shape(attrs, that.options.view.options.layer.scale, canvas, that.options.view.interactiveTexts, that.options.view.model.attributes.id % 2 == 0);

                that.options.view.shape = attributes.object;

                var diff = that.options.view.options.layer.size.width/* * that.options.view.options.layer.scale*/;
                if (_.isArray(that.options.view.snapshot.shapes.left)) {
                  attrs.x += diff;
                  that.options.view.addSnapshot(new Shape(attrs, that.options.view.options.layer.scale, canvas, that.options.view.interactiveTexts), that.options.view.snapshot.shapes.left);
                }
                if (_.isArray(that.options.view.snapshot.shapes.right)) {
                  attrs.x -= diff;
                  that.options.view.addSnapshot(new Shape(attrs, that.options.view.options.layer.scale, canvas, that.options.view.interactiveTexts), that.options.view.snapshot.shapes.right);
                }

                that.options.view.changed = (brush != "text-highlight" && brush != "text-underline" && brush != "text-select");
              }
            );
          }
        },

        save: (function () {
          var last;

          return function (name, options) {
            var draw = new DrawCollectionModel(),
                view = Marionette.getOption(this, "view");

            _.defaults(options || (options = {}), {
              name: name.name || last || "default",
              nameNormalized: name.nameNormalized || last || "default"
            });
            last = options.name;

            this.deselect();

            var json = view.layerToJSON();
            console.debug("[DEBUG] Current page "+view.options.model.attributes.id+" stored with "+json.children.length+" elements");
            view.model.set("drawing", JSON.stringify(json));

            draw.fetch().done(function () {
              draw.create({
                id: options.nameNormalized,
                name: options.name
              });
            });

            if(configuration.sgoReadOnlyObjectsURL){
              // Si estoy en Santillana Go, guardo figuras en servidor
              // No hago nada aqui
            } else if (window.location.href.indexOf("mynotebook") > -1) {
              // Si estoy en Mi Cuaderno, guardo figuras en AV
              App.notebook.layout.get("panel").currentView.save();
            } else if (window.location.href.indexOf("edit") > -1) {
              // Solo si estoy en modo previsualizacion o creacion de Mi Libro, guardo figuras en AV
              App.mybook.layout.get("panel").currentView.save();
            } else {
              // Esta en modo normal. Guardar objetos editables
                view.model.collection.url = "book/" + options.nameNormalized;
                view.model.save();
            }
          };
        })(),

        saveOtherPage: (function () {
          var last;

          return function (name, options) {
            var view = Marionette.getOption(this, "view");
            if (_.isArray(view.snapshot.shapes.left) || _.isArray(view.snapshot.shapes.right)) {
              var draw = new DrawCollectionModel();
              var that = this;
              var model;

              App.request("pages").done(function (pages) {
                var diff = _.isArray(view.snapshot.shapes.left) ? -1: 1;
                if ((view.model.attributes.id-1) + diff >= 0 && (view.model.attributes.id-1) + diff <= pages.models.length) {
                  model = pages.models[view.model.attributes.id - 1 + diff];
                  if (model) {
                    _.defaults(options || (options = {}), {
                      name: name || last || "default",
                      nameNormalized: name || last || "default"
                    });
                    last = options.name;
                    that.deselect();
                    var json = view.layerToJSON(false);
                    console.debug("[DEBUG] "+(diff>0?"Right":"Left")+" page "+model.attributes.id+" stored with "+json.children.length+" elements");
                    model.set("drawing", JSON.stringify(json));
                    draw.fetch().done(function () {
                      draw.create({
                        id: options.nameNormalized,
                        name: options.name
                      });
                    });
                    if(configuration.sgoReadOnlyObjectsURL){
                      // Si estoy en Santillana Go, guardo figuras en servidor
                      // No hago nada aqui
                    } else if (window.location.href.indexOf("mynotebook") > -1) {
                      // Si estoy en Mi Cuaderno, guardo figuras en AV
                      App.notebook.layout.get("panel").currentView.save();
                    } else if (window.location.href.indexOf("edit") > -1) {
                      // Solo si estoy en modo previsualizacion o creacion de Mi Libro, guardo figuras en AV
                      App.mybook.layout.get("panel").currentView.save();
                    } else {
                      model.collection.url = "book/" + options.nameNormalized;
                      // Esta en modo normal. Guardar objetos editables
                      model.save();
                    }
                  }
                }
              });
            }
          };
        })(),

        trash: function (group) {
          var view = Marionette.getOption(this, "view");
          group = group || view.selected;
          if (group) {
            var clearCanvas = view.clearInteractive(view.selected) || (view.selected && !view.selected.element);
            // Deseleccionar el elemento para quitar los tiradores
            view.selectElement(false);
            // El elemento a eliminar deja de estar seleccionado por abandonar su condición de existencia
            view.selected = null;
            // Se elimina el elemento de la lista
            view.snapshot.shapes.current = _.filter(view.snapshot.shapes.current,function(elem){return (typeof elem != "undefined" && elem != null)&&elem!==group;});
            if (_.isArray(view.snapshot.shapes.left)) {
              view.snapshot.shapes.left = _.filter(view.snapshot.shapes.left,function(elem){return (typeof elem != "undefined" && elem != null)&&elem.id!==group.id;});
            }
            if (_.isArray(view.snapshot.shapes.right)) {
              view.snapshot.shapes.right = _.filter(view.snapshot.shapes.right,function(elem){return (typeof elem != "undefined" && elem != null)&&elem.id!==group.id;});
            }
            // Se invalida la vista para que se visualice el borrado del elemento
            if (clearCanvas) {
              view.frontDraw();
            }
            return true;
          }
          return false;
        },

          selectText: function (options, view) {
              var deferred = $.Deferred();
              var that = this;

              // Obtengo la pagina actual para conseguir code e index
              App.request("page", {model: true}).done(function (pageModel) {

                  // Obtengo el documento de la pagina para conseguir texto y posiciones
                  App.request("document", {code: that.options.view.model.get("code"), delay: 0}).done(function (document) {

                      var page = that.options.view.model.attributes;
                      var selectedPositions = [];
                      var selectedText = "";

                      var initialTop = $("#p" + page.number).offset().top;
                      var initialLeft = $("#p" + page.number).offset().left;
                      var pageScale = parseFloat($("#p" + page.number).css('transform').split('(')[1].split(')')[0].split(',')[0]);
                      var viewerScale = parseFloat($("#viewer > li").css('transform').split('(')[1].split(')')[0].split(',')[0]);
                      var scale = pageScale;

                      var selection = {
                          'xMin': options.xMin / scale,
                          'yMin': options.yMin / scale,
                          'xMax': options.xMax / scale,
                          'yMax': options.yMax / scale
                      };

                      $("#p" + page.number).find("div").each(function(index) {

                          if ($(this)[0].innerText != "") {

                              var transform = $(this).css('transform').split('(')[1].split(')')[0].split(',');

                              // We discard rotated texts.
                              if (transform[1].trim() == "0") {

                                  var divPosition = {
                                      'text': $(this)[0].innerText,
                                      'xMin': ($(this).offset().left - initialLeft) / scale / viewerScale,
                                      'yMin': ($(this).offset().top - initialTop) / scale / viewerScale,
                                      'xMax': ((($(this).offset().left - initialLeft) ) / scale / viewerScale) + ($(this).width() * 0.25) ,
                                      'yMax': ((($(this).offset().top - initialTop) ) / scale / viewerScale) + ($(this).height() * 0.25)
                                  };

                                  if (divPosition.xMin < selection.xMax && divPosition.xMax > selection.xMin && divPosition.yMin < selection.yMax && divPosition.yMax > selection.yMin) {

                                      selectedPositions.push({
                                          'x1': (divPosition.xMin < selection.xMin) ? selection.xMin : divPosition.xMin,
                                          'y1': (divPosition.yMin < selection.yMin) ? selection.yMin : divPosition.yMin,

                                          'x2': (divPosition.xMax > selection.xMax) ? selection.xMax : divPosition.xMax,
                                          'y2': (divPosition.yMax > selection.yMax) ? selection.yMax : divPosition.yMax,

                                          'x3': (divPosition.xMax > selection.xMax) ? selection.xMax : divPosition.xMax,
                                          'y3': (divPosition.yMax > selection.yMax) ? selection.yMax : divPosition.yMax,

                                          'x4': (divPosition.xMax > selection.xMax) ? selection.xMax : divPosition.xMax,
                                          'y4': (divPosition.yMin < selection.yMin) ? selection.yMin : divPosition.yMin
                                      });

                                      selectedText += $(this)[0].innerText;

                                  }

                              }

                          }

                      });

                      deferred.resolve({'selectedText': selectedText, 'selectedPositions': selectedPositions});

                  });

              });

              return deferred.promise();
          },

        drawSelection: function (type, options) {
          var that = this;
          var view = Marionette.getOption(this, "view");
          var canvas = view.canvas;

          var group = {attrs: {selectedText: null}, elems: []};

          //var group = new Kinetic.Group();

          // Obtengo el texto seleccionado para calcular ubicaciones
          $.when(this.selectText(options,view)).done(function (selected) {

              if (selected.selectedText) {

                  // Almaceno el texto seleccionado en el grupo para recuperarlo luego
                  group.attrs.selectedText = selected.selectedText;

                  if (type == "text-select") {
                      // console.log(selected.selectedText);
                      // Almaceno el texto seleccionado en las propiedades de Tools
                      App.Tools.selectedText = selected.selectedText;
                  }

                  // Divido los resultados por posición Y

                  var multiline = _.groupBy(selected.selectedPositions, function (pos) {
                      return pos.y3;
                  });

                  var addAsShape = false; // Indica si se añade en el array de figuras

                  // Recorro todos los resultados. Cada elemento de este
                  // objeto es una línea nueva.
                  _.each(multiline, function (pos, key, list) {
                      var result = {x: 0, y: 0, height: 0, width: 0};

                      // Recorro todos los resultados de una misma línea
                      // para calcular posición y tamaño del DIV de resaltado
                      _.each(pos, function (letra, key, list) {
                          if (result.x == 0) {
                              result.x = Math.round(letra.x1*2);
                          }

                          if (letra.x2*2 > result.width) {
                              result.width = letra.x2*2;
                          }

                          if (result.y == 0) {
                              result.y = Math.round(letra.y1*2);
                          }

                          if (letra.y3*2 > result.height) {
                              result.height = letra.y3*2;
                          }
                      });

                      switch (type) {

                          case "text-underline":
                              // Se añade el subrayado
                              group.elems.push(new UnderLine({
                                  points: [
                                      result.x,
                                      result.height + 2,
                                      result.width,
                                      result.height + 2
                                  ],
                                  color: that.attributes.color,
                                  strokeWidth: 2
                              }, that.options.view.options.layer.scale));
                              // Se añaden resaltados transparentes para facilitar la selección del texto subrayado
                              group.elems.push(new Highlight({
                                  x: result.x - 1,
                                  y: result.y,
                                  height: result.height - result.y,
                                  width: result.width - result.x + 3,
                                  opacity: 0,
                                  color: "#FFFFFF",
                                  strokeWidth: 0
                              }, that.options.view.options.layer.scale, that.options.view.canvas));
                              break;

                          case "text-highlight":
                              // Dibujo cajas semi-transparentes para el resaltado
                              group.elems.push(new Highlight({
                                  x: result.x - 1,
                                  y: result.y,
                                  height: result.height - result.y,
                                  width: result.width - result.x + 3,
                                  color: that.attributes.color
                              }, that.options.view.options.layer.scale, that.options.view.canvas));
                              break;

                          case "text-select":
                              addAsShape = true;
                              // Dibujo cajas semi-transparentes para la seleccion
                              group.elems.push(new TextCopy({
                                  x: result.x - 1,
                                  y: result.y,
                                  height: result.height - result.y,
                                  width: result.width - result.x + 3,
                                  color: that.attributes.color
                              }, that.options.view.options.layer.scale, that.options.view.canvas));
                              break;
                      }

                  });

                  // Manejo de colisiones entre resaltados
                  _.each(view.snapshot.shapes.current, function (grupo) {
                      if (grupo.className == "Group") {
                          // Recorrer los elementos del antiguo grupo
                          if (_.find(grupo.children, function (oldItem) {
                                  // Recorrer los elementos del nuevo grupo
                                  return _.find(group.elems, function(newItem) {
                                      return (
                                              newItem.x >= oldItem.x
                                              && newItem.x <= oldItem.x + oldItem.width
                                              && newItem.y >= oldItem.y
                                              && newItem.y <= oldItem.y - oldItem.height
                                          ) || (
                                              oldItem.x >= newItem.x
                                              && oldItem.x <= newItem.x + newItem.width
                                              && oldItem.y >= newItem.y
                                              && oldItem.y <= newItem.y - newItem.height
                                          );
                                  });
                              })) {
                              // Elimino el grupo anterior
                              that.trash(grupo);
                          }
                      }
                  });

                  var grupo = {
                      attrs: { selectedText: selected.selectedText },
                      className: "Group",
                      children: group.elems,
                      isInsideCanvas: function() { return false; }
                  };
                  if (addAsShape) {
                      view.shape = grupo;
                  }
                  else {
                      // Añadir el grupo de resaltados y seleciones al conjunto de figuras del canvas
                      if (typeof grupo != "undefined" && grupo != null)
                          view.addSnapshot(grupo);
                  }

                  view.changed = true;
              }




          });

        }
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/controllers/draw',["underscore", "marionette", "modules/viewer/controllers/kinetic", "configuration"],
    function (_, Marionette, KineticController, configuration) {
      "use strict";

      return Marionette.Controller.extend({
        copiedObjects: null,
        history: null,
        historyStep: -1,
        modified: false,
        timeBetween: 100,

        initialize: function (options) {
          _.defaults(options || (options = {}), {
            canvas: [],
            drawing: []
          });

          // init vars
          this.history = [];
          this.copiedObjects = [];

          options.canvas = _.map(options.canvas, function(value) {
            var controller = new KineticController({
              view: value
            });

            App
                .on("history", this.makeHistory, this);

            controller
                .on("history", this.makeHistory, this)
                .on("select", this.deselect, this);

            return controller;
          }, this);
          if (options.drawing && !options.drawing.length)
            this.draw(options.drawing);

          // Inserto el layer vacío en el history
          this.makeHistory();
          this.modified = false;
        },

        onClose: function () {
          _.each(this.options.canvas, function(value) {
            value.close();
          });
        },

        brush: function (type) {
          _.each(this.options.canvas, function(value) {
            value.attributes.brush = type;
            value.attributes.select = false;

            value.options.view.draggable = !type;
          });
        },

        color: function (color) {
          _.each(this.options.canvas, function(value) {
            value.attributes.color = color;
          });
        },

        draw: function(drawing) {
          _.each(this.options.canvas, function(value, index) {
            if (!drawing[index]) {
              return true;
            }

            value.draw(drawing[index]);
            return true;
          }, this);
          this.makeHistory();
        },

        makeHistory: function () {
          var self = this,
              historyStep = this.historyStep+1;
          this.modified = true;

          // Si cambio la historia por la mitad, se reescribe. Back to the future!
          if (historyStep < this.history.length) {
            this.history.length = historyStep;
          }

          // moment es un array que contiene un snapshot por página
          var moment = [];

          _.each(this.options.canvas, function(value) {
            var layerAsString = JSON.stringify(value.options.view.layerToJSON());
            // Guardar una copia del estado actual de las figuras en el layer
            moment.push(layerAsString);
          }, this);

          if (moment.length > 0) {
            this.historyStep++;
            console.debug(this.historyStep,"history:make"/*+Date.now()*/);
            // Guardo en la historia el estado de todas las páginas visibles
            this.history.push({timestamp: Date.now(), moment: moment});
          }
        },

        redo: function () {
          if (this.historyStep < this.history.length - 1) {
            this.historyStep++;
            if (this.history[this.historyStep] && this.history[this.historyStep-1])
              console.debug(this.historyStep,"history:redo"/*+(this.history[this.historyStep].timestamp - this.history[this.historyStep-1].timestamp)*/);

            if (this.history.length > this.historyStep+1 && this.history[this.historyStep+1].timestamp - this.history[this.historyStep].timestamp < this.timeBetween) {
              this.historyStep++;
            }

            _.each(this.options.canvas, function(value, index) {
              value.clearInteractive();
              value.draw(this.history[this.historyStep].moment[index]);
            }, this);
          }
        },

        save: function (name) {
          this.modified = false;

          // Necesito saber si el nombre es un numero. En caso de serlo, le agrego un _ para 
          // transformarlo a string. Esto evita errores al guardar numeros muy largos (max int)
          // if(!isNaN(name))
          //   name = "_" + name;

          /**
           * Fix para algunos elementos cuando se edita a doble página (cuadros de texto, cuadros de enlace...)
           * El problema que resulta es que el texto se guarda en la instancia de la segunda página y no de la primera por lo que se puede añadir los datos de la figura en la instancia de la izquierda
           * @author Mario Poyato <mario.poyato@inmediastudio.es>
           */
          if (this.options.canvas.length > 1) {
            var leftLayer = this.options.canvas[0].options.view.snapshot.shapes.current,
              rightLayer = this.options.canvas[1].options.view.snapshot.shapes.current;
            _.each(rightLayer, function(shape) {
              // Si es un cuadro de texto
              switch (shape.className) {
                case "Character":
                case "TextArrow":
                case "TextDirection":
                case "LinkSquare":
                case "Clip":
                case "Note": {
                  var shape2 = _.find(leftLayer, function(shape2) {
                    return shape2.id == shape.id;
                  });
                  // En caso de encontrar la figura en la otra página
                  if (shape2) {
                    shape.copyData(shape2, false); // Copy Data
                  }
                }
              };
            }, this);
          }

          /**
           * FIX: Arreglo del identificador del guardado.
           * @author Mario Poyato <mario.poyato@inmediastudio.es>
           */
          var normalize = (function() {
            var from = "ÃÀÁÄÂÈÉËÊÌÍÏÎÒÓÖÔÙÚÜÛãàáäâèéëêìíïîòóöôùúüûÑñÇç",
              to   = "AAAAAEEEEIIIIOOOOUUUUaaaaaeeeeiiiioooouuuunncc",
              mapping = {};
            for(var i = 0, j = from.length; i < j; i++ )
              mapping[ from.charAt( i ) ] = to.charAt( i );
            return function( str ) {
              if (!str)
                return str;
              var ret = [];
              for( var i = 0, j = str.length; i < j; i++ ) {
                var c = str.charAt( i );
                if( mapping.hasOwnProperty( str.charAt( i ) ) )
                  ret.push( mapping[ c ] );
                else
                  ret.push( c );
              }
              return ret.join('').replace(/[^-A-Za-z0-9]+/g, '-');
            }
          })();
          var nameNormalized = normalize(name);
          normalize = null;

          // Guardo elementos
          _.each(this.options.canvas, function(value) {
            value.save({name: name, nameNormalized: nameNormalized});
            value.saveOtherPage({name: name, nameNormalized: nameNormalized});
          });

          // Si estoy como admin de Santillana GO y dibujo sobre el libro,
          // las figuras quedarán como readOnlyObjects de nivel 1, es decir,
          // no podrán ser modificadas po profesores ni por alumnos.
          // En este caso, las personalizaciones no se guardan en local, 
          // sino que se envían a Santillana GO para que las guarde.
          if(configuration.sgoReadOnlyObjectsURL){
            console.log("ATENCION: Guardando en Santillana Go");
            App.request("pages").done(function (pages) {
              var customizedPages = {};

              // Mapeo las paginas
              customizedPages.pages = pages.models.map(function(page){
                var custom = page.get("custom") ? page.get("custom").slice(0) : [];
                custom[0] = page.get("drawing");

                return { 
                  "idUnidad": page.get("code"),
                  "number": page.get("number"),
                  "index": page.get("index"),
                  "title": page.get("title"),
                  "custom": custom
                }
              });

              // Mantengo sólo las que tienen figuras
              customizedPages.pages = _.filter(customizedPages.pages, function(page){
                return page.custom[0];
              });

              // Envio las paginas con figuras para guardarlas en SGO
              $.ajax({
                type: 'POST',
                // contentType: "application/json; charset=utf-8",
                url: configuration.sgoReadOnlyObjectsURL,
                data: {xml: JSON.stringify(customizedPages)},
                success: function(msg) {
                  console.log("Guardado ok!", msg);
                }
              });
            });
          }
        },

        select: function (force) {
          _.each(this.options.canvas, function(value) {
            value.attributes.select = force;
          });
        },

        deselect: function () {
          _.each(this.options.canvas, function(value) {
            value.deselect();
            value.options.view.draggable = false;
          });
        },

        textDeselect: function () {
          var layer;

          _.each(this.options.canvas, function(value) {
            layer = value.options.view.layer;

            //console.log("TODO draw:textDeselect");
            /*
              layer.getChildren().each(function(value){
                if(value.nodeType == "Group"){
                  _.map(
                    value.getChildren(function(item){
                      return item.getClassName() == "TextCopy";
                    }).toArray(),
                    function(value) {
                      value.remove();
                    }
                  );
                }
              });

              layer.draw();
            */
          });
        },

        undo: function () {
          if (this.historyStep > 1 && this.history[this.historyStep]) {
            this.historyStep--;
            var difftime = this.history[this.historyStep+1].timestamp - this.history[this.historyStep].timestamp;
            console.debug(this.historyStep, "history:undo"/*+difftime*/);

            if (difftime < this.timeBetween) {
              this.historyStep--;
            }

            if (this.history[this.historyStep]) {
              _.each(this.options.canvas, function(value, index) {
                value.clearInteractive();
                value.draw(this.history[this.historyStep].moment[index]);
              }, this);
            }
          }
        },

        copy: function () {
          var isText = true;

          // Si no hay espacio reservar espacio para el objeto copiado
          if (!this.copiedObjects) {
            this.copiedObjects = new Array(2);
          }
          // En caso de doble página obtener la figura seleccionada
          _.each(this.options.canvas, function(page, index) {
            var view = page.options.view;
            if (view.selected) {
              // Indicar que hay una figura copiada (No es texto)
              isText = false;
              // Almacenar la figura copiada
              this.copiedObjects[index] = view.selected;
              // Deseleccionar el objeto
              view.selectElement(false);
              view.frontDraw();
              view.mainDraw();
            }
          }, this);

          // Si no habia nada seleccionado y tengo
          // texto en el buffer, copio el texto
          if(isText && App.Tools.selectedText){
            // Envío el texto copiado a AV
            App.request("av", {type: 'setClipboard', clipboard: App.Tools.selectedText});
            console.log("COPIADO:", App.Tools.selectedText);
            App.Tools.selectedText = null;
            _.each(this.options.canvas, function(page, index) {
              var view = page.options.view;
              view.shape = null;
              view.frontDraw();
            });
          }
        },

        paste: function () {
          if (this.copiedObjects && this.copiedObjects.length) {
            _.each(this.copiedObjects, function (obj, index) {
              //_.each(page, function (obj) {
              /*
               obj = JSON.parse(obj.compress());
               obj.attrs.x = obj.attrs.x + 20;
               obj.attrs.y = obj.attrs.y + 20;
               var shape = view.generaFigura(JSON.stringify(obj));
               view.addSnapshot(shape);
               */
              var view = this.options.canvas[index].options.view;

              obj = obj.compress();
              obj.attrs.x = obj.attrs.x + 20;
              obj.attrs.y = obj.attrs.y + 20;
              obj.attrs.id = view.snapshot.nextId++;
              obj = view.generaFigura(obj);

              /*obj = _.clone(obj);
              obj.x += 20;
              obj.y += 20;
              obj.id = view.snapshot.nextId++;*/
              if (typeof obj != "undefined" && obj != null) {
                view.addSnapshot(obj);

                var diff = view.options.layer.size.width * view.options.layer.scale;
                if (_.isArray(view.snapshot.shapes.left)) {
                  var otherObj = _.clone(obj);
                  otherObj.x += diff;
                  view.addSnapshot(otherObj, view.snapshot.shapes.left);
                }
                if (_.isArray(view.snapshot.shapes.right)) {
                  var otherObj = _.clone(obj);
                  otherObj.x -= diff;
                  view.addSnapshot(otherObj, view.snapshot.shapes.right);
                }

              }
              view.selected  = null;
              view.frontDraw();
              view.mainDraw();
              //});
            }, this);
            this.makeHistory();
          }
        },

        trash: function () {
          var changed = false;
          _.each(this.options.canvas, function(value) {
            changed = value.trash();
          });

          var _deletedObjects = [];

          _.each(this.options.canvas, function(page) {
            _.each(page.options.view.canvas.children, function (child) {
              if (typeof child.anchors !== 'undefined') {
                if (child.anchors.length > 0) {
                  _deletedObjects.push(child);
                } else {
                  if (typeof child.object !== 'undefined') {
                    if (typeof child.object.anchors !== 'undefined') {
                      if (child.object.anchors.length > 0) {
                        _deletedObjects.push(child);
                      }
                    }
                  }
                }
              }
            });
            while (_deletedObjects.length > 0) {
              _deletedObjects.pop().destroy();
            }
            //page.options.view.stage.draw();
          });

          if (changed)
            this.makeHistory();
        }
      });
    }
);

// noinspection JSHint
define(
    'modules/viewer/viewer.document.draw',[
      "app",
      "modules/viewer/viewer.document",
      "language",
      // controllers
      "modules/viewer/controllers/kinetic.builder",
      "modules/viewer/controllers/draw",
      // models
      "models/page",
      // regions
      "regions/append"
    ],
    function (
        App, parent, language, builder, IndexController, PageModel, AppendRegion
        ) {
      "use strict";

      return App.module(
          "viewer.document.draw",
          function (module, App, Backbone, Marionette, $, _) {
            /* Presets module */
            this.startWithParent = false;

            /* Private */

            /* Public */

            // Constructor
            this.addInitializer(function (options) {
              this.options = _.defaults(options || (options = {}), {
                drawing: [],
                pages: []
              });

              App.reqres.removeHandler("draw:exit");
              App.reqres.setHandler("draw:exit", function (options) {
                var deferred = $.Deferred();

                _.defaults(options || (options = {}), {
                  context: deferred
                });

                if (!module.controller || module.controller.modified) {
                  deferred.rejectWith(options.context);
                } else {
                  deferred.resolveWith(options.context);
                }

                return deferred.promise();
              });

              App.request("is:device", {boolean: true, context: this})
                  .done(function(isDevice) {
                    this.controller = new IndexController({
                      canvas: _.map(options.pages, function (value) {
                        var DRAWING = PageModel.prototype.STATUS.DRAWING;
                        var canvas = builder.construct(value, isDevice),
                            region = new AppendRegion({el: value.el});

                        value.model.status(-DRAWING);
                        value.model.set("interactive", value.el.getElementsByClassName("interactive")[0]/*value.el.getElementsByClassName("interactive")[0].cloneNode(false)*/);
                        region.show(canvas);

                        return canvas;
                      }),
                      drawing: options.drawing
                    });

                    App
                        .on("draw:brush", this.controller.brush, this.controller)
                        .on("draw:color", this.controller.color, this.controller)
                        .on("draw:copy", this.controller.copy, this.controller)
                        .on("draw:deselect", this.controller.deselect, this.controller)
                        .on("draw:textDeselect", this.controller.textDeselect, this.controller)
                        .on("draw:paste", this.controller.paste, this.controller)
                        .on("draw:redo", this.controller.redo, this.controller)
                        .on("draw:save", this.save, this)
                        .on("draw:select", this.controller.select, this.controller)
                        .on("draw:trash", this.controller.trash, this.controller)
                        .on("draw:undo", this.controller.undo, this.controller)
                        .on("fit", this.delay, this)
                        .on("goto", this.delay, this)
                        .on("resize", this.preResize, this);
                    $(document.body).addClass("kinetic");

                    $("#tools .active").trigger("touch");
                  });
            });

            this.addFinalizer(function () {
              var DRAWING = PageModel.prototype.STATUS.DRAWING;
              _.each(this.options.pages, function (value) {
                value.model.status(DRAWING * (App.menu.mode !== "hidden"));
              });

              this.controller.close();

              App
                  .off("draw:brush", this.controller.brush)
                  .off("draw:color", this.controller.color)
                  .off("draw:copy", this.controller.copy)
                  .off("draw:deselect", this.controller.deselect)
                  .off("draw:textDeselect", this.controller.textDeselect)
                  .off("draw:paste", this.controller.paste)
                  .off("draw:redo", this.controller.redo)
                  .off("draw:save", this.save)
                  .off("draw:select", this.controller.select)
                  .off("draw:trash", this.controller.trash)
                  .off("draw:undo", this.controller.undo)
                  .off("fit", this.delay)
                  .off("goto", this.delay)
                  .off("resize", this.preResize);

              $(document.body).removeClass("kinetic");
            });

            this.defer = _.chain(function() {
              this.stop();
              this.start();
            })
                .bind(this)
                .debounce(0)
                .value();

            // Fix para dispositivos Android, que al mostrar
            // el teclado, hace resize y pierde el contenido
            // de las cajas de texto.
            this.preResize = function(){
              var that = this;

              App.request("is:device")
                .done(function (type) {
                  console.log(type);
                  if(type[0] == "Android"){
                    return false;
                  } else {
                    that.delay();
                  }
                })
                .fail(function(type) {
                  that.delay();
                });
            }

            this.delay = _.chain(function() {
              var pages = this.options.pages,
                  status = this.controller.history.pop();

              this.stop();
              this.start({
                drawing: status,
                pages: pages
              });
            })
                .bind(this)
                .debounce(750)
                .value();

            this.save = function (name) {
              // Fix para guardar en el ultimo slot abierto o creado
              if(!name){
                name = window.configuration.lastSavedName;
              }
              
              this.controller.save(name);

              App.execute("notification", language("savedChanges") + " [" + (name || "default") + "]", {color: 6});
              App.execute("module:stop", "Tools");
              App.execute("module:stop", "viewer.document.draw");
            };
          }
      );
    }
);
// noinspection JSHint
/**
 * Figura del texto a resaltar
 * @class TextHighlight
 * @extends TextSelect
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/text-highlight',[
    	"app",
    	"underscore",
    	"modules/viewer/models/shape/text-select"
    ],
    function (App, _, TextSelect) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var TextHighlight = function(options, scale, canvas) {
        // Default properties of KineticJS
        _.defaults(options || (options = {}), {
          className: "TextHighlight",
          fill: "#FFFF00",
          opacity: 0.3,
          strokeWidth: 1
        });
        TextSelect.call(this,options,scale,canvas);
        if ("undefined" !== typeof(options))
        {
          scale = scale || 1;
          this.x = options.x * scale;
          this.y = options.y * scale;
          this.fill = options.fill;
          this.strokeWidth = options.strokeWidth;
          this.opacity = options.opacity;
        }
      };

      TextHighlight.prototype = _.extend({}, TextSelect.prototype, {
        /**
         * Acción del objeto.
         *
         * @property action
         * @type {String}
         * @default "textHighlight"
         */
        action: "textHighlight"
      });
      
      return TextHighlight;

    }
);
/**
 * Figura del texto a subrayar
 * @class TextUnderline
 * @extends Shape
 * @author Mario Poyato <mario.poyato@inmediastudio.es>
 */
define(
    'modules/viewer/models/shape/text-underline',[
    	"app",
    	"underscore",
    	"modules/viewer/models/shape/text-select"
    ],
    function (App, _, TextSelect) {
      "use strict";

      /**
       * Función constructura de que toma las opciones (o atributos), escala del canvas y el propio canvas e inicializa el objeto
       * @param options
       * @param scale
       * @param canvas
       * @constructor
       */
      var TextUnderline = function (options, scale, canvas) {
        // Default properties of KineticJS
        _.defaults(options || (options = {}), {
          className: "TextUnderline",
          fill: "#F2942E",
          opacity: 0.5,
          strokeWidth: 1
        });
        TextSelect.call(this,options,scale,canvas);
        if ("undefined" !== typeof(options))
        {
          scale = scale || 1;
          this.x = options.x * scale;
          this.y = options.y * scale;
          this.fill = options.fill;
          this.strokeWidth = options.strokeWidth;
          this.opacity = options.opacity;
        }
      };

      TextUnderline.prototype = _.extend({}, TextSelect.prototype, {
        /**
         * Acción del objeto.
         *
         * @property action
         * @type {String}
         * @default "textUnderline"
         */
        action: "textUnderline"
      });
      
      return TextUnderline;

    }
);
// noinspection JSHint
define(
    'modules/viewer/viewer.bookmark',["app", "modules/viewer/viewer", "views/chapters.dropdown"],
    function (App, parent, ChapterDropdownDeferred) {
      "use strict";

      return App.module(
          "viewer.bookmark",
          function (module, App, Backbone, Marionette, $, _) {
            /* Presets module */
            this.startWithParent = false;

            /* Private */

            /* Public */

            // Layout
            this.layout = parent.layout.get("bookmark");

            // Constructor
            this.addInitializer(function (options) {
              ChapterDropdownDeferred().done(function (View) {
                View = View.extend({
                  id: "index",
                  tagName: "h1",

                  title: function (title) {
                    this.model.set("title", title);
                  }
                });

                var view = new View();

                module.layout.show(view);

                view.on("select", function ($el) {
                  var page = parseInt($el.attr("data-page"));

                  // Elimino resultados de busqueda
                  App.trigger("search:removeResults");

                  parent.trigger("goto", page);
                });

                App.on("goto", module.refresh, module);

                module.refresh(options.page);
              });
            });

            // Destructor
            this.addFinalizer(function () {
              App.off("goto", this.refresh);
            });

            // Events

            this.refresh = function (page) {
              App.request(
                  "chapter",
                  {context: this.layout.currentView, page: page}
              )
                  .done(function (chapter) {
                    var titulo = (chapter && chapter.get("title")) || "";
                    this.title(titulo);
                  });
            };
          }
      );
    }
);
// noinspection JSHint
define(
    'modules/viewer/views/paginator',["underscore", "marionette", "modules/viewer/template"],
    function (_, Marionette, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        events: {
          "change @ui.page": function () {
            this.trigger("goto", this.ui.page.val(), false, true);
          },

          "keydown @ui.page": function (event) {
            event = event.originalEvent || event;

            event.stopPropagation();
            if (event.keyCode === 13) {
              this.ui.page.blur();
              // this.trigger("goto", parseInt(this.ui.page.val(), 10) || 1);
            }
          }
        },
        tagName: "article",
        template: JST.paginator,

        modelEvents: {
          "change:page": "render"
        },
        ui: {
          "previous": "a:first",
          "page": "input",
          "next": "a:last"
        },
        triggers: {
          "tap @ui.next": "next",
          "tap @ui.previous": "previous"
        },

        disable: function (first, last) {
          this.delay(function() {
            this.ui.previous.toggleClass("inactive", first);
            this.ui.next.toggleClass("inactive", last);
          });
        }
      });
    }
);

// noinspection JSHint
define(
    'modules/viewer/views/zoom',["underscore", "marionette", "modules/viewer/template"],
    function (_, Marionette, JST) {
      "use strict";

      return Marionette.ItemView.extend({
        behaviors: {
          Scrollbar: {
            color: 9,
            container: "aside",
            position: "bottom"
          }
        },
        className: "slider",
        events: {
          "touch i.icon-zoom": function () {
            this.trigger("step");
          },
		      "touch i.icon-zoom-center": function () {
            this.trigger("zoom", 1);
          }
        },
        tagName: "section",
        template: JST.zoom,

        initialize: function () {
          var callback = _.chain(function(progress) {
            this.trigger("zoom", progress * 2 / 100 + 1);
          })
              .bind(this)
              .debounce(300)
              .value();

          this.on("scroll:after", callback);
        }
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/regions/controls',["marionette", "regions/replace", "modules/viewer/template"],
    function (Marionette, ReplaceRegion, JST) {
      "use strict";

      return Marionette.Layout.extend({
        className: "norender",
        tagName: "footer",
        template: JST["controls.layout"],

        regions: {
          "navigation": "nav",
          "paginator": "article",
          "zoom": "section"
        },
        regionType: ReplaceRegion
      });
    }
);
// noinspection JSHint
define(
    'modules/viewer/viewer.controls',[
      "app",
      "modules/viewer/viewer",
      "views/icon.collection",
      "modules/viewer/views/paginator",
      "modules/viewer/views/zoom",
      "modules/viewer/regions/controls"
    ],
    function (App, parent, IconCollectionView, PaginatorView, ZoomView, IndexLayout) {
      "use strict";

      return App.module(
          "viewer.controls",
          function (module, App, Backbone, Marionette, $, _) {
            /* Presets module */
            this.startWithParent = false;

            /* Private */
            var buttons = {
                  mode: {type: "document", command: "mode"},
                  fit: {type: "fit", command: "fit"}
                },
                step = 1;

            /* Public */

            // Layout
            this.layout = (function () {
              var layout = new IndexLayout(),
                  region = parent.layout.get("controls");
              region.show(layout);

              return layout;
            })();

            // Constructor
            this.addInitializer(function (options) {
              _.defaults(options || (options = {}), {
                fit: true
              });

              App.request("is:document:landscape", {boolean: true, context: this})
                  .done(function (isDocumentLandscape) {
                    this.options.mode = options.mode =
                        _.has(options, "mode") ? options.mode : !isDocumentLandscape;

                    var navigation = new IconCollectionView({
                      collection: new Backbone.Collection(
                          _.compact(_.map(buttons, function (value, key) {
                            return (options[key]) ? value : null;
                          }))
                      )
                    });
                    this.layout.navigation.show(navigation);

                    var paginator = new PaginatorView({
                      model: new Backbone.Model({page: 0})
                    });
                    this.layout.paginator.show(paginator);

                    var zoom = new ZoomView();
                    this.layout.zoom.show(zoom);

                    /* View events */

                    navigation.on("itemview:touch", function (view) {
                      if ($(".kinetic").length) {
                        return false;
                      }
                      parent.trigger(view.model.get("command"));
                    });

                    paginator
						 .on("goto", function (page, force, searching) {
                          if ($(".kinetic").length) {
                            return false;
                          }

                          parent.trigger("goto", page, force, searching);
                        })
                        .on("next", function () {
                          if ($(".kinetic").length) {
                            return false;
                          }

                          parent.trigger("next");
                        })
                        .on("previous", function () {
                          if ($(".kinetic").length) {
                            return false;
                          }

                          parent.trigger("previous");
                        })
                      .on("fit", function () {
                        if ($(".kinetic").length) {
                          return false;
                        }

                        App.trigger("fit");
                      });

                    zoom
                        .on("step", function () {
                          step = (step) % 3 + 1;
                          parent.trigger("zoom", step);
                        })
                        .on("zoom", function (scale) {
                          parent.trigger("zoom", scale);
                        });

                    /* Events */

                    App
                        .on("fit", this.fit, this)
                        .on("goto", this.goto, this)
                        .on("mode", this.mode, this)
                        .on("zoom", this.zoom, this)
                        .on("zoom:external", this.zoomExternal, this);

                    /* Init */

                    try {
                        if ( typeof this.options.page != "undefined" ){
                            this.goto( this.options.page.get( "number" ) );
                        }
                        else{
                            this.goto( options.page );
                        }
                    } 
                    catch (Exception) {
                    }
                    this.layout.$el.removeClass("norender");
                  });
            });

            // Destructor
            this.addFinalizer(function () {
              this.layout.$el.addClass("norender");

              App
                  .off("fit", this.fit)
                  .off("goto", this.goto)
                  .off("mode", this.mode)
                  .off("zoom", this.zoom)
                  .off("zoom:external", this.zoomExternal);
            });

            /* Events */

            this.fit = function (force) {
              this.layout.navigation.currentView.collection
                  .findWhere({command: "fit"})
                  .set("type", (force) ? "fit-back" : "fit");
            };

            this.goto = function (page) {
              $.when(
                      App.request("chapters", {context: this}),
                      App.request(
                          "pages",
                          {
                            context: this,
                            filter: function () {
                              return true;
                            }
                          }
                      )
                  )
                  .done(function (book, all) {
                    var pages = book.pages(),
                        current = pages.get(page),
                        first = current === pages.first(),
                        last = current === pages.last(),
                        title = (current) ? current.get("title") : "";

                    App.request("viewer:is:mode:double", {context: this[0]})
                        .done(function () {
                          var match = _.isEqual(
                              (current) ? current.toJSON() : {},
                              all.first().toJSON()
                          );

                          if ((!first || !match) && !last) {
                            var next = pages.at(pages.indexOf(current) + 1);
                            last = next === pages.last();

                            title += " - " + next.get("title");
                          }
                        })
                        .always(function () {

                            if ( typeof this.layout.paginator.currentView == "undefined" ){
                                //console.log( "NO se pillo la pagina", current );
                                this.options.page = current;
                            }
                            else {
                                //console.log( "SI se pillo la pagina", current ) ;
                                var view = this.layout.paginator.currentView;
                                view.model.set("page", title);
                                view.disable(first, last);
                            }
                        });
                  });
            };

            this.mode = function (mode) {
              this.layout.navigation.currentView.collection
                  .findWhere({command: "mode"})
                  .set("type", (mode === "double") ? "document" : "double");

              App.request("page", {context: this})
                  .done(function (page) {
                    this.goto(page);
                  });
            };

            this.zoom = function (scale) {
              this.layout.zoom.currentView.scroll((scale - 1) / 2 * 100);
//              this.layout.zoom.currentView.$(".bar").css("width", (scale - 1) / 2 *
//                  100 + "%");
            };

            this.zoomExternal = function (scale) {
              parent.trigger("zoom", scale);
            };
          }
      );
    }
);

define(
    'modules/worktools/worktools',[
        "app",
        "modules/worktools/template",
        "controllers/scrollbar",
        "configuration",
        "encrypt",
        "language",
        "views/dialog",
        "modules/worktools/regions/index",
        "modules/worktools/controllers/worktool",
        "modules/worktools/views/listView",
        "modules/worktools/views/filtersContainer",
        "modules/worktools/views/listIconView",
        "modules/worktools/views/contentDialog",
        "models/resource.collection",
        "controllers/loader",
        "modules/traze/traze"
    ],
    function (App, JST, Scrollbar, Config, Encrypt,Language, DialogView, IndexLayout, Controller,ListView,FiltersContainer, ListIconView,ContentDialog,ResourceCollation, Loader, traze) {
        "use strict";

        return App.module("Worktools", function (Worktools, App, Backbone, Marionette, $, _) {

            this.startWithParent = false;

          // Dialog open resource

            this.addInitializer(function () {

                var dv = new DialogView({
                    color: 5
                });

                var ResourceContentView = new DialogView({
                    color: 5,
                    width: '100%',
                    height: '100%',
                    className: "dialog fullscreen",
                    draggable: false
                  });

                var layout = new IndexLayout(),
                    self = this;


                $.when(App.request("buttons"), App.request("chapter"), App.request("chapters", {
                    collection: true
                })).done(function (buttons, current, chapters) {
                    layout.on("render", function () {

                        var iconsetup = [];
                        _.each(buttons, function (item, index) {

                            if(item.class == "library"){
                                iconsetup.push({
                                    type: item.icon,
                                    iconClass: item.class,
                                    path: item.path,
                                    buttonid: index,
                                    active: item.active,
                                    name: Language("teachersLibrary"),
                                    porunidad: item.porunidad,
                                    iframe: item.iframe
                                });
                            }else{
                                if(item.class == "links"){
                                    iconsetup.push({
                                        type: item.icon,
                                        iconClass: item.class,
                                        path: item.path,
                                        buttonid: index,
                                        active: item.active,
                                        name: Language("links"),
                                        porunidad: item.porunidad,
                                        iframe: item.iframe
                                    });
                                }else{
                                    iconsetup.push({
                                        type: item.icon,
                                        iconClass: item.class,
                                        path: item.path,
                                        buttonid: index,
                                        active: item.active,
                                        name: item.name,
                                        porunidad: item.porunidad,
                                        iframe: item.iframe
                                    });
                                }
                            }

                            
                        });

                        var a = new ListIconView({
                            collection: new Backbone.Collection(iconsetup)
                        });


                        layout.buttons.show(a);

                        var resourceView = new ListView({
                            collection: new ResourceCollation()
                        });

                       // Data Mapper
                        
                        var b = new FiltersContainer({
                            collection: new Backbone.Collection()
                        });
                        
                        layout.filters.show(b);

                    var contentDialog = new ContentDialog ({model: new Backbone.Model()});

                    this.controller = new Controller({view:a,resource:resourceView, dropdown: b,dialog: ResourceContentView, contentDialog: contentDialog, loader:Loader});

                    layout.elements.show(resourceView);

                    });

                    this.layout = (function () {
                        dv.show(layout);
                        return layout;
                    })();   

                    dv.on("quit", function() {
                    App.execute("module:stop", "Worktools");

                  });


                    // ********************************************************************
                    // ***     Fix para deshabilitar actividades sin traza en SCORM     ***
                    // ********************************************************************
                    // cuando se está en SCORM se desactiva la apertura de actividades desde el propio libro
                    if( traze.mT.tipoTraza == 'SCORM' ){
                      $(".icon-check").parent().addClass('inactive disabled');
                    }
                    // ********************************************************************


                    // ********************************************************************
                    // ***    Fix para deshabilitar herramientas en SCORM y en Netex    ***
                    // ********************************************************************
                    if( traze.mT.tipoTraza == 'SCORM' || traze.mT.tipoTraza == 'WC'){
                      $('.icon-wrench').parent().remove(); // Quito herramientas
                    }
                    // ********************************************************************

                });

            });

        });
    }
);
//noinspection JSHint
require(
    [
      "jquery",
      "underscore",
      "app",
      "index",
      "helpers/backbone.sync",
      "helpers/handlebars",
      "helpers/underscore",
      "modules/favorites/favorites",
      "modules/fullscreen_worktools/fullscreen_worktools",
      "modules/help/help",
      "modules/index/index",
      "modules/loader/loader",
      "modules/menu/menu",
      "modules/search/search",
      "modules/tools/tools",
      "modules/mybook/mybook",
      "modules/notebook/notebook",
      "modules/viewer/viewer",
      "modules/traze/traze",
      "modules/viewer/viewer.document",
      "modules/viewer/viewer.document.draw",
      "modules/viewer/models/shape",
      "modules/viewer/models/shape/arrow",
      "modules/viewer/models/shape/character",
      "modules/viewer/models/shape/circle",
      "modules/viewer/models/shape/clip",
      "modules/viewer/models/shape/cross",
      "modules/viewer/models/shape/doodle",
      "modules/viewer/models/shape/edit",
      "modules/viewer/models/shape/transparent-circle-inverse",
      "modules/viewer/models/shape/transparent-circle",
      "modules/viewer/models/shape/transparent-square-inverse",
      "modules/viewer/models/shape/transparent-square",
      "modules/viewer/models/shape/fill-circle-inverse",
      "modules/viewer/models/shape/fill-circle",
      "modules/viewer/models/shape/fill-square-inverse",
      "modules/viewer/models/shape/fill-square",
      "modules/viewer/models/shape/line",
      "modules/viewer/models/shape/link-square",
      "modules/viewer/models/shape/scloud",
      "modules/viewer/models/shape/square",
      "modules/viewer/models/shape/text-direction",
      "modules/viewer/models/shape/tick",
      "modules/viewer/models/shape/triangle",
      "modules/viewer/models/shape/underline",
      "modules/viewer/models/shape/highlight",
      "modules/viewer/models/shape/text-select",
      "modules/viewer/models/shape/text-highlight",
      "modules/viewer/models/shape/text-underline",
      "modules/viewer/viewer.bookmark",
      "modules/viewer/viewer.controls",
      "modules/worktools/worktools"
    ],
    function ($, _, app) {
      "use strict";

      app.reqres.setHandler("module:load", function (module) {
        var deferred = $.Deferred(),
            path = module.split(".");
        module = app;

        try {
          _.each(path, function (value) {
            module = module[value];
          });

          deferred.resolve(module);
        }
        catch (Exception) {
          deferred.reject();
        }

        return deferred.promise();
      });
    }
);
define("build", function(){});


require(["build"]);
